#ifndef _CDAS2ADST_h_
#define _CDAS2ADST_h_

#include <StationStatus.h>
#include <vector>
#include <DetectorGeometry.h>
#include <FileInfo.h>

class RecEvent;
class SDEvent;
class TErEvent;
class TCalibStation;
class SdRecLpnhe;
class TBits;

namespace cdas{
  
  /**
     \class CDAS2ADST
     
     \brief Module to convert CDAS 2 ADST.
     
     \authors  Ioana Maris, Julien Aublin, Pierre Billoir, Moritz M\"unchmeyer
  */
  
  class CDAS2ADST{
    
  public:
    
    CDAS2ADST();
  
    virtual ~CDAS2ADST();
    
    void FillADSTEvent(RecEvent& adst, const TErEvent& cdas);
    void FillRecInfoADSTEvent(RecEvent& adst, const SdRecLpnhe& reco);
    void FillFileInfo();
    
    const DetectorGeometry& GetDetectorGeometry() const {return fDetectorGeometry;}
    
    const FileInfo& GetFileInfo() const {return fFileInfo;}
    
    void SetStoreVEMTraces(const bool what) { fStoreVEMTraces = what;}    
    void SetStoreFADCTraces(const bool what ) { fStoreFADCTraces = what;}
    void SetMinEnergyToStoreTraces(const double what ) { fMinEnergyToStoreTraces = what;}
    
  private:
    void FillSDEvent(SDEvent& sd, const TErEvent& cdas);
    void FillStationsInfo(SDEvent& event, TBits& bits,
			  const std::vector<TCalibStation*>& stations,
			  const EStationStatus stat, const TErEvent& cdas);
    
    
    DetectorGeometry fDetectorGeometry;
    FileInfo fFileInfo;

    double fMinEnergyToStoreTraces;
    bool fStoreVEMTraces;
    bool fStoreFADCTraces;
  };

}


#endif





