#include "EDArray.h"
#include "EDMainFrame.h"
#include "TCdasUtil.h"

EDMainFrame *Main;
/*-----------------------------------------------------------------------------*/
EDArrayFD::EDArrayFD(double x, double y, double angle) {
  fLine[0] = new TLine(x+FDSIZE*sin((180-angle)*kCdasUtil::DEG2RAD),y+FDSIZE*cos((180-angle)*kCdasUtil::DEG2RAD),x+FDSIZE*sin((-angle)*kCdasUtil::DEG2RAD),y+FDSIZE*cos((-angle)*kCdasUtil::DEG2RAD));
  fLine[0]->SetLineColor(29);
  fLine[0]->SetLineStyle(2);
  fLine[0]->SetLineWidth(3);
  for (int i=1;i<6;i++) {
    fLine[i]=new TLine(x+2*SIZE*sin((30*i-angle)*kCdasUtil::DEG2RAD),y+2*SIZE*cos((30*i-angle)*kCdasUtil::DEG2RAD),x+FDSIZE*sin((30*i-angle)*kCdasUtil::DEG2RAD),y+FDSIZE*cos((30*i-angle)*kCdasUtil::DEG2RAD));
    fLine[i]->SetLineColor(29);
    fLine[i]->SetLineStyle(2);
    fLine[i]->SetLineWidth(3);
  }
}

/*-----------------------------------------------------------------------------*/
EDArrayCanvas::EDArrayCanvas(const char *name, const TGWindow *p, const TGWindow *main, unsigned int w, unsigned int h, unsigned int options, unsigned long back) : TRootEmbeddedCanvas(name,p, w, h, options, back) {
  zoom = 15000;
  fCanvasArray = this->GetCanvas();
  fShowerCore.SetMarkerSize(1);
  fShowerCore.SetMarkerColor(2);
  fShowerCore.SetMarkerStyle(29);
  fShowerDirection.SetLineColor(2);
  fShowerDirection.SetLineWidth(2);
  fShowerShadow.SetLineWidth(1);
  fShowerShadow.SetLineStyle(3);
  fShowerCore.SetX(1000000);
  fShowerCore.SetY(1000000);
  fShowerShadow.SetX1(1000000);
  fShowerShadow.SetX2(1000000);
  fShowerShadow.SetY1(1000000);
  fShowerShadow.SetY2(1000000);
  fShowerDirection.SetX1(1000000);
  fShowerDirection.SetX2(1000000);
  fShowerDirection.SetY1(1000000);
  fShowerDirection.SetY2(1000000);
  // SDP line characteristics
  for (int i = 0; i < 4; i++) {
    fSDP[i].SetLineColor(150+2);
    fSDP[i].SetLineWidth(2);
    fSDP[i].SetLineStyle(4);
  }
  for (int i=1;i<MAXSTAT;i++) {
    fStation[i] = NULL;
    fMarker[i] = NULL;
  }
  // Station[0] -> For selected stations
  fStation[0] = new TEllipse(0,0,SIZE,SIZE,0,360,0);

  // Adding fluorescence detectors
  int orient[4]={-30,60,-172,-120};
  for (int i=0;i<4;i++)
    fFD.push_back(new EDArrayFD(kIoSd::FdX(i+1),kIoSd::FdY(i+1),orient[i]+90));

  // Drawing default
  /*fCanvasArray->SetEditable(kTRUE);
  Clear();
  fCanvasArray->Range(kIoSd::FdX(4),kIoSd::FdY(1),kIoSd::FdX(1),kIoSd::FdY(4));
  fCanvasArray->SetBorderSize(2);
  fCanvasArray->cd();
  unsigned int size = fFD.size();
  for (unsigned int i=0;i<size;i++)
    fFD[i]->Draw();
  Update();
  fCanvasArray->SetEditable(kFALSE);*/

  // mouse feature
  if(main) {
    Main = (EDMainFrame *)main;
    gVirtualX->GrabButton(fId,kButton1,kAnyModifier,kButtonPressMask,kNone,kNone);
  }
  //kButtonReleaseMask,kNone,kNone);
  //PressMask,kNone,kNone);

  // zoom features
  fPlus.SetMarkerColor(15);
  fMinus.SetMarkerColor(15);
  fPlus.SetMarkerSize(2.5);
  fMinus.SetMarkerSize(2.5);
  fPlus.SetMarkerStyle(22);
  fMinus.SetMarkerStyle(23);
  fPlusT.SetText(0,0,"+");
  fMinusT.SetText(0,0,"-");
  fPlusT.SetTextSize(0.057);
  fMinusT.SetTextSize(0.057);
  fPlusT.SetTextAlign(23);
  fMinusT.SetTextAlign(22);

  this->Resize(this->GetDefaultSize());

}

/*-----------------------------------------------------------------------------*/

void EDArrayCanvas::Update() {
  fCanvasArray->cd();
  fCanvasArray->Modified();
  fCanvasArray->Update();
}

/*-----------------------------------------------------------------------------*/
bool EDArrayCanvas::HandleButton(Event_t *event) {
  if(!fCanvasArray->GetListOfPrimitives()->GetSize())
    return kFALSE;
  if(event->fCode != kButton1)
    return kFALSE;
  double xcoord = fCanvasArray->AbsPixeltoX(event->fX);
  double ycoord = fCanvasArray->AbsPixeltoY(event->fY);

  double dx = fPlus.GetX() - xcoord,
    dy = fPlus.GetY() - ycoord,
    dym = fMinus.GetY() - ycoord;
  if(sqrt(sqr(dx)+sqr(dy))<600*zoom/15000) {
    if(zoom>4000) {
      zoom -= 2000;
      Select(Main->fLastStation);
    }
    return kTRUE;
  } else if(sqrt(sqr(dx)+sqr(dym))<600*zoom/15000) {
    zoom += 2000;
    Select(Main->fLastStation);
    return kTRUE;
  }
  double min = 1000000;
  unsigned int id = 0;
  for(unsigned int i=1; i<MAXSTAT; i++) {
    if(!fStation[i])
      continue;
    dx = fStation[i]->GetX1()-xcoord;
    dy = fStation[i]->GetY1()-ycoord;
    if(sqrt(sqr(dx)+sqr(dy)) < min) {
      min = sqrt(sqr(dx)+sqr(dy));
      id = i;
      if(min<SIZE)
        break;
    }
  }
  if(min > SIZE)
    return kTRUE;
  if(id) {
    Main->StationListBox()->Select(id);
    Main->StationListBox()->SetTopEntry(id);
    Main->StationSel(id);
    if(Main->fConfig.GetInt("Enable"))
      Main->fIds2Delete.push_back(id);
  }
  return kTRUE;
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::Select(int stat) {
  if (!fStation[stat]) return;
  fCanvasArray->cd();
  fCanvasArray->SetEditable(kTRUE);
  fStation[0]->SetX1(fStation[stat]->GetX1());
  fStation[0]->SetY1(fStation[stat]->GetY1());
  fStation[0]->SetR1(fStation[stat]->GetR1()+200);
  fStation[0]->SetR2(fStation[stat]->GetR2()+200);
  fStation[0]->SetPhimin(fStation[stat]->GetPhimin());
  fStation[0]->SetPhimax(fStation[stat]->GetPhimax());
  fStation[0]->SetTheta(fStation[stat]->GetTheta());
  fStation[0]->Draw();
  fStation[stat]->Draw();
  double x = fStation[stat]->GetX1();
  double y = fStation[stat]->GetY1();
  fCanvasArray->Range(x-zoom, y-zoom*Aspect(), x+zoom, y+zoom*Aspect());
  unsigned int size = fLines.size();
  for (unsigned int k=0; k<size; k++)
    fLines[k].Draw();

  // zoom
  fPlus.SetX(x-zoom*0.9);
  fPlus.SetY(y+zoom*Aspect()*0.9);
  fMinus.SetX(x-zoom*0.9);
  fMinus.SetY(y+zoom*Aspect()*0.75);
  fPlusT.SetX(x-zoom*0.9);
  fPlusT.SetY(y+zoom*Aspect()*0.9);
  fMinusT.SetX(x-zoom*0.9);
  fMinusT.SetY(y+zoom*Aspect()*0.75);
  fPlus.Draw();
  fMinus.Draw();
  fPlusT.Draw();
  fMinusT.Draw();

  Update();
  fCanvasArray->SetEditable(kFALSE);
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::UnTrigAll() {
  for (int i=1;i<MAXSTAT;i++) {
    if (fStation[i]) {
      fStation[i]->SetR1(150);//SIZE);
      fStation[i]->SetR2(150);//SIZE);
    }
  }
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::SetSignal(int stat, double signal) {
  if (!fStation[stat]) return;
  double size=log10(signal);
  if (size<0) size=0;
  size=sqrt(size);
  size*=400;
  size*= sqrt((GetCanvas()->GetUymax()-GetCanvas()->GetUymin())/20000);
  if (signal==0) size=SIZE;
  if (signal==-1) size=150;
  if (size<150) size=150;
  if (size>750) size=750;
  fStation[stat]->SetR1(size);
  fStation[stat]->SetR2(size);
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::Clear() {
  fCanvasArray->Clear();
  for (int i=1;i<MAXSTAT;i++) {
    if (fStation[i]) delete fStation[i];
    fStation[i] = NULL;
  }
/* Before it was stating: Not deleting TMarkers: there are a lot of them, and they always have the same position
   However, on summary archives stations may have moved. Let's do things properly */
  for (int i=1;i<MAXSTAT;i++) {
    if (fMarker[i]) delete fMarker[i];
    fMarker[i] = NULL;
  }
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::DrawProjArray(TErEvent *rec) {
  double xmax,ymax,xmin,ymin;
  xmax = ymax = -1e32;
  ymin = xmin = 1e32;

  unsigned int size = rec->fSelectedStations.size();
  for (unsigned int i=0; i<size; i++) {
    float distmax = distance2Core(rec->fSelectedStations[i]->fX-rec->fRec.fXCore, rec->fSelectedStations[i]->fY-rec->fRec.fYCore, rec->fRec.fU, rec->fRec.fV);
    if (ymax < distmax)
      ymax = distmax;
  }
  ymax+=500;
  ymin=-ymax;
  xmin=ymin*GetWidth()/GetHeight();
  xmax=ymax*GetWidth()/GetHeight();

  fCanvasArray->SetEditable(kTRUE);
  Clear();
  fCanvasArray->Range(xmin,ymin,xmax,ymax);
  fCanvasArray->SetBorderSize(2);
  fCanvasArray->cd();

  size = rec->fCalibStations.size();
  for (unsigned int i=0; i<size; i++) {
    TCalibStation *st = &rec->fCalibStations[i];
    double a=(st->fX-rec->fRec.fXCore);
    double b=(st->fY-rec->fRec.fYCore);
    double x=a*cos(M_PI/2+rec->fRec.Phi())+b*sin(M_PI/2+rec->fRec.Phi());
    double y=b*cos(M_PI/2+rec->fRec.Phi())-a*sin(M_PI/2+rec->fRec.Phi());
    y*=cos(rec->fRec.Theta());
    if (!fStation[st->Id]) {
      fStation[st->Id] = new TEllipse(0,0,SIZE,SIZE,0,360,0);
      fStation[st->Id]->SetLineColor(9);
    }
    fStation[st->Id]->SetX1(x);
    fStation[st->Id]->SetY1(y);
    if (st->Id==kIoSd::CARMEN || st->Id==kIoSd::MIRANDA || st->Id==kIoSd::DIA || st->Id==kIoSd::NOCHE || st->Id==kIoSd::MOULIN || st->Id==kIoSd::ROUGE) {
      fStation[st->Id]->SetPhimin(0);
      fStation[st->Id]->SetPhimax(180);
      if (st->Id==kIoSd::CARMEN || st->Id==kIoSd::NOCHE || st->Id==kIoSd::ROUGE)
        fStation[st->Id]->SetTheta(90);
      else
        fStation[st->Id]->SetTheta(270);
    }
    fStation[st->Id]->Draw();
  }
  double sig[5]={100.,10.,4.,2.,1.};
  int linest[5]={2,2,4,4,3};
  int linecl[5]={1,1,4,2,16};
  double m = rec->fRec.fSRef/pow(rec->fRec.fRefDist,rec->fRec.fBeta);
  double s=rec->fRec.fSRef;
  for (int i=0;i<5;i++) {
    double r = pow(m/sig[i],-3./(log10(s)-log10(m)));
    if (fStation[MAXSTAT-i-1]) delete fStation[MAXSTAT-i-1];
    fStation[MAXSTAT-i-1] = new TEllipse(0,0,r,0,360,0);
    fStation[MAXSTAT-i-1]->SetLineStyle(linest[i]);
    fStation[MAXSTAT-i-1]->SetLineColor(linecl[i]);
    fStation[MAXSTAT-i-1]->Draw("only");
  }
  fShowerCore.SetX(0);
  fShowerCore.SetY(0);
  fShowerCore.Draw();
  fShowerDirection.SetX1(0);
  fShowerDirection.SetX2(1000*sin(M_PI/2+rec->fRec.Phi()));
  fShowerDirection.SetY1(0);
  fShowerDirection.SetY2(1000*cos(M_PI/2+rec->fRec.Phi()));
  fShowerDirection.Draw();
  Update();
  fCanvasArray->SetEditable(kFALSE);
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::DrawArray(TErEvent *ev, double xc, double yc, int zooom) {
  zoom = zooom;
  double xmax,ymax,xmin,ymin;
  xmax=ymax=-1e32;
  ymin=xmin=1e32;
  unsigned int size = ev->fCalibStations.size();

  if (xc && yc) {
    xmin = xc-zoom;
    xmax = xc+zoom;
    ymin = yc-zoom*Aspect();
    ymax = yc+zoom*Aspect();
  } else {
    for (unsigned int i=0;i<size;i++) {
      TCalibStation *st = &ev->fCalibStations[i];
      if (!st->fX || !st->fY)
        continue;
      if (xmin > st->fX)
        xmin = st->fX-zoom;
      if (xmax < st->fX)
        xmax = st->fX+zoom;
      if (ymin > st->fY)
        ymin = st->fY-zoom*Aspect();
      if (ymax < st->fY)
        ymax = st->fY+zoom*Aspect();
    }
  }
  if (zooom>1000000) {
    if (Aspect()<1) {
      xmin=-40000/Aspect();
      xmax=40000/Aspect();
      ymin=-40000;
      ymax=40000;
    } else {
      xmin=-40000;
      xmax=40000;
      ymin=-40000*Aspect();
      ymax=40000*Aspect();
    }
  }
  fCanvasArray->SetEditable(kTRUE);
  Clear();
  fCanvasArray->Range(xmin,ymin,xmax,ymax);
  fCanvasArray->SetBorderSize(2);
  fCanvasArray->cd();
  fLines.clear();
  int count=0;
  for (unsigned int i=0;i<size;i++) {
    TCalibStation *st = &ev->fCalibStations[i];
    //if ( (xc && yc) && (st->fX>xmax || st->fX<xmin || st->fY>ymax || st->fY<ymin)) continue;
    if (st->Trigger.Window != 0 && st->Error==IoSdEvent::eT3NotFound) {
      if (!fMarker[st->Id]) {
	fMarker[st->Id] = new TMarker(st->fX,st->fY,8);
	fMarker[st->Id]->SetMarkerColor(38);
	fMarker[st->Id]->SetMarkerSize(0.8);
      }
      fMarker[st->Id]->Draw();
      continue;
    }
    if (!fStation[st->Id]) {
      fStation[st->Id] = new TEllipse(0,0,SIZE,SIZE,0,360,0);
      fStation[st->Id]->SetLineColor(9);
    }
    fStation[st->Id]->SetX1(st->fX);
    fStation[st->Id]->SetY1(st->fY);
    if (st->Id==kIoSd::CARMEN || st->Id==kIoSd::MIRANDA || st->Id==kIoSd::DIA || st->Id==kIoSd::NOCHE || st->Id==kIoSd::MOULIN || st->Id==kIoSd::ROUGE) {
      fStation[st->Id]->SetPhimin(0);
      fStation[st->Id]->SetPhimax(180);
      if (st->Id==kIoSd::CARMEN || st->Id==kIoSd::NOCHE || st->Id==kIoSd::ROUGE)
        fStation[st->Id]->SetTheta(90);
      else
        fStation[st->Id]->SetTheta(270);
    }
    fStation[st->Id]->Draw();
  }//*******************************//
   //****** stations deleted ******//
  size = ev->fRandomStations.size();
  for (unsigned int k=0; k<size; k++) {
    TLine linex, liney;
    fLines.push_back(linex);
    fLines.push_back(liney);
    int offset = 400;
    double x = ev->fRandomStations[k]->fX;
    double y = ev->fRandomStations[k]->fY;
    fLines[count*2].SetX1(x-offset);
    fLines[count*2].SetX2(x+offset);
    fLines[count*2].SetY1(y-offset);
    fLines[count*2].SetY2(y+offset);
    fLines[count*2+1].SetX1(x+offset);
    fLines[count*2+1].SetX2(x-offset);
    fLines[count*2+1].SetY1(y-offset);
    fLines[count*2+1].SetY2(y+offset);
    count++;
  }
  size = fLines.size();
  for (unsigned int k=0; k<size; k++) {
    fLines[k].SetLineColor(6);
    fLines[k].SetLineWidth(2);
    fLines[k].Draw();
  }
  //****** highest energy triangle ******//
  int n=0;
  int st[3];
  size = ev->fSelectedStations.size();
  for (unsigned int k=0;k<size;k++) {
    if (ev->fSelectedStations[k]->Trigger.Type & 0x400) {
      if (n<3)
        st[n]=k;
      n++;
    }
  }
  size = fLines.size();
  if (n==3) {
    TLine line1;
    TLine line2;
    TLine line3;
    fLines.push_back(line1);
    fLines.push_back(line2);
    fLines.push_back(line3);
    double x1 = ev->fSelectedStations[st[0]]->fX;
    double y1 = ev->fSelectedStations[st[0]]->fY;
    double x2 = ev->fSelectedStations[st[1]]->fX;
    double y2 = ev->fSelectedStations[st[1]]->fY;
    double x3 = ev->fSelectedStations[st[2]]->fX;
    double y3 = ev->fSelectedStations[st[2]]->fY;
    fLines[size].SetX1(x1);
    fLines[size].SetY1(y1);
    fLines[size].SetX2(x2);
    fLines[size].SetY2(y2);
    fLines[size+1].SetX1(x1);
    fLines[size+1].SetY1(y1);
    fLines[size+1].SetX2(x3);
    fLines[size+1].SetY2(y3);
    fLines[size+2].SetX1(x2);
    fLines[size+2].SetY1(y2);
    fLines[size+2].SetX2(x3);
    fLines[size+2].SetY2(y3);
    fLines[size].SetLineColor(IST2);
    fLines[size+1].SetLineColor(IST2);
    fLines[size+2].SetLineColor(IST2);
    fLines[size].SetLineWidth(2);
    fLines[size+1].SetLineWidth(2);
    fLines[size+2].SetLineWidth(2);
    fLines[size].Draw();
    fLines[size+1].Draw();
    fLines[size+2].Draw();
  }
  //*******************************//
  // Drawing FD
  size = fFD.size();
  for (unsigned int i=0;i<size;i++)
    fFD[i]->Draw();

  // Drawing shower if present
  fShowerCore.Draw();
  fShowerShadow.Draw();
  fShowerDirection.Draw();

  // Draw SDP line
  DrawSDP(ev->T3s);
  fPlus.Draw();
  fMinus.Draw();
  fPlusT.Draw();
  fMinusT.Draw();

  Update();
  fCanvasArray->SetEditable(kFALSE);
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::SetShower(TErEvent *rec) {
  fCanvasArray->cd();
  fCanvasArray->SetEditable(kTRUE);

  if (rec==NULL) {
    fShowerCore.SetX(1000000);
    fShowerCore.SetY(1000000);
    fShowerShadow.SetX1(1000000);
    fShowerShadow.SetX2(1000000);
    fShowerShadow.SetY1(1000000);
    fShowerShadow.SetY2(1000000);
    fShowerDirection.SetX1(1000000);
    fShowerDirection.SetX2(1000000);
    fShowerDirection.SetY1(1000000);
    fShowerDirection.SetY2(1000000);
  } else {
    fShowerCore.SetX(rec->fRec.fXCore);
    fShowerCore.SetY(rec->fRec.fYCore);
    fShowerShadow.SetX1(rec->fRec.fXCore);
    fShowerShadow.SetX2(rec->fRec.fXCore+5*SIZE*cos(rec->fRec.Phi()));
    fShowerShadow.SetY1(rec->fRec.fYCore);
    fShowerShadow.SetY2(rec->fRec.fYCore+5*SIZE*sin(rec->fRec.Phi()));
    fShowerDirection.SetX1(rec->fRec.fXCore);
    fShowerDirection.SetX2(rec->fRec.fXCore+5*SIZE*cos(rec->fRec.Phi())*sin(rec->fRec.Theta()));
    fShowerDirection.SetY1(rec->fRec.fYCore);
    fShowerDirection.SetY2(rec->fRec.fYCore+5*SIZE*sin(rec->fRec.Phi())*sin(rec->fRec.Theta()));
    DrawSDP(rec->T3s);
  }
  fShowerCore.Draw();
  fShowerShadow.Draw();
  fShowerDirection.Draw();

  fPlus.Draw();
  fMinus.Draw();
  fPlusT.Draw();
  fMinusT.Draw();
  fCanvasArray->SetEditable(kFALSE);
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::DrawSDP(std::vector<IoSdT3Trigger> &T3s) {
  // drawing line representing the SDPs (up to 4 lines)
  fCanvasArray->cd();
  fCanvasArray->SetEditable(kTRUE);
  unsigned int n = T3s.size();
  if (n >4)
    n = 4;
  for (unsigned int i = 0; i < n; i++) {
    float angle = T3s[i].sdpangle();
    fSDP[i].SetX1(kIoSd::FdX(T3s[i].EyeId()));
    fSDP[i].SetX2(kIoSd::FdX(T3s[i].EyeId()) + SDPLENGTH*cos(angle));
    fSDP[i].SetY1(kIoSd::FdY(T3s[i].EyeId()));
    fSDP[i].SetY2(kIoSd::FdY(T3s[i].EyeId()) + SDPLENGTH*sin(angle));
    if (T3s[i].EyeId())
      fSDP[i].Draw();
  }
  fCanvasArray->SetEditable(kFALSE);
}


void EDArrayCanvas::DrawSDP(IoSdT3Trigger &t) {
  // drawing line representing the SDP
  fCanvasArray->cd();
  fCanvasArray->SetEditable(kTRUE);

  float angle = t.sdpangle();
  fSDP[0].SetX1(kIoSd::FdX(t.EyeId()));
  fSDP[0].SetX2(kIoSd::FdX(t.EyeId()) + SDPLENGTH*cos(angle));
  fSDP[0].SetY1(kIoSd::FdY(t.EyeId()));
  fSDP[0].SetY2(kIoSd::FdY(t.EyeId()) + SDPLENGTH*sin(angle));
  if (t.EyeId())
    fSDP[0].Draw();

  fCanvasArray->SetEditable(kFALSE);
}

/*-----------------------------------------------------------------------------*/
void EDArrayCanvas::Set(int STAT, int color) {
  if (!fStation[STAT]) return;
  fCanvasArray->cd();
  fCanvasArray->SetEditable(kTRUE);
  fStation[STAT]->SetFillColor(color);
  fCanvasArray->SetEditable(kFALSE);
}
