#ifndef EDARRAY_H
#define EDARRAY_H

#include <list>
#include <string>
#include <iostream>
#include <iomanip>

#include <TRootEmbeddedCanvas.h>
#include <TCanvas.h>
#include <TEllipse.h>
#include <TMarker.h>
#include <TLine.h>
#include <TText.h>
#include <TArrow.h>

#include "AugerIoSd.h"
#include "Er.h"

#define SIZE 350
//#define BORDER 4000
#define FDSIZE 15000
#define SDPLENGTH 50000
#define MAXSTAT (2048 + 2048) //because of use of LsId>2048 indexes

// Colors
#define IST5 8
#define IST2 3
#define IST1 5
#define HASNODATA 38
#define ISBADLS 2
#define ISBADCOMMS 46
#define ISDOWN 1
#define ISSILENT 7
#define ISLIGHTN 94
#define NOT2 15
#define AMBIGUOUS 23
#define COLORDEFAULT 10

class EDArrayFD {
private:
  TLine               *fLine[6];

public:
  EDArrayFD(double, double, double);
  ~EDArrayFD() {
    for(int i=0;i<6;i++)
      if (fLine[i]) delete fLine[i];
  }
  void Draw() {
    for(int i=0;i<6;i++)
      fLine[i]->Draw();
  }
};

class EDArrayCanvas : public TRootEmbeddedCanvas {
private:
  TCanvas                 *fCanvasArray;
  TEllipse                *fStation[MAXSTAT];
  TMarker                 *fMarker[MAXSTAT];
  TMarker                 fShowerCore, fPlus, fMinus;
  TLine                   fShowerDirection, fSDPLL, fSDP[4];
  TLine                   fShowerShadow;
  TText                   fPlusT, fMinusT;
  std::vector<TLine>      fLines;
  std::vector<EDArrayFD*> fFD;

  void DrawFD(double, double, double, double);
  void SetDefault(int STAT) {
    Set(STAT,COLORDEFAULT);
    SetSignal(STAT,0);
  }

public:
  int zoom;
  EDArrayCanvas(const char *name, const TGWindow *p, const TGWindow *main, unsigned int w, unsigned int h, unsigned int options=kSunkenFrame|kDoubleBorder, unsigned long back=fgDefaultFrameBackground);
  ~EDArrayCanvas() {
    for (unsigned int i=0;i<MAXSTAT;i++)
      if (fStation[i]) delete fStation[i];
    unsigned int size = fFD.size();
    for (unsigned int i=0;i<size;i++) delete fFD[i];
    fLines.clear();
  };
  void DrawArray(TErEvent *ev, double xc=0., double yc=0.,int zooom=15000);
  void DrawProjArray(TErEvent *);
  void cd() {
    fCanvasArray->cd();
  }
  void Update();
  void UnTrigAll();
  void Clear(TErEvent *rec) {
    if(rec)
      for (unsigned int i=0;i<rec->Trigger.NumberOfStation;i++)
        SetDefault(rec->Stations[i].Id);
    SetShower(NULL);
  }
  void SetShower(TErEvent *);
  void SetSignal(int, double);
  void Set(int STAT, int color);
  void Select(int);
  void Clear();
  bool HandleButton(Event_t *);
  void DrawSDP(IoSdT3Trigger &);
  void DrawSDP(std::vector<IoSdT3Trigger> &list);
  double Aspect() {
    return GetHeight()*1./GetWidth();
  }

};

#endif
