#include "EDConfig.h"
#include <cstdlib>

const char* EDConfig::Get(const char *name) {
  char *cmpname=BuildName(name);
  unsigned int size = fName.size();
  for (unsigned int i=0;i<size;i++)
    if (strcmp(cmpname,fName[i].c_str())==0) {
      free(cmpname);
      return fValue[i].c_str();
    }
  free(cmpname);
  return NULL;
}

void EDConfig::Set(const char* name,const char* value) {
  char *cmpname=BuildName(name);
  unsigned int size = fName.size();
  for (unsigned int i=0;i<size;i++)
    if (strcmp(cmpname,fName[i].c_str())==0) {
      fValue[i]=value;
      if (atoi(value) && fMenu[i])
        fMenu[i]->CheckEntry(i+MENUOFFSET);
      if (!(atoi(value)) && fMenu[i])
        fMenu[i]->UnCheckEntry(i+MENUOFFSET);
      free(cmpname);
      return;
    };
}

int EDConfig::GetSig(const char*name) {
  char *cmpname=BuildName(name);
  unsigned int size = fName.size();
  for (unsigned int i=0;i<size;i++)
    if (strcmp(cmpname,fName[i].c_str())==0) {
      free(cmpname);
      return i+MENUOFFSET;
    }
  free(cmpname);
  return -1;
}

char *EDConfig::BuildName(const char *name) {
  char *out=strdup(name);
  char *tmp=out;
  while (*tmp!='\0')
    if (*tmp==' ') *tmp++='_';
    else tmp++;
  return out;
}

void EDConfig::Add(TGPopupMenu *menu,const char*name,const char*value) {
  fName.push_back(BuildName(name));
  fValue.push_back(value);
  fMenu.push_back(menu);
  menu->AddEntry(name,GetSig(name),(void*)0,(TGPicture*)0);
}

TGCheckButton * EDConfig::CheckButton(TGGroupFrame *frame,const char *name,const char*value) {
  fName.push_back(BuildName(name));
  fValue.push_back(value);
  fMenu.push_back(NULL);
  TGCheckButton * tmp = new TGCheckButton(frame,name,GetSig(name));
  return tmp;
}

void EDConfig::Add(const char*name,const char*value) {
  fName.push_back(BuildName(name));
  fValue.push_back(value);
  fMenu.push_back(NULL);
}

void EDConfig::Load(const char *file) {
  ifstream in;
  std::string input,val;
  in.open(file);
  while (1) {
    in>>input;
    in>>val;
    if (!in) break;
    if(!input.find("/")) {
      fName.push_back(input);
      fValue.push_back(val);
      fMenu.push_back(NULL);
    }
    else Set(input.c_str(),val.c_str());
  }
  in.close();
}

void EDConfig::Save(const char *file) {
  std::ofstream out;
  out.open(file);
  unsigned int size = fName.size();
  for (unsigned int i=0;i<size;i++)
    out<<fName[i]<<" "<<fValue[i]<<std::endl;
  out.close();
}

int EDConfig::GetInt(const char *name) {
  return atoi(Get(name));
}

int EDConfig::GetInt(int sig) {
  return atoi(fValue[sig-MENUOFFSET].c_str());
}

void EDConfig::SetInt(const char *name,int value) {
  char c[256];
  snprintf(c,256,"%d",value);
  Set(name,(const char*)c);
}

void EDConfig::SetInt(int sig,int value) {
  char c[256];
  snprintf(c,256,"%d",value);
  fValue[sig-MENUOFFSET]=c;
}

void EDConfig::CheckMenu(int sig) {
  if (GetInt(sig))
    fMenu[sig-MENUOFFSET]->UnCheckEntry(sig);
  else
    fMenu[sig-MENUOFFSET]->CheckEntry(sig);
  SetInt(sig,!GetInt(sig));
}

void EDConfig::CheckSetMenu(int sig) {
  fMenu[sig-MENUOFFSET]->CheckEntry(sig);
  SetInt(sig, 1);
}

void EDConfig::UnCheckUnSetMenu(int sig) {
  fMenu[sig-MENUOFFSET]->UnCheckEntry(sig);
  SetInt(sig, 0);
}
