#include "EDExpert.h"
#include <strstream>

enum MessageId {
  E_DEL_PMT1,
  E_DEL_PMT2,
  E_DEL_PMT3,
  E_DONE,
  E_VEARLY,
  E_VLATE,
  E_EARLY,
  E_LATE,
  E_APPLY,
  E_DEL_STATION
};

EDExpertFrame::EDExpertFrame(const TGWindow *p,const TGWindow *main, 
			     UInt_t w, UInt_t h) : TGTransientFrame(p, main, w, h) {

  char text[256];
  if(main) fMain = (EDMainFrame *)main;
  else return;
  if(fMain->fEvent) fEvent = fMain->fEvent;
  else {
    cerr<<"Please choose one event..."<<endl;
    return;
  }
  //if(fMain->fLastStation) fStation = fEvent->GetStationById(fMain->fLastStation);
  //else {cerr<<"Please choose one station..."<<endl;return;}
  fStation=fMain->fStation;

  //since ROOT 3.02.06 we don't need anymore to use this:
  //Connect("CloseWindow()", "EDExpertFrame", this, "CloseWindow()");

  ULong_t colordefault;
  ULong_t colormenu,colortab1,colortabelem1;
  ULong_t colorred, colorgreen, coloryellow, colorblue;
  gClient->GetColorByName("AntiqueWhite3",colordefault);
  gClient->GetColorByName("AntiqueWhite3",colortab1);
  gClient->GetColorByName("bisque3",colortabelem1);
  gClient->GetColorByName("azure2",colormenu);
  gClient->GetColorByName("IndianRed1",colorred);
  gClient->GetColorByName("PaleGreen3",colorgreen);
  gClient->GetColorByName("Khaki1",coloryellow);
  gClient->GetColorByName("dodgerblue",colorblue);

  std::ostrstream tmp;
  tmp.seekp(0,ios::beg);
  tmp<<"Station "<<fStation->name()<<" ("<<fStation->Id<<")"<<'\0';
  fGF1 = new TGGroupFrame(this, tmp.str(), kVerticalFrame);
  tmp.freeze(0);
  fGF1->ChangeBackground(colordefault);

  fHF = new TGHorizontalFrame(fGF1,100,30,kChildFrame,fgDefaultFrameBackground);
  fHF->ChangeBackground(colordefault);
  fHF->SetLayoutManager(new TGMatrixLayout(fHF,0,3,5,5));

  fMax = new TGLayoutHints(kLHintsExpandX);//| kLHintsExpandY);
  fCMax = new TGLayoutHints(kLHintsExpandX | kLHintsCenterX);
  fLayout = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY);

  for (int i=0;i<3;i++) {
    fDelPmt[i] = new TGTextButton(fHF," Turn Off ",E_DEL_PMT1+i);
    fDelPmt[i]->Associate(this);
    fDelPmt[i]->ChangeBackground(colorred);
    fPmt[i]= new TGTextEntry(fHF,new TGTextBuffer(10));
    fPmt[i]->Resize(65,fPmt[i]->GetDefaultHeight());
    snprintf(text,256,"%.3f",fStation->fPmt[i].SigInVEM());
    fPmt[i]->SetText(text);
    fPmt[i]->SetEnabled(kFALSE);
    snprintf(text,256,"PMT %d",i+1);
    fPmtL[i] = new TGLabel(fHF, text);
    fPmtL[i]->Resize(fDelPmt[0]->GetDefaultWidth(),fDelPmt[0]->GetDefaultHeight());
    fPmtL[i]->ChangeBackground(colordefault);
    fHF->AddFrame(fPmtL[i],fLayout);
    fHF->AddFrame(fPmt[i],fMax);
    fHF->AddFrame(fDelPmt[i],fMax);
  }
  fSig= new TGTextEntry(fHF,new TGTextBuffer(10));
  fSig->Resize(65,fSig->GetDefaultHeight());
  snprintf(text,256,"%.3f",fStation->fSigInVEM);
  fSig->SetText(text);
  fSigL = new TGLabel(fHF, "Average");
  fSigL->Resize(fDelPmt[0]->GetDefaultWidth(),fDelPmt[0]->GetDefaultHeight());
  fSigL->ChangeBackground(colordefault);
  fApply = new TGTextButton(fHF, "Apply",E_APPLY);
  fApply->Associate(this);
  fApply->Resize(fDelPmt[0]->GetDefaultWidth(),fDelPmt[0]->GetDefaultHeight());
  fApply->ChangeBackground(colorred);
  fHF->AddFrame(fSigL,fLayout);
  fHF->AddFrame(fSig,fCMax);
  fHF->AddFrame(fApply,fCMax);

  fHF2 = new TGHorizontalFrame(fGF1,100,30,kChildFrame,fgDefaultFrameBackground);
  fHF2->ChangeBackground(colordefault);
  fHF2->SetLayoutManager(new TGMatrixLayout(fHF2,0,5,5,5));

  fVEarly = new TGTextButton(fHF2, "  <<  ",E_VEARLY);
  fVEarly->Associate(this);
  fVEarly->ChangeBackground(colorred);
  fEarly = new TGTextButton(fHF2, "   <  ",E_EARLY);
  fEarly->Associate(this);
  fEarly->ChangeBackground(colorred);
  fLate = new TGTextButton(fHF2, "   >  ",E_LATE);
  fLate->Associate(this);
  fLate->ChangeBackground(colorred);
  fVLate = new TGTextButton(fHF2, "  >>  ",E_VLATE);
  fVLate->Associate(this);
  fVLate->ChangeBackground(colorred);
  fStartTimeL = new TGLabel(fHF2, "Timing");
  fStartTimeL->Resize(fLate->GetDefaultWidth(),fLate->GetDefaultHeight());
  fStartTimeL->ChangeBackground(colordefault);
  fHF2->AddFrame(fVEarly,fCMax);
  fHF2->AddFrame(fEarly,fCMax);
  fHF2->AddFrame(fStartTimeL,fLayout);
  fHF2->AddFrame(fLate,fCMax);
  fHF2->AddFrame(fVLate,fCMax);

  fDelStation = new TGTextButton(fGF1,"Delete station",E_DEL_STATION);
  fDelStation->Associate(this);
  fDelStation->ChangeBackground(colorred);

  fGF1->AddFrame(fHF,fMax);
  fGF1->AddFrame(fHF2,fMax);
  fGF1->AddFrame(fDelStation, fMax);

  fCancel = new TGTextButton(this, "     Done     ",E_DONE);
  fCancel->Associate(this);
  fCancel->ChangeBackground(colorred);

  AddFrame(fGF1,fMax);
  AddFrame(fCancel, fLayout);

  MapSubwindows();
  Layout();

  if (main) {
    int ax, ay;
    Window_t wdummy;
    gVirtualX->TranslateCoordinates(main->GetId(), GetParent()->GetId(),
                                    (Int_t)(((TGFrame *) main)->GetWidth() - fWidth) >> 1,
                                    (Int_t)(((TGFrame *) main)->GetHeight() - fHeight) >> 1,
                                    ax, ay, wdummy);
    if (ax < 0) ax = 10;
    if (ay < 0) ay = 10;
    Move(ax, ay);
    SetWMPosition(ax, ay);
  }

  ChangeBackground(colordefault);
  SetWindowName("Expert Menu");
  SetIconName("e. m.");
  MapWindow();
  fClient->WaitFor(this);
};

Bool_t EDExpertFrame::ProcessMessage(Long_t msg, Long_t parm1, Long_t) {
  char text[256];
  switch (GET_MSG(msg)) {
  case kC_COMMAND:
    switch (GET_SUBMSG(msg)) {
    case kCM_BUTTON:
      if(parm1==E_VEARLY) {
        fStation->fStartBin += 10;
        fMain->ListStations(fStation->Id,0);
        break;
      }
      if(parm1==E_VLATE) {
        fStation->fStartBin -= 10;
        fMain->ListStations(fStation->Id,0);
        break;
      }
      if(parm1==E_EARLY) {
        fStation->fStartBin += 1;
        fMain->ListStations(fStation->Id,0);
        break;
      }
      if(parm1==E_LATE) {
        fStation->fStartBin -= 1;
        fMain->ListStations(fStation->Id,0);
        break;
      }
      if(parm1==E_APPLY) {
        fStation->fSigInVEM = (atof(fSig->GetText()));
        fMain->ListStations(fStation->Id,0);
        break;
      }
      if(parm1==E_DONE) {
        CloseWindow();//since ROOT 3.02.06
        break;
      }
      if(parm1==E_DEL_STATION) {
        unsigned int id = fStation->Id;
        for (unsigned int i=0;i<fEvent->fSelectedStations.size();i++) {
          if (fEvent->fSelectedStations[i]->Id==id) {
            fEvent->fRandomStations.push_back(fEvent->fSelectedStations[i]);
            fEvent->fSelectedStations.erase(fEvent->fSelectedStations.begin()+i);
            i--;
          }
        }
        fMain->ListStations(0);
        CloseWindow();//since ROOT 3.02.06
        break;
      }
      if(parm1>=E_DEL_PMT1 && parm1<=E_DEL_PMT3) {
        int nb_tubes_ok=0;
        double sigtot=0;
        for (int i=0;i<3;i++) {
          if (parm1==E_DEL_PMT1+i) {
            fStation->fPmt[i].fSigInVEM = 0.00001;
            fPmt[i]->SetEnabled(kFALSE);
            fPmt[i]->SetText("Deleted");
          }
          if (fStation->fPmt[i].SigInVEM()>0.01)
            nb_tubes_ok++;
          sigtot+=fStation->fPmt[i].SigInVEM();
        }
        if (nb_tubes_ok)
          fStation->fSigInVEM = sigtot/nb_tubes_ok;
        else
          fStation->fSigInVEM = 0.00001;
        snprintf(text,256,"%.3f",fStation->fSigInVEM);
        fSig->SetText(text);
        fMain->ListStations(fStation->Id,0);
        break;
      }

    default:
      break;
    }
  default:
    break;
  }
  return kTRUE;
}
