#include "EDEye.h"
#include "EDMainFrame.h"
#include "TCdasUtil.h"
#include "TMarker.h"

#define SLIDEMAX 150
#define FDADC(k) FADCDataWordGetData(&fEvent->GetFADCData()->GetFADCData(fEvent->GetPixelList()->GetPixel(j))->GetFADCTrace()[k])

EDMainFrame *EDMain;

class TFDTubeDir {
public:
  TFDTubeDir() {
    char buf[256];
    std::string input=strdup(EDSRCDIR)+(std::string)"/FDtube_dir_ref.dat";
    FILE *f=fopen(input.c_str(),"r");
    for(int i=0;i<FdUtil::Fd::kFD_NEYES;i++)
      for(int j=0;j<FdUtil::Fd::kEYE_NEXTPIXELS;j++)
        fAlt[i][j]=fAz[i][j]=0;
    unsigned int j=0;
    int orient[4]={-30,60,-172,-120};
    while (fgets(buf, 255, f)) {
      double x,y;
      sscanf(buf,"%lf %lf",&x,&y);
      for (int i=0;i<FdUtil::Fd::kFD_NEYES;i++)
        for (int k=0;k<6;k++) {
          fAlt[i][k*480+j]=y*kCdasUtil::DEG2RAD;
          fAz[i][k*480+j]=(x+orient[i]+30*k+15)*kCdasUtil::DEG2RAD;
        }
      j++;
    }
    fclose(f);
  }
  double fAlt[FdUtil::Fd::kFD_NEYES][FdUtil::Fd::kEYE_NEXTPIXELS]; ///< Altitude in radians from ground
  double fAz[FdUtil::Fd::kFD_NEYES][FdUtil::Fd::kEYE_NEXTPIXELS];  ///< Azimuth in radians (0=east, positive northward)
  double Alt(int eye,int pix) {
    return fAlt[eye-1][pix-1]*kCdasUtil::RAD2DEG;
  } ///< Altitude in degrees
  double Az(int eye,int pix) {
    return fAz[eye-1][pix-1]*kCdasUtil::RAD2DEG;
  } ///< Azimuth in degrees
};


TFDTubeDir gFDTubeDir;

FDPixelCanvas::FDPixelCanvas(const char *name, const TGWindow *p, const TGWindow *main, unsigned int w, unsigned int h, unsigned int options, unsigned long back) : TRootEmbeddedCanvas(name,p, w, h, options, back) {
  fCanvas = GetCanvas();

  for (int i=0;i<10;i++) {
    PixelData[i]=new TH1F;
    PixelData[i]->SetStats(kFALSE);
    PixelData[i]->SetLineColor(i);
  }
  fColor=0;
  this->Resize(this->GetDefaultSize());
}

bool EDEyeEmbeddedCanvas::HandleButton(Event_t *event) {
  if (!fEvent) return false;
  double x = fCanvasEye->AbsPixeltoX(event->fX);
  double y = fCanvasEye->AbsPixeltoY(event->fY);
  double dmin=100;
  double id=-1;
  for (unsigned int j=0; j<fEvent->GetPixelList()->GetNumPixels(); j++ ) {
  //for (unsigned int j=0; j<6*440 ; j++ ) {
    int i=FdUtil::Fd::GetEyePixelNo(fEvent->GetPixelList()->GetPixel(j));
    int o=FdUtil::Fd::GetMirrorPixelNo(fEvent->GetPixelList()->GetPixel(j));
    if (o>440) continue;
    double alt0=gFDTubeDir.Alt(fEye,i);
    double az0=-gFDTubeDir.Az(fEye,i); // WARNING, x axis is -Azimuth
    double az=az0*cos(fRot)+alt0*sin(fRot);
    double alt=alt0*cos(fRot)-az0*sin(fRot);
    if ((az-x)*(az-x)+(alt-y)*(alt-y)<dmin) {
      dmin=(az-x)*(az-x)+(alt-y)*(alt-y);
      id=j;
    }
  }
  if (id==-1) return false;
  int j=id;
  int nb=1000, min=0;
  if (fRange) {
    min=230;
    nb=250;
  }
  fFDPixelCanvas->GetCanvas()->cd();
  if (fFDPixelCanvas->fColor==0) {
    fFDPixelCanvas->PixelData[0]->SetBins(nb,min,min+nb);
    for (int k=min; k<min+nb; k++) {
      fFDPixelCanvas->PixelData[fFDPixelCanvas->fColor]->SetBinContent(k-min+1,0);
    }
    fFDPixelCanvas->PixelData[0]->Draw();
    fFDPixelCanvas->fColor=1;
  }
  fFDPixelCanvas->PixelData[fFDPixelCanvas->fColor]->SetBins(nb,min,min+nb);
  for (int k=min; k<min+nb; k++) {
    fFDPixelCanvas->PixelData[fFDPixelCanvas->fColor]->SetBinContent(k-min+1,FDADC(k));
  }
  fFDPixelCanvas->PixelData[fFDPixelCanvas->fColor]->Draw("SAME");
  double hmin=4000,hmax=0;
  for (int i=1;i<10;i++) {
    if (!fMultiview && i>1) continue;
    if (fFDPixelCanvas->PixelData[i]->GetMaximum()>hmax) hmax=fFDPixelCanvas->PixelData[i]->GetMaximum();
    if (fFDPixelCanvas->PixelData[i]->GetMinimum()<hmin && fFDPixelCanvas->PixelData[i]->GetMinimum()>0) hmin=fFDPixelCanvas->PixelData[i]->GetMinimum();
  }
  fFDPixelCanvas->PixelData[0]->SetMaximum(hmax+100);
  fFDPixelCanvas->PixelData[0]->SetMinimum(hmin-100);
  if (fMultiview) {
    //int i=FdUtil::Fd::GetEyePixelNo(fEvent->GetPixelList()->GetPixel(j));
    //fPixel[i]->SetFillColor(fFDPixelCanvas->fColor);
    //fCanvasEye->Modified();
    fFDPixelCanvas->fColor++;
  }
  if (fFDPixelCanvas->fColor==5) fFDPixelCanvas->fColor++;
  if (fFDPixelCanvas->fColor>6) fFDPixelCanvas->fColor=1;
  fFDPixelCanvas->GetCanvas()->Update();
  //Update();
/*
  if(!fEDEyeEmbeddedCanvas->GetListOfPrimitives()->GetSize())
    return kFALSE;
  if(event->fCode != kButton1)
    return kFALSE;
  double xcoord = fEDEyeEmbeddedCanvas->AbsPixeltoX(event->fX);
  double ycoord = fEDEyeEmbeddedCanvas->AbsPixeltoY(event->fY);
  cout << xcoord << " " << ycoord << endl;
*/
  return true;
}

Bool_t EDEyeCanvas::ProcessMessage(long msg, long parm1, long parm2) {
  parm1+=0;
  switch (GET_MSG(msg)) {
  case kC_HSLIDER:
    switch (GET_SUBMSG(msg)) {
    case kSL_PRESS:
      fSliding=1;
      break;
    case kSL_RELEASE:
      fSliding=0;
      break;
    case kSL_POS:
      if (fSliding)
        SetView(parm2);
      else {
        if (parm2>fEDEyeEmbeddedCanvas->fView) {
          SetView(parm2+30);
          fSlide->SetPosition(parm2+30);
        } else {
          SetView(parm2-30);
          fSlide->SetPosition(parm2-30);
        }
      }
      break;
    }
    break;
    default:
      if (parm1==1) {
        fEDEyeEmbeddedCanvas->fRange=!fEDEyeEmbeddedCanvas->fRange;
        fEDEyeEmbeddedCanvas->fFDPixelCanvas->fColor=0;
      } else if (parm1==2) {
        fEDEyeEmbeddedCanvas->fMultiview=!fEDEyeEmbeddedCanvas->fMultiview;
        fEDEyeEmbeddedCanvas->fFDPixelCanvas->fColor=0;
      }
      break;
  }
  return true;
}

/*-----------------------------------------------------------------------------*/
EDEyeEmbeddedCanvas::EDEyeEmbeddedCanvas(const char *name,const TGWindow* p,int x,int y,int eye,double rot, double size) : TRootEmbeddedCanvas(name,p,x,y) {
  fCanvasEye = GetCanvas();
  fEvent=NULL;
  fMultiview=0;
  fEye=eye;
  fRot=rot;
  fCanvasEye->SetEditable(kTRUE);
  fCanvasEye->cd();
  //fSDP2=NULL;
  fSDP=NULL;
  for (int i=1;i<FdUtil::Fd::kEYE_NEXTPIXELS+1;i++) fPixel[i] = NULL;
  double xmin=360,xmax=-360,alt0,az0;
  double ymin=360,ymax=-360;
  for (int i=1;i<FdUtil::Fd::kEYE_NEXTPIXELS+1;i++) {
    alt0=gFDTubeDir.Alt(fEye,i);
    if (!(alt0>0)) continue;
    az0=-gFDTubeDir.Az(fEye,i); // WARNING, x axis is -Azimuth
    if (az0<xmin) xmin=az0;
    if (az0>xmax) xmax=az0;
    if (alt0<ymin) ymin=alt0;
    if (alt0>ymax) ymax=alt0;
    double az=az0*cos(fRot)+alt0*sin(fRot);
    double alt=alt0*cos(fRot)-az0*sin(fRot);
    TMarker *m=new TMarker(az,alt,2);
    m->SetMarkerSize(size);
    m->Draw();
  }
  fXmin=xmin-10;
  fYmin=ymin-10;
  fYmax=ymax+10;
  fCanvasEye->SetBorderSize(2);
  if (size==1) SetView(0);
  else SetFullView();
  fRange=1;
  gVirtualX->GrabButton(fId,kButton1,kAnyModifier,kButtonPressMask,kNone,kNone);
  //fCanvasEye->SetEditable(kTRUE);
}

/*-----------------------------------------------------------------------------*/
void EDEyeEmbeddedCanvas::Update() {
  fCanvasEye->cd();
  fCanvasEye->Modified();
  fCanvasEye->Update();
}

/*-----------------------------------------------------------------------------*/
void EDEyeEmbeddedCanvas::SetView(double val) {
  if (val<0) val=0;
  if (val>SLIDEMAX) val=SLIDEMAX;
  //Update();
  fView=val;
  fCanvasEye->Range(fXmin+val,0,fXmin+val+50,33);
  //this->Resize(this->GetDefaultSize());
  fCanvasEye->Modified();
  Update();
}

/*-----------------------------------------------------------------------------*/
void EDEyeEmbeddedCanvas::SetFullView() {
  //Update();
  double xmin=fXmin*cos(fRot)+0*sin(fRot);
  double ymin=0*cos(fRot)-fXmin*sin(fRot);
  double xmax=(fXmin+200)*cos(fRot)+33*sin(fRot);
  double ymax=33*cos(fRot)-(fXmin+200)*sin(fRot);
  double tmp=0;
  if (xmin>xmax) {
    tmp=xmin;
    xmin=xmax;
    xmax=tmp;
  }
  if (ymin>ymax) {
    tmp=ymin;
    ymin=ymax;
    ymax=tmp;
  }
  fCanvasEye->Range(xmin,ymin,xmax,ymax);
  Update();
}

/*-----------------------------------------------------------------------------*/
int EDEyeEmbeddedCanvas::DrawEye(TEyeEvent *ev) {
  fCanvasEye->cd();
  // if (!ev), call for cleaning only
  if (!ev) {
    fCanvasEye->SetEditable(kTRUE);
    for (int i=1;i<FdUtil::Fd::kEYE_NEXTPIXELS+1;i++) {
      if (fPixel[i]) {
        delete fPixel[i];
        fPixel[i]=NULL;
      }
    }
    if (fSDP) delete fSDP;
    fSDP=NULL;
    Update();
    fCanvasEye->SetEditable(kFALSE);
    return 0;
  }
  // if called with ev but not good eye, do nothing
  TEyeEventHeader *header = ev->GetEventHeader();
  if ((int)header->GetEyeNo()!=fEye) {
    return 0;
  }
  fEvent=ev;
  // ok. First cleaning canvas
  DrawEye(NULL);
  // Then drawing it
  fCanvasEye->SetEditable(kTRUE);
  int viewang=0;
  for (unsigned int j=0; j<ev->GetPixelList()->GetNumPixels(); j++ ) {
    int i=FdUtil::Fd::GetEyePixelNo(ev->GetPixelList()->GetPixel(j));
    int o=FdUtil::Fd::GetMirrorPixelNo(ev->GetPixelList()->GetPixel(j));
    if (o>440) continue;
    // a true pixel
    if (!fPixel[i]) fPixel[i]=NewPixel(i);
    fPixel[i]->Draw();
    // Pedestal calculation
    int x=0,x2=0,n=230;
    for (int k=0; k<n; k++ ) {
      x2+=FDADC(k)*FDADC(k);
      x+=FDADC(k);
    }
    double sigma=sqrt((1/(n-1.))*(x2-x*(x/n)));
    double ped=x/n;
    // removing 3 sigma fluctuations
    for (int k=0; k<230; k++) {
      if ((FDADC(k)-ped)*(FDADC(k)-ped) > 9*sigma*sigma) {
        n-- ;
        x -= FDADC(k) ;
        x2 -= FDADC(k)*FDADC(k) ;
      }
    }
    sigma=sqrt((1/(n-1.))*(x2-x*(x/n)));
    ped=x/n;
    fPixel[i]->SetFillColor(5);
    // search for pixel with 2 consecutive bins at 3 sigma
    int ok=0;
    double maxs=1;
    for (int k=0; k<1000; k++) {
      if ((FDADC(k)-ped)/sigma>maxs) maxs=(FDADC(k)-ped)/sigma;
    }
    fPixel[i]->SetR1(0.13*log(maxs));
    fPixel[i]->SetR2(0.13*log(maxs));

    for (int k=0; k<1000; k++) {
      if ((FDADC(k)-ped)>3*sigma && ok) {
        fPixel[i]->SetFillColor(3);
        viewang=i;
        break;
      }
      if (ok) ok--;
      if ((FDADC(k)-ped)>3*sigma) ok=1;
    }
    if (!viewang) viewang=i;
  }
  fCanvasEye->SetEditable(kFALSE);
  Update();
  return viewang;
}

void EDEyeEmbeddedCanvas::SetSDP (double a, double b, double c) {
  double n=sqrt(a*a+b*b+c*c);
  if (c<0) n*=-1;
  double A=a/n;
  double B=b/n;
  double C=c/n;
  if (!fSDP) fSDP=new TGraph(100);
  double x,y,X,Y;
  int i=0;
  for (x=fXmin;x<fXmin+200;x+=2) {
    y=kCdasUtil::RAD2DEG*atan2(B*sin(x*kCdasUtil::DEG2RAD)-A*cos(x*kCdasUtil::DEG2RAD),C);
    X=x*cos(fRot)+y*sin(fRot);
    Y=y*cos(fRot)-x*sin(fRot);
    fSDP->SetPoint(i++,X,Y);
  }
}

/*-----------------------------------------------------------------------------*/
/*
void EDEyeEmbeddedCanvas::NewSDP () {
  if (fSDP2)
    return;
  double xmin=fXmin*cos(fRot)+0*sin(fRot);
  double ymin=0*cos(fRot)-fXmin*sin(fRot);
  double xmax=(fXmin+200)*cos(fRot)+35*sin(fRot);
  double ymax=35*cos(fRot)-(fXmin+200)*sin(fRot);
  double tmp=0;
  if (xmin>xmax) {
    tmp=xmin;
    xmin=xmax;
    xmax=tmp;
  }
  if (ymin>ymax) {
    tmp=ymin;
    ymin=ymax;
    ymax=tmp;
  }
  char buf[10],buf2[1024];
  snprintf(buf,10,"SDP %d",fEye);
  // In case it is rotated, it becomes more complicated
  if (fabs(fRot)<0.1) // basic formula
    snprintf(buf2,1024,"%lf*atan2([1]*sin(x*%lf)-[0]*cos(x*%lf),[2])-y",kCdasUtil::RAD2DEG,kCdasUtil::DEG2RAD,kCdasUtil::DEG2RAD);
  else if (fabs(fRot-0.5*M_PI)<0.1) // x -> y, y -> -x (with x=-x first and last) => x -> -y, y -> x
    snprintf(buf2,1024,"%lf*atan2([1]*sin(-y*%lf)-[0]*cos(-y*%lf),[2])-x",kCdasUtil::RAD2DEG,kCdasUtil::DEG2RAD,kCdasUtil::DEG2RAD);
  else if (fabs(fRot-M_PI)<0.1) // x -> -x, y -> -y
    snprintf(buf2,1024,"%lf*atan2([1]*sin(-x*%lf)-[0]*cos(-x*%lf),[2])+y",kCdasUtil::RAD2DEG,kCdasUtil::DEG2RAD,kCdasUtil::DEG2RAD);
  else // x -> y, y -> -x
    snprintf(buf2,1024,"%lf*atan2([1]*sin(y*%lf)-[0]*cos(y*%lf),[2])+x",kCdasUtil::RAD2DEG,kCdasUtil::DEG2RAD,kCdasUtil::DEG2RAD);
  fSDP2=new TF2(buf,buf2,xmin,xmax,ymin,ymax);
  double contour=0.;
  fSDP2->SetContour(1,&contour);
  fSDP2->SetNpx(400);
  fSDP2->SetNpy(70);
  fSDP2->SetLineWidth(2);
}
*/

/*-----------------------------------------------------------------------------*/
TEllipse *EDEyeEmbeddedCanvas::NewPixel (int i) {
  double alt0=gFDTubeDir.Alt(fEye,i);
  double az0=-gFDTubeDir.Az(fEye,i); // WARNING, x axis is -Azimuth
  double az=az0*cos(fRot)+alt0*sin(fRot);
  double alt=alt0*cos(fRot)-az0*sin(fRot);
  TEllipse *e=new TEllipse(az,alt,0.6,0.6,0,360,0);
  e->SetLineColor(9);
  e->SetFillColor(10);
  return e;
}

/*-----------------------------------------------------------------------------*/
EDEyeCanvas::EDEyeCanvas(const TGWindow *p, int eye,TGTab * p2) : TGVerticalFrame(p) {
  gClient->GetColorByName("AntiqueWhite2",fColorData);
  gClient->GetColorByName("AntiqueWhite3",fColorNone);
  fTab=p2;
  fEye=eye;
  fSliding=0;
  string names[5];
  names[1]="Los Leones";
  names[2]="Los Morados";
  names[3]="Loma Amarilla";
  names[4]="Coihueco";
  TGLayoutHints *fMax = new TGLayoutHints(kLHintsTop | kLHintsExpandX | kLHintsExpandY, 1, 1, 1, 0);
  TGLayoutHints *fMaxX = new TGLayoutHints(kLHintsTop | kLHintsExpandX, 1, 1, 1, 0);
  fControl=new TGHorizontalFrame(this,100,30,kChildFrame,fgDefaultFrameBackground);
  this->AddFrame(fControl);
  fMultiviewButton= new TGCheckButton(fControl, "Multi View",2);
  fMultiviewButton->Associate(this);
  fMultiviewButton->ChangeBackground(fColorNone);
  fControl->AddFrame(fMultiviewButton);
  fPixelFullRange= new TGCheckButton(fControl, "Zoomed view",1);
  fPixelFullRange->Associate(this);
  fPixelFullRange->ChangeBackground(fColorNone);
  fControl->AddFrame(fPixelFullRange);
  fPixelFullRange->SetState(kButtonDown);
  fFDPixelCanvas = new FDPixelCanvas("Pixel Trace",this,this,500,200);
  this->AddFrame(fFDPixelCanvas,fMaxX);
  fEDEyeEmbeddedCanvas= new EDEyeEmbeddedCanvas(names[eye].c_str(),this,500,200,eye,0,1);
  this->AddFrame(fEDEyeEmbeddedCanvas,fMax);
  fEDEyeEmbeddedCanvas->SetPixelCanvas(fFDPixelCanvas);
  fSlide=new TGHSlider(this,100,kSlider2,eye);
  fSlide->SetRange(0,SLIDEMAX);
  fSlide->SetPosition(0);
  fSlide->ChangeBackground(fColorData);
  this->AddFrame(fSlide,fMaxX);
  this->ChangeBackground(fColorNone);
  fSlide->ChangeBackground(fColorNone);
  SetView(0);
  DrawEye(NULL);
}

/*-----------------------------------------------------------------------------*/
int EDEyeCanvas::DrawEye(TEyeEvent *ev) {
  // if (!ev), call for cleaning only
  if (!ev) {
    fEDEyeEmbeddedCanvas->DrawEye(NULL);
    ChangeColor(fColorNone);
    return 0;
  }
  // if called with ev but not good eye, do nothing
  TEyeEventHeader *header = ev->GetEventHeader();
  if ((int)header->GetEyeNo()!=fEye) return 0;
  // ok. Draw it
  int goodpix=fEDEyeEmbeddedCanvas->DrawEye(ev);
  ChangeColor(fColorData);
  fTab->SetTab(fEye-1);
  double viewang=-gFDTubeDir.Az(fEye,goodpix)-fEDEyeEmbeddedCanvas->fXmin; // X axis = -Azimuth
  fSlide->SetPosition((int)viewang-25);
  SetView(viewang-25);
  return 1;
}

void EDEyeCanvas::ChangeColor(ULong_t color) {
  //this->ChangeBackground(color);
  //fSlide->ChangeBackground(color);
  fTab->GetTabTab(fEye-1)->ChangeBackground(color);
}

int EDEyeEmbeddedCanvas::DrawShower(TShowerParams *sp) {
  fCanvasEye->cd();
  // if (!sp), call for cleaning only
  if (!sp) {
    if (fSDP)
      delete fSDP;
    fSDP=NULL;
    Update();
    return 0;
  }
  fCanvasEye->SetEditable(kTRUE);
  double xsh,ysh,zsh;  // cosines of shower direction
  double xce,yce,zce;  // cosines of core-eye direction
  xsh=cos(sp->Phi())*sin(sp->Theta());
  ysh=sin(sp->Phi())*sin(sp->Theta());
  zsh=cos(sp->Theta());
  xce=sp->fXCore-kIoSd::FdX(fEye);
  yce=sp->fYCore-kIoSd::FdY(fEye);
  zce=sp->fZCore-kIoSd::FdZ(fEye);
  double a,b,c;         // normal to SDP
  a=ysh*zce-zsh*yce;
  b=zsh*xce-xsh*zce;
  c=xsh*yce-ysh*xce;
  double n=sqrt(a*a+b*b+c*c);
  //NewSDP();
  //if (c>0) fSDP2->SetParameters(a/n,b/n,c/n);
  //else fSDP2->SetParameters(-a/n,-b/n,-c/n);
  SetSDP(a/n,b/n,c/n);
  //fSDP2->Draw("SAME");
  fSDP->Draw();
  fCanvasEye->SetEditable(kFALSE);
  Update();
  return 1;
}
