#ifdef FD
// Hybrid display, adapted for ED by Xavier Bertou. Original version:
// ROOT macro for geometrical analysis of FD data by Martina Bohacova
// needs Pixel.h, LoadData.cc, LinFit.C, tube_dir.dat, tank-coors.txt
// and data files
//version of 30th Oct 2002, drawing only the 3D reconstruction

#include "EDHybrid.h"
#include "MiEvent.hh"

#include <iostream>
#include <fstream>

#include "TROOT.h"
#include "TH1.h"
#include "TF1.h"
#include "TCanvas.h"
#include "TGraphErrors.h"
#include "TNode.h"
#include "TView.h"

int TEDHybrid::ReadCam() {
  int err;
  char file[512];
  sprintf(file,"%s/%s%c",EDSRCDIR,"FDtube_dir.dat",0);
  if(( err = ReadTube( file, cam )) < 0 ) {
    cout << "Error in reading tube directions! Exit. " << err << endl;
    exit( -1 );
  };
  return 1;
}

TEDHybrid::TEDHybrid() {
  // Reading tube direction information for LL camera
  ReadCam();

  // building 3D objects
  LL = new TTUBS ("LL","LL","void", 0.8,0.2,330,150);
  LL->SetNumberOfDivisions(6);
  LL->SetLineColor(4);
  LL->SetLineWidth(2);

  COI = new TTUBS ("COI","COI","void", 0.8,0.2,240,60);
  COI->SetNumberOfDivisions(6);
  COI->SetLineColor(3);
  COI->SetLineWidth(2);

  bay4LL=new TSPHE("bay4LL","bay4LL","void",0, 20,58,88,60,90);
  bay4LL->SetNumberOfDivisions(3);
  bay4LL->SetLineColor(40);
  bay4LL->SetLineWidth(2);
  bay5LL=new TSPHE("bay5LL","bay5LL","void",0, 20,58,88,90,120);
  bay5LL->SetNumberOfDivisions(3);
  bay5LL->SetLineColor(33);
  bay5LL->SetLineWidth(2);

  xbay4LL=new TSPHE("xbay4LL","xbay4LL","void",19.9, 20,58,88,60,90);
  xbay4LL->SetLineColor(4);
  xbay4LL->SetNumberOfDivisions(22);
  xbay5LL=new TSPHE("xbay5LL","xbay5LL","void",19.9, 20,58,88,90,120);
  xbay5LL->SetLineColor(4);
  xbay5LL->SetNumberOfDivisions(22);

  LLseg = new TTUBE ("LLseg","LLseg","void", 0.05,10);
  LLseg->SetLineColor(4);

  COIseg = new TTUBE ("COIseg","COIseg","void", 0.05,10);
  COIseg->SetLineColor(3);

  Shower=new TSPHE("Shower","Shower","void",0.2);
  Shower->SetLineColor(2);
  CameraPixel=new TSPHE("CameraPixel","CameraPixel","void",0.2);
  CameraPixel->SetLineColor(5);
  ImpactPoint=new TSPHE("ImpactPoint","ImpactPoint","void",0.2);
  ImpactPoint->SetLineColor(3);

  Sdoff = new TTUBE ("Sdoff","Sdoff","void", 0.35,0.18);
  Sdoff->SetLineColor(1);
  Sdon = new TTUBE ("Sdon","Sdon","void", 0.35,0.18);
  Sdon->SetLineColor(7);
  Sdbug = new TTUBE ("Sdbug","Sdbug","void", 0.35,0.18);
  Sdbug->SetLineColor(2);
  Sdt1 = new TTUBE ("Sdt1","Sdt1","void", 0.35,0.18);
  Sdt1->SetLineColor(5);
  Sdt2 = new TTUBE ("Sdt2","Sdt2","void", 0.35,0.18);
  Sdt2->SetLineColor(3);
  SdN = new TTUBE ("SdN","SdN","void", 0.2,0.1);
  SdN->SetLineColor(41);


  empty = new TF2("empty","0*x*y",-13,10, 0, 20);
  empty->SetMaximum(10);

  Sdp=NULL;
  ShowerPoints=HitTanks=OffTanks=NULL;
}

int TEDHybrid::FindEvent() {
  struct MujHeader Header;
  char File[512];
  int Seek;

  if (!GetFile(File,&Seek,Event->Trigger.Second,Event->Trigger.MicroSecond*1000))
    return 0;
  int StartSlice = 20;             //interval for looking for fired pixels-
  int EndSlice = 50;               //eliminates singles...
  NPixEvent = LoadData( Seek, File, StartSlice, EndSlice, &PixArray,&Header,cam);
  cout << "Data loaded... " << NPixEvent << " pixels" << endl;
  Bay = Header.MirrNum;
  return 1;
}

int TEDHybrid::Reconstruct() {

  float CutOnAmpl = 3.5;          // cut on amplitude of peaks (in sigma)
  int N_BIN_TOT = 1000;           // number of bins recorded
  Int_t N_BINS = 40;              // number of bins around a peak to consider

  MujPixel **DataSmooth = DuplPixArray( PixArray, NPixEvent );
  MujPixel **NewPixArray = (MujPixel **) malloc( sizeof( MujPixel *) * NPixEvent);
  MujPixel **FinalPixArray = (MujPixel **) malloc( sizeof( MujPixel *) * NPixEvent);

  int new_N_PIX = 0;
  NPixFinal = 0;

  char  tlab[20];
  char  sdp_fnc[100];
  char  ti_fit[100];
  Double_t  Rp, T0, Chi0, Ro, theta_n, fi_n;

  float Dr = 3.141592654/180;


  //////////////////////////////////////////////////////////////////////////////
  ////////////****   FITTING THE FADC DATA - PEAK FINDING  *******//////////////
  //////////////////////////////////////////////////////////////////////////////

  TF1 mygaus("mygaus","[0]*exp(-0.5*((x-[1])/[2])*((x-[1])/[2]))",18,80);
  TMinuit *minuit=new TMinuit(15); // BECAUSE OF !@$#$@#$! ROOT

  ///////////////////// loop over pixels ///////////////////////////////////////
  for (Int_t j = 0; j < NPixEvent; j++)
    cout << "o";
  cout << endl;
  for (Int_t j = 0; j < NPixEvent; j++) {

    sprintf(tlab,"pixel ( %d, %d )", PixArray[j]->ColNo,PixArray[j]->RowNo);

    ///////////////////// loop over bins of each pixel ///////////////////

    float FADC_MAX1 =0;       //max. found before smoothing
    int FADC_MAX_TIME1 =0;
    float FADC_MAX2 =0;       //max. found after smoothing
    int FADC_MAX_TIME2 =0;
    float OFFSET =0;
    float SIGMA2 =0;
    float SIGMA = 0;

    for (Int_t k = 0; k < N_BIN_TOT; k++) {
      if ( k>=500 ) {
        OFFSET += PixArray[j]->Fadcdata[k] ;
        SIGMA2 += PixArray[j]->Fadcdata[k]*PixArray[j]->Fadcdata[k];
      }
      if ( k>=200 && k<500 ) {
        if ( PixArray[j]->Fadcdata[k] > FADC_MAX1 ) {
          FADC_MAX1 = PixArray[j]->Fadcdata[k] ;
          FADC_MAX_TIME1 = k;
        }
      }
    }
    OFFSET /= 500;
    SIGMA = sqrt(SIGMA2/500 - OFFSET*OFFSET);
    PixArray[j]->Offset = OFFSET;

    //////////////////////// smooth data /////////////////////////////////

    for (Int_t k = 0; k < N_BIN_TOT-4; k++) {
      float Max1 = PixArray[j]->Fadcdata[k]-OFFSET, Max2 = 0, Med = 0 ;
      for (Int_t i = 1; i< 5 ; i++) {
        if ( PixArray[j]->Fadcdata[k+i]-OFFSET	> Max1 ) {
          Med  = Max2;
          Max2 = Max1;
          Max1 = PixArray[j]->Fadcdata[k+i]-OFFSET;
        } else if (  PixArray[j]->Fadcdata[k+i]-OFFSET >  Max2 ) {
          Med  = Max2;
          Max2 = PixArray[j]->Fadcdata[k+i]-OFFSET;
        } else if (  PixArray[j]->Fadcdata[k+i]-OFFSET >  Med ) {
          Med  = PixArray[j]->Fadcdata[k+i]-OFFSET;
        }

      }
      DataSmooth[j]->Fadcdata[k+1] = Med;
      if ( Med > FADC_MAX2 ) {
        FADC_MAX2 = Med;
        FADC_MAX_TIME2 = k;
      }
    }
    DataSmooth[j]->Fadcdata[0] =DataSmooth[j]->Fadcdata[1]	= DataSmooth[j]->Fadcdata[2];
    DataSmooth[j]->Fadcdata[N_BIN_TOT-1]=DataSmooth[j]->Fadcdata[N_BIN_TOT-2] = DataSmooth[j]->Fadcdata[N_BIN_TOT-3];

    TH1D fadchi2("fadchi2",tlab, N_BINS,(FADC_MAX_TIME2-N_BINS/2)*0.1,(FADC_MAX_TIME2+N_BINS/2)*0.1 );

    for (Int_t k = 0; k < N_BINS; k++) {
      fadchi2.SetBinContent(k+1,
                            k + FADC_MAX_TIME2-N_BINS/2 >= 0 ?
                            DataSmooth[j]->Fadcdata[k+FADC_MAX_TIME2-N_BINS/2] : 0.);
    }

    mygaus.SetParameters(30,FADC_MAX_TIME2*0.1,0.1);
    fadchi2.Fit("mygaus","+Q0 ","same");

    ////////////////////////////////////////////////////////////////////////////////
    /////////////////////************ SIGNAL FIT ***********//////////////////////
    //////////////////////////////////////////////////////////////////////////////

    TH1D fadchi("fadchi",tlab, N_BINS,(FADC_MAX_TIME2-N_BINS/2)*0.1,(FADC_MAX_TIME2+N_BINS/2)*0.1 );

    for (Int_t k = 0; k < N_BINS; k++) {

      fadchi.SetBinContent(k+1, //PixArray[j]->Fadcdata[k+FADC_MAX_TIME2-N_BINS/2]-OFFSET);
                           k + FADC_MAX_TIME2-N_BINS/2 >= 0 ?
                           PixArray[j]->Fadcdata[k+FADC_MAX_TIME2-N_BINS/2]-OFFSET : 0.);
    }

    TF1 *fit = fadchi2.GetFunction("mygaus");
    Double_t Const =fit->GetParameter(0);
    Double_t Mean =fit->GetParameter(1);
    Double_t Sigma =fit->GetParameter(2);

    mygaus.SetParameters(Const,Mean,Sigma);
    mygaus.SetParLimits(1,0,65);
    mygaus.SetParName(0,"Const");
    mygaus.SetParName(1,"Mean");
    mygaus.SetParName(2,"Sigma");
    fadchi.Fit("mygaus","+Q0","same");


    TF1 *fit2 = fadchi.GetFunction("mygaus");

    if ( fit2->GetParameter(0) > CutOnAmpl*SIGMA && fabs(fit2->GetParameter(2)) >
         0.1 && fit2->GetParameter(1) > 10) {
      NewPixArray[ new_N_PIX  ] = new MujPixel( *PixArray[j] );

      int from, to;

      from = (int)ceil(10.*(fit2->GetParameter(1) - 3*fit2->GetParameter(2))
                       - (FADC_MAX_TIME2 - N_BINS/2 ));
      if( from < 0)
        from=0;
      to = (int)ceil(10.*(fit2->GetParameter(1) + 3*fit2->GetParameter(2))
                     - (FADC_MAX_TIME2-N_BINS/2 ));
      if ( to > N_BINS )
        to = N_BINS;

      NewPixArray[new_N_PIX]->Area = fadchi.Integral( from, to, "" );
      NewPixArray[new_N_PIX]->Ti = fit2->GetParameter(1);
      NewPixArray[new_N_PIX]->Ampl = fit2->GetParameter(0);
      NewPixArray[new_N_PIX]->Sigma =fit2->GetParameter(2);
      NewPixArray[new_N_PIX]->TiErr = fit2->GetParError(1);

      new_N_PIX++;
      cout << "+";
    }	else
    cout << "-";
    cout.flush();
  }         /// end of j-loop (pixels)
  cout << endl;
  if (new_N_PIX < 3 ) {
    cout << new_N_PIX << "... Not enough pixels to reconstruct...exit" << endl;
    delete minuit;
    return 0;
  }
  //////////////////////////////////////////////////////////////////////////////
  /////////////******** FIRST FIT TIME VS ELEVATION **********//////////////////
  //////////////////////////////////////////////////////////////////////////////

  float _Titmp[NPixEvent];
  float _Elev[NPixEvent];
  float _ErrBox[NPixEvent];
  float _W[NPixEvent];

  for (Int_t j=0; j < new_N_PIX; j++) {
    _Titmp[j] = NewPixArray[j]->Ti;
    _Elev[j] =90 - NewPixArray[j]->Elev;
    _ErrBox[j] = 1/sqrt(NewPixArray[j]->NeibrNo+1) + 1/NewPixArray[j]->Ampl  ;
    _W[j] = 1/_ErrBox[j];
  }

  float a, b;

  LinFit( new_N_PIX, _Elev, _Titmp, _W, a, b );

  //////////////////////////////////////////////////////////////////////////////
  ////////////////******** CUT ON DIST. FROM THE CORE *******///////////////////
  //////////////////////////////////////////////////////////////////////////////

  float tmp[NPixEvent];
  float FirstPixTi=1000, LastPixTi=0;
  int FirstPix, LastPix;

  for (Int_t j=0; j < new_N_PIX; j++) {
    tmp[j] = fabs(NewPixArray[j]->Ti - a*(90 - NewPixArray[j]->Elev) -b);

    if ( tmp[j]<4 ) {
      FinalPixArray[ NPixFinal  ] = new MujPixel( *NewPixArray[j] );
      NPixFinal ++;
    }
  }
  cout << "NPixFinal " << NPixFinal << endl;
  for (Int_t j=0; j < NPixFinal; j++) {
    if (FinalPixArray[j]->Ti > LastPixTi ) {
      LastPixTi = FinalPixArray[j]->Ti;
      LastPix = j;
    }
    if (FinalPixArray[j]->Ti < FirstPixTi ) {
      FirstPixTi = FinalPixArray[j]->Ti;
      FirstPix = j;
    }
  }

  //////////////////////////////////////////////////////////////////////////////
  ///////////////////////************* SDP************//////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  float VxAux[NPixEvent], VyAux[NPixEvent], VzAux[NPixEvent], ErrPix[NPixEvent];

  for (Int_t j=0; j < NPixFinal; j++) {
    VxAux[j] = 1/(tan((FinalPixArray[j]->Azim)*Dr));
    VyAux[j] = 1;
    VzAux[j] = tan((90 - FinalPixArray[j]->Elev)*Dr)/sin((FinalPixArray[j]->Azim)*Dr);
    ErrPix[j] = 0.1*sqrt(1/FinalPixArray[j]->Ampl);
  }

  TGraphErrors sdp(NPixFinal, VxAux ,VzAux, ErrPix, ErrPix);
  sdp.Fit("pol1","+Q0 ","same");
  TF1 *sdpfit = sdp.GetFunction("pol1");

  // azimuth and zenith angle of SDP normal vector
  fi_n = atan(sdpfit->GetParameter(0)/sdpfit->GetParameter(1))/Dr;
  theta_n = atan( fabs(sdpfit->GetParameter(1))/ cos( fi_n*Dr ))/Dr;

  if( sdpfit->GetParameter(1) > 0 )
    fi_n += 180.;
  if( VxAux[LastPix] - VxAux[FirstPix]
      + sdpfit->GetParameter(1)*(VzAux[LastPix] - VzAux[FirstPix]) > 0 ) {
    // flip the SDP normal vector
    fi_n -= 180.;
    theta_n = 180. - theta_n;
  }
  if( fi_n < 0. )
    fi_n += 360.;

  ///// SDP coordinates////////////////

  float Nx, Ny, Nz;

  Nx = cos(fi_n*Dr) * sin(theta_n*Dr);
  Ny = sin(fi_n*Dr) * sin(theta_n*Dr);
  Nz = cos(theta_n*Dr);

  //////////////////////////////////////////////////////////////////////////////
  ////////////////************** FIT OF TANGENT ***************/////////////////
  //////////////////////////////////////////////////////////////////////////////

  float _TiErr[NPixEvent];
  float _XiErr[NPixEvent];
  float _Ti[NPixEvent];
  float _Xi[NPixEvent];
  float _Xin[NPixEvent], _Yin[NPixEvent];

  for (Int_t j=0; j < NPixFinal; j++) {
    float __elev = (90.-FinalPixArray[j]->Elev)*Dr;
    float __azim = (FinalPixArray[j]->Azim - fi_n -90)*Dr;
    _Xi[j] = asin(sin((90.-FinalPixArray[j]->Elev)*Dr)/sin(theta_n*Dr) )/Dr;
    _Xin[j] = acos(cos(__elev)*fabs(cos(__azim)))/Dr;
    _Yin[j] = (sin( __elev)*cos(theta_n*Dr) - cos(__elev)*sin(__azim)*sin(theta_n*Dr))/Dr;
    _Ti[j] = FinalPixArray[j]->Ti;
    _TiErr[j] = FinalPixArray[j]->TiErr + 0.1;
    _XiErr[j ] = 0.1;
  }

  TGraphErrors shower(NPixFinal, _Xin, _Ti, _XiErr, _TiErr);
  sprintf(ti_fit,"[0] +[1]/0.3/tan([2]*%f/2-x*%f/2)",Dr,Dr);
  TF1 tan_fit("tan_fit",ti_fit,0,67);
  tan_fit.SetParameters(-10,12,90);
  tan_fit.SetLineColor(46);
  shower.Fit("tan_fit","+RQ0 ","same");
  shower.Fit("pol1","+Q0 ","same");
  TF1 *tanfit = shower.GetFunction("tan_fit");
  T0 = tanfit->GetParameter(0);
  Rp = fabs(tanfit->GetParameter(1));
  Chi0 = tanfit->GetParameter(2) - 180.;
  if( Chi0 < -180. )
    Chi0 += 360.;

  float Theta_s, Fi_s, Chi0_s = Chi0, theta_ns = theta_n;
  if( Chi0 < 0 ) {
    Chi0_s += 180.;
    theta_ns = 180. - theta_n;
  }
  int inc_s = ( fabs( fi_n - 180.) < 90. ) ? 1 : -1;
  if( Chi0_s < 90. )
    inc_s = -inc_s;

  Theta_s = acos( sin(theta_ns*Dr)*sin(Chi0_s*Dr))/Dr;
  Fi_s = fi_n + 90.*inc_s - atan( cos( theta_ns*Dr )*tan(Chi0_s*Dr ))/Dr;
  if( Chi0 < 0 ) {  // Laser; flip the direction
    Theta_s = 180. - Theta_s;
    Fi_s += 180.;
  }
  Fi_s = Fi_s - 360.*(floor( Fi_s / 360.));

  cout << "T0= " << T0 <<endl;
  cout << "Rp= " << Rp <<endl;
  cout << "Chi0= " << Chi0 <<endl;
  cout << "Theta= " << theta_n <<endl;
  cout << "Phi= " << fi_n <<endl;
  cout << "shower Phi = " << Fi_s << endl;
  cout << "shower Theta = " << Theta_s << endl;

  //////////////////////////////////////////////////////////////////////////////
  //////////////************ POINTS OF THE SHOWER **************////////////////
  //////////////////////////////////////////////////////////////////////////////

  for (Int_t j=0; j < NPixFinal; j++) {
    vy[j] = cos( (90 - FinalPixArray[j]->Elev)*Dr)*sin((FinalPixArray[j]->Azim)*Dr);
    vx[j] = cos( (90 - FinalPixArray[j]->Elev)*Dr)*cos((FinalPixArray[j]->Azim)*Dr);
    vz[j] = sin( (90 - FinalPixArray[j]->Elev)*Dr);

    Ri[j] = Rp / fabs(sin( Chi0*Dr - asin( vz[j]/sin( theta_n*Dr ))));
  }

  Ro = Rp / sin( Chi0*Dr ); ///impact point
  ImpX = Ro*cos((fi_n-90)*Dr);
  ImpY = Ro*sin((fi_n-90)*Dr);
  ImpZ = 0;

  //////////////////////////////////////////////////////////////////////////////
  ////////                   Sdp function                  /////////////////////
  //////////////////////////////////////////////////////////////////////////////


  sprintf(sdp_fnc,"-(%f/%f)*x-(%f/%f)*y ", Nx,Nz,Ny,Nz);      // <<<<--- normal vector SDP

  if (Sdp)
    delete Sdp;
  Sdp = new TF2("sdp",sdp_fnc,-10,10, 0, 20);
  Sdp->SetMinimum(0);
  Sdp->SetMaximum(10);
  Sdp->SetTitle("");
  delete minuit;

  return 1;
}

int TEDHybrid::DrawTanks() {
  float xt,yt,zt;
  int l=0,r=0;
  //if (HitTanks) delete HitTanks; // done automatically on delete canvas
  //if (OffTanks) delete OffTanks;
  unsigned int size = Event->Trigger.NumberOfStation;

  HitTanks = new TPolyMarker3D(size);
  OffTanks = new TPolyMarker3D(size);

  for (unsigned int i=0;i<size;i++) {
    xt=(Event->Stations[i].Easting-kIoSd::E_LL)/1000;
    yt=(Event->Stations[i].Northing-kIoSd::N_LL)/1000;
    zt=0.1;
    TNode *inode;
    if (X3D) {
      if (Event->Stations[i].Error==0 && Event->Stations[i].Trigger.Window==0)
        inode  = new TNode("tanknode","tanknode","Sdt2",xt,yt,zt);
      else if (Event->Stations[i].Error==0)
        inode  = new TNode("tanknode","tanknode","Sdt1",xt,yt,zt);
      else if (Event->Stations[i].Trigger.Window==0)
        inode  = new TNode("tanknode","tanknode","Sdbug",xt,yt,zt);
      else if (Event->Stations[i].Error==2)
        inode  = new TNode("tanknode","tanknode","Sdon",xt,yt,zt);
      else
        inode  = new TNode("tanknode","tanknode","Sdoff",xt,yt,zt);
    } else {
      inode  = new TNode("tank","tank","SdN",xt,yt,zt);
      if (Event->Stations[i].Error==0)
        HitTanks->SetPoint( l++, xt, yt, zt);
      else if (Event->Stations[i].Error!=2)
        OffTanks->SetPoint( r++, xt, yt, zt);
    }
  }
  if (X3D)
    return 1;

  HitTanks->SetMarkerSize(1.5);
  HitTanks->SetMarkerStyle( 4 );
  HitTanks->SetMarkerColor(2);
  HitTanks->Draw("same");

  OffTanks->SetMarkerSize(1.);
  OffTanks->SetMarkerStyle( 5 );
  OffTanks->SetMarkerColor(1);
  OffTanks->Draw("same");

  return 1;
}

int TEDHybrid::InitPad() {
  if (Pad && gROOT->FindObject("Pad"))
    delete Pad;
  Pad = new TPad("Pad","main pad",0.,0.,1.,1.);
  Pad->Draw();
  Pad->cd();
  return 1;
}

int TEDHybrid::InitX3D() {
  empty->Draw("surf4");
  Pad->Update();
  TNode *node1;
  node1 = new TNode("NODEX","NODEX","LL");
  node1->cd();
  node1->Draw("same");
  if (Bay==4)
    node1 = new TNode("NODEX","NODEX","xbay4LL",0,0);
  if (Bay==5)
    node1 = new TNode("NODEX","NODEX","xbay5LL",0,0);
  double xc=(kIoSd::E_CO-kIoSd::E_LL)/1000;
  double yc=(kIoSd::N_CO-kIoSd::N_LL)/1000;
  node1 = new TNode("NODEX","NODEX","COI",xc,yc);
  // Should draw the bays
  return 1;
}

int TEDHybrid::InitN3D() {
  Sdp->Draw("surf4");
  Pad->Update();
  TNode *node1;
  node1 = new TNode("NODEN","NODEN","bay4LL",0,0);
  node1->cd();
  node1->Draw("same");
  node1 = new TNode("NODEN","NODEN","bay5LL",0,0);
  node1 = new TNode("NODEN","NODEN","LL"); // LL last so that it overlaps the bays
  return 1;
}

int TEDHybrid::DrawShower() {
  TNode *node1;
  node1 = new TNode("NODEI","NODEI","ImpactPoint",ImpX,ImpY,ImpZ);
  if (X3D) {
    for (Int_t j=0; j < NPixFinal; j++) {
      node1=new TNode("NODESH","NODESH","CameraPixel", 19.9*vx[j], 19.9*vy[j], 19.9*vz[j]);
      node1=new TNode("NODESH","NODESH","Shower", Ri[j]*vx[j], Ri[j]*vy[j], Ri[j]*vz[j]);
    }
    return 1;
  }
  ShowerPoints = new TPolyMarker3D(NPixFinal);
  for (Int_t j=0; j < NPixFinal; j++)
    ShowerPoints->SetPoint( j, Ri[j]*vx[j], Ri[j]*vy[j], Ri[j]*vz[j]);
  ShowerPoints->SetMarkerColor(4);
  ShowerPoints->SetMarkerStyle( 4 );
  ShowerPoints->SetMarkerSize( 1 );
  ShowerPoints->Draw("same");

  SDLine=new TPolyLine3D(2);
  SDLine->SetPoint(0,(Event->xCore-kIoSd::E_LL)/1000,(Event->yCore-kIoSd::N_LL)/1000,0);
  SDLine->SetPoint(1,(Event->xCore-kIoSd::E_LL)/1000+10*tan(Event->theta)*cos(Event->phi),(Event->yCore-kIoSd::N_LL)/1000+10*tan(Event->theta)*sin(Event->phi),10);
  SDLine->SetLineWidth(2);
  SDLine->SetLineColor(2);
  SDLine->Draw("same");
  SDCore = new TPolyMarker3D(1);
  SDCore->SetPoint(0,(Event->xCore-kIoSd::E_LL)/1000,(Event->yCore-kIoSd::N_LL)/1000,0);
  SDCore->SetMarkerSize(1);
  SDCore->SetMarkerStyle(20);
  SDCore->SetMarkerColor(2);
  SDCore->Draw("same");
  return 1;
}


int TEDHybrid::Draw(SdEvent *event, int x3d) {
  // affecting local variables
  X3D=x3d;
  Event=event;

  if (!FindEvent())
    return 0;

  if (!Reconstruct())
    return 0;

  InitPad();
  if (X3D)
    InitX3D();
  else
    InitN3D();

  DrawTanks();
  DrawShower();

  if (X3D)
    Pad->x3d();

  return 1;
};

#endif
