#include "EDMainFrame.h"
#include "ED.h"
#include "EDDoc.h"
#include "EDShowerAnim.h"
#include "EDSelection.h"
#include "TColor.h"
#include "TImage.h"
#include "TStyle.h"
#include "TSystem.h"
#include "SdRecLpnhe.h"
#include "TCdFReconstruct.h"
#include "PBRec.h"
#ifdef FD
#include "FD.h"
#endif

#include <sstream>
#include "KeySymbols.h"

#define DEFMIN 999999

std::vector <RFile> gRootfiles;

std::string ErrorMessages[8]={"No Error","T3 Lost","T3 Not Found","T3 Too Young","T3 Already","T1 stopped","Bad compressed data","Data Lost"};

int nstation_from_Db=0;
int veryfirst=1;

class TMyGGroupFrame : public TGGroupFrame {
public:
  EDMainFrame * fMain;
  TMyGGroupFrame(EDMainFrame* p2 = 0,const TGWindow* p = 0, const char* title = "0", UInt_t options = kVerticalFrame, GContext_t norm = GetDefaultGC()(), FontStruct_t font = GetDefaultFontStruct(), Pixel_t back = GetDefaultFrameBackground()) : TGGroupFrame(p,title,options,norm,font,back) {
    fMain=p2;
  }
  Bool_t ProcessMessage(Long_t a, Long_t b, Long_t c) {
    return fMain->ProcessMessage(a,b,c);
  }
};

/*-----------------------------------------------------------------------------*/
bool EDMainFrame::HandleKey(Event_t *event)
{
  unsigned int keysym;
  char str[2];
  gVirtualX->LookupString(event, str, sizeof(str), keysym);
  
  if (event->fType == kGKeyPress) {
    if (//(event->fState & kKeyControlMask) && 
	(EKeySym)keysym == kKey_q) CloseWindow();
    
  if ((EKeySym)keysym == kKey_Escape) CloseWindow();
  if ((EKeySym)keysym == kKey_u) UpdateFile();
  if ((EKeySym)keysym == kKey_Return
      || (EKeySym)keysym == kKey_Space) CallReconstruct();
  if ((EKeySym)keysym == kKey_n ||
      (EKeySym)keysym == kKey_Down) _NextEvent();
  if ((EKeySym)keysym == kKey_p ||
      (EKeySym)keysym == kKey_Up) _PreviousEvent();
  if ((EKeySym)keysym == kKey_PageDown) _NextEvent(5);
  if ((EKeySym)keysym == kKey_PageUp) _PreviousEvent(5);
  }
  return kTRUE;
}

// Defining colours
ULong_t colourdefault;
ULong_t colourmenu,colourtab1,colourtabelem1;
ULong_t colourred, red, colourgreen, colouryellow, colourblue, colorwhite,colorgrey;

/*-----------------------------------------------------------------------------*/
EDMainFrame::EDMainFrame (const TGWindow *p, UInt_t w, UInt_t h) : TGMainFrame(p, w, h) {

  gClient->GetColorByName("AntiqueWhite3",colourdefault);
  gClient->GetColorByName("AntiqueWhite3",colourtab1);
  gClient->GetColorByName("bisque3",colourtabelem1);
  gClient->GetColorByName("azure2",colourmenu);
  gClient->GetColorByName("IndianRed1",colourred);
  gClient->GetColorByName("PaleGreen3",colourgreen);
  gClient->GetColorByName("Khaki1",colouryellow);
  gClient->GetColorByName("dodgerblue",colourblue);
  gClient->GetColorByName("white",colorwhite);
  gClient->GetColorByName("LightGrey",colorgrey);
  gClient->GetColorByName("red",red);

  for(unsigned int i=0; i<kIoSd::NPMT; i++) {
    fArrow[i]=NULL;
    fArrow[i+3]=NULL;
    fArrowLine[i]=NULL;
    fPMTPave[i]=NULL;
  }
  //_eventPos = 0;
  fTextOutput = NULL;
  fEvent = NULL;
  fGraph = NULL;
  fGraph2 = NULL;
  fCdFLDF = NULL;
  fArrowSat = NULL;
  fTriggerName = "all";
  fStationName = "All stations";
  fCountEvents = 0;
  fSliding = 0;
  fFiles = NULL;
  fFileName = "No file loaded";
  fLeg = NULL;
  fLDF = NULL;
  fSuperpPmt = NULL;
  fSuperpPmtPave = NULL;
  fLastEventHadFD=false;
  _drawArray = true;
  _otherWindow = NULL;
  _expertWindow = NULL;
  _triggerWindow = NULL;
  _fileSelectWindow = NULL;
  _event4Stereo = NULL;

  // Some Predefined Layouts
  TGLayoutHints *fMax = new TGLayoutHints(kLHintsTop | kLHintsExpandX | kLHintsExpandY, 1, 1, 1, 0);
  TGLayoutHints *fMaxX = new TGLayoutHints(kLHintsTop | kLHintsExpandX, 1, 1, 0, 0);
  TGLayoutHints *fLeftCenterY = new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsCenterY | kLHintsExpandY, 1, 1, 1, 1);
  TGLayoutHints *fExpandX = new TGLayoutHints(kLHintsTop | kLHintsExpandX, 1, 1, 1, 0);
  TGLayoutHints *fExpandY = new TGLayoutHints(kLHintsTop | kLHintsExpandY, 1, 1, 1, 0);
  TGLayoutHints *Layout_MenuBar1 = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 3, 0, 0);
  TGLayoutHints *Layout_MenuBar2 = new TGLayoutHints(kLHintsTop | kLHintsRight, 0, 3, 0, 0);
  this->ChangeBackground(colourdefault);

  // Starting building of GUI
  fTab = new TGTab(this, 500, 500);
  fTab->Associate(this);
  fTab->ChangeBackground(colourdefault);

  // Control bar
  fControl = new TGHorizontalFrame(this);
  fControl->ChangeBackground(colourdefault);
  
  fHControl = new TGHorizontalFrame(fControl,GetWidth(),30,kChildFrame,fgDefaultFrameBackground);
  fHControl->ChangeBackground(colourdefault);
  // doesn't work on Suns...
  //fHControl->SetLayoutManager(new TGMatrixLayout(fHControl,1,20,1,5));

  // First Tab environment
  tf = fTab->AddTab("Sd Event Display");
  tf->ChangeBackground(colourtab1);
  // width=100 for the array display
  fF3 = new TGHorizontalFrame(tf,100,30,kChildFrame,fgDefaultFrameBackground);
  tf->AddFrame(fF3, fMax);
  fF3->ChangeBackground(colourtab1);

  fF4 = new TGVerticalFrame(fF3,100,30,kChildFrame,fgDefaultFrameBackground);
  fF3->AddFrame(fF4, fLeftCenterY);
  fF4->ChangeBackground(colourtab1);

  fF5 = new TGGroupFrame(fF4, " Events list                                                                                                 Stations list ", kVerticalFrame);
  fF4->AddFrame(fF5, fMax);//fExpandX);
  fF5->ChangeBackground(colourtab1);

  RootFilePopMenu = new TGPopupMenu(fClient->GetRoot());
  RootFilePopMenu->AddEntry("Change Data File Path",ED_FILE_DATAFILEPATH,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddSeparator();
  RootFilePopMenu->AddEntry("Load...",ED_FILE_LOAD,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddSeparator();
  RootFilePopMenu->AddEntry("Current file",ED_FILE_LOAD_CURRENT,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddEntry("Last file",ED_FILE_LOAD_LAST,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddEntry("Last 24h",ED_FILE_LOAD_DAY,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddEntry("Last 48h",ED_FILE_LOAD_TWO_DAY,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddEntry("Last week",ED_FILE_LOAD_WEEK,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddEntry("Last month",ED_FILE_LOAD_MONTH,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddEntry("All",ED_FILE_LOAD_ALL,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddSeparator();
  RootFilePopMenu->AddEntry("Select period",ED_FILE_LOAD_PERIOD,(void*)0,(TGPicture*)0);
  RootFilePopMenu->AddSeparator();
  RootFilePopMenu->AddEntry("Exit",ED_FILE_QUIT,(void*)0,(TGPicture*)0);
  RootFilePopMenu->Associate(this);
  RootFilePopMenu->ChangeBackground(colourtab1);

  //Select trigger to choose files
  fTriggerPopMenu = new TGPopupMenu(fClient->GetRoot());
  fTriggerPopMenu->AddEntry("Enter trigger name...",ED_CHOOSE_TRIGGER,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddSeparator();
  fTriggerPopMenu->AddEntry("All triggers",TRIG_ALL,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("Pattern 3C2",P_EA_A,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("Pattern 3C1H",P_EA_B,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("Pattern 3C1",P_EA_D,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("Pattern 4C4",P_FA_D,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("Pattern 2C1 and 3C2",P_EA_E,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("Pattern 3C2 and 4C4",P_EA_F,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("TOT",TOT_EA,(void*)0,(TGPicture*)0);
 fTriggerPopMenu->AddEntry("FD",TRIG_FD,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->AddEntry("Laser",TRIG_LASER,(void*)0,(TGPicture*)0);
  fTriggerPopMenu->Associate(this);
  fTriggerPopMenu->ChangeBackground(colourtab1);
  fTriggerPopMenu->CheckEntry(TRIG_ALL);

  fMenuRecLpnhe = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fMenuRecLpnhe,"Lpnhe rec. in use","0");
  fConfig.CheckMenu(fConfig.GetSig("Lpnhe rec. in use"));
  fMenuRecLpnhe->AddSeparator(); 
  fConfig.Add(fMenuRecLpnhe,"Old energy converter","0");
  fConfig.Add(fMenuRecLpnhe,"MC + 37% muons","0");
  fConfig.Add(fMenuRecLpnhe,"CIC + FD","0");
  fConfig.Add(fMenuRecLpnhe,"Official - New FD-CIC","0");
  fConfig.CheckMenu(fConfig.GetSig("Old energy converter"));
  fMenuRecLpnhe->AddSeparator();
  fConfig.Add(fMenuRecLpnhe,"Global parameters fit","0");
  fConfig.CheckMenu(fConfig.GetSig("Global parameters fit"));
  fMenuRecLpnhe->AddSeparator(); 
  fConfig.Add(fMenuRecLpnhe,"Plane Geometry","0");
  fConfig.Add(fMenuRecLpnhe,"Curve Geometry","0");
  fConfig.Add(fMenuRecLpnhe,"VCurve Geometry","0");
  fConfig.Add(fMenuRecLpnhe,"Optimal Geometry","0");
  fConfig.CheckMenu(fConfig.GetSig("Optimal Geometry")); 
  fMenuRecLpnhe->AddSeparator(); 
  fConfig.Add(fMenuRecLpnhe,"Log LDF", "0");
  fConfig.Add(fMenuRecLpnhe,"Variable slope LDF", "0");
  fConfig.Add(fMenuRecLpnhe,"NKG LDF", "0");
  fConfig.Add(fMenuRecLpnhe,"Optimal LDF", "0");
  fConfig.CheckMenu(fConfig.GetSig("Optimal LDF"));
  fMenuRecLpnhe->AddSeparator(); 
  fConfig.Add(fMenuRecLpnhe,"Optimize station selection","0");
  fConfig.CheckMenu(fConfig.GetSig("Optimize station selection"));
  fMenuRecLpnhe->AddSeparator();
  fConfig.Add(fMenuRecLpnhe,"Use proton LDF","0");
  fConfig.CheckMenu(fConfig.GetSig("Use proton LDF"));
  fConfig.Add(fMenuRecLpnhe,"Use iron LDF","0");
  fMenuRecLpnhe->AddSeparator();
  fConfig.Add(fMenuRecLpnhe,"Correct Asymmetries","0");
  fConfig.CheckMenu(fConfig.GetSig("Correct Asymmetries"));
  fConfig.Add(fMenuRecLpnhe,"Force reconstruction","0");
  fConfig.Add(fMenuRecLpnhe,"Impose core from FD stereo","0");
  fConfig.Add(fMenuRecLpnhe,"Use silent stations","0");
  fConfig.CheckMenu(fConfig.GetSig("Use silent stations"));
  
  fMenuRecLpnhe->Associate(this);
  fMenuRecLpnhe->ChangeBackground(colourtab1);

  fMenuRecCdF = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fMenuRecCdF,"CdF rec. in use (experimental)","0");
  fMenuRecCdF->AddSeparator();
  fConfig.Add(fMenuRecCdF,"Log-Log parabola LDF","0");
  fConfig.CheckMenu(fConfig.GetSig("Log-Log parabola LDF"));
  fConfig.Add(fMenuRecCdF,"2EXPO LDF","0");
  fConfig.Add(fMenuRecCdF,"Haverah Park LDF","0");
  fConfig.Add(fMenuRecCdF,"smoothed spline LDF","0");
  fMenuRecCdF->Associate(this);
  fMenuRecCdF->ChangeBackground(colourtab1);

  fMenuRecPB = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fMenuRecPB,"P. B. rec. in use","0");
  fMenuRecPB->AddSeparator();
  fConfig.Add(fMenuRecPB,"Asymmetry correction","0");
  fConfig.CheckMenu(fConfig.GetSig("Asymmetry correction"));
  fMenuRecPB->Associate(this);
  fMenuRecPB->ChangeBackground(colourtab1);

  fMenuHorRec = new TGPopupMenu(fClient->GetRoot());
  /*
  fMenuHorRec->AddEntry("Proton",PROT_TABLE,(void*)0,(TGPicture*)0);
  fMenuHorRec->AddEntry("Iron",IRON_TABLE,(void*)0,(TGPicture*)0);
  */
  fConfig.Add(fMenuHorRec,"NMu Contours","0");
  fConfig.Add(fMenuHorRec,"Like core contours","0");
  fConfig.Add(fMenuHorRec,"Likelihood contours","0");
  fConfig.CheckMenu(fConfig.GetSig("NMu Contours"));
  fMenuHorRec->AddSeparator();
 
  fConfig.Add(fMenuHorRec,"Do not perform horizontal rec.","0");
  fMenuHorRec->Associate(this);
  fMenuHorRec->ChangeBackground(colourtab1);

  fConfigMenu = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fConfigMenu,"All-in-one display","0");
  fConfig.Add(fConfigMenu,"Shower plane projection","0");
  fConfig.Add(fConfigMenu,"3D animated reconstruction","0");
  fConfig.Add(fConfigMenu,"Save 3D animation","0");
#ifdef FD

  fConfig.Add(fConfigMenu,"Hybrid display","0");
  fConfig.Add(fConfigMenu,"Hybrid X3D view","0");
#endif
  // New Menu in Configure for Reconstruction Type
  fRecTypSel = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fRecTypSel,"Infill Reconstruction","0");
  

  fConfigMenu->AddSeparator();
  fMenuSel = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fMenuSel,"Use QualityCuts","0");
  fConfig.Add(fMenuSel,"Use CheckT2","0");
  fConfig.Set("Use QualityCuts","1");
  fConfig.Set("Use CheckT2","1");
  fMenuSel->AddSeparator();
  fConfig.Add(fMenuSel,"No selection","0");
  fConfig.Add(fMenuSel,"Official selection","0");
  fConfig.Add(fMenuSel,"Official selection + T5","0");
  fConfig.Add(fMenuSel,"Top Down selection","0");
  fConfig.Add(fMenuSel,"Top Down selection + T5","0");
   fConfig.CheckMenu(fConfig.GetSig("Official selection"));
  //fConfig.Set("Official selection","1");  
  //fConfig.Add(fMenuSel,"Use Infilled","0");
  fConfig.Add(fMenuSel,"Use Infill Stations","0");
  fMenuSel->AddSeparator();
  fConfig.Add(fMenuSel,"Apply once before reconstruction","0");
  fConfig.Add(fMenuSel,"Apply before counting stations","0");
  fConfig.Add(fMenuSel,"Apply upon clicking an event","0");
  fConfig.Set("Apply upon clicking an event","1");
  fMenuSel->AddSeparator();
  fConfig.Add(fMenuSel,"Allow bad calibration/simulated tanks","0");
  fMenuSel->Associate(this);
  fMenuSel->ChangeBackground(colourtab1);
  fConfigMenu->AddPopup("Reconstruction Type",fRecTypSel);
  fConfigMenu->AddPopup("Event and Station Selection",fMenuSel);
  fConfigMenu->AddPopup("Horizontal options",fMenuHorRec);
  fConfigMenu->AddPopup("Lpnhe reconstruction options",fMenuRecLpnhe);
  fConfigMenu->AddPopup("CdF reconstruction options",fMenuRecCdF);
  fConfigMenu->AddPopup("P. B. reconstruction options",fMenuRecPB);

  fConfigMenu->AddSeparator();
  fConfig.Add(fConfigMenu,"Select events only","0");
  fConfigMenu->AddSeparator();
  fConfig.Add(fConfigMenu,"Show number of stations/events","0");
  fConfig.Set("Show number of stations/events","1");
  fConfig.Add(fConfigMenu,"Show station name","0");
  fConfig.Add(fConfigMenu,"Show start of trace","0");
  fConfigMenu->AddSeparator();
  fConfigMenu->AddEntry("Save configuration",ED_SAVECONFIG,(void*)0,(TGPicture*)0);
  fConfigMenu->Associate(this);
  fConfigMenu->ChangeBackground(colourtab1);

  fExpertMenu = new TGPopupMenu(fClient->GetRoot());
  fExpertMenu->AddEntry("Modify event...",ED_EXPERT_MODIFY,(void*)0,(TGPicture*)0);
  fConfig.Add(fExpertMenu,"Show all stations","0");
  fConfig.Add(fExpertMenu,"Show 6 FADC","0");
  fConfig.Add(fExpertMenu,"Show raw calibration","0");
  fConfig.Add(fExpertMenu,"Show raw histograms","0");
  fMenuRand = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fMenuRand,"T4 criteria","0");
  fConfig.Add(fMenuRand,"T5 criteria","0");
  fMenuRand->AddSeparator();
  fConfig.Add(fMenuRand,"Remove External Triggers","0");
  fConfig.Add(fMenuRand,"Remove Lightning","0");
  fConfig.Add(fMenuRand,"Use EA tanks","0");
  fConfig.Add(fMenuRand,"Use Radio infills","0");
  fConfig.Add(fMenuRand,"Use Doublets","0");
  fConfig.Add(fMenuRand,"Remove Lonely Tanks","0");
  fConfig.Add(fMenuRand,"Bottom Up Algorithm","0");
  fMenuRand->Associate(this);
  fMenuRand->ChangeBackground(colourtab1);
  fExpertMenu->AddPopup("Accidental suppression",fMenuRand);
  fConfig.Add(fExpertMenu,"Select station...","0");
  fExpertMenu->AddPopup("Select trigger",fTriggerPopMenu);
  fExpertMenu->Associate(this);
  fExpertMenu->ChangeBackground(colourtab1);

  fMenuMultiSel = new TGPopupMenu(fClient->GetRoot());
  fConfig.Add(fMenuMultiSel,"Enable","0");
  fConfig.Add(fMenuMultiSel,"Delete selected stations","0");
  fMenuMultiSel->Associate(this);
  fMenuMultiSel->ChangeBackground(colourtab1);

  MenuBar_one_station = new TGMenuBar(fHControl, (int)(GetWidth()/2), 25, kRaisedFrame | kFixedWidth);
  MenuBar_one_station->AddPopup(" File ",RootFilePopMenu,Layout_MenuBar1);
  MenuBar_one_station->AddPopup(" Configure ", fConfigMenu ,Layout_MenuBar1);
  MenuBar_one_station->AddPopup(" Experts only... ", fExpertMenu, Layout_MenuBar1);
  MenuBar_one_station->AddPopup(" Multiple selection ", fMenuMultiSel, Layout_MenuBar2);
  fHControl->AddFrame(MenuBar_one_station, fMaxX);

  // File Selection and Update
  fHF_ForFile = new TGHorizontalFrame(fHControl,(int)(GetWidth()/2.2),25,kFixedWidth,fgDefaultFrameBackground);
  fHF_ForFile->ChangeBackground(colourtab1);
  
  fButtonRec = new TGTextButton(fHF_ForFile, " &Reconstruct ", ED_RECONSTRUCT);
  fButtonRec->Associate(this);
  fButtonRec->ChangeBackground(colourred);

  fButtonPrev = new TGTextButton(fHF_ForFile, "&Previous", ED_PREVEVENT);
  fButtonPrev->Associate(this);
  fButtonPrev->ChangeBackground(colouryellow);

  fButtonNext = new TGTextButton(fHF_ForFile, "&Next", ED_NEXTEVENT);
  fButtonNext->Associate(this);
  fButtonNext->ChangeBackground(colouryellow);
  
  fgetFile = new TGTextButton(fHF_ForFile,"&Search event...",ED_GET_FILE);
  fgetFile->Associate(this);
  fgetFile->ChangeBackground(colourgreen);

  fButtonFileUpdate = new TGTextButton(fHF_ForFile, "  &Update  ", ED_FILEBUTTONUPDATE);
  fButtonFileUpdate->Associate(this);
  fButtonFileUpdate->ChangeBackground(colourred);

  fTriggeredStation = new TGTextEntry(fHF_ForFile,new TGTextBuffer(4));
  fTriggeredStation->Resize(30,fTriggeredStation->GetDefaultHeight());
  fTriggeredStation->SetToolTipText("Minimum number of stations triggered");
  fConfig.Add("Minimum number of stations triggered","8");
 
  fHF_ForFile->AddFrame(fButtonRec,fExpandY);
  fHF_ForFile->AddFrame(fButtonPrev,fMax);
  fHF_ForFile->AddFrame(fButtonNext,fMax);
  fHF_ForFile->AddFrame(fButtonFileUpdate, new TGLayoutHints(kLHintsBottom | kLHintsExpandY, 1, 1, 1, 1));
  fHF_ForFile->AddFrame(fTriggeredStation, new TGLayoutHints(kLHintsBottom | kLHintsExpandY, 1, 1, 1, 1));
  fHF_ForFile->AddFrame(fgetFile,fExpandY);

  fHControl->AddFrame(fHF_ForFile,fMaxX);
  fControl->AddFrame(fHControl,fMaxX);
  AddFrame(fControl,fMaxX);

  // Event and Station selection
  fHF_Evt = new TGHorizontalFrame(fF5,100,30,kChildFrame,fgDefaultFrameBackground);
  fHF_Evt->ChangeBackground(colourtab1);
  fEventListBox = new TGListBox(fHF_Evt, ED_EVENTSEL);
  fEventListBox->Resize(250,500);
  fEventListBox->Associate(this);
  fStationListBox = new TGListBox(fHF_Evt, ED_STATIONSEL);
  fStationListBox->Resize(200,500);
  fStationListBox->Associate(this);

  fHF_Evt->AddFrame(fEventListBox,new TGLayoutHints(kLHintsBottom | kLHintsExpandY | kLHintsExpandX, 1, 0, 0, 0));
  fHF_Evt->AddFrame(fStationListBox,new TGLayoutHints(kLHintsBottom | kLHintsExpandY , 1, 0, 0, 0));

  fF5->AddFrame(fHF_Evt,fMax);

  fF6 = new TGGroupFrame(fF4, "Array", kVerticalFrame);
  fF6->ChangeBackground(colourtab1);
  fEventArray = new EDArrayCanvas("EventArray",fF6,this,(int)(GetWidth()/2),(int)((3./5.)*GetWidth()/2)); //please keep this ratio
  //fEventArray = new EDArrayCanvas("EventArray",fF6,this,500,300);
  //image
  fF6->AddFrame(fEventArray,fMax);
  fF4->AddFrame(fF6,fMaxX);

  fF7 = new TGGroupFrame(fF4, "Status", kVerticalFrame);
  fF7->ChangeBackground(colourtab1);
  fTextInfo = new TGTextView(fF7,400,100,new TGText(""));
  fF7->AddFrame(fTextInfo,fMax);
  fF4->AddFrame(fF7,fExpandX);
  //###### The Right canvas ######
  fF2 = new TGGroupFrame(fF3, "Display", kVerticalFrame);
  fF2->ChangeBackground(colourtab1);
  fF3->AddFrame(fF2,fMax);

  fFADC = new TRootEmbeddedCanvas("FADC",fF2,500,700);
  fF2->AddFrame(fFADC,fMax);
  fProgressBar= new TGHProgressBar(fF2, 0, 20);
  fProgressBar->ShowPosition(1,1,"");
  fProgressBar->SetBarColor(colouryellow);
  //keep size in y
  fF2->AddFrame(fProgressBar,fExpandX);
  tf->Resize(tf->GetDefaultSize());

  // FD tab
  tf = fTab->AddTab("FD Event Display");
  tf->ChangeBackground(colourtab1);

  TGTab *fTabFD = new TGTab(tf, 500, 500);
  tf->AddFrame(fTabFD,fMax);
  fTabFD->ChangeBackground(colourtab1);

  string names[5];
  names[1]="Los Leones";
  names[2]="Los Morados";
  names[3]="Loma Amarilla";
  names[4]="Coihueco";
  for (int eye=1;eye<5;eye++) {
    tf = fTabFD->AddTab(names[eye].c_str());
    tf->ChangeBackground(colourtab1);
    fFDEye[eye] = new EDEyeCanvas(tf,eye,fTabFD);
    tf->AddFrame(fFDEye[eye],fMax);
  }

  // Hybrid tab
  tf = fTab->AddTab("Hybrid Event Display");
  tf->ChangeBackground(colourtab1);
  TGHorizontalFrame *fF8 = new TGHorizontalFrame(tf,100,100,kChildFrame,fgDefaultFrameBackground);
  TGHorizontalFrame *fF9 = new TGHorizontalFrame(tf,100,30,kChildFrame,fgDefaultFrameBackground);
  TGHorizontalFrame *fFa = new TGHorizontalFrame(tf,100,100,kChildFrame,fgDefaultFrameBackground);
  tf->AddFrame(fF8,fMaxX);
  tf->AddFrame(fF9,fMax);
  tf->AddFrame(fFa,fMaxX);
  TRootEmbeddedCanvas *c1=new TRootEmbeddedCanvas("a1",fF8,100,100);
  fFDHybrid[3]=new EDEyeEmbeddedCanvas("a2",fF8,600,100,3,M_PI);
  TRootEmbeddedCanvas *c3=new TRootEmbeddedCanvas("a3",fF8,100,100);

  std::string picture1=strdup(EDSRCDIR)+(std::string)"/Auger_logo.xpm";
  TGPicture *fPicture1 = (TGPicture *)gClient->GetPicture(picture1.c_str());
  TGIcon *Icon3 = new TGIcon(tf,fPicture1,fPicture1->GetWidth(),fPicture1->GetHeight());
  c1->AddFrame(Icon3,new TGLayoutHints(kLHintsCenterX | kLHintsCenterY,4,4,4,4));

  fFDHybrid[4]=new EDEyeEmbeddedCanvas("a4",fF9,100,600,4,0.5*M_PI);
  fHybridArray = new EDArrayCanvas("HybridArray",fF9,NULL,600,600);
  fFDHybrid[2]=new EDEyeEmbeddedCanvas("a2",fF9,100,600,2,1.5*M_PI);
  TRootEmbeddedCanvas *c7=new TRootEmbeddedCanvas("a7",fFa,100,100);
  fFDHybrid[1]=new EDEyeEmbeddedCanvas("a1",fFa,600,100,1,0.);
  TRootEmbeddedCanvas *c9=new TRootEmbeddedCanvas("a9",fFa,100,100);
  fF8->AddFrame(c1);
  fF8->AddFrame(fFDHybrid[3],fExpandX);
  fF8->AddFrame(c3);
  fF9->AddFrame(fFDHybrid[4],fExpandY);
  fF9->AddFrame(fHybridArray,fMax);
  fF9->AddFrame(fFDHybrid[2],fExpandY);
  fFa->AddFrame(c7);
  fFa->AddFrame(fFDHybrid[1],fExpandX);
  fFa->AddFrame(c9);

  // Help Tab environment
  tf = fTab->AddTab("Help");
  EDDoc* Doc = new EDDoc(tf,500,500);
  tf->AddFrame(Doc,fMax);
  tf->ChangeBackground(colourtab1);

  fShowHelp = new TGCheckButton(tf, "Show help on startup  ",ED_STARTHELP);
  fConfig.Add("Show help v3r0 on startup","1");
  fShowHelp->Associate(this);
  fShowHelp->ChangeBackground(colourtab1);
  tf->AddFrame(fShowHelp,fExpandX);
  
  for (int i=0;i<fTab->GetNumberOfTabs();i++) {
    fTabElement = fTab->GetTabTab(i);
    fTabElement->ChangeBackground(colourtabelem1);
  }

  AddFrame(fTab,fMax);

  std::string s="Event Display, ";
  s+=VERSION;
  SetWindowName(s.c_str());
  SetIconName("ED");

  MapSubwindows();

  Layout();

  MapWindow();

  fConfig.Add("DataFilePath","");
  /*************** reading configuration ***************/
  std::string input;
  input=getenv("HOME")+(std::string)"/.EDrc";
  fConfig.Load(input.c_str());
 
  kEc::StrictCalib(!fConfig.GetInt("Allow bad calibration/simulated tanks"));
  unsigned int size = fConfig.Names().size();
  for (unsigned int i=0;i<size;i++) {
    if(!fConfig.Names()[i].find("/")) {
      fDirectories.push_back((char *)fConfig.Names()[i].c_str());
      if (fConfig.Values()[i]=="1")
	fDir2Check.push_back((char *)fConfig.Names()[i].c_str());
    }
  }
  /*****************************************************/

  if(fConfig.GetInt("Show help v3r0 on startup")) {
    fShowHelp->SetState(kButtonDown);
    fTab->SetTab(3);
  }
  fTriggeredStation->SetText(fConfig.Get("Minimum number of stations triggered"));

  if (strcmp(fConfig.Get("DataFilePath"),"")==0) {
    _otherWindow = new EDDataFilePath (fClient->GetRoot(),this, 500, 400);
  } else gDataFilePath=fConfig.Get("DataFilePath");

  fHorMap = NULL;
  for(int i=0; i<MAXSTAT; i++) fHorStat[i] = NULL;

  if(startEvent != "") {
    long long id = atoll(startEvent.c_str());
    SearchEvent(id);
  }
  fTabElement = fTab->GetCurrentTab();
  fTabElement->ChangeBackground(colourtab1);


  // basic initialisation of reconstructors
  fCdFRec.SetVerbosity(-1);
  theLpnheRec().SetVerbosity(1);

  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Q), kAnyModifier);//KeyControlMask);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Escape), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_u), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Return), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Space), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_n), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_p), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Down),kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Up), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_PageDown),kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_PageUp), kAnyModifier);

}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::_RecordConfig() 
/*-----------------------------------------------------------------------------*/
{
  //  unsigned int size = fDirectories.size();
  //if (!size) return;

  std::string s = getenv("HOME")+(std::string)"/.EDrc";
  ifstream in(s.c_str());
  ofstream out(".EDrc.tmp");
  
  std::string input,val;
  while (1) {
    in>>input;
    in>>val;
    if (!in) break;
    if(input.find("/")) out<<input.c_str()<<" "<<val.c_str()<<endl;;
  }
  in.close();
  out.close();

  s = (std::string)"mv -f .EDrc.tmp "+getenv("HOME")+(std::string)"/.EDrc";
  system(s.c_str());

  s=getenv("HOME")+(std::string)"/.EDrc";
  out.open(s.c_str(),ofstream::app);  
  for (unsigned int i=0;i<fDirectories.size();i++){
    int checked = 0;
    for (unsigned int j=0; j<fDir2Check.size(); j++)
      if(fDirectories[i]==fDir2Check[j]) {
	checked = 1; 
	break;
      }
    out<<fDirectories[i]<<" "<<checked<<endl;
  }
  out.close();
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::CheckTrig(int trig_id) 
/*-----------------------------------------------------------------------------*/
{
  for(int i=0; i<100 ; i++) {
    fTriggerPopMenu->UnCheckEntry(i);
  }
  //  fTriggerPopMenu->UnCheckEntry(TRIG_ALL);
  fTriggerPopMenu->CheckEntry(trig_id);
  //if(trig_id==TRIG_ALL) fTriggerPopMenu->CheckEntry(trig_id);
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::CheckHorTable (int table_id) 
/*-----------------------------------------------------------------------------*/
{
  /*    fMenuHorRec->UnCheckEntry(PROT_TABLE);
      fMenuHorRec->UnCheckEntry(IRON_TABLE);*/
  fConfig.UnCheckUnSetMenu(fConfig.GetSig("NMu Contours"));
  fConfig.UnCheckUnSetMenu(fConfig.GetSig("Like core contours"));
  fConfig.UnCheckUnSetMenu(fConfig.GetSig("Likelihood contours"));
  fConfig.CheckSetMenu(table_id);
}

/*-----------------------------------------------------------------------------*/
float EDMainFrame::NbTriggeredStation () 
/*-----------------------------------------------------------------------------*/
{
  return atof(fTriggeredStation->GetBuffer()->GetString());
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::_NextEvent (int iter) 
/*-----------------------------------------------------------------------------*/
{
  EventPos tmp = _eventPos+iter;
  while(! fEventListBox->Select(tmp)) {
    tmp ++;
    if(tmp>_maxEvent) break;
  }
  if(!(tmp>_maxEvent)) {
    _eventPos = tmp;
    EventSelection(_eventPos);
  }
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::_PreviousEvent (int iter) 
/*-----------------------------------------------------------------------------*/
{
  EventPos tmp = _eventPos-iter;
  while(! fEventListBox->Select(tmp)) {
    tmp--;
    if(tmp<_minEvent) break;
  }
  if(! (tmp<_minEvent)) {
    _eventPos = tmp;
    EventSelection(_eventPos);
  }
}

/*-----------------------------------------------------------------------------*/
int EDMainFrame::Select() 
/*-----------------------------------------------------------------------------*/
{
  unsigned int size = fEvent->fSelectedStations.size();

  if (fConfig.GetInt("Use QualityCuts"))
    fEvent->UseQualityCuts=true;
  else
    fEvent->UseQualityCuts=false;
  if (fConfig.GetInt("Use CheckT2"))
    fEvent->UseCheckT2=true;
  else 
   fEvent->UseCheckT2=false;
  
  fEvent->UseInfillRec=false;
  bool UseInfillRec = fConfig.GetInt("Infill Reconstruction");
  cerr<<"In Select () "<< UseInfillRec << endl;
  if (UseInfillRec) {// infill
    fConfig.Set("Use Infill Stations","1");
    kEs::UseInfillStations(fConfig.GetInt("Use Infill Stations"));
    fEvent->UseInfillRec=true;
    cerr<<"Setting "<<fEvent->UseInfillRec<<endl;
  }

  if (fConfig.GetInt("Official selection") || fConfig.GetInt("Official selection + T5")) {
    fEvent->OfficialSelection();
  } 
  else if (fConfig.GetInt("Top Down selection") || fConfig.GetInt("Top Down selection + T5")) {
    if (!fEvent->TopDownSelection()) {
      fEvent->fSelectedStations.clear();
    } 
  }

  if(fConfig.GetInt("T4 criteria")) {
    if (!fEvent->IsT4()) {
      for (unsigned int i=0;i<size;i++)
	fEvent->fRandomStations.push_back(fEvent->fSelectedStations[i]);
      fEvent->fSelectedStations.clear();
    }
  }
  else if(fConfig.GetInt("T5 criteria")||fConfig.GetInt("Official selection + T5")) {
    if (!fEvent->IsT5()) {
      for (unsigned int i=0;i<size;i++)
	fEvent->fRandomStations.push_back(fEvent->fSelectedStations[i]);
      fEvent->fSelectedStations.clear();
    }
  }
  if(fConfig.GetInt("Remove External Triggers")) fEvent->RemoveExternalTriggers();
  if(fConfig.GetInt("Remove Lonely Tanks")) fEvent->RemoveLonelyStations();
  if(fConfig.GetInt("Remove Lightning")) fEvent->RemoveLightning();
  kEs::UseEA(fConfig.GetInt("Use EA tanks"));
  kEs::UseRadio(fConfig.GetInt("Use Radio infills"));
  kEs::UseDoublets(fConfig.GetInt("Use Doublets"));
  if(fConfig.GetInt("Bottom Up Algorithm")) fEvent->BottomUpSelection();
  return 1;
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::CallReconstruct () 
/*-----------------------------------------------------------------------------*/
{
  int tobreak = 0;
  if(!Reconstruct()) {
    cerr << "Can't perform reconstruction!" << endl;
    fFADC->GetCanvas()->Clear();
    fFADC->GetCanvas()->cd();
    gPad->Range(0,0,1,1);
    if(fTextOutput) delete fTextOutput;
    fTextOutput=new TPaveText(0,0,1,1);
    fTextOutput->Clear();
    fTextOutput->SetFillColor(19);
    fTextOutput->SetTextFont(32);
    fTextOutput->SetTextColor(2);
    fTextOutput->SetTextSize(0.08);
    fTextOutput->AddText("Reconstruction failed!");
    fTextOutput->Draw();
    fFADC->GetCanvas()->Update();
    DrawArray();
    tobreak = 1;
  }
  if(tobreak) return;
  DrawRec();
  if(fConfig.GetInt("3D animated reconstruction"))
    ShowerAnimate(fEvent,fConfig.GetInt("Save 3D animation"));
}

/*-----------------------------------------------------------------------------*/
int EDMainFrame::Reconstruct () 
/*-----------------------------------------------------------------------------*/
{
  if (!fConfig.GetInt("Apply once before reconstruction")) _selectOnce = true;
  if (!_selectOnce) {
    Select();
    _selectOnce = true;
  }
  if(fConfig.GetInt("NMu Contours")) fEvent->SetHorPlot(NMU);
  else if(fConfig.GetInt("Like core contours")) fEvent->SetHorPlot(LIKECORE);
  else if(fConfig.GetInt("Likelihood contours")) fEvent->SetHorPlot(LIKELIHOOD);
  fEvent->SetDoHorizontalRec(!fConfig.GetInt("Do not perform horizontal rec."));
  
  if (fConfig.GetInt("P. B. rec. in use")) {
    thePBRec().CorrectAsym = fConfig.GetInt("Asymmetry correction");
    fEvent->Reconstruct(&thePBRec());
    _currentRec=&thePBRec();
  } 
  else if (fConfig.GetInt("CdF rec. in use (experimental)")) { // CdF Rec
    if (fConfig.GetInt("Log-Log parabola LDF")) fCdFRec.SetLDF("LOG");
    if (fConfig.GetInt("2EXPO LDF")) fCdFRec.SetLDF("2EXPO");
    if (fConfig.GetInt("Haverah Park LDF")) fCdFRec.SetLDF("HP");
    if (fConfig.GetInt("smoothed spline LDF")) fCdFRec.SetLDF("SMSP");
    fCdFRec.SetEstimated(fEvent->fEstimated);
    fEvent->Reconstruct(&fCdFRec);
    _currentRec=&fCdFRec;
    cout<<fCdFRec<<endl;
  } 
  else if (fConfig.GetInt("Lpnhe rec. in use")) { // LPNHE Reconstruction
    theLpnheRec().SetAsymmetry(fConfig.GetInt("Correct Asymmetries"));
    theLpnheRec().SetForce(fConfig.GetInt("Force reconstruction"));
    //int already = 0;
    
    //before reconstruction search for best configuration
    theLpnheRec().SetOptimize(false);
    if (fConfig.GetInt("Optimize station selection")) theLpnheRec().SetOptimize(true);
    theLpnheRec().SetSilent(false);
    if (fConfig.GetInt("Use silent stations")) theLpnheRec().SetSilent(true);
    theLpnheRec().SetOfficial(false);
    // energy converter
    if (fConfig.GetInt("Official - New FD-CIC")) theLpnheRec().SetEnergyConverter("NEWFD");
    if (fConfig.GetInt("CIC + FD")) theLpnheRec().SetEnergyConverter("CIC");
    if (fConfig.GetInt("Old energy converter")) theLpnheRec().SetEnergyConverter("LOG");
    if (fConfig.GetInt("MC + 37% muons")) theLpnheRec().SetEnergyConverter("MCPLUSMU");

    TGeometry rec1 = eGeomOptimal;
    if (fConfig.GetInt("Plane Geometry")) rec1 = ePlane;
    if (fConfig.GetInt("Curve Geometry")) rec1 = eCurve;
    if (fConfig.GetInt("VCurve Geometry")) rec1 = eVCurve;
    if (fConfig.GetInt("Optimal Geometry")) rec1 = eGeomOptimal;
    TLDF  rec2 = eLDFOptimal;
    if (fConfig.GetInt("Log LDF")) rec2 = eLog;
    if (fConfig.GetInt("Variable slope LDF")) rec2 = eLogXmax;
    if (fConfig.GetInt("NKG LDF")) rec2 = eNKG;
    if (fConfig.GetInt("Optimal LDF")) rec2 = eLDFOptimal;
    theLpnheRec().SetRec(rec1, rec2);
    
    theLpnheRec().SetGlobal(false);
    if (fConfig.GetInt("Global parameters fit")) theLpnheRec().SetGlobal(true);
    if (fConfig.GetInt("Impose core from FD stereo") && CheckForStereo(_eventPos)) {
      double x1=0,x2=0,y1=0,y2=0;
      int i1=0,i2=0;
      unsigned int size = fEvent->T3s.size();
      for(unsigned int i=0;i<size;i++) {
        if (!fEvent->T3s[i].EyeId())
          continue;
        if (!x1) {
          x1=kIoSd::FdX(fEvent->T3s[i].EyeId());
          y1=kIoSd::FdY(fEvent->T3s[i].EyeId());
          i1=i;
        } else {
          x2=kIoSd::FdX(fEvent->T3s[i].EyeId());
          y2=kIoSd::FdY(fEvent->T3s[i].EyeId());
          i2=i;
        }
      }
      double x=x2*tan(fEvent->T3s[i2].sdpangle())-x1*tan(fEvent->T3s[i1].sdpangle())+y1-y2;
      x/=tan(fEvent->T3s[i2].sdpangle())-tan(fEvent->T3s[i1].sdpangle());
      double y=y2+(x-x2)*tan(fEvent->T3s[i2].sdpangle());
      theLpnheRec().SetCore(x,y,1400.0);
    }
    fEvent->Reconstruct(&theLpnheRec());
    _currentRec=&theLpnheRec();
  } // End of DEFAULT LPNHE Reconstruction
  if(fEvent->fRecError) return 0;
  if(_drawArray && fEvent->fSelectedStations.size()) ListStations(fLastStation);
  return 1;
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::SearchEvent(long long id)
/*-----------------------------------------------------------------------------*/
{
  unsigned int size = fDirectories.size();
  unsigned int d = 0;
  for (unsigned int i=0; i<size; i++) {
    string tmp = fDirectories[i];
    if (tmp.length()>d) d = tmp.length(); 
  }
  unsigned int width = 300;
  if (d*6>width) width += (unsigned int) (d*2.4); 
  selectionFrame = new EDSelectionFrame(fClient->GetRoot(),this,width,200+size*25, id);
}

/*-----------------------------------------------------------------------------*/
Bool_t EDMainFrame::ProcessMessage(long msg, long parm1, long parm2) 
/*-----------------------------------------------------------------------------*/
{
  // Process events generated by the buttons in the frame.
  switch (GET_MSG(msg)) {
  case kC_COMMAND:
    switch (GET_SUBMSG(msg)) {
    case kCM_MENU:
      // First... : Root file to open
      if(parm1 == ED_FILE_DATAFILEPATH) {
	_otherWindow = new EDDataFilePath (fClient->GetRoot(),this, 500, 400);
      }
      if(parm1 == ED_FILE_QUIT) CloseWindow();
      if(parm1 == ED_FILE_LOAD) LoadFile();
      if(parm1 >= ED_FILE_LOAD_CURRENT && parm1 <= ED_FILE_LOAD_ALL) {
        SelectFiles(gRootfiles,parm1);
        UpdateFile();
      }
      if(parm1 == ED_FILE_LOAD_PERIOD) {
	int haschanged = 0;
        _fileSelectWindow = new EDFileSelect (fClient->GetRoot(),this, 350, 110, gRootfiles, &haschanged);
        if(haschanged) UpdateFile();
      }
      // selecting trigger...
      if(parm1 == TRIG_ALL) {
        fTriggerName="all"; CheckTrig(TRIG_ALL);
      }
      //if(parm1 == TRIG_OBSERVER)  {fTriggerName="OBSERVER";CheckTrig(TRIG_OBSERVER);}
      //if(parm1 == TRIG_RANDOM)  {fTriggerName="RANDOM";CheckTrig(TRIG_RANDOM);}
      if(parm1 == P_EA_A) {
        fTriggerName="3C2"; CheckTrig(P_EA_A);
      }//TrgP_EA_A
      if(parm1 == P_EA_B) {
        fTriggerName="3C1H"; CheckTrig(P_EA_B);
      }//TrgP_EA_B
      if(parm1 == P_EA_D) {
        fTriggerName="3C1"; CheckTrig(P_EA_D);
      }//TrgP_EA_D
      if(parm1 == P_FA_D) {
        fTriggerName="4C4"; CheckTrig(P_FA_D);
      }//TrgP_FA_D++
      if(parm1 == P_EA_E) {
        fTriggerName="2C1&3C2"; CheckTrig(P_EA_E);
      }//TrgP_EA_E
      if(parm1 == P_EA_F) {
        fTriggerName="3C2&4C4"; CheckTrig(P_EA_F);
      }//TrgP_EA_F
      if(parm1 == TOT_EA) {
        fTriggerName="TOT"; CheckTrig(TOT_EA);
      }
      if(parm1 == TRIG_FD)  {
        fTriggerName="FD"; CheckTrig(TRIG_FD);
      }
      if(parm1 == TRIG_LASER)  {
        fTriggerName="Laser"; CheckTrig(TRIG_LASER);
      }
      if(parm1 == ED_EXPERT_MODIFY) {
	_expertWindow = new EDExpertFrame (fClient->GetRoot(),this, 250, 220);
      }
      if(parm1 == ED_CHOOSE_TRIGGER) {
        _triggerWindow = new EDTrigger (fClient->GetRoot(),this, 220, 60);
        CheckTrig(ED_CHOOSE_TRIGGER);
      }
      if(parm1==fConfig.GetSig("Select station...")) {
	_stationWindow = new EDSelectStation (fClient->GetRoot(),this, 240, 60);
	break;
      }
      if(parm1>=fConfig.GetMinSig() && parm1<=fConfig.GetMaxSig())
	fConfig.CheckMenu(parm1);
      if(parm1 == fConfig.GetSig("NMu Contours")) {
	fEvent->SetHorPlot(NMU);
        CheckHorTable(parm1);
      }
      if(parm1 == fConfig.GetSig("Like core contours")) {
        fEvent->SetHorPlot(LIKECORE);
        CheckHorTable(parm1);
      }
      if(parm1 == fConfig.GetSig("Likelihood contours")) {
        fEvent->SetHorPlot(LIKELIHOOD);
        CheckHorTable(parm1);
      }
      if (parm1==fConfig.GetSig("P. B. rec. in use") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("CdF rec. in use (experimental)"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Lpnhe rec. in use"));
	fConfig.CheckSetMenu(parm1);
      }
      if (parm1==fConfig.GetSig("CdF rec. in use (experimental)") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("P. B. rec. in use"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Lpnhe rec. in use"));
	fConfig.CheckSetMenu(parm1);
      }
      if (parm1==fConfig.GetSig("Lpnhe rec. in use") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("P. B. rec. in use"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("CdF rec. in use (experimental)"));
	fConfig.CheckSetMenu(parm1);
      }

   if (parm1==fConfig.GetSig("No selection") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection + T5"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection + T5"));
	fConfig.CheckSetMenu(parm1);
      }

      if (parm1==fConfig.GetSig("Official selection") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("No selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection + T5"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection + T5"));
	fConfig.CheckSetMenu(parm1);
      }

     if (parm1==fConfig.GetSig("Official selection + T5") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("No selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection + T5"));
	fConfig.CheckSetMenu(parm1);
      }

      if (parm1==fConfig.GetSig("Top Down selection") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("No selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection + T5"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection + T5"));
	fConfig.CheckSetMenu(parm1);
      }

      if (parm1==fConfig.GetSig("Top Down selection + T5") ) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("No selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official selection + T5"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Top Down selection"));
	fConfig.CheckSetMenu(parm1);
      } 

    
      // For simulations --->
      if(parm1==fConfig.GetSig("Allow bad calibration/simulated tanks"))
        kEc::StrictCalib(-1);
      //<---

      // P. B. only --->
      if(parm1==fConfig.GetSig("Asymmetry correction"))
	thePBRec().CorrectAsym = fConfig.GetInt("Asymmetry correction");
      //<---
      
      // CdF only --->
      if(parm1>=fConfig.GetSig("Log-Log parabola LDF") && 
	 //parm1<=fConfig.GetSig("Haverah Park LDF")) {
	 parm1<=fConfig.GetSig("smoothed spline LDF")) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Log-Log parabola LDF"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("2EXPO LDF"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Haverah Park LDF"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("smoothed spline LDF"));
	fConfig.CheckSetMenu(parm1);
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("P. B. rec. in use"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Lpnhe rec. in use"));
	fConfig.CheckSetMenu(fConfig.GetSig("CdF rec. in use (experimental)"));
      }
      //<---
      
      // LPNHE only --->
      if (parm1 == fConfig.GetSig("Global parameters fit")) theLpnheRec().SetGlobal(fConfig.GetInt("Global parameters fit"));
      if (parm1 >= fConfig.GetSig("Plane Geometry") && parm1 <= fConfig.GetSig("Optimal Geometry")) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Plane Geometry"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Curve Geometry"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("VCurve Geometry"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Optimal Geometry"));
	fConfig.CheckSetMenu(parm1); 
	}
      if (parm1 >= fConfig.GetSig("Log LDF") && parm1 <= fConfig.GetSig("Optimal LDF")) {
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Log LDF"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Variable slope LDF"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("NKG LDF"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Optimal LDF"));
	fConfig.CheckSetMenu(parm1); 
	}
      if (parm1==fConfig.GetSig("Correct Asymmetries") )
        theLpnheRec().SetAsymmetry(fConfig.GetInt("Correct Asymmetries"));
      if (parm1==fConfig.GetSig("Use silent stations"))
        theLpnheRec().SetForce(fConfig.GetInt("Use silent stations"));
      if (parm1==fConfig.GetSig("Force reconstruction"))
        theLpnheRec().SetForce(fConfig.GetInt("Force reconstruction"));
      if (parm1==fConfig.GetSig("Use proton LDF")){
	theLpnheRec().SetPrimary(eProton);
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Use iron LDF"));
	fConfig.CheckSetMenu(parm1);
      }
      if (parm1==fConfig.GetSig("Use iron LDF")){
	theLpnheRec().SetPrimary(eIron);
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Use proton LDF"));
	fConfig.CheckSetMenu(parm1);
      }
      if (parm1==fConfig.GetSig("Official - New FD-CIC")){
	theLpnheRec().SetEnergyConverter("NEWFD");
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("CIC + FD"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Old energy converter"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("MC + 37% muons"));
	fConfig.CheckSetMenu(parm1);
      }
      if (parm1==fConfig.GetSig("CIC + FD")){
	theLpnheRec().SetEnergyConverter("CIC");
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official - New FD-CIC"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Old energy converter"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("MC + 37% muons"));
	fConfig.CheckSetMenu(parm1);
      }
      if (parm1==fConfig.GetSig("Old energy converter")){
	theLpnheRec().SetEnergyConverter("LOG");
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official - New FD-CIC"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("MC + 37% muons"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("CIC + FD"));
	fConfig.CheckSetMenu(parm1);
      }
      if (parm1==fConfig.GetSig("MC + 37% muons")){
	theLpnheRec().SetEnergyConverter("MCPLUSMU");
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Official - New FD-CIC"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("Old energy converter"));
	fConfig.UnCheckUnSetMenu(fConfig.GetSig("CIC + FD"));
	fConfig.CheckSetMenu(parm1);
      }
      if(parm1==fConfig.GetSig("Do not perform horizontal rec."))
        fEvent->SetDoHorizontalRec(!fEvent->fDoHorizontalRec);
      if((parm1==fConfig.GetSig("Show station name")||
          parm1==fConfig.GetSig("Show all stations") ||
          parm1==fConfig.GetSig("All-in-one display")) && fEvent)
        ListStations(fLastStation,0);
      if((parm1==fConfig.GetSig("Show raw calibration")||
          parm1==fConfig.GetSig("Show start of trace") ||
          parm1==fConfig.GetSig("Show raw histograms") ||
          parm1==fConfig.GetSig("Show 6 FADC")) && fEvent && fLastStation)
        StationSel(fLastStation);
      if(parm1==fConfig.GetSig("Enable")) {
        if(fConfig.GetInt("Enable") && fStationListBox->GetSelectedEntry()) {
          fStationListBox->GetSelectedEntry()->Activate(kFALSE);
          fIds2Delete.clear();
        }
        fStationListBox->SetMultipleSelections(fConfig.GetInt("Enable"));
        fStationListBox->SelectionChanged();
        fStationListBox->Layout();
      }
      if(parm1==fConfig.GetSig("Delete selected stations")) {
	unsigned int size = fIds2Delete.size();
	for(unsigned int k=0; k<size; k++) {
          for (unsigned int l=0;l<fEvent->fSelectedStations.size();l++) {
            if (fEvent->fSelectedStations[l]->Id==fIds2Delete[k]) {
              fEvent->fRandomStations.push_back(fEvent->fSelectedStations[l]);
              fEvent->fSelectedStations.erase(fEvent->fSelectedStations.begin()+l);
              l--;
            }
          }
        }
        fIds2Delete.clear();
        ListStations(fLastStation);
        fConfig.CheckMenu(fConfig.GetSig("Delete selected stations"));
      }

      if(parm1==ED_SAVECONFIG) {
        std::string s;
        s=getenv("HOME")+(std::string)"/.EDrc";
        fConfig.Set("Minimum number of stations triggered",fTriggeredStation->GetBuffer()->GetString());
        if (!fConfig.Get("DataFilePath"))
          fConfig.Add("DataFilePath",gDataFilePath.c_str());
        else
          fConfig.Set("DataFilePath",gDataFilePath.c_str());
        fConfig.Save(s.c_str());
      }

      break;

    case kCM_BUTTON:
      if(parm1 == ED_NEXTEVENT) _NextEvent();
      if(parm1 == ED_PREVEVENT) _PreviousEvent();
      if( parm1 == ED_FILEBUTTONUPDATE ) UpdateFile();
      if( parm1 == ED_RECONSTRUCT ) CallReconstruct();
      if(parm1==ED_GET_FILE) SearchEvent();
      break;

    case kCM_CHECKBUTTON:
      switch (parm1) {
      case ED_STARTHELP:
        fConfig.SetInt("Show help v3r0 on startup",!fConfig.GetInt("Show help v3r0 on startup"));
        break;
      }
      break;

    case kCM_LISTBOX:
      if(parm1 == ED_EVENTSEL) EventSelection(parm2);
      if(parm1 == ED_STATIONSEL) {
        if(fConfig.GetInt("Enable"))
          fIds2Delete.push_back(parm2);
        StationSel(parm2);
        fLastStation = parm2;
      }
      break;

    case kCM_TAB:
      for (int i=0;i<fTab->GetNumberOfTabs();i++) {
        fTabElement = fTab->GetTabTab(i);
        fTabElement->ChangeBackground(colourtabelem1);
      }
      fTabElement = fTab->GetTabTab(fTab->GetCurrent());
      fTabElement->ChangeBackground(colourtab1);
      if(fEvent && !fConfig.GetInt("Shower plane projection") && 
	 !isnan(fEvent->fRec.fE) && fEvent->fRec.fE) {
	if (fTab->GetCurrent() == 1)
	  for (int i=1;i<5;i++) fFDEye[i]->DrawShower(&fEvent->fRec);
	if (fTab->GetCurrent() == 2)
	  for (int i=1;i<5;i++) fFDHybrid[i]->DrawShower(&fEvent->fRec);
      }
      break;
      
    default:
      break;
    }
  default:
    break;
  }
  
  // because opening a popup menu unbind these keys
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Return), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Down),kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Up), kAnyModifier);
  BindKey((const TGWindow*)this, gVirtualX->KeysymToKeycode(kKey_Escape), kAnyModifier);

  return kTRUE;
}

/*-----------------------------------------------------------------------------*/
static const char* OpenFileDialog (const char *data) 
/*-----------------------------------------------------------------------------*/
{
  static int again=0;
  const char* gOpenAsTypes[] = {
                                 "ROOT files",   "*.root*",
                                 0,              0
                               };
  static TGFileInfo fi;
  fi.fFileTypes = gOpenAsTypes;
  if(!again) {
    fi.fIniDir = strdup(data);
    again=1;
  }
  new TGFileDialog(gClient->GetRoot(), gClient->GetRoot(), kFDOpen, &fi);
  return fi.fFilename;
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::LoadFile () 
/*-----------------------------------------------------------------------------*/
{
  //fFileName = OpenFileDialog(fConfig.Get("DataFilePath"));
  fFileName = OpenFileDialog(gDataFilePath.c_str());
  if(!fFileName) return;
  gRootfiles.clear();
  gRootfiles.push_back(fFileName);
  UpdateFile();
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::AddEvent (EventPos pos, IoSdKey key) 
/*-----------------------------------------------------------------------------*/
{
  std::ostringstream tmp;

  if(_minEvent>pos)
    _minEvent = pos;
  if(fConfig.GetInt("Show number of stations/events"))
    tmp << std::setw(3)<<std::setfill('0')<<fCountEvents << ") ";
  tmp << std::setw(8)<< std::setfill('0')<< key.EvtId<< ", ";
  tmp << key.Multiplicity<< " stations, "<< key.T3Algo;
  fEventListBox->AddEntry(tmp.str().c_str(), pos);
  if(_maxEvent<pos)
    _maxEvent = pos;
}

/*-----------------------------------------------------------------------------*/
int EDMainFrame::CheckForStereo (EventPos pos) 
/*-----------------------------------------------------------------------------*/
{
  if (strstr(fEvent->Trigger.Algo.c_str(), "Stereo") != 0)
    return 1;

  if (!fEvent->Trigger.IsFd()) return 0;
  EventPos min=0;
  if (min<pos-10) min=pos-10;
  EventPos max=fFiles->LastEvent();
  if (max>pos+10) max=pos+10;
  for (EventPos i=min;i<max;i++) {
    if (i==pos) continue;
    if (_event4Stereo) delete _event4Stereo;
    _event4Stereo = new IoSdEvent(i);
    if ((_event4Stereo->Trigger.IsFd()) &&
        (_event4Stereo->Trigger.Algo != fEvent->Trigger.Algo) &&
        (abs((int)((_event4Stereo->Trigger.second()-fEvent->Trigger.second())*1000000+
                   _event4Stereo->Trigger.microsecond()-fEvent->Trigger.microsecond()))<500)) {
      fEvent->T3s.push_back(_event4Stereo->Trigger);
      return 1;
    }
  }
  return 0;
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::AddRecEvent (EventPos pos) 
/*-----------------------------------------------------------------------------*/
{
  _drawArray = false;
  theLpnheRec().SetVerbosity(0);

  if(fEvent) delete fEvent;
  fEvent = new TErEvent(pos);
  if(!Reconstruct()) return;

  std::ostringstream tmp;

  if(_minEvent>pos) _minEvent = pos;
  tmp.setf(ios::fixed,ios::floatfield);
  tmp << std::setw(8)<< std::setfill('0')<< fEvent->Id << ": "<< std::setprecision(1)<< fEvent->fRec.fE<< " EeV, "<< fEvent->fRec.Theta("deg")<< " deg, "<< fEvent->fSelectedStations.size()<< " stations";
  
  if (fEvent->Trigger.IsFd()) {
    if (CheckForStereo(pos)) tmp << ", Stereo";
    else tmp << ", Hybrid";
  }

  fEventListBox->AddEntry(tmp.str().c_str(), pos);
  if(_maxEvent<pos)  _maxEvent = pos;
  
  cerr<<".";
  _drawArray = true;
  theLpnheRec().SetVerbosity(1);
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::UpdateFile () 
/*-----------------------------------------------------------------------------*/
{
  std::ostringstream message;
  unsigned int file;
  unsigned int size = gRootfiles.size();
  if(!size)
    SelectFiles(gRootfiles,ED_FILE_LOAD_CURRENT);
  _minEvent = DEFMIN;
  _maxEvent = 0;
  fTextInfo->Clear();

  ClearLastFile();
  size = gRootfiles.size();
  
  std::vector<std::string> names;
  for(file=0; file<size; file++)
    names.push_back( (std::string)((gRootfiles[file]).filename()) );
  if(fFiles) delete fFiles;
  fFiles = new AugerIoSd(names);
  if(!fFiles->NumberOfEvents()) return;
  
  fProgressBar->Reset();
  fProgressBar->SetRange(0,fFiles->NumberOfEvents());
  fCountEvents = 0;
  vector<IoSdKey> keys = *fFiles->Keys();
  for(KeyPos pos=fFiles->FirstKey(); pos<fFiles->LastKey(); pos=fFiles->NextKey()) {
    fProgressBar->Increment(1.);
    if(keys[pos].Multiplicity<NbTriggeredStation()) continue;
    if(fTriggerName != "all" && strstr(keys[pos].T3Algo.c_str(), fTriggerName.c_str())  == NULL)
      continue;
    if(fStationName!="All stations") {
      IoSdEvent event(pos);
      bool isthere = 0;
      for(unsigned int j=0; j<event.Trigger.NumberOfStation; j++)
        if(fStationName == event.Stations[j].Name && !event.Stations[j].Error) {
          isthere = 1;
          break;
        }
      if(!isthere) continue;
    }
    if(fConfig.GetInt("Apply before counting stations")) {
      if (fEvent) delete fEvent;
      fEvent = new TErEvent(pos);
      Select(); _selectOnce = true;
      size = fEvent->fSelectedStations.size();
      if(size<NbTriggeredStation()) continue;
      keys[pos].Multiplicity=size;
    }
    fCountEvents ++;
    //if(!(fCountEvents%50)) gSystem->ProcessEvents();
    if(!fConfig.GetInt("Select events only")) AddEvent(pos, keys[pos]);
    else AddRecEvent(pos);
  }
  cerr<<endl; 
  fEventListBox->Layout();
  EventSelection(_eventPos);
  fProgressBar->SetPosition(fFiles->NumberOfEvents());
  // test...
  fHorMap = NULL;
}

/*-----------------------------------------------------------------------------*/
int EDMainFrame::AddStation (IoSdStation* st) 
/*-----------------------------------------------------------------------------*/
{
  std::ostringstream tmp;
  unsigned int error= st->Error;
  unsigned int lsId = st->Id;

  if (!error && ((TCalibStation*)st)->IsLightningEvent())
    SetTank(lsId,ISLIGHTN);
  else if(!st->Trigger.Window) {// Station was requested
    if(error == IoSdEvent::eDataLost) SetTank(lsId,ISBADCOMMS);
    else if(error) SetTank(lsId,ISBADLS);
    //else if(st->Trigger.Type&0x400) SetTank(lsId,IST5);
    else SetTank(lsId,IST2);
  } else {// Station was not requested
    if(error == IoSdEvent::eDataLost) SetTank(lsId,ISDOWN);
    else if(error) SetTank(lsId,HASNODATA);
    else SetTank(lsId,IST1);
  }

  if(error && (!fConfig.GetInt("Show all stations")))
    return 0;

  tmp.setf(ios::fixed,ios::floatfield);
  bool inrec = 0;
  TCalibStation *recst=NULL;
  unsigned int size = fEvent->fSelectedStations.size();
  for(unsigned int i=0; i<size; i++)
    if(fEvent->fSelectedStations[i]->Id==lsId) {
      inrec = 1;
      recst = fEvent->fSelectedStations[i];
      break;
    }
  if(!inrec)
    tmp <<"/* ";
  if(fConfig.GetInt("Show number of stations/events"))
    tmp<< std::setw(2)<< std::setfill('0')<< fCountStations << ") ";
  if(fConfig.GetInt("Show station name"))
    tmp<< st->name();
  else
    tmp << std::setw(4)<< std::setfill('0')<< lsId;
  if(!inrec)
    tmp<< " "<< ErrorMessages[error].c_str()<< " ("<< error<< ")";
  else if(recst->Gps->NanoSecond < 1001000000) {
    if(!fGPSOffset)
      fGPSOffset = recst->StartTime();
    tmp<< " ("<< (int)(1.e9*(recst->StartTime()-fGPSOffset))<< " ns, "<< std::setprecision(1)<< recst->fSigInVEM<< " VEM)";
  } else
    tmp<< " (### ns, "<< std::setprecision(1)<< recst->fSigInVEM<< " VEM)";
  if(!inrec)
    tmp <<" */";
  tmp<< '\0';

  fStationListBox->AddEntry(tmp.str().c_str(),lsId);
  return 1;
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::ListStations(unsigned int selected, unsigned int sort) 
/*-----------------------------------------------------------------------------*/
{
  int first = 0;
  fGPSOffset = 0.;
  unsigned int lsId, j;
  unsigned int size;
  std::ostringstream tmp;

  ClearLastEvent();
  DrawArray();
  if(sort) fEvent->SortStationsByTime();
  
  // stations with error=0
  size = fEvent->fSelectedStations.size();  
  fCountStations=0;
  for(j=0 ; j<size; j++) {
    fCountStations++;
    lsId = fEvent->fSelectedStations[j]->Id;
    if(AddStation(fEvent->fSelectedStations[j]) && ((!first && !selected) || lsId==selected)) {
      //if (veryfirst)
      //StationSel(lsId); // FIXME, else I lose 1 PMT in display for first event, don't know why
      //veryfirst=0;
      StationSel(lsId);
      fStationListBox->Select(lsId);
      first = lsId;
    }
  }
  // ambiguous tanks
  size = fEvent->fAmbiguousStations.size();
  for(j=0 ; j<size; j++) {
    fCountStations++;
    AddStation(fEvent->fAmbiguousStations[j]);
    SetTank(fEvent->fAmbiguousStations[j]->Id,AMBIGUOUS);
  }
  // deleted stations
  size = fEvent->fRandomStations.size();
  for(j=0 ; j<size; j++) {
    fCountStations++;
    lsId = fEvent->fRandomStations[j]->id();
    tmp.str("");
    if(fConfig.GetInt("Show number of stations/events"))
      tmp<< std::setw(2)<< std::setfill('0')<< fCountStations << ") ";
    tmp<<std::setw(4)<< std::setfill('0')<< lsId<< ", station deleted" ;
    fStationListBox->RemoveEntry(lsId);
    fStationListBox->AddEntry(tmp.str().c_str(),lsId);
    SetTank(lsId,COLORDEFAULT);
  }

  // stations with error!=0
  for(j=0 ; j<fEvent->Trigger.NumberOfStation; j++) {
    fCountStations++;
    if(!fEvent->Stations[j].Error) continue;
    int good=1;
    size = fEvent->fSelectedStations.size();
    for (unsigned int k=0 ; k<size; k++)
      if (fEvent->fSelectedStations[k]->Id == fEvent->Stations[j].Id) {
        good=0;
        continue;
      }
    if (!good) continue;
    lsId = fEvent->Stations[j].Id;
    if(AddStation(&(fEvent->Stations[j])) && ((!first && !selected) || lsId==selected)) {
      StationSel(lsId);
      fStationListBox->Select(lsId);
      first = lsId;
    }
  }
  fStationListBox->MapSubwindows();
  fStationListBox->Layout();
  fStationListBox->MapWindow();
  fEventArray->Update();
  fHybridArray->Update();
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::DrawSignals () 
/*-----------------------------------------------------------------------------*/
{
  unsigned int i, j;
  unsigned int size = fEvent->fSelectedStations.size();
  int ny = 1, nx = 1;
  if(size < 4)
    nx = 1;
  else if(size < 11)
    nx = 2;
  else if(size < 16)
    nx = 3;
  else if(size < 25)
    nx = 4;
  else if(size < 36)
    nx = 5;
  else if(size < 42)
    nx = 6;
  ny = (int)(round((double)size/(double)nx));
  if((unsigned int)(nx*ny)<size)
    ny++;

  fFADC->GetCanvas()->Clear();
  fFADC->GetCanvas()->Divide(nx, ny, 0, 0);
  if(fLeg) delete fLeg;
  if(fSuperpPmt) delete[] fSuperpPmt;
  if(fSuperpPmtPave) delete[] fSuperpPmtPave;
  fSuperpPmt = new TH1F[size*3];
  fSuperpPmtPave = new TPaveText[size];

  fLeg = new TLegend(0.76,0.2,0.89,0.45);
  fLeg->SetHeader("PMT:");
  fLeg->SetFillStyle(0);
  fLeg->SetLineColor(19);

  for(j=0; j<size; j++) {
    fFADC->GetCanvas()->cd(j+1);
    int min = fEvent->fSelectedStations[j]->fStartBin-20;
    if(min<0)
      min = 0;
    int max = min+NBBIN;
    if(max>=(int)fEvent->fSelectedStations[j]->Fadc->NSample || !min)
      max = fEvent->fSelectedStations[j]->Fadc->NSample-1;
    if(min+NBBIN > max+2)
      min = 0;
    float maxi = 0;
    int offset=0;
    int col[3]={2,4,8};
    if(min)
      offset=-500;
    for(i=0; i<kIoSd::NPMT; i++) {
      if(fEvent->fSelectedStations[j]->fPmt[i].SigInVEM() < 0.01)
        continue;
      float* values = fEvent->fSelectedStations[j]->fPmt[i].fTrace;
      fSuperpPmt[j*3+i].SetLineColor(col[i]);
      fSuperpPmt[j*3+i].SetLineWidth(1);
      fSuperpPmt[j*3+i].SetBins(max-min+1,0+offset,(max-min)*kEc::TimeSlot+offset);
      fSuperpPmt[j*3+i].SetMaximum();
      fSuperpPmt[j*3+i].SetMinimum();
      fSuperpPmt[j*3+i].SetFillStyle(0);
      for(int k=0; k<max-min+1; k++)
        fSuperpPmt[j*3+i].SetBinContent(k+1,(values[k+min]));
      fSuperpPmt[j*3+i].SetStats(kFALSE);
      if(fSuperpPmt[j*3+i].GetMaximum()>maxi)
        maxi = fSuperpPmt[j*3+i].GetMaximum();
    }
    TH1F *frame = fFADC->GetCanvas()->DrawFrame(0+offset,0,(max-min)*kEc::TimeSlot+offset,maxi+0.5);
    frame->GetXaxis()->SetLabelSize(0.05);
    frame->GetYaxis()->SetLabelSize(0.05);
    for(i=0; i<kIoSd::NPMT; i++) {
      fSuperpPmt[j*3+i].Draw("SAME");
      std::ostringstream pm;
      pm << "  " << i+1 ;
      if(!j) {
        fLeg->AddEntry((TObject *)&(fSuperpPmt[j*3+i]),pm.str().c_str(),"l");
      }
    }
    fSuperpPmtPave[j].SetX1NDC(0.45);
    fSuperpPmtPave[j].SetY1NDC(0.5);
    fSuperpPmtPave[j].SetX2NDC(0.85);
    fSuperpPmtPave[j].SetY2NDC(0.89);
    fSuperpPmtPave[j].SetTextAlign(21);
    std::ostringstream buf;
    buf.setf(ios::fixed,ios::floatfield)
      ;
    buf<<fEvent->fSelectedStations[j]->name()<<" ("<<fEvent->fSelectedStations[j]->Id<<")";
    fSuperpPmtPave[j].AddText(buf.str().c_str());
    buf.str("");
    buf<<"Signal: "<<std::setprecision(1)<<fEvent->fSelectedStations[j]->fSigInVEM<<" VEM";
    fSuperpPmtPave[j].AddText(buf.str().c_str());
    buf.str("");
    buf<<"Area/Peak: "<<std::setprecision(2)<<fEvent->fSelectedStations[j]->fSigInVEM/fEvent->fSelectedStations[j]->fPeakInVEM;
    fSuperpPmtPave[j].AddText(buf.str().c_str());
    buf.str("");
    buf<< fEvent->fSelectedStations[j]->Trigger.Name.c_str();
    fSuperpPmtPave[j].AddText(buf.str().c_str());
    if(fEvent->fRecError!=-1 && fEvent->fRec.fU && fEvent->fRec.fV) {
      buf.str("");
      buf<< "r: "<< fEvent->Dist(*(fEvent->fSelectedStations[j]))<< " m";
      fSuperpPmtPave[j].AddText(buf.str().c_str());
      buf.str("");
      buf<< "dt: "<< fEvent->Dt(*(fEvent->fSelectedStations[j]))<< " ns";
      fSuperpPmtPave[j].AddText(buf.str().c_str());
    } else
      fSuperpPmtPave[j].SetY1NDC(0.65);
    fSuperpPmtPave[j].Draw("SAME");
    fLeg->Draw("SAME");
  }
  fFADC->GetCanvas()->cd(0);
  fFADC->GetCanvas()->Update();
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::EventSelection (EventPos pos) 
/*-----------------------------------------------------------------------------*/
{
  std::ostringstream message;
  unsigned int size;

  //kEs::UseInfill(fConfig.GetInt("Use Infilled"));
  kEs::UseInfillStations(fConfig.GetInt("Use Infill Stations"));
  _eventPos = pos;
  fTextInfo->Clear();
  if (fLastEventHadFD)
    for(int k=1;k<5;k++) {
      fFDEye[k]->DrawEye(NULL);
      fFDHybrid[k]->DrawEye(NULL);
    }
  
  if(!fEventListBox->Select(_eventPos)) {
    if(_eventPos == _minEvent) {// No event...
      message <<"No event found with following selection(s): ";
      fTextInfo->AddLine(message.str().c_str());
      message.str("");
      message.setf(ios::fixed,ios::floatfield);
      message << "- minimum number of triggered stations: " <<std::setprecision(0) << NbTriggeredStation();
      fTextInfo->AddLine(message.str().c_str());
      if(fTriggerName != "all") {
        message.str("");
        message <<"- trigger: "<< fTriggerName;
        fTextInfo->AddLine(message.str().c_str());
      }
      if(fStationName!="All stations") {
        message.str("");
        message << "- station: " << fStationName;
        fTextInfo->AddLine(message.str().c_str());
      }
      return;
    }
    EventSelection(_minEvent);
    return;
  }
  int count = 0;
  int tmp = _eventPos;
  for(;tmp>_minEvent;tmp--) {
    if(fEventListBox->Select(tmp)) count++;
    if(count>4) break;
  }
  if(count==5) fEventListBox->SetTopEntry(tmp); 
  fEventListBox->Select(_eventPos);
  fEventListBox->MapSubwindows();
  fEventListBox->Layout();
  fEventListBox->MapWindow();
  
  if(fEvent) delete fEvent;
  fEvent = new TErEvent(_eventPos);
  
  // printing file associated to event
  /* if(!strstr(fFileName,"/")) 
     message<< fFileName ;
     else message<< (strrchr(fFileName,'/')+1) ;
     fTextInfo->AddLine(message.str().c_str()) ;*/
  message.str("");
  message<< "selected file: ";
  if(((string)fFiles->CurrentFileName()).length()>55) {
    fTextInfo->AddLine(message.str().c_str()) ;
    message.str("");
  }
  message<< fFiles->CurrentFileName();
  fTextInfo->AddLine(message.str().c_str()) ;
  
  message.str("");
  message.setf(ios::fixed,ios::floatfield);
  message<< "Minimum number of triggered stations: "<< std::setprecision(0) << NbTriggeredStation();
  fTextInfo->AddLine(message.str().c_str());
  message.str("");
  if(fTriggerName == "all")
    message << "Trigger selected: all of them";
  else
    message << "Trigger selected: " << fTriggerName;
  if(fCountEvents==1)
    message << ", "<< fCountEvents<< " event.";
  else if(fCountEvents > 1)
    message << ", "<< fCountEvents<< " events.";
  fTextInfo->AddLine(message.str().c_str());

  // treating event...
  _selectOnce = false;
  if(fEvent) {
    // FD display
    //fLastEventHadFD=false;
    if (fEvent->RawAugerEvent()) {
      size = fEvent->RawAugerEvent()->Eyes.size();
      for(unsigned int i=0;i<size;i++) {
        TEyeEvent *fdevent=&fEvent->RawAugerEvent()->Eyes[i];
        for(int k=1;k<5;k++) {
          fFDEye[k]->DrawEye(fdevent);
          fFDHybrid[k]->DrawEye(fdevent);
        }
        fLastEventHadFD=true;
      }
    }
    message.str("");
    message << "Date of this event: " << fEvent->UTCDate<< " (GPS " << fEvent->Trigger.Second << ")" << ends;
    fTextInfo->AddLine(message.str().c_str());
    if(fStationName!="All stations") {
      message.str("");
      message << "Station selected: " << fStationName;
      fTextInfo->AddLine(message.str().c_str());
    }
    unsigned int id = 0;
    //fEvent->Signals();//to calculate each Pmt value
    if(fEvent->fSelectedStations.size()>2) {
      if(fConfig.GetInt("Apply before counting stations") || fConfig.GetInt("Apply upon clicking an event")) Select();
      size = fEvent->fSelectedStations.size();
      float dist2 = 1e+12, dx, dy;
      unsigned int i, found = 0;
      for(i=0; i<size; i++) {
        dx = fEvent->fSelectedStations[i]->fX - fEvent->fEstimated.fXCore;
        dy = fEvent->fSelectedStations[i]->fY - fEvent->fEstimated.fYCore;
        if(dist2>sqr(dx)+sqr(dy)) {
          dist2 = sqr(dx)+sqr(dy);
          found = i;
        }
      }
      id = fEvent->fSelectedStations[found]->Id;
    }
    if(fEvent->fSelectedStations.size() != 0) ListStations(id);
    else {
      ListStations(0);
      fFADC->GetCanvas()->Clear();
      fFADC->GetCanvas()->cd();
      gPad->Range(0,0,1,1);
      if(fTextOutput) delete fTextOutput;
      fTextOutput=new TPaveText(0,0,1,1);
      fTextOutput->Clear();
      fTextOutput->SetFillColor(19);
      fTextOutput->SetTextFont(32);
      fTextOutput->SetTextColor(2);
      fTextOutput->SetTextSize(0.06);
      fTextOutput->AddText("");
      fTextOutput->AddText("");
      fTextOutput->AddText("");
      fTextOutput->AddText("");
      fTextOutput->AddText("No station survived cuts");
      fTextOutput->AddText("");
      fTextOutput->AddText("Event classified as noise");
      fTextOutput->AddText("");
      fTextOutput->AddText("");
      fTextOutput->AddText("");
      fTextOutput->AddText("");
      fTextOutput->Draw();
      fFADC->GetCanvas()->Update();
    }
    return;
  }
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::DrawFADC (int nb, short* values, int maxsample) 
/*-----------------------------------------------------------------------------*/
{
  TH1F* tmp = &fFADCHisto[nb];
  tmp->UseCurrentStyle();
  tmp->SetBins(maxsample,0,maxsample);
  tmp->SetMaximum();
  tmp->SetMinimum();
  for(int i=0;i<maxsample;i++)
    tmp->SetBinContent(i+1,values[i]);
  tmp->SetStats(kFALSE);
  tmp->GetXaxis()->SetLabelSize(0.10);
  tmp->GetYaxis()->SetLabelSize(0.10);
  tmp->SetFillStyle(0);
  tmp->SetXTitle("");
  tmp->SetYTitle("");
  tmp->SetMinimum(tmp->GetMinimum()-(tmp->GetMaximum()-tmp->GetMinimum())*0.1);
  tmp->Draw();
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::DrawFADC (int nb, int nbbin) 
/*-----------------------------------------------------------------------------*/
{
  double tot = fStation->fPmt[nb].SigInVEM();
  double smax = fStation->fPmt[nb].PeakInVEM();

  std::ostringstream buf;
  buf<<fStation->name()<<": PMT "<<nb+1;

  if(tot<0.01) {
    buf<<" off  ";
    if(fPMTPave[nb]) delete fPMTPave[nb];
    gPad->Range(0,0,1,1);
    fPMTPave[nb] = new TPaveText(0,0,1,1);
    fPMTPave[nb]->SetFillStyle(0);
    TText *tmp = fPMTPave[nb]->AddText(buf.str().c_str());
    tmp->SetTextAngle(12);
    tmp->SetTextAlign(22);
    tmp->SetTextColor(17);
    fPMTPave[nb]->Draw();
    return;
  }

  float *values = fStation->fPmt[nb].fTrace;
  int min = fStation->fStartBin-20;
  if(min < 0)
    min = 0;
  int max = min+nbbin;
  if(max >= (int)fStation->Fadc->NSample || !min)
    max = fStation->Fadc->NSample-1;
  if(min+nbbin>max+2)
    min = 0;

  int offset = 0;
  if(min)
    offset = -500;
  fFADCHisto[nb].UseCurrentStyle();
  fFADCHisto[nb].SetBins(max-min+1,0+offset,(max-min)*kEc::TimeSlot+offset);
  fFADCHisto[nb].SetMaximum();
  fFADCHisto[nb].SetMinimum();
  for(int i=0; i<max-min+1; i++)
    fFADCHisto[nb].SetBinContent(i+1,(values[i+min]));
  fFADCHisto[nb].SetStats(kFALSE);
  fFADCHisto[nb].GetXaxis()->SetLabelSize(0.05);
  fFADCHisto[nb].GetYaxis()->SetLabelSize(0.05);
  fFADCHisto[nb].SetFillStyle(1001);
  fFADCHisto[nb].SetFillColor(29);
  fFADCHisto[nb].Draw();

  int nblines=4 + ((fStation->Calib->TubeOk[nb] && abs((int)fStation->Calib->Rate[nb]-7000)<200) || (!fStation->Calib->TubeOk[nb]));

  if(fPMTPave[nb]) delete fPMTPave[nb];
  fPMTPave[nb] = new TPaveText(kEc::TimeSlot*.7*(max-min)+offset,(1.1-nblines*.1)*smax,kEc::TimeSlot*1.1*(max-min)+offset,1.1*smax);

  fPMTPave[nb]->AddText(buf.str().c_str());
  fPMTPave[nb]->AddText("");
  fPMTPave[nb]->AddLine(0,0,0,0);

  if(fStation->Calib->TubeOk[nb] && abs((int)fStation->Calib->Rate[nb]-7000)<200)
    fPMTPave[nb]->AddText("Tube Calibrated");
  else if(!fStation->Calib->TubeOk[nb])
    fPMTPave[nb]->AddText("Tube Not Calibrated");

  buf.str("");
  buf.setf(ios::fixed,ios::floatfield)
    ;
  buf<<"Signal: "<<std::setprecision(2)<<tot<<" VEM";
  fPMTPave[nb]->AddText(buf.str().c_str());

  buf.str("");
  buf.setf(ios::fixed,ios::floatfield)
    ;
  buf<<"Area/Peak: "<<std::setprecision(2)<<((1.*tot)/smax);
  fPMTPave[nb]->AddText(buf.str().c_str());

  buf.str("");
  buf << fStation->Trigger.Name;
  fPMTPave[nb]->AddText(buf.str().c_str());

  fPMTPave[nb]->Draw();

  if(fConfig.GetInt("Show start of trace"))
    DrawFADCArrows(nb,offset);
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::DrawFADCArrows (int nb, int offset) 
/*-----------------------------------------------------------------------------*/
{
  offset+=0;
  double smax = fStation->fPmt[nb].PeakInVEM();
  double start = 0;
  double end = (fStation->fEndIntegration-fStation->fStartIntegration)*kEc::TimeSlot;
  if(fArrow[nb]) delete fArrow[nb];
  fArrow[nb] = new TArrow(start,-smax*0.1,end,-smax*0.1,0.03,"|>");
  fArrow[nb]->SetLineColor(2);
  fArrow[nb]->SetLineWidth(1);
  fArrow[nb]->Draw();

  if(fArrowLine[nb]) delete fArrowLine[nb];
  fArrowLine[nb] = new TLine(start,-smax*0.1,start,smax*1.15);
  fArrowLine[nb]->SetLineColor(2);
  fArrowLine[nb]->SetLineWidth(1);
  fArrowLine[nb]->SetLineStyle(2);
  fArrowLine[nb]->Draw();

  if(fArrow[nb+3]) delete fArrow[nb+3];
  fArrow[nb+3] = new TArrow(start,smax*1.15,end,smax*1.15,0.03,"|>");
  fArrow[nb+3]->SetLineColor(2);
  fArrow[nb+3]->SetLineWidth(1);
  fArrow[nb+3]->Draw();

}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::StationSel (unsigned int number) 
/*-----------------------------------------------------------------------------*/
{
  unsigned int i, size;
  if(fConfig.GetInt("All-in-one display")) {
    if(!fLastStation)
      DrawSignals();
    size = fEvent->fSelectedStations.size();
    for(i=0; i<size; i++) {
      if(number == fEvent->fSelectedStations[i]->Id) {
        fFADC->GetCanvas()->cd(i+1);
        gPad->SetFillColor(188);
        gPad->Modified();
      }
      if(fLastStation == fEvent->fSelectedStations[i]->Id) {
        fFADC->GetCanvas()->cd(i+1);
        gPad->SetFillColor(19);
        gPad->Modified();
      }
    }
    fFADC->GetCanvas()->cd(0);
    fFADC->GetCanvas()->Update();
    fEventArray->Select(number);
    fEventArray->Update();
    fStation=NULL;
    size = fEvent->fCalibStations.size();
    for (i=0;i<size;i++) {
      if (fEvent->fCalibStations[i].Id==number && fStation==NULL)
        fStation=&(fEvent->fCalibStations[i]);
      if (fEvent->fCalibStations[i].Id==number && fEvent->fCalibStations[i].Error==0)
        fStation=&(fEvent->fCalibStations[i]);
    }
    /*if(fEvent->Trigger.Algo.substr(0,2) == "FD" && fStation && !fStation->Error){
      std::ostringstream message;
      message<<"Delta Time (" << fStation->Id << "-FD): " << fEvent->Trigger.MicroSecond+fStation->Gps->Offset-fStation->Gps->NanoSecond/1000.<< " us"	;
      fTextInfo->AddLine(message.str().c_str()) ;
      }*/

    if(!fStation)
      return; // Station not found?
    fLastStation = number;
    return;
  }
  // deleted stations
  size = fEvent->fRandomStations.size();
  for(i=0; i<size; i++)
    if(fEvent->fRandomStations[i]->id() == number) {
      fFADC->GetCanvas()->Clear();
      fFADC->GetCanvas()->Update();
      fEventArray->Select(number);
      fEventArray->Update();
      return;
    }

  fEventArray->Select(number);
  fEventArray->Update();
  fStation=NULL;
  size = fEvent->fCalibStations.size();
  for (i=0;i<size;i++) {
    if (fEvent->fCalibStations[i].Id==number && fStation==NULL)
      fStation=&(fEvent->fCalibStations[i]);
    if (fEvent->fCalibStations[i].Id==number && fEvent->fCalibStations[i].Error==0)
      fStation=&(fEvent->fCalibStations[i]);
  }
  if(!fStation)
    return; // Station not found or deleted
  if(fStation->Error) {
    fFADC->GetCanvas()->Clear();
    fFADC->GetCanvas()->Update();
    return;
  }
  fLastStation = number;
  TCanvas *tmp=fFADC->GetCanvas();
  tmp->Clear();
  /*************** Show the raw calibration ***************/
  if(fConfig.GetInt("Show raw calibration")) {
    if(fTextOutput) delete fTextOutput;
    tmp->Range(0,0,1,1);
    tmp->cd();
    fTextOutput = new TPaveText(0,0,1,1);
    fTextOutput->SetTextFont(22);
    fTextOutput->AddText(fStation->name().c_str());
    std::ostringstream message;
    message<< "Id "<< fStation->Id;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    fTextOutput->AddText("");
    message << "100MHz: current " << fStation->Gps->Current100;
    message << " next " << fStation->Gps->Next100;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "40MHz: current " << fStation->Gps->Current40;
    message << " next " << fStation->Gps->Next40;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "SawTooth: Previous " << (int)fStation->Gps->PreviousST;
    message << " Current " << (int)fStation->Gps->CurrentST;
    message << " Next " << (int)fStation->Gps->NextST;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "GPS Offset " << 0.01*(short)(fStation->Gps->Offset & 0xffff);
    if (fStation->CanHaveJumps())
      message << "   (with 100ns jumps)";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Nanosecond (EOT) " << fStation->Gps->nanosecond();
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    fTextOutput->AddText("");
    message << "Trigger1Algo " << fStation->Trigger.Name;
    message << " Type " << fStation->Trigger.Type;
    if (fStation->Trigger.IsTOT())
      message << " (TOT) ";
    message << " NSamples " << fStation->Fadc->NSample;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Event Clock Tick " << fStation->Gps->Tick;
    message << "  offset " << ((short)(fStation->Gps->Offset>>16));
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Calib version " <<fStation->Calib->Version;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Calib start " <<fStation->Calib->StartSecond;
    message << " end " << fStation->Calib->EndSecond;
    int caltime=(int)fStation->Calib->EndSecond-(int)fStation->Calib->StartSecond-1;
    message << " (" << caltime << " s)" ;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "T1/T2/TOT rate " << std::setprecision(4) << fStation->Calib->T1Rate <<" / " << std::setprecision(3) << fStation->Calib->T2Rate << " / " << fStation->Calib->TOTRate << " Hz" ;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    fTextOutput->AddText("");
    message << "Tubes OK:   ";
    for(i=0; i<kIoSd::NPMT; i++)
      message << fStation->Calib->TubeOk[i] << "  ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Dynodes Baselines:" ;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    for(i=0; i<kIoSd::NPMT; i++)
      message<< std::setprecision(4)<< fStation->Calib->Base[i]<< "  "<< std::setprecision(2)<< fStation->Calib->SigmaBase[i]<< "   ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Anodes Baselines:" ;
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    for(i=3; i<6; i++)
      message<< std::setprecision(4)<< fStation->Calib->Base[i]<< "  "<< std::setprecision(2)<< fStation->Calib->SigmaBase[i]<< "   ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    fTextOutput->AddText("");
    message<< "VEM peak   "<< std::setprecision(3);
    for(i=0; i<kIoSd::NPMT; i++)
      message<< fStation->fPmt[i].fVemPeak<< "   ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "VEM charge   " << std::setprecision(3);
    for(i=0;i<3;i++)
      message << fStation->fPmt[i].fVemCharge << "   ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Area/Peak   " ;
    for(i=0;i<3;i++)
      message << fStation->fPmt[i].fVemCharge/fStation->fPmt[i].fVemPeak << "  ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Pulse Decay Time   " ;
    for(i=0;i<3;i++)
      message << fStation->fPmt[i].fPulseSlope << "  ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Stability: ";
    for(i=0; i<kIoSd::NPMT; i++)
      message<< fStation->Calib->ShowEvolution[i]<< " ("<< fStation->Calib->Stability[i]<<")  ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "70Hz calib sigma    "<< std::setprecision(2);
    for(i=0; i<kIoSd::NPMT; i++)
      message << (fStation->Calib->Rate[i]-70.)/sqrt(70./caltime) << "   ";
    fTextOutput->AddText(message.str().c_str());
    fTextOutput->AddText("");
    message.str("");
    message << "Dynode/Anode";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Rates   "<< std::setprecision(4);
    for(i=0; i<kIoSd::NPMT; i++)
      message << fStation->Calib->NbTDA[i]*1./caltime << "  ";
    fTextOutput->AddText(message.str().c_str());
    message.str("");
    message << "Ratio   ";
    for(i=0; i<kIoSd::NPMT; i++)
      message << std::setprecision(4) << fStation->Calib->DA[i] << "  " << std::setprecision(2) << fStation->Calib->SigmaDA[i] << "   ";
    fTextOutput->AddText(message.str().c_str());
    fTextOutput->AddText("");
    fTextOutput->Draw();
  }
  /*************** Show the raw histograms ***************/
  else if(fConfig.GetInt("Show raw histograms") && fStation->Calib->Version>8) {
    tmp->Divide(3,3,0,0);
    for(i=0; i<kIoSd::NPMT; i++) {
      tmp->cd(i+1);
      fStation->HBase(i)->Draw();
      tmp->cd(i+4);
      fStation->HCharge(i)->Draw();
    }
    tmp->cd(7);
    fStation->HCharge(3)->Draw();
    tmp->cd(8);
    fStation->HPeak(0)->Draw();
    gPad->RangeAxis(0,0,400,2000);
    for(i=1; i<kIoSd::NPMT; i++)
      fStation->HPeak(i)->Draw("SAME");
    tmp->cd(9);
    gPad->SetLogy();
    for(i=0; i<kIoSd::NPMT; i++) {
      TH1 *h=fStation->HShape(i);
      double xx=h->GetBinContent(1);
      double yy=1;
      for (int j=1;j<20;j++)
      if (h->GetBinContent(j)-xx>yy) yy=h->GetBinContent(j)-xx;
      for (int j=1;j<20;j++) {
        h->SetBinContent(j,(h->GetBinContent(j)-xx)/yy);
        if (h->GetBinContent(j)<=0) h->SetBinContent(j,1e-4);
        if (yy<50000) h->SetBinContent(j,1e-4);
      }
      if (i==0) h->Draw(); else h->Draw("SAME");
    }
    tmp->cd(0);
  }
  /*************** Show the 6 FADC traces ***************/
  else if(fConfig.GetInt("Show 6 FADC")) {
    tmp->Divide(1,6,0,0);
    for(i=0; i<kIoSd::NPMT; i++) {
      tmp->cd(i+1);
      DrawFADC(i, fStation->Fadc->Trace[i][IoSdEvent::eHigh], fStation->Fadc->NSample);
      tmp->cd(i+4);
      DrawFADC(i+3, fStation->Fadc->Trace[i][IoSdEvent::eLow], fStation->Fadc->NSample);
    }
    tmp->cd(0);
  }
  /*************** Show the 3 PMT traces ***************/
  else {
    tmp->Divide(1,3,0,0);
    for(i=0; i<kIoSd::NPMT; i++) {
      tmp->cd(i+1);
      DrawFADC(i,NBBIN);
    }
    tmp->cd(0);
  }
  tmp->Update();
  /*
    if(fEvent->Trigger.Algo.substr(0,2) == "FD" && fStation && !fStation->Error){
    std::ostringstream message;
    message<<"Delta Time (" << fStation->Id << "-FD): " << fEvent->Trigger.MicroSecond+fStation->Gps->Offset-fStation->Gps->NanoSecond/1000.<< " us"	;
    fTextInfo->AddLine(message.str().c_str()) ;
    }
  */
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::ClearLastFile () 
/*-----------------------------------------------------------------------------*/
{
  fEventListBox->RemoveEntries(-1<<31,1<<31-1);
  ClearLastEvent();
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::ClearLastEvent () 
/*-----------------------------------------------------------------------------*/
{
  fStationListBox->RemoveEntries(-1<<31, 1<<31-1);
  fEventArray->Clear(fEvent);
  fHybridArray->Clear(fEvent);
  fLastStation=0;
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::ClearAll () 
/*-----------------------------------------------------------------------------*/
{
  ClearLastFile();
  fFADC->GetCanvas()->Clear();
  fFADC->GetCanvas()->Update();
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::GetEventById (vector <string> files, long long eventId) 
/*-----------------------------------------------------------------------------*/
{
  ClearAll();
  gRootfiles.clear();
  unsigned int size = files.size();
  for (unsigned int i=0; i<size;i++) gRootfiles.push_back(files[i].c_str());
  if(fFiles) delete fFiles;
  fFiles = new AugerIoSd(files);
  if(!fFiles->NumberOfEvents()) return;
  vector<IoSdKey> keys = *fFiles->Keys();
  KeyPos pos, last=fFiles->LastKey();
  _minEvent = fFiles->FirstKey();
  _maxEvent =  fFiles->LastKey()-1;
  for(pos=fFiles->FirstKey(); pos<fFiles->LastKey(); pos=fFiles->NextKey())
    if(keys[pos].EvtId == (unsigned long long)eventId) {
      AddEvent(pos, keys[pos]);
      last = pos;
    }
  if(last == fFiles->LastKey()) {
    fTextInfo->Clear();
    fTextInfo->AddLine("Not found in database.");
    return;
  }
  std::ostringstream tmp;
  tmp<<"event "<<eventId<<" is in file(s): ";
  fTextInfo->Clear();
  fTextInfo->AddLine(tmp.str().c_str());
  for (unsigned int i=0; i<size;i++) 
    fTextInfo->AddLine(strrchr(files[i].c_str(),'/'));
  fTextInfo->AddLine("getting it...");
  fEventListBox->Layout();
  EventSelection(last);
 
  fProgressBar->Reset();
  fProgressBar->SetRange(0,1);
  fProgressBar->SetPosition(1);
  
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::DrawArray () 
/*-----------------------------------------------------------------------------*/
{
  double xc=0.,yc=0.;
  unsigned int size = fEvent->fSelectedStations.size();
  //if(fFiles->Array()->GetArray()->size()>100)
  if(size>0) {
    for(unsigned int j=0 ; j<size; j++) {
      xc += fEvent->fSelectedStations[j]->fX;
      yc += fEvent->fSelectedStations[j]->fY;
    }
    xc/=size;
    yc/=size;
  } else {
    xc = fEvent->fCalibStations[0].fX;
    yc = fEvent->fCalibStations[0].fY;
  }

  if(fConfig.GetInt("Shower plane projection"))
    fEventArray->DrawProjArray(fEvent);
  else {
    fEventArray->DrawArray(fEvent, xc, yc);
    if (CheckForStereo(_eventPos)) fEventArray->DrawSDP(fEvent->T3s);
  }
  fHybridArray->DrawArray(fEvent, xc, yc, 1000001);
  if (CheckForStereo(_eventPos)) fHybridArray->DrawSDP(fEvent->T3s);
}

/*-----------------------------------------------------------------------------*/
void EDMainFrame::DrawRec () 
/*-----------------------------------------------------------------------------*/
{
  unsigned int size;
  fLastStation=0;
  if(fLDF) {
    delete fLDF; fLDF = NULL;
  }
  if(fCdFLDF) {
    delete fCdFLDF; fCdFLDF = NULL;
  }
  size =  fMarker.size();
  for(unsigned int i=0; i<size; i++) delete fMarker[i];
  fMarker.clear();
  double dt=0,dt2=0;

  double rcut = fEvent->fRec.fLDFRcut;
  size =  fEvent->fSelectedStations.size();
  for(unsigned int i=0; i<size; i++) {
    dt += fEvent->Dt(*(fEvent->fSelectedStations[i]));
    dt2 += pow(fEvent->Dt(*(fEvent->fSelectedStations[i])),2);
  }
  int zero = 0;
  if (_currentRec == &theLpnheRec()) {
    for(unsigned int i=0; i<size; i++) if (theLpnheRec().Wgt(fEvent->fSelectedStations[i]) != 0.0) {
      dt += fEvent->Dt(*(fEvent->fSelectedStations[i]));
      dt2 += pow(fEvent->Dt(*(fEvent->fSelectedStations[i])),2);
      }
    else ++zero;
    }
  dt /= (size-zero);
  dt2 /= (size-zero);
  if(fEvent->fRec.Theta() < 1.2217 || !fEvent->fDoHorizontalRec) {
    if (_currentRec==&fCdFRec && fCdFRec.fLDFType != "LOG") {
      fCdFLDF = new TGraph(fCdFRec.fLDF.size());
      for(unsigned int i=0; i<fCdFRec.fLDF.size(); i++)
	fCdFLDF->SetPoint(i, fCdFRec.fLDF[i][0],fCdFRec.fLDF[i][1]);
      fCdFLDF->SetLineColor(3);
      fCdFLDF->SetLineWidth(3);
    }
    else {
    if (fConfig.GetInt("NKG LDF")) fLDF = new TF1("LDF","[0]*((x/1000)*(x+700.0)/(1000.0+700.0))**[1]",0,5000);
      else  fLDF = new TF1("LDF","(x>[4])*([0]*((x/[3])**([1]+[2]*log(x/[3]))))+(x<=[4])*[0]*([4]/[3])**([1]+[2]*log([4]/[3]))*(x/[4])**([1]+2*[2]*log([4]/[3]))",0,5000);
      fLDF->SetTitle("Lateral distribution function fit");
      fLDF->SetLineColor(3);
      fLDF->SetParameter(0, fEvent->fRec.fSRef);
      fLDF->SetParameter(1, fEvent->fRec.fBeta);
      if (!fConfig.GetInt("NKG LDF")) fLDF->SetParameter(2, fEvent->fRec.fGamma);
      fLDF->SetParameter(3,fEvent->fRec.fRefDist);
      fLDF->SetParameter(4,rcut);
    }
    if(fGraph) delete fGraph;
    if(fGraph2) delete fGraph2;
    fGraph = new TGraphErrors(size);
    fGraph->SetTitle("Lateral distribution function fit");
    fGraph->SetMarkerStyle(21);
    fGraph->SetMarkerColor(4);
    fGraph2 = new TGraph(201);
    fGraph2->SetFillStyle(1001);
    fGraph2->SetFillColor(179);
    fGraph2->SetLineColor(179);
    fGraph2->SetLineStyle(3);

    fFADC->GetCanvas()->Clear();
    fFADC->GetCanvas()->Divide(1,2);
    fFADC->GetCanvas()->cd(1);

    gPad->SetLogy();

    /**************************** drawing the LDF ****************************/
    if(fEvent->fRec.fE>0) {
      double min=1,max=0;
      double dist;
      for(unsigned int i=0; i<size; i++) {
	dist = fEvent->Dist(*(fEvent->fSelectedStations[i]));
	double dx = fEvent->fSelectedStations[i]->fX - fEvent->fRec.fXCore;
	double dy = fEvent->fSelectedStations[i]->fY - fEvent->fRec.fYCore;
	double s = fEvent->fSelectedStations[i]->fSigInVEM;
	if (_currentRec==&thePBRec() && fConfig.GetInt("Asymmetry correction")) 
	  s /= thePBRec().AsymFact(dx,dy);
	if( (_currentRec==&theLpnheRec() && fConfig.GetInt("Correct Asymmetries")) 
	   || (_currentRec==&fCdFRec && fCdFRec.fLDFType == "LOG") )
	  s *= compute_asymetry(dx,dy,dist,fEvent->fRec.fU,fEvent->fRec.fV);

	if(s<min) min=s;
        if(s>max) max=s;
	fGraph->SetPoint(i, dist,s);
        if (_currentRec==&theLpnheRec() && fEvent->fSelectedStations[i]->fSat) fGraph->SetPointError(i,0,s*fEvent->fSelectedStations[i]->fSat*0.01);
        else fGraph->SetPointError(i,0,sqrt(s));
      }
      if(min>=1) fGraph->SetMinimum(1);
      fGraph->SetMaximum(3*max);
      fGraph->Draw("AP");
      double xmin=fGraph->GetXaxis()->GetXmin();
      double xmax=fGraph->GetXaxis()->GetXmax();
      for (int i=0; i<201;i++) {
        double d = abs(i-100)*(xmax-xmin)/100+xmin;
        double s = fEvent->fRec.fSRef *
	  pow(d/fEvent->fRec.fRefDist, fEvent->fRec.fBeta+fEvent->fRec.fGamma*log(d/fEvent->fRec.fRefDist));
        if (d<rcut) {
          double fact = 1/pow(rcut/fEvent->fRec.fRefDist,fEvent->fRec.fBeta+fEvent->fRec.fGamma*log(rcut/fEvent->fRec.fRefDist));
          double slope = fEvent->fRec.fBeta+2*fEvent->fRec.fGamma*log(rcut/fEvent->fRec.fRefDist);
          s=fEvent->fRec.fSRef * pow(d/rcut,slope)/fact;
	  /*
	  pow(d/DISTREF, fEvent->fRec.fBeta+fEvent->fRec.fGamma*log(d/DISTREF));
        if (d<300) {
          double fact = 1/pow(300./DISTREF,fEvent->fRec.fBeta+fEvent->fRec.fGamma*log(300./DISTREF));
          double slope = fEvent->fRec.fBeta+2*fEvent->fRec.fGamma*log(300./DISTREF);
          s=fEvent->fRec.fSRef * pow(d/300.,slope)/fact;
	  */
        }
	if (fConfig.GetInt("NKG LDF")) s = fEvent->fRec.fSRef * pow(d/1000.0*(d+700.0)/1700.0, fEvent->fRec.fBeta);
	if (i<100 && d>100) fGraph2->SetPoint(i, d, pow(sqrt(0.25+s)-0.5,2));
        else if (d>100) fGraph2->SetPoint(i, d, pow(sqrt(0.25+s)+0.5,2));
      }
      if(fLDF) {
	fGraph2->Draw("SLF");
	fLDF->Draw("SAME");
      }
      else fCdFLDF->Draw("L");
      fGraph->Draw("P");
      if (_currentRec!=&theLpnheRec()) {
        for(unsigned int i=0; i<size; i++) {
          if(fEvent->fSelectedStations[i]->fPmt[0].fLowGainSat ||
              fEvent->fSelectedStations[i]->fPmt[1].fLowGainSat ||
              fEvent->fSelectedStations[i]->fPmt[2].fLowGainSat) {
            dist = fEvent->Dist(*(fEvent->fSelectedStations[i]));
            double s =  fEvent->fSelectedStations[i]->fSigInVEM;
            if(fArrowSat) delete fArrowSat;
            fArrowSat = new TArrow(dist,s,dist,1.5*s,0.02, "|>");
            fArrowSat->SetLineWidth(3);
            fArrowSat->SetLineColor(4);
            fArrowSat->SetFillColor(4);
            fArrowSat->Draw();
            break;
          }
        }
      }
      // including silent stations...
      unsigned int size2 = fEvent->fSilentStations.size();
      for(unsigned int i=0;i<size2;i++) {
        double d = fEvent->Dist(*(fEvent->fSilentStations[i]));
	double s = fEvent->fRec.fSRef *
	  pow(d/fEvent->fRec.fRefDist, fEvent->fRec.fBeta+fEvent->fRec.fGamma*log(d/fEvent->fRec.fRefDist));
        if(fConfig.GetInt("Use silent stations") && s >= SIGTHRESH)
          SetTank(fEvent->fSilentStations[i]->Id,ISSILENT);
        if(d<xmax) {
          fFADC->GetCanvas()->cd(1);
          TMarker *tmp = new TMarker(d,SILSIGTHRESH,23);
          tmp->SetMarkerColor(ISSILENT);
          tmp->Draw();
          fMarker.push_back(tmp);
        }
      }
    }
  } else {
    /**************************** Horizontal drawing ****************************/
    if(fHorMap && gROOT->FindObject("HorMap")) delete fHorMap;
    for(int i=0; i<MAXSTAT; i++)
      if(fHorStat[i]) {
        delete fHorStat[i];
        fHorStat[i] = NULL;
      }

    fFADC->GetCanvas()->Clear();
    fFADC->GetCanvas()->Divide(1,2);
    fFADC->GetCanvas()->cd(1);
    //fFADC->GetCanvas()->SetFillColor(50);
#define NLEVELS 50
    double dLog10i, maxval, levels[NLEVELS];
    if (fConfig.GetInt("Like core contours")) {
      fHorMap = new TH2F("HorMap","Likelihood L(E=E0,x,y)",51,
			 -fEvent->fEDdmax,fEvent->fEDdmax, 51,
			 -fEvent->fEDdmax,fEvent->fEDdmax);
      for(int ix=0; ix<52; ix++)
        for(int iy=0; iy<52; iy++) {
          double sig = fEvent->fEDMapMu[ix][iy];
          fHorMap->SetBinContent(ix,iy,sig);
        }
      //maxval = fHorMap->GetMaximum(100000);
      maxval = fHorMap->GetMaximum();
      dLog10i = log10(maxval/fHorMap->GetMinimum())/(NLEVELS-1.);
      for (int i=0; i<NLEVELS; i++)
        levels[i]=fHorMap->GetMinimum()*pow(10,i*dLog10i);
      fHorMap->SetFillColor(46);
      fHorMap->SetContour(NLEVELS,levels);
      fHorMap->SetStats(kFALSE);
      fHorMap->Draw("surf3");
      fFADC->GetCanvas()->Update();
    } else if (fConfig.GetInt("NMu Contours")) {
      gStyle->SetPalette(1);
      fHorMap = new TH2F("HorMap","Muon map N(E=E0,x=xc,y=yc) at ground", 51, 
			 -fEvent->fEDdmax, fEvent->fEDdmax, 51,
			 -fEvent->fEDdmax, fEvent->fEDdmax);
      for(int ix=0; ix<52; ix++) {
        for(int iy=0; iy<52; iy++) {
          double sig = fEvent->fEDMapMu[ix][iy];
          fHorMap->SetBinContent(ix,iy,sig);
        }
      }
      //maxval = fHorMap->GetMaximum(100000);
      maxval = fHorMap->GetMaximum();
      dLog10i = log10(maxval/1.)/(NLEVELS-1.);
      for (int i=0; i<NLEVELS; i++)
        levels[i]=0.5*pow(10,i*dLog10i);
      fHorMap->SetFillColor(46);
      fHorMap->SetContour(NLEVELS,levels);
      fHorMap->SetStats(kFALSE);
      fHorMap->Draw("contz");
      unsigned int is;
      for(is=0; is<size; is++) {
        double dx = fEvent->fSelectedStations[is]->fX - fEvent->fRec.fXCore;
        double dy = fEvent->fSelectedStations[is]->fY - fEvent->fRec.fYCore;
        double sig = 200*log10(fEvent->fSelectedStations[is]->fSigInVEM);
        fHorStat[is] = new TEllipse(dx,dy,sig);
        fHorStat[is]->SetFillColor(38);
        fHorStat[is]->SetLineColor(9);
        fHorStat[is]->SetFillStyle(4050);
        fHorStat[is]->Paint();
        fHorStat[is]->Draw("same");
      }
      fFADC->GetCanvas()->Update();
      gStyle->SetPalette(0);
    } else if (fConfig.GetInt("Likelihood contours")) {
      fHorMap = new TH2F("HorMap","Likelihood L(E=E0(x,y),x,y)", 51,
			 -fEvent->fEDdmax,fEvent->fEDdmax, 51,
			 -fEvent->fEDdmax,fEvent->fEDdmax);
      for(int ix=0; ix<52; ix++)
        for(int iy=0; iy<52; iy++) {
          double sig = fEvent->fEDMapMu[ix][iy];
          fHorMap->SetBinContent(ix,iy,sig);
        }
      //maxval = fHorMap->GetMaximum(100000);
      maxval = fHorMap->GetMaximum();
      dLog10i = log10(maxval/fHorMap->GetMinimum())/(NLEVELS-1.);
      for (int i=0; i<NLEVELS; i++)
        levels[i]=fHorMap->GetMinimum()*pow(10,i*dLog10i);
      fHorMap->SetFillColor(46);
      fHorMap->SetContour(NLEVELS,levels);
      fHorMap->SetStats(kFALSE);
      fHorMap->Draw("contz");
      fFADC->GetCanvas()->Update();
      fFADC->GetCanvas()->cd(2);
      fHorMap->SetTitle("E=E0(x,y)");
      for(int ix=0; ix<52; ix++) {
        for(int iy=0; iy<52; iy++) {
          double sig = fEvent->fEDMapMuBis[ix][iy];
          fHorMap->SetBinContent(ix,iy,sig);
        }
      }
      //maxval = fHorMap->GetMaximum(100000);
      maxval = fHorMap->GetMaximum();
      dLog10i = log10(maxval/1.)/(NLEVELS-1.);
      for (int i=0; i<NLEVELS; i++)
        levels[i]=fHorMap->GetMinimum()+i*(maxval-fHorMap->GetMinimum())/(NLEVELS-1.);
      fHorMap->SetFillColor(46);
      fHorMap->SetContour(NLEVELS,levels);
      fHorMap->SetStats(kFALSE);
      fHorMap->Draw("contz");
      fFADC->GetCanvas()->Update();
    }
  }
  /**************************** draw the text ****************************/
  if (fEvent->fRec.Theta() < 1.2217 || !fEvent->fDoHorizontalRec ||
      !(fMenuHorRec->IsEntryChecked(LIKELIHOOD_CONT))) {
    fFADC->GetCanvas()->cd(2);
    gPad->Range(0,0,1,1);
    if(fTextOutput) delete fTextOutput;
    fTextOutput = new TPaveText(0,0,1,1);
    fTextOutput->SetFillColor(19);
    fTextOutput->SetTextFont(32);

    std::ostringstream buf;
    buf << fEvent->UTCDate;
    fTextOutput->AddText(buf.str().c_str());
    buf.str("");
    buf.setf(ios::fixed,ios::floatfield)
      ;
    buf << std::setprecision(0) << "X = " << fEvent->fRec.fXCore << " #pm " << fEvent->fRec.dXCore() << "m" << '\0';
    fTextOutput->AddText(buf.str().c_str());
    buf.str("");
    buf << "Y = " << fEvent->fRec.fYCore << " #pm " << fEvent->fRec.dYCore() <<"m";
    fTextOutput->AddText(buf.str().c_str());
    if( size>3 ) {
      buf.str("");
      buf << std::setprecision(1) << "#Deltat = " << sqrt(dt2-dt*dt) << "ns";
      fTextOutput->AddText(buf.str().c_str());
    }
    fTextOutput->AddText("");
    buf.str("");
    buf << "#theta = " << fEvent->fRec.Theta("deg") << " #pm " << fEvent->fRec.dTheta("deg") << " deg";
    fTextOutput->AddText(buf.str().c_str());
    buf.str("");
    buf << "#varphi = " << fEvent->fRec.Phi("deg") << " #pm " << fEvent->fRec.dPhi("deg") * sin(fEvent->fRec.Theta()) << "/sin(#theta) deg";
    fTextOutput->AddText(buf.str().c_str());
    if( fEvent->fRec.fR) {
      fTextOutput->AddText("");
      buf.str("");
      if( fabs(fEvent->fRec.fR/1000)<500 )
        buf << std::setprecision(1) << "R = " << fEvent->fRec.fR/1000 << " #pm " << fEvent->fRec.dR()/1000 << " km";
      else
        buf << std::setprecision(1) << "R > 500 km";
      fTextOutput->AddText(buf.str().c_str());
    }
    fTextOutput->AddText("");
    buf.str("");
    buf << std::setprecision(1) << "Ra " << fEvent->fRec.Ra() << " deg   --   Dec " << fEvent->fRec.Dec() << " deg";
    fTextOutput->AddText(buf.str().c_str());
    fTextOutput->AddText("");
    if(fEvent->fRec.Theta() < 1.2217 || !fEvent->fDoHorizontalRec) {
      //#ifdef EXPERIMENTAL
      if(fEvent->fRec.fXmax) {
        if(theLpnheRec().XmaxOK()) {
          buf.str("");
          buf << "Xmax (slant) = " << std::setprecision(2) << fEvent->fRec.fXmax/cos(fEvent->fRec.Theta())
          << " g/cm";
          fTextOutput->AddText(buf.str().c_str());
        } else {
          buf.str("");
          buf << "Xmax (bad constraints) = " << std::setprecision(2) << fEvent->fRec.fXmax/cos(fEvent->fRec.Theta())
          << " g/cm";
          fTextOutput->AddText(buf.str().c_str());
        }
      }
      //#endif
      buf.str("");
      if(fEvent->UseInfillRec) // infill
	
      buf << "S(450)= " << std::setprecision(2) << fEvent->fRec.fSRef << " #pm " << fEvent->fRec.dSRef() << " VEM";
      else
      buf << "S(1000)= " << std::setprecision(2) << fEvent->fRec.fSRef << " #pm " << fEvent->fRec.dSRef() << " VEM";
      fTextOutput->AddText(buf.str().c_str());
    }
    buf.str("");
    buf <<"E= " << fEvent->fRec.fE << " EeV #pm " << std::setprecision(0) << (fEvent->fRec.dSRef()/fEvent->fRec.fSRef)*100 << "% (statistical)"; // Using S1000
    fTextOutput->AddText(buf.str().c_str());
    fTextOutput->AddText("");
    fTextOutput->Draw();
    fFADC->GetCanvas()->Update();
  }
  
 /**************************** draw array and the rest ****************************/
#ifdef FD
  fFD->GetCanvas()->cd();
  if(fConfig.GetInt("Hybrid display") && !fConfig.GetInt("Hybrid X3D view")) {
    if(fHybridDisplay && gROOT->FindObject("HybridDisplay")) 
      delete fHybridDisplay;
    fHybridDisplay = NULL;
    fHybridDisplay = new TCanvas("HybridDisplay","Hybrid Display",10,10,800,500);
  }
  if(fConfig.GetInt("Hybrid display") || fConfig.GetInt("Hybrid X3D view"))
    EDHybrid.Draw(fEvent,fConfig.GetInt("Hybrid X3D view"));
#endif

  fEventArray->UnTrigAll();
  fHybridArray->UnTrigAll();
  for(unsigned int i=0; i<size; i++) {
    fEventArray->SetSignal(fEvent->fSelectedStations[i]->Id, fEvent->fSelectedStations[i]->fSigInVEM);
    fHybridArray->SetSignal(fEvent->fSelectedStations[i]->Id, fEvent->fSelectedStations[i]->fSigInVEM);
  }
  //    fEventArray->SetHasData(fEvent->fSelectedStations[i]->Id);
  if(!fConfig.GetInt("Shower plane projection") ) {
    if( !isnan(fEvent->fRec.fE) && fEvent->fRec.fE) {
      fEventArray->SetShower(fEvent);
      fHybridArray->SetShower(fEvent);
      if (fTab->GetCurrent() == 1) 
	for (int i=1;i<5;i++) fFDEye[i]->DrawShower(&fEvent->fRec);
      if (fTab->GetCurrent() == 2)
	for (int i=1;i<5;i++) fFDHybrid[i]->DrawShower(&fEvent->fRec);
    } else {
      fFADC->GetCanvas()->cd(1);
      fTextOutput->Clear();
      fTextOutput->SetTextColor(2);
      fTextOutput->SetTextSize(0.08);
      fTextOutput->AddText("Reconstruction failed!");
      fTextOutput->Draw();
      fFADC->GetCanvas()->Update();
    }
  }

  if(fEvent->Trigger.IsFd()) {
    for (int i=1;i<5;i++) {
      std::ostringstream message;
      message << "Distance to eye " << i << " " << std::setprecision(5) << pow(pow(fEvent->fRec.fXCore-kIoSd::FdX(i),2)+pow(fEvent->fRec.fYCore-kIoSd::FdY(i),2),0.5) << " m, angle " << std::setprecision(4) << atan2(fEvent->fRec.fYCore-kIoSd::FdY(i), fEvent->fRec.fXCore-kIoSd::FdX(i)) << " rad";
      fTextInfo->AddLine(message.str().c_str()) ;
    }
  }

  fEventArray->Update();
  fHybridArray->Update();
}

