#ifndef EDMAINFRAME_H
#define EDMAINFRAME_H

#include <dirent.h>
#include <string>
#include <iostream>
#include <fstream>
#include <sys/types.h>
#include <sys/stat.h>

#include <TROOT.h>
#include <TGLabel.h>
#include <TGMenu.h>
#include <TGTextEntry.h>
#include <TGTextView.h>
#include <TG3DLine.h>
#include <TGClient.h>
#include <TGButton.h>
#include <TGTab.h>
#include <TGFileDialog.h>
#include <TGListBox.h>
#include <TGraph.h>
#include <TGraphErrors.h>
#include <TPaveLabel.h>
#include <TPaveText.h>
#include <TArrow.h>
#include <TH1.h>
#include <TH2.h>
#include <TF1.h>
#include <TGProgressBar.h>
#include <TLegend.h>
#include <TArrow.h>

#include "EDArray.h"
#include "EDEye.h"
#include "EDConfig.h"
#include "EDDataFilePath.h"
#include "ED_Files.h"
#include "EDExpert.h"
#include "EDTrigger.h"
#include "EDSelection.h"
#include "TCdFReconstruct.h"
#ifdef FD
#include "EDHybrid.h"
#endif

#define MAXNBOFEVENTS 500000
#define NBBIN 150

#ifndef VERSION
#define VERSION "standalone version"
#endif

enum ETHistCommandsId {
  ED_FILE_DATAFILEPATH,
  ED_FILE_LOAD,
  ED_FILE_LOAD_CURRENT,
  ED_FILE_LOAD_LAST,
  ED_FILE_LOAD_DAY,
  ED_FILE_LOAD_TWO_DAY,
  ED_FILE_LOAD_WEEK,
  ED_FILE_LOAD_MONTH,
  ED_FILE_LOAD_ALL,
  ED_FILE_LOAD_PERIOD,

  ED_FILEBUTTON,
  ED_NEXTEVENT,
  ED_PREVEVENT,
  ED_EVENTSEL,
  ED_STATIONSEL,
  ED_PMTONLY,
  ED_PRINTCALIB,
  ED_SHOWNAME,
  ED_BAD_STATIONS,
  ED_FILEBUTTONUPDATE,
  ED_SHOW_T2,
  ED_SHOW_T3,
  ED_RECONSTRUCT,
  ED_ANIMATE,
  ED_3DMAP,
  ED_REC_REPULSIVE,
  ED_REC_CURVE,

  ED_EXPERT_MODIFY,
  ED_CHOOSE_TRIGGER,
  ED_GET_FILE,
  // old triggers
  TRIG_ALL,
  TRIG_FD,
  TRIG_LASER,
  TRIG_OBSERVER,
  TRIG_RANDOM,
  // new triggers
  P_EA_A,
  P_EA_B,
  P_EA_D,
  P_FA_D,
  P_EA_E,
  P_EA_F,
  TOT_EA,
  // horizontal rec. options
  PROT_TABLE,
  IRON_TABLE,
  NMU_CONT,
  LIKECORE_CONT,
  LIKELIHOOD_CONT,

  ED_STARTHELP,
  ED_SAVECONFIG,
  ED_FILE_QUIT,
};

class EDFileSelect;
class EDExpertFrame;
class EDTrigger;
class EDSelectStation;
class EDDataFilePath;
class EDSelectionFrame;

extern std::string startEvent;
extern std::string gDataFilePath;

class EDMainFrame : public TGMainFrame {

public:
  AugerIoSd *fFiles;

#ifdef FD
  TEDHybrid EDHybrid;
#endif
  TRootEmbeddedCanvas *fFADC,*fFD;
  EDEyeCanvas *fFDEye[5];
  EDEyeEmbeddedCanvas *fFDHybrid[5];
  TGCompositeFrame *tf;
  TGPopupMenu *RootFilePopMenu, *fTriggerPopMenu, *fMenuRecLpnhe, 
    *fMenuRecCdF, *fMenuRecPB, *fMenuHorRec, *fMenuRand, *fExpertMenu, 
    *fConfigMenu, *fMenuMultiSel, *fMenuSel, *fRecTypSel;
  TGHorizontalFrame *fControl;
  TGGroupFrame *fF2, *fF5, *fF6, *fF7; 
  TGHorizontalFrame *fHControl, *fF3, *fHF_ForFile, *fHF_Evt, *fF8, *fF9, *fFa ;
  TGVerticalFrame *fF4;
  TGMenuBar *MenuBar_one_station;

  // Tab
  TGTab          *fTab;       // GUI tabular environment
  TGTabElement   *fTabElement;

  // 1st Tab
  TGRadioButton *fStationDefault, *fStationNumber;
  TGCheckButton *fShowT2, *fShowT3, *fShowHelp;

  // 2nd Tab
  TH1F           fFADCHisto[6], *fSuperpPmt;
  TPaveText      *fPMTPave[3], *fTextOutput, *fSuperpPmtPave;
  TLegend        *fLeg;
  TArrow         *fArrow[6], *fArrowSat;
  TLine          *fArrowLine[3];
  TGListBox      *fEventListBox;
  TGListBox      *fStationListBox;
  TGTextButton   *fButtonNext, *fButtonPrev, *fButtonFileUpdate, *fButtonRec, *fgetFile;
  TGTextEntry    *fTriggeredStation;
  TGTextView     *fTextInfo;
  TGHProgressBar *fProgressBar;

  const char *fFileName;
  int fCountEvents;
  int fSliding;
  int fCountStations;
  double fGPSOffset;
  bool fLastEventHadFD;

  int fCanvasArrayEvent;
  int fCanvasArrayEventX;
  int fCanvasArrayEventY;

  TF1          *fLDF;
  TGraphErrors *fGraph;
  TGraph       *fGraph2, *fCdFLDF;
  std::vector<TMarker*> fMarker;
  TCanvas      *fHybridDisplay;
  TH2F         *fHorMap;
  TEllipse     *fHorStat[MAXSTAT];

  bool HandleKey(Event_t *);
  void CloseWindow() {
    delete this;
    exit(0);
  }
  void CheckTrig(int);
  void CheckStat(int);
  void CheckHorTable(int);
  Bool_t ProcessMessage(long, long, long);
  void LoadFile();
  int CheckForStereo(EventPos);
  void AddEvent(EventPos, IoSdKey);
  void AddRecEvent(EventPos);
  int AddStation(IoSdStation*);
  void DrawFADC(int, short*, int);
  void DrawFADC(int, int);
  void DrawFADCArrows(int, int);
  void DrawArray();
  void DrawSignals();
  void CallReconstruct();
  int Reconstruct();
  int Select();
  void SearchEvent(long long id = -1);
  void GetEventById (vector <string>, long long);

  void ClearLastFile ();
  void ClearLastEvent ();
  void ClearAll ();

  EDMainFrame(const TGWindow*, UInt_t, UInt_t);
  ~EDMainFrame() {
    _RecordConfig();
    if(fFiles) delete fFiles;
    if(fEvent) delete fEvent;
    if(fLDF) delete fLDF;
    if(fGraph) delete fGraph;
    if(fGraph2) delete fGraph2;
    if(fTextOutput) delete fTextOutput;
  }

  void UpdateFile();
  TGListBox *StationListBox() {
    return fStationListBox;
  }
  float NbTriggeredStation ();
  void DrawTrigger ();
  void DrawRec ();
  void ListStations (unsigned int, unsigned int sort=1);
  void StationSel (unsigned int);
  void EventSelection (EventPos);
  void SetTank(int i,int j) {
    fEventArray->Set(i,j);
    fHybridArray->Set(i,j);
  }

  EDArrayCanvas *fEventArray;
  EDArrayCanvas *fHybridArray;
  EDConfig fConfig;
  TErEvent *fEvent;
  TCalibStation *fStation;
  unsigned int fLastStation;
  std::string fTriggerName;
  std::string fStationName;
  std::vector<unsigned int> fIds2Delete;
  std::vector<char *> fDirectories;
  std::vector<char *> fDir2Check;
  EDSelectionFrame *selectionFrame;

  TCdFReconstruct fCdFRec;
   
 private :

  bool _selectOnce;
  bool _drawArray;
  TReconstruct *_currentRec;
  EventPos _eventPos;
  EventPos _maxEvent;
  EventPos _minEvent;
  EDDataFilePath *_otherWindow;
  EDTrigger *_triggerWindow;
  EDSelectStation *_stationWindow;
  EDFileSelect *_fileSelectWindow;
  EDExpertFrame *_expertWindow;
  IoSdEvent *_event4Stereo;
  void _RecordConfig();
  void _NextEvent (int iter = 1);
  void _PreviousEvent (int iter = 1);
};

#endif
