#include "EDSelection.h"
#include <sstream>

enum MessageId {
  ADDDIRECTORY,
  DELDIRECTORY,
  GET_EVENT,
  CANCEL
};

/*-----------------------------------------------------------------------------*/
EDSelectionFrame::EDSelectionFrame(const TGWindow *p,const TGWindow *main, UInt_t w, UInt_t h, long long id) : TGTransientFrame(p, main, w, h) 
/*-----------------------------------------------------------------------------*/
{

  if(main) fMain = (EDMainFrame *)main;
  else return;
 
  ULong_t colordefault, colorred;
  gClient->GetColorByName("AntiqueWhite3",colordefault);
  gClient->GetColorByName("IndianRed1",colorred);
  
  /******** Layouts ********/
  fMaxXY = new TGLayoutHints(kLHintsExpandX | kLHintsExpandY);
  fMaxX = new TGLayoutHints(kLHintsExpandX);
  fCenter = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY);
  fAll = new TGLayoutHints(kLHintsCenterX | kLHintsCenterY | kLHintsExpandX);
  fLayout = new TGLayoutHints(kLHintsCenterX | kLHintsBottom);
  /*************************/

  fHF = new TGHorizontalFrame(this,GetWidth()/2,25,kChildFrame,fgDefaultFrameBackground);
  fHF->ChangeBackground(colordefault);

  fIdInFile = new TGTextEntry(fHF,new TGTextBuffer(16));
  fIdInFile->Resize(150,fIdInFile->GetDefaultHeight());
  if(!fMain->fDirectories.size()) fIdInFile->SetText("Data directory missing...");
  else if (id != -1) {
    char tmp[64];
    snprintf(tmp,64,"%llu",id);
    fIdInFile->SetText(tmp);
  }
  
  fGetEvent = new TGTextButton(fHF,"&Get event #",GET_EVENT);
  fGetEvent->Associate(this);
  fGetEvent->ChangeBackground(colorred);

  fHF->AddFrame(fGetEvent);
  fHF->AddFrame(fIdInFile, fCenter);
  AddFrame(fHF,fCenter);


  /********** directories stuff **********/
  unsigned int size = fMain->fDirectories.size();
  if (size) { 
    fGF1 = new TGGroupFrame(this," Search event in... ", kVerticalFrame);
    fGF1->ChangeBackground(colordefault);
    
    for (unsigned int i=0; i<size; i++) {
      TGCheckButton *tmp = fConfig.CheckButton(fGF1,fMain->fDirectories[i],"0");
      for (unsigned int j=0; j<fMain->fDir2Check.size(); j++)
	if(!strcmp(fMain->fDir2Check[j],fMain->fDirectories[i]))
	  tmp->SetState(kButtonDown);
      
      tmp->Associate(this);
      tmp->ChangeBackground(colordefault);
      fGF1->AddFrame(tmp,fMaxX);
      fCheckDir.push_back(tmp);
    }
    AddFrame(fGF1,fMaxX);
  }
  
  fGF2 = new TGGroupFrame(this, (const char*)NULL, kChildFrame);
  fGF2->ChangeBackground(colordefault);
  
  fAddDir = new TGTextButton(fGF2, " Add a new data directory to search in... ",ADDDIRECTORY);
  fAddDir->Associate(this); fAddDir->ChangeBackground(colorred);
  fDelDir = new TGTextButton(fGF2, " Delete checked directories... ",DELDIRECTORY);
  fDelDir->Associate(this); fDelDir->ChangeBackground(colorred);
  fGF2->AddFrame(fAddDir,fAll);
  fGF2->AddFrame(fDelDir,fAll);
  AddFrame(fGF2,fMaxXY);
  
  fCancel = new TGTextButton(this, "  Done  ",CANCEL);
  fCancel->Associate(this);
  fCancel->ChangeBackground(colorred);
  AddFrame(fCancel, fLayout);
 
  MapSubwindows();
  Layout();
  if (main) {
    int ax, ay;
    Window_t wdummy;
    gVirtualX->
      TranslateCoordinates(main->GetId(), GetParent()->GetId(),
			   (Int_t)(((TGFrame *) main)->GetWidth() - fWidth) >> 1,
			   (Int_t)(((TGFrame *) main)->GetHeight() - fHeight) >> 1,
			   ax, ay, wdummy);
    if (ax < 0) ax = 10;
    if (ay < 0) ay = 10;
    Move(ax, ay);
    SetWMPosition(ax, ay);
  }

  ChangeBackground(colordefault);
  SetWindowName("Selection Menu");
  SetIconName("S. M.");
  MapWindow();
  fClient->WaitFor(this);

};

/*-----------------------------------------------------------------------------*/
bool EDSelectionFrame::ProcessMessage(long msg, long parm1, long) 
/*-----------------------------------------------------------------------------*/
{
  switch (GET_MSG(msg)) {
    
  case kC_COMMAND:
    switch (GET_SUBMSG(msg)) {
    case kCM_BUTTON:
      if(parm1==ADDDIRECTORY) {
	static int againmychild=0;
	const char* gOpenAsTypes[] = {
	  "directories",   "*/",
	  "all files",   "*.*",
	  0,              0
	};
	static TGFileInfo fichild;
	fichild.fFileTypes = gOpenAsTypes;
	if(!againmychild) {
	  fichild.fIniDir = strdup(gDataFilePath.c_str());//getenv("HOME"));
	  againmychild=1;
	}
	new TGFileDialog(gClient->GetRoot(), gClient->GetRoot(), kFDOpen, &fichild);
	if(fichild.fFilename) {
	  fMain->fDirectories.push_back(fichild.fFilename);
	  fMain->fDir2Check.push_back(fichild.fFilename);
	}
	break;
      }
      if(parm1==DELDIRECTORY) {
	for (unsigned int i=0; i<fCheckDir.size(); i++) {
	  if (fCheckDir[i]->GetState()== kButtonDown) {
	    for (unsigned int j=0; j<fMain->fDir2Check.size(); j++) 
	      if(fConfig.GetSig(fMain->fDir2Check[j])==fCheckDir[i]->WidgetId())
		fMain->fDir2Check.erase(fMain->fDir2Check.begin()+j);
	    for (unsigned int j=0; j<fMain->fDirectories.size(); j++) 
	      if(fConfig.GetSig(fMain->fDirectories[j])==fCheckDir[i]->WidgetId())
		fMain->fDirectories.erase(fMain->fDirectories.begin()+j);
	  }
	}
	CloseWindow();
	break;
      }
      if(parm1==GET_EVENT) {
	std::string text;
	text = fIdInFile->GetBuffer()->GetString();
	if(text=="") break;
	long long id = atoll(text.c_str());
	fMain->fTextInfo->Clear();
	unsigned int size = fMain->fDir2Check.size();
	vector <string> files;
	files.reserve(size);
	for(unsigned int i=0; i<size; i++) {
	  string tmp;
	  tmp = FromId2File(fMain->fDir2Check[i], id);  
	  if(tmp!="") files.push_back(tmp);
 	}
	fMain->GetEventById(files,id); 
        break;
      }
      if(parm1==CANCEL) {
	CloseWindow();
        break;
      }
      break;//break for kCM_BUTTON:
      
    case kCM_CHECKBUTTON:
      for (unsigned int i=0; i<fMain->fDirectories.size(); i++) {
	if(parm1 == fConfig.GetSig(fMain->fDirectories[i])) {
	  if (fCheckDir[i]->GetState()== kButtonDown)  {//checked
	    if(!fMain->fDir2Check.size()) 
	      fMain->fDir2Check.push_back(fMain->fDirectories[i]);
	    else {
	      bool found = false; 
	      for (unsigned int j=0; j<fMain->fDir2Check.size(); j++) 
		if (!strcmp(fMain->fDir2Check[j],fMain->fDirectories[i])) found = true;
	      if (!found) fMain->fDir2Check.push_back(fMain->fDirectories[i]);
	    }
	  }
	  else if (fCheckDir[i]->GetState()== kButtonUp)  {//unchecked
	    for (unsigned int j=0; j<fMain->fDir2Check.size(); j++)
	      if(!strcmp(fMain->fDir2Check[j],fMain->fDirectories[i]))
		fMain->fDir2Check.erase(fMain->fDir2Check.begin()+j);
	  }
	  
	}
      }
      break;//break for kCM_CHECKBUTTON:
      
    default:
      break;
      
    }// end case kC_COMMAND
  default:
    break;
    
  }
  return kTRUE;
}


/*-----------------------------------------------------------------------------*/
string EDSelectionFrame::FromId2File (const char *directory, long long eventId, int redo) 
/*-----------------------------------------------------------------------------*/
{
  string input = "";
  char tmp[128];
  fMain->fTriggeredStation->SetText("0");// if selected event has less stations than required
  if (eventId/1000000>200000) { // this division is for some stupid compilers not accepting large constants
    // new Auger Id. Easier to find
    input = GetFileName(eventId,directory); // memory leak?
    if(input=="") {
      snprintf(tmp,128,"New id %llu not found in directory %s.",eventId,directory);
      fMain->fTextInfo->AddLine(tmp);
      return input;
    }
    snprintf(tmp,128,"New id %llu found in directory %s.",eventId,directory);
    fMain->fTextInfo->AddLine(tmp);
  } 
  else {// treating usual Sd ids...
    ifstream in;
    unsigned int Id = 0;
    string dir = directory;
    unsigned int pos = dir.find("/");
    while(pos<=dir.length()) {
      dir.replace(pos,1,"_");
      pos = dir.find("/");    
    }
    input=strdup(EDSRCDIR)+(string)"."+dir;
    in.open(input.c_str());
    if(!in && redo) {
      fMain->fTextInfo->AddLine("Could not create database. Can you write in ");
      input = strdup(EDSRCDIR)+(string)"  ?";
      fMain->fTextInfo->AddLine(input.c_str());
      return (string)"";
    } else if(!in) {
      snprintf(tmp,128,"No database for directory %s, creating it...",directory);
      fMain->fTextInfo->AddLine(tmp);
      AssociateId2File((string)directory);
      return FromId2File(directory, eventId, 1);
    }
    unsigned int previous;
    bool found = false;
    while (1) {
       previous = Id;
       in>>Id;
       if(previous==Id) {
         if (eventId>Id && redo) break; // current file?
	 snprintf(tmp,128,"Id %llu not found in current database of directory",eventId);
         fMain->fTextInfo->AddLine(tmp);
	 snprintf(tmp,128,"%s.",directory);
	 fMain->fTextInfo->AddLine(tmp);
         in.close();
         if(redo) return (string)"";
         fMain->fTextInfo->AddLine("Updating it...");
         AssociateId2File((string)directory);
         return FromId2File(directory,eventId,1);
       }
       if(!previous && Id>eventId) {
	 fMain->fTextInfo->AddLine("Requested Id smaller than the first Id of the list.");
	 in.close();
	 return (string)"";
       }
       if(previous<=eventId && eventId<Id) {found = true; break;}
       in>>input;
    }
    in.close(); 
    if (found) {
      snprintf(tmp,128,"Id %llu found in directory %s.",eventId,directory);
      fMain->fTextInfo->AddLine(tmp);
    }
    else {
      snprintf(tmp,128,"Id %llu not found in directory %s.",eventId,directory);
      fMain->fTextInfo->AddLine(tmp);
    }

  }
  return input;
}

/*-----------------------------------------------------------------------------*/
void EDSelectionFrame::AssociateId2File (string directory) 
/*-----------------------------------------------------------------------------*/
{
  ifstream in;
  string file_tmp;
  unsigned int Id;
  unsigned int size;
  
  string dir = directory;
  unsigned int pos = dir.find("/");
  while(pos<=dir.length()) {
    dir.replace(pos,1,"_");
    pos = dir.find("/");    
  }
  string input=strdup(EDSRCDIR)+(string)"."+dir;
  
  in.open(input.c_str());
  size = BuildAllRootFiles(directory);
  fMain->fProgressBar->Reset();
  fMain->fProgressBar->SetRange(0, size);
  
  int flag=0;
  ofstream out(input.c_str(),ios::app);
  if(in) {
    while (in>>Id) in>>file_tmp;
    in.close();
  } 
  else {
    chmod(input.c_str(),0666);
    flag=1;
  }
  RFile last(file_tmp.c_str());
  
  for(unsigned int i=0; i<size; i++) {
    fMain->fProgressBar->Increment(1);
    if(flag) {
      if(fMain->fFiles) delete fMain->fFiles;
      fMain->fFiles = new AugerIoSd((char *)(gAllRootFiles[i].filename()));
      if(fMain->fFiles->NumberOfEvents()) {
	IoSdEvent event(fMain->fFiles->FirstEvent());
	out << (long long)event.AugerId << " " << gAllRootFiles[i].filename() << endl;
	if(i==size-1){
	  IoSdEvent lastevent(fMain->fFiles->LastEvent()-1);
	  out << (long long)lastevent.AugerId << " " << gAllRootFiles[i].filename() << endl;
	}
      }
    } else if(last==gAllRootFiles[i]) flag=1;
  }
  out.close();
  fMain->fProgressBar->SetPosition(size);
}
