#include "TCanvas.h"
#include "TRandom.h"
#include "TPolyMarker.h"
#include "TMarker.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TPolyLine.h"
#include "TColor.h"
#include "TBox.h"
#include "TROOT.h"
#include <vector>
#include <iostream>

#include "Er.h"

float L = 10000.;
float H = 3000.;
float SINA = .25;
float COSA = sqrt(1-SINA*SINA);

float XGraph(float x, float y, float z) {
  return L*x/(L+y*COSA-z*SINA);
}
float YGraph(float x, float y, float z) {
  x+=0;
  return L*(z*COSA+(y-H)*SINA)/(L+y*COSA-z*SINA);
}

void ShowerAnimate(TErEvent *ev,int save) {
  float theta=ev->fRec.Theta();
  float phi=ev->fRec.Phi();

  double day=(((ev->Trigger.Second-3*3600)%86400)-43200.)/86400.; // UTC to Argentina time difference
  day=1-2*fabs(day);
  std::vector<float> XSTA;
  std::vector<float> YSTA;
  std::vector<int> ISINEVENT;
  for (unsigned int i=0;i<ev->Trigger.NumberOfStation;i++) {
    XSTA.push_back(ev->Stations[i].Easting-ev->fRec.Easting());
    YSTA.push_back(ev->Stations[i].Northing-ev->fRec.Northing());
    int isok=0;
    for (unsigned int j=0;j<ev->Trigger.NumberOfStation;j++)
      if(ev->Stations[j].Id == ev->Stations[i].Id)
        isok=1;
    ISINEVENT.push_back(isok);
  }
  float PI = 3.1416;

  int NSTAT=ev->Trigger.NumberOfStation;
  int NPMAX = (int)(100*ev->fRec.fE);
  if (NPMAX>10000)
    NPMAX=10000;
  int NTNEG=(int)(100./cos(theta));
  int NTPOS=(int)(100./cos(theta));
  if (NTNEG>200)
    NTNEG=200;
  if (NTPOS>200)
    NTPOS=200;

  std::vector<TPolyMarker*> GrdPcles;
  std::vector<TEllipse*> EllClean;
  std::vector<TBox*> BoxClean;
  std::vector<TLine*> LinClean;
  float C = .2998;
  float DT = 100.;
  int NCSTEP = 2;
  int NMEM = 3;
  float LFRONT = 2000;


  float RSTA = 200;
  float HSTA = 100.;


  float cth=cos(theta);
  float sth=sin(theta);
  float cph=cos(phi);
  float sph=sin(phi);

  float XMIN = -5000;
  float XMAX = 5000;
  float YMIN = -3500;
  float YMAX = 4500;

  float xp[4][10000],yp[4][10000],xps[4][10000],yps[4][10000],
  xm[4][10000],ym[4][10000],xpsp[4*10000],ypsp[4*10000];
  TRandom rg;
  TEllipse* elc[500];

  //  graphical objects
  TPolyMarker *pm[4];
  TPolyMarker *pmmu[4];
  for(int i=0; i<4; i++) {
    pm[i] = new TPolyMarker(0);
    pmmu[i] = new TPolyMarker(0);
    GrdPcles.push_back(pm[i]);
    GrdPcles.push_back(pmmu[i]);
  }
  TLine axe;
  double xfr[5],yfr[5],xfru[5],yfru[5];
  TPolyLine *plfr=NULL, *plfru=NULL;
  TLine *lfr=NULL;
  //  colors
  TBox bxt[50], bxc[50];
  TColor *colc[50], *colt[50];
  float levR,levG,levB,xc,yc,zc,xx,yy,zz,xgr,ygr;
  //  variables
  int npt,npsp,nc,nl,nmk[2],nptmu;
  int np[4] = {0,0,0,0};
  int nps[4] = {0,0,0,0};
  int nmu[4] = {0,0,0,0};
  //
  //   background  ------------------------------------------------------
  TPad* padm = new TPad("padm","main pad",0.,0.,1.,1.);
  padm->Draw();
  padm->cd();
  gPad->Range(XMIN*1.01,YMIN*1.01,XMAX*1.01,YMAX*1.01);
  //gPad->SetFillColor(kBlack);

  float ygr_horiz = L*SINA/COSA;

  int ncol=25; // root uses various number colors. Using 50->100 and 200->228
  int icoltnk = 203;
  TColor *coltnk = gROOT->GetColor(icoltnk);
  coltnk->SetRGB(1.,1.,.5);
  for(int i=0; i<2*ncol; i++) {
    colt[i] = gROOT->GetColor(100-i);
    bxt[i].SetX1(XMIN);
    bxt[i].SetX2(XMAX);
    bxt[i].SetY1(ygr_horiz+i/(1.*2*ncol)*(YMIN-ygr_horiz));
    bxt[i].SetY2(ygr_horiz+(i+1)/(1.*2*ncol)*(YMIN-ygr_horiz));
    levR = (1+3*day)*i/(6.*2*ncol);
    levG = levR;
    levB = levR/3.;
    colt[i]->SetRGB(levR,levG,levB);
    bxt[i].SetFillColor(100-i);
    bxt[i].Draw();

    if (i<ncol) {
      colc[i] = gROOT->GetColor(228-i);
      bxc[i].SetX1(XMIN);
      bxc[i].SetX2(XMAX);
      bxc[i].SetY1(ygr_horiz+i/(1.*ncol)*(YMAX-ygr_horiz));
      bxc[i].SetY2(ygr_horiz+(i+1)/(1.*ncol)*(YMAX-ygr_horiz));
      levB = day/2+(ncol-i)/(6.*ncol);
      levG = levB;
      levR = levB/3.;
      colc[i]->SetRGB(levR,levG,levB);
      bxc[i].SetFillColor(228-i);
      bxc[i].Draw();
    }
  }
  //
  //   ground stations  ---------------------------------
  float xgrsta,ygrsta,hgrsta,r1sta,r2sta;
  for(int i=0; i<NSTAT; i++) {
    xgrsta = XGraph(XSTA[i],YSTA[i],0);
    ygrsta = YGraph(XSTA[i],YSTA[i],0);
    r1sta = RSTA * L/(L+YSTA[i]*COSA);
    r2sta = RSTA * L*SINA*(L+H*COSA)/pow(L+YSTA[i]*COSA,2);
    hgrsta = HSTA *
             L*(L*COSA+H*SINA+YSTA[i]*(COSA*COSA-SINA))/pow(L+YSTA[i]*COSA,2);

    TEllipse* elbot = new TEllipse(xgrsta,ygrsta,r1sta,r2sta);
    EllClean.push_back(elbot);
    elbot->SetLineColor(22);
    if (ISINEVENT[i])
      elbot->SetFillColor(31);
    else
      elbot->SetFillColor(kBlack);
    elbot->Draw();

    TBox* bxsta = new TBox(double(xgrsta-r1sta),double(ygrsta),
                           double(xgrsta+r1sta),double(ygrsta+hgrsta));
    BoxClean.push_back(bxsta);
    if (ISINEVENT[i])
      bxsta->SetFillColor(31);
    else
      bxsta->SetFillColor(kBlack);
    bxsta->Draw();

    TEllipse* eltop = new TEllipse(xgrsta,ygrsta+hgrsta,r1sta,r2sta);
    EllClean.push_back(eltop);
    eltop->SetLineColor(22);
    if (ISINEVENT[i])
      eltop->SetFillColor(31);
    else
      eltop->SetFillColor(kBlack);
    eltop->Draw();
    TLine* lnsta = new TLine(xgrsta-r1sta,ygrsta,xgrsta-r1sta,ygrsta+hgrsta);
    LinClean.push_back(lnsta);
    lnsta->SetLineColor(22);
    lnsta->Draw();
    lnsta = new TLine(xgrsta+r1sta,ygrsta,xgrsta+r1sta,ygrsta+hgrsta);
    LinClean.push_back(lnsta);
    lnsta->SetLineColor(22);
    lnsta->Draw();
  }
  //
  //  axes and planes  ------------------------------------------
  //  starting point
  float length = NTNEG*C*DT;
  xx = length*sth*cph;
  yy = length*sth*sph;
  zz = length*cth;
  xgr = XGraph(xx,yy,zz);
  ygr = YGraph(xx,yy,zz);
  axe.SetX1(0.);
  axe.SetY1(-H*SINA);
  axe.SetX2(xgr);
  axe.SetY2(ygr);
  axe.SetLineStyle(2);
  axe.SetLineColor(kWhite);
  axe.Draw();
#ifdef FLUO
  // fluorescence variables
  int NLSTEP = 10;
  int NCMAX = NTNEG/NCSTEP+1;
  int NLMAX = NTNEG/NLSTEP+1;
  float RTEL=500.;
  float ATTEN_L = 10000.;

  std::vector<float> XTEL;
  std::vector<float> YTEL;
  XTEL.push_back(10000);
  YTEL.push_back(10000);
  int NTEL=1;

  float dx[2][100],dy[2][100],dz[2][100],
  xl[2][100],yl[2][100],zl[2][100],
  xf[2][100],yf[2][100],zf[2][100],
  dist[2][100],al[2][100];
  Bool_t act[2][100];
  Bool_t thit[2] = {
                     kFALSE, kFALSE
                   };
  TMarker* tmk[2][100];
  TLine *ll[2][100],*lt[2][100];
  TPad* pad[2];
  float xgrtel[61],ygrtel[61];
  TLine axt[2],axtp[2];

  //  shower-telescope plane
  for(int i=0; i<NTEL; i++) {
    xgr = L*XTEL[i]/(L+YTEL[i]*COSA);
    ygr = L*(YTEL[i]-H)*SINA/(L+YTEL[i]*COSA);
    axt[i].SetX1(xgr);
    axtp[i].SetX1(xgr);
    axt[i].SetY1(ygr);
    axtp[i].SetY1(ygr);
  }
  for(int i=0; i<NTEL; i++) {
    axt[i].SetX2(xgr);
    axt[i].SetY2(ygr);
    axtp[i].SetX2(0.);
    axtp[i].SetY2(-H*SINA);
  }
  for(int i=0; i<NTEL; i++) {
    axt[i].SetLineStyle(2);
    axt[i].SetLineColor(9);
    axtp[i].SetLineStyle(2);
    axtp[i].SetLineColor(9);
    axt[i].Draw();
    axtp[i].Draw();
  }
  //
  //   telescopes  --------------------------------------
  for(int i=0; i<NTEL; i++) {
    for(int iph=0; iph<25; iph++) {
      float ph_tel = PI*iph/12;
      float cph_tel = cos(ph_tel);
      float sph_tel = sin(ph_tel);
      for (int ith=0; ith<5; ith++) {
        float cth_tel = cos(PI*ith/16);
        xx = XTEL[i] + RTEL*cth_tel*cph_tel;
        yy = YTEL[i] + RTEL*cth_tel*sph_tel;
        zz = RTEL*sqrt(1-cth_tel*cth_tel);
        xgrtel[ith] = XGraph(xx,yy,zz);
        ygrtel[ith] = YGraph(xx,yy,zz);
      }
      TPolyLine* tpl = new TPolyLine(5,xgrtel,ygrtel);
      tpl->SetLineColor(13);
      tpl->Draw();
    }
    //   parallels
    for (int ith=0; ith<5; ith++) {
      float th_tel = PI*ith/16;
      float cth_tel = cos(th_tel);
      float sth_tel = sin(th_tel);
      for (int iph=0; iph<25; iph++) {
        float ph_tel = PI*iph/12;
        xx = XTEL[i] + RTEL*cth_tel*cos(ph_tel);
        yy = YTEL[i] + RTEL*cth_tel*sin(ph_tel);
        zz = RTEL*sth_tel;
        xgrtel[iph] = L*xx/(L+yy*COSA-zz*SINA);
        ygrtel[iph] = L*(zz*COSA+(yy-H)*SINA)/(L+yy*COSA-zz*SINA);
      }
      TPolyLine* tpl = new TPolyLine(25,xgrtel,ygrtel);
      tpl->SetLineColor(13);
      tpl->Draw();
    }
  }
#endif
  padm->Update();

  //
  //   progression by steps  -------------------------------------------------
  nc=0;
  nl = 0;
  npsp = 0;
  nmk[0]=nmk[1]=0;
  //
  for (int it=-NTNEG+1; it<NTPOS; it++) {
    float distl = it*DT*C;
    xc = -distl*sth*cph;
    yc = -distl*sth*sph;
    zc = -distl*cth;
    float depth = 900/cth + .12*distl;
    float curv = .09/depth;
    float rlat = 400*pow(depth/900,2);
    int ipm = (it+NTNEG)%4;
    float alight;
    //
    //   parametrization of intensity -----------------------
    if (depth<1000) {
      npt = (int)(NPMAX*exp(-pow((depth-1000)/300,2)));
      nptmu = (NPMAX/10);
      alight = 5*exp(-pow((depth-1000)/300,2));
    } else {
      nptmu = (int)(NPMAX*100/depth);
      npt = (int)(NPMAX*exp(-(depth-1000)/300));
      alight = 5*exp(-(depth-1000)/200);
    }
    //
    //   particles  -----------------------------------------
    np[ipm] = 0;
    nps[ipm] = 0;
    nmu[ipm] = 0;
    for (int i=0; i<npt+nptmu; i++) {
      float rad;
      if (i>=npt)
        rlat=500;
      do {
        rad = -rlat*(log(rg.Rndm()));
      } while(rad<20*alight);
      float azi = 2*PI*rg.Rndm();
      float xsh = rad*cos(azi);
      float ysh = rad*sin(azi);
      float zsh = -distl + curv*rad*rad;
      xx =  xsh*cth*cph - ysh*sph + zsh*sth*cph;
      yy =  xsh*cth*sph + ysh*cph + zsh*sth*sph;
      zz = -xsh*sth               + zsh*cth;
      xgr = XGraph(xx,yy,zz);
      ygr = YGraph(xx,yy,zz);
      //  above ground
      if(zz>HSTA) {
        if (i<npt) {
          xp[ipm][np[ipm]] = xgr;
          yp[ipm][np[ipm]] = ygr;
          np[ipm]++;
        } else {
          xm[ipm][nmu[ipm]] = xgr;
          ym[ipm][nmu[ipm]] = ygr;
          nmu[ipm]++;
        }
        //  on the ground (within station height)
      } else if(zz>0) {
        for (int is=0; is<NSTAT; is++) {
          if(pow(xx-XSTA[is],2)+pow(yy-YSTA[is],2) < RSTA*RSTA) {
            xps[ipm][nps[ipm]] = xgr;
            yps[ipm][nps[ipm]] = ygr;
            nps[ipm]++;
          }
        }
      }
    }
    //   particles in atmosphere
    pm[ipm]->SetPolyMarker(np[ipm],xp[ipm],yp[ipm]);
    pm[ipm]->SetMarkerColor(kGreen);
    pm[ipm]->Draw();
    pmmu[ipm]->SetPolyMarker(nmu[ipm],xm[ipm],ym[ipm]);
    pmmu[ipm]->SetMarkerColor(kRed);
    pmmu[ipm]->Draw();
    // particles at ground
    TPolyMarker *tmppol = new TPolyMarker(0);
    GrdPcles.push_back(tmppol);
    tmppol->SetPolyMarker(nps[ipm],xps[ipm],yps[ipm]);
    tmppol->SetMarkerStyle(20);
    tmppol->SetMarkerSize(.5);
    tmppol->SetMarkerColor(icoltnk);
    tmppol->Draw();
    //
#ifdef FLUO
    //   fluorescence  -------------------------------
    //  new ray starting from axis ---------
    if(it<0 && it%NLSTEP==0) {
      //
      //   light rays
      for(int i=0; i<NTEL; i++) {
        act[i][nl] = kTRUE;
        al[i][nl] = alight;
        xgr = L*xc/(L+yc*COSA-zc*SINA);
        ygr = L*(zc*COSA+(yc-H)*SINA)/(L+yc*COSA-zc*SINA);
        xl[i][nl] = xc;
        yl[i][nl] = yc;
        zl[i][nl] = zc;
        float dl = sqrt(pow(xc-XTEL[i],2)+pow(yc-YTEL[i],2)+pow(zc,2));
        dist[i][nl] = dl;
        dx[i][nl] = C*DT*(XTEL[i]-xc)/dl;
        dy[i][nl] = C*DT*(YTEL[i]-yc)/dl;
        dz[i][nl] = C*DT*(-zc)/dl;
        xf[i][nl] = xl[i][nl]+5*dx[i][nl];
        yf[i][nl] = yl[i][nl]+5*dy[i][nl];
        zf[i][nl] = zl[i][nl]+5*dz[i][nl];
        ll[i][nl] = new TLine();
        ll[i][nl]->SetLineColor(kGrey);
        ll[i][nl]->SetLineWidth(al[i][nl]);
        ll[i][nl]->Draw();
        lt[i][nl] = new TLine();
        lt[i][nl]->SetX1(xgr);
        lt[i][nl]->SetY1(ygr);
        lt[i][nl]->SetLineColor(9);
        lt[i][nl]->SetLineStyle(3);
        lt[i][nl]->Draw();
      }
      nl++;
    }
    //  moving active rays towards the telescopes -----
    for (int j=0; j<nl; j++) {
      for (int i=0; i<NTEL; i++) {
        if(act[i][j]) {
          if(pow(xf[i][j]-XTEL[i],2)+pow(yf[i][j]-YTEL[i],2)+pow(zf[i][j],2)
              > RTEL*RTEL) {
            xgr = L*xl[i][j]/(L+yl[i][j]*COSA-zl[i][j]*SINA);
            ygr = L*(zl[i][j]*COSA+(yl[i][j]-H)*SINA)
                  /(L+yl[i][j]*COSA-zl[i][j]*SINA);
            lt[i][j]->SetX2(xgr);
            lt[i][j]->SetY2(ygr);

            ll[i][j]->SetX1(xgr);
            ll[i][j]->SetY1(ygr);
            xgr = L*xf[i][j]/(L+yf[i][j]*COSA-zl[i][j]*SINA);
            ygr = L*(zf[i][j]*COSA+(yf[i][j]-H)*SINA)
                  /(L+yf[i][j]*COSA-zf[i][j]*SINA);
            ll[i][j]->SetX2(xgr);
            ll[i][j]->SetY2(ygr);
            xl[i][j] += dx[i][j];
            yl[i][j] += dy[i][j];
            zl[i][j] += dz[i][j];
            xf[i][j] += dx[i][j];
            yf[i][j] += dy[i][j];
            zf[i][j] += dz[i][j];
            al[i][j] *= exp(-C*DT/ATTEN_L);
            ll[i][j]->SetLineWidth(al[i][j]);
          }
          //  if telescope reached: create a spot on it  -------------------------
          else {
            if(zf[i][j]<.71*RTEL) {
              //  first hit: open the side pad and draw the telescope
              if(!thit[i]) {
                thit[i] = kTRUE;
                if(i==0)
                  pad[i] = new TPad("pad1","telescope 1",.01,.68,.41,.98);
                if(i==1)
                  pad[i] = new TPad("pad2","telescope 2",.59,.68,.99,.98);
                int icolpad = 53;
                TColor *colpad = gROOT->GetColor(icolpad);
                colpad->SetRGB(.1,.3,0.);
                pad[i]->SetFillColor(icolpad);
                pad[i]->Draw();
                pad[i]->cd();
                pad[i]->Range(-1.2*RTEL,-.6*RTEL,1.2*RTEL,1.2*RTEL);
                float ph_view = atan2(XTEL[i],YTEL[i]+L);
                //   meridians
                for(int iph=0; iph<61; iph++) {
                  float ph_tel = PI*iph/30;
                  float cph_tel = cos(ph_tel-ph_view);
                  float sph_tel = sin(ph_tel-ph_view);
                  for (int ith=0; ith<8; ith++) {
                    float cth_tel = cos(PI*ith/30);
                    float sth_tel = sqrt(1-cth_tel*cth_tel);
                    xgrtel[ith] = RTEL*cth_tel*cph_tel;
                    ygrtel[ith] = RTEL*(sth_tel*COSA+cth_tel*sph_tel*SINA);
                  }
                  TPolyLine* tpl = new TPolyLine(8,xgrtel,ygrtel);
                  tpl->SetLineColor(13);
                  tpl->Draw();
                }
                //   parallels
                for (int ith=0; ith<8; ith++) {
                  float cth_tel = cos(PI*ith/30);
                  float sth_tel = sqrt(1-cth_tel*cth_tel);
                  for (int iph=0; iph<61; iph++) {
                    float ph_tel = PI*iph/30;
                    float cph_tel = cos(ph_tel-ph_view);
                    float sph_tel = sin(ph_tel-ph_view);
                    xgrtel[iph] = RTEL*cth_tel*cph_tel;
                    ygrtel[iph] = RTEL*(sth_tel*COSA+cth_tel*sph_tel*SINA);
                  }
                  TPolyLine* tpl = new TPolyLine(61,xgrtel,ygrtel);
                  tpl->SetLineColor(13);
                  tpl->Draw();
                }
              }
              if(thit[i])
                pad[i]->cd();
              //  spot  -----------------------------------
              if(nmk[i]) {
                tmk[i][nmk[i]-1]->SetMarkerStyle(24);
              }
              xx = xf[i][j]-XTEL[i];
              yy = yf[i][j]-YTEL[i];
              zz = zf[i][j];
              float norm = sqrt(xx*xx + yy*yy +zz*zz)/RTEL;
              tmk[i][nmk[i]] = new TMarker(xx/norm, (SINA*yy+COSA*zz)/norm, 20);
              int icolpix = 50;
              TColor *colpix = gROOT->GetColor(icolpix);
              colpix->SetRGB(.5,.5,1.);
              tmk[i][nmk[i]]->SetMarkerColor(icolpix);
              tmk[i][nmk[i]]->SetMarkerSize(al[i][j]*pow(7000/dist[i][j],2));
              tmk[i][nmk[i]]->Draw();
              nmk[i]++;
            }
            act[i][j]=kFALSE;
            lt[i][j]->SetX2(ll[i][j]->GetX2());
            lt[i][j]->SetY2(ll[i][j]->GetY2());
            delete ll[i][j];
          }
        }
        padm->cd();
      }
    }
    //
#endif
    //   core  ----------------------------------------------
    if(it<=0 && it%NCSTEP==0) {
      //if(nc) {elc[nc-1]->SetFillStyle(0); elc[nc-1]->SetLineColor(23);}
      if (nc>=NMEM) {
        delete elc[nc-NMEM];
      }
      float den = L+yc*COSA-zc*SINA;
      xgr = L*xc/den;
      ygr = L*(zc*COSA+(yc-H)*SINA)/den;
      float ax1x = -sph;
      float ax2x = cth*cph;
      float ax1y = SINA*cph;
      float ax2y = (COSA*sth+SINA*cth*sph);
      float aa = ax1x*ax1x + ax1y*ax1y;
      float bb = ax2x*ax2x + ax2y*ax2y;
      float cc = ax1x*ax2x + ax1y+ax2y;
      float discr = sqrt(pow(aa-bb,2)+pow(cc,2));
      double r1 = 20*alight*L/den*sqrt(aa+bb+discr);
      double r2 = 20*alight*L/den*sqrt(aa+bb-discr);
      double th_el = atan2(cc,aa-bb);
      elc[nc] = new TEllipse(double(xgr),double(ygr),r1,r2,0,360,th_el);
      elc[nc]->SetFillColor(kGreen);
      elc[nc]->SetLineColor(kGreen);
      elc[nc]->Draw();
      nc++;
    } else if (it%NCSTEP==0 && it<=NMEM*NCSTEP) {
      delete elc[nc-NMEM];
      nc++;
    }
    //  front plane (+-LFRONT, bounded by ground) --------------
    float xlt = xc+LFRONT*( sph-cth*cph);
    float xrt = xc+LFRONT*(-sph-cth*cph);
    float ylt = yc+LFRONT*(-cph-cth*sph);
    float yrt = yc+LFRONT*( cph-cth*sph);
    float zlt = zc+LFRONT*sth;
    float zrt = zlt;
    float xlb = xc+LFRONT*( sph+cth*cph);
    float xrb = xc+LFRONT*(-sph+cth*cph);
    float ylb = yc+LFRONT*(-cph+cth*sph);
    float yrb = yc+LFRONT*( cph+cth*sph);
    float zlb = zc-LFRONT*sth;
    float zrb = zlb;
    float llow = zc/sth;
    float xlg = xc+LFRONT*sph+llow*cth*cph;
    float xrg = xc-LFRONT*sph+llow*cth*cph;
    float ylg = yc-LFRONT*cph+llow*cth*sph;
    float yrg = yc+LFRONT*cph+llow*cth*sph;
    //  above ground -----
    if(plfr)
      delete plfr;
    if(plfru)
      delete plfru;
    if(lfr)
      delete lfr;
    plfr=NULL;
    plfru=NULL;
    lfr=NULL;
    if(llow>-LFRONT) {
      // left-bottom
      xfr[0] = XGraph(xlb,ylb,zlb);
      yfr[0] = YGraph(xlb,ylb,zlb);
      // left-top
      xfr[1] = XGraph(xlt,ylt,zlt);
      yfr[1] = YGraph(xlt,ylt,zlt);
      // right-top
      xfr[2] = XGraph(xrt,yrt,zrt);
      yfr[2] = YGraph(xrt,yrt,zrt);
      // right-bottom
      xfr[3] = XGraph(xrb,yrb,zrb);
      yfr[3] = YGraph(xrb,yrb,zrb);
      if(llow>LFRONT) {
        xfr[4] = xfr[0];
        yfr[4] = yfr[0];
        plfr = new TPolyLine(5,xfr,yfr);
        plfr->SetLineStyle(2);
        plfr->SetLineColor(kWhite);
        plfr->Draw();
      } else {
        xfr[0] = XGraph(xlg,ylg,0);
        yfr[0] = YGraph(xlg,ylg,0);
        xfr[3] = XGraph(xrg,yrg,0);
        yfr[3] = YGraph(xrg,yrg,0);
        plfr = new TPolyLine(4,xfr,yfr);
        plfr->SetLineStyle(2);
        plfr->SetLineColor(kWhite);
        plfr->Draw();
      }
    }
    //  under ground -----
    if(llow<LFRONT) {
      // left-top
      xfru[0] = XGraph(xlt,ylt,zlt);
      yfru[0] = YGraph(xlt,ylt,zlt);
      // left-bottom
      xfru[1] = XGraph(xlb,ylb,zlb);
      yfru[1] = YGraph(xlb,ylb,zlb);
      // right-bottom
      xfru[2] = XGraph(xrb,yrb,zrb);
      yfru[2] = YGraph(xrb,yrb,zrb);
      // right-top
      xfru[3] = XGraph(xrt,yrt,zrt);
      yfru[3] = YGraph(xrt,yrt,zrt);
    }
    //  ground line -----
    if(fabs(llow)<LFRONT) {
      lfr = new TLine();
      lfr->SetX1(L*xlg/(L+ylg*COSA));
      lfr->SetY1(L*(ylg-H)*SINA/(L+ylg*COSA));
      lfr->SetX2(L*xrg/(L+yrg*COSA));
      lfr->SetY2(L*(yrg-H)*SINA/(L+yrg*COSA));
      lfr->SetLineStyle(3);
      lfr->SetLineColor(kWhite);
      lfr->Draw();
      plfru = new TPolyLine(4,xfru,yfru);
      plfru->SetLineStyle(3);
      plfru->SetLineColor(15);
      plfru->Draw();
    } else if (llow<LFRONT) {
      xfru[4] = xfru[0];
      yfru[4] = yfru[0];
      plfru = new TPolyLine(5,xfru,yfru);
      plfru->SetLineStyle(3);
      plfru->SetLineColor(15);
      plfru->Draw();
    }
    // ---------------------------------------------------------------
    padm->Update();
    if (save && ((it+NTNEG)%3)==0) {
      char file[256];
      sprintf(file,"anim_%03d.gif",(it+NTNEG)/3);
      padm->SaveAs(file);
    }
    for(int is=0; is<nps[ipm]; is++) {
      if(npsp<4*NPMAX) {
        xpsp[npsp]=xps[ipm][is];
        ypsp[npsp]=yps[ipm][is];
        npsp++;
      }
    }
  }
  unsigned int size = GrdPcles.size();
  for (unsigned int i=0; i<size; i++)
    delete GrdPcles[i];
  size = EllClean.size();
  for (unsigned int i=0;i<size;i++)
    delete EllClean[i];
  size = BoxClean.size();
  for (unsigned int i=0;i<size;i++)
    delete BoxClean[i];
  size = LinClean.size();
  for (unsigned int i=0;i<size;i++)
    delete LinClean[i];
  if(plfr)
    delete plfr;
  if(plfru)
    delete plfru;
  if(lfr)
    delete lfr;
  delete padm;
  if (save)
    system("convert anim_*.gif Anim.gif && rm anim_*.gif");
  return;
}

