#define ED__FILES
#include <stdio.h>
#include <ctime>
#include <vector>
#include <algorithm>
#include "ED_Files.h"

#include <dirent.h>

vector<RFile> gAllRootFiles;

/*-----------------------------------------------------------------*/

time_t StringToSeconds(const char *name) {
  int year, month, day, hour, min;
  const char *buf = name+strlen(name);
  int lookforunderscore=4;
  while (lookforunderscore) {
    if (*(--buf)=='_')
      lookforunderscore--;
    if (buf==name)
      break; // for badly formated entries
  }
  sscanf(buf,"_%d_%d_%d_%dh%d.root",&year,&month,&day,&hour,&min);
  struct tm timestructure;
  timestructure.tm_year = year - 1900;
  timestructure.tm_mon = month - 1;
  timestructure.tm_mday = day;
  timestructure.tm_hour = hour;
  timestructure.tm_min = min;
  timestructure.tm_sec = 0;
  return mktime(&timestructure);
}

// Methods of the RFile Class
/*----------------------------------------------------------------------*/
RFile::RFile(const char* name) {
  fileName = name;
  seconds = StringToSeconds(name);
}

/*----------------------------------------------------------------------*/
bool RFile::operator<(const RFile& f) const {
  return (seconds<f.seconds);
}
bool RFile::operator==(const RFile& f) const {
  return (seconds == f.seconds);
}
bool RFile::operator!=(const RFile& f) const {
  return (seconds != f.seconds);
}
bool RFile::operator<=(const RFile& f) const {
  return (seconds <= f.seconds ) ;
}
bool RFile::operator>(const RFile& f) const {
  return (seconds > f.seconds) ;
}
bool RFile::operator>=(const RFile& f) const {
  return (seconds >= f.seconds) ;
}

/*----------------------------------------------------------------------*/
const char* RFile::filename() {
  return fileName.c_str();
}

int BuildAllRootFiles() {
  static std::string dataFilePath="";
  if (dataFilePath == gDataFilePath) return gAllRootFiles.size();
  dataFilePath = gDataFilePath;
  gAllRootFiles.clear();
  DIR *dirp;
  struct dirent *direntp;
  std::string rootFileDir, rootFileNameForSelection;
  char tempString[20];

  // We make a loop on each directory which can contain Root file
  // in the gDataFilePath directory. Ignores anything before 2003....
  for (int i=2003; i<2030; i++) {
    for (int j=1; j<13; j++) {
      sprintf(tempString,"/%d/%.2d/",i,j);
      rootFileDir = gDataFilePath + tempString;
      if ((dirp = opendir(rootFileDir.c_str())) != NULL)
        while ((direntp = readdir (dirp)) != NULL) {
          if((strstr(direntp->d_name,".root") != NULL)) {
            rootFileNameForSelection = rootFileDir + direntp->d_name;
            gAllRootFiles.push_back(rootFileNameForSelection.c_str());
          }
        }
      if(dirp)
        closedir(dirp);
    }
  }

  if (!gAllRootFiles.size())
    return 0;

  std::sort<vector<RFile>::iterator>(gAllRootFiles.begin(),gAllRootFiles.end());
  return gAllRootFiles.size();
}

/*-----------------------------------------------------------------*/
int SelectFilesInRange(time_t start, time_t end, vector<RFile>& rootFiles) {
  // get the list of available files under gDataFilePath
  int nFiles = BuildAllRootFiles();
  int i = 0;
  rootFiles.clear();
  while (i < nFiles && gAllRootFiles[i].seconds < start)
    i++;
  if (i == nFiles)
    return 0;
  int j = i;
  if (i != 0)
    i--;
  while (j < nFiles && gAllRootFiles[j].seconds < end)
    j++;
  for (int k = i; k < j; k++)
    rootFiles.push_back(gAllRootFiles[k]);
  return rootFiles.size();
}

/*----------------------------------------------------------------------*/
void SelectFiles(vector<RFile>& rootFiles,int flag) {
  time_t start=0, end=0;

  int size = BuildAllRootFiles();
  rootFiles.clear();
  if (size == 0)
    return;
  switch(flag) {
    // easy cases
  case ED_FILE_LOAD_LAST:
    if (size > 1)
      rootFiles.push_back(gAllRootFiles[size-2]);
  case ED_FILE_LOAD_CURRENT:
    rootFiles.push_back(gAllRootFiles[size-1]);
    return;

    // else...
  case ED_FILE_LOAD_DAY:
    end = time((time_t *)NULL);
    start = end - 86400;
    break;

  case ED_FILE_LOAD_TWO_DAY:
    end = time((time_t *)NULL);
    start = end - 86400*2;
    break;
  case ED_FILE_LOAD_WEEK:
    end = time((time_t *)NULL);
    start = end - 86400*7;
    break;

  case ED_FILE_LOAD_MONTH:
    end = time((time_t *)NULL);
    start = end - 86400*31;
    break;

  case ED_FILE_LOAD_ALL:
    end = time((time_t *)NULL);
    start = 0;
    break;

  default:
    break;
  }

  SelectFilesInRange(start, end, rootFiles);
}

/*-----------------------------------------------------------------*/
EDFileSelect::EDFileSelect(const TGWindow *p,const TGWindow *main, UInt_t w, UInt_t h, vector<RFile>& rootfiles, int *haschanged) : TGTransientFrame(p, main, w, h) {

  tofill=&rootfiles;
  fhaschanged=haschanged;
  ULong_t colordefault,colorred;
  gClient->GetColorByName("AntiqueWhite3",colordefault);
  gClient->GetColorByName("IndianRed1",colorred);
  fLayout=new TGLayoutHints(kLHintsCenterX|kLHintsCenterY);
  fMax=new TGLayoutHints(kLHintsCenterX|kLHintsCenterY|kLHintsExpandX|kLHintsExpandY);


  fGF1 = new TGCompositeFrame(this, 200,50, kHorizontalFrame);
  fYear1=new TGComboBox(fGF1,1);
  fMonth1=new TGComboBox(fGF1,2);
  fDay1=new TGComboBox(fGF1,3);
  TGLabel l1(fGF1,"From:");
  l1.ChangeBackground(colordefault);
  fGF1->AddFrame(&l1,fMax);
  fGF1->AddFrame(fDay1, fMax);
  fGF1->AddFrame(fMonth1, fMax);
  fGF1->AddFrame(fYear1, fMax);

  fGF2 = new TGCompositeFrame(this, 200,50, kHorizontalFrame);
  fYear2=new TGComboBox(fGF2,1);
  fMonth2=new TGComboBox(fGF2,2);
  fDay2=new TGComboBox(fGF2,3);
  TGLabel l2(fGF2,"To:");
  l2.ChangeBackground(colordefault);
  fGF2->AddFrame(&l2,fMax);
  fGF2->AddFrame(fDay2, fMax);
  fGF2->AddFrame(fMonth2, fMax);
  fGF2->AddFrame(fYear2, fMax);

  {
    char c[8];
    for (int i=2003;i<2031;i++) {
      snprintf(c,5,"%d",i);
      fYear1->AddEntry(c,i);
      fYear2->AddEntry(c,i);
    }
    for (int i=1;i<32;i++) {
      snprintf(c,5,"%.2d",i);
      fDay1->AddEntry(c,i);
      fDay2->AddEntry(c,i);
    }
  }
  fMonth1->AddEntry("January",1);
  fMonth2->AddEntry("January",1);
  fMonth1->AddEntry("February",2);
  fMonth2->AddEntry("February",2);
  fMonth1->AddEntry("March",3);
  fMonth2->AddEntry("March",3);
  fMonth1->AddEntry("April",4);
  fMonth2->AddEntry("April",4);
  fMonth1->AddEntry("May",5);
  fMonth2->AddEntry("May",5);
  fMonth1->AddEntry("June",6);
  fMonth2->AddEntry("June",6);
  fMonth1->AddEntry("July",7);
  fMonth2->AddEntry("July",7);
  fMonth1->AddEntry("August",8);
  fMonth2->AddEntry("August",8);
  fMonth1->AddEntry("September",9);
  fMonth2->AddEntry("September",9);
  fMonth1->AddEntry("October",10);
  fMonth2->AddEntry("October",10);
  fMonth1->AddEntry("November",11);
  fMonth2->AddEntry("November",11);
  fMonth1->AddEntry("December",12);
  fMonth2->AddEntry("December",12);

  fGF1->ChangeBackground(colordefault);
  fGF2->ChangeBackground(colordefault);

  fYear1->Select(2004);
  fYear2->Select(2004);
  fMonth1->Select(1);
  fMonth2->Select(8);
  fDay1->Select(1);
  fDay2->Select(31);

  fButton1 = new TGTextButton(this, "&Select",0);
  fButton1->Associate(this);
  fButton1->ChangeBackground(colorred);

  fButton2 = new TGTextButton(this, "&Cancel",1);
  fButton2->Associate(this);
  fButton2->ChangeBackground(colorred);

  AddFrame(fGF1, fMax);
  AddFrame(fGF2, fMax);
  AddFrame(fButton1, fLayout);
  AddFrame(fButton2, fLayout);

  MapSubwindows();
  Layout();

  ChangeBackground(colordefault);
  SetWindowName("Select period");
  MapWindow();

  fClient->WaitFor(this);
}


/*-----------------------------------------------------------------*/
Bool_t EDFileSelect::ProcessMessage(Long_t msg, Long_t parm1, Long_t) {
  switch (GET_MSG(msg)) {
  case kC_COMMAND:
    switch (GET_SUBMSG(msg)) {
    case kCM_BUTTON:
      if(parm1==0) {
        (*tofill).clear(); // Reset the rootfile list content
        struct tm start, end;
        start.tm_year = fYear1->GetSelected() - 1900;
        start.tm_mon = fMonth1->GetSelected() - 1;
        start.tm_mday = fDay1->GetSelected() - 1;
        start.tm_hour = start.tm_min = start.tm_sec = 0;
        time_t startSec  = mktime(&start);
        end.tm_year = fYear2->GetSelected() - 1900;
        end.tm_mon = fMonth2->GetSelected() - 1;
        end.tm_mday = fDay2->GetSelected() + 1;
        end.tm_hour = end.tm_min = end.tm_sec = 0;
        time_t endSec  = mktime(&end);
        SelectFilesInRange(startSec, endSec,*tofill);
        (*fhaschanged)=1;
        CloseWindow();
        break;
      }
      if(parm1==1) {
        CloseWindow();
        break;
      }
    }
  }
  return kTRUE;
}


// For new Ids
const char *GetFileName(unsigned long long id, const char * dir) {
  struct tm mytime;
  char dirstring[1024],matchstring[1024];
  DIR *dirp;
  struct dirent *direntp;
  mytime.tm_year=id/100000000;
  mytime.tm_mon=0;
  mytime.tm_sec=0;
  mytime.tm_min=0;
  mytime.tm_hour=0;
  mytime.tm_mday=id/100000 - 1000*mytime.tm_year;
  mytime.tm_year-=1900;
  mktime(&mytime);
  snprintf(dirstring,1024,"%s/%d/%02d/",dir,mytime.tm_year+1900,mytime.tm_mon+1);
  snprintf(matchstring,1024,"_%d_%02d_%02d_12h00.root",mytime.tm_year+1900,mytime.tm_mon+1,mytime.tm_mday);
  if ((dirp = opendir(dirstring)) != NULL)
    while ((direntp = readdir (dirp)) != NULL) {
      if((strstr(direntp->d_name,matchstring) != NULL)) {
        closedir(dirp);
        snprintf(matchstring,1024,"%s/%s",dirstring,direntp->d_name);
        return strdup(matchstring);
      }
    }
  if(dirp) closedir(dirp);
  strcpy(matchstring,"");
  // trying with the official Auger directory just in case...
  if (strstr(dir,"/../Auger/")) return strdup(matchstring); // already tried
  snprintf(dirstring,1024,"%s/../Auger/",dir);
  return GetFileName(id,dirstring);

}

int BuildAllRootFiles(string directory) {
  gAllRootFiles.clear();
  DIR *dirp;
  struct dirent *direntp;
  string rootFileDir, rootFileNameForSelection;
  char tempString[20];
  // first we look for root files in the current directory
  rootFileDir = directory+(string)"/";
  if ((dirp = opendir(rootFileDir.c_str())) != NULL)
    while ((direntp = readdir (dirp)) != NULL) {
      if((strstr(direntp->d_name,".root") != NULL)) {
	rootFileNameForSelection = rootFileDir + direntp->d_name;
	gAllRootFiles.push_back(rootFileNameForSelection.c_str());
      }
    }
  if(dirp) closedir(dirp);
  
  // then we look in subdirectories with pattern ./20xx/xx/*.root
  for (int i=2003; i<2030; i++) {
    for (int j=1; j<13; j++) {
      sprintf(tempString,"/%d/%.2d/",i,j);
      rootFileDir = directory + tempString;
      if ((dirp = opendir(rootFileDir.c_str())) != NULL)
	while ((direntp = readdir (dirp)) != NULL) {
	  if((strstr(direntp->d_name,".root") != NULL)) {
	    rootFileNameForSelection = rootFileDir + direntp->d_name;
	    gAllRootFiles.push_back(rootFileNameForSelection.c_str());
	  }
	}
      if(dirp) closedir(dirp);
    }
  }

  if (!gAllRootFiles.size()) return 0;

  sort<vector<RFile>::iterator>(gAllRootFiles.begin(),gAllRootFiles.end());
  return gAllRootFiles.size();
}
