#ifdef FD
//functions for loading data in EventRec.C
//Martina Bohacova, OCT 30, 2002

#include <MiEvent.hh>       // event format definition
#include <MiEventFile.hh>   // file class for read/write
#include <FdNumbering.hh>
using namespace FdUtil;
#include "FDPixel.h"          //class declaration for EventRec.C

// -------------------------------------------------------------------------

int LoadData(int Seek, char *File, int TimeSlice, int TimeSliceEnd,
             MujPixel*** _polepixelu , struct MujHeader* mh,
             struct pixpoint cam[][N_PIX_TOT+1] ) {
  TMirrorEventFile *file;
  TMirrorRunHeader *header = new TMirrorRunHeader;
  TMirrorRunTrailer *trailer = new TMirrorRunTrailer;
  int COUNT=0;
  int NumPixel=0;

  file = new TMirrorEventFile( File);

  if ( file->ReadRunHeader( &header )) {
    cout << "RunHeader successfully read" << endl;
    header->Dump();
  }

  if ( file->ReadRunTrailer( &trailer )) {
    cout << "RunTrailer successfully read" << endl;
    trailer->Dump();
  }

  file->Seek( Seek );

  TMirrorEvent *event = new TMirrorEvent();

  if ( file->ReadMirrorEvent( &event ) == kFALSE ) {
    cout << "ReadMirrorEvent failed, exiting" << endl;
    return 0;
  }

  // --- event processing ...

  event->EventHeader()->Dump();
  //event->PixelList()->Dump();
  event->PixelData()->Dump();

  TMirrorBitArray ba( event->GetPixelData(), TimeSlice, TimeSliceEnd );

  // --- loop over pixels and their data

  //------------------------------------------------------------

  for ( UInt_t n_pix = 0 ; n_pix < event->GetPixelList()->GetNumPixels(); n_pix++ ) {

    Fd::PixelNumber pixel = event->GetPixelList()->GetPixel( n_pix );

    if ( ba.Test( Fd::GetColumnNo( pixel ), Fd::GetRowNo( pixel ) ) ) {
      NumPixel++;
      continue;
    }
  }

  if(( *_polepixelu = (MujPixel **) malloc( NumPixel * sizeof(MujPixel *))) == NULL ) {
    cout << "can't allocate memory, exit " << endl;
    exit(1);
  }

  mh->MirrNum =event->GetEventHeader()->GetMirrorNo();
  mh->RunNum =event->GetEventHeader()->GetRunNo();
  mh->EventNum =event->GetEventHeader()->GetEventNo();
  mh->GPSTime =event->GetEventHeader()->GetGPSTime();
  mh->GPSNanoTime =event->GetEventHeader()->GetGPSNanoTime();
  mh->PixNum = NumPixel;

  for ( UInt_t n_pix = 0 ; n_pix < event->GetPixelList()->GetNumPixels(); n_pix++ ) {

    Fd::PixelNumber pixel = event->GetPixelList()->GetPixel( n_pix );

    if ( !ba.Test( Fd::GetColumnNo( pixel ), Fd::GetRowNo( pixel ) ) ) {
      continue;
    }
    COUNT++;

    int bracha =0;
    if ( Fd::GetRowNo( pixel ) %2 == 1) {
      if ( ba.Test( Fd::GetColumnNo( pixel )+1, Fd::GetRowNo( pixel ) ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel )-1, Fd::GetRowNo( pixel ) ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel )-1, Fd::GetRowNo( pixel )-1 ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel ), Fd::GetRowNo( pixel )-1 ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel )-1, Fd::GetRowNo( pixel )+1 ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel ), Fd::GetRowNo( pixel )+1 ) )
        bracha ++;
    }
    if ( Fd::GetRowNo( pixel ) %2 == 0) {
      if ( ba.Test( Fd::GetColumnNo( pixel )+1, Fd::GetRowNo( pixel ) ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel )-1, Fd::GetRowNo( pixel ) ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel )+1, Fd::GetRowNo( pixel )-1 ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel ), Fd::GetRowNo( pixel )-1 ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel )+1, Fd::GetRowNo( pixel )+1 ) )
        bracha ++;
      if ( ba.Test( Fd::GetColumnNo( pixel ), Fd::GetRowNo( pixel )+1 ) )
        bracha ++;
    }

    (*_polepixelu)[COUNT-1] = new MujPixel( Fd::GetPixelNo( pixel ), Fd::GetRowNo( pixel ),
                                            Fd::GetColumnNo( pixel ),
                                            cam[event->GetEventHeader()->GetMirrorNo()][Fd::GetPixelNo(pixel)].elev,
                                            cam[event->GetEventHeader()->GetMirrorNo()][Fd::GetPixelNo(pixel)].azim,
                                            bracha
                                          );

    (*_polepixelu)[COUNT-1]->CopyFadcdata(
      event->FADCData()->GetFADCData( pixel ),
      cam[mh->MirrNum][Fd::GetPixelNo(pixel)].calconst
    );
  }


  //  delete event;

  file->Close();
  delete file;

  delete header;
  delete trailer;
  return COUNT;
}

///////////////////////////////////////////
/////////// Class MujPixel  ///////////////
///////////////////////////////////////////

MujPixel::MujPixel( int No, int Row, int Col,
                    float Elevace, float Azimut, int brother ) {
  PixNo = No;
  ColNo = Col;
  RowNo = Row;
  Elev = Elevace;
  Azim = Azimut;
  NeibrNo = brother;
  Area = 0;
}

MujPixel::MujPixel( MujPixel * p ) {
  PixNo = p->PixNo;
  ColNo = p->ColNo;
  RowNo = p->RowNo;
  NeibrNo = p->NeibrNo;
  Elev = p->Elev;
  Azim = p->Azim;
  CalConst = p->CalConst;
  Offset = p->Offset;

  Ti = p->Ti;
  TiErr = p->TiErr;
  Ampl = p->Ampl;
  Sigma = p->Sigma;
  Area = p->Area;

  for( int k = 0; k < 1000; k ++ )
    Fadcdata[k] = p->Fadcdata[k];
}

MujPixel::~MujPixel() {
  return;
}

void MujPixel::CopyFadcdata( TFADCData::FADCData f, float calconst ) {
  int k;

  for( k = 0; k < 1000; k ++ )
    Fadcdata[k] = calconst*(f->fFadcData[k].fBits.fData);
}

MujPixel ** DuplPixArray( MujPixel ** src, int count ) {
  MujPixel ** arr;

  if(( arr = (MujPixel **) malloc( count * sizeof( MujPixel *))) == NULL ) {
    fprintf( stderr, "alloc failed" );
    exit(1);
  }

  for( int i = 0; i < count; i++ )
    arr[i] = new MujPixel( src[i] );

  return arr;
}

void DeletePixArray( MujPixel ** src, int count ) {

  for( int i = 0; i < count; i++ )
    delete src[i];
  free( src );
}

// Returns 1 if succeeded, < 0 if failed
int ReadTube( char * filename, struct pixpoint cam[][N_PIX_TOT+1] ) {
  FILE * f;
  char buffer[1000];
  int ii, cc, c;
  float f1,f2,f3,f4,f5,f6;

  if(( f = fopen( filename, "r" )) == NULL )
    return -1;
  for( c = 4; c <=5; c++ ) {
    for( int i = 1; i <= N_PIX_TOT; i++ ) {
      if( fgets( buffer, 250, f ) == NULL ) {
        fclose( f );
        return -11000-i;
      }

      if( sscanf( buffer, "4 %d%d%f%f%f%f%f%f", &cc, &ii,
                  &f1,&f2,&f3,&f4,&f5,&f6) < 8) {
        fclose( f );
        return -cc*1000-i;
      }
      cam[cc][i].sx=f1;
      cam[cc][i].sy=f2;
      cam[cc][i].sz=f3;
      cam[cc][i].azim=f4;
      cam[cc][i].elev=f5;
      cam[cc][i].calconst=f6;

      if( ii != i ) {
        fclose( f );
        return -1500-i;
      }
    }
  }
  fclose( f );
  return 1;
}
#endif
