#include "Ec.h"
#include "STCconstants.h"


static int gEcOldCalib=0;

int Ec_UseOldCalib(int i) {
  if (i!=-1) gEcOldCalib=i;
  return gEcOldCalib;
}

// TTraceData Tx() functions

float TTraceData::Tx(float percent) const {
  float binvalue, signal = 0., total = 0., tot_prec = 0., tx = 0.;
  int k, k0 = 0;
  for (k = fStartIntegration; k < fEndIntegration; k++) {
    signal += fTrace[k];
  }
  if (!signal)  return 0;
  if (signal < 0) {
    cerr << "Pmt trace seems wrong, signal negative (" << signal <<
    ")..." << endl;
    return 0;
  }
  signal *= (percent / 100);
  for (k = fStartIntegration; k < fEndIntegration; k++) {
    binvalue = fTrace[k];
    tot_prec = total;
    total += binvalue;
    if (total >= signal) break;
    if (fTrace[k] == 0) k0++; else k0 = 0;
  }
  tx = (signal - total) / (total - tot_prec) * (1.0 + k0) + (float) (k);
  
  return tx * kEc::TimeSlot;
}

float TTraceData::T90()  const {
  if (fT90 < 0) fT90 = Tx(90);
  return fT90;
}

float TTraceData::T70()  const {
  if (fT70 < 0) fT70 = Tx(70);
  return fT70;
}

float TTraceData::T50()  const {
  if (fT50 < 0) fT50 = Tx(50);
  return fT50;
}

float TTraceData::T10()  const {
  if (fT10 < 0) fT10 = Tx(10);
  return fT10;
}

float TTraceData::SigInVEM() {
  if (fSigInVEM < 0) cerr <<"fSigInVEM is not calculated, please call TCalibStation::PmtSignal()."<< endl;
  return fSigInVEM;
}

float TTraceData::PeakInVEM() {
  if (fPeakInVEM < 0) cerr << "fPeakInVEM is not calculated, please call TCalibStation::PmtSignal()."<< endl;
  return fPeakInVEM;
}

// TCalib* functions

TCalibSdPmt::TCalibSdPmt():TTraceData() {
  fHighGainSat = 0;
  fLowGainSat = 0;
  fPulseSlope = 0;
  fHistSlope = 0;
  fBaseline[0] = fBaseline[1] = 0;
}


TCalibStation::TCalibStation():IoSdStation(), TTraceData() {
  fStartIntegration = 0;
  if (Fadc != 0x0)
    fEndIntegration = Fadc->NSample;
  else
    fEndIntegration = 0;
  fStartBin = 0;
  fSat = 0;
  fSeveralSignals = 0;
  //fWgt = 1.0;
  for (unsigned int i = 0; i < kIoSd::NPMT; i++) {
    fPmt[i].fVemPeak = fPmt[i].fVemCharge = -1;
    fPmt[i].fCalibratedState = 0;
    fPmt[i].fStartIntegration = fStartIntegration;
    fPmt[i].fEndIntegration = fEndIntegration;
  }
}

TCalibStation::TCalibStation(const IoSdStation & st):IoSdStation(st),
    TTraceData() {
  fStartIntegration = 0;
  if (Fadc != 0x0)
    fEndIntegration = Fadc->NSample;
  else
    fEndIntegration = 0;
  fStartBin = 0;
  fSeveralSignals = 0;
  //fWgt = 1.0;
  fSat = 0;
  for (unsigned int i = 0; i < kIoSd::NPMT; i++) {
    fPmt[i].fVemPeak = fPmt[i].fVemCharge = -1;
    fPmt[i].fCalibratedState = 0;
    fPmt[i].fStartIntegration = fStartIntegration;
    fPmt[i].fEndIntegration = fEndIntegration;
  }
}


void TCalibStation::Print(ostream& os) {
  os << endl;
  os.setf(ios::fixed);
  int initprec = os.precision();
  os.precision(2);
  os << "LsId             = " << id() << endl;
  os << "Ls name          = " << name() << endl;
  os << "Easting          = " << easting() << endl;
  os << "Northing         = " << northing() << endl;
  os << "X                = " << fX << endl;
  os << "Y                = " << fY << endl;
  os << "Z                = " << fZ << endl;
  os << "Altitude         = " << altitude() << endl;
  os << "Ls error code    = " << error() << endl;
  os << "Ls trigger type  = " << trigger().Name << endl;
  os << "avg signal       = " << fSigInVEM << endl;
  os << "avg peak         = " << fPeakInVEM << endl;
  os << "sig/peak         = " << SigOverPeak() << endl;
  //if( fDist != 0.0 ) os << "distance to axis = " << fDist << endl;
  //if( fDcore != 0.0 ) os << "distance to core = " << fDcore << endl;
  os << "start bin        = " << fStartBin << endl;
  os << "start time       = " << StartTime() << endl;
  os << "start second     = " << gps()->second() << endl;
  os << "start nano       = " << gps()->nanosecond() << endl;
  os.precision(initprec);
}

TEcEvent::TEcEvent():IoSdEvent() {
}

TEcEvent::~TEcEvent() {
  fCalibStations.clear();
}

TEcEvent::TEcEvent(EventPos pos):IoSdEvent(pos, (char *) "") {
  Init();
}

TEcEvent::TEcEvent(EventId id):IoSdEvent(id) {
  Init();
}

TEcEvent::TEcEvent(const IoSdEvent & sdevent):IoSdEvent(sdevent) {
  Init();
}

void TEcEvent::Signals() {
  unsigned int size = fCalibStations.size();
  for (unsigned int i = 0; i < size; i++) {
    if(fCalibStations[i].Error) continue;
    fCalibStations[i].Signal();
  }
}

void TEcEvent::Init() {
  NumberOfErrorZeroStation = 0;
  fCalibStations.reserve(Stations.size());
  for (unsigned int i = 0; i < Trigger.NumberOfStation; i++) {
    /* Check first if there is not a Eb bug */
    if (!Stations[i].Error && abs((int) Stations[i].Gps->Second - (int) Trigger.Second) > 1) {
      cerr << Stations[i].Id << " has a corrupted data bloc" << endl;
      Stations[i].Error = IoSdEvent::eBadCompress;
    }
    fCalibStations.push_back(Stations[i]);
    if (!Stations[i].Error) NumberOfErrorZeroStation++;
  }
  Signals();

  for (unsigned int i = 0; i < fCalibStations.size(); i++) {
    TCalibStation & stat = fCalibStations[i];
    stat.fRefSecond = RefSecond();
    UTMtoLL(kSTC::WGS84, stat.Northing, stat.Easting, kSTC::AugerUTMZone, stat.fLatitude, stat.fLongitude);
  }
  SetReference(kSTC::N0, kSTC::E0);
}

void TEcEvent::SetReference(double north, double east) {
  /// cf GAP-2001-038
  double x, y;
  ne2xy(north, east, &x, &y);
  for (unsigned int i = 0; i < fCalibStations.size(); i++) {
    TCalibStation & stat = fCalibStations[i];
    nea2xyz(stat.Northing, stat.Easting, stat.Altitude, &stat.fX,
            &stat.fY, &stat.fZ);
    // applying altitude correction
    stat.fZ += (x * x + y * y) * kSTC::Gamma;
  }
  fRefNorthing = north;
  fRefEasting = east;
}

// Specific functions from calibration
// Not used for tank calibration by itself but for tank diagnostics

// From Numerical Recipes, adpated: see http://www.codeproject.com/cpp/howtofft.asp
#include <math.h>
#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
//data -> float array that represent the array of complex samples
//number_of_complex_samples -> number of samples (N^2 order number) 
//isign -> 1 to calculate FFT and -1 to calculate Reverse FFT
void FFT (float data[], unsigned long number_of_complex_samples, int isign)
{
    //variables for trigonometric recurrences
    unsigned long n,mmax,m,j,istep,i;
    double wtemp,wr,wpr,wpi,wi,theta,tempr,tempi;
    //the complex array is real+complex so the array 
    //as a size n = 2* number of complex samples
    // real part is the data[index] and 
    //the complex part is the data[index+1]
    n=number_of_complex_samples * 2; 

    //binary inversion (note that the indexes 
    //start from 0 witch means that the
    //real part of the complex is on the even-indexes 
    //and the complex part is on the odd-indexes
    j=0;
    for (i=0;i<n/2;i+=2) {
        if (j > i) {
            //swap the real part
            SWAP(data[j],data[i]);
            //swap the complex part
            SWAP(data[j+1],data[i+1]);
            // checks if the changes occurs in the first half
            // and use the mirrored effect on the second half
            if((j/2)<(n/4)){
                //swap the real part
                SWAP(data[(n-(i+2))],data[(n-(j+2))]);
                //swap the complex part
                SWAP(data[(n-(i+2))+1],data[(n-(j+2))+1]);
            }
        }
        m=n/2;
        while (m >= 2 && j >= m) {
            j -= m;
            m = m/2;
        }
        j += m;
    }
    //Danielson-Lanzcos routine 
    mmax=2;
    //external loop
    while (n > mmax)
    {
        istep = mmax<<  1;
        theta=isign*(6.28318530717959/mmax);
        wtemp=sin(0.5*theta);
        wpr = -2.0*wtemp*wtemp;
        wpi=sin(theta);
        wr=1.0;
        wi=0.0;
        //internal loops
        for (m=1;m<mmax;m+=2) {
            for (i= m;i<=n;i+=istep) {
                j=i+mmax;
                tempr=wr*data[j-1]-wi*data[j];
                tempi=wr*data[j]+wi*data[j-1];
                data[j-1]=data[i-1]-tempr;
                data[j]=data[i]-tempi;
                data[i-1] += tempr;
                data[i] += tempi;
            }
            wr=(wtemp=wr)*wpr-wi*wpi+wr;
            wi=wi*wpr+wtemp*wpi+wi;
        }
        mmax=istep;
    }
}

bool AA(TCalibStation *t)
{
  int count[3] = { 0, 0, 0 };
  int cross[3] = { 1, 1, 1 };
  for (unsigned int j = 0; j < kIoSd::NPMT; j++) {
    for (unsigned int i = 0; i < kIoSd::MAXSAMPLE; i++) {
      if (t->Fadc->Trace[j][IoSdEvent::eHigh][i] > 1000)
        return false;
      if (t->Fadc->Trace[j][IoSdEvent::eHigh][i] < t->fPmt[j].fADCBaseline[IoSdEvent::eHigh][i] - 4
          && cross[j]) {
        count[j]++;
        cross[j] = 0;
      }
      if (t->Fadc->Trace[j][IoSdEvent::eHigh][i] > t->fPmt[j].fADCBaseline[IoSdEvent::eHigh][i] + 4
          && !cross[j])
        cross[j] = 1;
    }
  }
  if (count[0] > 2 && count[1] > 3 && count[2] > 3)
    return true;            // PMT 1 has smaller signal
  return false;
}


// New lightning algorithm, using FFT of signal
// Lightning events appear to have peaks in Fourrier transform at high
// frequency and at 2.5-3 MHz... This is about indexes 32-40...
bool TCalibStation::IsLightningEvent() {
  return AA(this);
  for (unsigned int pm=0;pm<3;pm++) {
    float arr[512];
    for (unsigned int i=0;i<512;i++) arr[i]=Fadc->Trace[pm][IoSdEvent::eHigh][i]-1.0*fPmt[pm].fADCBaseline[IoSdEvent::eHigh][i];
    if (1) FFT(arr,256,1);
    float v,tot=0,tot2=0,tot3=0;
    for (unsigned int i=0;i<128;i++) {
      v=sqrt(arr[2*i]*arr[2*i]+arr[2*i+1]*arr[2*i+1]);
      tot+=v;
      if (i>31 && i<41) tot2+=v;
      if (i>100) tot3+=v;
      //cout << OldIsLightningEvent() << " " << i << " " << v << endl;
    }
    //cout << OldIsLightningEvent() << " " << tot << " " << tot2 << " " << tot3 <<" " << fPmt[pm].fBaselineSigma[IoSdEvent::eHigh] << " " << AA(this) << " " << Id << endl;
  }
  return false;
}

void TCalibStation::WaterEstimatesFit(int n) {
  if (fPmt[n].fCalibratedState<3) return; // No good Vem from histo => no water fit
  TH1F *h = HCharge(n);
  float base = Calib->Base[n];
  if (Calib->Version == 13)
    base *= 19;     // 19 bins in version 13
  else
    base *= 20;
  h->Fit("expo", "Q0", "",
         base + 1.5 * fPmt[n].fVemCharge,
         base + 2 * fPmt[n].fVemCharge);
  TF1 *f2 = h->GetFunction("expo");
  fPmt[n].fHistSlope =
    f2->GetParameter(1) * fPmt[n].fVemCharge;
  // and pulse shape fit
  if (Calib->Version < 12)
    HShape(n)->Fit("expo", "Q0", "", 50, 125);
  else
    HShape(n)->Fit("expo", "Q0", "", 125, 200);
  f2 = HShape(n)->GetFunction("expo");
  fPmt[n].fPulseSlope = -1. / f2->GetParameter(1);
  fPmt[n].fCalibratedState = 4;
}

