#include "Ec.h"

namespace kEc {
  int gDebug=0;
  int gStrictCalib=1;
}

void kEc::StrictCalib(int v) {
  if (v==-1) kEc::gStrictCalib=!kEc::gStrictCalib;
  else kEc::gStrictCalib=v;
}

void kEc::Debug(int v) {
  kEc::gDebug=v;
}
//
// Cleaner New calibration
// Everything needed is in this file. Old calibration is in EcOld.cc
//

void TCalibStation::Signal() {
  if (Ec_UseOldCalib()) // backward compatible
    OldSignal();
  else {
    // new calib
    Calibrate();
    ComputeBaselines();
    BuildSignals();
    MergeSignals();
    SelectSignal();
    TreatSaturated();
  }
}

void TCalibStation::ChargeHistogramFit(int n, int force=0) {
  if (fPmt[n].fCalibratedState>1 && !force) return; // already calibrated
  // extracting VEM from histograms
  TH1F *h = HCharge(n);
  if (h->GetMaximum() < 500) return; // to few entries in histogram

  fPmt[n].fCalibratedState = 2;
  float base = Calib->Base[n];
  if (Calib->Version == 13) base *= 19;     // 19 bins in version 13
  else  base *= 20;
  int max = 0;
  int maxbin = 0;
  int bin = h->GetNbinsX();
  int nbbin = bin;
  int v = 0, v1 = 0, v2 = 0;
  int binmin = 0, binmax = 0;
  int dir = -1;
  while (!binmax) {
    v = (int) h->GetBinContent(bin);
    v1 = (int) h->GetBinContent(bin + dir);
    v2 = (int) h->GetBinContent(bin + 2 * dir);
    if (v > 300 && v > max) {
      max = v;
      maxbin = bin;
    };
    if (binmin && (1. * max) / v > 1.3 && (1. * max) / v1 > 1.3 && (1. * max) / v2 > 1.3) 
      binmax = bin;
    if (!binmin && max && (1. * max) / v > 1.3 && (1. * max) / v1 > 1.3 && (1. * max) / v2 > 1.3) {
      binmin = bin;
      dir = 1;
    };
    bin += dir;
    if (bin == nbbin)  binmax = -1;
    if (bin == 2)  binmax = -1;
  }
  if (Calib->VemCharge[n] < binmax && Calib->VemCharge[n] > binmin && binmin > 0 && binmax < h->GetNbinsX() && binmin < binmax) {     // Good for fit
    double x = 0, x2 = 0, x3 = 0, x4 = 0, y = 0, xy = 0, x2y =
                                         0, v = 0, a = 0;
    int nb = 0;
    for (int i = binmin; i < binmax; i++) {
      v = h->GetBinContent(i);
      a = h->GetBinCenter(i);
      x += a;
      x2 += a * a;
      x3 += a * a * a;
      x4 += a * a * a * a;
      y += v;
      xy += a * v;
      x2y += a * a * v;
      nb++;
    }
    double B =
      (y * (x4 * x - x2 * x3) + xy * (x2 * x2 - nb * x4) +
       x2y * (nb * x3 - x * x2));
    double A =
      (y * (x2 * x2 - x * x3) + xy * (nb * x3 - x * x2) +
       x2y * (x * x - nb * x2));
    double res = -B / (2 * A);
    if (res > h->GetBinCenter(binmin) && res < h->GetBinCenter(binmax)) {
      fPmt[n].fVemCharge = res - base;
      fPmt[n].fCalibratedState = 3;
    }
  }
}

void TCalibStation::Calibrate() {
  if (Calib->Version <12) { // Old data, throw away
    if (Id>100 && kEc::gDebug) cerr << "Warning: Old calib data for tank " << Id << " (calib version " << Calib->Version << ")" << endl;
    if (kEc::gStrictCalib) {
      for (unsigned int n=0;n<3;n++) Calib->TubeOk[n]=0;
      if (Id>100 && kEc::gDebug) cerr << "Throwing away data" << endl;
      if (Id>4000) cerr << "Warning: Tank " << Id << " looks like simulated tank. Please add kEc::StrictCalib(0); in your code to recover the tank " << endl;
    } else {
      for (unsigned int n=0;n<3;n++) OldCalibrate(n,0);
    }
    return;
  }
  if (Calib->Version > 32000) {
    // Thanks Patrick for putting data in the FADC trace :S
    // Just removing tanks...
    for (unsigned int n=0;n<3;n++) Calib->TubeOk[n]=0;
    return;
  }
  for (unsigned int n=0;n<3;n++) {
    // Trust online calibration to use one PMT
    if (!Calib->TubeOk[n]) continue;
    ChargeHistogramFit(n);
    // In case fit failed: use LS online estimate corrected
    if (fPmt[n].fVemCharge < 0) fPmt[n].fVemCharge = Calib->VemCharge[n] * 1.045;       // From hump to LS
    // VEM Peak is not fitted
    fPmt[n].fVemPeak = Calib->VemPeak[n];
    // from Hump to VEM: Laura & Capisa & Larabelle experiments
    fPmt[n].fVemCharge /= 1.01;
    fPmt[n].fVemPeak /= 0.87;
    // D/A stuff
    fPmt[n].fDA=Calib->DA[n];
    // from rate DA method
    if (Calib->Version == 12) fPmt[n].fDA /= 1.07;
  }
}

// Baseline computation: algorithm looks for a succession of bins
//          within +-2 of first bin to define a baseline piece.
//          Baseline is extrapolated in between proportionnally to
//          the signal. For start and end of trace, extrapolation
//          if needed uses a fixed correction factor coming from
//          understanding of the electronics (by Maldera and Navarra)
//          Baseline sigma is extracted from the biggest baseline piece,
//          and if none was found the +-2 is changed until something is
//          found.

// FIXME
#define CORRFACT 0.000158
#define SATUR 1020
#define THRESH 3
// replacing kIoSd_MAXSAMPLE because of some buggued traces
#define kIoSd_MAXSAMPLE 760

int TCalibStation::ComputeBaselines() {
  int ret=0;
  for (int i=0;i<3;i++) {
    ret+=ComputeBaselines(i,IoSdEvent::eLow);
    ret+=ComputeBaselines(i,IoSdEvent::eHigh);
  }
  return ret;
}

int TCalibStation::ComputeBaselines(int pmt,int gain) {
  int is_satured=0,reaches_zero=0; // FIXME
  int val=0,sigma=2;
  unsigned int MINBIN=40;
  for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++)
    fPmt[pmt].fADCBaseline[gain][i]=Calib->Base[pmt + (gain==IoSdEvent::eLow ? 3 : 0)];
  vector<unsigned int> startv,endv;
  startv.reserve(20);
  endv.reserve(20);
  int flag=0;
  while (!startv.size() && ++sigma<=1024) {
    unsigned int bin=0,sbin=0;
    while (bin<kIoSd_MAXSAMPLE) {
      val=Fadc->Trace[pmt][gain][sbin];
      while (bin<kIoSd_MAXSAMPLE && val-(int)Fadc->Trace[pmt][gain][bin]>-sigma && val-(int)Fadc->Trace[pmt][gain][bin]<sigma) bin++;
      if (val>SATUR) {
        bin=sbin; // don't look for baseline in saturated part
        MINBIN=10; // after saturation, consider 10 following bins as baseline
        if (sigma<4) sigma=4; // and allow for more fluctuations
        is_satured=1; //FIXME
      }
      if (val<1 && is_satured) { // super undershoot, consider from now on baseline = 0
        startv.push_back(bin);
        endv.push_back(kIoSd_MAXSAMPLE-1);
        sbin=bin=kIoSd_MAXSAMPLE;
        reaches_zero=1;
      }
      if (bin>MINBIN+sbin) { // FIXME
        startv.push_back(sbin);
        endv.push_back(bin);
        sbin=bin;
      } else {
        sbin++;
        bin=sbin;
      }
    } // I have a list of baseline pieces.
    if (startv.size() && startv[0]>200 && flag!=1) {// First piece of baseline after 200, strange... start again with a bigger sigma...
      while (startv.size()) {
        startv.erase(startv.begin());
        endv.erase(endv.begin());
      }
      sigma=4;
      flag=1;
      if (kEc::gDebug) cerr << "No baseline up to 200 for station " << Id << ", pmt " << pmt << ", gain " << gain << "... strange..." << endl;
    }
  }
  if (!startv.size()) {
    if (kEc::gDebug) cerr << "No baseline piece found for station " << Id << endl;
    return 0;
  }
  vector<double> basev,basev2;
  basev.reserve(startv.size());
  basev2.reserve(startv.size());
  for (unsigned int i=0;i<startv.size();i++) { // computing average and sigma in each piece
    double v=0,v2=0;
    for (unsigned int j=startv[i];j<endv[i];j++) {
      v+=Fadc->Trace[pmt][gain][j];
      v2+=Fadc->Trace[pmt][gain][j]*Fadc->Trace[pmt][gain][j];
    }
    basev[i]=v/(endv[i]-startv[i]);
    basev2[i]=v2/(endv[i]-startv[i]);
  }
  if (reaches_zero) basev[startv.size()-1]=0;
  // Compute sigma from each baseline piece, taking biggest piece
  unsigned int maxpiece=0;
  for (unsigned int i=0;i<startv.size();i++)
    if (endv[i]-startv[i]>maxpiece) {
      maxpiece=endv[i]-startv[i];
      fPmt[pmt].fBaselineSigma[gain]=sqrt(basev2[i]-basev[i]*basev[i]);
    }
  if (sigma!=3 && !is_satured && kEc::gDebug) // FIXME
    cerr << "Noisy baseline piece found for station " << Id << " (sigma= " << sigma << ", sigma " << fPmt[pmt].fBaselineSigma[gain] << ")" << endl;
  for (unsigned int i=0;i<startv.size();i++) // in baseline pieces, put computed value
    for (unsigned int j=startv[i];j<endv[i];j++) fPmt[pmt].fADCBaseline[gain][j]=basev[i];
  // treating intermediate values: one baseline piece, signal, another baseline piece
  if (startv.size()>1) {
    for (unsigned int i=0;i<startv.size()-1;i++) {
      if (fPmt[pmt].fADCBaseline[gain][startv[i]]==fPmt[pmt].fADCBaseline[gain][startv[i+1]]) {// same baseline, no interpolation
        for (unsigned int j=endv[i];j<startv[i+1];j++) fPmt[pmt].fADCBaseline[gain][j]=fPmt[pmt].fADCBaseline[gain][startv[i]];
      } else {
        double v=0;
        double b=fPmt[pmt].fADCBaseline[gain][startv[i]]; // baseline of starting piece
        // computing integrated charge
        for (unsigned int j=endv[i];j<startv[i+1];j++) // from end of i to start of i+1
          if (Fadc->Trace[pmt][gain][j]>b) v+=Fadc->Trace[pmt][gain][j]-b;
        if (v==0) { // linear interpolation
          for (unsigned int j=endv[i];j<startv[i+1];j++)
            fPmt[pmt].fADCBaseline[gain][j]=fPmt[pmt].fADCBaseline[gain][startv[i+1]]+(fPmt[pmt].fADCBaseline[gain][startv[i]]-fPmt[pmt].fADCBaseline[gain][startv[i+1]])*(startv[i+1]-j)*1./(startv[i+1]-endv[i]);
        } else {
          double dv=(fPmt[pmt].fADCBaseline[gain][startv[i+1]]-b)/v;
          v=0;
          for (unsigned int j=endv[i];j<startv[i+1];j++) {
            if ((Fadc->Trace[pmt][gain][j])>b) v+=Fadc->Trace[pmt][gain][j]-b;
            fPmt[pmt].fADCBaseline[gain][j]=b+v*dv;
          }
        }
      }
    }
  }
  // treating case of starting or ending signal, use theoretical factor from Simone and Gianni
  if (startv[0]!=0) {
    double b=basev[0];
    double v=0;
    for (int j=startv[0]-1;j>=0;j--) {
      if (Fadc->Trace[pmt][gain][j]>b) v+=Fadc->Trace[pmt][gain][j]-b;
      fPmt[pmt].fADCBaseline[gain][j]=b+v*CORRFACT;
    }
  }
  if (endv[endv.size()-1]!=kIoSd_MAXSAMPLE) {
    double b=basev[endv.size()-1];
    double v=0;
    for (unsigned int j=endv[endv.size()-1];j<kIoSd_MAXSAMPLE;j++) {
      if (Fadc->Trace[pmt][gain][j]>b) v+=Fadc->Trace[pmt][gain][j]-b;
      fPmt[pmt].fADCBaseline[gain][j]=b-v*CORRFACT;
    }
  }
  // fill bins 760-768 with baseline
  for (unsigned int i=kIoSd_MAXSAMPLE;i<kIoSd::MAXSAMPLE;i++)
    Fadc->Trace[pmt][gain][i]=fPmt[pmt].fADCBaseline[gain][kIoSd_MAXSAMPLE-1];
  return 1;
}

// End of Baseline computation


// Once Baseline has been computed, one looks for signals in dynode and/or
//      anode, depending on saturation. Portions of signals are defined
//      based on a modified version of the CleanFilter of Pierre Billoir.
//      One then fills the appropriate vectors of signals for each Pmt.

int TCalibStation::BuildSignals() {
  int ret=0;
  for (int i=0;i<3;i++) {
    ret+=BuildSignals(i);
  }
  // building Station trace
  int nb=0;
  for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++)
    fTrace[i]=0;
  for (int j=0;j<3;j++) {
    if (!Calib->TubeOk[j]) continue;
    nb++;
    for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++)
      fTrace[i]+=fPmt[j].fTrace[i];
  }
  if (nb) for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++)
      fTrace[i]/=nb;
  return ret;
}

int TCalibStation::BuildSignals(int pmt) {
  // first, build PMT trace
  // look for saturated dynode, if any...
  int gain=IoSdEvent::eHigh;
  double gainV=1.0;
  for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++) {
    if (Fadc->Trace[pmt][IoSdEvent::eHigh][i]>SATUR){
      fPmt[pmt].fHighGainSat++;
      if (Fadc->Trace[pmt][IoSdEvent::eLow][i]>SATUR) 
	fPmt[pmt].fLowGainSat++;
    }
  }
  if (fPmt[pmt].fHighGainSat) {
    gain=IoSdEvent::eLow;
    gainV=fPmt[pmt].fDA;
    // check there is signal on anode or put TubeOk=0
    int big=0,sig=0;
    for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++) {
      if (Fadc->Trace[pmt][gain][i]>300) big++;
      if (Fadc->Trace[pmt][gain][i]-1.0*fPmt[pmt].fADCBaseline[gain][i]>10) sig++;
    }
    // if less than 2 bins with reasonable signal or mora than 500 with big baseline, turn tube off.
    if (sig<2 || big>500) Calib->TubeOk[pmt]=0;
  }
  for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++) // fTrace[i] en VemPeak
    fPmt[pmt].fTrace[i]=(Fadc->Trace[pmt][gain][i]-1.0*fPmt[pmt].fADCBaseline[gain][i])*gainV/fPmt[pmt].fVemPeak;
  // if PMT is masked, don't use, trust online calibration...
  if (!Calib->TubeOk[pmt]) return 0;
  // now PMT trace is Ok, look for signals. Note: this is done on the FADC
  // trace since what matters is ADC counts for fluctuations...
  fPmt[pmt].fSignals.reserve(20);
  //---------------- variables for signal search --------------
  int nb_bin = 0;
  int start_bin = 0;
  int intrace = 0;
  float sig_piece = 0.;
  float max_piece = 0.;

  for (unsigned int i=0;i<kIoSd_MAXSAMPLE;i++) {
    // Always use high gain to look for piece of signal
    if (Fadc->Trace[pmt][IoSdEvent::eHigh][i]>fPmt[pmt].fADCBaseline[IoSdEvent::eHigh][i]+THRESH) { // if signal
      nb_bin++;
      sig_piece += fPmt[pmt].fTrace[i]; // FIXME *fPmt[pmt].fVemPeak/fPmt[pmt].fVemCharge;
      if (fPmt[pmt].fTrace[i]>max_piece) max_piece=fPmt[pmt].fTrace[i];
      if(intrace==0 && nb_bin==2) start_bin = i-1; // if 2 consecutive bins with signal => true signal
      intrace=(nb_bin>=2)? 1 : 0;
      if(intrace==1 && i==kIoSd_MAXSAMPLE-1) {// End of trace
        TSignal *s=new TSignal();
        s->fNbBin  = nb_bin;
        s->fSignal = sig_piece;
        s->fPeak = max_piece;
        s->fStart  = start_bin;
        s->fGain   = gain;
        s->fEnd    = i;
        if (s->fSignal>0) fPmt[pmt].fRawSignals.push_back(*s);
        delete s;
      }
    } else {
      if(intrace==1) { // End of trace
        TSignal *s=new TSignal();
        s->fNbBin  = nb_bin;
        s->fSignal = sig_piece;
        s->fPeak = max_piece;
        s->fStart  = start_bin;
        s->fGain   = gain;
        s->fEnd    = i;
        if (s->fSignal>0) fPmt[pmt].fRawSignals.push_back(*s);
        delete s;
      }
      intrace = 0; start_bin = 0; sig_piece = 0.; nb_bin = 0; max_piece = 0.;// we're out of trace now in any case
    }
  }

  // Part where pieces are joined:
  // 2nd piece must be less than 20 + nb_bin(first piece) bins apart
  // AND ((( 1st piece must be at least 30% of signal of second piece
  // OR 2nd piece has peak below 5 )))
  // (else one merges small muon peaks before real physics) 
  fPmt[pmt].fSignals.reserve(fPmt[pmt].fRawSignals.size());
  for (unsigned int i=0;i<fPmt[pmt].fRawSignals.size();i++) fPmt[pmt].fSignals.push_back(fPmt[pmt].fRawSignals[i]);
  int dowork = 1; unsigned int curr_pos = 0;
  while(dowork && fPmt[pmt].fSignals.size()!=0) {
    for(unsigned int n=0+curr_pos; n<(fPmt[pmt].fSignals.size()-1); n++) {
      if( ((fPmt[pmt].fSignals[n+1].fStart-fPmt[pmt].fSignals[curr_pos].fEnd)
           <(20+fPmt[pmt].fSignals[curr_pos].fNbBin))
          && ((0.3 < fPmt[pmt].fSignals[curr_pos].fSignal/fPmt[pmt].fSignals[n+1].fSignal) || fPmt[pmt].fSignals[n+1].fPeak<5)
          &&(fPmt[pmt].fSignals[n+1].fSignal!= 0) ) {// conditions for merging
        // Total signal is sum of 2 signals
        fPmt[pmt].fSignals[curr_pos].fSignal += fPmt[pmt].fSignals[n+1].fSignal;
        // + baseline in between (needed as 1- could be non strickly 0
        //                                  2- will count as noise )
        for (int i=fPmt[pmt].fSignals[curr_pos].fEnd+1;i<fPmt[pmt].fSignals[n+1].fStart;i++)
          fPmt[pmt].fSignals[curr_pos].fSignal += fPmt[pmt].fTrace[i];
        // fPmt[pmt].fSignals[curr_pos].fStart does not change
        fPmt[pmt].fSignals[curr_pos].fEnd     = fPmt[pmt].fSignals[n+1].fEnd;
        fPmt[pmt].fSignals[curr_pos].fNbBin  += fPmt[pmt].fSignals[n+1].fNbBin;
        fPmt[pmt].fSignals.erase(fPmt[pmt].fSignals.begin()+(n+1));
        n--;
      } else {
        break;
      }
    }
    curr_pos++;
    dowork = (fPmt[pmt].fSignals.size()!=curr_pos) ? 1 : 0;
  }
  return 1;
}
// End of Signal search

// Now, one has to merge the selected signals in individual PMTs
int TCalibStation::MergeSignals() {
  int nb_pmt=0;
  for (unsigned int j=0;j<3;j++) {
    if (!Calib->TubeOk[j]) continue;
    nb_pmt++;
  }
  // one only has "working" PMTs signals
  for (unsigned int pmt=0;pmt<3;pmt++) {
    for (unsigned int i=0;i<fPmt[pmt].fSignals.size();i++) {
      int st=fPmt[pmt].fSignals[i].fStart;
      int en=fPmt[pmt].fSignals[i].fEnd;
      // try to merge with others until none worked
      int loop=1;
      while (loop) {
        loop=0;
        for (unsigned int pmt2=0;pmt2<3;pmt2++)
          for (unsigned int j=0;j<fPmt[pmt2].fSignals.size();j++) {
            if (pmt==pmt2 && i==j) continue;
            int st2=fPmt[pmt2].fSignals[j].fStart;
            int en2=fPmt[pmt2].fSignals[j].fEnd;
            if ((st2<=st && en2>=st) || (st2<=en && st2>=st)) {
              // they overlap...
              int mst=(st<st2 ? st : st2); // min st
              int Men=(en>en2 ? en : en2); // max en
              if (mst!=st || Men!=en) {
                // they overlap and the union is bigger than the current piece
                st=mst;
                en=Men;
                loop=1;
              }
            }
          }
      }
      // we have a piece st-en, add it to the station
      // check if it's not included in another signal
      int ok=1;
      for (unsigned int j=0;j<fSignals.size();j++)
        if (st>=fSignals[j].fStart && en<=fSignals[j].fEnd) ok=0;
      if (ok) {
        TSignal *s=new TSignal();
        s->fStart=st;
        s->fEnd=en;
        double nb_bin=0;
        double signal=0;
        for (unsigned int j=0;j<3;j++) {
          if (!Calib->TubeOk[j]) continue;
          for (int k=st; k<en;k++) {
            if (Fadc->Trace[j][IoSdEvent::eHigh][k]>=fPmt[j].fADCBaseline[IoSdEvent::eHigh][k]+THRESH) nb_bin++;
            signal+=fPmt[j].fTrace[k]*fPmt[j].fVemPeak/fPmt[j].fVemCharge;
          }
        }
        s->fNbBin=nb_bin/nb_pmt;
        s->fSignal=signal/nb_pmt;
        fSignals.push_back(*s);
        delete s;
      }
    }
  }
  // adding 10 bins (if possible) after each signal
  // warning, they are not necessarily ordered...
  for (unsigned int k=0;k<fSignals.size();k++) {
    int extra=10;
    for (unsigned int l=0; l<fSignals.size(); l++) {
      if (k!=l && fSignals[k].fStart < fSignals[l].fStart && fSignals[k].fEnd+10>=fSignals[l].fStart)
      extra=fSignals[l].fStart-fSignals[k].fEnd-1;
    }
    if (fSignals[k].fEnd+extra>(int)kIoSd_MAXSAMPLE) extra=kIoSd_MAXSAMPLE-fSignals[k].fEnd;
    if (extra>0) for (unsigned int j=0;j<3;j++) {
      if (!Calib->TubeOk[j]) continue;
      for (int i=0;i<extra;i++) {
        int bin=fSignals[k].fEnd-1+extra;
        if (Fadc->Trace[j][IoSdEvent::eHigh][bin]>=fPmt[j].fADCBaseline[IoSdEvent::eHigh][bin]+THRESH) fSignals[k].fNbBin+=1./nb_pmt;
        fSignals[k].fSignal+=fPmt[j].fTrace[bin]*fPmt[j].fVemPeak/fPmt[j].fVemCharge/nb_pmt;
      }
    }
    fSignals[k].fEnd+=extra;
  }
  return 1;
}

// Finally, select best signal and fill relevant parameters
int TCalibStation::SelectSignal() {
  int ok=0;
  double v=0;
  for (unsigned int i=0;i<fSignals.size();i++) {
    if (fSignals[i].fNbBin*fSignals[i].fSignal>v) {
      v=fSignals[i].fNbBin*fSignals[i].fSignal;
      ok=i;
    }
  }
  // TODO what to do if fSignals.size()==0
  if (v==0) {
    fStartIntegration=0;
    fEndIntegration=0;
    fStartBin=0;
    fSigInVEM=0.;
    return 0;
  }
  // filling individual PMTs stuff
  for (unsigned int j=0;j<3;j++) {
    fPmt[j].fSigInVEM=0;
    fPmt[j].fPeakInVEM=0;
    if (!Calib->TubeOk[j]) continue;
    for (int i=fSignals[ok].fStart;i<fSignals[ok].fEnd;i++) {
      double x=fPmt[j].fTrace[i];
      if (x>fPmt[j].fPeakInVEM) fPmt[j].fPeakInVEM=x;
      fPmt[j].fSigInVEM+=x*fPmt[j].fVemPeak/fPmt[j].fVemCharge;
    }
  }
  fStartIntegration=fSignals[ok].fStart;
  fEndIntegration=fSignals[ok].fEnd;
  fStartBin=fSignals[ok].fStart;
  fSigInVEM=fSignals[ok].fSignal;
  return 1;
}


//
int TCalibStation::TreatSaturated() {
  for (unsigned int i=0;i<3;i++) {
    if (Calib->TubeOk[i] && (!fPmt[i].fLowGainSat || !fPmt[i].fHighGainSat)) return 1;
  }
  // low gain saturated
  // using GAP 2005-025 to recover saturated anodes
  // Warning, error is large in this case, not sqrt(signal)!
  fSat = 1;
  for (unsigned int j = 0; j < kIoSd::NPMT; j++) {
    if (!Calib->TubeOk[j]) continue;
    if (!fPmt[j].fLowGainSat) continue;
    // computing anode undershoot
    float anode_begin=fPmt[j].fADCBaseline[IoSdEvent::eLow][0];
    float anode_end=fPmt[j].fADCBaseline[IoSdEvent::eLow][kIoSd_MAXSAMPLE-1];
    float newsig=0;
    if (anode_begin-anode_end>1) {
      fSat=16; // percent of error
      newsig = (1669 + 14755*(anode_begin-anode_end)) * fPmt[j].fDA / fPmt[j].VemCharge();
    } else {
      fSat = 10; // percent of error
      // computing dynode undershoot
      float dynode_begin=fPmt[j].fADCBaseline[IoSdEvent::eHigh][0];
      float dynode_end=fPmt[j].fADCBaseline[IoSdEvent::eHigh][kIoSd_MAXSAMPLE-1];
      // computing dynode charge
      float qd=0;
      for (int i = fStartIntegration; i < fEndIntegration; i++) {
        float s=Fadc->Trace[j][IoSdEvent::eHigh][i];
        if (s-dynode_begin>1) qd+=s-dynode_begin;
      }
      newsig = (770*((dynode_begin - dynode_end) - 1.35e-4 *qd + 0.055 *fPmt[j].fHighGainSat) -222)* fPmt[j].fDA/fPmt[j].VemCharge();
    }
    if (newsig>fPmt[j].fSigInVEM) fPmt[j].fSigInVEM=newsig;
  }
  fSigInVEM = (Calib->TubeOk[0]*fPmt[0].fSigInVEM + Calib->TubeOk[1]*fPmt[1].fSigInVEM + Calib->TubeOk[2]*fPmt[2].fSigInVEM)/(Calib->TubeOk[0]+Calib->TubeOk[1]+Calib->TubeOk[2]);
  return 1;
}
