#include "Ec.h"
#include "STCconstants.h"

#define SATUREXTRAP 1

void OldComplain(const char * msg) {
  static bool already=false;
  if (already) return;
  if (Ec_UseOldCalib()) {
    already=true;
    cerr << "WARNING!!!! Using old calibration..." << endl;
    return;
  }
  already=true;
  cerr << "WARNING!!!! Calling an old function from EcOld.cc (" << msg << ")" << endl;
  cerr << "This function is not supported and could disappear in next CDAS release" << endl;
}

#define sqr(x) ((x)*(x))

/* --------------------------------------------------------- */
int TCalibStation::Anomaly()
/* --------------------------------------------------------- */
{
  OldComplain("Anomaly");
  unsigned int j;
  int k, over, under, slot_over, slot_under, nsatur_dynode[3],
  nsatur_anode[3];
  int n_dynode_under[3], n_anode_under[3], n_dynode_cross[3],
  n_anode_cross[3];
  int anomaly_level = 0;
  int n_missing_PMT = 0;

  // counting missing PMTs and setting counters to zero
  for (k = 0; k < 3; k++) {
    n_dynode_under[k] = n_dynode_cross[k] = n_anode_under[k] =
                                              n_anode_cross[k] = 0;
    n_missing_PMT += !Calib->TubeOk[k];
  }
  anomaly_level += n_missing_PMT;

  // checking saturations
  for (k = 0; k < 3; k++)
    if (Calib->TubeOk[k]) {
      nsatur_dynode[k] = nsatur_anode[k] = 0;
      for (j = fStartBin; j < fStartBin + kEc::NBIN_LOOK; j++) {
        nsatur_dynode[k] += Fadc->Trace[k][1][j] == 1023;
        nsatur_anode[k] += Fadc->Trace[k][0][j] == 1023;
      }
    }
  // Anomalies in dynode FADCs
  // counting number of slots "significantly under" baseline within kEc::NBIN_LOOK slots
  // counting of "crosses" (transitions between "significantly over" and "significantly under")
  // (the significance is modulated by the degree of saturation)
  for (k = 0; k < 3; k++)
    if (Calib->TubeOk[k]) {
      over = under = 0;
      for (j = fStartBin; j < fStartBin + kEc::NBIN_LOOK; j++) {
        double toler =
          kEc::SHIFT_FROM_BASE * (1 + .2 * nsatur_dynode[k]);
        slot_over = Fadc->Trace[k][1][j] > Calib->Base[k] + toler;
        slot_under = Fadc->Trace[k][1][j] < Calib->Base[k] - toler;
        n_dynode_under[k] += slot_under;
        if ((over && slot_under) || (under && slot_over))
          n_dynode_cross[k]++;
        if (slot_over) {
          under = 0;
          over = 1;
        }
        if (slot_under) {
          over = 0;
          under = 1;
        }
      }
    }
  // Anomalies in anode FADCs
  for (k = 0; k < 3; k++)
    if (Calib->TubeOk[k]) {
      over = under = 0;
      for (j = fStartBin; j < fStartBin + kEc::NBIN_LOOK; j++) {
        double toler =
          kEc::SHIFT_FROM_BASE * (1 + .2 * nsatur_anode[k]);
        slot_over =
          Fadc->Trace[k][0][j] > Calib->Base[k + 3] + toler;
        slot_under =
          Fadc->Trace[k][0][j] < Calib->Base[k + 3] - toler;
        n_anode_under[k] += slot_under;
        if ((over && slot_under) || (under && slot_over))
          n_anode_cross[k]++;
        if (slot_over) {
          under = 0;
          over = 1;
        }
        if (slot_under) {
          over = 0;
          under = 1;
        }
      }
    }
  if (n_dynode_under[0] + n_dynode_under[1] + n_dynode_under[2] >
      kEc::NMAX_UNDERSHOOT)
    anomaly_level += 0;
  if (n_anode_under[0] + n_anode_under[1] + n_anode_under[2] >
      kEc::NMAX_UNDERSHOOT)
    anomaly_level += 100;
  if (n_dynode_cross[0] + n_dynode_cross[1] + n_dynode_cross[2] >
      kEc::NMAX_CROSS)
    anomaly_level += 1000;
  if (n_anode_cross[0] + n_anode_cross[1] + n_anode_cross[2] >
      kEc::NMAX_CROSS)
    anomaly_level += 10000;
  if (anomaly_level) {
    if (1) {
      printf("!!!! stat %3d  FADC anomaly level %d :", Id,
             anomaly_level);
      printf(" dynode under");
      for (k = 0; k < 3; k++) {
        if (Calib->TubeOk[k])
          printf(" %d", n_dynode_under[k]);
        else
          printf(" -");
      }
      printf("  cross");
      for (k = 0; k < 3; k++) {
        if (Calib->TubeOk[k])
          printf(" %d", n_dynode_cross[k]);
        else
          printf(" -");
      }
      printf("  anode under");
      for (k = 0; k < 3; k++) {
        if (Calib->TubeOk[k])
          printf(" %d", n_anode_under[k]);
        else
          printf(" -");
      }
      printf("  cross");
      for (k = 0; k < 3; k++) {
        if (Calib->TubeOk[k])
          printf(" %d", n_anode_cross[k]);
        else
          printf(" -");
      }
      printf("\n");
    }
  }
  // negative signal
  if (fSigInVEM < 0) {
    if (1)
      printf("!!!! stat %d  sigVEM %f\n", Id, fSigInVEM);
    anomaly_level += 100000;
  }
  // abnormal trigger type
  if (Trigger.Type == 16) {
    anomaly_level += 1000000;
    if (1)
      printf("!!!! stat %d trigger 16\n", Id);
  }
  // abnormal Id (including 37 (Laura) for real events)
  //  if(Id<=0 || (!Simulation && Id==37)) {
  if (Id <= 0 || (Id == 37)) {
    anomaly_level += 10000000;
    if (1)
      printf("!!!! stat Id %d\n", Id);
  }
  return anomaly_level;
}

/* Adapted from P. Billoir, additional bugs by Carla  -------------------------------------*/
#define NCOEF 2
#define START_SLOT 100
#define END_SLOT 500
#define NSLOT_WINDOW_UP 30
#define NSLOT_WINDOW_DOWN 50
#define THRESHOLD_CLEAN_UP 0.1
#define THRESHOLD_CLEAN_DOWN 0.05
#define MARGIN_STOP 5
#define THRESHOLD_SUM 3.
#define maxval(a,b) ( (a)>(b) ? (a) : (b) )
#define minval(a,b) ( (a)<(b) ? (a) : (b) )

/*----------------------------------------------------------------------*/
int TCalibStation::CleanFilter(int& start, int& stop, double *trace)
/*----------------------------------------------------------------------*/
{
  OldComplain("CleanFilter");
  //  double trace[kIoSd::MAXSAMPLE];
  double ftrace[kIoSd::MAXSAMPLE];
  double coef[NCOEF] = {3.1,-2.1}; // corresponds to a decay time of 64 ns for the light
  int  above, nabove, npeak, first, last;
  //  double fact[3];
  double sum;

  for (unsigned int j=0; j<kIoSd::MAXSAMPLE; j++) ftrace[j] = 0;
#if 0
  for (j=0; j<kIoSd::MAXSAMPLE; j++) trace[j] = ftrace[j] = 0;
  ntubeOK = 0;

  for(k=0; k<3; k++) {
    if(Calib->TubeOk[k]) {
      ntubeOK++;
      fact[k] = fPmt[k].VemPeak() / fPmt[k].VemCharge();
    }
  }

  for(j=0; j<kIoSd::MAXSAMPLE; j++) for(k=0; k<3; k++) if(Calib->TubeOk[k]) {
        double val =  fPmt[k].fTrace[j];
        if(j>0 && j<kIoSd::MAXSAMPLE-1) {
          double valp =  fPmt[k].fTrace[j-1];
          double valf =  fPmt[k].fTrace[j+1];
          // suppress isolated PMT pulse (1 slot well above the previous and the next one)
          if (val>3*valp && val>3*valf) val = (valp+valf)/2;
        }
        trace[j] += maxval(0,val) * fact[k] / ntubeOK;
      }
#endif
  // build the filtered trace
  for(unsigned int j = NCOEF-1; j < kIoSd::MAXSAMPLE; j++) for(unsigned int k = 0; k < NCOEF; k++)
      ftrace[j] += coef[k]*trace[j-k];

  // search for the "start" point -------------------------------------

  //int start_peak = END_SLOT;

  start = stop = -1;
  // ascending search for start using a sliding window
  for(int j = START_SLOT; j < END_SLOT-NSLOT_WINDOW_UP; j++) {
    first = -1;
    above = 0;
    nabove = 0;
    npeak = 0;
    sum = 0;
    // scan a window of length NSLOT_WINDOW_UP
    for(int k = 0; k < NSLOT_WINDOW_UP; k++) {
      if(ftrace[j+k]>THRESHOLD_CLEAN_UP) {
        if(first<0) first = j+k;
        if(!above) nabove = 0;
        above = 1;
        nabove++;
        //if(start_peak==END_SLOT) start_peak = j+k;
        sum += ftrace[j+k];
        if(nabove>2 || sum>THRESHOLD_SUM) {
          start = first; break;
        }
      } else {
        if(above) npeak++;
        if(npeak>1) {
          start = first; break;
        }
        above = 0;
        sum = 0;
      }
    }
    if(start>=0) break;
  }
  if(start<0) start = END_SLOT;

  // search for the "stop" point -------------------------------------

  //int stop_peak = 0;
  // descending search for stop using a sliding window
  for(int j = END_SLOT; j > START_SLOT+NSLOT_WINDOW_DOWN; j--) {
    last = -1;
    above = 0;
    nabove = 0;
    npeak = 0;
    sum = 0;
    // scan a window of length NSLOT_WINDOW_DOWN
    for(int k = 0; k < NSLOT_WINDOW_DOWN; k++) {
      if(ftrace[j-k]>THRESHOLD_CLEAN_DOWN) {
        if(last<0) last = j-k;
        if(!above) nabove = 0;
        above = 1;
        nabove++;
        //if(stop_peak==0) stop_peak = j-k;
        sum += ftrace[j+k];
        if(nabove>2 || sum>THRESHOLD_SUM) {
          stop = last; break;
        }
      } else {
        if(above) npeak++;
        if(npeak>1) {
          stop = last; break;
        }
        above = 0;
        sum = 0;
      }
    }
    if(stop>=0) break;
  }
  if(stop<start) return 0;

  stop += MARGIN_STOP;
  return 1;
}

#define NSIGMA_BEFORE 3
#define NSIGMA_AFTER 5

void TCalibStation::CorrectTraceTime() {
  OldComplain("CorrectTraceTime");
  double trace[kIoSd::MAXSAMPLE], fact[3];
  double sum, sum1, sum2, average, sigma, dsum, tc10, tc50, tc90, tc70;
  int ntubeOK,j1,j2,j1n,j2n;

  for (unsigned int j = 0; j < kIoSd::MAXSAMPLE; j++) trace[j] = 0;
  ntubeOK = 0;
  for(unsigned int k = 0; k < kIoSd::NPMT; k++) {
    if(Calib->TubeOk[k]) {
      ntubeOK++;
      fact[k] = fPmt[k].VemPeak() / fPmt[k].VemCharge();
    }
  }

  for(unsigned int j = 0; j < kIoSd::MAXSAMPLE; j++) for(unsigned int k = 0; k < kIoSd::NPMT; k++) if(Calib->TubeOk[k]) {
        double val =  fPmt[k].fTrace[j];

        // suppress isolated PMT pulse (1 slot well above the previous and the next one)
        if(j>0 && j<kIoSd::MAXSAMPLE-1) {
          double valp = fPmt[k].fTrace[j-1];
          double valf = fPmt[k].fTrace[j+1];
          if (val>3*valp && val>3*valf) val = (valp+valf)/2;
        }
        trace[j] += maxval(0,val) * fact[k] / ntubeOK;
      }

  int start, stop;
  if (!CleanFilter(start, stop, trace)) return;

  j1n = start;
  j2n = stop;

  do {
    j1 = j1n; j2 = j2n;
    sum = sum1 = sum2 = 0;
    for(int j = j1; j < j2; j++) {
      sum += trace[j];
      sum1 += j*trace[j];
      sum2 += j*j*trace[j];
    }
    average = sum1/sum;
    sigma = sqrt(sum2/sum-average*average);
    j1n = maxval(start,(int)(average-NSIGMA_BEFORE*sigma));
    j2n = minval(stop,(int)(average+NSIGMA_AFTER*sigma));
  } while(j1n<j1 || j2n>j2);

  sum = 0;
  for(int j = j1n; j < j2n; j++) sum += trace[j];
  sum1 = tc10 = tc50 = tc90 = tc70 = 0;
  for(int j = j1n; j < j2n; j++) {
    sum1 += trace[j];
    if(!tc10) if((dsum=sum1-.1*sum)>0) tc10 = j-dsum/trace[j];
    if(!tc50) if((dsum=sum1-.5*sum)>0) tc50 = j-dsum/trace[j];
    if(!tc70) if((dsum=sum1-.7*sum)>0) tc70 = j-dsum/trace[j];
    if(!tc90) if((dsum=sum1-.9*sum)>0) tc90 = j-dsum/trace[j];
  }

  fT10 = tc10*kEc::TimeSlot;
  fT50 = tc50*kEc::TimeSlot;
  fT70 = tc70*kEc::TimeSlot;
  fT90 = tc90*kEc::TimeSlot;
  fStartIntegration = start;
  fEndIntegration = stop;
  fStartBin = start;
  fSigInVEM = sum;
}


/* End of adapted from P. Billoir, additionnal bugs by Carla ------------------- */


/* --------------------------------------------------------- */
bool TCalibStation::OldIsLightningEvent()
/* --------------------------------------------------------- */
// voir le signal vu par Piera qui n'est pas un eclair mais declare tel par cette fonction
{
  OldComplain("OldIsLightningEvent");
  int count[3] = { 0, 0, 0 };
  int cross[3] = { 1, 1, 1 };
  for (unsigned int j = 0; j < kIoSd::NPMT; j++) {
    for (unsigned int i = 0; i < kIoSd::MAXSAMPLE; i++) {
      if (Fadc->Trace[j][IoSdEvent::eHigh][i] > 1000)
        return false;
      if (Fadc->Trace[j][IoSdEvent::eHigh][i] < Calib->Base[j] - 4
          && cross[j]) {
        count[j]++;
        cross[j] = 0;
      }
      if (Fadc->Trace[j][IoSdEvent::eHigh][i] > Calib->Base[j] + 4
          && !cross[j])
        cross[j] = 1;
    }
  }
  if (count[0] > 2 && count[1] > 3 && count[2] > 3)
    return true;            // PMT 1 has smaller signal
  return false;
}

/* --------------------------------------------------------- */
double TCalibStation::ShapeParameter(double dist)
/* --------------------------------------------------------- */
{
  OldComplain("ShapeParameter");
  int k, nPMT = 0;
  double fact[3];
  double sum1 = 0, sum2 = 0;
  for (k = 0; k < 3; k++) {
#if 0
    PmtSignal(k);
#endif
    if (Calib->TubeOk[k]) {
      fact[k] = Calib->VemPeak[k] / Calib->VemCharge[k];
      //      cout << fact[k] << " " << fPmt[k].PeakInVEM()/fPmt[k].SigInVEM() << "----" << endl;
      nPMT++;
    }
  }

  for (unsigned int j = 0; j < kIoSd::MAXSAMPLE; j++) {
    double val = 0;
    for (k = 0; k < 3; k++)
      if (Calib->TubeOk[k])
        val += fPmt[k].fTrace[j] * fact[k];
    val /= nPMT;
    if ((j - fStartBin) / (dist / kEc::SHAPE_DIST_REF) <
        kEc::SHAPE_CUT)
      sum1 += val;
    else
      sum2 += val;
  }
  return sum1 / (sum1 + sum2);
}

/* --------------------------------------------------------- */
void TCalibStation::EstimateCalibration(int n)  // 0, 1 or 2
/* --------------------------------------------------------- */
{
  OldComplain("EstimateCalibration");
  int i, iter = 0;
  float moy1, moy2, var1, var2;
  float samples = Fadc->NSample;

  if (samples > 768)
    samples = 768;
  do {
    moy1 = moy2 = 0;
    for (i = (iter * 50); i < 50 * (iter + 1); i++) {
      moy1 += Fadc->Trace[n][IoSdEvent::eHigh][i];
      moy2 += Fadc->Trace[n][IoSdEvent::eLow][i];
    }
    moy1 /= 50;
    moy2 /= 50;
    var1 = 0;
    var2 = 0;
    for (i = (iter * 50); i < 50 * (iter + 1); i++) {
      var1 += sqr(Fadc->Trace[n][IoSdEvent::eHigh][i] - moy1);
      var2 += sqr(Fadc->Trace[n][IoSdEvent::eLow][i] - moy2);
    }
    var1 /= 49;
    var2 /= 49;
    iter++;
    if (i > (int) (samples - 250)) {
      moy1 = moy2 = 0;
      for (i = 0; i < 50; i++) {
        moy1 += Fadc->Trace[n][IoSdEvent::eHigh][i];
        moy2 += Fadc->Trace[n][IoSdEvent::eLow][i];
      }
      moy1 /= 50;
      moy2 /= 50;
      break;
    }
  } while (sqrt(var1) > 2 || sqrt(var2) > 2);
  bool satuseen = 0;
  for (i = 0; i < (int) (samples); i++) {
    if (!satuseen && Fadc->Trace[n][IoSdEvent::eHigh][i] == 0x3FF) {
      i = -1;             //we begin from the start
      fPmt[n].fHighGainSat = 1;
      satuseen = 1;
      continue;
    }
    if (fPmt[n].fHighGainSat) {
      if (!fPmt[n].fLowGainSat
          && Fadc->Trace[n][IoSdEvent::eLow][i] == 0x3FF)
        fPmt[n].fLowGainSat = 1;
      fPmt[n].fTrace[i] =
        (Fadc->Trace[n][IoSdEvent::eLow][i] - moy2) * 32;
      if (fPmt[n].fTrace[i] < kEc::SignalCut * 32)
        fPmt[n].fTrace[i] = 0;
    } else {
      fPmt[n].fTrace[i] =
        (Fadc->Trace[n][IoSdEvent::eHigh][i] - moy1);
      if (fPmt[n].fTrace[i] < kEc::SignalCut)
        fPmt[n].fTrace[i] = 0;
    }
    fPmt[n].fTrace[i] /= fPmt[n].VemPeak(); //(53.);
  }
}

/* --------------------------------------------------------- */
static void _ApplyCalib(TCalibStation * T, int n)
/* --------------------------------------------------------- */
{
  OldComplain("_ApplyCalib");
  int samples = T->Fadc->NSample;
  T->fPmt[n].fDA=T->Calib->DA[n];
  if (T->Calib->Version == 12) T->fPmt[n].fDA /= 1.07;             // from rate DA method
  if (samples > 768) samples = 768;
  bool satuseen = 0;
  if (T->Fadc->Trace[n][IoSdEvent::eHigh][1]>T->Calib->Base[n]+300 ||
      T->Fadc->Trace[n][IoSdEvent::eLow][1]>T->Calib->Base[n+3]+300) {
    T->Calib->TubeOk[n]=0;  // modifying Calib block, this is evil, should have an offline equivalent
    T->Calib->Base[n]=T->Fadc->Trace[n][IoSdEvent::eHigh][1];
    T->Calib->Base[n+3]=T->Fadc->Trace[n][IoSdEvent::eLow][1];
    cerr << "Problem in baseline for station " << T->Id << ", PMT " << n << endl;
  }
#if SATUREXTRAP
  for (int i = 0; i < samples; i++) {
    if (!satuseen && T->Fadc->Trace[n][IoSdEvent::eHigh][i] > 1020) {
      i = -1;             //we begin from the start
      T->fPmt[n].fHighGainSat = 1;
      satuseen = 1;
      continue;
    }
    if (T->fPmt[n].fHighGainSat) {
      if (!T->fPmt[n].fLowGainSat && T->Fadc->Trace[n][IoSdEvent::eLow][i] > 1020) T->fPmt[n].fLowGainSat = 1;
      if (T->Fadc->Trace[n][IoSdEvent::eLow][i] < (int) T->Calib->Base[n + 3] + kEc::SignalCut) T->fPmt[n].fTrace[i] = 0.;
      else T->fPmt[n].fTrace[i] = ((T->Fadc->Trace[n][IoSdEvent::eLow][i] - T->Calib->Base[n + 3]) * T->fPmt[n].fDA);
    } else {
      if (T->Fadc->Trace[n][IoSdEvent::eHigh][i] < (int) T->Calib->Base[n] + kEc::SignalCut) T->fPmt[n].fTrace[i] = 0.;
      else T->fPmt[n].fTrace[i] = (T->Fadc->Trace[n][IoSdEvent::eHigh][i] - T->Calib->Base[n]);
    }
    T->fPmt[n].fTrace[i] /= T->fPmt[n].VemPeak();
  }
#else
  for (int i = 0; i < samples; i++) {
    if (!satuseen && T->Fadc->Trace[n][IoSdEvent::eHigh][i] == 0x3FF) {
      i = -1;             //we begin from the start
      T->fPmt[n].fHighGainSat = 1;
      satuseen = 1;
      continue;
    }
    if (T->fPmt[n].fHighGainSat) {
      if (!T->fPmt[n].fLowGainSat && T->Fadc->Trace[n][IoSdEvent::eLow][i] == 0x3FF) T->fPmt[n].fLowGainSat = 1;
      if (T->Fadc->Trace[n][IoSdEvent::eLow][i] < (int) T->Calib->Base[n + 3] + kEc::SignalCut) T->fPmt[n].fTrace[i] = 0.;
      else
        T->fPmt[n].fTrace[i] = ((T->Fadc->Trace[n][IoSdEvent::eLow][i] - T->Calib->Base[n + 3]) * T->fPmt[n].fDA);
    } else {
      if (T->Fadc->Trace[n][IoSdEvent::eHigh][i] < (int) T->Calib->Base[n] + kEc::SignalCut) T->fPmt[n].fTrace[i] = 0.;
      else T->fPmt[n].fTrace[i] = (T->Fadc->Trace[n][IoSdEvent::eHigh][i] - T->Calib->Base[n]);
    }
    T->fPmt[n].fTrace[i] /= T->fPmt[n].VemPeak();
  }
#endif
  for (int i = 0; i < samples; i++) if (T->fPmt[n].fTrace[i] < 0.1) T->fPmt[n].fTrace[i] = 0.0;
}

void TCalibStation::OldChargeHistogramFit(int n, int force=0) {
  OldComplain("OldChargeHistogramFit");
  if (fPmt[n].fCalibratedState>1 && !force) return; // already calibrated
  if (Calib->Version < 8) return; // old data, no histogram
  if (!Calib->TubeOk[n]) return; // bad PMT
  // extracting VEM from histograms
  TH1F *h = HCharge(n);
  if (h->GetMaximum() < 500) return; // to few entries in histogram

  fPmt[n].fCalibratedState = 2;
  float base = Calib->Base[n];
  if (Calib->Version == 13) base *= 19;     // 19 bins in version 13
  else  base *= 20;
  int max = 0;
  int maxbin = 0;
  int bin = h->GetNbinsX();
  int nbbin = bin;
  int v = 0, v1 = 0, v2 = 0;
  int binmin = 0, binmax = 0;
  int dir = -1;
  while (!binmax) {
    v = (int) h->GetBinContent(bin);
    v1 = (int) h->GetBinContent(bin + dir);
    v2 = (int) h->GetBinContent(bin + 2 * dir);
    if (v > 300 && v > max) {
      max = v;
      maxbin = bin;
    };
    if (binmin && (1. * max) / v > 1.3 && (1. * max) / v1 > 1.3 && (1. * max) / v2 > 1.3) {
      binmax = bin;
    };
    if (!binmin && max && (1. * max) / v > 1.3 && (1. * max) / v1 > 1.3 && (1. * max) / v2 > 1.3) {
      binmin = bin;
      dir = 1;
    };
    bin += dir;
    if (bin == nbbin)  binmax = -1;
    if (bin == 2)  binmax = -1;
  }
  if (Calib->VemCharge[n] < binmax && Calib->VemCharge[n] > binmin && binmin > 0 && binmax < h->GetNbinsX() && binmin < binmax) {     // Good for fit
    double x = 0, x2 = 0, x3 = 0, x4 = 0, y = 0, xy = 0, x2y =
                                         0, v = 0, a = 0;
    int nb = 0;
    for (int i = binmin; i < binmax; i++) {
      v = h->GetBinContent(i);
      a = h->GetBinCenter(i);
      x += a;
      x2 += a * a;
      x3 += a * a * a;
      x4 += a * a * a * a;
      y += v;
      xy += a * v;
      x2y += a * a * v;
      nb++;
    }
    double B =
      (y * (x4 * x - x2 * x3) + xy * (x2 * x2 - nb * x4) +
       x2y * (nb * x3 - x * x2));
    //double C=(y*(x3*x3-x4*x2)+xy*(x*x4-x2*x3)+x2y*(x2*x2-x*x3));
    double A =
      (y * (x2 * x2 - x * x3) + xy * (nb * x3 - x * x2) +
       x2y * (x * x - nb * x2));
    double res = -B / (2 * A);

    if (res > h->GetBinCenter(binmin) && res < h->GetBinCenter(binmax)) {
      fPmt[n].fVemCharge = res - base;
      fPmt[n].fCalibratedState = 3;
    }
  }
}

/* --------------------------------------------------------- */
void TCalibStation::OldCalibrate(int n, int domonit = 0)   // 0, 1 or 2
/* --------------------------------------------------------- */
{
  OldComplain("OldCalibrate");
  // Old data, should not be used
  if (!Calib->TubeOk[n] && !Calib->Base[n]) {
    fPmt[n].fCalibratedState = 1;
    EstimateCalibration(n);
    return;
  }
  OldChargeHistogramFit(n);
  if (domonit) WaterEstimatesFit(n);
  if (fPmt[n].fVemCharge < 0) fPmt[n].fVemCharge = Calib->VemCharge[n] * 1.045;       // From hump to LS
  if (fPmt[n].fVemPeak < 0) fPmt[n].fVemPeak = Calib->VemPeak[n];   // we don't care...
  // from Hump to VEM
  fPmt[n].fVemCharge /= 1.01; // From Laura
  fPmt[n].fVemPeak /= 0.87;   // From Laura
  // Old data
  if (Calib->Version < 12) fPmt[n].fCalibratedState = 2;
  _ApplyCalib(this, n);
}

bool TCalibStation::LoadCalib() {
  OldComplain("LoadCalib");
  size_t calibPos = Name.find("(Calib:");
  if (calibPos > Name.size())
    return false;           // Calib was not saved
  if (fPmt[0].fCalibratedState > 0)
    return true;            // Calib was already done
  std::string values = Name.substr(calibPos, Name.find(')', calibPos));
  sscanf(values.c_str(), "(Calib: %f %f %f %f %f %f)",
         &fPmt[0].fVemCharge, &fPmt[1].fVemCharge, &fPmt[2].fVemCharge,
         &fPmt[0].fVemPeak, &fPmt[1].fVemPeak, &fPmt[2].fVemPeak);
  for (unsigned int i = 0; i < kIoSd::NPMT; i++)
    _ApplyCalib(this, i);
  return true;
}

float TCalibStation::GetBaseline(int pmt, int gain, int start, int end) {
  OldComplain("GetBaseline");
  float a=0;
  for (int i = start; i < end; i++)
    a+=Fadc->Trace[pmt][gain][i];
  a/=(end-start);
  float s,b=0;
  int n=0;
  for (int i = start; i < end; i++) {
    s=Fadc->Trace[pmt][gain][i];
    if (s>a-4 && s<a+4) {
      b+=s;
      n++;
    }
  }
  if (n) a=b/n;
  return a;
}

/* --------------------------------------------------------- */
void TCalibStation::OldSignal(int loadCalib)
/* --------------------------------------------------------- */
{
  OldComplain("OldSignal");
  int nb, startbin = 0;
  unsigned int i, j;
  float peak = 0;
  //  float som[kIoSd::MAXSAMPLE];
  //  float sompmt[kIoSd::MAXSAMPLE][kIoSd::NPMT];
  unsigned int samples = Fadc->NSample;
  if (samples > kIoSd::MAXSAMPLE) samples = kIoSd::MAXSAMPLE;

  //calibrating traces...
  if (!loadCalib || !LoadCalib()) for (i = 0; i < kIoSd::NPMT; i++) OldCalibrate(i);

  /*** if station is not calibrated... ***/
  if (!Calib->NTubesOk) {
    for (i = 0; i < samples; i++) {
      fTrace[i] = 0;
      nb = 0;
      for (j = 0; j < kIoSd::NPMT; j++) {
        //        if (fPmt[j].fTrace[i] > 0.1) {  /* 0.1 VEM at least per bin */
        nb++;
        fTrace[i] += fPmt[j].fTrace[i];
        //        }
      }
      if (nb) fTrace[i] /= nb;
      if (peak < fTrace[i]) peak = fTrace[i];
    }
    /* old stuff... */
    if (Trigger.Name == "Unknown Trigger (?)") {
      if (fTrace[256] > kEc::OldThresh) Trigger.Name = "Threshold";
      else Trigger.Name = "Time Over Threshold";
    }
    if (Trigger.Name == "Threshold") startbin = 256 - kEc::StartThresh;  //120;
    for (i = startbin; i < samples - 2; i++)
      if (fTrace[i] >= kEc::SigThresh && fTrace[i + 1] >= kEc::SigThresh && fTrace[i + 2] >= kEc::SigThresh) {
        fStartBin = i;
        break;
      }
  }
  /*** if station is calibrated (at least one pmt...) ***/
  else {
    for (i = 0; i < samples; i++) {
      double th = kEc::SigThresh;
      int startedPmt = 0;
      //      som[i] = sompmt[i][0] = sompmt[i][1] = sompmt[i][2] = nb = 0;
      fTrace[i] = nb = 0;
      for (j = 0; j < kIoSd::NPMT; j++) {
        //        if (Calib->TubeOk[j] || fPmt[j].fTrace[i] > 0.1) {
        if (Calib->TubeOk[j]) {
          nb++;
          fTrace[i] += fPmt[j].fTrace[i];
          //          sompmt[i][j] += fPmt[j].fTrace[i];
        }
        if (fTrace[i] >= th && !fStartBin) {
          if (i < 256 - 120) th += kEc::SigThresh;   /*if necessary, increment for each pmt outside 120 bins window */
          else if (fPmt[j].fTrace[i] >= kEc::SigThresh)  startedPmt++;   /*StartBin = i; */
        }
      }                   //end of j
      //we want a real signal in ALL (nb) the working pmts
      if (!fStartBin && startedPmt == nb) fStartBin = i;
      if (nb) fTrace[i] /= nb;
      if (peak < fTrace[i]) peak = fTrace[i];
    }                       //end of i

    if (Trigger.Name == "Threshold") {
      if ((int) (256 - fStartBin) < kEc::StartThresh) startbin = fStartBin;
      else  startbin = 256 - kEc::StartThresh;
      for (i = startbin; i < samples - 2; i++) {  //we look for the Thres. start
        if (fTrace[i] >= kEc::SigThresh && fTrace[i + 1] >= kEc::SigThresh && fTrace[i + 2] >= kEc::SigThresh) {
          fStartBin = i;
          break;
        }
      }
    }
  }

  if (fStartBin > 20) fStartIntegration = fStartBin - 20; else fStartIntegration = 0;
  fStartIntegration = fStartBin;
  if (!fStartIntegration) fEndIntegration = samples; else fEndIntegration = fStartIntegration + kEc::IntegrationWindow;
  if (fEndIntegration >= (int) (samples)) {   /* case of bugged data */
    fEndIntegration = samples;
    fStartBin = 0;
    fStartIntegration = 0;
  }

  fPeakInVEM = peak;
  fSigInVEM = 0;
#if 0
  for (int i = fStartIntegration; i < fEndIntegration; i++)  fSigInVEM += som[i];
  fSigInVEM *= (fPmt[0].VemPeak() + fPmt[1].VemPeak() + fPmt[2].VemPeak());
  fSigInVEM /= (fPmt[0].VemCharge() + fPmt[1].VemCharge() +  fPmt[2].VemCharge());
#endif
  fPmt[0].fSigInVEM = fPmt[1].fSigInVEM = fPmt[2].fSigInVEM = 0;
  for (int i = fStartIntegration; i < fEndIntegration; i++) {
    for (unsigned int j = 0; j < kIoSd::NPMT; j++) {
      //      fSigInVEM += Calib->TubeOk[j]*sompmt[i][j]*fPmt[j].VemPeak()/fPmt[j].VemCharge();
      //      if (fPmt[j].fPeakInVEM < sompmt[i][j]) fPmt[j].fPeakInVEM=sompmt[i][j];
      //      fPmt[j].fSigInVEM += Calib->TubeOk[j]*sompmt[i][j]*fPmt[j].VemPeak()/fPmt[j].VemCharge();
      fSigInVEM += Calib->TubeOk[j]*fPmt[j].fTrace[i]*fPmt[j].VemPeak()/fPmt[j].VemCharge();
      if (fPmt[j].fPeakInVEM < fPmt[j].fTrace[i]) fPmt[j].fPeakInVEM = fPmt[j].fTrace[i];
      fPmt[j].fSigInVEM += Calib->TubeOk[j]*fPmt[j].fTrace[i]*fPmt[j].VemPeak()/fPmt[j].VemCharge();
    }
  }
  if (Calib->TubeOk[0]+Calib->TubeOk[1]+Calib->TubeOk[2]) fSigInVEM /= Calib->TubeOk[0]+Calib->TubeOk[1]+Calib->TubeOk[2];
  if (fPmt[0].fLowGainSat || fPmt[1].fLowGainSat || fPmt[2].fLowGainSat) { // low gain saturated
    // using GAP 2005-025 to recover saturated anodes
    // Addition: don't use PMT with base > 200... These are bad PMTs (ex: event 200502200461 tank 176 PMT 2)
    // Warning, error is large in this case, not sqrt(signal)!
    fSat = 1;
    for (unsigned int j = 0; j < kIoSd::NPMT; j++) {
      if (!fPmt[j].fLowGainSat) continue;
      // computing anode undershoot
      float anode_begin=GetBaseline(j,IoSdEvent::eLow,0,100);
      // Bad PMT
      if (anode_begin>200) {
        fPmt[j].fSigInVEM=0;
        Calib->TubeOk[j]=0; // modifying Calib block: this is evil. Should have an offline equivalent...
        continue;
      }
      float anode_end=GetBaseline(j,IoSdEvent::eLow,668,768);
      float newsig=0;
      //cerr << "AE-AB= " << anode_end-anode_begin << endl;
      if (anode_begin-anode_end>1) {
        fSat=16; // percent of error
        newsig = (1669 + 14755*(anode_begin-anode_end)) * fPmt[j].fDA / fPmt[j].VemCharge();
      } else {
        fSat = 10; // percent of error
        // computing anode undershoot
        float dynode_begin=GetBaseline(j,IoSdEvent::eHigh,0,100);
        float dynode_end=GetBaseline(j,IoSdEvent::eHigh,668,768);
        // computing dynode charge
        float qd=0;
        for (int i = fStartIntegration; i < fEndIntegration; i++) {
          float s=Fadc->Trace[j][IoSdEvent::eHigh][i];
          if (s-dynode_begin>1) qd+=s-dynode_begin;
        }
        //cerr << "DE-DB= " << dynode_end-dynode_begin << endl;
        //cerr << "Qd/kd= " << qd << " / " << fPmt[j].fHighGainSat << endl;
        newsig = (770*((dynode_begin - dynode_end) - 1.35e-4 *qd + 0.055 *fPmt[j].fHighGainSat) -222)* fPmt[j].fDA/fPmt[j].VemCharge();
      }
      //cerr << fPmt[j].fSigInVEM << " => " << newsig << " (" << fSat << ")" << endl;
      if (newsig>fPmt[j].fSigInVEM) fPmt[j].fSigInVEM=newsig;
    }
    //cerr << "Old SigInVem: " << fSigInVEM << ", New: ";
    fSigInVEM = (Calib->TubeOk[0]*fPmt[0].fSigInVEM + Calib->TubeOk[1]*fPmt[1].fSigInVEM
                 + Calib->TubeOk[2]*fPmt[2].fSigInVEM)/(Calib->TubeOk[0]+Calib->TubeOk[1]+Calib->TubeOk[2]);
    //cerr << fSigInVEM << endl;
  }
  fSeveralSignals = 0;
  //another signal before?
  FindOtherSignal(0, fStartBin - 2, fTrace);
  //another signal after?
  if (Trigger.Name == "Threshold") FindOtherSignal(fStartBin + 20, samples - 2, fTrace);
  else if (Trigger.Name == "Time Over Threshold") FindOtherSignal(fStartBin + 120, samples - 2, fTrace);
}

/* --------------------------------------------------------- */
void TCalibStation::PmtSignal(int n)
/* --------------------------------------------------------- */
{
  OldComplain("PmtSignal");
  n+=0;
#if 0
  int k;
  double sum = 0, peak = 0, binvalue = 0;

  if (fSigInVEM == -1)
    Signal();
  for (k = fStartIntegration; k < fEndIntegration; k++) {
    binvalue = fPmt[n].fTrace[k];
    if (binvalue > 0.1) {
      sum += binvalue;
      if (peak < binvalue)
        peak = binvalue;
    }
  }
  fPmt[n].fPeakInVEM = peak;
  fPmt[n].fSigInVEM = sum * fPmt[n].VemPeak() / fPmt[n].VemCharge();
#endif
}

/* --------------------------------------------------------- */
void TCalibStation::FindOtherSignal(int begin, int end, float *sum)
/* --------------------------------------------------------- */
{
  OldComplain("FindOtherSignal");
  int count = 0, i;

  for (i = begin; i < end; i++) {
    if (count != 10) {
      if (sum[i] < kEc::SigThresh)
        count++;
      else
        count = 0;
    }
    if (count == 10 && sum[i] > kEc::SigThresh
        && sum[i + 1] > kEc::SigThresh && sum[i + 2] > kEc::SigThresh) {
      fSeveralSignals = 1;
      break;
    }
  }
}

/*---------------------------------------------------------*/
bool TEcEvent::EnoughTanks()
/*---------------------------------------------------------*/
{
  OldComplain("EnoughTanks");
  if (fNCalibStations < 3)
    return false;
  else
    return true;
}

void TEcEvent::SaveCalib() {
  OldComplain("SaveCalib");
  for (unsigned int i = 0; i < Stations.size(); i++)
    if (Stations[i].Error == 0) {
      TCalibStation toCalib(Stations[i]);
      toCalib.OldSignal(0);
      for (unsigned int pm = 0; pm < kIoSd::NPMT; pm++)
        if (toCalib[pm].fVemCharge < 0 || toCalib[pm].fVemPeak < 0)
          return;
      std::ostringstream newName;
      newName << Stations[i].name() << " (Calib: ";
      newName << toCalib[0].VemCharge() << " " << toCalib[1].
      VemCharge() << " " << toCalib[2].VemCharge() << " ";
      newName << toCalib[0].VemPeak() << " " << toCalib[1].
      VemPeak() << " " << toCalib[2].VemPeak() << ")";
      Stations[i].Name = newName.str();
    }
}


