#include "Er.h"
#include "TReconstruct.h"
#include "ToolsLib.h"
#include "SdRecLpnhe.h"
#include "PBRec.h"

bool sortbytime (TCalibStation* x, TCalibStation* y) {
  return x->StartTime() < y->StartTime();
}

bool sortbysignal (TCalibStation* x, TCalibStation* y) {
  return x->fSigInVEM < y->fSigInVEM;
}

TErEvent::TErEvent(EventPos pos):TEsEvent(pos) {
  fRecError=-1;
  fHorPlot = NMU;
  fDoHorizontalRec = true;
  theLpnheRec().SetErEvent(this);
  thePBRec().fErEvent = this;
}

TErEvent::TErEvent(EventId id):TEsEvent(id) {
  fRecError=-1;
  fHorPlot = NMU;
  fDoHorizontalRec = true;
  theLpnheRec().SetErEvent(this);
  thePBRec().fErEvent = this;
}

TErEvent::TErEvent():TEsEvent() {
  fRecError=-1;
  fHorPlot = NMU;
  fDoHorizontalRec = true;
  theLpnheRec().SetErEvent(this);
  thePBRec().fErEvent = this;
}

TErEvent::~TErEvent() {}

TErEvent::TErEvent(const TEsEvent & sdevent):TEsEvent(sdevent) {
  fRecError=-1;
  fHorPlot = NMU;
  fDoHorizontalRec = true;
  theLpnheRec().SetErEvent(this);
  thePBRec().fErEvent = this;
}

/*---------------------------------------------------------*/
bool TErEvent::Reconstruct(TReconstruct * rec)
/*---------------------------------------------------------*/
{
  return rec->Reconstruct(*this);
}

/*---------------------------------------------------------*/
void TErEvent::SetHorPlot(HorPlot plot)
/*---------------------------------------------------------*/
{
  fHorPlot = plot;
}

/*---------------------------------------------------------*/
void TErEvent::SetDoHorizontalRec(bool rec)
/*---------------------------------------------------------*/
{
  fDoHorizontalRec = rec;
}

/*----------------------------------------------------------*/
void TErEvent::Print()
/*----------------------------------------------------------*/
{
  cout << "---------------- TErEvent --------------------" << endl;
  cout.setf(ios::fixed);
  cout << "Evt       = " << id() << endl;
  cout << "Algo      = " << trigger().algo() << endl;
  cout << "Number of tanks removed : " << fRandomStations.size() << endl;
  //if( fRandomIds.size() ) cout << " removed ids = " << fRandomIds << endl;
  if (fRecError) cout << fRecError << endl;
}

/*----------------------------------------------------------*/
void TErEvent::EventList()
/*----------------------------------------------------------*/
{
  static FILE *evtlist;
  char filename[100];
  //  sprintf(filename,"%6d_event.txt",id());
  sprintf(filename, "eventlist.txt");
  if (!evtlist) evtlist = fopen(filename, "w+");
  fprintf(evtlist, "%7d  ", (int) id());
  /*
    fprintf(evtlist,"%8.2f  ",fTheta);
    fprintf(evtlist,"%8.2f  ",fPhi);
    fprintf(evtlist,"%14.2f  ",fE);
    fprintf(evtlist,"%9.2f  ",fNorthing-kN0);
    fprintf(evtlist,"%9.2f  ",fEasting-kE0);
    if( fTheta > 70 ) fprintf(evtlist,"HORIZONTAL");
    else fprintf(evtlist,"          ");
  */
  fprintf(evtlist, "%3d  ", (int) fSelectedStations.size());
  for (unsigned int i = 0; i < fSelectedStations.size(); i++)
    fprintf(evtlist, "%4d ", (int) fSelectedStations[i]->id());
  fprintf(evtlist, "\n");
  fflush(evtlist);
}

/*-------------------------------------------------------------------------------------*/
void TErEvent::FitEllipse(double& xm, double& ym, double& a, double& b, double& tilt,
        string method_orientation, string method_area) const
/*-------------------------------------------------------------------------------------*/
{
  unsigned int npts = fSelectedStations.size();
  double * x = new double[npts];
  double * y = new double[npts];
  double * w = new double[npts];
  
  for( unsigned int i=0;i<npts;i++ )
    {
      //double distance=sqrt((x[i]-xm1)*(x[i]-xm1)+(y[i]-ym1)*(y[i]-ym1));
      x[i] = fSelectedStations[i]->fX;
      y[i] = fSelectedStations[i]->fY;
      w[i] = 1;//1/(exp(distance/distancemax));//sqrt(gvTCS[i][SIGOVERPEAK].Value());
    }
  //}
  fit_ellipse(npts,x,y,w,xm,ym,a,b,tilt,method_orientation,method_area);
  delete [] x;
  delete [] y;
  delete [] w;
}

/*-------------------------------------------------------------------------------------*/
double TErEvent::GetPolygonArea() const
/*-------------------------------------------------------------------------------------*/
{
  unsigned int npts = fSelectedStations.size();
  double * x = new double[npts];
  double * y = new double[npts];
  double * w = new double[npts];

  for( unsigned int i=0;i<npts;i++ )
    {
      //double distance=sqrt((x[i]-xm1)*(x[i]-xm1)+(y[i]-ym1)*(y[i]-ym1));
      x[i] = fSelectedStations[i]->fX;
      y[i] = fSelectedStations[i]->fY;
      w[i] = 1;//1/(exp(distance/distancemax));//sqrt(gvTCS[i][SIGOVERPEAK].Value());
    }
  vector<int> edges = get_edges(npts,x,y);
  double * xe = new double[edges.size()];
  double * ye = new double[edges.size()];
  for( unsigned int i=0;i<edges.size();i++ )
    {
      xe[i]=x[edges[i]];
      ye[i]=y[edges[i]];
    }
  double area = ComputePolygonArea((int)edges.size(),xe,ye);
  return area;
}


double TErEvent::Dist(const TCalibStation & st) const
{
  
  const double x = st.fX-fRec.fXCore; 
  const double y = st.fY-fRec.fYCore; 
  const double z = st.fZ-fRec.fZCore;
  return  sqrt(x*x+y*y+z*z-(x*fRec.fU+y*fRec.fV+z*fRec.W())*(x*fRec.fU+y*fRec.fV+z*fRec.W()));
}

double TErEvent::Dcore(const TCalibStation & st) const
{

  const double x = st.fX-fRec.fXCore; 
  const double y = st.fY-fRec.fYCore; 
  const double z = st.fZ-fRec.fZCore;
  return  sqrt(x*x+y*y+z*z);
}

double TErEvent::CosZeta(const TCalibStation & st) const
{
  const double x = st.fX-fRec.fXCore; 
  const double y = st.fY-fRec.fYCore;
  const double z = st.fZ-fRec.fZCore;
  if((x*x+y*y)>0 && (fRec.fU*fRec.fU+fRec.fV*fRec.fV)>0)
    return (x*fRec.fU+y*fRec.fV)/(sqrt(x*x+y*y+z*z)*sqrt(fRec.fU*fRec.fU+fRec.fV*fRec.fV));
  return  0;
}

double TErEvent::Dt(const TCalibStation & st) const
{
  if (st.Error) 
    return 0;
  const double x = st.fX-fRec.fXCore; 
  const double y = st.fY-fRec.fYCore; 
  const double z = st.fZ-fRec.fZCore;
  const double costh = sqrt(maxval(0, 1 - sqr(fRec.fU) - sqr(fRec.fV)));
  const double correctime = (costh * (st.fZ - fRec.fZCore)) / kCdasUtil::CSPEED;
  const double r2 = sqr(x) + sqr(y) - sqr(fRec.fU * x + fRec.fV * y); 
  const double dt = st.StartTime() * 1e+9 + correctime;
  const double curve = (fRec.fR == 0)? fRec.W()/20000.0 : 1.0/(2*fRec.fR);
  return (fRec.fT0 - (x * fRec.fU + y * fRec.fV - r2*curve) / kCdasUtil::CSPEED - dt);
}

