#ifndef TPBREC_H
#define TPBREC_H

#include "IoSd.h"
#include "Es.h"
#include "TReconstruct.h"

#define NSTATMAX 2000
#define MXID_EASTING_ORIGIN 439005
#define MXID_EASTING_STEP 750
#define MXID_NORTHING_ORIGIN 6064244
#define MXID_NORTHING_STEP 649.5

#define HORIZ_ENER 10
#define HORIZ_EXPONENT 1.05
#define DIST_MIN_LDF 300.


///Pierre Billoir's reconstruction
class TPBRec: public TReconstruct {

public:

  static TPBRec * Instance();
  bool _Reconstruct()///Reconstruct function inherited from TReconstruct
  {
    bool res = Process();
    fErEvent->fRec = *this;
    return res;
  }

  void OpenOutputFiles();
  void CloseOutputFiles();
  void InitializeRec();


  int MonitDay, TimeInDay;

  FILE *list_file, *res1, *res2, *res3, *aux1, *aux2, *aux3, *aux4, *diag, *disp, *user_file;
  TFile *truthroot;
  char truthroot_name[100];
  int truth_root;

  // easting and northing of Los Leones
  double EASTING_ORIGIN;
  double NORTHING_ORIGIN;
  double ALTITUDE_ORIGIN;

  // user parameters
  char user_file_name[100];
  int n_user_file;
  double Something;

  // running conditions (defined or read in ReadRequest)
  int NevMax, Simulation, ReadTruth;
  int Input_sel_evts, Write_sel_evts, Process_3st, Process_gen, Reconstruct, FitHoriz;
  int List_event, *list_event;
  int PrintLevel, Write1_result, Write2_result, Write3_result,
  Auxil1_output, Auxil2_output, Auxil3_output, Auxil4_output,
  Diagno_output, Display_output, ROOT_output;
  int RemoveInfillStation, RemoveDoublet, RemoveEA, SelectContained, CorrectAsym;

  double MAGNETIC_NORTH;
  double ISOLATION_DISTANCE, ISOLATION_DISTANCE2, NMIN_AROUND_HIGHEST_SIGNAL;
  double SELECT_TOLRES, SELECT_TOLRESM, SELECT_CURV0, SELECT_DISTMAX_PROJ_ST;
  double ANGREC_TOLRESC, TIME_ERR, TIME_ERR0, TIME_ERR_FACTD2, TIME_ERR_CURV;
  int TIME_START, TIME_START_SELECT, TIME_START_PLAN, TIME_START_CURV, TIME_START_GLOB;
  double LDF_DISTREF, LDF_SINTHMAX, LDF_CHI2MAX, LDF_DCHI2MAX, LDF_DISTMAX_BARY_CORE,
  LDF_DISTMAX_SILENT, LDF_SIGTHRESH, BARY_POWER;
  int NSTAT_IN_BARY;
  double GLOB_CHI2MAX, GLOB_DCHI2MAX;
  double HORIZ_MUON, HORIZ_SIGTHRESH;
  double ANA3_RATMIN, ANA3_AREAMAX, ANA3_AREAMIN, ANA3_DISTMAX,
  ANA3_CHI2_LDF_MAX, ANA3_CHI2_LDF_SILENT_MAX, ANA3_DISTMAX_BARY_CORE;

  // switches
  int SilentLDF, FitLDFSlope;

  // event features
  int NumEvent, TimeEvent, UTCTimeEvent, GlobalTriggerType, T4Flag, AnomEvent;
  int true_ievt, true_prim;
  double true_energy, true_theta, true_phi, true_xmax, true_north, true_east;
  int nActiveStation, nSilentStation, nStationInFit, nPositionInFit, nWindow, RejectCode;
  int nCarmenMiranda, nDiaNoche, nMoulinRouge, nDoubletInFit;

  // reconstructed quantities
  TCalibStation *stat[NSTATMAX];
  double X[NSTATMAX], Y[NSTATMAX], Z[NSTATMAX], T[NSTATMAX], TCorr[NSTATMAX];
  double Signal[NSTATMAX], Peak[NSTATMAX];
  double XSilent[NSTATMAX], YSilent[NSTATMAX], ZSilent[NSTATMAX];
  int stId[NSTATMAX],selId[NSTATMAX],mxId[NSTATMAX],mxIdSilent[NSTATMAX];
  double XBarycenter, YBarycenter, ZBarycenter, TBarycenter;
  int Anom[NSTATMAX], Satur1[NSTATMAX], Satur2[NSTATMAX],  Satur3[NSTATMAX],
  LocalTriggerType[NSTATMAX], InFit[NSTATMAX], SilentInFit[NSTATMAX];
  double resTPlan[NSTATMAX],resTCurv[NSTATMAX],distBary[NSTATMAX],
  distCore[NSTATMAX],azimCore[NSTATMAX],sigFit[NSTATMAX],
  Tshift[NSTATMAX],Tfrac10[NSTATMAX],Tfrac50[NSTATMAX],Tfrac90[NSTATMAX],Shape[NSTATMAX];
  int nMuPeak[NSTATMAX];
  double AsymPMT[NSTATMAX];
  int CleanStart[NSTATMAX],CleanStop[NSTATMAX];
  int CleanFilter(TCalibStation *st, int& start, int& stop);
  int CleanSignal(TCalibStation *st, int start, int stop, double& signal, double& peak);
  double Tclean10[NSTATMAX],Tclean50[NSTATMAX],Tclean90[NSTATMAX];

  double covPlan[6], covCurv[10], covCurrent[10];
  double uCurrent,vCurrent;
  double t0Plan,uPlan,vPlan,residPlan,chi2Plan;
  double t0Curv,uCurv,vCurv,residCurv,chi2Curv;
  double u3st,v3st;
  int ContainedOK, PlanFitOK, CurvFitOK, LDFFitOK, Fit3stOK, GlobFitOK, GeomFlag, LDFFlag;
  double Theta, dTheta, Phi, dPhi, Curv, dCurv;
  double dXCore, dYCore, dS1000;
  //double T0, XCore, YCore, S1000, Energy, dEnergy, betaLDF, gammaLDF, RCurv;
  double dbetaLDF, chi2LDF, chi2LDFwithSilent, chi2Glob, chi2GlobwithSilent;
  double CosxSideral, CosySideral, CoszSideral, RightAscension, Declination, GalacticL, GalacticB;
  int nNeighboursOfMax;
  double AvgRiseTime, AsymRiseTime, AvgFallTime, AsymFallTime, AvgShape, AsymShape;

  // tabulation for an horizontal event
  char HorizModel[20];
  int HorizFit;
  double dx_grid,dy_grid,energ_hor,phi_hor,cosphi_hor,sinphi_hor,
  nmu_hor[51][51],signal_hor[51][51];
  double Horiz_alpha_tab[10][5][7][5], Horiz_beta_tab[10][5][7][5],
  Horiz_a[5][7],  Horiz_b[5][7];

  int Process();
  void ConvertS1000ToEnergy(double costh,double s1000,double ds1000);
  int AnomalyEvent();
  int ActiveStations();
  int ContainedEvent();
  void SilentStations();
  void WriteDisplay();
  void PrintStations();
  int Select();
  int TryWithAllStations();
  int TryRemoveOneStation(int *ord) {
    _indexes.clear();
    _indexes.push_back(nActiveStation);
    return _TryRemove1Station(ord);
  }
  int TryRemoveTwoStations(int *ord) {
    _indexes.clear(); _indexes.resize(2);
    _indexes[1] = nActiveStation;
    return _TryRemove2Stations(ord);
  }
  int TryRemoveThreeStations(int *ord) {
    _indexes.clear(); _indexes.resize(3);
    _indexes[2] = nActiveStation;
    return _TryRemove3Stations(ord);
  }
  int TryRemoveFourStations(int *ord) {
    _indexes.clear(); _indexes.resize(4);
    _indexes[3] = nActiveStation;
    return _TryRemove4Stations(ord);
  }
  int TryRemoveFiveStations(int *ord) {
    _indexes.clear(); _indexes.resize(5);
    _indexes[4] = nActiveStation;
    return _TryRemove5Stations(ord);
  }
  int TryRemoveSixStations(int *ord) {
    _indexes.clear(); _indexes.resize(6);
    _indexes[5] = nActiveStation;
    return _TryRemove6Stations(ord);
  }
  int TryRemoveSevenStations(int *ord) {
    _indexes.clear(); _indexes.resize(7);
    _indexes[6] = nActiveStation;
    return _TryRemove7Stations(ord);
  }
  int TryRemoveEightStations(int *ord) {
    _indexes.clear(); _indexes.resize(8);
    _indexes[7] = nActiveStation;
    return _TryRemove8Stations(ord);
  }

  int Anal3st(int is1, int is2, int is3);
  int AnalGen();
  int AngularFit();
  void ComputeBarycenter();
  void CorrectTimeAltitude(double u,double v);
  void CorrectTimeAltitudeCurvature(double u,double v, double xc, double yc);
  int compact_config(double u, double v);
  int Recover3st();
  void SetTime(int time_start);
  int LDFFit();
  void PrintLDFfail(const char *stage, int errflag, double chi2, int ndof, double dist);
  void SaveLDFParam(double xc, double dxc, double yc, double dyc, double s1000, double ds1000,
                    double *distc, double beta, double dbeta, double gamma);
  int GlobalFit();
  int LDFhorFit();
  int ReadTableHoriz(unsigned int, char *model);
  int FindPeak();
  int MuonCount();
  int CleanTrace();
  void RiseFallTime();

  void FitPlan (int tcorr, int errt, double& t0, double& u, double& v,
                double *res_t, double& res, double& chi2, double *cov, int& errflag);
  void FitCurved (int tcorr, int errt, double& t0, double& u, double& v, double& curv,
                  double *res_t, double& res, double& chi2, double *cov, int& errflag);
  void FitLDFMinuit
  (double& xc,double& dxc,double& yc,double& dyc,double& s1000,double& ds1000,
   double *distc, double& chi2, double& beta, double& dbeta, double& gamma, int& errflag);

  void FitGlobMinuit
  (double& t0, double& u, double& v, double& curv, double& xc, double& yc,
   double& s1000, double *distc, double *cov, double& chi2, int& errflag);
  void FitGlobFixCurvMinuit
  (double& t0, double& u, double& v, double& xc, double& yc,
   double& s1000, double *distc, double *cov, double& chi2, int& errflag);
  void FitLDFhorMinuit
  (double& xc,double& dxc,double& yc,double& dyc,double& fact,double& dfact,
   double& chi2, int& errflag);

  double ldf_hor(double x,double y);
  double NKGfunc(double r, double age);
  double AsymFact(double dx, double dy);
  void SideralCoord();
  int ReadHorizTables();
  int accept_station(IoSdStation& st, int window_condition);
  int select_input();



private:

  TPBRec();                    ///< TPBRec constructor
  ~TPBRec();

  static TPBRec * _instance;   ///< instance of the singleton (initialized only once)
  vector <int> _indexes;

  int _TryRemove1Station(int *ord);
  int _TryRemove2Stations(int *ord);
  int _TryRemove3Stations(int *ord);
  int _TryRemove4Stations(int *ord);
  int _TryRemove5Stations(int *ord);
  int _TryRemove6Stations(int *ord);
  int _TryRemove7Stations(int *ord);
  int _TryRemove8Stations(int *ord);
};


TPBRec & thePBRec();




#endif
