#include <math.h>

#include "TRandom.h"
#include "TH1F.h"
#include "TH2F.h"

#include "TCdFReconstruct.h"
#include "TCommonStation.h"
#include "TCdasUtil.h"
#include "STClibrary.h"

#define LDF_DISTREF 1000.

using namespace TSdParams;

TvCommonStation gvTCS;

// initialize static member
TDataToFit * TCdFReconstruct::gDataBackUp = NULL;

/*TCdFReconstruct *recCdF(){
  if (TCdFReconstruct::instance == 0) TCdFReconstruct::instance = new TCdFReconstruct();
  return TCdFReconstruct::instance;
  }*/

/*-----------------------------------------------*/
TCdFReconstruct::TCdFReconstruct()
/*-----------------------------------------------*/
{
  gDataBackUp = new TDataToFit();
  InitReconstruct();
  InitFitting();
}

/*-----------------------------*/
TCdFReconstruct::~TCdFReconstruct()
/*-----------------------------*/
{
  if( gDataBackUp != 0x0 )
    delete gDataBackUp;
  gDataBackUp = 0x0;
}

/*---------------------------------*/
void TCdFReconstruct::InitReconstruct()
/*---------------------------------*/
{
  fNoReconstruction = 2;
  fNFit             = 0;
  fNTanks           = 0;
  fPrimary          = 0; // proton
  fVerbosity        = -1;
  fLDFType          = "LOG"; // default
  fUseEstimate      = "barycenter";
  fHorizontal       = false;
  fBetaFree         = false;
  fCorrectAltitude  = false;
  fFitStep.clear();
}

/*---------------------------------*/
void TCdFReconstruct::InitTCS()
/*---------------------------------*/
{
  gvTCS.Zero();
  gvTCS.resize(fErEvent->fSelectedStations.size()); // resize the pure vector part of gvTCS
  gvTCS.fNParticipatingTanks = fErEvent->fSelectedStations.size();// first time
  for( unsigned int i=0;i<fErEvent->fSelectedStations.size();i++ ) {
    TCalibStation& thestat = *fErEvent->fSelectedStations[i];
    int id = thestat.id();
    gvTCS.fLsMapEventIndex[id] = i;
    gvTCS.fLsMapActualEventIndex[id] = i;
    gvTCS[i][XPOSITION]        = thestat.fX;
    gvTCS[i][YPOSITION]        = thestat.fY;
    gvTCS[i][LSTIME]           = thestat.StartTime();
    gvTCS[i][LSID]             = id;
    gvTCS[i][EVENTINDEX]       = (int)i;
    gvTCS[i][ACTUALEVENTINDEX] = (int)i;
    gvTCS[i][SIGNAL]           = thestat.SigInVEM();
    gvTCS[i][SIGOVERPEAK]      = thestat.SigOverPeak();
    gvTCS[i][T10] = 1e-9*(thestat.Gps->NanoSecond-kEc::FADCDepth*kEc::TimeSlot+thestat.T10());
    gvTCS[i][T50] = 1e-9*(thestat.Gps->NanoSecond-kEc::FADCDepth*kEc::TimeSlot+thestat.T50());
    gvTCS[i][T70] = 1e-9*(thestat.Gps->NanoSecond-kEc::FADCDepth*kEc::TimeSlot+thestat.T70());
    gvTCS[i][T90] = 1e-9*(thestat.Gps->NanoSecond-kEc::FADCDepth*kEc::TimeSlot+thestat.T90());
  }
  // save data
  //  CopyInstance(gDataBackUp,dynamic_cast<TDataToFit*>(this));
  gvTCS.BackUp();
  if( fVerbosity > 1 )
    gvTCS.Print();
}

/*---------------------------------------------------------*/
void TCdFReconstruct::SetDataToFit()
/*---------------------------------------------------------*/
{
  *(TDataToFit*)this = GetDataToFit();
}

/*---------------------------------------------------------*/
TDataToFit TCdFReconstruct::GetDataToFit()
/*---------------------------------------------------------*/
{
  vector<int> ListIds, ListIdsSilent;
  vector<double> ListSignals, ListSigOverPeak, ListTimes;
  vector<vector<double> > ListCoords;
  vector<vector<double> > ListCoordsSilent;
  vector<bool> ListSaturated;
  unsigned int NTanks = gvTCS.size();

  if( NTanks < 3 ) {
    TDataToFit toto;
    return toto;
  }

  unsigned int NSilTanks = fErEvent->fSilentStations.size();

  ListIds.resize(NTanks);
  ListIdsSilent.resize(NSilTanks);
  ListCoords.resize(NTanks);
  ListCoordsSilent.resize(NSilTanks);
  for( unsigned int i=0;i<NTanks;i++ )
    ListCoords[i].resize(3);
  for( unsigned int i=0;i<NSilTanks;i++ )
    ListCoordsSilent[i].resize(3);
  ListSignals.resize(NTanks);
  ListSigOverPeak.resize(NTanks);
  ListTimes.resize(NTanks);
  ListSaturated.resize(NTanks);
  vector<double> xyz(3);
  for (unsigned int i=0;i<NTanks;i++) {
    TCalibStation& thestat = *fErEvent->fSelectedStations[gvTCS.fLsMapEventIndex[gvTCS[i][LSID].Int()]];
    ListIds[i]         = thestat.Id;
    ListTimes[i]       = thestat.StartTime();
    xyz[0]             = thestat.fX;
    xyz[1]             = thestat.fY;
    xyz[2]             = thestat.fZ;
    ListCoords[i]      = xyz;
    ListSignals[i]     = thestat.SigInVEM();
    ListSigOverPeak[i] = thestat.SigOverPeak();
    ListSaturated[i]   = thestat.fSat;
  }
  fListTimesSave = ListTimes;
  for (unsigned int i=0;i<NSilTanks;i++) {
    TCalibStation& thestat = *fErEvent->fSilentStations[i];
    ListIdsSilent[i] = thestat.Id;
    xyz[0] = thestat.fX;
    xyz[1] = thestat.fY;
    xyz[2] = thestat.fZ;
    ListCoordsSilent[i] = xyz;
  }
  TDataToFit tmp(ListIds,ListTimes,ListCoords,ListCoordsSilent,ListSignals);
  tmp.fNTanks = NTanks;
  tmp.ComputeConstTimesErrors(35e-9);
  tmp.SetSaturated(ListSaturated);
  tmp.fListIdsSilent = ListIdsSilent;
  return tmp;
}

/*----------------------------------------------------------------------------------------------*/
bool TCdFReconstruct::_Reconstruct()
/*----------------------------------------------------------------------------------------------*/
{
  bool status;
  InitTCS();
  SetDataToFit();
  if( fLDFType == "LOG" )
    SetPrimary(0); // assume proton in LOG-LOG reconstruction
  IterativeReconstruction();
  TShowerParams::fT0 = 1e9*fT0s;
  if( fNoReconstruction == 0 && fTheta > 70 && fErEvent->fDoHorizontalRec == true ) {
    cout << "\n\tHorizontal reconstruction" << endl;
    cout << "\t=========================" << endl;
    cout << "SKIP !!!!" << endl;
  fGPSTime = fGPSSecond+fT0s;
    return false;
    /*
      fBeta = 0;
      fGamma = 0;
      HorizontalReconstruction();
      fHorizontal = true;
    */
  }
  fGPSTime = fGPSSecond+fT0s;
//  int leap = 13;
//  gps2utcdate(fGPSSecond,&fYear,&fMonth,&fDay,&fHour,leap);
	// dans cette version du soft de STCoordinates sans LeapGPS.h,.cc ne PAS passer par gps2utcdata
	unsigned int utcsec = kCdasUtil::UTCsecond(fGPSSecond);
	unix2date(utcsec+fT0s,&fYear,&fMonth,&fDay,&fHour);
  fElevation =  90. - fTheta;
  fAzimuth   = 270. - fPhi;
  Double_t jd;
  date2jd(fYear,fMonth,fDay,fHour,&jd);
  xy2ne(fXCore,fYCore,fdXCore,fdYCore,&fNorthing,&fEasting,&fdNorthing,&fdEasting);
  UTMtoLL(kSTC::WGS84,fNorthing,fEasting,kSTC::AugerUTMZone,fLatitude,fLongitude);
  azel2gal(fLatitude,fLongitude,fAzimuth,fElevation,jd,&fL,&fB,kSTC::RefEquinox);
  gal2radec(fL,fB,&fra,&fdec,kSTC::RefEquinox);
  FillCovarianceMatrix();
  if( fNoReconstruction == 0 )
    status = true;
  else
    status = false;
  return status;
}


/*---------------------------------------------------*/
void TCdFReconstruct::CorrectTimeAltitude()
/*---------------------------------------------------*/
{
  double cth = cos(fTheta*kCdasUtil::DEG2RAD);
  fTimeCorrection.resize(fNTanks);
  for( unsigned int i=0;i<fNTanks;i++ ) {
    fTimeCorrection[i] = cth*(fListCoords[i][2]-fZCore)/kCdasUtil::CSPEEDSI; // in ns
  }
}

/*---------------------------------------------------*/
void TCdFReconstruct::CorrectTimeAltitudeCurvature()
/*---------------------------------------------------*/
{
  if( fsqfU2fV2 >= 1 ) {
    cout << "TCdFReconstruct::CorrectTimeAltitudeCurvature BAD GEOMETRY !!!!" << endl;
    return;
  }
  double costh = maxval(0,fsqfU2fV2);
  // rough estimation of curvature 1/R
  for(unsigned int i=0; i<fNTanks; i++) {
    double dx = fListCoords[i][0] - fXCore;
    double dy = fListCoords[i][1] - fYCore;
    double dist2 = sqr(dx)+sqr(dy)-sqr(fU*dx + fV*dy);
    fTimeCorrection[i] = (costh*(fListCoords[i][2]-fZCore) - dist2/(2.*fR)) / kCdasUtil::CSPEEDSI;
  }
}

/*-------------------------------------------------------------------------------*/
double TCdFReconstruct::GetS1000Estimate()
/*-------------------------------------------------------------------------------*/
{
  double s1000 = 10;
  double distmin = 99999;
  double dist_refstat = LDF_DISTREF;
  double sig_refstat = fListSignals[0];
  for( unsigned int i=0;i<fNTanks;i++ ) {
    double dx = fListCoords[i][0]-fXCore;
    double dy = fListCoords[i][1]-fYCore;
    double dist = compute_distUV(fU,fV,dx,dy,0);
    if( fabs(dist-LDF_DISTREF) < distmin ) {
      distmin = fabs(dist-LDF_DISTREF);
      sig_refstat = fListSignals[i];
      dist_refstat = dist;
    }
  }
  double costh = sqrt(1-fU*fU-fV*fV);
  double beta_start = 3 + .7*atan(6*(costh-.65));
  s1000 = sig_refstat*pow(dist_refstat/LDF_DISTREF,beta_start);
  return s1000;
}

/*-------------------------------------------------------------------------------*/
double TCdFReconstruct::GetEnergyEstimate()
/*-------------------------------------------------------------------------------*/
{
  double costh = sqrt(1-fU*fU - fV*fV);
  string ldf = (fLDFType == "SMSP" ) ? "LOG" : fLDFType ;
  return compute_energy( ldf, maxval(0,GetS1000Estimate()), acos(costh),fPrimary);
}

/*-------------------------------------------------------------------------------*/
void TCdFReconstruct::IterativeReconstruction()
/*-------------------------------------------------------------------------------*/
{
  static int first = 1;
  if( (fLDFType == "LOG" || fLDFType == "2EXPO") && first && fVerbosity > -1 ) {
    cout << "In this case of reconstruction, the algorithms have to make" << endl;
    cout << "an asumption on the nature of the primary." << endl;
    cout << "For the moment you can choose proton (0, the default) or iron (1) for LOG LDF Type" << endl;
    cout << "Or from proton (0, the default), iron (1) or photon (2) for 2EXPO LDF Type" << endl;
    cout << "with the function TCdFReconstruct::SetPrimary(int primary)." << endl;
    first = 0;
  }

  if( fNoReconstruction == 1 ) {
    cout << "No reconstruction possible in this event. Returning..." << endl;
    return;
  }
  fSRef = 10;
  fE = 10;
  SetMinimizationMethod("MINOS");
  //  SetMinimizationMethod("MIGRAD");

  compute_uv(fListCoords,fListTimes,&fT0s,&fU,&fV);
  compute_barycentre(fListCoords,fListSignals,fListTimes,&fXCore,&fYCore,&fZCore,&fT0s);
  if( fUseEstimate == "Es" ) {
    fXCore = fEs.fXCore;
    fYCore = fEs.fYCore;
    fU     = fEs.fU;
    fV     = fEs.fV;
  }

  uv2thetaphi(fU,fV,fTheta,fPhi);
  fTheta *= kCdasUtil::RAD2DEG;
  fPhi *= kCdasUtil::RAD2DEG;

  CorrectTimeAltitude();

  if( fVerbosity > 0 ) {
    cout << "initall = " << fXCore << " " << fYCore << " " << fU << " " << fV << " " << fTheta << " " << fPhi << endl;
  }
  int param[11] = {kFitX0,kFitY0,kFitZ0,kFitT0,kFitU,kFitV,kFitC,kFitS,kFitE,kFitBeta,kFitXMAX};
  double step[11] = {10,10,10,1e-4,1e-4,1e-4,1e-5,1,0.1,0.01,10};
  double start[11] = {fXCore,fYCore,fZCore,fT0s,fU,fV,1e-4,fSRef,fE,-3.,600.};

  if( fLDFType == "HP" || fLDFType == "LOG" || fLDFType == "2EXPO" || fLDFType == "SMSP" ) {
    SetFreeParameters(11,param,start,step);
  } else {
    cout << "LDF function (" << fLDFType << ") not recognized." << endl;
    cout << "Returning." << endl;
    fNoReconstruction = 1;
    return;
  }

  // --------------- first estimation of u,v -----------------------
  gvTCS.Sort("gt",SIGNAL);
  if( fVerbosity > 0 ) {
    for( unsigned int i=0;i<fNTanks;i++ )
      cout << gvTCS[i][LSID].Int() << " "
      << fListIds[gvTCS.fLsMapActualEventIndex[gvTCS[i][LSID].Int()]] << " "
      << fListSignals[gvTCS.fLsMapActualEventIndex[gvTCS[i][LSID].Int()]] << endl;
  }
  fCorrectAltitude = false;
  if( !Step0() )
    return; // barycenter and direction with plane front
/*  fCorrectAltitude = true;
  if( !Step0() )
    return;

  if( fVerbosity > 0 ) {
    cout << "altitude correction (plane front) : " << endl;
    cout << (TCdFShowerParams)*this << endl;
    for( unsigned int i=0;i<fNTanks;i++ )
      cout << fListIds[i] << " " << fListTimesResiduals[i] << endl;
  }
*/  /*
    CorrectTimeAltitudeCurvature();
    if( !Step0(1) )
    return; // barycenter and direction with plane front
    if( fVerbosity > 0 )
    {
    cout << "curvature and altitude correction : " << endl;
    cout << (TCdFShowerParams)*this << endl;
    for( unsigned int i=0;i<fNTanks;i++ ) cout << fListIds[i] << " " << fListTimesResiduals[i] << endl;
    }
  */

  // --------------- at this step, u, v should be quite good ---------------

  // --------------- recompute core position -------------------------------
  if( !Step1(step) )
    return; // compute S and core position given u,v,zcore
  if( fVerbosity > 0 ) {
    cout << "after step 1 : " << endl;
    cout << (TCdFShowerParams)*this << endl;
  }
  //  cout << (TShowerParams)*this << endl;
  if( !Step2(step) )
    return; // compute u,v,C,T0 given core position and S
  if( fVerbosity > 0 ) {
    cout << "after step 2 : " << endl;
    cout << (TCdFShowerParams)*this << endl;
  }

  if( fNTanks > 3 ) {
    Step3(step);               // compute u, v, curvature, T0 with variable R
    if( fVerbosity > 0 ) {
      cout << "after step 3 : " << endl;
      cout << (TCdFShowerParams)*this << endl;
    }
    Step4(step);               // compute S, core position and T0 given new u, v and curvature
    if( fVerbosity > 0 ) {
      cout << "after step 4 : " << endl;
      cout << (TCdFShowerParams)*this << endl;
    }
    Step5(step);               // compute other curvature radius
    if( fVerbosity > 0 ) {
      cout << "after step 5 : " << endl;
      cout << (TCdFShowerParams)*this << endl;
      }
   

  }

  // then  compute energy ===>> Already done at each step (TFitting::ComputeResult)
  /*   -> that is why it is commented out
    if( fLDFType != "SMSP" && fLDFType != "2EXPO" ) {
      fE = compute_energy(fLDFType,maxval(0,fS1000),fTheta*kCdasUtil::DEG2RAD,fPrimary,fdS1000,&fdE);
    }
  */
  RecordFitResults();
  if( fHasConverged )
    fNoReconstruction = 0;
  else
    fNoReconstruction = 1;
}

/*-------------------------------------------------------------------------------*/
bool TCdFReconstruct::Step0()
/*-------------------------------------------------------------------------------*/
{
  if( fCorrectAltitude ) {
    fListTimes = fListTimesSave;
    for( unsigned int i=0;i<fNTanks;i++ )
      fListTimes[i] += fTimeCorrection[i];
  }
  if( fVerbosity > 0 ) {
    cout << "\t* step 0 : Compute barycentre and direction (plane front)" << endl;
    cout << "\t\t===>" << endl;
  }

  // compute core position
  if( fVerbosity > 0 ) {
    cout << "init bary = " << fXCore << " " << fYCore << " " << fZCore << " " << 1e9*fT0s << endl;
  }

  SetFixParameter(kFitX0,fXCore);
  SetFixParameter(kFitY0,fYCore);
  SetFixParameter(kFitZ0,fZCore);
  SetFixParameter(kFitC,0.);
  SetFixParameter(kFitS,fSRef);
  SetFixParameter(kFitE,fE);
  SetFixParameter(kFitBeta,fBeta);
  SetFixParameter(kFitXMAX,fXmax);

  SetFCN(FCNPlane);
  Minimize();
  ComputeResults();

  if( fConvergenceStatus < 3 ) {
    fNoReconstruction = 1;
    return fHasConverged;
  }
  if( fHasConverged == false ) {
    fNoReconstruction = 1;
    return fHasConverged;
  }
  RecordFitSteps(); // 0 :store results of this fit step
  if( fVerbosity > 0 )
    cout << *(TFitting*)this << endl;

  if( fCorrectAltitude )
    fListTimes = fListTimesSave;

  return fHasConverged;
}

#define DCMAX 500.
double DCMAXTILTED;

/*-------------------------------------------------------------------------------*/
bool TCdFReconstruct::Step1(double*step)
/*-------------------------------------------------------------------------------*/
{
  if( fCorrectAltitude ) {
    fListTimes = fListTimesSave;
    for( unsigned int i=0;i<fNTanks;i++ )
      fListTimes[i] += fTimeCorrection[i];
  }
  if( fVerbosity > 0 ) {
    cout << "\t* step 1 : find S, XCore, YCore, T0 given u, v and ZCore (plane front)" << endl;
    cout << "\t\t===>" << endl;
  }
  // at this point, U, V are known (plane front)
  // compute S1000, Xcore, Ycore, T0 given estimated u, v, Zcore for a plane front
  SetFixParameter(kFitZ0,fZCore);
  SetFixParameter(kFitU,fU);
  SetFixParameter(kFitV,fV);
  SetFixParameter(kFitC,0.);
  SetFixParameter(kFitXMAX,fXmax);
  SetFreeParameter(kFitT0,fT0s,step[kFitT0]);

  if ( fLDFType != "2EXPO" ) {
    SetFreeParameter(kFitX0,fXCore,step[kFitX0]);
    SetFreeParameter(kFitY0,fYCore,step[kFitY0]);
    
    // start point for beta
    GetBetaGammaFromTheta(0,fTheta*kCdasUtil::DEG2RAD,fBeta,fGamma);
    if( fNTanks > 4 && fBetaFree )
      SetFreeParameter(kFitBeta,fBeta,step[kFitBeta]);
    else
      SetFixParameter(kFitBeta,fBeta);
  }
  else {
    DCMAXTILTED=DCMAX/cos(fTheta*kCdasUtil::DEG2RAD);
    SetFreeParameter(kFitX0,fXCore,step[kFitX0],(fXCore-DCMAXTILTED),(fXCore+DCMAXTILTED));
    SetFreeParameter(kFitY0,fYCore,step[kFitY0],(fYCore-DCMAXTILTED),(fYCore+DCMAXTILTED));
  }
  
  fSRef = GetS1000Estimate();
  fE     = GetEnergyEstimate(); //e.g. for smsp or 2expo;

  if( fVerbosity > 0 ) {
    cout << "estimate s1000 = " << fSRef
    << " , E estimate " << fE
    << " and beta estimate = " << fBeta << endl;
  }

  if ( fLDFType != "SMSP" && fLDFType != "2EXPO" ) {
    SetFixParameter(kFitE,fE);
    SetFreeParameter(kFitS,fSRef,step[kFitS]);
  } else {
    SetFixParameter(kFitS,fSRef);
    SetFreeParameter(kFitE,fE,step[kFitE]);
  }

  if( fLDFType == "HP" ) SetFCN(FCNGlobalHP);
  else if( fLDFType == "LOG" ) SetFCN(FCNGlobalLOG);
  else if( fLDFType == "SMSP" ) {
     //SetMin(kFitE,gpEstim->MinMap(SMSP_TYPE_ENERGY));
     //SetMax(kFitE,gpEstim->MaxMap(SMSP_TYPE_ENERGY));

    SetFCN(FCNSMSP);
  } else if( fLDFType == "2EXPO" ) SetFCN(FCNGlobal2EXPO);

  Minimize();
  ComputeResults();
  if( fHasConverged == false ) {
    fNoReconstruction = 1;
    return fHasConverged;
  }
  RecordFitSteps(); // 1 :store results of this fit step
  if( fVerbosity > 0 )
    cout << *(TFitting*)this << endl;

  if( fCorrectAltitude )
    fListTimes = fListTimesSave;

  return fHasConverged;
}

/*-------------------------------------------------------------------------------*/
bool TCdFReconstruct::Step2(double*step)
/*-------------------------------------------------------------------------------*/
{
  if( fCorrectAltitude ) {
    fListTimes = fListTimesSave;
    for( unsigned int i=0;i<fNTanks;i++ )
      fListTimes[i] += fTimeCorrection[i];
  }
  if( fVerbosity > 0 ) {
    cout << "\t* step 2 : find u, v, curvature, T0 given S and core position" << endl;
    cout << "\t\t===>" << endl;
  }
  // recompute u, v and T0 with a curvature using the new core position
  // make U, V, C, T0 free again

  SetFixParameter(kFitX0,fXCore);
  SetFixParameter(kFitY0,fYCore);
  SetFixParameter(kFitZ0,fZCore);
  SetFixParameter(kFitS,fSRef);
  SetFixParameter(kFitE,fE);
  SetFixParameter(kFitBeta,fBeta);
  SetFixParameter(kFitXMAX,fXmax);
  // keep same steps
  SetFreeParameter(kFitT0,fT0s,step[kFitT0]);
  SetFreeParameter(kFitU,fU,step[kFitU]);
  SetFreeParameter(kFitV,fV,step[kFitV]);
  if( fNTanks < 4 ) {
    SetFixParameter(kFitC,0.); // not enough tanks to compute curvature
    if( fVerbosity > -1 )
      cout << "No curvature will be computed, not enough tanks for that." << endl;
  } else
    SetFreeParameter(kFitC,1e-4,step[kFitC],0.,1.);
  SetFCN(FCNCurvature);
  Minimize();
  ComputeResults();
  if( fHasConverged == false )
    fNoReconstruction = 1;
  RecordFitSteps(); // 2 : store results of this fit step
  if( fVerbosity > 0 )
    cout << *(TFitting*)this << endl;

  if( fCorrectAltitude )
    fListTimes = fListTimesSave;

  return fHasConverged;
}

/*-------------------------------------------------------------------------------*/
bool TCdFReconstruct::Step3(double*step)
/*-------------------------------------------------------------------------------*/
{
  if( fCorrectAltitude ) {
    fListTimes = fListTimesSave;
    for( unsigned int i=0;i<fNTanks;i++ )
      fListTimes[i] += fTimeCorrection[i];
  }
  if( fVerbosity > 0 ) {
    cout << "\t* step 3 : again find u, v, curvature, T0 with variable R" << endl;
    cout << "\t\t===>" << endl;
  }

  SetFixParameter(kFitX0,fXCore);
  SetFixParameter(kFitY0,fYCore);
  SetFixParameter(kFitZ0,fZCore);
  SetFixParameter(kFitS,fSRef);
  SetFixParameter(kFitE,fE);
  SetFixParameter(kFitBeta,fBeta);
  SetFixParameter(kFitXMAX,fXmax);
  // keep same steps
  SetFreeParameter(kFitT0,fT0s,step[kFitT0]);
  SetFreeParameter(kFitU,fU,step[kFitU]);
  SetFreeParameter(kFitV,fV,step[kFitV]);
  SetFreeParameter(kFitC,1/(2.*fR),step[kFitC]);
  SetFCN(FCNCurvatureVariable);
  //  SetFCN(FCNCurvature);

  Minimize();
  ComputeResults();
  if( fHasConverged == false ) {
    fNoReconstruction = 1;
  } else {
    RecordFitSteps(); // 3 : store results of this fit step
    if( fVerbosity > 0 )
      cout << *(TFitting*)this << endl;
  }
  if( fCorrectAltitude )
    fListTimes = fListTimesSave;
  return fHasConverged;
}

/*-------------------------------------------------------------------------------*/
bool TCdFReconstruct::Step4(double*step)
/*-------------------------------------------------------------------------------*/
{
  if( fCorrectAltitude ) {
    fListTimes = fListTimesSave;
    for( unsigned int i=0;i<fNTanks;i++ )
      fListTimes[i] += fTimeCorrection[i];
  }
  if( fVerbosity > 0 ) {
    cout << "\t* step 4 : recompute S, XCore, YCore and T0 given new estimations of u, v and curvature" << endl;
    cout << "\t\t===>" << endl;
  }
  // remember errors on last variable parameters
  // since they will be fixed next time
  double erru = fdU;
  double errv = fdV;
  double errr = fdR;
  // recompute S1000, Xcore, Ycore, T0 given estimated u, v, Zcore for a curved front
  SetFixParameter(kFitZ0,fZCore);
  SetFixParameter(kFitU,fU);
  SetFixParameter(kFitV,fV);
  SetFixParameter(kFitC,1/(2.*fR));

  if ( fLDFType != "2EXPO" ) {
    SetFreeParameter(kFitX0,fXCore,step[kFitX0]);
    SetFreeParameter(kFitY0,fYCore,step[kFitY0]); 
   
    if( fNTanks > 4 && fBetaFree )
      SetFreeParameter(kFitBeta,fBeta,step[kFitBeta]);
    else
      SetFixParameter(kFitBeta,fBeta);
  }
  else {
    DCMAXTILTED=DCMAX/cos(fTheta*kCdasUtil::DEG2RAD);
    SetFreeParameter(kFitX0,fXCore,step[kFitX0],(fXCore-DCMAXTILTED),(fXCore+DCMAXTILTED));
    SetFreeParameter(kFitY0,fYCore,step[kFitY0],(fYCore-DCMAXTILTED),(fYCore+DCMAXTILTED));
  }

  if ( fLDFType != "SMSP" && fLDFType != "2EXPO" ) {
    SetFixParameter(kFitE,fE);
    SetFreeParameter(kFitS,fSRef,step[kFitS]);
  } else {
    SetFixParameter(kFitS,fSRef);
    if ( fLDFType == "SMSP" ){
      SetFreeParameter(kFitE,fE,step[kFitE]);
      SetMin(kFitE,fpSmspEstim->MinMap[SMSP_TYPE_ENERGY]);
      SetMax(kFitE,fpSmspEstim->MaxMap[SMSP_TYPE_ENERGY]);
    }  
    else if ( fLDFType == "2EXPO" ) {
      //SetFreeParameter(kFitE,fE,step[kFitE]);
      double theta = asin(sqrt(fU*fU+fV*fV));
      SetFreeParameter(kFitE,compute_energy("2EXPO",maxval(0,fSRef),theta,fPrimary,0,NULL),step[kFitE]);
    }
  }
  
  if( fLDFType == "HP" )
    SetFCN(FCNGlobalHP);
  else if( fLDFType == "LOG" )
    SetFCN(FCNGlobalLOG);
  else if( fLDFType == "SMSP" )
    SetFCN(FCNSMSP);
  else if( fLDFType == "2EXPO" )
    SetFCN(FCNGlobal2EXPO);
  Minimize();
  ComputeResults();
  if( fHasConverged == false )
    fNoReconstruction = 1;
  else {
    RecordFitSteps(); // 4 : store results of this fit step
    if( fVerbosity > 0 )
      cout << *(TFitting*)this << endl;

    // and restore correct error values
    fdU = erru;
    fdV = errv;
    fdR = errr;
  }

  if( fCorrectAltitude )
    fListTimes = fListTimesSave;

  return fHasConverged;
}

/*-------------------------------------------------------------------------------*/
bool TCdFReconstruct::Step5(double*step)
/*-------------------------------------------------------------------------------*/
{
  if( fNTanks < 4 )
    return false;
  TCdFShowerParams fitresults = *(TCdFShowerParams*)this;

  double r = fR;
  double errr = fdR;
  double r10,r50,r70,r90;
  double dr10,dr50,dr70,dr90;

  SetFixParameter(kFitX0,fXCore);
  SetFixParameter(kFitY0,fYCore);
  SetFixParameter(kFitZ0,fZCore);
  SetFixParameter(kFitS,fSRef);
  SetFixParameter(kFitE,fE);
  SetFixParameter(kFitU,fU);
  SetFixParameter(kFitV,fV);
  SetFreeParameter(kFitT0,fT0s,step[kFitT0]);
  SetFCN(FCNCurvatureVariable);

  SetListTime(10);
  SetFreeParameter(kFitC,1/(2.*fR),step[kFitC]);
  Minimize();
  ComputeResults();
  if( fVerbosity > -1 )
    cout << "fR10 = " << fR << endl;
  r10 = fR;
  dr10 = fdR;

  SetListTime(50);
  SetFreeParameter(kFitC,1/(2.*fR),step[kFitC]);
  Minimize();
  ComputeResults();
  if( fVerbosity > -1 )
    cout << "fR50 = " << fR << endl;
  r50=fR;
  dr50=fdR;

  SetListTime(70);
  SetFreeParameter(kFitC,1/(2.*fR),step[kFitC]);
  Minimize();
  ComputeResults();
  if( fVerbosity > -1 )
    cout << "fR70 = " << fR << endl;
  r70=fR;
  dr70=fdR;

  SetListTime(90);
  SetFreeParameter(kFitC,1/(2.*fR),step[kFitC]);
  Minimize();
  ComputeResults();
  if( fVerbosity > -1 )
    cout << "fR90 = " << fR << endl;
  r90=fR;
  dr90=fdR;

  fR = r;     //restore fR value
  fdR = errr;
  // restore global chi2 and errors
  CopyInstance((TCdFShowerParams*)this,&fitresults);
  //THIS DOES NOT WORK !!! find out WHY ! *(TCdFShowerParams*)this = fitresults;

  fR10  = r10;
  fdR10 = dr10;
  fR50  = r50;
  fdR50 = dr50;
  fR70  = r70;
  fdR70 = dr70;
  fR90  = r90;
  fdR90 = dr90;
  fChi2 = fitresults.fChi2;
  frChi2 = fitresults.frChi2;

  // restore times
  fListTimes = fListTimesSave;

  return fHasConverged;}

#include <THorReconstruct.h>
/*-------------------------------------------------------------------------------*/
void TCdFReconstruct::HorizontalReconstruction()
/*-------------------------------------------------------------------------------*/
{
  THorReconstruct horrec;

  *((TReconstruct *) & horrec) = *((TReconstruct *) this);
  horrec.FillInputs(Theta("deg"), Phi("deg"), fXCore, fYCore);
  horrec.Verbose = fVerbosity;
  horrec.SetEDPlot(fErEvent->fHorPlot);
  if ((horrec.Reconstruct(*fErEvent)) == false ) return;
  *((TReconstruct *) this) = *((TReconstruct *) & horrec);

  // we need one angular iteration...

  if (Theta("deg") > 84) {
    horrec.FillInputs(Theta("deg"), Phi("deg"), fXCore, fYCore);
    if ( (horrec.Reconstruct(*fErEvent)) == false ) return;
    *((TReconstruct *) this) = *((TReconstruct *) & horrec);
  }

  fVarCov[eXCore][eXCore] = sqr(horrec.dDXCore);
  fVarCov[eYCore][eYCore] = sqr(horrec.dDYCore);
  //fPhi = horrec.Phi*kCdasUtil::DEG2RAD;
  fVarCov[eSRef][eSRef] = sqr(fErEvent->fRec.fdE);

  for (int iy = 0; iy < 52; iy++) {
    for (int ix = 0; ix < 52; ix++) {
      fErEvent->fEDMapMu[ix][iy] = horrec.EDMapMu[ix][iy];
      fErEvent->fEDMapMuBis[ix][iy] = horrec.EDMapE[ix][iy];
    }
  }
  fErEvent->fEDdmax = (horrec.EDdxmax>horrec.EDdymax) ? horrec.EDdxmax: horrec.EDdymax;
  fErEvent->fEDdmax += 200;

  fSRef = 0;
  fdS1000 = 0;
  fsqfU2fV2 = sqrt( fU * fU + fV * fV );
  fXmax = 0;
  compute_distancesUV(fU,fV,fXCore,fYCore,fZCore,fListCoords,fAxisDistances);
}

/*-----------------------------------------*/
void TCdFReconstruct::RecordFitSteps()
/*-----------------------------------------*/
{
  compute_error_angle(fU,fdU,fV,fdV,&fdTheta,&fdPhi);
  if( fLDFType == "HP" )
    compute_error_energy(fLDFType,fSRef,fdS1000,fTheta,fdTheta,&fdE);
  else if ( fLDFType == "2EXPO" )
    compute_dE_dS1000_2EXPO(fE,fTheta,fSRef,&fdE,&fdS1000);
  fFitStep.push_back(*dynamic_cast<TCdFShowerParams*>(this));
  fNFit +=1;
}

/*-----------------------------------------*/
void TCdFReconstruct::RecordFitResults()
/*-----------------------------------------*/
{
  // copy fit results (u, v, core, energy ....)
  compute_error_angle(fU,fdU,fV,fdV,&fdTheta,&fdPhi);
  if( fLDFType == "HP" )
    compute_error_energy(fLDFType,fSRef,fdS1000,fTheta,fdTheta,&fdE);
  else if ( fLDFType == "2EXPO" )
    compute_dE_dS1000_2EXPO(fE,fTheta,fSRef,&fdE,&fdS1000);
  gvTCS.Sort("lt",EVENTINDEX);
  double x1,y1,z1;
  for( int i=0;i<gvTCS.fNParticipatingTanks;i++ ) {
    gvTCS[i][TIMERESIDUAL] = fListTimesResiduals[i];
    gvTCS[i][LSFITTEDTIME] = fListFittedTimes[i];
    gvTCS[i][TIMESHIFT] = 1e9*(gvTCS[i][LSTIME].Value() - fT0s);
    x1 = fListCoords[i][0] - fXCore;
    y1 = fListCoords[i][1] - fYCore;
    z1 = fListCoords[i][2] - fZCore;
    double cdist = sqrt(x1*x1+y1*y1+z1*z1);
    gvTCS[i][ANGLETOAXIS] = acos((x1*fU+y1*fV)/(cdist*sqrt(fU*fU+fV*fV)))*kCdasUtil::RAD2DEG;
    gvTCS[i][COREDISTANCE] = cdist;
  }
  if( fAxisDistances.size() > 0 ) {
    for( int i=0;i<gvTCS.fNParticipatingTanks;i++ ) {
      gvTCS[i][AXISDISTANCE] = fAxisDistances[i];
    }
  }
}

/*----------------------------------------------*/
void TCdFReconstruct::FillCovarianceMatrix()
/*----------------------------------------------*/
{
  fVarCov[eT0][eT0]       = sqr(fdT0s);
  fVarCov[eU][eU]         = sqr(fdU);
  fVarCov[eV][eV]         = sqr(fdV);
  fVarCov[eXCore][eXCore] = sqr(fdXCore);
  fVarCov[eYCore][eYCore] = sqr(fdYCore);
  fVarCov[eSRef][eSRef] = sqr(fdS1000);
  fVarCov[eR][eR]         = sqr(fdR);
}

/*----------------------------------------------*/
void TCdFReconstruct::Print(ostream& os) const
/*----------------------------------------------*/
{
  os << "---------------- TCdFReconstruct --------------------" << endl;
  os.setf(ios::fixed)
    ;
  int initprec = os.precision();
  os.precision(2);
  os << "NTanks    = " << fNTanks << endl;
  double no, ea;
  xy2ne(fXCore,fYCore,&ea,&no);
  os << "Northing  = " << no << endl;
  os << "Easting   = " << ea << endl;
  os.precision(0);
  os << "reconstruction (0 is good) = " << fNoReconstruction << endl;
  if( fNoReconstruction == 0 )
    TCdFShowerParams::Print(os);
  os.precision(initprec);
}

/*---------------------------------------------------*/
void TCdFReconstruct::RestoreData()
/*---------------------------------------------------*/
{
  if( gDataBackUp != 0x0 ) {
    CopyInstance(dynamic_cast<TDataToFit*>(this),gDataBackUp);
    fNoReconstruction = 2;
  } else
    cout << "TCdFReconstruct::RestoreData() : Data was not backuped. Returning." << endl;
}

/*---------------------------------------------------*/
void TCdFReconstruct::UseEstimate(string estim)
/*---------------------------------------------------*/
{
  fUseEstimate = estim;
}

/*---------------------------------------------------*/
void TCdFReconstruct::SetFluctuationStartTime(double trms)
/*---------------------------------------------------*/
{
  TRandom rand;
  for( unsigned int i=0;i<fNTanks;i++ )
    fListTimes[i]=rand.Gaus(fListTimes[i],trms);
}

/*---------------------------------------------------*/
void TCdFReconstruct::SetListTime(int percent)
/*---------------------------------------------------*/
{
  fListTimes = fListTimesSave;
  gvTCS.Sort("lt",EVENTINDEX);
  for( unsigned int i=0;i<fNTanks;i++ ) {
    if( percent==10 )
      fListTimes[i] = gvTCS[i][T10].Value();
    else if( percent==50 )
      fListTimes[i] = gvTCS[i][T50].Value();
    else if( percent==70 )
      fListTimes[i] = gvTCS[i][T70].Value();
    else if( percent==90 )
      fListTimes[i] = gvTCS[i][T90].Value();
    else
      cout << "TCdFReconstruct::SetListTime() : bad asked time" << endl;
  }
  if( fCorrectAltitude ) {
    for( unsigned int i=0;i<fNTanks;i++ )
      fListTimes[i] += fTimeCorrection[i];
  }
}


