#include "TCdFShowerParams.h"

/*------------------------------------*/
void TCdFShowerParams::InitShowerParams()
/*------------------------------------*/
{
  for (unsigned int i = 0; i<8; i++)
    for (unsigned int j = 0; j<8; j++) fVarCov[i][j] = 0.;
  fU = 0.;
  fdU = 0.;
  fV = 0.;
  fdV = 0.;
  fW = 0.;
  fdW = 0.;
  fsqfU2fV2 = 0.;
  fT0s = 0.;
  fdT0s = 0.;
  fTheta = 0.;
  fdTheta = 0.;
  fPhi = 0.;
  fdPhi = 0.;
  fE = 0.;
  fdE = 0.;
  fSRef= 0.;
  fdS1000 = 0.;
  fXCore = 0.;
  fdXCore = 0.;
  fYCore = 0.;
  fdYCore = 0.;
  fZCore = 0.;
  fdZCore = 0.;
  fR = 0.;
  fdR = 0.;
  fR10 = 0.;
  fdR10 = 0.;
  fR50 = 0.;
  fdR50 = 0.;
  fR70 = 0.;
  fdR70 = 0.;
  fR90 = 0.;
  fdR90 = 0.;
  fXmax   = 0.;
  fdXmax   = 0.;
  fBeta = 0.;
  fdBeta = 0.;
  fGamma = 0.;
  fChi2 = 0.;
  frChi2 = 0.;
  fddl    = 0.;
  fConvergenceStatus = 0;
  fHasConverged = 0.;
  fGPSTime = 0.;
  fYear = 0;
  fMonth = 0;
  fDay = 0;
  fHour = 0.;
  fNorthing = 0.;
  fdNorthing = 0.;
  fEasting = 0.;
  fdEasting = 0.;
  fLongitude = 0.;
  fdLongitude = 0.;
  fLatitude = 0.;
  fdLatitude = 0.;
  fElevation = 0.;
  fdElevation = 0.;
  fAzimuth = 0.;
  fdAzimuth = 0.;
  fL = 0.;
  fdL = 0.;
  fB = 0.;
  fdB = 0.;
  fra = 0.;
  fdec = 0.;
}

/*-------------------------------------------*/
void TCdFShowerParams::Print(ostream & os) const
/*-------------------------------------------*/
{
  char tmp[3];
  char out[100];
  sprintf(tmp,"%c",0xb1);
  string pm(tmp);
  sprintf(tmp,"%c",0xb0);
  string deg(tmp);
  os << "---------------- TCdFShowerParams --------------------" << endl;
  os.setf(ios::fixed)
    ;
  int initprec = os.precision();
  os << "X core    = " << fXCore << " (" << pm << " " << fdXCore << ")"<< endl;
  os << "Y core    = " << fYCore << " (" << pm << " " << fdYCore << ")"<< endl;
  os << "Z core    = " << fZCore << " (" << pm << " " << fdZCore << ")"<< endl;
  sprintf(out,"u         = %5.4f (%s %7.5f = %s %7.5f %%)",fU,pm.c_str(),fdU,pm.c_str(),100.*fabs(fdU/fU));
  os << out << endl;
  sprintf(out,"v         = %5.4f (%s %7.5f = %s %7.5f %%)",fV,pm.c_str(),fdV,pm.c_str(),100.*fabs(fdV/fV));
  os << out << endl;
  os.precision(0);
  os << "t0        = " << floor(1e9*fT0s) << " ns ";
  os << pm << " " << floor(1e9*fdT0s) << " ns" << endl;
  os.precision(4);
  os << "theta     = " << fTheta << deg << " (" << pm << " " << fdTheta << deg << ")" << endl;
  os << "phi       = " << fPhi << deg << " (" << pm << " " << fdPhi << deg << ")" << endl;
  os << "energy    = " << fE << " EeV" << " (" << pm << " " << fdE << " EeV" << ")"<< endl;
  os << "S1000     = " << fSRef << " (" << pm << " "
  << fdS1000 << " = " << pm << " " << 100.*fabs(fdS1000/fSRef) << " %)" << endl;
  os << "beta      = " << fBeta << " (" << pm << " " << fdBeta << ")" << endl;
  os << "gamma     = " << fGamma << endl;
  os.precision(3);
  os << "curvature (T0) = " << fR/1e3 << " km (" << pm << " " << fdR/1e3 << " km)" << endl;
  os << "curvature (T10)= " << fR10/1e3 << " km (" << pm << " " << fdR10/1e3 << " km)" << endl;
  os << "curvature (T50)= " << fR50/1e3 << " km (" << pm << " " << fdR50/1e3 << " km)" << endl;
  os << "curvature (T70)= " << fR70/1e3 << " km (" << pm << " " << fdR70/1e3 << " km)" << endl;
  os << "curvature (T90)= " << fR90/1e3 << " km (" << pm << " " << fdR90/1e3 << " km)" << endl;

  if( fXmax != 0) {
    os << "Xmax       = " << fXmax << "  " << pm << " " << fdXmax << endl;
  }

  os << "Chi2      = " << fChi2 << endl;
  os << "Chi2/ddl  = " << frChi2 << endl;
  os << "conv stat = " << fConvergenceStatus << endl;
  os << "conv glob = " << fHasConverged << endl;
  os.precision(initprec);
}

/*----------------------------------------------------------------*/
void CopyInstance(TCdFShowerParams * dest, const TCdFShowerParams * src)
/*----------------------------------------------------------------*/
{
  (*dest).fU = (*src).fU;
  (*dest).fdU = (*src).fdU;
  (*dest).fV = (*src).fV;
  (*dest).fdV = (*src).fdV;
  (*dest).fW = (*src).fW;
  (*dest).fdW = (*src).fdW;
  (*dest).fsqfU2fV2 = (*src).fsqfU2fV2;
  (*dest).fT0s = (*src).fT0s;
  (*dest).fdT0s = (*src).fdT0s;
  (*dest).fTheta = (*src).fTheta;
  (*dest).fdTheta = (*src).fdTheta;
  (*dest).fPhi = (*src).fPhi;
  (*dest).fdPhi = (*src).fdPhi;
  (*dest).fE = (*src).fE;
  (*dest).fdE = (*src).fdE;
  (*dest).fSRef = (*src).fSRef;
  (*dest).fdS1000 = (*src).fdS1000;
  (*dest).fXCore = (*src).fXCore;
  (*dest).fdXCore = (*src).fdXCore;
  (*dest).fYCore = (*src).fYCore;
  (*dest).fdYCore = (*src).fdYCore;
  (*dest).fZCore = (*src).fZCore;
  (*dest).fdZCore = (*src).fdZCore;
  (*dest).fR = (*src).fR;
  (*dest).fdR = (*src).fdR;
  (*dest).fR10 = (*src).fR10;
  (*dest).fdR10 = (*src).fdR10;
  (*dest).fR50 = (*src).fR50;
  (*dest).fdR50 = (*src).fdR50;
  (*dest).fR70 = (*src).fR70;
  (*dest).fdR70 = (*src).fdR70;
  (*dest).fR90 = (*src).fR90;
  (*dest).fdR90 = (*src).fdR90;
  (*dest).fXmax = (*src).fXmax;
  (*dest).fdXmax = (*src).fdXmax;
  (*dest).fBeta = (*src).fBeta;
  (*dest).fdBeta = (*src).fdBeta;
  (*dest).fGamma = (*src).fGamma;
  (*dest).fChi2 = (*src).fChi2;
  (*dest).frChi2 = (*src).frChi2;
  (*dest).fddl = (*src).fddl;
  (*dest).fConvergenceStatus = (*src).fConvergenceStatus;
  (*dest).fHasConverged = (*src).fHasConverged;
  (*dest).fGPSTime = (*src).fGPSTime;
  (*dest).fYear = (*src).fYear;
  (*dest).fMonth = (*src).fMonth;
  (*dest).fDay = (*src).fDay;
  (*dest).fHour = (*src).fHour;
  (*dest).fNorthing = (*src).fNorthing;
  (*dest).fdNorthing = (*src).fdNorthing;
  (*dest).fEasting = (*src).fEasting;
  (*dest).fdEasting = (*src).fdEasting;
  (*dest).fLongitude = (*src).fLongitude;
  (*dest).fdLongitude = (*src).fdLongitude;
  (*dest).fLatitude = (*src).fLatitude;
  (*dest).fdLatitude = (*src).fdLatitude;
  (*dest).fElevation = (*src).fElevation;
  (*dest).fdElevation = (*src).fdElevation;
  (*dest).fAzimuth = (*src).fAzimuth;
  (*dest).fdAzimuth = (*src).fdAzimuth;
  (*dest).fL = (*src).fL;
  (*dest).fdL = (*src).fdL;
  (*dest).fB = (*src).fB;
  (*dest).fdB = (*src).fdB;
  (*dest).fra = (*src).fra;
  (*dest).fdec = (*src).fdec;
}
