#ifndef _TCDF_SHOWER_PARAMS_
#define _TCDF_SHOWER_PARAMS_

#include "ToolsLib.h"
#include "TShowerParams.h"
/**
   @file TCdFShowerParams.h
*/

//! TCdFShowerParams class : contains shower parameters after fit

class TCdFShowerParams : virtual public TShowerParams {

public:

  TCdFShowerParams() {
    InitShowerParams();
  }

  //  double fU;
  double fdU;
  //  double fV;
  double fdV;
  double fTheta;
  double fdTheta;
  double fPhi;
  double fdPhi;
  //  double fE;
  //  double fdE;
  //  double fS1000;
  double fdS1000;
  //  double fXCore;
  double fdXCore;
  //  double fYCore;
  double fdYCore;
  //double fZCore;
  double fdZCore;
  //  double fXmax;
  double fdXmax;
  double fdUU;
  double fdVV;
  double fdUV;
  double fW;
  double fdW;
  double fsqfU2fV2; ///< sqrt(u^2+v^2) = sin^2(theta)
  double fT0s;
  /**< impact time in fraction of second since the
     smallest second value of the tanks in the event
   */
  double fdT0s;      ///< error on fT0s
  //  double fR;        ///< curvature radius
  double fdR;       ///< error on curvature radius
  double fR10;      ///< curvature radius
  double fdR10;     ///< error on curvature radius
  double fR50;      ///< curvature radius
  double fdR50;     ///< error on curvature radius
  double fR70;      ///< curvature radius
  double fdR70;     ///< error on curvature radius
  double fR90;      ///< curvature radius
  double fdR90;     ///< error on curvature radius
  //  double fBeta;
  double fdBeta;
  //  double fGamma;

  int fConvergenceStatus; ///< convergence status given by Minuit
  bool fHasConverged; ///< convergence true if fConvergenceStatus != 0

  double fGPSTime;      ///< GPS Time of event impact
  int    fYear;         ///< year of the event (UTC)
  int    fMonth;        ///< month of the event (UTC)
  int    fDay;          ///< day of the event (UTC)
  double fHour;         ///< decimal value of the hour (UTC)
  double fNorthing;     ///< Northing of the core
  double fdNorthing;    ///< error on Northing
  double fEasting;      ///< Easting of the core
  double fdEasting;     ///< error on Easting
  double fLongitude;    ///< Longitude of the core
  double fdLongitude;   ///< error on Longitude
  double fLatitude;     ///< Latitude of the core
  double fdLatitude;    ///< error on Latitude
  double fElevation;    ///< elevation of the shower (angle above the horizon)
  double fdElevation;   ///< elevation of the shower (angle above the horizon)
  double fAzimuth;
  /**< azimuth of the shower (from south measured westwards), faz =
     270 - fPhi is the azimuth which must be used with the
     STCoordinates functions
  */
  double fdAzimuth;     ///< error on Azimuth
  double fL;            ///< galactic longitude
  double fdL;           ///< error on L
  double fB;            ///< galactic latitude
  double fdB;           ///< error on B
  double fra;           ///< right ascension
  double fdec;          ///< declination

  //  double fChi2;               ///< chi2 square of the last fit
  //  double frChi2;              ///< is the reduced chi-square (chi2/ddl)
  //  double fddl;                ///< number of degrees of freedom (is the number of tanks of the event)


  void Print(ostream&) const;
  void InitShowerParams();

};

inline ostream& operator<<(ostream& s, const TCdFShowerParams& tsp) {
  tsp.Print(s);
  return s;
}

void CopyInstance(TCdFShowerParams * dest, const TCdFShowerParams * src);

#endif
