#include "TCommonStation.h"

/******************* TClassMember *************************/
TClassMember& TClassMember::operator=(int i) {
  fIsInt = true;
  fIsDouble = false;
  fInt = i;
  return *this;
}

TClassMember& TClassMember::operator=(double i) {
  fIsInt = false;
  fIsDouble = true;
  fDouble = i;
  return *this;
}

bool TClassMember::operator==(const TClassMember& cm) const {
  return ((fIsInt&&cm.fIsInt)&&(fInt==cm.fInt))||((fIsDouble&&cm.fIsDouble)&&(fDouble==cm.fDouble));
}

bool TClassMember::operator<(const TClassMember& cm) const {
  return ((fIsInt&&cm.fIsInt)&&(fInt<cm.fInt))||((fIsDouble&&cm.fIsDouble)&&(fDouble<cm.fDouble));
}

bool TClassMember::operator>(const TClassMember& cm) const {
  return ((fIsInt&&cm.fIsInt)&&(fInt>cm.fInt))||((fIsDouble&&cm.fIsDouble)&&(fDouble>cm.fDouble));
}

bool TClassMember::operator<=(const TClassMember& cm) const {
  return ((fIsInt&&cm.fIsInt)&&(fInt<=cm.fInt))||((fIsDouble&&cm.fIsDouble)&&(fDouble<=cm.fDouble));
}

bool TClassMember::operator>=(const TClassMember& cm) const {
  return ((fIsInt&&cm.fIsInt)&&(fInt>=cm.fInt))||((fIsDouble&&cm.fIsDouble)&&(fDouble>=cm.fDouble));
}

void TClassMember::Print(ostream& s) const {
  s << fIsInt*fInt+fIsDouble*fDouble;
}

/********************** TCommonStation ********************/
void TCommonStation::Zero() {
  fFilledRanks.clear();
  fvcm.resize(NCLASSMEMBERS);
  for( unsigned int i=0;i<NCLASSMEMBERS;i++ )
    fvcm[i] = 0;
  // specific initialization values :
  fvcm[PARTICIPATE] = 1; // very important
}

void TCommonStation::SetClassMember(int rank, int value) {
  vector<int>::iterator vi;
  vi = find(fFilledRanks.begin(),fFilledRanks.end(),rank);
  if( vi == fFilledRanks.end() ) {
    fFilledRanks.push_back(rank);
    sort(fFilledRanks.begin(),fFilledRanks.end());
  }
  fvcm[rank] = value;
}

void TCommonStation::SetClassMember(int rank, double value) {
  vector<int>::iterator vi;
  vi = find(fFilledRanks.begin(),fFilledRanks.end(),rank);
  if( vi == fFilledRanks.end() ) {
    fFilledRanks.push_back(rank);
    sort(fFilledRanks.begin(),fFilledRanks.end());
  }
  fvcm[rank] = value;
}

void TCommonStation::Print(ostream& s) const {
  s << "TCommonStation : " << endl;
  //  for( unsigned int i=0;i<3;i++ )//NCLASSMEMBERS;i++ )
  for( unsigned int i=0;i<NCLASSMEMBERS;i++ ) {
    cout << i << " : " << fvcm[i] << endl;
  }
}
/********************** TLsOrdering ********************/
bool TLsOrdering::operator()(const TCommonStation& ls0, const TCommonStation& ls1) const {
  bool result = false;
  if( fOrder == "lt" )
    result = ( ls0.fvcm[fRank]  <  ls1.fvcm[fRank] );
  else if( fOrder == "gt" )
    result = ( ls0.fvcm[fRank]  >  ls1.fvcm[fRank] );
  else if( fOrder == "le" )
    result = ( ls0.fvcm[fRank]  <= ls1.fvcm[fRank] );
  else if( fOrder == "ge" )
    result = ( ls0.fvcm[fRank]  >= ls1.fvcm[fRank] );
  else
    cout << "Ordering string not recognized : " << fOrder << endl;
  return result;
}

/********************** TvCommonStation ********************/

// initialize statics
TvCommonStation * gCSBackUp = 0x0;

TvCommonStation::~TvCommonStation() {
  if( gCSBackUp == 0x0 )
    delete gCSBackUp;
}

int TvCommonStation::Remove(int rank, TClassMember value) {
  // FIXME : case where this->size() == 1
  int initprec = cout.precision();
  cout.precision(0);
  int out = -1;
  //unsigned int size = this->size();
  fvTCS = find_if(begin(),end(),TLsEq(rank,value));
  if( fvTCS != end() ) {
    out = fvTCS-begin();
    if( (*this)[out][PARTICIPATE] == (TClassMember)(int)0 ) {
      cout << "This station has already been removed." << endl;
      cout.precision(initprec);
      return -1;
    }
    cout << "Setting tank " << (*this)[out][LSID] << " (" << out << ") to false." << endl;
    (*this)[out][PARTICIPATE] = false;
    int out_actualevent = (*this)[out][ACTUALEVENTINDEX].Int();
    (*this)[out][ACTUALEVENTINDEX] = -1;
    for( int i=0;i<fNParticipatingTanks;i++ ) {
      if( i == out )
        continue;
      int newvalue = (*this)[i][ACTUALEVENTINDEX].Int();
      if( newvalue > out_actualevent )
        fLsMapActualEventIndex[(*this)[i][LSID].Int()] = --newvalue;
      cout << "lsid " << (*this)[i][LSID].Int() << " was " << (*this)[i][ACTUALEVENTINDEX].Int() << " becomes " << newvalue << endl;
      (*this)[i][ACTUALEVENTINDEX] = newvalue;
    }
    fRemovedTanks.push_back((*this)[out]);
    erase(begin()+out);
  } else
    cout << "Station not found." << endl;
  fNParticipatingTanks--;
  cout.precision(initprec);
  return out;
}

int TvCommonStation::Remove(int rank, int value) {
  return Remove(rank,(TClassMember)(int)value);
}

int TvCommonStation::Remove(int rank, double value) {
  return Remove(rank,(TClassMember)(double)value);
}

void TvCommonStation::Sort(string order, int rank) {
  sort(begin(),end(),TLsOrdering(order,rank));
}

void TvCommonStation::Print() {
  unsigned int size = this->size();
  if( size == 0 ) {
    cout << "TvCommonStation instance is empty." << endl;
    return;
  }
  // participating tanks
  int initprec = cout.precision();
  cout.precision(0);
  for( unsigned int i=0;i<size;i++ ) {
    cout << (*this)[i][LSID] << " (" << (*this)[i][EVENTINDEX]
    << "," << (*this)[i][ACTUALEVENTINDEX] << ")";
    if( i < size-1 )
      cout << " --- ";
  }
  size = fRemovedTanks.size();
  // removed tanks
  if( size > 0 ) {
    cout << " out : ";
    for( unsigned int i=0;i<size;i++ ) {
      cout << fRemovedTanks[i][LSID] << " (" << fRemovedTanks[i][EVENTINDEX]
      << "," << fRemovedTanks[i][ACTUALEVENTINDEX] << ")";
      if( i < size-1 )
        cout << " --- ";
    }
  }
  cout << endl;
  cout.precision(initprec);
}

void TvCommonStation::Zero() {
  clear();
  fNParticipatingTanks = 0;
  fLsMapEventIndex.clear();
  fLsMapActualEventIndex.clear();
  fPairDistances.clear();
  fParticipatingPairDistances.clear();
  fRemovedTanks.clear();
}

void TvCommonStation::BackUp() {
  if( gCSBackUp == 0x0 )
    gCSBackUp = new TvCommonStation();
  *gCSBackUp = *this;
}

void TvCommonStation::RestoreData() {
  if( gCSBackUp != 0x0 )
    *this = *gCSBackUp;
  else
    cout << "TvCommonStation has not been saved. Can't restore data." << endl;
}
