#ifndef _TDATATOFIT_H_
#define _TDATATOFIT_H_

#include <vector>
#include "ToolsLib.h"

using namespace std;
/**
   @file TDataToFit.h
*/

//! TDataToFit class : contains all data used for fit

class TDataToFit {

public:

  TDataToFit() {
    InitDataToFit();
  }

  TDataToFit(const vector<int>& ListIds,
             const vector<double>& ListTimes,
             const vector< vector<double> >& ListCoords);

  TDataToFit(const vector<int>& ListIds,
             const vector< vector<double> >& ListCoords,
             const vector<double>& ListSignals);

  TDataToFit(const vector<int>& ListIds,
             const vector<double>& ListTimes,
             const vector< vector<double> >& ListCoords,
             const vector<double>& ListSignals);

  TDataToFit(const vector<int>& ListIds,
             const vector<double>& ListTimes,
             const vector< vector<double> >& ListCoords,
             const vector< vector<double> >& ListCoordsSilent,
             const vector<double>& ListSignals);

  void SetLsIds(const vector<int>& ListIds);
  void SetTimes(const vector<double>& ListTimes);
  void SetTimesErrors(const vector<double>& ListTimesErrors);
  void SetCoords(const vector< vector<double> >& ListCoords);
  void SetSignals(const vector<double>& ListSignals);
  void SetSignalsErrors(const vector<double>& ListSignalsErrors);
  void SetSaturated(const vector<bool>& ListSaturated);
  void ComputeConstTimesErrors(double sigma);
  void ComputeVarTimeError(int i, double r, double costheta2);
  void ComputeSignalsErrors();

  void Remove(int); // rank is given for instance by gvTCS.Remove(int)

  // what we get from raw data
  unsigned int fNTanks; ///< number of tanks
  vector<int> fListIds; ///< Tank Id
  vector<int> fListIdsSilent; ///< silent Tank Id
  vector<double> fListTimes; ///< Tank times
  vector<double> fListTimesErrors;
  vector<bool> fListSaturated;
  vector< vector<double> > fListCoords; ///< vector (x,y,z) coordinates
  vector< vector<double> > fListCoordsSilent; ///< vector (x,y,z) coordinates of silent stations
  vector<double> fListSignals; ///< Tank signals
  vector<double> fListSigOverPeak;
  vector<double> fListSignalsErrors;
  vector<double> fUserWeights;

  // what we get from the fit
  vector<double> fListTimesResiduals;
  vector<double> fListFittedTimes;
  vector<double> fAxisDistances; ///< distances
  vector<double> fCoreDistances;
  vector< vector<double> > fLDF; ///< values for LDF fit

  void InitDataToFit(); ///< init. everything
  void InitFitResults(); ///< init. only the part we get from the fit

};

void CopyInstance(TDataToFit * dest, const TDataToFit * src);


#endif
