#include <THorReconstruct.h>
#include <ToolsLib.h>
#include "STCconstants.h"


MuonParamSignal *THorReconstruct::MuParamSigP = new MuonParamSignal();
MuonParamTable *THorReconstruct::MuParamTabP = new MuonParamTable();

double Interpol(double x1, double x2, double y1, double y2, double x) {
  return (((y2 - y1) / (x2 - x1)) * (x - x1) + y1);
}

int GetIndex(double theta, char *mode) {
  int index;

  index = (int) (((int) (theta) - 70) / 2);
  if (strcmp(mode, "sup") == 0)
    index++;
  if (index > 9)
    index = 9;              // case theta>88deg.

  return index;
}

static THorReconstruct *_horrec_instance = NULL;
static int _troubles = 0;

MuonParamSignal::MuonParamSignal() {
  char files[256], tmp[256];
  int thetas[NBINTH] = { 70, 72, 74, 76, 78, 80, 82, 84, 86, 88 };
  for (int i = 0; i < NBINTH; i++)
    for (int j = 0; j < NBINMUSIG; j++) {
      strcpy(files, "\0");
      strcpy(files, SIGMUPATH);
      sprintf(tmp, "proton/sig_muon_%d/sig_%d.dat", thetas[i],
              j + 1);
      strcat(files, tmp);
      FILE *fin = fopen(files, "r");
      if (fin == NULL) {
        cerr <<
        "Warning: Can't open files needed for horizontal reconstruction !!! "
        << endl;
        _troubles = 1;
        return;
      }
      NBinSig[i][j] = 0;
      double a, b;
      while (fgets(tmp, 256, fin)) {
        sscanf(tmp, "%lf %lf\n", &a, &b);
        Signal[i][j][NBinSig[i][j]] = (double) a;
        Occurence[i][j][NBinSig[i][j]] = (double) b;
        NBinSig[i][j]++;
      }
      fclose(fin);
    }

  for (int i = 0; i < NBINTH; i++) {
    strcpy(files, "\0");
    strcpy(files, SIGMUPATH);
    sprintf(tmp, "proton/sig_muon_%d/param.dat", thetas[i]);
    strcat(files, tmp);
    FILE *fin = fopen(files, "r");
    fgets(tmp, 256, fin);
    sscanf(tmp, "%lf %lf\n", &CoeffMeanOffset[i], &CoeffMeanPente[i]);
    fgets(tmp, 256, fin);
    sscanf(tmp, "%lf\n", &CoeffSigma[i]);
    fclose(fin);
  }
}

MuonParamSignal::~MuonParamSignal() {}

double MuonParamSignal::Value(int index, int k, double X) {
  if (X < 1.75)
    return Occurence[index][k][1];  // threshold trigger
  int i=0;

  for (; i < NBinSig[index][k]-1000; i += 1000)
    if (Signal[index][k][i] < X && X < Signal[index][k][i+1000]) break;

  for (; i < NBinSig[index][k]-100; i += 100)
    if (Signal[index][k][i] < X && X < Signal[index][k][i+100]) break;

  for (; i < NBinSig[index][k]-10; i += 10)
    if (Signal[index][k][i] < X && X < Signal[index][k][i+10]) break;

  for (; i < NBinSig[index][k]-1; i++)
    if (Signal[index][k][i] < X && X < Signal[index][k][i+1]) {
      return Occurence[index][k][i] + (Occurence[index][k][i+1] - Occurence[index][k][i]) /
             (Signal[index][k][i+1] - Signal[index][k][i]) * (X - Signal [index] [k][i]);
    }

  return 0.;

}


double MuonParamSignal::GetSignalProb(double theta, double sig_tank, int k) {
  char mode[256];
  strcpy(mode, "inf");
  int i_inf = GetIndex(theta, mode);
  strcpy(mode, "sup");
  int i_sup = GetIndex(theta, mode);
  int thetas[NBINTH] = { 70, 72, 74, 76, 78, 80, 82, 84, 86, 88 };
  double prob;

  if (k < NBINMUSIG) {
    double p_inf = Value(i_inf, k, sig_tank);
    double p_sup = Value(i_sup, k, sig_tank);
    prob = Interpol(thetas[i_inf], thetas[i_sup], p_inf, p_sup, theta);
  } else {
    double mean =
      Interpol(thetas[i_inf], thetas[i_sup], CoeffMeanPente[i_inf],
               CoeffMeanPente[i_sup],
               theta) * k + Interpol(thetas[i_inf], thetas[i_sup],
                                     CoeffMeanOffset[i_inf],
                                     CoeffMeanOffset[i_sup], theta);
    double sigma =
      Interpol(thetas[i_inf], thetas[i_sup], CoeffSigma[i_inf],
               CoeffSigma[i_sup], theta) * sqrt(k*1.0);
    prob =
      (1 / sqrt(2 * M_PI * pow(sigma, 2.))) *
      exp(-pow(sig_tank - mean, 2) / (2 * pow(sigma, 2)));
  }

  return prob;
}

MuonParamTable::MuonParamTable() {
  char files[256], tmp[256];
  int thetas[NBINTH] = { 70, 72, 74, 76, 78, 80, 82, 84, 86, 88 };
  for (int i = 0; i < NBINTH; i++) {
    strcpy(files, "\0");
    strcpy(files, SIGMUPATH);
    sprintf(tmp, "proton/sig_muon_%d/prot_10_%d.par", thetas[i],
            thetas[i]);
    strcat(files, tmp);

    FILE *fin = fopen(files, "r");
    if (fin == NULL) {
      cerr <<
      "Warning: Can't open files needed for horizontal reconstruction !!! "
      << endl;
      _troubles = 1;
      return;
    }
    for (int k = 0; k < NLEGENDREDEG; k++) {
      for (int m = 0; m < NFOURIERPHIDEG; m++) {
        fscanf(fin, "%lf", &AlphaNMu[i][k][0][m]);
        BetaNMu[i][k][0][m] = 0.;
      }
    }
    for (int l = 1; l < NFOURIERPSIDEG; l++) {
      for (int k = 0; k < NLEGENDREDEG; k++) {
        for (int m = 0; m < NFOURIERPHIDEG; m++) {
          fscanf(fin, "%lf", &AlphaNMu[i][k][l][m]);
        }
      }
      for (int k = 0; k < NLEGENDREDEG; k++) {
        BetaNMu[i][k][l][0] = 0;
        for (int m = 1; m < NFOURIERPHIDEG; m++) {
          fscanf(fin, "%lf", &BetaNMu[i][k][l][m]);
        }
      }
    }
    fclose(fin);
  }
}


MuonParamTable::~MuonParamTable() {}

void MuonParamTable::BuildCurrentMaps(double theta, double phi, int flag) {
  if (flag == 0) {
    _CurrentThetaRad = theta * M_PI / 180.;
    _CurrentPhiRad = phi * M_PI / 180.;
  }
  char mode[256];
  strcpy(mode, "inf");
  int i_inf = GetIndex(theta, mode);
  strcpy(mode, "sup");
  int i_sup = GetIndex(theta, mode);
  double phi_aires = (phi - (90. - kSTC::NorthDeclination)) * M_PI / 180.;
  double alpha, beta;         // interpolated values
  int theta_inf = (int) (theta);
  if ((theta_inf % 2) != 0)
    theta_inf++;
  double dth = (theta - theta_inf) / 2.;

  for (int k = 0; k < NLEGENDREDEG; k++) {
    for (int l = 0; l < NFOURIERPSIDEG; l++) {
      CurrentAlphaNMu[k][l] = CurrentBetaNMu[k][l] = 0.;
      for (int m = 0; m < NFOURIERPHIDEG; m++) {
        alpha =
          (1. - dth) * AlphaNMu[i_inf][k][l][m] +
          dth * AlphaNMu[i_sup][k][l][m];
        CurrentAlphaNMu[k][l] += alpha * cos(m * phi_aires);
      }
      if (l > 0) {
        for (int m = 1; m < NFOURIERPHIDEG; m++) {
          beta =
            (1. - dth) * BetaNMu[i_inf][k][l][m] +
            dth * BetaNMu[i_sup][k][l][m];
          CurrentBetaNMu[k][l] += beta * sin(m * phi_aires);
        }
      }
    }
  }

}

double MuonParamTable::NMuAtGround(double x, double y, double dphi) {
  static const double krmax = 5000.;
  //  static const double knslice = 40;
  double x_prime = x * cos(_CurrentPhiRad+dphi) + y * sin(_CurrentPhiRad+dphi);
  double y_prime = -x * sin(_CurrentPhiRad+dphi) + y * cos(_CurrentPhiRad+dphi);
  double x_sh = x_prime * cos(_CurrentThetaRad);
  double y_sh = y_prime;

  double r = sqrt(pow(x_sh, 2.) + pow(y_sh, 2.));
  if (r > krmax)
    return 1e-6;
  double psi = atan2(y_sh, x_sh);
  double rho = 2 * sqrt(r / krmax) - 1;

  if (dphi != 0.) {
    BuildCurrentMaps(_CurrentThetaRad*180./M_PI, (_CurrentPhiRad+dphi)*180./M_PI, 1);
  }

  double n_mu = 0, xx = 1;
  for (int k = 0; k < NLEGENDREDEG; k++) {
    for (int l = 0; l < NFOURIERPSIDEG; l++) {
      n_mu += xx * CurrentAlphaNMu[k][l] * cos(l * psi);
      if (l > 0)
        n_mu += xx * CurrentBetaNMu[k][l] * sin(l * psi);
    }
    xx *= rho;
  }
  return exp(n_mu);
}

double MuonParamTable::ThetaAtGround(double x, double y) {
  static const double krmax = 5000.;
  //  static const double knslice = 40;
  double x_prime = x * cos(_CurrentPhiRad) + y * sin(_CurrentPhiRad);
  double y_prime = -x * sin(_CurrentPhiRad) + y * cos(_CurrentPhiRad);
  double x_sh = x_prime * cos(_CurrentThetaRad);
  double y_sh = y_prime;

  double r = sqrt(pow(x_sh, 2.) + pow(y_sh, 2.));
  if (r > krmax)
    return _CurrentThetaRad*180./M_PI;
  //double psi = atan2(y_sh, x_sh);
  double rho = 2 * sqrt(r / krmax) - 1;


  double th_mu = 0, xx = 1;
  for (int k = 0; k < 5; k++) {
    for (int l = 0; l < 7; l++) {
      //            th_mu += xx * CurrentAlphaAngSpread[l][k] * cos(l * psi);
      //            if (l > 0)
      //                th_mu += xx * CurrentBetaAngSpread[l][k] * sin(l * psi);
    }
    xx *= rho;
  }

  return acos(th_mu);

}

double MuonParamTable::MeanMuEnerAtGround(double x, double y) {
  static const double krmax = 5000.;
  //  static const double knslice = 40;
  double x_prime = x * cos(_CurrentPhiRad) + y * sin(_CurrentPhiRad);
  double y_prime = -x * sin(_CurrentPhiRad) + y * cos(_CurrentPhiRad);
  double x_sh = x_prime * cos(_CurrentThetaRad);
  double y_sh = y_prime;

  double r = sqrt(pow(x_sh, 2.) + pow(y_sh, 2.));
  if (r > krmax)
    return 1.;
  //double psi = atan2(y_sh, x_sh);
  double rho = 2 * sqrt(r / krmax) - 1;


  double e_mu = 0, xx = 1;
  for (int k = 0; k < 5; k++) {
    for (int l = 0; l < 7; l++) {
      //            e_mu += xx * CurrentAlphaMeanMuEner[l][k] * cos(l * psi);
      //            if (l > 0)
      //                e_mu += xx * CurrentBetaMeanMuEner[l][k] * sin(l * psi);
    }
    xx *= rho;
  }

  return pow(10, e_mu);

}

void fcnStep1(int &, double *, double &f, double *par, int) {
  f = _horrec_instance->FCNStep1(par);
}

void fcnStep2(int &, double *, double &f, double *par, int) {
  f = _horrec_instance->FCNStep2(par);
}

void fcnStep3(int &, double *, double &f, double *par, int) {
  f = _horrec_instance->FCNStep3(par);
}

THorReconstruct::THorReconstruct() {}

THorReconstruct::~THorReconstruct() {}

void THorReconstruct::FillInputs(double theta, double phi, double xcore,
                                 double ycore) {
  Theta = theta;
  Phi = phi;
  XCore = xcore;
  YCore = ycore;
}

void THorReconstruct::SetEDPlot(int plot) {
  _EDPlot = plot;
}

bool THorReconstruct::_Reconstruct() {
  if (_troubles)
    return false;

  _horrec_instance = this;
  MuParamTabP->BuildCurrentMaps(Theta, Phi);
  if (Step1() == 0)
    return false;

  if (Step2() == 0)
    return false;

  if (Step3() == 0)
    return false;

  EDdxmax = 0., EDdymax = 0.;
  for (unsigned int is = 0; is < fErEvent->fSelectedStations.size();
       is++) {
    double dx = fErEvent->fSelectedStations[is]->fX - XCore;
    double dy = fErEvent->fSelectedStations[is]->fY - YCore;
    if (fabs(dx) > EDdxmax)
      EDdxmax = fabs(dx);
    if (fabs(dy) > EDdymax)
      EDdymax = fabs(dy);
  }

  double eddmax = (EDdxmax>EDdymax) ? EDdxmax: EDdymax;
  double xcoresave=XCore, ycoresave=YCore;
  double esave=fE, desave=fdE;
  double x = -eddmax, y = -eddmax;
  for (int iy = 0; iy < 52; iy++) {
    x = -eddmax;
    for (int ix = 0; ix < 52; ix++) {
      if (_EDPlot == 1) {
        double par[3]={fE, x, y};
        EDMapMu[ix][iy] = FCNStep3(par);
        cout<<"dx: "<<x<<" dy: "<<y<<" L(dx,dy,E): "<<EDMapMu[ix][iy]<<endl;
      } else if (_EDPlot == 0) {
        EDMapMu[ix][iy] = fE * MuParamTabP->NMuAtGround(x, y, 0.);
      } else if (_EDPlot == 2) {
        XCore=xcoresave+x;
        YCore=ycoresave+y;
        Step1();
        EDMapE[ix][iy] = fE;
        double par[1]={fE};
        EDMapMu[ix][iy] = FCNStep1(par);
        cout<<"dx: "<<x<<" dy: "<<y<<" E0(dx,dy): "<<fE<<" L(dx,dy,E0(dx,dy): "<<EDMapMu[ix][iy]<<endl;
      }
      x += 2 * eddmax / 51;
    }
    y += 2 * eddmax / 51;
  }
  XCore=xcoresave;
  YCore=ycoresave;
  fE=esave;
  fdE=desave;


  double ener, dener;
  ener = 10*pow(fE,0.95)*.9;
  dener = 0.95*ener*fdE;///fE;
  fE = ener;
  fdE = dener;
  fXCore = XCore;
  fYCore = YCore;

  return true;
}

int THorReconstruct::Step1() {
  int errflag = 0;
  double arglist[10];

  TMinuit minuit(1);
  minuit.SetPrintLevel(-1);
  minuit.SetFCN(fcnStep1);
  minuit.mnparm(0, "energy", .1, .1, 0, 0, errflag);
  arglist[0] = 500;           // iterations number
  arglist[1] = 1.;
  int fd = 0;
  if(Verbose<2) {
    fd = dup(1); close(1);
  }
  minuit.mnexcm("MIGRAD", arglist, 2, errflag);
  if(Verbose<2) {
    dup2(fd, 1); close(fd);
  }
  // minuit->mnexcm("MINOS", arglist, 0, errflag);
  minuit.GetParameter(0, fE, fdE);

  return 1;
}

int THorReconstruct::Step2() {
  int errflag = 0;
  double arglist[10];
  double ddum;

  TMinuit minuit(2);
  minuit.SetPrintLevel(-1);
  minuit.SetFCN(fcnStep2);
  minuit.mnparm(0, "xcore", 0., 100, 0, 0, errflag);
  minuit.mnparm(1, "ycore", 0., 100, 0, 0, errflag);
  arglist[0] = 500;           // iterations number
  arglist[1] = 1.;
  int fd = 0;
  if(Verbose<2) {
    fd = dup(1); close(1);
  }
  minuit.mnexcm("MIGRAD", arglist, 2, errflag);
  if(Verbose<2) {
    dup2(fd, 1); close(fd);
  }
  if (errflag !=0 && errflag != 4) return 0;
  // minuit->mnexcm("MINOS", arglist, 0, errflag);
  minuit.GetParameter(0, DXCore, ddum);
  minuit.GetParameter(1, DYCore, ddum);

  return 1;
}

int THorReconstruct::Step3() {
  int errflag = 0;
  double arglist[10];

  TMinuit minuit(3);
  minuit.SetPrintLevel(-1);
  minuit.SetFCN(fcnStep3);
  minuit.mnparm(0, "energy", fE, .1, 0, 0, errflag);
  minuit.mnparm(1, "xcore", DXCore, 100, 0, 0, errflag);
  minuit.mnparm(2, "ycore", DYCore, 100, 0, 0, errflag);
  arglist[0] = 500;           // iterations number
  arglist[1] = 1.;
  int fd = 0;
  if(Verbose<2) {
    fd = dup(1); close(1);
  }
  minuit.mnexcm("MIGRAD", arglist, 2, errflag);
  if(Verbose<2) {
    dup2(fd, 1); close(fd);
  }
  if (errflag !=0 && errflag != 4) return 0;
  // Minuit->mnexcm("MINOS", arglist, 0, errflag);
  minuit.GetParameter(0, fE, fdE);
  minuit.GetParameter(1, DXCore, dDXCore);
  minuit.GetParameter(2, DYCore, dDYCore);

  XCore += DXCore;
  YCore += DYCore;
  return 1;
}

static void _limits(double mu, int &min, int &max)
{int sigma = (int) (sqrt(mu) + 0.5), nMu = (int) (mu + 0.5);
 if (sigma < 1) sigma = 1;
 min = nMu - 6*sigma; if (min < 0 ) min = 0;
 max = nMu + 6*sigma;
//ancienne formule OD 
//if (mu < 30.0) {min = 0; max = 50; return;}
//min = 0; max = 2*nMu;
}

double THorReconstruct::FCNStep1(double *par) {
  double likefunc = 0.;       // the -Log(L) function
  int nMax, nMin;
  double dx, dy, sig_tank, nmu_th;

  unsigned int size = fErEvent->fSelectedStations.size();
  for (unsigned int is = 0; is < size; is++) {
    // data
    sig_tank = fErEvent->fSelectedStations[is]->fSigInVEM;
    dx = fErEvent->fSelectedStations[is]->fX - XCore;
    dy = fErEvent->fSelectedStations[is]->fY - YCore;
    // extract or compute theoritical expectations (MC)
    nmu_th = par[0] * MuParamTabP->NMuAtGround(dx, dy);
    if (nmu_th == 0.)
      continue;
    // ready
    double prob_stat = 0.;
    _limits(nmu_th, nMin, nMax);
    for (int k = nMin; k < nMax; k++) {
      prob_stat +=
        ProbPoisson(k + 1,
                    nmu_th) * MuParamSigP->GetSignalProb(Theta,
                                                         sig_tank,
                                                         k);
    }
    if (prob_stat == 0.)
      prob_stat = 1e-323;
    likefunc += -log(prob_stat);
  }
  size = fErEvent->fSilentStations.size();
  for (unsigned int is = 0; is < size; is++) {
    sig_tank = 0.;
    dx = fErEvent->fSilentStations[is]->fX - XCore;
    dy = fErEvent->fSilentStations[is]->fY - YCore;
    nmu_th = par[0] * MuParamTabP->NMuAtGround(dx, dy);
    if (nmu_th == 0)
      continue;
    double prob_stat = 0.;
    prob_stat += ProbPoisson(0, nmu_th);
    for (int k = 0; k <= 3; k++)
      prob_stat +=
        ProbPoisson(k + 1,
                    nmu_th) * MuParamSigP->GetSignalProb(Theta,
                                                         sig_tank,
                                                         k);
    if (prob_stat == 0.)
      prob_stat = 1e-323;
    likefunc += -log(prob_stat);

  }

  return likefunc;
}

double THorReconstruct::FCNStep2(double *par) {
  double likefunc = 0.;       // the -Log(L) function
  int nMax, nMin;
  double dx, dy, sig_tank, nmu_th;

  unsigned int size = fErEvent->fSelectedStations.size();
  for (unsigned int is = 0; is < size; is++) {
    // data
    sig_tank = fErEvent->fSelectedStations[is]->fSigInVEM;
    dx = fErEvent->fSelectedStations[is]->fX - XCore - par[0];
    dy = fErEvent->fSelectedStations[is]->fY - YCore - par[1];
    // extract or compute theoritical expectations (MC)
    nmu_th = fE * MuParamTabP->NMuAtGround(dx, dy);
    if (nmu_th == 0.)
      continue;
    // ready
    double prob_stat = 0.;
    _limits(nmu_th, nMin, nMax);
    for (int k = nMin; k <= nMax; k++) {
      prob_stat +=
        ProbPoisson(k + 1,
                    nmu_th) * MuParamSigP->GetSignalProb(Theta,
                                                         sig_tank,
                                                         k);
    }
    if (prob_stat == 0.)
      prob_stat = 1e-323;
    likefunc += -log(prob_stat);
  }
  size = fErEvent->fSilentStations.size();
  for (unsigned int is = 0; is < size; is++) {
    sig_tank = 0.;
    dx = fErEvent->fSilentStations[is]->fX - XCore - par[0];
    dy = fErEvent->fSilentStations[is]->fY - YCore - par[1];
    nmu_th = fE * MuParamTabP->NMuAtGround(dx, dy);
    if (nmu_th == 0)
      continue;
    double prob_stat = 0.;
    prob_stat += ProbPoisson(0, nmu_th);
    for (int k = 0; k <= 3; k++)
      prob_stat +=
        ProbPoisson(k + 1,
                    nmu_th) * MuParamSigP->GetSignalProb(Theta,
                                                         sig_tank,
                                                         k);
    if (prob_stat == 0.)
      prob_stat = 1e-323;
    likefunc += -log(prob_stat);

  }

  return likefunc;

}

double THorReconstruct::FCNStep3(double *par) {
  double likefunc = 0.;       // the -Log(L) function
  int nMax, nMin;
  double dx, dy, sig_tank, nmu_th;

  unsigned int size = fErEvent->fSelectedStations.size();
  for (unsigned int is = 0; is < size; is++) {
    // data
    sig_tank = fErEvent->fSelectedStations[is]->fSigInVEM;
    dx = fErEvent->fSelectedStations[is]->fX - XCore - par[1];
    dy = fErEvent->fSelectedStations[is]->fY - YCore - par[2];
    // extract or compute theoritical expectations (MC)
    nmu_th = par[0] * MuParamTabP->NMuAtGround(dx, dy);
    if (nmu_th == 0.)
      continue;
    // ready
    double prob_stat = 0.;
    _limits(nmu_th, nMin, nMax);
    for (int k = nMin; k <= nMax; k++) {
      prob_stat +=
        ProbPoisson(k + 1,
                    nmu_th) * MuParamSigP->GetSignalProb(Theta,
                                                         sig_tank,
                                                         k);
    }
    if (prob_stat == 0.)
      prob_stat = 1e-323;
    likefunc += -log(prob_stat);
  }
  size = fErEvent->fSilentStations.size();
  for (unsigned int is = 0; is < size; is++) {
    sig_tank = 0.;
    dx = fErEvent->fSilentStations[is]->fX - XCore - par[1];
    dy = fErEvent->fSilentStations[is]->fY - YCore - par[2];
    nmu_th = par[0] * MuParamTabP->NMuAtGround(dx, dy);
    if (nmu_th == 0)
      continue;
    double prob_stat = 0.;
    prob_stat += ProbPoisson(0, nmu_th);
    for (int k = 0; k <= 3; k++)
      prob_stat +=
        ProbPoisson(k + 1,
                    nmu_th) * MuParamSigP->GetSignalProb(Theta,
                                                         sig_tank,
                                                         k);
    if (prob_stat == 0.)
      prob_stat = 1e-323;
    likefunc += -log(prob_stat);

  }

  return likefunc;

}
