#include <iostream>
#include <signal.h>
#include "Er.h"
#include "TSTSky.h"
#include "SdRecLpnhe.h"
#include "TCdFReconstruct.h"
#include "STCconstants.h"
#include <IoAuger.h>
#include <TMath.h>

int main (int argc, char *argv[]) {
  if (argc==1) {
    cout << "Usage: " << argv[0] << " <files>" << endl;
    cout << "   will produce a verbose output of the content of each file and reconstruct the Sd part." << endl;
    return 1;
  }
  int n1 = 0, n2 = 0 , n3 = 0, n4 = 0;
  double coslim=cos(15*kCdasUtil::DEG2RAD);
  AugerIoSd input(argc-1, argv+1); //creates the object holding the files
  //IoSd output("bad.root","w");

  for (EventPos pos=input.FirstEvent(); pos<input.LastEvent(); pos=input.NextEvent()) {
    TErEvent event(pos); //creates the event object
    //by default the signals on the good stations are calculated.
    // cerr<<"event "<<event.AugerId<<" date "<<event.UTCDate<<" trigger:";
    //cerr<<event.Trigger.Algo<<" number of stations: "<<event.Trigger.NumberOfStation;
    // cerr <<" position: "<< pos;
    // cerr<<endl;

#define CDF
#ifdef CDF
    TCdFReconstruct cdfrec;
    int verbosity = 1;
    // apply event selection
    event.OfficialSelection();
    cerr << "Official selection : "
    << event.fSelectedStations.size()
    << " (active) "
    << event.fSilentStations.size()
    << " (silent)"
    << endl;
    if (event.fSelectedStations.size()<5) continue;

    event.OfficialSelection();

    double xm, ym, a, b, tilt;
    string method_orientation="distances";
    string method_area="smax";
    event.FitEllipse(xm,ym,a,b,tilt,method_orientation,method_area);
    cout << "ellipse=" << event.fSelectedStations.size() << " " << M_PI*a*b << " " << b/a << endl;

    //    cdfrec.SetLDF("LOG");
    //cdfrec.SetLDF("SMSP");
        cdfrec.SetLDF("2EXPO");
     cdfrec.SetPrimary(0); //(0: proton, 2: photon)
    cdfrec.SetVerbosity(verbosity);
    cdfrec.SetEstimated(event.fEstimated);

    //	cdfrec.UseEstimate("Es"); // can be "Es"
    //	cdfrec.UseBetaFree(false);
    cdfrec.UseEstimate("barycenter");
    cdfrec.UseBetaFree(false);

    event.Reconstruct(&cdfrec);
    if( verbosity > -1 ) {
      cout << "-------------------- " << event.AugerId << " --------------------" << endl;
      cout << cdfrec << endl;
      cout << "s1000ldf " << event.Id << " " << cdfrec.fSRef << endl;
      for( unsigned int i=0; i<event.fSelectedStations.size();i++ )
        cout << event.fSelectedStations[i]->Id << " " << event.fSelectedStations[i]->fDist << endl;
    }
#else
    // Use old RecSd reconstruction

    // apply event selection
    if (!event.IsICRC2005()) continue;
    ((n1 = event.fSelectedStations.size()) > 3) ? n4++ : n3++;
    if (pos % 100 == 0) {
      cerr<<"event "<<event.AugerId<<" date "<<event.UTCDate;
      cerr << " Official selection : " << event.fSelectedStations.size() << " (active) " << event.fSilentStations.size() << " (silent)" <<endl;
    }
    //    continue;
    int n1 = event.fSelectedStations.size();
    theLpnheRec().SetDefault();
    int res = theLpnheRec().GeometricalRec(eGeomOptimal);
    if (res) {
      cout << n1 << " " << event.fSelectedStations.size() << " " << theLpnheRec().fChi2 << " " << theLpnheRec().ndof << " ";
      cout << event.fRec.Theta("deg") << " " << event.fRec.dTheta("deg") << " " << event.fRec.Phi("deg") << " " << event.fRec.dPhi("deg") << " ";
      cout << sqrt(sqr(event.fRec.dTheta("deg")) + sqr(sin(event.fRec.Theta())*event.fRec.dPhi("deg"))) << endl;
    } else cout << event.AugerId << " Failed...." << endl;
    //    if (TMath::Prob(theLpnheRec().fChi2, theLpnheRec().ndof) < 0.01) output.Write(event);
    continue;
    theLpnheRec().SetGlobal(false);
    theLpnheRec().SetRec(ePlane,eLog);
    event.Reconstruct(&theLpnheRec());
    TShowerParams plan(event.fRec);

    event.OfficialSelection();
    theLpnheRec().SetDefault();
    theLpnheRec().SetVerbosity(1);
    event.Reconstruct(&theLpnheRec());
    TShowerParams basic(event.fRec);

    // reset event to OfficialSelection in case reconstruction removed some tanks
    event.OfficialSelection();
    theLpnheRec().SetRec(eCurve, eLogXmax);
    event.Reconstruct(&theLpnheRec());
    TShowerParams xmax(event.fRec);

    // Comparing theta
    cout << event.fEstimated.Theta("deg") << " " << basic.Theta("deg") << " "
    << xmax.Theta("deg") << " " << plan.Theta("deg") << " " << endl;
#endif

    double dcos=event.fEstimated.fU*event.fRec.fU+
                event.fEstimated.fV*event.fRec.fV+
                sqrt(1-event.fEstimated.fU*event.fEstimated.fU-event.fEstimated.fV*event.fEstimated.fV)*
                sqrt(1-event.fRec.fU*event.fRec.fU-event.fRec.fV*event.fRec.fV);
    double dgrnd=sqrt(pow(event.fEstimated.fXCore-event.fRec.fXCore,2)+
                      pow(event.fEstimated.fYCore-event.fRec.fYCore,2));
    if ( (!(event.fRec.Theta()>=0 && event.fRec.Theta()<=M_PI)) ||
         (!(event.fRec.fE>0)) ||
         (!(event.fRec.fSRef>0)))
      // bad event
      cout <<event.AugerId<<" -1 "
      << event.Stations.size() << " "
      << event.fSelectedStations.size() << " "
      << event.fEstimated.Theta("deg") << " "
      << acos(dcos)*kCdasUtil::RAD2DEG << " "
      << dgrnd << endl;
    else {
      int ok=1;
      if ( (dcos<coslim) || (dgrnd>800) )
        ok=0;
      cout<<event.AugerId<< " "
      << event.fSelectedStations.size() << " "
      << event.fRec.LocalTheta("deg") << " "
      << event.fRec.LocalPhi("deg") << " "
      << event.fRec.fE << " "
      << event.fRec.L() << " "
      << event.fRec.B() << " "
      << event.UTCTime << " ";
      printf("%09.0f ",event.fRec.fT0);
      cout << event.fRec.fXCore << " "
      << event.fRec.fYCore << " "
      << event.fRec.fSRef << " "
      << event.fRec.Ra() << " "
      << event.fRec.Dec() << " "
      << event.fRec.Theta("deg") << " "
      << event.fRec.Phi("deg") << " "
      << event.trigger().IsFd() << " "
      << ok << " "
      << event.IsT5() << " "
      << event.Id << " " << endl;
    }
    //    break;
  }
  cout<< " N3 = " << n3 << " N4 = " << n4 <<endl;
  //  output.Close();
  return 0;
}
