#include <algorithm>
#include "Es.h"
#include "TCdasUtil.h"
#include "T2Life.h"
#include <math.h>

namespace kEs {
  vector < unsigned int > BadStations;
  bool IsInit=false;
 
}

unsigned int kEs::Crown(double x1, double x2, double y1, double y2, bool infilldist) {
  double dist = 1500; //tank distance, hexagone side
  double apoth = 1300; // apothem
  if (infilldist) {
    dist /= 2.; apoth /= 2.;
  } 
  double apoth2 = 2*apoth; //2600 ou 1300 (infill)
  double d1 = fabs(y1 - y2) / apoth2 + fabs(x1 - x2) / dist;
  double d2 = fabs(y1 - y2) / apoth;
  double max = (d2 > d1 ? d2 : d1);
  int ret=(int)(floor(max));
  const double rounding = 0.2;
  if (max-ret>rounding)
    ret++; // rounding at 0.2 crowns, ie 300 m
  
  return ret;
}

const char *EsVersion() {
  return ESVERSION;
}

// TEsEvent stuff
TEsEvent::TEsEvent(EventPos pos):t4level(0),
				 fIs5T5Pos(0),
				 fIs6T5(0),
				 TEcEvent(pos) {
  Init();
 
}

TEsEvent::TEsEvent(EventId id):t4level(0),
			       fIs5T5Pos(0),
			       fIs6T5(0),
			       TEcEvent(id) {
  Init();
  
}

TEsEvent::TEsEvent():t4level(0),
		     fIs5T5Pos(0),
		     fIs6T5(0),
		     TEcEvent() {
  Init();
 
}

TEsEvent::~TEsEvent() {}

TEsEvent::TEsEvent(const IoSdEvent & ev):t4level(0),
					 fIs5T5Pos(0),
					 fIs6T5(0),
					 TEcEvent(ev) {
  Init();
  
}

TEsEvent::TEsEvent(const TEcEvent &ev):t4level(0),
				       fIs5T5Pos(0),
				       fIs6T5(0),
				       TEcEvent(ev) {
  Init();
 
}

#define COPYVECTOR(a) \
  {for(unsigned int i=0;i<ev.a.size();i++) {\
    for(unsigned int j=0;j<ev.fCalibStations.size();j++) {\
      if (ev.a[i]==&ev.fCalibStations[j]) {\
        tmp.push_back(j);\
        break;\
      }\
    }\
  }\
  a.clear();\
  for (unsigned int i=0;i<tmp.size();i++)\
    a.push_back(&fCalibStations[tmp[i]]);\
  tmp.clear();}

TEsEvent::TEsEvent(const TEsEvent & ev):t4level(0),
					fIs5T5Pos(0),
					fIs6T5(0),
					TEcEvent(ev) {

  vector < int >tmp;
  tmp.reserve(fCalibStations.size());
  
  t4level = ev.t4level;
  COPYVECTOR(fSelectedStations);
  COPYVECTOR(fRandomStations);
  COPYVECTOR(fSilentStations);
  COPYVECTOR(fBadStations);
}

TEsEvent & TEsEvent::operator =(const TEsEvent & ev) {
  cerr << " Cannot = event " << ev.Id << endl;
  cerr << " = operator on TEsEvent not defined!!!! " << endl;
  cerr << " If you code it, please send the code to us :) " << endl;
  cerr << " Else use copy constructor " << endl;
  return *this;
}


void TEsEvent::Init() {
  if (!kEs::IsInit) {
    kEs::UseEA(0);
    kEs::UseRadio(0);
    kEs::UseInfillStations(0);
    kEs::UseBatata(0);
    kEs::UseDoublets(0);
     UseTestTanks(0);
     }
  memset(&fEstimated, 0, sizeof(TShowerParams));
  fEstimated.fGPSSecond=RefSecond();
  fSelectedStations.clear();
  fRandomStations.clear();
  fSilentStations.clear();
  fBadStations.clear();
  fSelectedStations.reserve(fCalibStations.size());
  fSilentStations.reserve(fCalibStations.size());
  fBadStations.reserve(fCalibStations.size());
  vector<unsigned int> & StationsT2 = StationsStatusFromT2Files(Trigger.Second);
  if (StationsT2.size() > 0)
    for (unsigned int i = 0; i < fCalibStations.size(); i++) {
      if (fCalibStations[i].Error == eNoError && !StationsT2[fCalibStations[i].Id]) fCalibStations[i].Error = eT3Lost;
      if (fCalibStations[i].Error == eDataLost && StationsT2[fCalibStations[i].Id]) fCalibStations[i].Error = eT3NotFound;
    }
  for (unsigned int i = 0; i < fCalibStations.size(); i++) {
    if (!fCalibStations[i].Error)
      fSelectedStations.push_back(&(fCalibStations[i]));
    else if (fCalibStations[i].Error == IoSdEvent::eT3NotFound
	     && fCalibStations[i].Trigger.Window != 0)
      fSilentStations.push_back(&(fCalibStations[i]));
    else
      fBadStations.push_back(&(fCalibStations[i]));
  }
  
  
  if (UseCheckT2CommsCrisis && IsCommsCrisis) CheckT2CommsCrisis();
  if (UseCheckT2 && !IsCommsCrisis) CheckT2();
  if (QualityFlag && UseQualityCuts) QualityCuts();
   t4level = -1;
}


// Ct bug, 20% 3TOT lost. Introduced 18/08 at 18h45.
// Main bug was corrected 25/11 at 21h00. Event tagging bug in Ct was
// corrected Dec 20 23h00, but it doesn't affect event selection.
#define D18_08_2004 1092787200
#define D26_11_2004 1101427200
// PLD bug discard data from March 27 to April 6
#define D27_03_2005 1111881600
#define D07_04_2005 1112832000
// CDAS/Comms bug discard data from April 22 to May 3
#define D22_04_2005 1114128000
#define D03_05_2005 1115078400

bool TEsEvent::IsICRC2005(){
  static unsigned int TriggerBugStart = D18_08_2004;
  static unsigned int TriggerBugStop  = D26_11_2004;
  static unsigned int PLDBugStart = D27_03_2005;
  static unsigned int PLDBugStop  = D07_04_2005;
  static unsigned int PmBugStart = D22_04_2005;
  static unsigned int PmBugStop  = D03_05_2005;
  unsigned int s=kCdasUtil::UNIXsecond(RefSecond());
  if (s > PLDBugStart && s < PLDBugStop) return false;
  if (s > PmBugStart && s < PmBugStop) return false;
  if (s > TriggerBugStart && s < TriggerBugStop) return ((IsT4() & kEs::T4_4C1) && (IsT5() > 0));
  return (IsT5() > 0) ;
}

static bool _SameSide(double p1[2], double p2[2], double a[2], double b[2]) {
  double z1, z2;
  z1 = (b[0]-a[0])*(p1[1]-a[1]) - (b[1]-a[1])*(p1[0]-a[0]);
  z2 = (b[0]-a[0])*(p2[1]-a[1]) - (b[1]-a[1])*(p2[0]-a[0]);
  return (z1*z2 >= 0);
}

static bool _Inside(double p[2],  double a[2], double b[2], double c[2]) {
  return (_SameSide(p, a, b, c) && _SameSide(p, b, c, a) && _SameSide(p, c, a, b));
}

static bool _InsideTheTriangle(double xc, double yc, vector<double> &x, vector<double> &y, int i1, int i2, int i3) {
  double p[2]; p[0] = xc; p[1] = yc;
  double a[2]; a[0] = x[i1]; a[1] = y[i1];
  double b[2]; b[0] = x[i2]; b[1] = y[i2];
  double c[2]; c[0] = x[i3]; c[1] = y[i3];
  return _Inside(p, a, b, c);
}

#define GOOD_CORE_RCUT (1700*1700)
bool TEsEvent::OldIsICRC2005Posterior(double xc, double yc) {
  vector<double> x, y, dist; double d,d1,d2,d3;
  for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
    d = pow(xc-fSelectedStations[i]->fX,2) + pow(yc-fSelectedStations[i]->fY,2);
    if (d < GOOD_CORE_RCUT) {
      x.push_back(fSelectedStations[i]->fX); y.push_back(fSelectedStations[i]->fY); dist.push_back(sqrt(d));;
    }
  }
  for (unsigned int i = 0; i < fSilentStations.size(); i++) {
    d = pow(xc-fSilentStations[i]->fX,2) + pow(yc-fSilentStations[i]->fY,2);
    if (d < GOOD_CORE_RCUT) {
      x.push_back(fSilentStations[i]->fX); y.push_back(fSilentStations[i]->fY); dist.push_back(sqrt(d));
    }
  }
  for (unsigned int i = 0; i < fRandomStations.size(); i++) {
    d = pow(xc-fRandomStations[i]->fX,2) + pow(yc-fRandomStations[i]->fY,2);
    if (d < GOOD_CORE_RCUT) {
      x.push_back(fRandomStations[i]->fX); y.push_back(fRandomStations[i]->fY); dist.push_back(sqrt(d));
    }
  }
  if (dist.size() < 3) return false;
  int i1 = 0; d1 = 1700;
  for (unsigned int i = 0; i < dist.size(); i++) if (dist[i] < d1) {
      d1 = dist[i]; i1 = i;
    }
  int i2 = 0; d2 = 1700;
  for (unsigned int i = 0; i < dist.size(); i++) if (dist[i] < d2 && dist[i] > d1) {
      d2 = dist[i]; i2 = i;
    }
  int i3 = 0; d3 = 1700;
  for (unsigned int i = 0; i < dist.size(); i++) if (dist[i] < d3 && dist[i] > d2) {
      d3 = dist[i]; i3 = i;
    }
  vector<double> xt, yt;
  // check if the 3 station are on an equilateral triangle
  if ((x[i1]- x[i2])*(x[i1]-x[i2]) + (y[i1]-y[i2])*(y[i1]-y[i2]) > GOOD_CORE_RCUT) return false;
  if ((x[i2]- x[i3])*(x[i2]-x[i3]) + (y[i2]-y[i3])*(y[i2]-y[i3]) > GOOD_CORE_RCUT) return false;
  if ((x[i3]- x[i1])*(x[i3]-x[i1]) + (y[i3]-y[i1])*(y[i3]-y[i1]) > GOOD_CORE_RCUT) return false;
  // now check if inside the triangle
  return _InsideTheTriangle(xc, yc, x, y, i1, i2, i3);
}

bool TEsEvent::IsICRC2005Posterior(double xc, double yc) {
  vector<double> x, y;
  x.reserve(10);
  y.reserve(10);
  fIs5T5Pos = false;
  // look for tanks within one crown of core
  for (unsigned int i = 0; i < fSelectedStations.size(); i++)
    if (kEs::Crown(xc,fSelectedStations[i]->fX,yc,fSelectedStations[i]->fY,UseInfillRec)<=1) {
      x.push_back(fSelectedStations[i]->fX);
      y.push_back(fSelectedStations[i]->fY);
    }
  for (unsigned int i = 0; i < fSilentStations.size(); i++)
    if (kEs::Crown(xc,fSilentStations[i]->fX,yc,fSilentStations[i]->fY,UseInfillRec)<=1) {
      x.push_back(fSilentStations[i]->fX);
      y.push_back(fSilentStations[i]->fY);
    }
  for (unsigned int i = 0; i < fRandomStations.size(); i++)
    if (kEs::Crown(xc,fRandomStations[i]->fX,yc,fRandomStations[i]->fY,UseInfillRec)<=1) {
      x.push_back(fRandomStations[i]->fX);
      y.push_back(fRandomStations[i]->fY);
    }
  if (x.size()<3) return false;
  // search for equilateral triangles (1 crown for each pair)
  for (unsigned int i = 0; i < x.size() ; i++)
    for (unsigned int j = 0; j < x.size() ; j++) {
      if (i==j) continue;
      if (kEs::Crown(x[i],x[j],y[i],y[j],UseInfillRec)>1) continue;
      for (unsigned int k = 0; k < x.size() ; k++) {
        if (k==i || k==j) continue;
        if (kEs::Crown(x[i],x[k],y[i],y[k],UseInfillRec)>1) continue;
        if (kEs::Crown(x[k],x[j],y[k],y[j],UseInfillRec)>1) continue;
        // I have an equilateral triangle. Check if core inside
        if (_InsideTheTriangle(xc, yc, x, y, i, j, k)){ 
	  fIs5T5Pos = true;
	  return true;
	}
	}
    }
  return false;
}

bool TEsEvent::OfficialSelection() {
  Init();
  IsT4();
  RemoveDuplicatedStations();
 RemoveXbee();
 BottomUpSelection();
   if (UseInfillRec)
    RemoveLonelyStations(0.5*kEs::D1_CUT,0.5*kEs::D2_CUT);
  else
    RemoveLonelyStations();  
  t4level = -1;
  return (T4Criteria());
}

int TEsEvent::IsT4() {
  if (t4level != -1)
    return t4level;
  RemoveExternalTriggers();
  RemoveLightning();
  RemoveBadStations();
  RemoveBadAltitude();
  return T4Criteria();
}

int TEsEvent::T4Criteria() {
  if (t4level != -1)
    return t4level;
  double IST4DMIN=2000.;
  double IST4DMAX=2800.;
  if (UseInfillRec){
    IST4DMIN=1000.;
    IST4DMAX=1400.;
  }
#define TIMETOLERANCE 000.
#define CROWN(i,j, k) kEs::Crown(fSelectedStations[i]->fX,fSelectedStations[j]->fX,fSelectedStations[i]->fY,fSelectedStations[j]->fY,k)

#define IST4DTIME(i,j) fabs(1e9*((double)fSelectedStations[i]->StartTime()-(double)fSelectedStations[j]->StartTime()))
#define IST4DIST(i,j) sqrt(pow(fSelectedStations[i]->fY-fSelectedStations[j]->fY,2)+pow(fSelectedStations[i]->fX-fSelectedStations[j]->fX,2))
#define IST4ISBADSTATION(i) (fSelectedStations[i]->Trigger.Window!=0&&(!Trigger.IsFd()))
#define IST4STATID(i) fSelectedStations[i]->Id
#define IST4DTIMETOLERANCEBAD(i,j) ((IST4DTIME(i,j)-TIMETOLERANCE-IST4DIST(i,j)/kCdasUtil::CSPEED)>0)

  bool IR=UseInfillRec;

  t4level = 0;
  for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
    if (!fSelectedStations[i]->IsTOT())
      continue;
    if (IST4ISBADSTATION(i))
      continue;
    for (unsigned int j = 0; j < fSelectedStations.size(); j++) {
      if (j == i)
        continue;
      if (!fSelectedStations[j]->IsTOT())
        continue;
      if (IST4ISBADSTATION(j))
        continue;
      if (CROWN(i, j, IR) > 1)
        continue;
      if (IST4DTIMETOLERANCEBAD(i, j))
        continue;
      if (IST4DIST(i,j) < 100) //do not take doublets
        continue;
      for (unsigned int k = 0; k < fSelectedStations.size(); k++) {
        if (k == j || k == i)
          continue;
        if (!fSelectedStations[k]->IsTOT())
          continue;
        if (IST4ISBADSTATION(k))
          continue;
        if (CROWN(i, k, IR) > 1)
          continue;
        if (IST4DTIMETOLERANCEBAD(j, k) || IST4DTIMETOLERANCEBAD(i, k))
          continue;
	if (IST4DIST(i, k) < 100 || IST4DIST(j, k) < 100) //do not take doublets
	  continue;
        // Found a 3TOT
        k = j = i = fSelectedStations.size();
        t4level |= kEs::T4_TOT;
      }
    }
  }
  for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
    if (IST4ISBADSTATION(i))
      continue;
    for (unsigned int j = 0; j < fSelectedStations.size(); j++) {
      if (j == i)
        continue;
      if (IST4ISBADSTATION(j))
        continue;
      if (CROWN(i, j, IR) > 1)
        continue;
      if (IST4DTIMETOLERANCEBAD(i, j))
        continue;
      if (IST4DIST(i,j) < 100) //do not take doublets
        continue;
      for (unsigned int k = 0; k < fSelectedStations.size(); k++) {
        if (k == i || k == j)
          continue;
        if (IST4ISBADSTATION(k))
          continue;
        if (CROWN(i, k, IR) > 1)
          continue;
        if (IST4DTIMETOLERANCEBAD(j, k))
          continue;
        if (IST4DTIMETOLERANCEBAD(i, k))
          continue;
        if (IST4DIST(i, k) < 100 || IST4DIST(j, k) < 100) //do not take doublets
	  continue;
        for (unsigned int l = 0; l < fSelectedStations.size(); l++) {
          if (l == i || l == j || l == k)
            continue;
          if (IST4ISBADSTATION(l))
            continue;
          if (CROWN(i, l, IR) > 1)
            continue;
          if (IST4DTIMETOLERANCEBAD(i, l))
            continue;
          if (IST4DTIMETOLERANCEBAD(l, j))
            continue;
          if (IST4DTIMETOLERANCEBAD(l, k))
            continue;
	  if (IST4DIST(i, l) < 100 || IST4DIST(j, l) < 100 || IST4DIST(k, l) < 100) //do not take doublets
	    continue;
          // Found a 4C1
          l = k = j = i = fSelectedStations.size();
          t4level |= kEs::T4_4C1;
        }
      }
    }
  }
  if (t4level && Trigger.IsFd())
    t4level |= kEs::T4_FD;  // If it is also FD
  return t4level;
}

#define EDSELECT_IS_4C1(i) (find(StList.begin(),StList.end(),i)!=StList.end())

/*----------------------------------------------------------*/
void TEsEvent::BottomUpSelection(double rCutActive, double rCutSilent)
/*----------------------------------------------------------*/
{
  vector < unsigned int >StList;
  StList.clear();
  StList.reserve(4000);
  int tr0 = 0, tr1 = 0, tr2 = 0;
  float sig = 0;
  double ps0 = 0, ps1 = 0, ps2 = 0, tps1 = 0, tps2 = 0, tps0 = 0;
  double x1 = 0, x2 = 0, y1 = 0, y2 = 0, x0 = 0, y0 = 0;
  double t0 = 0, t1 = 0, t2 = 0;
  double xCore, yCore, u, v;
  
  double IST4DMIN=2000.;
  double IST4DMAX=2800.;
  if (UseInfillRec){
    IST4DMIN=1000.;
    IST4DMAX=1400.;
  }

  if (!(T4Criteria() & kEs::T4_TOT))  // looking for 4C1 if ! 3TOT
    for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
      if (IST4ISBADSTATION(i))
        continue;
      for (unsigned int j = 0; j < fSelectedStations.size(); j++) {
        if (j == i)
          continue;
        if (IST4ISBADSTATION(j))
          continue;
        if (IST4DIST(i, j) > IST4DMIN)
          continue;
        if (IST4DTIMETOLERANCEBAD(i,j))
          continue;
	if (IST4DIST(i,j) < 100) //do not take doublets
	  continue;
        for (unsigned int k = 0; k < fSelectedStations.size(); k++) {
          if (k == i || k == j)
            continue;
          if (IST4ISBADSTATION(k))
            continue;
          if (IST4DIST(i, k) > IST4DMIN)
            continue;
          if (IST4DTIMETOLERANCEBAD(j, k))
            continue;
          if (IST4DTIMETOLERANCEBAD(i, k))
            continue;
          if (IST4DIST(i, k) < 100 || IST4DIST(j, k) < 100) //do not take doublets
	    continue;
        for (unsigned int l = 0; l < fSelectedStations.size();
               l++) {
            if (l == i || l == j || l == k)
              continue;
            if (IST4ISBADSTATION(l))
              continue;
            if (IST4DIST(i, l) > IST4DMIN)
              continue;
            if (IST4DTIMETOLERANCEBAD(i, l))
              continue;
            if (IST4DTIMETOLERANCEBAD(l, j))
              continue;
            if (IST4DTIMETOLERANCEBAD(l, k))
              continue;
	    if (IST4DIST(i, l) < 100 || IST4DIST(j, l) < 100 || IST4DIST(k, l) < 100) //do not take doublets
            // Found a 4C1
            StList.push_back(i);
            StList.push_back(j);
            StList.push_back(k);
            StList.push_back(l);
          }
        }
      }
    }
  for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
    if (IST4ISBADSTATION(i))
      continue;
    if (!fSelectedStations[i]->IsTOT() && !EDSELECT_IS_4C1(i))
      continue;
    for (unsigned int j = 0; j < fSelectedStations.size(); j++) {
      if (j == i)
        continue;
      if (IST4ISBADSTATION(j))
        continue;
      if (IST4DIST(i, j) > IST4DMIN)
        continue;
      if (IST4DTIMETOLERANCEBAD(i, j))
        continue;
      if (!fSelectedStations[j]->IsTOT() && !EDSELECT_IS_4C1(j))
        continue;
      if (IST4DIST(i,j) < 100) //do not take doublets
	continue;
      for (unsigned int k = 0; k < fSelectedStations.size(); k++) {
        if (k == j || k == i)
          continue;
        if (IST4ISBADSTATION(k))
          continue;
        if (IST4DIST(i, k) > IST4DMIN)
          continue;
        if (IST4DIST(j, k) > IST4DMAX)
          continue;
        if (IST4DTIMETOLERANCEBAD(j,k) || IST4DTIMETOLERANCEBAD(i,k))
          continue;
        if (!fSelectedStations[k]->IsTOT() && !EDSELECT_IS_4C1(k))
          continue;
	if (IST4DIST(i, k) < 100 || IST4DIST(j, k) < 100) //do not take doublets
	  continue;
        tps0 = fSelectedStations[i]->SigInVEM();
        tps1 = fSelectedStations[j]->SigInVEM();
        tps2 = fSelectedStations[k]->SigInVEM();
        if (tps0 + tps1 + tps2 > sig) {
          sig = tps0 + tps1 + tps2;
          ps0 = tps0;
          ps1 = tps1;
          ps2 = tps2;
          x0 = fSelectedStations[i]->fX;
          y0 = fSelectedStations[i]->fY;
          t0 = 1e9 * fSelectedStations[i]->StartTime();
          x1 = fSelectedStations[j]->fX;
          y1 = fSelectedStations[j]->fY;
          t1 = 1e9 * fSelectedStations[j]->StartTime();
          x2 = fSelectedStations[k]->fX;
          y2 = fSelectedStations[k]->fY;
          t2 = 1e9 * fSelectedStations[k]->StartTime();
          tr0 = i;
          tr1 = j;
          tr2 = k;
        }
      }
    }
  }

  if (sig > 0) {
    xCore = (x0 * ps0 + x1 * ps1 + x2 * ps2) / (ps0 + ps1 + ps2) - x0;
    yCore = (y0 * ps0 + y1 * ps1 + y2 * ps2) / (ps0 + ps1 + ps2) - y0;
    x1 = x1 - x0;
    y1 = y1 - y0;
    x2 = x2 - x0;
    y2 = y2 - y0;
    t1 = t1 - t0;
    t2 = t2 - t0;
    u = kCdasUtil::CSPEED * ((t1 * y2 - t2 * y1) / (x2 * y1 - x1 * y2));
    v = kCdasUtil::CSPEED * ((t1 * x2 - t2 * x1) / (x1 * y2 - x2 * y1));
    fSelectedStations[tr0]->Trigger.Type |= 0x400;
    fSelectedStations[tr1]->Trigger.Type |= 0x400;
    fSelectedStations[tr2]->Trigger.Type |= 0x400;
    if (rCutSilent > 0) {
      for (int k = 0; k < (int) fSilentStations.size(); k++) {
        x1 = fSilentStations[k]->fX - x0;
        y1 = fSilentStations[k]->fY - y0;
        x1 -= xCore;
        y1 -= yCore;
        double dr = sqrt(x1 * x1 + y1 * y1 -
                         (u * x1 + v * y1) * (u * x1 + v * y1));
        if (dr > rCutSilent) {  // removing station
          fBadStations.push_back(fSilentStations[k]);
          fSilentStations.erase(fSilentStations.begin() + k);
          k--;
        }
      }
    }
    double tmin,tmax; 
    if (UseInfillRec==1) {tmin=kEs::TMIN_CUT_INF;tmax=kEs::TMAX_CUT_INF;}
    else {tmin=kEs::TMIN_CUT_REG;tmax=kEs::TMAX_CUT_REG;}
    
    for (int k = 0; k < (int) fSelectedStations.size(); k++) {
      x1 = fSelectedStations[k]->fX - x0;
      y1 = fSelectedStations[k]->fY - y0;
      x1 -= xCore;
      y1 -= yCore;
      double dr = sqrt(x1 * x1 + y1 * y1 -
                       (u * x1 + v * y1) * (u * x1 + v * y1));
      t1 = 1e9 * fSelectedStations[k]->StartTime() - t0;
      x1 += xCore;
      y1 += yCore;
      double dt = t1 + (u * x1 + v * y1) / kCdasUtil::CSPEED;
      
      // limits for ambigous
      double Adt_min, Adt_max;
      if (UseInfillRec){ //infill
	Adt_min = -500.; Adt_max = 1400.;}
      else { // normal array (old values Adt_min = -1000.; Adt_max = 2000.;
	Adt_min = -700.; Adt_max = 2600.;}
      
      if ((rCutActive > 0
           && !(dr < rCutActive && dt > tmin && dt < tmax))
          || (rCutActive < 0 && !(dt > tmin && dt < tmax))) {    // removing station
        fRandomStations.push_back(fSelectedStations[k]);
        fSelectedStations.erase(fSelectedStations.begin() + k);
        k--;
      } 
      // no need to use ambiguous stations
      //       else if ((rCutActive > 0
      //                   && !(dr < rCutActive && dt > -1000 && dt < 2000))
      //                  || (rCutActive < 0 && !(dt > -1000 && dt < 2000))) {    // marking as ambiguous
      //         fAmbiguousStations.push_back(fSelectedStations[k]);
      //         fSelectedStations.erase(fSelectedStations.begin() + k);
      //         k--;
      //       }

    }
    xCore += x0;
    yCore += y0;
    fEstimated.fXCore = xCore;
    fEstimated.fYCore = yCore;
    fEstimated.fU = u;
    fEstimated.fV = v;
  } else {
    for (int k = 0; k < (int) fSelectedStations.size(); k++) {
      fRandomStations.push_back(fSelectedStations[k]);
      fSelectedStations.erase(fSelectedStations.begin() + k);
      k--;
    }
  }
}

void TEsEvent::RemoveLightning() {
  int n=0;
  for (int i = 0; i < (int) fSelectedStations.size(); i++)
    if (fSelectedStations[i]->IsLightningEvent()) n++;
  if (n) { //remove whole event
    for (int i = 0; i < (int) fSelectedStations.size(); i++) {
      fRandomStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin() + i);
      i--;
    }
  }
  /*
    for (int i = 0; i < (int) fSelectedStations.size(); i++) {
      if (fSelectedStations[i]->IsLightningEvent()) {
        fRandomStations.push_back(fSelectedStations[i]);
        fSelectedStations.erase(fSelectedStations.begin() + i);
        i--;
      }
    }
  */
}

void TEsEvent::RemoveDuplicatedStations() {
  for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
    for (unsigned int j = 0; j < fSilentStations.size(); j++) {
      if (fSelectedStations[i]->Id == fSilentStations[j]->Id) {
        fBadStations.push_back(fSilentStations[j]);
        fSilentStations.erase(fSilentStations.begin() + j);
        j--;
      }
    }
  }
  // to correct bug in some event where tanks are tripled
 for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
   for (unsigned int j = i+1; j < fSelectedStations.size(); j++) {
     if (fSelectedStations[i]->Id == fSelectedStations[j]->Id) {
        fSelectedStations.erase(fSelectedStations.begin() + j);
        j--;
      }
    }
  }  /** Correction pointed out by Haris **/
  for (unsigned int i = 0; i < fSilentStations.size(); i++) {
    for (unsigned int j = i+1; j < fSilentStations.size(); j++) {
      if (fSilentStations[i]->Id == fSilentStations[j]->Id) {
        fBadStations.push_back(fSilentStations[j]);
        fSilentStations.erase(fSilentStations.begin() + j);
        j--;
      }
    }
  }
  for (unsigned int i = 0; i < fRandomStations.size(); i++) {
    for (unsigned int j = 0; j < fSilentStations.size(); j++) {
      if (fRandomStations[i]->Id == fSilentStations[j]->Id) {
        fBadStations.push_back(fSilentStations[j]);
        fSilentStations.erase(fSilentStations.begin() + j);
        j--;
      }
    }
  }
}
 

void TEsEvent::RemoveExternalTriggers() {
  for (int i = 0; i < (int) fSelectedStations.size(); i++) {
    if (fSelectedStations[i]->Trigger.Type == 16) { // FIXME
      fRandomStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin() + i);
      i--;
    }
  }
}

void TEsEvent::RemoveBadStations()       // They are really suppressed, not considered random
{
  for (int i = 0; i < (int) fSelectedStations.size(); i++)
    for (int j=0; j< (int) kEs::BadStations.size();j++)
      if (fSelectedStations[i]->id() == kEs::BadStations[j]) {
        fBadStations.push_back(fSelectedStations[i]);
        fSelectedStations.erase(fSelectedStations.begin() + i);
        i--; j=kEs::BadStations.size();
      }
  for (int i = 0; i < (int) fSilentStations.size(); i++)
    for (int j=0; j< (int) kEs::BadStations.size();j++)
      if (fSilentStations[i]->id() == kEs::BadStations[j]) {
        fBadStations.push_back(fSilentStations[i]);
        fSilentStations.erase(fSilentStations.begin() + i);
        i--; j=kEs::BadStations.size();
      }
}

void TEsEvent::RemoveBadAltitude()       // They are really suppressed, not considered random
{
  for (int i = 0; i < (int) fSelectedStations.size(); i++) {
    if (fSelectedStations[i]->Altitude < 1000) {
      fBadStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin() + i);
      i--;
    }
  }
  for (int i = 0; i < (int) fSilentStations.size(); i++) {
    if (fSilentStations[i]->Altitude < 1000) {
      fBadStations.push_back(fSilentStations[i]);
      fSilentStations.erase(fSilentStations.begin() + i);
      i--;
    }
  }
}

void kEs::UseEA(int yes) {
  kEs::IsInit=true;
  if (yes) {
    for (int i = 0; i < (int) kEs::BadStations.size(); i++)
      if (kEs::BadStations[i]<=70) {
        kEs::BadStations.erase(kEs::BadStations.begin() + i);
        i--;
      }
  } else
    for (int i=1;i<=70;i++) kEs::BadStations.push_back(i);
}

void kEs::UseRadio(int yes) {
  kEs::IsInit=true;
  if (yes) {
    for (int i = 0; i < (int) kEs::BadStations.size(); i++)
      if (kEs::BadStations[i] == 1351
        || kEs::BadStations[i] == 1325
	|| kEs::BadStations[i] == 97
        || kEs::BadStations[i] == 98
        || kEs::BadStations[i] == 99
	  ) {
        kEs::BadStations.erase(kEs::BadStations.begin() + i);
        i--;
      }
  } else {
    kEs::BadStations.push_back(1325);
    kEs::BadStations.push_back(1351);
    kEs::BadStations.push_back(97);
    kEs::BadStations.push_back(98);
    kEs::BadStations.push_back(99);
 }
}

void kEs::UseInfillStations(int yes) {
  
  vector<unsigned int> infill(infilllist,infilllist+sizeof(infilllist)/sizeof(int));
  // Code
  kEs::IsInit=true;
  if (yes) {
    for (int i = 0; i < (int) kEs::BadStations.size(); i++)
      for (int j = 0; j < (int) infill.size(); j++)
        if (kEs::BadStations[i] == infill[j]) {
	  kEs::BadStations.erase(kEs::BadStations.begin() + i);
          i--;
	}
    
  } else {
    for (int j = 0; j < (int) infill.size(); j++)
      kEs::BadStations.push_back(infill[j]);
  }
}



void kEs::UseBatata(int yes) {
  vector<unsigned int> batata;
  batata.push_back(1575);
  batata.push_back(1618);
  // Code
  kEs::IsInit=true;
  if (yes) {
    for (int i = 0; i < (int) kEs::BadStations.size(); i++)
      for (int j = 0; j < (int) batata.size(); j++)
        if (kEs::BadStations[i] == batata[j]) {
          kEs::BadStations.erase(kEs::BadStations.begin() + i);
          i--;
        }
  } else {
    for (int j = 0; j < (int) batata.size(); j++)
      kEs::BadStations.push_back(batata[j]);
  }
}


void kEs::UseDoublets(int yes) {
  // now we deal with tanks 1243, 1258 and 1255 that were not taken accoutn as doublets
  kEs::IsInit=true;
  if (yes) {
    for (int i = 0; i < (int) kEs::BadStations.size(); i++)
      if (kEs::BadStations[i] == kIoSd::MIRANDA
        || kEs::BadStations[i] == kIoSd::ROUGE
        || kEs::BadStations[i] == kIoSd::NOCHE
        || (kEs::BadStations[i]>70 && kEs::BadStations[i]<=100)
	  || kEs::BadStations[i] ==1847
	  || kEs::BadStations[i] ==1243
	|| kEs::BadStations[i] ==1258
	|| kEs::BadStations[i] ==1255) {
        kEs::BadStations.erase(kEs::BadStations.begin() + i);
        i--;
      }
  } else {
    kEs::BadStations.push_back(kIoSd::MIRANDA);
    kEs::BadStations.push_back(kIoSd::ROUGE);
    kEs::BadStations.push_back(kIoSd::NOCHE);
    for (int i=71;i<=100;i++) kEs::BadStations.push_back(i);
    kEs::BadStations.push_back(1243);
    kEs::BadStations.push_back(1258);
    kEs::BadStations.push_back(1255);
    kEs::BadStations.push_back(1847);
  }
}




//

// Since April 1 2009, some tanks are condidered test tanks and they should be
// removed from the reconstruction and the acceptance calculation
//
#define D01_04_2009 1238544000

void TEsEvent::UseTestTanks(int yes) {
  static unsigned int TestTankStart = D01_04_2009;
  // Built the list of the test tanks that should be removed since 01/04/2009
  unsigned int testtankslist[]={644,648,944,945,948,949,956};
  vector<unsigned int> testtanks(testtankslist,testtankslist+sizeof(testtankslist)/sizeof(int));
  unsigned int s=kCdasUtil::UNIXsecond(RefSecond());
  // Code
  kEs::IsInit=true;
  if (s > TestTankStart){
    if (yes) {
      for (int i = 0; i < (int) kEs::BadStations.size(); i++)
	for (int j = 0; j < (int) testtanks.size(); j++)
	  if (kEs::BadStations[i] == testtanks[j]) {
	    kEs::BadStations.erase(kEs::BadStations.begin() + i);
	    i--;
	  }
    } else {
      for (int j = 0; j < (int) testtanks.size(); j++)
	kEs::BadStations.push_back(testtanks[j]);
    }
  }
}

void TEsEvent::RemoveLonelyStations(double d1Cut, double d2Cut) {
  double dist = 0;
  double x1, y1, x2, y2;
  for (int i = 0; i < (int) fSelectedStations.size(); i++) {
    unsigned int n = 0, n2 = 0;
    x1 = fSelectedStations[i]->fX;
    y1 = fSelectedStations[i]->fY;
    for (int j = 0; j < (int) fSelectedStations.size(); j++) {
      if (j == i)
        continue;
      x2 = fSelectedStations[j]->fX;
      y2 = fSelectedStations[j]->fY;
      dist = sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
      if (dist < d1Cut)
        n++;
      if (dist < d2Cut)
        n2++;
    }
    if (n == 0 || n2 < 2) {
      fRandomStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin() + i);
      i--;
    }
  }
}

unsigned int TEsEvent::IsT5(unsigned int crown) {
  if (!IsT4())
    return false;
  const int stationId = T5Criteria(1); 
  // station ID = 1 means 5T5, larger means 6T5
  
  fIs6T5 = (stationId > 1);
  if (crown == 1 )
    return stationId;
  
  return T5Criteria(crown);
}

static bool CheckNeigh(vector<TCalibStation*> *v, TCalibStation *s,unsigned int ncrown, bool infilldist) {
  static vector<unsigned int> neibourgh;
  if (!v) {
    bool toto = (neibourgh.size() > 4);
    neibourgh.clear();
    neibourgh.reserve(50);
    return toto;
  }
  for (unsigned int i = 0; i < v->size(); i++)
    if ((*v)[i]->Id!=s->Id && kEs::Crown(s->fX,(*v)[i]->fX,s->fY,(*v)[i]->fY, infilldist)<=ncrown){
      neibourgh.push_back((*v)[i]->Id);
    }
  sort(neibourgh.begin(),neibourgh.end());
  neibourgh.erase(unique(neibourgh.begin(),neibourgh.end()),neibourgh.end());
  return (neibourgh.size()>=(3*ncrown*(ncrown+1))); // number of neighbor at n crowns = 6*n*(n+1)/2... Don't know what to do if people include infills... Use >= in order not to remove a T5 because of infilled, at least...

}
unsigned int TEsEvent::T5Criteria(unsigned int ncrown){
  if (fSelectedStations.size() < 3)
    return 0;           // need 3 stations for a T5
  double sMax = -1.0;
  TCalibStation *s=NULL;
  for (unsigned int i = 0; i < fSelectedStations.size(); i++)
    if (fSelectedStations[i]->SigInVEM() > sMax) {
      sMax = fSelectedStations[i]->SigInVEM();
      s = fSelectedStations[i];
    }
  if (!s) return 0;
  bool IR=UseInfillRec;
  CheckNeigh(NULL,s,ncrown,IR);
  if (CheckNeigh(&fSelectedStations,s,ncrown,IR)) return s->Id;
  if (CheckNeigh(&fSilentStations,s,ncrown,IR)) return s->Id;
  if (CheckNeigh(&fRandomStations,s,ncrown,IR)) return s->Id;
  if (CheckNeigh(&fAmbiguousStations,s,ncrown,IR)) return s->Id;
  return CheckNeigh(NULL, NULL,0,IR);
  //return 0;
}

void TEsEvent::Prune() {
  Stations.clear();
  for (unsigned int i = 0; i < fSelectedStations.size(); ++i) {
    Stations.push_back(*fSelectedStations[i]);
    if (Stations.back().Histo && Stations.back().CalibSaved()) {
      delete Stations.back().Histo;
      Stations.back().Histo = NULL;
    }
  }
  for (unsigned int i = 0; i < fSilentStations.size(); ++i)
    Stations.push_back(*fSilentStations[i]);
  if (Stations.size() !=
      fSelectedStations.size() + fSilentStations.size())
    cerr << "Problem in Prune !! " << endl;
}

/* NEW stuff, still not official */

int TSeed::DistInCrown(unsigned int pos) {
  int d = 1000;
  for (unsigned int i = 0; i < 3; i++) {
    int t = 0;
    int n =
      (int) round(fabs(fEvent->fSelectedStations[pos]->fY - fStations[i]->fY) /
                  1300.);
    int e =
      (int) round(fabs(fEvent->fSelectedStations[pos]->fY - fStations[i]->fY) /
                  2600. + fabs(fEvent->fSelectedStations[pos]->fX -
                               fStations[i]->fX) / 1500.);
    t = (n > e ? n : e);
    if (d > t)
      d = t;
  }
  return d;
}

bool TSeed::IsPhysics() {
  if (fTot)
    return true;
  int n = 0;
  for (unsigned int i = 0; i < fEvent->fSelectedStations.size(); i++)
    if (DistInCrown(i) < 5)
      n++;
  if (n > 3)                  // the 3 seed tank + one more
    return true;
  return false;
}

void TSeed::Select() {
  fStations[0]->Trigger.Type |= 0x400;
  fStations[1]->Trigger.Type |= 0x400;
  fStations[2]->Trigger.Type |= 0x400;
}

double TSeed::DeltaTime(unsigned int pos) {
  double x1 = fEvent->fSelectedStations[pos]->fX - fStations[0]->fX;
  double y1 = fEvent->fSelectedStations[pos]->fY - fStations[0]->fY;
  double t1 =
    fEvent->fSelectedStations[pos]->StartTime() * 1.e+9 -
    fStations[0]->StartTime() * 1.e+9;
  return t1 + (fU * x1 + fV * y1) / kCdasUtil::CSPEED;
}

double TSeed::DeltaAng(TSeed * s) {
  return fU * s->fU + fV * s->fV + W() * s->W();
}

bool TSeed::operator==(const TSeed & s) const {
  return ((fStations[0] == s.fStations[0]) && (fStations[1] == s.fStations[1])
          && (fStations[2] == s.fStations[2]));
}

bool TSeed::operator<(const TSeed & s) const {
  if (fNbCompat == s.fNbCompat) {                               // if same number of neighbours, prefer tot, or order by signal
    if (fTot && (!s.fTot))
      return false;
    if ((!fTot) && s.fTot)
      return true;
    return (fVem < s.fVem);
  }
  return (fNbCompat < s.fNbCompat);
}

void TSeed::AddStat(unsigned int pos) {
  fStations.push_back(fEvent->fSelectedStations[pos]);
  if (fStations.size() != 3)
    return;
  sort(fStations.begin(),fStations.end()); // sort in any way...
  double x0 = fStations[0]->fX;
  double y0 = fStations[0]->fY;
  double t0 = fStations[0]->StartTime() * 1.e+9;
  double s0 = pow((double) fStations[0]->SigInVEM(), (double) 0.5);
  double x1 = fStations[1]->fX;
  double y1 = fStations[1]->fY;
  double t1 = fStations[1]->StartTime() * 1.e+9;
  double s1 = pow((double) fStations[1]->SigInVEM(), (double) 0.5);
  double x2 = fStations[2]->fX;
  double y2 = fStations[2]->fY;
  double t2 = fStations[2]->StartTime() * 1.e+9;
  double s2 = pow((double) fStations[2]->SigInVEM(), (double) 0.5);
  fVem = s0 + s1 + s2;
  fTot = (fStations[0]->IsTOT() && fStations[1]->IsTOT() && fStations[2]->IsTOT());
  fXCore = (x0 * s0 + x1 * s1 + x2 * s2) / fVem;
  fYCore = (y0 * s0 + y1 * s1 + y2 * s2) / fVem;
  // computing u,v
  x1 -= x0;
  y1 -= y0;
  t1 -= t0;
  x2 -= x0;
  y2 -= y0;
  t2 -= t0;
  fU = kCdasUtil::CSPEED * ((t1 * y2 - t2 * y1) / (x2 * y1 - x1 * y2));
  fV = kCdasUtil::CSPEED * ((t1 * x2 - t2 * x1) / (x1 * y2 - x2 * y1));
  if (!isnan(W(0.1))) fValid=1;
}

int TEsEvent::ExperimentalT4() {
  RemoveExternalTriggers();
  RemoveLightning();
  RemoveBadStations();
  RemoveBadAltitude();
  fSeeds.reserve(100);
  
  double IST4DMIN=2000.;
  double IST4DMAX=2800.;
  if (UseInfillRec){
    IST4DMIN=1000.;
    IST4DMAX=1400.;
  }

  for (unsigned int i = 0; i < fSelectedStations.size(); i++) {
    for (unsigned int j = 0; j < fSelectedStations.size(); j++) {
      if (j == i)
        continue;
      if (IST4DIST(i, j) > IST4DMIN)
        continue;
      if (IST4DTIMETOLERANCEBAD(i, j))
        continue;
      for (unsigned int k = 0; k < fSelectedStations.size(); k++) {
        if (k == j || k == i)
          continue;
        if (IST4DIST(i, k) > IST4DMIN)
          continue;
        if (IST4DIST(j, k) > IST4DMAX)
          continue;
        if (IST4DTIMETOLERANCEBAD(i, k) || IST4DTIMETOLERANCEBAD(j, k))
          continue;
        // Found one seed
        TSeed s(this);
        s.AddStat(i);
        s.AddStat(j);
        s.AddStat(k);
        if (!s.fValid)
          continue;
        int ok = 1;
        for (unsigned int l = 0; l < fSeeds.size(); l++) {
          if (fSeeds[l] == s) {
            ok = 0;
            continue;
          }
        }
        if (ok)
          fSeeds.push_back(s);
      }
    }
  }
  if (fSeeds.size()) {
    // looking for how many neighbourgs the seeds has.
    double coslim = cos(10 * kCdasUtil::DEG2RAD);
    for (unsigned int i = 0; i < fSeeds.size(); i++) {
      for (unsigned int j = 0; j < fSeeds.size(); j++) {
        if (i == j)
          continue;
        double dang = fSeeds[i].DeltaAng(&(fSeeds[j]));
        if (dang > coslim)
          fSeeds[i].fNbCompat++;
      }
    }
    // sort seeds to keep the best candidate
    sort(fSeeds.begin(), fSeeds.end());
    int goodseed = fSeeds.size() - 1;        // the best one is the biggest one
    fSeeds[goodseed].Select();
    // debug to see angular distance distribution
    //for (unsigned int j=0;j<goodseed;j++) {
    //double dang=seeds[goodseed].DeltaAng(&(seeds[j]));
    //cout << "%% " << dang << endl;
    //}
    // keep stations compatible in time
    for (unsigned int k = 0; k < fSelectedStations.size(); k++) {
      double dt = fSeeds[goodseed].DeltaTime(k);
      if (!(dt > -1000 && dt < 2000)) {
        fRandomStations.push_back(fSelectedStations[k]);
        fSelectedStations.erase(fSelectedStations.begin() + k);
        k--;
      }
    }
    // good events have a tot seed or a 4th tank within 4 crowns
    if (fSeeds[goodseed].IsPhysics()) {
      fEstimated.fU = fSeeds[goodseed].fU;
      fEstimated.fV = fSeeds[goodseed].fV;
      fEstimated.fXCore = fSeeds[goodseed].fXCore;
      fEstimated.fYCore = fSeeds[goodseed].fYCore;
      ExperimentalRemoveLonelyStations(1800, 5000, 4000);
      // good events still have 4 stations or a tot seed
      if (fSeeds[goodseed].IsPhysics()) {
        t4level=1;
        return 1;
      }
    }
  }
  for (unsigned int k = 0; k < fSelectedStations.size(); k++)
    fRandomStations.push_back(fSelectedStations[k]);
  fSelectedStations.clear();
  t4level=0;
  return 0;
}

#define ESDIST2CORE(i,j,k,l) (sqrt(i*i+j*j-(i*k+j*l)*(i*k+j*l)))

void TEsEvent::ExperimentalRemoveLonelyStations(double d1Cut, double d2Cut,
    double d3Cut) {
  double dist = 0;
  double x1, y1, x2, y2;
  for (int i = 0; i < (int) fSelectedStations.size(); i++) {
    unsigned int n = 0, n2 = 0;
    x1 = fSelectedStations[i]->fX;
    y1 = fSelectedStations[i]->fY;
    for (int j = 0; j < (int) fSelectedStations.size(); j++) {
      if (j == i)
        continue;
      x2 = fSelectedStations[j]->fX;
      y2 = fSelectedStations[j]->fY;
      dist = sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
      if (dist < d1Cut)
        n++;
      if (dist < d2Cut)
        n2++;
    }
    if (n == 0 || n2 < 2) { // action depending on angle
      if (fEstimated.Theta("deg") > 60
          || (fEstimated.Theta("deg") > 40
              && ESDIST2CORE(fSelectedStations[i]->fX -
                             fEstimated.fXCore,
                             fSelectedStations[i]->fY -
                             fEstimated.fYCore, fEstimated.fU,
                             fEstimated.fV) < d3Cut))
        fAmbiguousStations.push_back(fSelectedStations[i]);
      else
        fRandomStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin() + i);
      i--;
    }
  }
}



void TEsEvent::QualityCuts() {
  // here the quality mask is independant of the orignal mask, so before killing a PMT, we need to check if it was one. The flag is used to check if a change was finally performed in the station 
  unsigned int sz;
  sz = fSelectedStations.size() ;
  int flag=0;  // to check if a PMT was killed for each station
  for (unsigned int i = 0; i < sz; i++) {
    flag=0;
    //check Full PMT to remove
    if(fSelectedStations[i]->PMQuality->TubeMask<7) {
      for (unsigned int ipm = 0; ipm < kIoSd::NPMT; ipm++) {
	if(fSelectedStations[i]->Calib->TubeOk[ipm]) {
	  fSelectedStations[i]->Calib->TubeOk[ipm] = 
	    ((( fSelectedStations[i]->Calib->TubeMask & (1 << ipm)) == (1 << ipm))  && 
	     (( fSelectedStations[i]->PMQuality->TubeMask & (1 << ipm)) == (1 << ipm)));
	  
	  //clear PMT signal when identified as bad
	  if(fSelectedStations[i]->Calib->TubeOk[ipm]==0) { 
	    fSelectedStations[i]->fPmt[ipm].fSignals.clear(); 
	    flag=1;
	  }
	}//end case of PMT ON before quality
      }
    }
    
    //check Anode to remove   (only for high gain saturated signals)
    
    if(fSelectedStations[i]->PMQuality->AnodeMask<7  && fSelectedStations[i]->IsHighGainSaturated()) {
      if(fSelectedStations[i]->PMQuality->AnodeMask<7) {
	for (unsigned int ipm = 0; ipm < kIoSd::NPMT; ipm++) {
	  if( fSelectedStations[i]->Calib->TubeOk[ipm] ){
	    fSelectedStations[i]->Calib->TubeOk[ipm] = 
	      ((( fSelectedStations[i]->Calib->TubeMask & (1 << ipm)) == (1 << ipm))  && (( fSelectedStations[i]->PMQuality->AnodeMask & (1 << ipm)) == (1 << ipm)));
	    
	    if(fSelectedStations[i]->Calib->TubeOk[ipm]==0){   
	      fSelectedStations[i]->fPmt[ipm].fSignals.clear();
	      flag=1;
	    }
	  }//end case of PMT ON before quality
	}
      }
    }
    
    
    
    if((fSelectedStations[i]->Calib->TubeOk[0]+fSelectedStations[i]->Calib->TubeOk[1]+fSelectedStations[i]->Calib->TubeOk[2])>0){
      if(flag==1){
	fSelectedStations[i]->fSignals.clear();  
	fSelectedStations[i]->Signal();
      }
    }
    else{ // station to kill
       fBadStations.push_back(fSelectedStations[i]);
     fSelectedStations.erase(fSelectedStations.begin()+i);
      i--;
      sz--;
      
    }
  }

}

void TEsEvent::UseNonRainingOnly(){
  unsigned int sz;
   sz = fSelectedStations.size() ;
   int flag=0;
   for (unsigned int i = 0; i < sz; i++) {
     flag=0;
     if(fSelectedStations[i]->PMQuality->RainingMask<7) {
       for (unsigned int ipm = 0; ipm < kIoSd::NPMT; ipm++) {
	 fSelectedStations[i]->Calib->TubeOk[ipm] = 
	   ((( fSelectedStations[i]->Calib->TubeMask & (1 << ipm)) == (1 << ipm))  && (( fSelectedStations[i]->PMQuality->RainingMask & (1 << ipm)) == (1 << ipm)));
	 if(fSelectedStations[i]->Calib->TubeOk[ipm]==0){
	   fSelectedStations[i]->fPmt[ipm].fSignals.clear();
	   flag=1;
	 }
       }
       
       if((fSelectedStations[i]->Calib->TubeOk[0]+fSelectedStations[i]->Calib->TubeOk[1]+fSelectedStations[i]->Calib->TubeOk[2])>0){
	 if(flag==1){
	   fSelectedStations[i]->fSignals.clear();  
	   fSelectedStations[i]->Signal();
	 }
       }
       else{ // station to kill
	 fBadStations.push_back(fSelectedStations[i]);
	 fSelectedStations.erase(fSelectedStations.begin()+i);
	 i--;
	 sz--;

       }
     }
   }
}


void TEsEvent::RemoveXbee() {
  
  unsigned int sz;
  int id;
  UInt_t gps=Trigger.Second;  
  //int flag=0;  // to check if a PMT was killed for each station
  sz = fSelectedStations.size() ;
  for (unsigned int i = 0; i < sz; i++) {
    id=fSelectedStations[i]->Id;
    if (((id==1570 || id==1764) && (gps>926640000 && gps<961977600)) || 
	((id==1574 || id==1622 || id==1773) && (gps>926640000 && gps<960249600)) ||
	((id==1810 || id==1813 || id==1816) && (gps>960508800 && gps<990576000)) ||
	((id==1811 || id==1812) && (gps>961891200 && gps<990576000)) ||
	((id==1826 || id==1827) && (gps>976579200 && gps<990576000))){
      
      fBadStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin()+i);
      i--;
      sz--;
    }
    
  }
  sz = fSilentStations.size() ;
  for (unsigned int i = 0; i < sz; i++) {
    id=fSilentStations[i]->Id;
    if (((id==1570 || id==1764) && (gps>926640000 && gps<961977600)) || 
	((id==1574 || id==1622 || id==1773) && (gps>926640000 && gps<960249600)) ||
	((id==1810 || id==1813 || id==1816) && (gps>960508800 && gps<990576000)) ||
	((id==1811 || id==1812) && (gps>961891200 && gps<990576000)) ||
	((id==1826 || id==1827) && (gps>976579200 && gps<990576000))){
      
      fBadStations.push_back(fSilentStations[i]);
      fSilentStations.erase(fSilentStations.begin()+i);
      i--;
      sz--;
    }
    
  }
}
void TEsEvent::CheckT2() {
  unsigned int sz;
  //int id;
  sz = fSelectedStations.size() ;
  for (unsigned int i = 0; i < sz; i++) {
    if(fSelectedStations[i]->T2Life==0){
      fBadStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin()+i);
      i--;
      sz--;
    }
  }
    

  sz = fSilentStations.size() ;
 for (unsigned int i = 0; i < sz; i++) {
    if(fSilentStations[i]->T2Life==0){
       fBadStations.push_back(fSilentStations[i]);
     fSilentStations.erase(fSilentStations.begin()+i);
      i--;
      sz--;
    }
    
  }


 sz = fAmbiguousStations.size() ;
 for (unsigned int i = 0; i < sz; i++) {
   if(fAmbiguousStations[i]->T2Life==0){
         fBadStations.push_back(fAmbiguousStations[i]);
     fAmbiguousStations.erase(fAmbiguousStations.begin()+i);
      i--;
      sz--;
    }
    
  }

 sz = fRandomStations.size() ;
 for (unsigned int i = 0; i < sz; i++) {
   if(fRandomStations[i]->T2Life==0){
     fBadStations.push_back(fRandomStations[i]);
     fRandomStations.erase(fRandomStations.begin()+i);
      i--;
      sz--;
    }
  }

 int id;
 
 if(AugerId<200600000000LL){ // this is here only to correct for 2005 bug
   
   sz = fBadStations.size() ;
   for (unsigned int i = 0; i < sz; i++) {
     id=fBadStations[i]->Id;
     if(id<100)continue;
     if(fBadStations[i]->T2Life==1){
       fSilentStations.push_back(fBadStations[i]);
       fBadStations.erase(fBadStations.begin()+i);
       i--;
       sz--;
     }
   }
 }
 
}
void TEsEvent::CheckT2CommsCrisis() {
  
  // to apply between 7/6/2009 0:00 (gps=928368000) and  16/11/2009 0:00(gps=942364800) 
  unsigned int sz;
  //int id;
  sz = fSelectedStations.size() ;
  for (unsigned int i = 0; i < sz; i++) {
    if(fSelectedStations[i]->T2Life120==0){
     fBadStations.push_back(fSelectedStations[i]);
      fSelectedStations.erase(fSelectedStations.begin()+i);
       i--;
      sz--;
    }
  }
    
  
  sz = fSilentStations.size() ;
  for (unsigned int i = 0; i < sz; i++) {
      if(fSilentStations[i]->T2Life120==0){
	fBadStations.push_back(fSilentStations[i]);
	fSilentStations.erase(fSilentStations.begin()+i);
	i--;
	sz--;
      }
  }
}


int TEsEvent::GetNeighbours() 
{

  if (!IsT4())    return 0;
  double sMax = -1.0;
  double x0,y0;
  int imax;
  int num_nei=0;
  fIdTankHottest=0;

  for (unsigned int i = 0; i < fSelectedStations.size(); i++)
    if (fSelectedStations[i]->SigInVEM() > sMax) {
      sMax = fSelectedStations[i]->SigInVEM();
      x0=fSelectedStations[i]->fX;
      y0=fSelectedStations[i]->fY;
      fIdTankHottest=fSelectedStations[i]->Id;
      imax=i;
    }

  if(!fIdTankHottest) return 0;
  else{
   for (unsigned int i = 0; i < fSelectedStations.size(); i++){
      if(fSelectedStations[i]->Id==fIdTankHottest)continue;
      if (kEs::Crown(x0,fSelectedStations[i]->fX,y0,fSelectedStations[i]->fY,UseInfillRec)<=1)   
	num_nei++;
    }
   
    for (unsigned int i = 0; i < fSilentStations.size(); i++)
      if (kEs::Crown(x0,fSilentStations[i]->fX,y0,fSilentStations[i]->fY,UseInfillRec)<=1)
	num_nei++;

    
    
    
    for (unsigned int i = 0; i < fRandomStations.size(); i++)
      if (kEs::Crown(x0,fRandomStations[i]->fX,y0,fRandomStations[i]->fY,UseInfillRec)<=1) num_nei++;
    for (unsigned int i = 0; i < fAmbiguousStations.size(); i++)
      if (kEs::Crown(x0,fAmbiguousStations[i]->fX,y0,fAmbiguousStations[i]->fY,UseInfillRec)<=1)  num_nei++;
  }
 

  return num_nei;  
}


int TEsEvent::IsPos(double xc, double yc) {
  vector<double> x, y;
  vector<int> id_stat;
  x.reserve(10);
  y.reserve(10);
  id_stat.reserve(10);
  
   unsigned int mult_sel = fSelectedStations.size();
  unsigned int mult_sil = fSilentStations.size();
  unsigned int mult_ran = fRandomStations.size();
  unsigned int mult_amb = fAmbiguousStations.size();
  
  if (mult_sel < 3)    return 0;           // need 3 stations for a T5
  
  // look for tanks within the first crown of the core
  
  for (unsigned int i = 0; i < mult_sel; i++){
    if (kEs::Crown(xc,fSelectedStations[i]->fX,yc,fSelectedStations[i]->fY,UseInfillRec)<=2) {
      x.push_back(fSelectedStations[i]->fX);
      y.push_back(fSelectedStations[i]->fY);
      id_stat.push_back(fSelectedStations[i]->Id);
    }
  }
  for (unsigned int i = 0; i < mult_sil; i++)
    if (kEs::Crown(xc,fSilentStations[i]->fX,yc,fSilentStations[i]->fY,UseInfillRec)<=2) {
      x.push_back(fSilentStations[i]->fX);
      y.push_back(fSilentStations[i]->fY);
      id_stat.push_back(fSilentStations[i]->Id);
    }
  for (unsigned int i = 0; i < mult_ran; i++)
    if (kEs::Crown(xc,fRandomStations[i]->fX,yc,fRandomStations[i]->fY,UseInfillRec)<=2) {
      x.push_back(fRandomStations[i]->fX);
      y.push_back(fRandomStations[i]->fY);
      id_stat.push_back(fRandomStations[i]->Id);
     
    }
  for (unsigned int i = 0; i < mult_amb; i++)
    if (kEs::Crown(xc,fAmbiguousStations[i]->fX,yc,fAmbiguousStations[i]->fY,UseInfillRec)<=2) {
      x.push_back(fAmbiguousStations[i]->fX);
      y.push_back(fAmbiguousStations[i]->fY);
      id_stat.push_back(fAmbiguousStations[i]->Id);
      
    }
  if (x.size()<4) return 0;

  // search for equilateral and isoceles triangles (1 crown for each pair)
  bool TriangleIsEquilateral=false;
  bool TriangleIsIsoceles=false;
  bool TriangleFound=false;
  bool EqTriangleFound=false;
  bool IsoTriangleFound=false;
  
  for (unsigned int i = 0; i < x.size() ; i++){
    for (unsigned int j = i+1; j < x.size() ; j++){
      if(j==i)continue;
      bool ij_1stCrown =false;
      bool ij_neighbours =false;
      if (kEs::Crown(x[i],xc,y[i],yc,UseInfillRec)<=1 && kEs::Crown(x[j],xc,y[j],yc,UseInfillRec)<=1) ij_1stCrown=true; 
      if (kEs::Crown(x[i],x[j],y[i],y[j],UseInfillRec)<=1)ij_neighbours =true;// j in the 1st crown of i
      for (unsigned int k = 0; k < x.size() ; k++) {
	if(k==i || k==j) continue;
	TriangleIsEquilateral=false;
	TriangleIsIsoceles=false;
	
	if (ij_1stCrown && ij_neighbours && kEs::Crown(x[k],x[j],y[k],y[j],UseInfillRec)<=1 && kEs::Crown(x[i],x[k],y[i],y[k],UseInfillRec)<=1)  TriangleIsEquilateral=true;
	if (ij_neighbours && ij_1stCrown && ( (kEs::Crown(x[k],x[j],y[k],y[j],UseInfillRec)<=1 && kEs::Crown(x[i],x[k],y[i],y[k],UseInfillRec)<=2)|| (kEs::Crown(x[k],x[j],y[k],y[j],UseInfillRec)<=2 && kEs::Crown(x[i],x[k],y[i],y[k],UseInfillRec)<=1) )  ) 	  TriangleIsIsoceles=true;

	if (_InsideTheTriangle(xc, yc, x, y, i, j, k))
	  { TriangleFound=true;
	    if(TriangleIsEquilateral)       EqTriangleFound=true;
	    else if(TriangleIsIsoceles)     IsoTriangleFound=true;
	  }
      }
    }
  }
  
  if( EqTriangleFound )       return 1;
  else if( IsoTriangleFound)  return 2;
  else if(TriangleFound)      return 3;
  else                        return 0;
}




