#include <sys/types.h>
#include <sys/stat.h>
#include "Es.h"
#include "STCconstants.h"

int main(int argc, char *argv[]) {
  bool overwrite = false, help = false, verbose = false;
  int noptions = 0;

  for (int i = 1; i < argc; i++) {
    if (strncmp(argv[i], "-v", 2) == 0) {
      verbose = true;
      noptions++;
    }
    if (strncmp(argv[i], "-h", 2) == 0) {
      help = true;
      noptions++;
    }
    if (strncmp(argv[i], "-force", 6) == 0) {
      overwrite = true;
      noptions++;
    }
  }
  argc -= noptions;
  if (argc < 2 || help) {
    cout << "Usage : " << endl;
    cout <<
    "------------------------------------------------------------------------"
    << endl;
    cout << argv[0] << " [-v] [-h] [-force] files... " << endl;
    cout <<
    "produce sdt4 output files for each input file and place them in the appropriate /Raid/data/.SdT4_virj"
    << endl;
    cout <<
    "directory where virj is the version of the Es package used to produce them. "
    << endl;
    cout << " Options  " << endl;
    cout << "-v  produces verbose output" << endl;
    cout << "-h prints this usage " << endl;
    cout <<
    "-force overwrite of existing t4 files if any [default is no]"
    << endl;
    cout <<
    "------------------------------------------------------------------------"
    << endl;
    exit(1);
  }

  argc -= 1;
  argv += noptions + 1;

  char T4DirName[256], cmd[300];
  struct stat toto;


  // Treat the input files one by one :
  for (int i = 0; i < argc; i++) {
    char *pos = strstr(argv[i], "sdm_");
    if (pos == NULL) {
      cerr << "Skipping file " << argv[i] <<
      " which is not and sdm file" << endl;
      continue;
    }
    IoSd input(argv[i]);
    char sdt4FileName[512];
    char yearMonth[8];
    strncpy(yearMonth, pos - 8, 7);
    yearMonth[7] = 0;
    snprintf(T4DirName, 255, "%s%s/%s/", "/Raid/data/nobackup/.SdT4_",
             ESVERSION, yearMonth);
    snprintf(cmd, 299, "%s%s", "mkdir -p ", T4DirName);
    system(cmd);
    snprintf(sdt4FileName, 512, "%s%s%s%s%s", T4DirName, "sdt4_",
             ESVERSION, "_", pos + 4);
    if (stat(sdt4FileName, &toto) == 0 && !overwrite) {
      cerr << "Skipping file " << argv[i] <<
      " because output file already exits en overwrite option was not used."
      << endl;
      continue;
    }
    IoSd output(sdt4FileName, "w");
    for (EventPos j = input.FirstEvent(); j < input.LastEvent();
         j = input.NextEvent()) {
      TEsEvent event(j);
      if (event.IsT4()) {
        if (verbose)
          output.Write(event, "VERBOSE");
        else
          output.Write(event);
      }
    }
    output.Close();
    input.Close();
  }
}
