#include "T2Life.h"
#include <string>
#include <fstream>
#include <sstream>
#include <vector>
#include <iostream>

using namespace std;

T2Life* 
T2Life::gfT2Life = 0;

T2Life::T2Life(const string& name)
  :  fCurrentStart(0),
     fCurrentStop(0),
     fFile(name.c_str()),
     fValidStations()
{if (!fFile) cerr << "No T2 Life file found. Will run Without!" << endl;  }

T2Life& 
T2Life::GetT2Life()
{
  if (!gfT2Life) gfT2Life = new T2Life("T2Life.txt");

  return *gfT2Life;
}


vector<unsigned int>& 
T2Life::GetValidStations(unsigned int theGPSSecond)
{

  if (theGPSSecond < fCurrentStop && theGPSSecond >= fCurrentStart) 
    return fValidStations;
  
  fValidStations.clear();
  if (!fFile) return fValidStations;

  if (theGPSSecond < fCurrentStart) { // requested earlier time
    fFile.seekg(0);
    cerr << "Rewinding T2Life file. Why?" << endl;
  }

  string line;
  istringstream buffer;
  bool found = false;
  unsigned int start;
  unsigned int end;
  while (getline(fFile, line)) { // find next valid line
    buffer.str(line);
    buffer >> start >> end;

    if (theGPSSecond < start)
      break;
    if (theGPSSecond < end) {
      found = true;
      fCurrentStart = start;
      fCurrentStop   = end;
      break;
    }
  }
  if (!found) {
    cerr << theGPSSecond << " out of range in T2Life. Why? (" << start << "-" << end << ")" << endl;
    fCurrentStart= 0;
    fCurrentStop  = 0;
    fFile.seekg(0);
    for (unsigned i = 0; i<1601; ++i) fValidStations.push_back(0);
    return fValidStations;
  }
    
  for (unsigned i = 0; i<101; ++i)
    fValidStations.push_back(0);

  int flag;
  while ( buffer >> flag)
    fValidStations.push_back(flag);
    for (unsigned i = fValidStations.size(); i < 1601; ++i) fValidStations.push_back(0);
  return fValidStations;
}


std::vector<unsigned int>& StationsStatusFromT2Files(unsigned int theGPSSecond)
{
  return T2Life::GetT2Life().GetValidStations(theGPSSecond);
}
