#include <iostream>
#include <signal.h>
#include "Es.h"
#include "IoAuger.h"
#include "STCconstants.h"

int main(int argc, char *argv[]) {
  char *outputName = NULL;
  bool out = false;
  int nT4 = 0, nT5 = 0, nTot = 0;
  if (argc == 1) {
    cout << "Usage: " << argv[0] << "[-o output] <files>" << endl;
    cout <<
    "   will list the AugerId in files satisfying OfficialSelection and T5"
    << endl;
    cout <<
    "   if -o option is present, create an output file with selected stations "
    << endl;
    cout <<
    "   using BottomUpSelection(3000.0, 5000.0) and saved calibration. "
    << endl;
    return 1;
  }
  if (strstr(argv[1], "-o")) {
    outputName = argv[2];
    argv += 2;
    argc -= 2;
    out = true;
  }
  AugerIoSd input(argc - 1, argv + 1);        //creates the object holding the files
  IoSd *outFile = NULL;
  if (out)
    outFile = new IoSd(outputName, "w");
  for (EventPos pos = input.FirstEvent(); pos < input.LastEvent();
       pos = input.NextEvent()) {
    TEsEvent event(pos);
    ++nTot;
    if (event.OfficialSelection() && event.IsT5()) {
      cout << event.AugerId << endl;
      ++nT5;
      if (out) {
        event.BottomUpSelection(3000.0, 5000.0);
        cout << "# " << event.IsT5() << endl;
        event.SaveCalib();
        event.Prune();
        outFile->WriteModified(event);
      }
    }
    if (event.IsT4())
      ++nT4;
  }
  if (out)
    outFile->Close();
  cout << " Total number of event treated : " << nTot << ". Total T4 : "
  << nT4 << ". Total T5 : " << nT5 << "." << endl;
  return 0;
};
