#include "Es.h"
#include "TCdasUtil.h"
#include "T2Life.h"
#include <math.h>
using namespace kEs;
// Everything from GAP 2006-072, by PB

static unsigned int _NMinStations=3;
static unsigned int _NMaxRemovedStations=9;
static int _Verbose=0;
static bool _LastTimingIteration;
const double kIsoDist = 4700.;
const double kIsoDist2 = 6200.;
const double kBaryPower = 1./3.;
const double kCurv0 = 0.00014;
const double kTolRes = 200.;
const double kTolResM = 250.;
const double kDistMaxProj = 1300.;
const double kDistMax = 1800; //3100.;
const double kDistMax3folds = 3100.;
const double kAreaMin = 0.2E6;
const double kAreaMax = 1.2E6;
const double kAna3RatMin = 1.6;

#define MIN(a,b) ((a)<=(b)?(a):(b))
#define MAX(a,b) ((a)>=(b)?(a):(b))

bool TEsEvent::TopDownSelection() 
{
  Init();
  RemoveExternalTriggers();
  RemoveLightning();
  RemoveBadStations();
  RemoveBadAltitude();
  //RemoveLonelyStations(kIsoDist, kIsoDist2);
  RemoveDuplicatedStations();	  
  RemoveLonelyStations();
  RemoveXbee();
  OrderStationsBySignals();
 
  unsigned int is, ns=fSelectedStations.size();
  unsigned int k=(ns-_NMinStations)<_NMaxRemovedStations?ns-_NMinStations:_NMaxRemovedStations; 

  // easy cases
  if(ns<_NMinStations) return false;
  if(_Verbose>0) cout<<"  Top Down selection on event #"<<Id<<endl;
  if(PatternRecovering()) {
    if(_Verbose>1) cout<<"  Found directly a "<<fSelectedStations.size()<<"-fold configuration"<<endl<<"    Estimated zenith: "<<fEstimated.Theta("deg")<<" deg."<<endl;
    RemoveDuplicatedStations();	  
    return true;
  }

  // then, top-down algorithm
  for(is=1;is<=k;is++) {
    fIndexes.clear(); fIndexes.resize(is); fIndexes[is-1]=ns;
    if(_Verbose>1) {
      cout<<"  New try by removing "<<is<<" stations" <<endl;
    }
    if(RecoverNStations(is)) {
      Remove777Stations();
      RemoveDuplicatedStations();
      if(_Verbose>1) {
        cout<<"  Found a "<<fSelectedStations.size()<<"-fold configuration by removing "<<is<<" stations"<<endl<<"    Estimated zenith: "<<fEstimated.fU<<"  "<<fEstimated.fV<<"  "<<fEstimated.Theta("deg")<<" deg."<<endl;
      }
      return true;
    }
  }
  
  return false;
}

void TEsEvent::OrderStationsBySignals()
{
  unsigned int size = fSelectedStations.size();
  unsigned int is,js;
  fOrd.clear();
  for(is=0;is<size;is++) fOrd.push_back(is);
  for(is=1;is<size;is++) for(js=is;js>0;js--) {
    if(fSelectedStations[fOrd[js-1]]->fSigInVEM>=fSelectedStations[fOrd[js]]->fSigInVEM) {
      unsigned int temp = fOrd[js]; fOrd[js] = fOrd[js-1]; fOrd[js-1] = temp;
    }
  }
}

void TEsEvent::Remove777Stations()
{
  for(unsigned int is=0;is<fSelectedStations.size();is++) if(fSelectedStations[is]->Error==777) {
    fRandomStations.push_back(fSelectedStations[is]);
    fSelectedStations.erase(fSelectedStations.begin() + is);
    --is;
  }
}

bool TEsEvent::PatternRecovering()
{
  TopDownDiagnostics[0]=TopDownDiagnostics[1]=TopDownDiagnostics[2]=0;
  unsigned int size=fSelectedStations.size(), ns=0;
  for(unsigned int is=0;is<size;is++) if(fSelectedStations[is]->Error==0) ns++;
  if(ns == 3) {
    if(_NMinStations==3) return Recover3Stations();
    else return false;
  }
  TopDownEstimateCore(); 
  _LastTimingIteration=false;
  if(!TopDownIsGoodTimeConfig(0)) return false;
  if(!TopDownIsGoodSpaceConfig()) return false;
  _LastTimingIteration=true;
  if(!TopDownIsGoodTimeConfig(1)) return false;
  //if(TopDownIsSubThreshold()) TopDownDiagnostics[1]=0;
  if(TopDownIsAligned()) TopDownDiagnostics[1]=1;

  return true;
}


bool TEsEvent::RecoverNStations(int n) // N>3
{
  if(n<2) {
    for(unsigned int is=0;is<fIndexes[0];is++) {
      if(_Verbose>2) cout<<"    trying without ls #"<<fSelectedStations[fOrd[is]]->Id<<endl;
      fSelectedStations[fOrd[is]]->Error=777;
      if(PatternRecovering()) return true;
      fSelectedStations[fOrd[is]]->Error=0;
    }
  } else {
    for(unsigned int is=n-1;is<fIndexes[n-1];is++) {
      if(_Verbose>2) cout<<"    trying without ls #"<<fSelectedStations[fOrd[is]]->Id<<endl;
      fSelectedStations[fOrd[is]]->Error=777;
      fIndexes[n-2]=is;
      if(RecoverNStations(n-1)) return true;
      fSelectedStations[fOrd[is]]->Error=0;
    }
  }
  return false;
}

bool TEsEvent::Recover3Stations() // directly from PBRec.cc, but we could maybe just return Is3TOT ?
{
//return Is3ToT();
  int nstsel=0;
  double ratio[3];
  double ratio_avg = 0.;  
  TCalibStation *st1=0,*st2=0,*st3=0;
  unsigned int is,ss=0,size=fSelectedStations.size();
  for(is=0;is<size;is++) if(fSelectedStations[is]->Error==0) {
    st1 = fSelectedStations[is]; ss = is; break;
  }
  for(is=ss+1;is<size;is++) if(fSelectedStations[is]->Error==0) {
    st2 = fSelectedStations[is]; ss = is; break;
  }
  for(is=ss+1;is<size;is++) if(fSelectedStations[is]->Error==0) {
    st3 = fSelectedStations[is]; break;
  }
  // shape of signals : at least 2 stations with ToT trigger or with "wide" signals
  ratio[0] = st1->fSigInVEM/st1->fPeakInVEM;
  ratio[1] = st2->fSigInVEM/st2->fPeakInVEM;
  ratio[2] = st3->fSigInVEM/st3->fPeakInVEM;
  nstsel = (st1->IsTOT() || ratio[0]>kAna3RatMin)
	  + (st2->IsTOT() || ratio[1]>kAna3RatMin)
	  + (st3->IsTOT() || ratio[2]>kAna3RatMin);
  ratio_avg = (ratio[0]+ratio[1]+ratio[2])/3;
  if(_Verbose>2) cout<<"TEsEvent::Recover3Stations:nstsel="<<nstsel<<endl;
  if(nstsel<2) return false;
  // area and maximal distance
  double deter = st1->fX*st2->fY-st1->fY*st2->fX
                +st2->fX*st3->fY-st2->fY*st3->fX
                +st3->fX*st1->fY-st3->fY*st1->fX;
  double dist2 = pow(st1->fX-st2->fX,2.) + pow(st1->fY-st2->fY,2.);
  dist2 = MAX(dist2, pow(st2->fX-st3->fX,2.)+pow(st2->fY-st3->fY,2.));
  dist2 = MAX(dist2, pow(st3->fX-st1->fX,2.)+pow(st3->fY-st1->fY,2.));
  // geometrical extension : cut on area of the triangle
  if(_Verbose>2) cout<<"TEsEvent::Recover3Stations:geometrical extension="<<fabs(deter/2)<<endl;
  if(fabs(deter/2) > kAreaMax || fabs(deter/2) < kAreaMin ) return false;
  // cut on distance between stations
  if(_Verbose>2) cout<<"TEsEvent::Recover3Stations:distance between stations="<<dist2<<endl;
  if(dist2 > pow(kDistMax3folds,2.))  return false;
  //--- computation of direction
  double dt1 = st2->StartTime()-st1->StartTime();
  double dt2 = st3->StartTime()-st1->StartTime();
  fEstimated.fU = kCdasUtil::CSPEED*((st1->fY-st3->fY)*dt1-(st1->fY-st2->fY)*dt2)/deter;
  fEstimated.fV = kCdasUtil::CSPEED*((st3->fX-st1->fX)*dt1-(st2->fX-st1->fX)*dt2)/deter;
  
  TopDownEstimateCore();
  _LastTimingIteration=false; 
  if(!TopDownIsGoodTimeConfig(1)) return false;

 return true;
}

void TEsEvent::TopDownEstimateCore()
{
  unsigned int size = fSelectedStations.size();
  fEstimated.fXCore=fEstimated.fYCore=fEstimated.fZCore=fEstimated.fTCore=0.;
  double wgt, sumw=0.;
  for(unsigned int is=0; is<size; is++) {
    TCalibStation *st = fSelectedStations[is];
    if(st->Error==777) continue;
    wgt = exp(kBaryPower*log(st->fSigInVEM));
    sumw += wgt;
    fEstimated.fXCore += wgt * st->fX; 
    fEstimated.fYCore += wgt * st->fY; 
    fEstimated.fZCore += wgt * st->fZ;
    fEstimated.fTCore += wgt * st->StartTime();
  }
  if(sumw==0.) {};
  fEstimated.fTCore = fEstimated.fTCore * 1e+9 / sumw;
  fEstimated.fXCore /= sumw; fEstimated.fYCore /= sumw; fEstimated.fZCore /= sumw;

}

double TEsEvent::TopDownCorrectTimeAltitude(int is)
{
  double costh=sqrt(MAX(0.,1.-fEstimated.fU*fEstimated.fU-fEstimated.fV*fEstimated.fV));
  return (costh*(fSelectedStations[is]->fZ-fEstimated.fZCore))/kCdasUtil::CSPEED;
}

double TEsEvent::TopDownCorrectTimeAltitudeCurv(int is)
{
  double costh=sqrt(MAX(0.,1.-fEstimated.fU*fEstimated.fU-fEstimated.fV*fEstimated.fV));
  double dx=fSelectedStations[is]->fX-fEstimated.fXCore;
  double dy=fSelectedStations[is]->fY-fEstimated.fYCore;
  double d2=dx*dx+dy*dy-pow(fEstimated.fU*dx+fEstimated.fV*dy,2.);
  double curv=kCurv0*costh;
  return (costh*(fSelectedStations[is]->fZ-fEstimated.fZCore) - curv*d2/2.)/kCdasUtil::CSPEED;
}

bool TEsEvent::TopDownIsGoodTimeConfig(int corrt)
{
  unsigned int is, ns=0;
  double wgt, dx, dy, dt, sumw, sumx, sumy, sumx2, sumy2, sumxy, sumt, sumxt, sumyt;
  double rxx, ryy, rxy, rx, ry, rr, deter;
  int corrtalt=0, corrtaltcurv=0;
  if(corrt==1) {
    corrtalt=1;
  } else if(corrt==2) {
    corrtaltcurv=1;
  }
  unsigned int size = fSelectedStations.size();
  sumw = sumx = sumy = sumx2 = sumy2 = sumxy = sumt = sumxt = sumyt = 0;
  for (is = 0; is < size; is++) {
    TCalibStation *st=fSelectedStations[is];
    if(st->Error!=0) continue; // case of 777 !!!
    dx = st->fX - fEstimated.fXCore;
    dy = st->fY - fEstimated.fYCore;
    dt=0.;
    if(corrtalt) dt+=corrtalt*TopDownCorrectTimeAltitude(is);
    if(corrtaltcurv) dt+=corrtaltcurv*TopDownCorrectTimeAltitudeCurv(is);
    dt += st->StartTime() * 1e+9 - fEstimated.fTCore;
    wgt = 1.;
    sumw += wgt;
    sumx += wgt * dx;
    sumy += wgt * dy;
    sumx2 += wgt * dx*dx;
    sumy2 += wgt * dy*dy; 
    sumxy += wgt * dx * dy;
    sumt += wgt * dt;
    sumxt += wgt * dt * dx;
    sumyt += wgt * dt * dy;
    ns++;
  }
  rxx = sumw * sumy2 - sumy*sumy;
  ryy = sumw * sumx2 - sumx*sumx;
  rxy = sumx * sumy - sumw * sumxy;
  rx = sumxy * sumy - sumx * sumy2;
  ry = sumxy * sumx - sumy * sumx2;
  rr = sumx2 * sumy2 - sumxy*sumxy;
  deter = sumx2 * rxx + sumxy * rxy + sumx * rx;  
  if (deter == 0 || isnan(deter)) return false;
  fEstimated.fU = (rxx * sumxt + rxy * sumyt + rx * sumt) / deter;
  fEstimated.fV = (rxy * sumxt + ryy * sumyt + ry * sumt) / deter;
  fEstimated.fT0 = (rx * sumxt + ry * sumyt + rr * sumt) / deter;
  fEstimated.fU *= (-1.0*kCdasUtil::CSPEED);
  fEstimated.fV *= (-1.0*kCdasUtil::CSPEED);

  if(ns==3) {
    if(_Verbose>1) cout<<"u*u+v*v="<<fEstimated.fU*fEstimated.fU+fEstimated.fV*fEstimated.fV<<"  "<<fEstimated.Theta("deg")<<endl;
    if(isnan(fEstimated.Theta("deg"))) return false;
    return true;
  }
  if(_LastTimingIteration==false) return true;

  // conditions on the residuals
  double sumdt2=0.;
  double costh=sqrt(MAX(0.,1.-fEstimated.fU*fEstimated.fU-fEstimated.fV*fEstimated.fV));
  for(is=0; is<size; is++) {
    TCalibStation *st=fSelectedStations[is];
    if(st->Error!=0) continue; // trying patterns...
    double dx = st->fX - fEstimated.fXCore;
    double dy = st->fY - fEstimated.fYCore;
    double dt = st->StartTime() * 1e+9 -fEstimated.fTCore - (fEstimated.fT0 -fEstimated.fU*dx/kCdasUtil::CSPEED - fEstimated.fV*dy/kCdasUtil::CSPEED);
    sumdt2 += dt*dt;
    if(fabs(dt) > (ns-2)*kTolResM*MAX(costh,0.2)) {
      if(_Verbose>3) cout<<"    residuals of ls #"<<st->Id<<" : "<<fabs(dt)<<" > "<<ns<<" "<<costh<<"  "<<(ns-2)*kTolResM*MAX(costh,0.2)<<endl;
      return false;    
    }
  }  
  if( sqrt(sumdt2/(ns-3)) > (ns-2)*kTolRes*MAX(costh,0.2) ) {
    if(_Verbose>3) cout<<"    global residual : "<<sqrt(sumdt2/(ns-3))<<" "<<(ns-2)*kTolRes*MAX(costh,0.2)<<endl;
    return false;
  }

  return true;
}

bool TEsEvent::TopDownIsGoodSpaceConfig()
{
  unsigned int ns=fSelectedStations.size();
  double dist2_max_proj = pow(1300.,2.) * (ns-2);
  for(int i=ns-1; i>=0; i--) {
    if(fSelectedStations[i]->Error!=0) continue;
    double dx = fSelectedStations[i]->fX - fEstimated.fXCore;
    double dy = fSelectedStations[i]->fY - fEstimated.fYCore;
    double dist2 = pow(dx,2.)+pow(dy,2.) - pow(fEstimated.fU*dx+fEstimated.fV*dy,2.);
    if(dist2 > dist2_max_proj) return false;
  }
  return true;
}

bool TEsEvent::TopDownIsAligned()
{
  unsigned int is,size=fSelectedStations.size(),ns=0;
  double xmean=0., ymean=0.;
  for(is=0;is<size;is++) {
    if(fSelectedStations[is]->Error!=0) continue;
    xmean+=fSelectedStations[is]->fX;
    ymean+=fSelectedStations[is]->fY;
    ns++;
  }
  xmean/=ns; ymean/=ns;
  vector<double> dx,dy;
  for(is=0;is<size;is++) {
    if(fSelectedStations[is]->Error!=0) continue;
    dx.push_back(fSelectedStations[is]->fX-xmean);
    dy.push_back(fSelectedStations[is]->fY-ymean);
  }
  double ixx=0., ixy=0., iyy=0.;
  for(is=0;is<ns;is++) {
    ixx += dx[is]*dx[is];
    iyy += dy[is]*dy[is];
    ixy += dx[is]*dy[is];
  }
  double zeta=atan2(2.*ixy,ixx-iyy)/2.;
  double lg=0., tr=0.;
  for(is=0;is<ns;is++) {
    lg+=pow(dx[is]*cos(zeta) + dy[is]*sin(zeta),2.);
    tr+=pow(dx[is]*sin(zeta) - dy[is]*cos(zeta),2.);
  }
  lg=sqrt(lg);  tr=sqrt(tr);
  if(lg < 300. || tr < 300.) return true;
  return false;
}


