/**
   \file
   Declaration of AugerKey
 
   \author Stefano Argir Antoine Letessier-Selvon
*/

#ifndef __AugerKey_h_
#define __AugerKey_h_

#include <string>
#include <AugerEvent.h>


/**
 * \class AugerKey AugerKey.h "/AugerKey.h"
 *
 * \brief A key to retrive objects of kind AugerEvent from AugerEventFile
 *  Currently the only available indexing is by EventId
 * 
*/
class AugerKey {

public:

  enum EventType { eSdOnly,
                   eFdOnly,
                   eHybrid,
                   eLaser,
                   eFdLaser,
                   eNone
                 };

  AugerKey();

  /// Create the key for the event
  AugerKey(AugerEvent & event);

  /// generate the string for this key
  std::string AsString();

  AugerEventId EventId;
  unsigned int NumberOfStations;
  unsigned int NumberOfEyes;
  unsigned int NumberOfPixels;

  /// Bit field with the id of the eyes
  unsigned int EyesId;
  /// Sd Trigger Algo string if Sd event exits
  std::string SdTriggerAlgo;

  EventType Type;


private:
  static const unsigned int fgNTypes = eNone + 1;
  static const std::string fgStringTypes[fgNTypes];

}
;                              // AugerKey



#endif                          // __AugerKey_h_
