
//
// File   : EyeGeometryData.hh
//
// Purpose: Declaration of the class TEyeGeometryData
//
// $Id: EyeGeometryData.hh 12083 2011-02-21 17:11:42Z mathes $
//



/** @file EyeGeometryData.hh
  * Declaration of the class TEyeGeometryData.
  * @author H.-J. Mathes, FzK
  */


#ifndef _EyeGeometryData_hh_
#define _EyeGeometryData_hh_

#include <iostream>

#include <TObject.h>

#include <FdNumbering.hh>
#ifndef __CINT__
# if (FDUTIL_VERSION_CODE >= ModuleVersionCode(4,1,0))
#  include <FdGeometry.hh>
# endif // FDUTIL_VERSION_CODE
#endif // __CINT__
#include <EyeEvent.hh>

/** Contains some geometry information for the current Eye which is for
  * instance used inside the T3
  */
class TEyeGeometryData : public TObject {
public:
    
    /** Return values of getters when the object has not been set properly. */
    static const float kInvalidValue /* = 0.0 */;
    
    /** Default constructor of the class TEyeGeometryData. */
    TEyeGeometryData();

    /** Constructor for the class TEyeGeometryData. 
      *
      * @param eye_no Eye number of the telescope 1...Fd::kFD_NEYES
      */
    TEyeGeometryData(unsigned int);

    /** Get the pointing direction of the axis of camera 'tel_id'. 
      *
      * @param tel_id the telescope id 1..6
      *
      * @return azimuth angle and elevation angle
      *
      * The angles (az,el) are expressed in degrees starting from east and
      * counter-clockwise for the azimuth angle and from the horizontal for 
      * the elevation angle.
      */
    virtual void GetAxisDirection(unsigned int,float&,float&);
    
#ifndef __CINT__
# if (FDUTIL_VERSION_CODE >= ModuleVersionCode(4,1,0))
    /** Get the pointing direction of the axis of camera 'tel_id'. */
    virtual void GetAxisDirection(unsigned int,
                                  FdUtil::FdGeometry::CameraPointing_t&);
# endif // FDUTIL_VERSION_CODE
#endif // __CINT__

    /** Reset all values to their defaults. */
    virtual void Reset();

    /** Set the axis directions of the current FD Eye to the values stored
      * in the geometry 'database' (= from ini-file).
      *
      * Note: Use this method only on native FD hardware !!!
      */
    virtual void SetAxisDirection();

    /** Set the axis direction of the specified Eye.
      *
      * This method might be useful for simulation and test purposes.
      */
    virtual void SetAxisDirection(unsigned int);
    
    /** Set the axis direction of the specified telescope to the passed values.
      *
      * @see GetAxisDirection() for more details!
      */
    virtual void SetAxisDirection(unsigned int,float,float);

    /** Print the contents of this object verbosely to the passed ostream. */
    virtual void Show(std::ostream& ostr=std::cout,
                      unsigned int eye_no=FdUtil::Fd::kFD_FIRST_EYE);

private:

    float fAxisAzimuth[FdUtil::Fd::kEYE_NMIRRORS];   // axis azimuth values
    float fAxisElevation[FdUtil::Fd::kEYE_NMIRRORS]; // axis elevation values
    
    ClassDef(TEyeGeometryData,EyeEVENTVERSIONv1)
};

#endif  // _EyeGeometryData_hh_
