
//
// File   : EyePixelList.cc
//
// Purpose: Implementation of Eyes class TEyePixelLis2t
//
// $Id: EyePixelList.cc 8757 2010-09-27 12:16:09Z mathes $
//


#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cstring>
#include <vector>


#include <TObject.h>
#include <TClass.h>  // TClass::ReadBuffer(), TClass::WriteBuffer()
#include <TBuffer.h>

#include <FdConfig.hh>
#include <ErrorLogging.hh>
#include <FdNumbering.hh>

#include <MiEvent.hh>

/** @file EyePixelList.cc
  * Implementation of the class TEyePixelList.
  * @author H.-J. Mathes, FzK
  */

#include "CheckVersion.hh"

#include "EyePixelList.hh"

using namespace std;
using FdUtil::Fd;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class TEyePixelList - implementation
//

TEyePixelList::TEyePixelList()
 : fStreamerVersion(0), 
   fEyeNo(FdUtil::GetEyeId()-Fd::kFD_FIRST_EYE), 
   fNPixels(0), fPixelVector(NULL)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TEyePixelList(): " << this << endl;
#endif // TRACE_CONSTRUCTOR

  fPixelVectorSize = Fd::kEYE_NEXTPIXELS/(8 * sizeof(unsigned int));

  if ( TEyeEvent::fgPrintLevel > 1 ) {
    FD_CDEBUG(1) << "Fd::kEYE_NEXTPIXELS= " << Fd::kEYE_NEXTPIXELS << endl;
    FD_CDEBUG(1) << "fPixelVectorSize=    " << fPixelVectorSize << endl;
  }
  
  fPixelVector = new UInt_t[fPixelVectorSize];
  
  for ( unsigned int i=0; i<fPixelVectorSize; i++ )
    fPixelVector[i] = 0;
}

// -------------------------------------------------------------------------

TEyePixelList::TEyePixelList(const TEyePixelList& rhs)
 : TObject(rhs), fStreamerVersion(0)
 { 
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TEyePixelList(TEyePixelList&): " << this << endl;
#endif // TRACE_CONSTRUCTOR
  *this = rhs;
}

// -------------------------------------------------------------------------

TEyePixelList& TEyePixelList::operator=(const TEyePixelList& rhs)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TEyePixelList::operator=(" << &rhs 
           << " -> " << this << ")" << endl;
#endif // TRACE_CONSTRUCTOR

  if ( this == &rhs ) return *this;
  
  fNPixels         = rhs.fNPixels;
  fPixelVectorSize = rhs.fPixelVectorSize;
  
  memcpy( (char *)fPixelVector, (char *)rhs.fPixelVector, 
          fPixelVectorSize * sizeof( UInt_t) );
  
  fPixelList.assign( rhs.fPixelList.begin(), rhs.fPixelList.end() );
  
  fStreamerVersion = rhs.fStreamerVersion;
  
  // temporary variable fPixelNumber not copied
  
  return *this;
}

// -------------------------------------------------------------------------

bool TEyePixelList::operator==(const TEyePixelList& rhs) const
 {
  if ( this == &rhs ) return true;
  
  for ( unsigned int i=0; i<fPixelVectorSize; i++ )
    if ( fPixelVector[i] != rhs.fPixelVector[i] ) return false;

  return true;
}

// -------------------------------------------------------------------------

TEyePixelList::~TEyePixelList()
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "~TEyePixelList(): " << this << endl;
#endif // TRACE_CONSTRUCTOR
  delete [] fPixelVector;
  
  fPixelList.clear();
}

// -------------------------------------------------------------------------

//#ifndef DPA

Fd::FdPixelNumber TEyePixelList::operator[](unsigned int at)
 {
  if ( at == 0 ) return NULL;
  if ( at > Fd::kEYE_NEXTPIXELS ) return NULL;

  unsigned int wordpos = (at - Fd::kEYE_FIRST_PIXEL) / (sizeof(unsigned int) * 8);
  unsigned int bitpos = (at - Fd::kEYE_FIRST_PIXEL) % (sizeof(unsigned int) * 8);
  
  unsigned int mask = 1 << bitpos;
  
  if ( !(fPixelVector[wordpos] & mask) ) { // not set
    return NULL;
  }
  
#if 1
  if ( fPixelList.empty() ) ConstructPixelList();
  
  // Attention !!! address will become invalid after adding new pixels
  
  return &fPixelList[wordpos];
#else
  // Attention !!! only one instance per class !
  
  Fd::SetPixelNo( &fPixelNumber, at );
  
  return &fPixelNumber;
#endif
}

//#endif // DPA

// -------------------------------------------------------------------------

void TEyePixelList::AddPixel(Fd::FdPixelNumber pixel)
 {
  if ( TEyeEvent::fgPrintLevel ) {
    FD_COUT << "TEyePixelList::AddPixel((" << Fd::GetEyeNo(pixel) << ","
                         << Fd::GetEyeMirrorNo(pixel) << ")="
                         << Fd::GetMirrorNo(pixel) << ","
	  	         << Fd::GetPixelNo(pixel) << ") ..." << endl;
  }
  
  if ( Fd::GetEyePixelNo( pixel ) > Fd::kEYE_NEXTPIXELS ) {
    FD_CWARN << "TEyePixelList::AddPixel() - tried to add illegal pixel " 
             << Fd::GetPixelNo( pixel ) << " - ignored!" << endl;
    return;
  }
  
  if ( (Fd::GetEyeNo( pixel ) - Fd::kFD_FIRST_EYE) != fEyeNo ) {
    FD_CWARN << "TEyePixelList::AddPixel() - tried to add pixel " 
             << Fd::GetPixelNo( pixel ) 
	     << "from illegal Eye " << Fd::GetEyeNo( pixel ) 
	     << " - ignored!" << endl;
    return;
  }
  
  if ( fPixelList.size() ) {  // erase this list during add operation
    fPixelList.clear();
  }
  
  unsigned int pix_no = Fd::GetEyePixelNo( pixel ) - Fd::kEYE_FIRST_PIXEL;
  
  unsigned int wordpos = pix_no / (sizeof(unsigned int) * 8);
  unsigned int bitpos = pix_no % (sizeof(unsigned int) * 8);
  
  unsigned int mask = 1 << bitpos;

#if 0
  FD_CDEBUG(1) << "pix_no= " << pix_no << " wordpos= " << wordpos
               << " bitpos= " << bitpos << " mask= " << hex << mask << dec 
	       << endl;
#endif
  
  // pixel already set before ?
  
  if ( fPixelVector[wordpos] & mask ) {
    FD_CWARN << "TEyePixelList::AddPixel(" << Fd::GetEyePixelNo( pixel ) 
             << ") - " 
	     << "tried to add pixel again to list - ignored!" << endl;
  }
  else {
    fPixelVector[wordpos] |= mask;

    fNPixels++;
  }
}

// -------------------------------------------------------------------------

void TEyePixelList::AddPixelList(TMirrorPixelList* pixel_list)
 {
  Fd::FdPixelNumber fdpixel;
  
  if ( TEyeEvent::fgPrintLevel ) {
    FD_COUT << "TMirrorPixelList contains " << pixel_list->GetNumPixels()
            << " pixels !" << endl;
  }
  
  for ( unsigned int i=0; i<pixel_list->GetNumPixels(); i++ ) {
    fdpixel = Fd::GetFdPixelNumber( pixel_list->GetPixel( i ) );
    AddPixel( fdpixel );

    if ( TEyeEvent::fgPrintLevel ) {
      FD_COUT << "TEyePixelList::AddPixelList(seq=" << fNPixels << "): " 
           << Fd::GetPixelNo( fdpixel ) << ","
           << Fd::GetEyePixelNo( fdpixel ) << "," 
           << Fd::GetMirrorPixelNo( fdpixel ) << endl;
    }
  }
}

// -------------------------------------------------------------------------

void TEyePixelList::Clear(Option_t *)
 {
  fStreamerVersion = 0; 
  fNPixels = 0;

  fPixelVectorSize = Fd::kEYE_NEXTPIXELS/(8 * sizeof(unsigned int));

  for ( unsigned int i=0; i<fPixelVectorSize; i++ )
    fPixelVector[i] = 0;
}

// -------------------------------------------------------------------------

void TEyePixelList::ConstructPixelList()
 {
  Fd::FdPixelNumberRec pixel;
  unsigned int wordpos, mask, no = 0;
  
  fPixelList.clear();
    
  for ( unsigned int i=0; i<Fd::kEYE_NEXTPIXELS; i++ ) {
    
    wordpos = i / (sizeof(unsigned int) * 8);
    mask = 1 << (i % (sizeof(unsigned int) * 8));
  
    if ( fPixelVector[wordpos] & mask ) {  // scan the bit vector

      if ( TEyeEvent::fgPrintLevel >= 2 )
        FD_COUT << "Pixel " << i << " is set !" << endl;

      // --- range for all Eyes: 1 .. 2880
      Fd::SetPixelNo( &pixel, i + Fd::kEYE_FIRST_PIXEL );
      
      // --- mask in the correct Eye Id
      Fd::SetEyeNo( &pixel, fEyeNo + Fd::kFD_FIRST_EYE );

      if ( TEyeEvent::fgPrintLevel >= 2 )
        FD_COUT << "TEyePixelList:: adding pixel " << i 
	        << " as no. " << no << endl;

      no++;

      fPixelList.push_back( pixel );
    }
  }
    
  if ( fPixelList.size() != fNPixels ) {
    FD_CFATAL << "TEyePixelList::ConstructPixelList() - "
              << "Size mismatch whilst constructing of internal PixelList !" 
	      << endl;
    FD_CFATAL << "Size= " << fPixelList.size() 
              << " vs. expected size= " << fNPixels << endl;
    exit( EXIT_FAILURE );
  }
}

// -------------------------------------------------------------------------

Fd::FdPixelNumber TEyePixelList::GetPixel(unsigned int sequence)
 {
  if ( sequence >= fNPixels ) return NULL;
  
  // setup eventually the internal pixel list
  
  if ( fPixelList.empty() ) ConstructPixelList();
  
  if ( sequence >= fPixelList.size() ) return NULL;

#if 1
  // Attention !!! address will become invalid after adding new pixels

  return &fPixelList[sequence];
#else
  // Attention !!! only one instance per class !

  fPixelNumber = fPixelList[sequence];
  
  return &fPixelNumber;
#endif
}

// -------------------------------------------------------------------------

void TEyePixelList::GetPosition(Fd::FdPixelNumber pixel, UInt_t* word, UInt_t *bit) const
 {
  // don't check for the Eye number ... ?
  
  *word = Fd::GetEyePixelNo( pixel ) / (sizeof( UInt_t ) * 8);
  *bit  = Fd::GetEyePixelNo( pixel ) % (sizeof( UInt_t ) * 8);
}

// -------------------------------------------------------------------------

void TEyePixelList::RemovePixel(Fd::FdPixelNumber pixel)
 {
  unsigned int word, bit;
  
  GetPosition( pixel, &word, &bit );
  
  unsigned int mask = 1 << bit;
  
  if ( Fd::GetEyePixelNo( pixel ) > Fd::kEYE_NEXTPIXELS ) {
    FD_CWARN << "Tried to remove illegal pixel " 
             << Fd::GetPixelNo( pixel ) << endl;
    return;
  }

  // pixel has been set before ?
  
  if ( !fPixelVector[word] & mask ) {
    FD_CWARN << "TEyePixelList::RemovePixel(" << Fd::GetEyePixelNo( pixel ) 
             << "): " << "Tried to remove non-existing pixel from the list !" 
	     << endl;
  }
  else {
    fPixelVector[word] &= ~mask;

    fNPixels--;
  }
}
 
// -------------------------------------------------------------------------

void TEyePixelList::SetEyeNo(unsigned int eye_no)
 {
  if ( TEyeEvent::fgPrintLevel ) {
    FD_COUT << "TEyePixelList::SetEyeNo(" << eye_no << ")" << endl;
  }
  
  if ( eye_no < Fd::kFD_FIRST_EYE || eye_no > Fd::kFD_LAST_EYE ) {
    FD_CWARN << "TEyePixelList::SetEyeNo(" << eye_no << ") - " 
             << "EyeNo is outside valid range "
             << Fd::kFD_FIRST_EYE << " .. " << Fd::kFD_LAST_EYE << " !" 
	     << endl;
    return;
  }
  
  fEyeNo = eye_no - Fd::kFD_FIRST_EYE;
}

// -------------------------------------------------------------------------

void TEyePixelList::Show(ostream& ostr)
 {
  ostr << "--- TEyePixelList ---" << endl;
  
  ostr << " Eye number      : " << fEyeNo + Fd::kFD_FIRST_EYE << endl;
  ostr << " Number of pixels: " << fNPixels << endl;
  ostr << " Streamer version: " << fStreamerVersion << endl;
  ostr << " Class version   : " << Class_Version() << endl;

  ostr << " Pixel numbers   : ";

  for ( unsigned int i=0; i<GetNumPixels(); i++ ) {
    Fd::FdPixelNumber fdpixel = GetPixel( i );
    if ( i && !(i % 10) ) {
      ostr << endl;
      ostr << "                   ";
    }
//    ostr << setw(6) << Fd::GetEyePixelNo( fdpixel ); // 1 .. 2880
    ostr << setw(6) << Fd::GetPixelNo( fdpixel );    // 1 .. 11520
  }
  
  ostr << endl;
  
  ostr << "---------------------" << endl;
}

// -------------------------------------------------------------------------

void TEyePixelList::Streamer(TBuffer &R__b)
 {
  // Stream an object of class TEyePixelList.

  UInt_t R__s, R__c;
  
  if ( R__b.IsReading() ) {
    
    Version_t R__v = fStreamerVersion = R__b.ReadVersion(&R__s, &R__c);
    
    if ( R__v == EyeEVENTVERSIONv2 ) {
    
      TObject::Streamer(R__b);

      if ( TEyeEvent::fgPrintLevel )
        FD_CINFO << "TEyePixelList::Streamer() - fix for version= 2 !" << endl;
    
      R__b >> fNPixels;
      R__b >> fPixelVectorSize;
      
      delete [] fPixelVector; 
      fPixelVector = new UInt_t[fPixelVectorSize];
      
      R__b.ReadFastArray(fPixelVector,fPixelVectorSize); 
      R__b.CheckByteCount(R__s, R__c, TEyePixelList::IsA());
    }
    // new Streamer scheme of ROOT v 3.x
    else if ( R__v > EyeEVENTVERSIONv2 ) {
      TEyePixelList::Class()->ReadBuffer(R__b, this, R__v, R__s, R__c);
    }
    else { // no support for *** VERY OLD *** versions !
      FD_CERR << "TEyePixelList::Streamer() - version == 1 not streamable !"
              << endl;
    }

    if ( TEyeEvent::fgPrintLevel > 1 ) {
      FD_CINFO << "TEyePixelList::Streamer() - "
               << "read fStreamerVersion= " << fStreamerVersion
               << ", fEyeNo= " << fEyeNo << endl;
    }
  }
  else { // support new Streamer scheme of ROOT v 3.x
    
    TEyePixelList::Class()->WriteBuffer(R__b, this);
  
    if ( TEyeEvent::fgPrintLevel > 1 ) {
      FD_CINFO << "TEyePixelList::Streamer() - "
               << "wrote fEyeNo= " << fEyeNo << endl;
    }
  }
}

// -------------------------------------------------------------------------

bool TEyePixelList::TestPixel(Fd::FdPixelNumber pixel)
 {
  unsigned int word, bit;
  
  GetPosition( pixel, &word, &bit );
  
  unsigned int mask = 1 << bit;
  
  if ( word >= fPixelVectorSize ) return false;
  
  return ((fPixelVector[word] & mask) > 0 ) ? true : false;
}

ClassImp(TEyePixelList)

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
