
//
// File  : EyeRunTrailer.cc
//
// Purpose: Declaration file for class TEyeRunTrailer
//
// $Id: EyeRunTrailer.cc 8757 2010-09-27 12:16:09Z mathes $
//

#include <iostream>
#include <cassert>


/** @file EyeRunTrailer.cc
  * Declaration file for class TEyeRunTrailer.
  * @author H.-J. Mathes, FzK
  */

#include <TTimeStamp.hh>
#include <FdNumbering.hh>
#include <MiRunTrailer.hh>

#include "CheckVersion.hh"

#include "EyeRunTrailer.hh"

using namespace std;
using namespace FdUtil;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class TEyeRunTrailer - implementation
//

TEyeRunTrailer::TEyeRunTrailer()
 {
  Clear();
}

// -------------------------------------------------------------------------

TEyeRunTrailer::~TEyeRunTrailer()
 { }

// -------------------------------------------------------------------------

TEyeRunTrailer& TEyeRunTrailer::operator=(const TEyeRunTrailer& rhs)
 {
  if ( this == &rhs ) return *this; 

  // now copy all data members, TObject stuff is not copied !
  
  fMirrorStatus         = rhs.fMirrorStatus;
  fNEvents              = rhs.fNEvents;
 
  fVetoTimeCounter      = rhs.fVetoTimeCounter;
  
  fEyeDaqDeadTime       = rhs.fEyeDaqDeadTime;
  
  for ( unsigned int i=0; i<FdUtil::Fd::kEYE_NMIRRORS; i++ ) {
    fDeadTimeCounter[i]        = rhs.fDeadTimeCounter[i];
    fLifeTimeCounter[i]        = rhs.fLifeTimeCounter[i];
    fMirrorDaqDeadTime[i]      = rhs.fMirrorDaqDeadTime[i];
    fMirrorNSkippedTriggers[i] = rhs.fMirrorNSkippedTriggers[i];
  }

  fStartTime            = rhs.fStartTime;
  fStopTime             = rhs.fStopTime;
  
  fTotalTime            = rhs.fTotalTime;
  
  return *this;
}
// -------------------------------------------------------------------------

void TEyeRunTrailer::AddMirrorRunTrailer(const TMirrorRunTrailer* trailer,
                                         unsigned int mirror_id)
 {
  assert( trailer != NULL );

  if ( mirror_id < Fd::kEYE_FIRST_MIRROR || mirror_id > Fd::kEYE_LAST_MIRROR ) {
    cerr << "TEyeRunTrailer::AddMirrorRunTrailer(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
  }
  
  // should be the same for all telescopes, what if not ?
  SetVetoTimeCounter( trailer->GetVetoTimeCounter() );

  SetDeadTimeCounter( trailer->GetDeadTimeCounter(), mirror_id );
  // ??? SetLifeTimeCounter( );
  SetMirrorDaqDeadTime( trailer->GetDaqDeadTime(), mirror_id );
#warning incomplete !!!  
  //SetMirrorNSkippedTriggers( trailer->GetDaqNSkippedTriggers(), mirror_id );
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::AddToLifeTimeCounter(int life_time,unsigned int mirror_id)
 {
  if ( mirror_id < Fd::kEYE_FIRST_MIRROR || mirror_id > Fd::kEYE_LAST_MIRROR ) {
    cerr << "TEyeRunTrailer::AddToTimeCounter(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
  }

  fLifeTimeCounter[mirror_id - Fd::kEYE_FIRST_MIRROR] += life_time;
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::Clear(Option_t *)
 {
  FdRoot::TTimeStamp t((time_t)0,0); 
  fStartTime = fStopTime = t;
  
  fMirrorStatus    = 0;
  fNEvents         = 0;
  fTotalTime       = 0;
  fVetoTimeCounter = 0;

  for ( unsigned int i=0; i<Fd::kEYE_NMIRRORS; i++ ) {
    fDeadTimeCounter[i]        = 0LL;
    fLifeTimeCounter[i]        = 0LL;
    fMirrorDaqDeadTime[i]      = 0LL;
    fMirrorNSkippedTriggers[i] = 0;
  }

  fEyeDaqDeadTime  = 0LL;
  fNT3             = 0;
  fNRejectedT3     = 0;
  fTotalT3DeadTime = 0.0;
}

// -------------------------------------------------------------------------

unsigned long long 
TEyeRunTrailer::GetDeadTimeCounter(unsigned int mirror_id) const
 {
  if ( mirror_id >= Fd::kEYE_FIRST_MIRROR && mirror_id <= Fd::kEYE_LAST_MIRROR )
    return fDeadTimeCounter[mirror_id - Fd::kEYE_FIRST_MIRROR];
  else {
    cerr << "TEyeRunTrailer::GetDeadTimeCounter(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
    return 0;
  }
}

// -------------------------------------------------------------------------

unsigned int TEyeRunTrailer::GetLifeTimeCounter(unsigned int mirror_id) const
 {
  if ( mirror_id >= Fd::kEYE_FIRST_MIRROR && mirror_id <= Fd::kEYE_LAST_MIRROR )
    return fLifeTimeCounter[mirror_id - Fd::kEYE_FIRST_MIRROR];
  else {
    cerr << "TEyeRunTrailer::GetLifeTimeCounter(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
    return 0;
  }
}

// -------------------------------------------------------------------------

unsigned long long 
TEyeRunTrailer::GetMirrorDaqDeadTime(unsigned int mirror_id) const
 {
  if ( mirror_id < Fd::kEYE_FIRST_MIRROR || mirror_id > Fd::kEYE_LAST_MIRROR ) {
    cerr << "TEyeRunTrailer::GetMirrorDaqDeadTime(" << mirror_id 
         << "): illegal mirror id passed !" << endl;
    return 0LL;
  }
  else
    return fMirrorDaqDeadTime[mirror_id - Fd::kEYE_FIRST_MIRROR];
}

// -------------------------------------------------------------------------

unsigned int 
TEyeRunTrailer::GetMirrorNSkippedTriggers(unsigned int mirror_id) const
 {
  if ( mirror_id < Fd::kEYE_FIRST_MIRROR || mirror_id > Fd::kEYE_LAST_MIRROR ) {
    cerr << "TEyeRunTrailer::GetMirrorNSkippedTriggers(" << mirror_id 
         << "): illegal mirror id passed !" << endl;
    return 0;
  }
  else
    return fMirrorNSkippedTriggers[mirror_id - Fd::kEYE_FIRST_MIRROR];
}

// -------------------------------------------------------------------------

Bool_t TEyeRunTrailer::IsMirrorPresent(unsigned int mirror_id) const
 {
  if ( mirror_id < Fd::kEYE_FIRST_MIRROR && mirror_id > Fd::kEYE_LAST_MIRROR ) {
    cerr << "TEyeRunTrailer::IsMirrorPresent(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
    return kFALSE;
  }
  
  if ( fMirrorStatus & ( 1 << (mirror_id - Fd::kEYE_FIRST_MIRROR) ) )
    return kTRUE;
  else
    return kFALSE;
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetDeadTimeCounter(unsigned long long dead_time,
                                        unsigned int mirror_id)
 {
  if ( mirror_id >= Fd::kEYE_FIRST_MIRROR && mirror_id <= Fd::kEYE_LAST_MIRROR )
    fDeadTimeCounter[mirror_id - Fd::kEYE_FIRST_MIRROR] = dead_time;
  else {
    cerr << "TEyeRunTrailer::SetDeadTimeCounter(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
  }
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetLifeTimeCounter(unsigned int life_time,
                                        unsigned int mirror_id)
 {
  if ( mirror_id >= Fd::kEYE_FIRST_MIRROR && mirror_id <= Fd::kEYE_LAST_MIRROR )
    fLifeTimeCounter[mirror_id - Fd::kEYE_FIRST_MIRROR] = life_time;
  else {
    cerr << "TEyeRunTrailer::SetLifeTimeCounter(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
  }
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetMirrorDaqDeadTime(unsigned long long dead_time,
                                          unsigned int mirror_id)
 {
  if ( mirror_id >= Fd::kEYE_FIRST_MIRROR && mirror_id <= Fd::kEYE_LAST_MIRROR )
    fMirrorDaqDeadTime[mirror_id - Fd::kEYE_FIRST_MIRROR] = dead_time;
  else {
    cerr << "TEyeRunTrailer::SetMirrorDaqDeadTime(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
  }
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetMirrorNSkippedTriggers(unsigned int n_skipped,
                                               unsigned int mirror_id)
 {
  if ( mirror_id >= Fd::kEYE_FIRST_MIRROR && mirror_id <= Fd::kEYE_LAST_MIRROR )
    fMirrorNSkippedTriggers[mirror_id - Fd::kEYE_FIRST_MIRROR] = n_skipped;
  else {
    cerr << "TEyeRunTrailer::SetMirrorNSkippedTriggers(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
  }
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetMirrorPresent(unsigned int mirror_id)
 {
  if ( mirror_id >= Fd::kEYE_FIRST_MIRROR && mirror_id <= Fd::kEYE_LAST_MIRROR )
    fMirrorStatus |= ( 1 << (mirror_id - Fd::kEYE_FIRST_MIRROR) );
  else {
    cerr << "TEyeRunTrailer::SetMirrorPresent(" << mirror_id << ") - "
         << "illegal Mirror id passed !" << endl;
  }
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetStartTime()
 {
  FdRoot::TTimeStamp t; // now
  fStartTime = t;
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetStopTime()
 {
  FdRoot::TTimeStamp t; // now
  fStopTime = t;
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::SetTotalTime(unsigned int total_time)
 {
  if ( !total_time ) {
    fTotalTime = fStopTime.GetSec() - fStartTime.GetSec();
  }
  else
    fTotalTime = total_time;
}

// -------------------------------------------------------------------------

void TEyeRunTrailer::Show(ostream& ostr) const
 {
  ostr << "--- TEyeRunTrailer ---" << endl;
  
  ostr << " RUN started at  : " << fStartTime.AsString() << endl;
  ostr << " RUN ended at    : " << fStopTime.AsString() << endl;
  ostr << " Total RUN time  : " << fTotalTime << " [s]" << endl;
  ostr << " Number of events: " << fNEvents << endl;
  
  if ( GetNumberOfT3() > 0 ) {
    ostr << " Number of T3    : " << GetNumberOfT3() << endl;
    ostr << "  rejected T3    : " << GetNumberOfRejectedT3() << endl;
    ostr << "  T3 dead time   : " << GetTotalT3DeadTime() << " [s]" << endl;
  }

  ostr << " Mirrors present : ";
  for ( unsigned int i=0; i<Fd::kEYE_NMIRRORS; i++ )
    if ( fMirrorStatus & (1<<i) ) ostr << i + 1 << " ";
  ostr << endl;
  
  ostr << " acc. VETO time : " << setw(20) << fVetoTimeCounter << " [* ]" 
       << endl;
  
  for ( unsigned int i=0; i<Fd::kEYE_NMIRRORS; i++ ) {
  
    if ( fMirrorStatus & (1<<i) ) {
      ostr << " Dead time[" << i + Fd::kEYE_FIRST_MIRROR << "]   : " 
           << setw(20) << fDeadTimeCounter[i] << " [* 100 ns]" 
	   << endl;
      ostr << " Life time[" << i + Fd::kEYE_FIRST_MIRROR << "]   : " 
           << setw(10) << fLifeTimeCounter[i] << " [* sec]" 
	   << endl;
      if ( fMirrorDaqDeadTime[i] > 0.0 )
        ostr << "Daq Dead time[" << i + Fd::kEYE_FIRST_MIRROR << "]: " 
             << setw(20) << fMirrorDaqDeadTime[i] << " [* 100 ns]" 
	     << endl;
      if ( fMirrorNSkippedTriggers[i] > 0 )
        ostr << "Skpd triggers[" << i + Fd::kEYE_FIRST_MIRROR << "]: " 
             << fMirrorNSkippedTriggers[i]
	     << endl;
    }
  }
  
  if ( fEyeDaqDeadTime > 0LL )
    ostr << " Eye DAQ Dead time: " << setw(20) 
         << fEyeDaqDeadTime << " [* 100 ns]" << endl;
}

ClassImp(TEyeRunTrailer)

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
