
//
// File   : MiEvent.cc
//
// Purpose: Implementation of Mirror event classes
//
// $Id: MiEvent.cc 8759 2010-09-27 12:16:36Z mathes $
//


#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>

#include <TClass.h>
#include <TBuffer.h>


/** @file MiEvent.cc
  * Implementation of class TMirrorEvent
  * @author H.-J. Mathes, FzK
  */

#include <ErrorLogging.hh>
#include <FdNumbering.hh>

#include "CheckVersion.hh"

#include "MiEvent.hh"

using namespace std;
using FdUtil::Fd;
using FdUtil::ErrorLogger;
using namespace MiEvent;

// ------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class TMirrorEvent - implementation
//

unsigned int TMirrorEvent::fgPrintLevel = 0;
ErrorLogger::ESeverityLevel TMirrorEvent::fgLoggingLevel = ErrorLogger::eInfo;

// -------------------------------------------------------------------------

TMirrorEvent::TMirrorEvent()
 : fConstructorType(kCREATE),
   fEventHeader(NULL), fPixelData(NULL), fPixelList(NULL), fFADCData(NULL),
   fTriggerData(NULL),
   fSaveOption(kSaveAll)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TMirrorEvent(): " << this << endl;
#endif
}

// -------------------------------------------------------------------------

TMirrorEvent::TMirrorEvent(const TMirrorEvent& rhs)
 : VMirrorEvent(rhs), 
   fConstructorType(kCREATE),
   fEventHeader(NULL), fPixelData(NULL), fPixelList(NULL), fFADCData(NULL),
   fTriggerData(NULL),
   fSaveOption(kSaveAll)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TMirrorEvent(TMirrorEvent&): " << this << endl;
#endif
  *this = rhs;
}

// -------------------------------------------------------------------------

TMirrorEvent::~TMirrorEvent()
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "~TMirrorEvent(): " << this << endl;
#endif
  switch ( fConstructorType ) {
    case kCREATE:
         delete fEventHeader;
         delete fPixelData;
         delete fPixelList;
         delete fFADCData;
	 delete fTriggerData;
	 break;

    case kSHMEVENT:
         break;
	 
    default:
         FD_CERR << "TMirrorEvent::~TMirrorEvent() - unknown fConstructorType= " 
	      << (int)fConstructorType << endl;
  }
}

// -------------------------------------------------------------------------

TMirrorEvent& TMirrorEvent::operator=(const TMirrorEvent& rhs)
 {
  // fConstructorType must not copied !
  
  // fMiEvent must not copied !
  
#if 0
  FD_CDEBUG(1) << "TMirrorEvent::operator=(): lhs   ---   rhs" << endl;

  FD_CDEBUG(1) << "\t\t\t" << hex << fEventHeader << " - " 
       << rhs.fEventHeader << dec << endl;
  FD_CDEBUG(1) << "\t\t\t" << hex << fPixelList << " - " 
       << rhs.fPixelList << dec << endl;
  FD_CDEBUG(1) << "\t\t\t" << hex << fPixelData << " - " 
       << rhs.fPixelData << dec << endl;
  FD_CDEBUG(1) << "\t\t\t" << hex << fFADCData << " - " 
       << rhs.fFADCData << dec << endl;
  FD_CDEBUG(1) << "\t\t\t" << hex << fTriggerData << " - " 
       << rhs.fTriggerData << dec << endl;
#endif

  // copy the other sub-objects (be aware, pointers might be NULL !) ...
 
  if ( rhs.fEventHeader ) { 
    TMirrorEventHeader *header = GetEventHeader();
    *header = *rhs.fEventHeader;
  }
  else {
    delete fEventHeader;
    fEventHeader = NULL;
  }
  
  if ( rhs.fPixelData ) {
    TMirrorPixelData *pdata = GetPixelData();
    *pdata = *rhs.fPixelData;
  }
  else {
    delete fPixelData;
    fPixelData = NULL;
  }
  
  if ( rhs.fPixelList ) {
    TMirrorPixelList *plist = GetPixelList();
    *plist = *rhs.fPixelList;
  }
  else {
    delete fPixelList;
    fPixelList = NULL;
  }
  
  if ( rhs.fFADCData ) {
    TMirrorFADCData *fadc = GetFADCData();
    *fadc = *rhs.fFADCData;
  }
  else {
    delete fFADCData;
    fFADCData = NULL;
  }

  if ( rhs.fTriggerData ) {
    TMirrorTriggerData *tdata = GetTriggerData();
    *tdata = *rhs.fTriggerData;
  }
  else {
    delete fTriggerData;
    fTriggerData = NULL;
  }
  
  fSaveOption = rhs.fSaveOption;
  
  return *this;
}

// -------------------------------------------------------------------------

void TMirrorEvent::Clear(Option_t* /* option */)
 {
  if ( fEventHeader ) fEventHeader->Clear();
  if ( fTriggerData ) fTriggerData->Clear();
  if ( fPixelData ) fPixelData->Clear();
  if ( fPixelList ) fPixelList->Clear();
  if ( fFADCData ) fFADCData->Clear();
}

// -------------------------------------------------------------------------

TMirrorEventHeader* TMirrorEvent::GetEventHeader()
 {
  if ( !fEventHeader ) 
    fEventHeader = new TMirrorEventHeader();

  if ( !fEventHeader ) {
    FD_CERR << "TMirrorEvent::GetEventHeader() - "
            << "new TMirrorEventHeader() failed!" << endl;
  }
  
  return fEventHeader;
}

// -------------------------------------------------------------------------

TMirrorFADCData* TMirrorEvent::GetFADCData()
 {
  if ( !fFADCData )
    fFADCData = new TMirrorFADCData();
  
  if ( !fFADCData ) {
    FD_CERR << "TMirrorEvent::GetFADCData() - "
            << "new TMirrorFADCData() failed!" << endl;
  }
  
  return fFADCData;
}
    
// -------------------------------------------------------------------------

TMirrorPixelData* TMirrorEvent::GetPixelData()
 {
  if ( !fPixelData )
    fPixelData = new TMirrorPixelData();
  
  if ( !fPixelData ) {
    FD_CERR << "TMirrorEvent::GetPixelData() - "
            << "new TMirrorPixelData() failed!" << endl;
  }
  
  return fPixelData;
}
    
// -------------------------------------------------------------------------

TMirrorPixelList* TMirrorEvent::GetPixelList()
 {
  if ( !fPixelList )
    fPixelList = new TMirrorPixelList();

  if ( !fPixelList ) {
    FD_CERR << "TMirrorEvent::GetPixelList() - "
            << "new TMirrorPixelList() failed!" << endl;
  }
  
  return fPixelList;
}

// -------------------------------------------------------------------------

TMirrorTriggerData* TMirrorEvent::GetTriggerData()
 {
  if ( !fTriggerData )
    fTriggerData = new TMirrorTriggerData();
  
  if ( !fTriggerData ) {
    FD_CERR << "TMirrorEvent::GetTriggerData() - "
            << "new TMirrorTriggerData() failed!" << endl;
  }
  
  return fTriggerData;
}

// --------------------------------------------------------------
// this part is required by the VMirrorEvent interface

TFADCData* TMirrorEvent::GetFADCData(FdUtil::Fd::PixelNumber pixel)
 {
  return GetFADCData()->GetPixelFADCData( pixel );
}

// -------------------------------------------------------------------------

TFADCData* TMirrorEvent::GetFADCData(FdUtil::Fd::PixelNumber pixel,
        			    TFADCData::FADCTraceInfo /* info */)
 {
#warning Tbd: must cut FADC data according to FADCTraceInfo !?
  return GetFADCData()->GetPixelFADCData( pixel );
}

// -------------------------------------------------------------------------

TFADCData::PixelMonitorData 
TMirrorEvent::GetMonitorData(FdUtil::Fd::PixelNumber pixel)
 {
  return GetFADCData()->GetMonitorData( pixel );
}

// -------------------------------------------------------------------------

TMirrorEventHeader::EMiTriggerSource TMirrorEvent::GetTriggerSource()
 {
  return GetEventHeader()->GetTriggerSource();
}

// -------------------------------------------------------------------------

void TMirrorEvent::Show(ostream& ostr) const
 { 
  ((TMirrorEvent*)this)->GetEventHeader()->Show( ostr );
  
  if ( fTriggerData ) fTriggerData->Show( ostr );
  if ( fPixelData ) fPixelData->Show( ostr );
  if ( fPixelList ) fPixelList->Show( ostr );
  
  if ( fFADCData && fPixelList ) fFADCData->Show( fPixelList, ostr );
}

// -------------------------------------------------------------------------

/** Streamer method for the TMirrorEvent class.
  * 
  * Streaming a complete event does not allow to access later 
  *       only subcomponents of it ...
  */
void TMirrorEvent::Streamer(TBuffer &b)
 {
  if ( b.IsReading() ) {                 // --- READING FROM STREAMER

    unsigned int b_count, start;
    Version_t vers = b.ReadVersion( &start, &b_count );

    // Note: - v1 and v2 are the same only a type has changed from an enum to
    //         an Int_t which results in a different StreamerInfo checksum
    //       - v3 has the object TMirrorTriggerData added, but it is not
    //         streamed
    //       - v4 inherits from VMirrorEvent, streaming TObject is obsolete
    
    if ( vers > MiEVENTVERSIONv4 ) {
      FD_CSEV << "TMirrorEvent::Streamer() - "
              << "version mismatch ! Expected <= " << MiEVENTVERSIONv3 
              << " found: " << vers << endl;
      return;
    }

    if ( vers <= MiEVENTVERSIONv3 ) {
      TObject::Streamer( b );
    }

    if ( vers >= MiEVENTVERSIONv4 ) {
      VMirrorEvent::Streamer( b );
    }
    
    b >> fSaveOption;

    // FD_CDEBUG(1) << "SaveOption read= " << hex << fSaveOption << dec << endl;
    
    // this accounts for old files ...
    
    if ( fSaveOption & kSaveAll ) {
      
      // v1 and v2
      fSaveOption |= (   kSaveEventHeader | kSavePixelData | kSavePixelList
                       | kSaveFadcData );
      
      // v3 and v4
      if ( vers == MiEVENTVERSIONv3 || vers == MiEVENTVERSIONv4 )
        fSaveOption |= kSaveTriggerData;
    }

    if ( fSaveOption & kSaveEventHeader ) {
      GetEventHeader()->Streamer( b );
    }

    if ( fSaveOption & kSavePixelData ) {
      GetPixelData()->Streamer( b );
      
      // set the MirrorNumber
      
      GetPixelData()->fPixelData->fMirrorNumber = *GetEventHeader()->GetMirrorNumber();
    }

    if ( fSaveOption & kSavePixelList ) {
      GetPixelList()->Streamer( b );
    }

    // Attention: when data is stored in unsorted order an the PixelList is 
    //            sorted after reading from stream, then FADC data gets 
    //            assigned to different pixels ... !

    if ( fSaveOption & kSaveFadcData ) {

      Fd::PixelNumber pixel;

      for ( UInt_t i=0; i<fPixelList->GetNumPixels(); i++ ) {

        pixel = fPixelList->GetPixel(i);
    
        TFADCData* fadc_data = GetFADCData()->GetPixelFADCData( pixel );
        fadc_data->Streamer( b );

        if ( Fd::GetPixelNo( pixel ) != 
	     Fd::GetPixelNo( fadc_data->GetPixelNumber() ) ) {
	  FD_CSEV << "TMirrorEvent::Streamer() - "
	          << "expected pixel no from pixel_list= " 
	          << Fd::GetPixelNo( pixel ) << endl
	          << "                         - found in data: " 
	          << Fd::GetPixelNo( fadc_data->GetPixelNumber() ) << endl;
	  FD_CSEV << "Continue at your own risk !!!" << endl;
	}

//	FD_CDEBUG(1) << "Pixel no from pixel list= " << Fd::GetPixelNo( pixel )
//	             << " from FADC= "
//	             << Fd::GetPixelNo( fadc_data->GetPixelNumber() ) << endl;
	     

        delete fadc_data;     // inefficient !?
      }
    }
    
    if ( fSaveOption & kSaveTriggerData ) {
#warning Tbd: TMirrorTriggerData is not loaded !
      // GetTriggerData()->Streamer( b );
    }

    b.CheckByteCount( start, b_count, TMirrorEvent::IsA() );

#if 0
    FD_CDEBUG(1) << "TMirrorEvent::Streamer() - after reading:" << endl;
    
    FD_CDEBUG(1) << "fEventHeader= " << hex << fEventHeader << dec << endl;
    FD_CDEBUG(1) << "fPixelList=   " << hex << fPixelList << dec << endl;
    FD_CDEBUG(1) << "fPixelData=   " << hex << fPixelData << dec << endl;
    FD_CDEBUG(1) << "fFADCData=    " << hex << fFADCData << dec << endl;
    FD_CDEBUG(1) << "fTriggerData= " << hex << fTriggerData << dec << endl;
#endif

    // do some post-processing of the data ...
  
    // GetPixelList()->Sort();
  } 
  else {                               // --- WRITING TO STREAMER

    // do some pre-processing of the data ...
  
    // GetPixelList()->Sort();
    
    // now stream the data
    
    unsigned int b_count = b.WriteVersion(TMirrorEvent::IsA(), kTRUE);

    // new for version v4
    
    VMirrorEvent::Streamer( b );
    
    // FD_CDEBUG(1) << "SaveOption written= " << hex << fSaveOption << dec << endl;

    b << fSaveOption;
    
    if ( fSaveOption & kSaveAll ) 
      fSaveOption |= (   kSaveEventHeader | kSavePixelData | kSavePixelList
                       | kSaveFadcData | kSaveTriggerData );
    
    if ( fSaveOption & kSaveEventHeader )
      GetEventHeader()->Streamer( b );
    
    if ( fSaveOption & kSavePixelData ) {
      // copy the MirrorNumber
      
      GetPixelData()->SetMirrorNumber( GetEventHeader()->GetMirrorNumber() );

      GetPixelData()->Streamer( b );
    }
      
    if ( fSaveOption & kSavePixelList )
      GetPixelList()->Streamer( b );

    if ( fSaveOption & kSaveFadcData ) {
    
      TMirrorFADCData *adc = GetFADCData(); // force allocation
      
      Fd::PixelNumber pixel;
      for ( UInt_t i=0; i<fPixelList->GetNumPixels(); i++ ) {
        pixel = GetPixelList()->GetPixel(i);
    
        TFADCData* fadc_data = adc->GetPixelFADCData( pixel );
        fadc_data->Streamer( b );
        delete fadc_data;     // inefficient !?
      }
    }
    
    if ( fSaveOption & kSaveTriggerData ) {
#warning Tbd: TMirrorTriggerData is not saved !
      // GetTriggerData()->Streamer( b );
    }
    
    b.SetByteCount( b_count, kTRUE );
  }
}

ClassImp(TMirrorEvent)

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
