
//
// File   : MiFadcData.cc
//
// Purpose: Implementation of Mirror FADC data class TMirrorFADCData
//          and TFADCData
//
// $Id: MiFadcData.cc 8759 2010-09-27 12:16:36Z mathes $
//

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>


/** @file MiFadcData.cc
  * Implementation of class TMirrorFADCData.
  * @author H.-J. Mathes, FzK
  */

#include <TClass.h>
#include <TBuffer.h>

#include <ErrorLogging.hh>
#include <FdNumbering.hh>
#include <FdConfig.hh>

#include "CheckVersion.hh"
#include "MiEvent.hh"

using namespace std;
using FdUtil::Fd;
using namespace MiEvent;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class TMirrorFADCData - implementation
//

TMirrorFADCData::TMirrorFADCData()
 : fConstructorType(kCREATE)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TMirrorFADCData(): " << this << endl;
#endif

  Construct();
}

// -------------------------------------------------------------------------

TMirrorFADCData::TMirrorFADCData(void *ptr)
  : fConstructorType(kMIFADCDATA)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TMirrorFADCData(void*): " << this << endl;
#endif

  fMiFADCData = (MiFADCData)ptr;
}
 
// -------------------------------------------------------------------------

TMirrorFADCData::TMirrorFADCData(const TMirrorFADCData& rhs)
 : TObject(rhs), fConstructorType(kCREATE)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TMirrorFADCData(TMirrorFADCData&): " << this << endl;
#endif

  Construct();
  
  *this = rhs;
}

// -------------------------------------------------------------------------

TMirrorFADCData::~TMirrorFADCData()
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "~TMirrorFADCData(): " << this << endl;
#endif
  switch ( fConstructorType ) {
    case kCREATE:
         Destruct();

    case kSHMEVENT:
    case kMIFADCDATA:
         fMiFADCData = NULL;
         break;

    default:
         FD_CERR << "TMirrorFADCData::~TMirrorFADCData() - "
	         << "unknown fConstructorType= " 
	         << (int)fConstructorType << endl;
  }
}

// -------------------------------------------------------------------------

void TMirrorFADCData::Clear(Option_t* /* option */)
 {
  Init();
  
  TFADCData::FADCData fadc;
  
  unsigned int column, row;
  unsigned int nbins;
  
  for ( column=0; column<Fd::kMI_LAST_COL; column++ ) {
    for ( row=0; row<Fd::kMI_LAST_VCROW; row++ ) {
      fadc = &fMiFADCData->fFadc[row][column];
      nbins = TFADCData::GetNFADCBins( (TFADCData::EFADCTraceResolution)fadc->fTraceInfo.fBits.fResolution );
      memset( fadc->fFadcData, 0x00, 
              nbins * sizeof( TFADCData::FADCDataWordRec ) );
    }
  }
}

// -------------------------------------------------------------------------

void TMirrorFADCData::Construct()
 {
  fMiFADCData = (MiFADCData)malloc( sizeof( MiFADCDataRec ) );;
  if ( fMiFADCData ) {
    Init();
  }
}

// -------------------------------------------------------------------------

void TMirrorFADCData::Destruct()
 {
  free( fMiFADCData );
  fMiFADCData = NULL;
}

// -------------------------------------------------------------------------

TFADCData::FADCData
TMirrorFADCData::_GetFADCData(unsigned int column, unsigned int row) const
 {
  if ( !fMiFADCData ) return NULL;
  if ( column > Fd::kMI_LAST_COL ) return NULL;
  if ( column < Fd::kMI_FIRST_COL ) return NULL;
  if ( row > Fd::kMI_LAST_VCROW ) return NULL;
  if ( row < Fd::kMI_FIRST_ROW ) return NULL;
  
  return &fMiFADCData->fFadc[row - Fd::kMI_FIRST_ROW][column - Fd::kMI_FIRST_COL];
}

// -------------------------------------------------------------------------

void TMirrorFADCData::Init()
 {                                                          /* thjm 19.02.01 */
  int column, row;
  TFADCData::FADCData fadc;
  
  fMiFADCData->fVersion = MiEVENTVERSION;

  Fd::PixelNumberRec pixel;
  
  Fd::SetEyeNo( &pixel, FdUtil::GetEyeId() );
  Fd::SetMirrorNo( &pixel, FdUtil::GetMirrorId() );
  
  for ( column=Fd::kMI_FIRST_COL; column<=Fd::kMI_LAST_COL; column++ ) {

    for ( row=Fd::kMI_FIRST_ROW; row<=Fd::kMI_LAST_VCROW; row++ ) {
  
      fadc = _GetFADCData( column, row );

      if ( fadc ) {

        Fd::SetPixelNoFromColumnRow( &pixel, column, row );
        
	TFADCData::Init( fadc, &pixel );
      }
      else
        FD_CERR << "TMirrorFADCData::Init()- "
	        << "could not access FADC trace (" << row 
	        << "," << column << ")!" << endl;
    }
  }
}

// -------------------------------------------------------------------------

void TMirrorFADCData::SetPixelFADCData(FdUtil::Fd::PixelNumber pixel,
                                       const TFADCData* src_fadc)
 { 
  unsigned int col = Fd::GetColumnNo( pixel );
  unsigned int row = Fd::GetRowNo( pixel );
  
  if ( Fd::kMI_FIRST_COL < col || col > Fd::kMI_LAST_COL ) {
    FD_CERR << "TMirrorFADCData::SetPixelFADCData() - "
            << "invalid col number!" << endl;
    return;
  }

  if ( Fd::kMI_FIRST_ROW < row || row > Fd::kMI_VC2_ROW ) {
    FD_CERR << "TMirrorFADCData::SetPixelFADCData() - "
            << "invalid row number!" << endl;
    return;
  }
  
  TFADCData::FADCData dest_fadc = 
     &fMiFADCData->fFadc[row - Fd::kMI_FIRST_ROW][col - Fd::kMI_FIRST_COL];
  
  *dest_fadc = *(src_fadc->fFADCData);
}

// -------------------------------------------------------------------------

void TMirrorFADCData::Show(ostream& ostr) const
 {
  TMirrorBitArray bit_array;
  unsigned int col_data = 0;
  
  for ( unsigned int row=0; row<Fd::kMI_NEXTROWS; row++)
    col_data |= (1 << row);
    
  for ( unsigned int col=0; col<Fd::kMI_NCOLS; col++ )
    bit_array.SetColumnData( col + Fd::kMI_FIRST_COL, col_data );
    
  TMirrorPixelList plist( &bit_array );
  
  Show( &plist, ostr );
}

// -------------------------------------------------------------------------

void TMirrorFADCData::Show(TMirrorPixelList* plist,ostream& ostr) const
 {
  if ( !plist ) {
    FD_CERR << "TMirrorFADCData::Show('TMirrorPixelList') - "
            << "'plist' pointer is NULL!" << endl;
    return;
  }
  
  ostr << "--- TMirrorFADCData ---" << endl;

  ostr << " Streamer version : " << fMiFADCData->fVersion << endl;
  ostr << " Class version    : " << Class_Version() << endl;
  
  Fd::PixelNumber pixel;
  
  for ( unsigned int npix=0; npix<plist->GetNumPixels(); npix++ ) {
  
    pixel = plist->GetPixel( npix );
    
    TFADCData* fadc = GetPixelFADCData( pixel );
    
    if ( fadc ) fadc->Show( ostr );
    
    delete fadc;
  }
}

// -------------------------------------------------------------------------

void TMirrorFADCData::Show(TMirrorPixelData* pdata,ostream& ostr) const
 {
  if ( !pdata ) {
    FD_CERR << "TMirrorFADCData::Show('TMirrorPixelData') - "
            << "'pdata' pointer is NULL!" << endl;
    return;
  }
  
  TMirrorPixelList plist( pdata );
  
  Show( &plist, ostr );
}

// -------------------------------------------------------------------------

/** Stream a TMirrorFADCData object (SA). */
void TMirrorFADCData::Streamer(TBuffer &b)
 {
#ifdef __ppc__
# warning TMirrorFADCData::Streamer() - no conversion done for PPC!
#endif // __ppc__
  if ( b.IsReading() ) {
  
    Version_t vers = b.ReadVersion();
    TObject::Streamer( b );
    
    if ( vers > MiEVENTVERSIONv3 ) {
      FD_CSEV << "TMirrorFADCData::Streamer() - version mismatch!" << endl;
      return;
    }
    
    b >> fMiFADCData->fVersion;
    
    TFADCData::FADCData fadc;

    for ( int i=0; i<Fd::kMI_NEXTROWS; i++ ) { 
      
      for ( int j=0; j<Fd::kMI_NCOLS; j++ ) {

        fadc = &fMiFADCData->fFadc[i][j];
	
        b >> fadc->fPixelNumber;
	
        if ( vers >= MiEVENTVERSIONv3 ) {
          b >> fadc->fTraceInfo.fWord;
        }
        else if ( vers == MiEVENTVERSIONv2 ) {
          TFADCData::FADCTraceInfoV3Rec trace_infoV3;
	  
          b >> trace_infoV3.fWord;
	  
	  fadc->fTraceInfo.fBits.fStartBin   = trace_infoV3.fBits.fStartBin;
	  fadc->fTraceInfo.fBits.fEndBin     = trace_infoV3.fBits.fEndBin;
	  fadc->fTraceInfo.fBits.fResolution = TFADCData::kFADC_RESOLUTION_1000_BINS;
	}
        else {
          fadc->fTraceInfo.fBits.fStartBin   = 0;
          fadc->fTraceInfo.fBits.fEndBin     = Fd::kMI_NFADC_BINS_v3 - 1;
          fadc->fTraceInfo.fBits.fResolution = TFADCData::kFADC_RESOLUTION_1000_BINS;
	}

        b >> fadc->fMonitor.fPrevThreshold;
        b >> fadc->fMonitor.fActualThreshold;
        b >> fadc->fMonitor.fActualRate;
	
	if ( vers >= MiEVENTVERSIONv2 ) {
          b >>  fadc->fMonitor.fMean;
          b >>  fadc->fMonitor.fVariance;
	}
	
        // need to read 'fTraceInfo' first !
	
	int start_idx = fadc->fTraceInfo.fBits.fStartBin;
	int end_idx = fadc->fTraceInfo.fBits.fEndBin;
    
	b.ReadFastArray( &fadc->fFadcData[start_idx].fWord,
	                  end_idx - start_idx + 1 );
      }
    }
  }
  else {
    b.WriteVersion( TMirrorFADCData::IsA() );
    
    TObject::Streamer( b );
    
    TFADCData::FADCData fadc;

    for ( int i=0; i<Fd::kMI_NEXTROWS; i++ ) { 

      for ( int j=0; j<Fd::kMI_NCOLS; j++ ) {

        fadc = &fMiFADCData->fFadc[i][j];
	
        b <<  fadc->fPixelNumber;
	
        b <<  fadc->fTraceInfo.fWord;
	
        b <<  fadc->fMonitor.fPrevThreshold;
        b <<  fadc->fMonitor.fActualThreshold;
        b <<  fadc->fMonitor.fActualRate;
        b <<  fadc->fMonitor.fMean;
        b <<  fadc->fMonitor.fVariance;
	
	int start_idx = fadc->fTraceInfo.fBits.fStartBin;
	int end_idx = fadc->fTraceInfo.fBits.fEndBin;

	b.WriteFastArray( &fadc->fFadcData[start_idx].fWord, 
	                   end_idx - start_idx + 1 );
      }  // n_cols
    }  // n_rows
  }
}

// -------------------------------------------------------------------------

bool TMirrorFADCData::IsValidPixel(unsigned int col,unsigned int row ) const
 {
  if ( col < Fd::kMI_FIRST_COL || col > Fd::kMI_LAST_COL ) return false;
  if ( row < Fd::kMI_FIRST_ROW || row > Fd::kMI_LAST_VCROW ) return false;
  
  return true;
}

ClassImp(TMirrorFADCData)

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
