
//
// File   : MiId.hh
//
// Purpose: Declaration of the class TMirrorIdentity
//
// $Id: MiId.hh 12082 2011-02-21 17:11:29Z mathes $
//


#ifndef _MiId_hh_
#define _MiId_hh_

#include <TObject.h>

#include <FdNumbering.hh>
#include <MiEvent.hh>

/** @file MiId.hh
  * Declaration of class TMirrorIdentity.
  * @author H.-J. Mathes, FzK
  */


/** Class to transmit the mirror identity.
  *
  * Objects of this type are sent to the Eye event builder when the connection
  * to the event builder is opened in order to inform the event builder about
  * the identity of the connecting mirror.
  */
class TMirrorIdentity : public TObject {
public:
    /** Constructor for the class TMirrorIdentity. If not explicitely specified
      * the mirror id of the newly created object is set to Fd::kEYE_FIRST_MIRROR.
      */
    TMirrorIdentity(UInt_t mirror_id=FdUtil::Fd::kEYE_FIRST_MIRROR)
     : fMirrorId(mirror_id) { }

    /** Virtual destructor of the class TMirrorIdentity.
      */
    virtual ~TMirrorIdentity() { }

    // --- getters ...
    
    /** Get the mirror_id of the actual object. */
    unsigned int GetMirrorId()
     { return fMirrorId; }

    // --- setters ...
    
    /** Set the mirror id contained in the actual object. */
    void SetMirrorId(unsigned int mirror_id)
     { fMirrorId = mirror_id; }

private:
    unsigned int         fMirrorId;
 
    ClassDef(TMirrorIdentity,MiEVENTVERSIONv1)
};

#endif  // _MiId_hh_
