
//
// File   : MiTriggerData.cc
//
// Purpose: Implementation of the class TMirrorTriggerData
//
// $Id: MiTriggerData.cc 8759 2010-09-27 12:16:36Z mathes $
//


#include <iostream>


/** @file MiTriggerData.cc
  * Implementation of the class TMirrorTriggerData.
  * @author H.-J. Mathes, FzK
  */


#include <ErrorLogging.hh>

#include "CheckVersion.hh"

#include "MiEvent.hh"
#include "MiTriggerData.hh"

using namespace std;
using namespace MiEvent;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// 
// class TMirrorTriggerData - implementation
// 

TMirrorTriggerData::TMirrorTriggerData()
 : fConstructorType(kCREATE)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TMirrorTriggerData(): " << this << endl;
#endif

  Construct();
  Init();
}

// -------------------------------------------------------------------------

TMirrorTriggerData::TMirrorTriggerData(void *ptr)
 : fConstructorType(kMITRIGGERDATA)
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "TMirrorTriggerData(): " << this << endl;
#endif

  fTriggerData = (MiTriggerData)ptr;
}

// -------------------------------------------------------------------------

TMirrorTriggerData::~TMirrorTriggerData()
 {
#ifdef TRACE_CONSTRUCTOR
  FD_CINFO << "~TMirrorTriggerData(): " << this << endl;
#endif

  switch ( fConstructorType ) {

    case kCREATE:
         Destruct();
	 break;

    case kSHMEVENT:
    case kMITRIGGERDATA:
         fTriggerData = NULL;
	 break;

    default:
         FD_CERR << "TMirrorTriggerData::~TMirrorTriggerData() - "
	         << "unknown constructor type " 
	         << (int)fConstructorType << endl;
  }
}

// -------------------------------------------------------------------------

TMirrorTriggerData& 
TMirrorTriggerData::operator=(const TMirrorTriggerData& rhs)
 { 
  if ( this == &rhs ) return *this;
  
  memcpy( fTriggerData, rhs.fTriggerData, sizeof( *rhs.fTriggerData ) );
  
  return *this;
}

// -------------------------------------------------------------------------

void TMirrorTriggerData::Clear(Option_t* /* option */)
 {
  Init();
}

// -------------------------------------------------------------------------

void TMirrorTriggerData::Init()
 {
  if ( !fTriggerData ) {
    FD_CFATAL << "TMirrorTriggerData::Init() - "
              << "no structure defined !" << endl;
    throw 1;
  }

  fTriggerData->fVersion       = Class_Version(); 

  fTriggerData->fNPixels       = 0;
  fTriggerData->fPatternNumber = 0;
  fTriggerData->fTriggerColumn = 0;
  fTriggerData->fTriggerRow    = 0;
  
  fTriggerData->fCutVar1       = 0.0;
  fTriggerData->fCutVar2       = 0.0;
}

// -------------------------------------------------------------------------

void TMirrorTriggerData::Show(std::ostream& ostr) const
 {
  if ( fTriggerData ) {
    ostr <<  "-- TriggerData --" << endl;
    ostr << "Version:          " << fTriggerData->fVersion << endl;
    ostr << "Number of pixels: " << fTriggerData->fNPixels << endl;
    ostr << "PatternNumber:    " << (Int_t)fTriggerData->fPatternNumber << endl;
  }
}

// -------------------------------------------------------------------------

/** Stream a TMirrorTriggerData object. 
  */
void TMirrorTriggerData::Streamer(TBuffer& b)
 {
  if ( b.IsReading() ) {
    
    Version_t vers = b.ReadVersion();
    TObject::Streamer( b );
    
    if ( vers > MiEVENTVERSIONv1 ) {
      FD_CCRIT << "Version mismatch in TMirrorTriggerData::Streamer() !" << endl;
      FD_CCRIT << "Found version: " << vers << " expected <= " 
               << MiEVENTVERSIONv1 << endl;
      return;
    }
    
    b >>   fTriggerData->fVersion;
    
    b >>   fTriggerData->fNPixels;
    b >>   fTriggerData->fPatternNumber;
    b >>   fTriggerData->fTriggerColumn;
    b >>   fTriggerData->fTriggerRow;
    
    b >>   fTriggerData->fCutVar1;
    b >>   fTriggerData->fCutVar2;
    
    if ( vers >= MiEVENTVERSIONv1 ) {
      // for later extension(s)
    }
  }
  else {
    
    b.WriteVersion( TMirrorTriggerData::IsA() );
    TObject::Streamer( b );

    b <<   fTriggerData->fVersion;
    
    b <<   fTriggerData->fNPixels;
    b <<   fTriggerData->fPatternNumber;
    b <<   fTriggerData->fTriggerColumn;
    b <<   fTriggerData->fTriggerRow;
    
    b <<   fTriggerData->fCutVar1;
    b <<   fTriggerData->fCutVar2;

    // new for MiEVENTVERSIONv2
  }
}

ClassImp(TMirrorTriggerData)

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
