
//
// File   : HoughSel.hh
//
// Purpose: Declaration of the class HoughSel
//
// $Id: HoughSel.hh 11539 2010-09-30 14:31:54Z mathes $
//



/** @file HoughSel.hh
  * Declaration of the class HoughSel.
  * @author F. Salamida, INFN
  */


#ifndef _HoughSel_hh_
#define _HoughSel_hh_

// --- forward declaration(s)

class TTRee;
class TFile;

class TMirrorBitArray;
class TMirrorEventHeader;
class TMirrorPixelList;
class TMirrorPixelData;
class EyeEventClassifier;

/**
  */
class HoughSel {
public:
    /** Constructor for the class HoughSel. 
      *
      * The passed filename is used to compute a name for the
      * diagnostic/debugging TTree.
      */
    HoughSel(const char *name=NULL,bool fill_tree=true);

    /** Virtual destructor of the class HoughSel.  */
    virtual ~HoughSel();
    
    /** Do the Hough analysis.
      *
      * In principle only the TMirrorBitArray is required for this analysis, 
      * the remaining parameters are only for diagnostic purposes.
      */
    void DoHoughSel(TMirrorBitArray&,int,int,int,int);
    
    /**  */
    void FillTree(int);  

    /**  */
    int FindFLTrigger(TMirrorPixelData *,int col,int row);
    
    /**  */
    void Finish();  
    
    /**  */
    TFile * GetHoughFile();
    
    /**  */
    TTree * GetHoughTree();

protected:
    /**  */
    void Cluster(TMirrorBitArray&, int &N);
    
    /**  */
    void Translate(float x[], int &N);
    
private:
    // Output tree variables 
    TFile *fHoughFile;
    TTree *fHoughTree;

    //! Event number
    int    fEventNo;

    //! Number of mirror
    int    fMirror;
    
    //! Number of read-out pixels
    int    fNPixels;    

    //! Number of T1 pixels
    int   nT1;

    //! Number of T1 clusters
    int   nC1;

   //! Cluster coordinates (Azimuth, Elevation) 
    float xh[440];
    float yh[440];

    //! Number of cluster pairs
    int   nPair;

    //! Slope of pair in (Azimuth, Elevation) plane
    float hsl[100000];

    //! Intercept of pair in (Azimuth, Elevation) plane
    float hin[100000];

    //! Number of bins to search for peak
    int   nbin;

    //! Peak value
    float hcmax;

    //! Mean content in window
    float hcwin;

    //! Baseline outside window
    float hcbas;

    //! RMS outside window
    float hcrms;

    //! Angle value from slope
    float hazim;

    //! T3 output parameters to fill analysis tree
    float aziG, azisl, azi0, SDPth, SDPph, SDPphG;
 
    //! T3 Event Class
    int fEventType;
};

#endif  // _HoughSel_hh_

