
//
// File   : T3PixelList.cc
//
// Purpose:Implementation of the class T3PixelList.
//
// $Id: T3PixelList.cc 11539 2010-09-30 14:31:54Z mathes $
//


#include <iostream>
#include <iomanip>
#include <vector>

#include <cstdio>
#include <cstdlib>

/** @file T3PixelList.cc
  * Implementation of the class T3PixelList.
  * @author P. Privitera & P. Facal, INFN Rome
  */

#include <FdNumbering.hh>
#include <ErrorLogging.hh>
#include <FdStdioStream.hh>

#include "T3PixelList.hh"

using namespace std;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class T3PixelList - implementation
//

void T3PixelList::DeleteT3Pixel(unsigned int i)
 {
  if ( !fT3List.empty() && i < fT3List.size() ) {
    delete fT3List[i];
    fT3List.erase(fT3List.begin() + i);
  }
  else {
    FD_CWARN << "T3PixelList::DeleteT3Pixel() - "
             << "denied request to delete pixel " << i << " !" << endl;
  }
}

// -------------------------------------------------------------------------

void T3PixelList::Show(FILE *str) 
 {
  if ( str == NULL ) {
    FD_CERR << "T3PixelList::Show() - FILE is NULL !" << endl;
    return;
  }
  
  FdUtil::OstdioStream ostr( str );
    
  Show( ostr );
}

// -------------------------------------------------------------------------

void T3PixelList::Show(ostream& ostr) 
 {
  T3PixelListIterator iter; 
  
  for ( iter = fT3List.begin(); iter != fT3List.end(); iter++ ) {
    T3PixelSel *sel = *iter;
    ostr << "Col " << setw(2) << sel->GetCol() 
         << " Row " << setw(2) << sel->GetRow() 
	 << " T1 " << setw(2) << sel->GetT1()
	 << " TPulse " << setw(3) << sel->GetTPulse() 
	 << " Integral " << setw(3) << sel->GetIntegral() 
	 << endl;
  }

  ostr.flush(); 
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
