
//
// File   : T3PixelSel.cc
//
// Purpose:Implementation of the class T3PixelSel.
//
// $Id: T3PixelSel.cc 11539 2010-09-30 14:31:54Z mathes $
//


#include <iostream>
#include <cstdio>

/** @file T3PixelSel.cc
  * Implementation of the class T3PixelSel.
  * @author P. Privitera & P. Facal, INFN Rome
  */

#include <FdConfig.hh>
#include <FdNumbering.hh>
#include <ErrorLogging.hh>
#if (FDUTIL_VERSION_CODE >= ModuleVersionCode(3,6,0))
# include <FdGeometry.hh>
#endif // FDUTIL_VERSION_CODE

#include "T3PixelSel.hh"

using namespace std;
using FdUtil::Fd;
using FdUtil::FdConfig;
#if (FDUTIL_VERSION_CODE >= ModuleVersionCode(3,6,0))
using FdUtil::FdGeometry;
#endif // FDUTIL_VERSION_CODE

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class T3PixelSel - implementation
//

#if (FDUTIL_VERSION_CODE >= ModuleVersionCode(4,0,1))

// this works only with the improved functionality available in
// FdGeometry 4.1.0+
T3PixelSel::T3PixelSel(FdGeometry::CameraPointing_t& axis, 
                       int mirror, int col, int row, int t1)
 : fMirror(mirror), fCol(col), fRow(row), fT1(t1), fTPulse(0), fPeak(0.)
 {
  const FdGeometry& geo = FdGeometry::GetInstance( axis );

  fAzimuth   = geo.GetPixelAzimuth( fCol, fRow );
  fElevation = geo.GetPixelElevation( fCol, fRow );
}
#endif // FDUTIL_VERSION_CODE

#if (FDUTIL_VERSION_CODE >= ModuleVersionCode(3,6,1))

// -------------------------------------------------------------------------

// this works only with the improved functionality available in
// FdGeometry 3.6.1+ or 4.0.1+
T3PixelSel::T3PixelSel(int eye, int mirror, int col, int row, int t1)
 : fMirror(mirror), fCol(col), fRow(row), fT1(t1), fTPulse(0), fPeak(0.)
 {
  const FdGeometry& geo = FdGeometry::GetInstance( eye, mirror );
   
  fAzimuth   = geo.GetPixelAzimuth( fCol, fRow );
  fElevation = geo.GetPixelElevation( fCol, fRow );
}
#endif // FDUTIL_VERSION_CODE

#if (FDUTIL_VERSION_CODE >= ModuleVersionCode(4,1,0))

// -------------------------------------------------------------------------

// code for the latest version, a reference to the FdGeometry object must be
// passed in, this will work also with Offline (and with FdUtil 4.0.x)
T3PixelSel::T3PixelSel(FdUtil::FdGeometry& geo,
                       int mirror, int col, int row, int t1)
 : fMirror(mirror), fCol(col), fRow(row), fT1(t1), fTPulse(0), fPeak(0.)
 {
  fAzimuth   = geo.GetPixelAzimuth( fCol, fRow );
  fElevation = geo.GetPixelElevation( fCol, fRow );
}

#elif (   (FDUTIL_VERSION_CODE >= ModuleVersionCode(3,6,0)) \
       && (FDUTIL_VERSION_CODE < ModuleVersionCode(4,1,0)))

// -------------------------------------------------------------------------

// new code: works for >= 3.6.0 when FdGeometry class was in place
T3PixelSel::T3PixelSel(int mirror, int col, int row, int t1)
 : fMirror(mirror), fCol(col), fRow(row), fT1(t1), fTPulse(0), fPeak(0.)
 {
# if (   (FDUTIL_VERSION_CODE == ModuleVersionCode(3,6,0)) \
      || (FDUTIL_VERSION_CODE == ModuleVersionCode(4,0,0)))
  // initial version of class FdGeometry, just one table for all telescopes
  fAzimuth   = FdGeometry::GetPixelAzimuth( fCol, fRow );
  fElevation = FdGeometry::GetPixelElevation( fCol, fRow );
# else
  // this code works vor FdUtil 3.6.1 and above as well as 4.0.1 and above
  // here we access the global table again
  // fAzimuth   = FdGeometry::GetPixelAzimuth0( fCol, fRow );
  // fElevation = FdGeometry::GetPixelElevation0( fCol, fRow );
  
  // but we have to account for different tables somehow
  unsigned int eye = FdUtil::GetEyeId();

  const FdGeometry& geo = FdGeometry::GetInstance( eye, mirror );
  
  fAzimuth   = geo.GetPixelAzimuth( fCol, fRow );
  fElevation = geo.GetPixelElevation( fCol, fRow );
}
# endif // FDUTIL_VERSION_CODE == 3.6.0 || 4.0.0

#else // FDUTIL_VERSION_CODE < 3.6.0

// -------------------------------------------------------------------------

// old code: use local table of az,el constants valid for all telescopes
//           and rotate later the azimuth angle

T3PixelSel::T3PixelSel(int mirror, int col, int row, int t1)
 : fMirror(mirror), fCol(col), fRow(row), fT1(t1), fTPulse(0), fPeak(0.)
 {
  // cout << "T3PixelSel::T3PixelSel(" << fMirror << ") ..." << endl;
  fAzimuth   = gPixelAzimuth[22*(fCol-1)+fRow-1];
  fElevation = gPixelElev[22*(fCol-1)+fRow-1];
}
#endif // FDUTIL_VERSION_CODE

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
