/***************************************************************************
    AKSltPattern.cc  -  description

    begin                : Tue Sep 27 2001
    copyright            : (C) 2001 by Andreas Kopmann
    email                : kopmann@ipe.fzk.de
 ***************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <cmath>

/** @file AkSltPattern.cc
  * Implementation of the class Ak::SltPattern.
  * @author A.Kopmann, IPE
  */

#include "AkSltPattern.hh"


// 0 real view, 1 upside down according to numbering scheme
#define TECHNICAL_VIEW 0
//#define TECHNICAL_VIEW 1

using namespace std;
using namespace Ak;

int SltPattern::track[128][3] = { 
    {0,0,0},  {8,1,1},  {1,8,1},  {1,1,8},  {1,1,1},  // 0
    {10,2,2}, {2,10,2}, {2,2,10}, {2,2,2},  {12,3,3}, // 5
    {3,12,3}, {3,3,12}, {3,3,3},  {8,1,2},  {1,8,2},  // 10
    {1,1,9},  {1,1,6},  {10,2,3}, {2,10,3}, {2,2,11},
    {6,8,6},  {12,3,4}, {3,12,4}, {3,3,13}, {2,3,3},  // 20
    {9,1,1},  {2,8,1},  {2,1,8},  {2,1,1},  {11,2,2},
    {3,10,2}, {3,2,10}, {3,2,2},  {13,3,3}, {4,12,3}, // 30
    {4,3,12}, {4,3,3},  {8,1,6},  {1,8,6},  {1,1,7}, 
    {3,4,13}, {10,2,1}, {2,10,1}, {2,2,9},  {6,1,7},  // 40
    {12,3,2}, {3,12,2}, {3,3,11}, {2,3,11}, {7,1,1},
    {6,8,1},  {6,1,8},  {6,1,1},  {9,2,2},  {1,10,2}, // 50 
    {1,2,10}, {1,2,2},  {11,3,3}, {2,12,3}, {2,3,12},
    {1,10,1}, {8,2,1},  {1,9,1},  {2,12,2}, {1,1,2},  // 60
    {10,3,2}, {2,11,2}, {4,3,13}, {2,2,3},  {12,4,3},
    {3,13,3}, {1,2,9},  {3,3,4},  {9,2,1},  {4,12,4}, // 70
    {1,2,8},  {1,2,1},  {3,2,11}, {3,2,3},  {2,3,10},
    {2,3,2},  {13,3,4}, {3,11,2}, {3,4,12}, {3,4,3},  // 80
    {8,6,1},  {1,7,1},  {2,9,1},  {11,3,2}, {10,1,2},
    {2,9,2},  {11,2,3}, {2,2,1},  {12,2,3}, {3,11,3}, // 90
    {2,1,9},  {3,3,2},  {9,1,2},  {3,10,3}, {1,6,8},
    {1,6,1},  {1,9,2},  {2,8,2},  {2,1,10}, {2,1,2},  // 100
    {4,13,3}, {2,11,3}, {3,2,12}, {3,13,4}, {0,0,0},
    {0,0,0},  {0,0,0},  {0,0,0},  {0,0,0},  {0,0,0},  // 110
    {0,0,0},  {0,0,0},  {0,0,0},  {0,0,0},  {0,0,0},
    {0,0,0},  {0,0,0},  {0,0,0},  {0,0,0},  {0,0,0},  // 120
    {0,0,0},  {0,0,0},  {0,0,0}
};


int SltPattern::vectorEvenRow[19][2] = {
    {0,0}, // 0, center 
    {1,-1}, {1,0},  { 1,1},  {0,1},  {-1,0},  {0,-1},  // 1-6, inner circle
    {0,-2}, {1,-2},  {2,-1}, {2,0},  {2,1},   {1,2},   // 7-18 outer cirle
    {0,2},  {-1,-2}, {-1,1}, {-2,0}, {-1,-1}, {-1,-2}
};


int SltPattern::vectorOddRow[19][2] = {
    {0,0}, // 0, center 
    {0,-1}, {1,0},   {0,1},  {-1,1}, {-1,0},  {-1,-1}, // 1-6, inner circle
    {0,-2}, {1,-2},  {1,-1}, {2,0},  {1,1},   {1,2},   // 7-18 outer cirle
    {0,2},  {-1,-2}, {-2,1}, {-2,0}, {-2,-1}, {-1,-2}
};

// -------------------------------------------------------------------------

void SltPattern::display(uint32_t *mem, int n, int, FILE *fout, int){
  // int iFirst, FILE *fout, int fLen
  
  int i, j, k;
  uint32_t xyProj[20];
  uint32_t tyProj[100];

  // Calculate the xy-projection and yt-projection
  for (i=0;i<20;i++)
    xyProj[i]=0;

  for (k=0;k<100;k++)
    tyProj[k]=0;

  for (k=0;k<n;k++){
    xyProj[k%20] = xyProj[k%20] | (mem[k] & 0x3fffff);
    tyProj[k/20] = tyProj[k/20] | (mem[k] & 0x3fffff);
  }


  // Display header line
  fprintf(fout,"\n");
  fprintf(fout,"   | ");
  for (i=0;i<20;i++)
    if ((i+1)%5 == 0)
       fprintf(fout,"        %2d",i+1);

  fprintf(fout," | ");

  for (k=0;k<100;k++)
    if ((k+1)%10 == 0)
       fprintf(fout,"       %3d",k+1);

  fprintf(fout,"\n");


  // Display 22 data lines
  for (j=0;j<22;j++){
    if ((j+1)%5 == 0)
      fprintf(fout,"%3d|",j+1);
    else
      fprintf(fout,"   |");

    // Start a half pixel later
    if (j%2 == 0) fprintf(fout," ");

    // Display the xy projection
    for (i=0;i<20;i++){

      if ((xyProj[i] & 0x1))
         fprintf(fout,"[]");
      else {
        if ((i+1)%5 == 0)
          fprintf(fout," :");
        else
          fprintf(fout," .");
      }
      
      xyProj[i] = xyProj[i] >> 1;
    }

    // Fill in last half pixel
    if (j%2 == 1) fprintf(fout," ");

    fprintf(fout," | ");

    // Display the yt  projection
    for (k=0;k<100;k++){
      if ((tyProj[k] & 0x1))
         fprintf(fout,"=");
      else {
        if ((k+1)%10 == 0)
          fprintf(fout,":");
        else
          fprintf(fout,".");
      }

      tyProj[k] = tyProj[k] >> 1;
    }

    fprintf(fout,"\n");
  }
}

// -------------------------------------------------------------------------

int SltPattern::getPatternId(uint32_t mem){
  return ( (mem >> 23) & 0x7f); // bits 23-29
}

// -------------------------------------------------------------------------

int SltPattern::getTriggered(uint32_t *mem, int n)
 {
  int i, j, k;
  uint32_t fltTrigger[20];
  int nTriggered;


  // Make a projection of the trigger
  for (i=0;i<20;i++)
    fltTrigger[i] = 0;

    
  for (k=0;k<n;k++){ 
    fltTrigger[k%20] = fltTrigger[k%20] | (mem[k] & 0x3fffff);
  }

  // Count the number of triggered pixel
  nTriggered = 0;
  for (i=0;i<20;i++) {
    for (j=0;j<22;j++) {
      if ((fltTrigger[i] & 0x1) == 1) {
        nTriggered = nTriggered +1;
      }

      fltTrigger[i] = fltTrigger[i] >> 1;
    }
  }

  return nTriggered;
}

// -------------------------------------------------------------------------

int SltPattern::getFirstTrigger(uint32_t *mem, int n){
  int i;

  for (i=0;i<n;i++){
    if ((mem[i] >> 23) & 0x7f){
      return(i);
    }
  }

  return -1;
}

// -------------------------------------------------------------------------

void SltPattern::getFltIntervall(uint32_t *mem, int n, int t0[][22], int t1[][22]){
  int i, j;
  int row, col, t;
  uint32_t buf;


  // Clear Matrices
  for (i=0;i<20;i++) {
    for (j=0;j<22;j++){
      t0[i][j]=-1;
      t1[i][j]=-1;
    }
  }


  // Go through the trigger memory
  for(i=0;i<n;i++){
    // Determine row and time bin from trigger mem entry
    col = i % 20;
    t = i / 20; // 1us

    // Is there any triggered pixel in this column?
    if ( (mem[i] & 0x003fffff) > 0){
      buf = mem[i];

      // Check the triggered pixel in every row
      for (j=0; j<22; j++){
        row = j;

        if ( (buf & 0x1) > 0 ){
          if (t0[col][row] == -1){
            // First trigger in this pixel
            t0[col][row] = t;
            t1[col][row] = t;
          } else {
            // Further trigger, may be last one?
            t1[col][row] = t;
          }
        }

        // Go to next row
        buf = buf >> 1;
      }
    } // if ( (mem[i] & 0x003fffff) > 0)

  } // for (i=..)
}

// -------------------------------------------------------------------------

void SltPattern::getSltIntervall(uint32_t *mem, int n, int *t0, int *t1){
  int i, t;

  // Clear t0, t1
  *t0 = -1;
  *t1 = -1;


  // Go through the trigger memory
  for(i=0;i<n;i++){
    // Determine row and time bin from trigger mem entry
    t = i / 20; // 1us

    // Is there any triggered pixel in this column?
    if ( ((mem[i] >> 23) & 0x7f) > 0 ){

      if (*t0 == -1){
        // First trigger
        *t0 = t;
        *t1 = t;
      } else {
        // Further trigger, may be last one?
        *t1 = t;
      }
    }
  }
}

// -------------------------------------------------------------------------

void SltPattern::getPixelList(int pattern, int col, int row, 
			      int * colList, int *rowList, int){
  // int nList

  int i;
  int vectorId;

  // Every track starts with pixel itself
  colList[0] = col;
  rowList[0] = row;

  // Follow the track
  for (i=0;i<3;i++){
    
    vectorId = track[pattern][i];

    if (rowList[i] %2 == 0) {// even row
      colList[i+1] = colList[i] + vectorEvenRow[vectorId][0]; 
      rowList[i+1] = rowList[i] + vectorEvenRow[vectorId][1]; 
    } else { // odd row
      colList[i+1] = colList[i] + vectorOddRow[vectorId][0]; 
      rowList[i+1] = rowList[i] + vectorOddRow[vectorId][1]; 
    }
  }
}

// -------------------------------------------------------------------------

int SltPattern::getPixelList(uint32_t *mem, int n, int *iTrigger,
			     int *colList, int *rowList, int){

  int i, j, col, row, testRow; 
  int iPattern, iImage, iMulti;
  int patternId;
  int fltTrigger;
  bool multi;

  // Scan for the first occurrence of a trigger
  iPattern = -1;
  patternId = 0;
  multi = false;
  iMulti = 0;
  for (i=0; i<n; i++){
    patternId = getPatternId(mem[i]);
    if ( (patternId > 0)  && (patternId < 127)) {
      iPattern = i;
      break;
    }

    if ((!multi) && (patternId == 127)){
      multi = true;
      iMulti = i;
    }
  }

  // We can't give more information for multiple patterns
  if ( multi ){
    //printf("Information: First multiple pattern found in time bin %d\n",
    //  	iMulti);

    // Try to find some (4 Pixel) that definitly belong to the 
    // multiple track!!! With these the same analysis can be
    // performed! 
    // The first start could be to collect all pixel in this column?!
    // But how to handle that we don't have the direction???
  }

  //printf("Trigger %3d,  time bin %4d", patternId, iPattern);
  if ((patternId == 127)  || (patternId == 0)){
    // printf("... Error: No single pattern found?\n");

    *iTrigger = iMulti;

    return 1; // no single pattern detected
  }

	
  // Get the coordinates of the reference pixel
  // This is always the last pixel in the triggered row
  // With some bend pattern the reference pixel is not in 
  // most left column. In this cases the next column
  // has to be analysed too!
  for (i=0;i<2;i++){
    fltTrigger = mem[iPattern+i] & 0x3fffff; // bits 0-21
    col = (iPattern+i) % 20; 
    iImage = iPattern - iPattern % 20;
    row = -1;
    for (j=0; j<22; j++){
      if ((fltTrigger & 0x1) == 1){
        testRow = j;

	// Get the coordinates of the pixel in the track
        getPixelList(patternId, col, testRow, colList, rowList);

	// Check the validity of the pixel list 
	// Show all pixel in the list realy a first level trigger?
	//printf(", mem %08x, pixel (%2d,%2d)",
	//	mem[iImage + col],col,testRow);

	if (checkPixelList(mem + iImage, n, colList, rowList) == -1) {
 	  row = j;
	  break;
        }

      }

      fltTrigger = fltTrigger >> 1;
    }

    // Does the first iteration has detected the ref. pixel?
    if (row >= 0) break;

  }

  if (row == -1) {
    //printf("Error: No triggered pixel found (col %d, i %d: %08x)\n",
    //         col, iPattern, mem[iPattern]);
    return 2; // Given pattern (SLT) not found in FLT data 
  }


  //printf("\n");

  // Return the column with the first trigger!
  if (multi)
    *iTrigger = iMulti;
  else
    *iTrigger = iPattern;

  return 0; // Successful with single pattern
}

// -------------------------------------------------------------------------

int SltPattern::checkPixelList(uint32_t *mem, int /* n */, 
			       int *colList, int *rowList, int len){
          
  int i, col, row;
  int iErr;

  iErr = -1;
  for (i=0;i<len;i++){

    col = colList[i];
    row = rowList[i];

    // Check for valid coordinates!
    if ((row < 0) || (col < 0)) {
      iErr = i;
      break;
    }

    //printf(" (%2d,%2d)", col, row);

    if ((mem[col] & (0x1 << row)) == 0 ) {
      iErr = i;
      //printf("... Error: Pixel not triggered (%08x - %08x)!\n",
      //	      mem[col], (0x1 << row));
      break;
    }
  }

  return iErr;
}

// -------------------------------------------------------------------------

int SltPattern::findPixelList(uint32_t *mem, int n,
			      int iPattern, int patternId, int firstRow,
			      int *colList, int *rowList, int nList)
 {
  int j, k, kEnd, col;
  uint32_t fltPattern;
  int nPixel = 0;
  int jMin, jFirst, jMax, kMax;

  // Find the pixel for a multi pattern
  // Try to get the first pixel available
  if (patternId == 127) {

    // In the beginning there are no pixel in 
    // the list and all rows are possible
    nPixel = 0;
    jFirst = 22;
    jMin = firstRow;
    jMax = 21;
    kMax = iPattern + 2;
    kEnd = iPattern - iPattern%20 + 20; // Only consider this image

    for (k=iPattern;k<kEnd;k++) {
      fltPattern = mem[k] & 0x3fffff;
      col = k %20;

      for (j=0;j<22;j++) {
	if ((j>=jMin) && (j<=jMax) && (k <= kMax)) {
	  if ((fltPattern & 0x1) == 1) {
	    // Save the coordinates if possible
	    if (nPixel < nList) {

	      if (nPixel < 2) {
		// Store the first two pixel
		colList[nPixel] = col;
		rowList[nPixel] = j;

	      } else{

		// Do not sort, just store the next pixel!
		//colList[nPixel] = col;
		//rowList[nPixel] = j;
					  
		// Contruct the linear track!?
                insertByProjection(col,j,colList,rowList,nPixel);

		// Avoid some disordering - faster ?!
		//insertByDistance(col,j,colList,rowList,nPixel);
	      } 
	    }

	    // Increase the number of pixel
	    nPixel = nPixel + 1;

	    // Define the search area
	    // Restrict it to the neighborhood
	    if (j < jFirst) jFirst = j; // Save the first pixel
	    jMax = j + 2;
	    kMax = k + 2;

	    if (jMax > 21) jMax = 21;

	    //printf("(%d,%d)",col,j);
	  }
        }

	fltPattern = fltPattern >> 1;
      }

      // Calculate the start for the next column
      jMin = jFirst - 3;
      if (jMin < 0) jMin = 0;

    }
  }

  // If there are not enough pixel found 
  // try to find another start point!!!
  if ((nPixel> 0) && (nPixel < 4)) {
    if ((colList[0] == iPattern%20) && (rowList[0] + 3 < 22) )
      nPixel = findPixelList(mem,n,iPattern,patternId,
			     rowList[0] + 3,colList,rowList,nList);
  }

  // Try to find more pixel in the direction of 
  // a initial pattern !?

  return nPixel;
}

// -------------------------------------------------------------------------

int SltPattern::insertByProjection(int col, int row,
			           int *colList, int *rowList, int nList){

  int i, entry;
  int rRef, rProj, rProjNew;

  // Check upper and low bound
  rProjNew = getScalarProd(colList[0],rowList[0],
  		           colList[nList-1],rowList[nList-1], col, row);

  // Replace the first pixel in the list?
  if (rProjNew < 0) {
    entry = 0;
  } else {
	
    // Add at the end of the list?
    rRef = getR2(colList[0],rowList[0],colList[nList-1],rowList[nList-1]);

    if (rProjNew >= rRef){
      entry = nList;
    } else {

      // Scan all pixel in between
      entry = nList - 1;
      for (i=1;i<nList-1;i++){
   
        rProj = getScalarProd(colList[0],rowList[0],
			      colList[nList-1],rowList[nList-1], 
			      colList[i], rowList[i]);

        // Insert before pixel i?
	if (rProjNew < rProj){
	  entry = i;
	  break;
	}
      }
    }
  }

  //printf("Insert (%02d,%02d) at #%d (%02d,%02d) ... (%02d,%02d)\n",
  //	col,row,entry,colList[0],rowList[0],
  //	colList[nList-1],rowList[nList-1]);

  // Insert the new pixel
  for (i=nList;i>entry;i--){
    colList[i] = colList[i-1];
      rowList[i] = rowList[i-1];
  }

  colList[entry] = col;
  rowList[entry] = row;

  // Success
  return 0;
}

// -------------------------------------------------------------------------

int SltPattern::insertByDistance(int col, int row,
			         int *colList, int *rowList, int nList){

  int i, entry;
  int rRef, rNew;

  rNew = getCircle(colList[0],rowList[0],col,row);

  entry = 1;
  for (i=nList-1;i>0;i++){
    rRef = getCircle(colList[0],rowList[0],colList[i],rowList[i]);

    if (rNew >= rRef) {
      entry = i+1;
      break;
    }
  }


  // Insert the new pixel
  for (i=nList;i>entry;i--){
    colList[i] = colList[i-1];
      rowList[i] = rowList[i-1];
  }

  colList[entry] = col;
  rowList[entry] = row;


  // Success
  return 0;
}

// -------------------------------------------------------------------------

int SltPattern::getX(int col, int row){

	if (row % 2 == 0)
		return(  (col << 1)  +1);
	else
		return( col << 1);

}

// -------------------------------------------------------------------------

int SltPattern::getY(int, int row){
  return (row << 1);
}

// -------------------------------------------------------------------------

float SltPattern::getR(int colRef, int rowRef, int col, int row){
  int dx, dy;  

  dx = getX(col, row) - getX(colRef, rowRef);
  dy = getY(col, row) - getY(colRef, rowRef);

  return ((float) sqrt( (double) (dx * dx + dy * dy) ));
}

// -------------------------------------------------------------------------

int SltPattern::getR2(int colRef, int rowRef, int col, int row){
  int dx, dy;  

  dx = getX(col, row) - getX(colRef, rowRef);
  dy = getY(col, row) - getY(colRef, rowRef);

  return (dx * dx + dy * dy);
}

// -------------------------------------------------------------------------

int SltPattern::getCircle(int colRef, int rowRef, int col, int row){
  int drow, dx, r;

  dx = abs (getX(col, row) - getX(colRef, rowRef));
  drow = abs(row - rowRef);

  if ( dx <= drow) {
     // Segment of 45 degree from the vertical axis
  r = drow << 1; 
  } else {
     // Segment of 45 degree from the horizontal axis
  // Here a linear law is used.
     r = dx + drow;
  }

  return r;
}

// -------------------------------------------------------------------------

int SltPattern::getScalarProd(int colRef, int rowRef,
			      int col1, int row1 , int col2, int row2){

  int dx1, dx2, dy1, dy2;

  dx1 = getX(col1, row1) - getX(colRef, rowRef);
  dx2 = getX(col2, row2) - getX(colRef, rowRef);

  dy1 = getY(col1, row1) - getY(colRef, rowRef);
  dy2 = getY(col2, row2) - getY(colRef, rowRef);

  return (dx1 * dx2 + dy1 * dy2);
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
