
//
// File   : AnalysedFadcData.cc
//
// Purpose: Implementation of class TAnalysedFADCData
//
// $Id: AnalysedFadcData.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <iostream>
#include <iomanip>


/** @file AnalysedFadcData.cc
  * Implementation of the class TAnalysedFADCData.
  * @author H.-J. Mathes, FzK
  */

#include <FdNumbering.hh>
#include <ErrorLogging.hh>

#include <MiEvent.hh>

#include "PFPedro.hh"
#include "AnalysedFadcData.hh"

using namespace std;
using namespace FdUtil;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class TAnalysedADCData - implementation
//

/** @todo
  * need reference counting to delete the default pulse finder ...
  */
Int_t TAnalysedFADCData::fgPrintLevel = 0;

VPulseFinder* TAnalysedFADCData::fgDefaultPulseFinder = NULL;

TAnalysedFADCData::TAnalysedFADCData(TFADCData* fadc)
 : TFADCData(*fadc), fPulseFinder(NULL), 
   fCalculatedPulseTime(0), fPulseFound(false)
 {
  if ( fgDefaultPulseFinder ) {
    fPulseFinder = fgDefaultPulseFinder->Clone();
  }
  else {
    fPulseFinder = new PFPedro();
  }
}

// -------------------------------------------------------------------------

TAnalysedFADCData::~TAnalysedFADCData()
 {
  if ( fgPrintLevel > 1 )
    FD_COUT << "~TAnalysedFADCData" << endl;

  delete fPulseFinder;
}

// -------------------------------------------------------------------------

void TAnalysedFADCData::Analyse()
 {
  FindPulse();
}

// -------------------------------------------------------------------------

// currently this method is not called
void TAnalysedFADCData::CalculatePulseTime(int timestamp_low)
 {
  int nxpg_time =   (timestamp_low & 0x7ff) * 50
                  + ((timestamp_low >> 11) & 0x3fff ) * 10000;
  int t0 = nxpg_time - GetNFADCBins() * 100;
  
  // possibly there are more pulses ...
  
  if ( int n_pulses = fPulseFinder->GetNPulses() ) {
    fPulseFound = true;
    
    if ( n_pulses > 1 ) {
      FD_COUT << "... " << n_pulses << " pulses found !" << endl;
    }
    else {
      fCalculatedPulseTime = fPulseFinder->GetPulseStart() * 100 + t0;
    }
  }
  
  // for ( Int_t i=0; i<n_pulses; i++ ) {
  if ( fgPrintLevel >= 1 )
    FD_COUT << "Pulse[" << 0 << "] at " 
            << fCalculatedPulseTime << " nsec" << endl;
  // }
}

// -------------------------------------------------------------------------

void TAnalysedFADCData::FindPulse()
 {
  fPulseFound = false;
  fCalculatedPulseTime = 0;
  
  fPulseFinder->FindPulse( this );
  
  if ( GetNPulses() > 0 ) {
    fPulseFound = true;
    
  //  CalculatePulseTime();  // ???
  }
}

// -------------------------------------------------------------------------

void TAnalysedFADCData::SetPulseFinder(VPulseFinder* finder)
 {
  if ( fgDefaultPulseFinder != NULL )
    delete fgDefaultPulseFinder;
    
  fgDefaultPulseFinder = finder;
}

// -------------------------------------------------------------------------

void TAnalysedFADCData::Show(std::ostream& ostr) const
 {
  ostr << "Pixel " <<  Fd::GetPixelNo( GetPixelNumber() ) 
       << " has " << GetNPulses() << " pulse(s)." << endl;
  ostr << "\tPedestal = " << GetPedestal() << endl;
  ostr << "\tSigma    = " << GetSigmaPedestal() << endl;

  if ( GetNPulses() > 0 )
    ostr << "\tPulse  Start  Integral    Time" << endl;
    
  for ( int n=0; n<GetNPulses(); n++ ) {
    ostr << "\t " << setw(2) << n;
    ostr << "    " << setw(4) << GetPulseStart(n);
    ostr << "  " << setw(8) << setprecision(2) << GetPulseIntegral(n);
    ostr << " " << setw(10) << GetPulseTime() << endl;
  }
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
