
//
// File   : ExtendedPixelList.cc
//
// Purpose: Implementation file for class ExtendedPixelList
//
// $Id: ExtendedPixelList.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <iostream>
#include <algorithm>  // min()


/** @file ExtendedPixelList.cc
  * Implementation of the class ExtendedPixelList.
  * @author H.-J. Mathes, Fzk
  */

#include <FdNumbering.hh>
#include <MiEvent.hh>

#include "CheckVersion.hh"

#include "ExtendedPixelList.hh"  

using namespace std;
using FdUtil::Fd;

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
//
// class ExtendedPixelList - implementation
//

ExtendedPixelList::ExtendedPixelList()
 {
  fTraceInfo = new TFADCData::FADCTraceInfoRec[Fd::kMI_NEXTPIXELS];

  unsigned int n_fadc_bins = TFADCData::GetNFADCBins( TFADCData::GetDefaultResolution() );
  
  for ( unsigned int i = 0; i<Fd::kMI_NEXTPIXELS; i++ ) {
    FADCTraceSetStartBin( &fTraceInfo[i], 0 );
    FADCTraceSetEndBin( &fTraceInfo[i], n_fadc_bins - 1 );
  }
}

// --------------------------------------------------------------------------

ExtendedPixelList::ExtendedPixelList(TMirrorPixelList* pixel_list)
 : TMirrorPixelList( *pixel_list ), fTraceInfo(NULL)
 {
  fTraceInfo = new TFADCData::FADCTraceInfoRec[Fd::kMI_NEXTPIXELS];

#warning must set TFADCData::fgDefaultResolution before
  // Note: It is assumed that TFADCData::fgDefaultResolution has been set
  //       properly before, for instance from ini-file
  unsigned int n_fadc_bins = TFADCData::GetNFADCBins( TFADCData::GetDefaultResolution() );
  
  for ( unsigned int i = 0; i<GetNumPixels(); i++ ) {
    FADCTraceSetStartBin( &fTraceInfo[i], 0 );
    FADCTraceSetEndBin( &fTraceInfo[i], n_fadc_bins - 1 );
  }
}

// --------------------------------------------------------------------------

ExtendedPixelList::ExtendedPixelList(TMirrorPixelList* pixel_list,
                                     unsigned int start, unsigned int end)
 : TMirrorPixelList( *pixel_list ), fTraceInfo(NULL)
 {
  fTraceInfo = new TFADCData::FADCTraceInfoRec[Fd::kMI_NEXTPIXELS];

#warning must set TFADCData::fgDefaultResolution before
  // Note: It is assumed that TFADCData::fgDefaultResolution has been set
  //       properly before, for instance from ini-file
  unsigned int n_fadc_bins = TFADCData::GetNFADCBins( TFADCData::GetDefaultResolution() );

  for ( unsigned int i = 0; i<GetNumPixels(); i++ ) {
    if ( start > end || start > (n_fadc_bins - 1) )
      FADCTraceSetStartBin( &fTraceInfo[i], 0 );
    else
      FADCTraceSetStartBin( &fTraceInfo[i], start );
    FADCTraceSetEndBin( &fTraceInfo[i], min(n_fadc_bins - 1,end) );
  }
}

// --------------------------------------------------------------------------

ExtendedPixelList::ExtendedPixelList(TMirrorBitArray* bit_array,
                                     unsigned int start, unsigned int end)
 : TMirrorPixelList( bit_array ), fTraceInfo(NULL)
 {
  fTraceInfo = new TFADCData::FADCTraceInfoRec[Fd::kMI_NEXTPIXELS];

#warning must set TFADCData::fgDefaultResolution before
  // Note: It is assumed that TFADCData::fgDefaultResolution has been set
  //       properly before, for instance from ini-file
  unsigned int n_fadc_bins = TFADCData::GetNFADCBins( TFADCData::GetDefaultResolution() );

  for ( unsigned int i = 0; i<GetNumPixels(); i++ ) {
    if ( start > end || start > (n_fadc_bins - 1) )
      FADCTraceSetStartBin( &fTraceInfo[i], 0 );
    else
      FADCTraceSetStartBin( &fTraceInfo[i], start );
    FADCTraceSetEndBin( &fTraceInfo[i], min(n_fadc_bins - 1,end) );
  }
}

// --------------------------------------------------------------------------

ExtendedPixelList::~ExtendedPixelList()
 {
  delete [] fTraceInfo;
}

// --------------------------------------------------------------------------

unsigned int ExtendedPixelList::GetStartBin(unsigned int seq)
 {
  if ( seq < GetNumPixels() )
    return FADCTraceGetStartBin( &fTraceInfo[seq] );
  else
    return 0;
}

// --------------------------------------------------------------------------

unsigned int ExtendedPixelList::GetEndBin(unsigned int seq)
 {
  if ( seq < GetNumPixels() )
    return FADCTraceGetEndBin( &fTraceInfo[seq] );
  else
    return 0;
}

// --------------------------------------------------------------------------

void ExtendedPixelList::SetStartBin(int start_bin)
 {
  for ( unsigned int i = 0; i<GetNumPixels(); i++ ) {
    FADCTraceSetStartBin( &fTraceInfo[i], start_bin );
  }
}
  
// --------------------------------------------------------------------------

void ExtendedPixelList::SetStartBin(unsigned int seq,int start_bin)
 {
  if ( seq < GetNumPixels() )
    FADCTraceSetStartBin( &fTraceInfo[seq], start_bin );
}
  
// --------------------------------------------------------------------------

void ExtendedPixelList::SetEndBin(int end_bin)
 {
  for ( unsigned int i = 0; i<GetNumPixels(); i++ ) {
    FADCTraceSetEndBin( &fTraceInfo[i], end_bin );
  }
}
  
// --------------------------------------------------------------------------

void ExtendedPixelList::SetEndBin(unsigned int seq,int end_bin)
 {
  if ( seq < GetNumPixels() )
    FADCTraceSetEndBin( &fTraceInfo[seq], end_bin );
}
  
// --------------------------------------------------------------------------

void ExtendedPixelList::Show(std::ostream& ostr) const
 {
  Fd::PixelNumber pix;
  
  ostr << "-- ExtendedPixelList --" << endl;
  ostr << "Version:      " << GetVersion() << endl;
  ostr << "ActualSize:   " << GetSize() << endl;
  ostr << "MaxSize:      " << GetMaxSize() << endl;
  ostr << "NPixels:      " << GetNumPixels() << endl;
    
  for ( unsigned int i=0; i<GetNumPixels(); i++ ) {
    pix = GetPixel( i );
    ostr << " - Pixel[" << i << "] = " << Fd::GetPixelNo( pix ) << endl;
  }
}
  
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
