
//
// File   : FEMiEvent.hh
//
// Purpose: Implementation of the VMirrorEvent interface for FE access
//
// $Id: FEMiEvent.hh 9725 2010-09-28 07:31:17Z mathes $
//



/** @file FEMiEvent.hh
  * Declaration of the class FEMiEvent
  * @author R. Ulrich, FzK
  */


#ifndef _FEMiEvent_hh_
#define _FEMiEvent_hh_

#include <VMiEvent.hh>

// --- forward declaration(s)

class TMirrorEventHeader;
class TMirrorPixelData;
class FEPageDesc;
class VirtualReadout;

/** FEMiEvent access for Tlt from all FE type classes
  *
  * This is the implementation of the VMirrorEvent interface to the FE 
  * classes, to be used by the TLT
  *
  * Actually event data might be extracted from:
  *
  * @li the frontend hardware (class FEReadout or class FECachedReadout)
  * @li a file (class FEFileReadout)
  */
class FEMirrorEvent : public VMirrorEvent {
public:
    /**  */
    FEMirrorEvent(VirtualReadout* r, FEPageDesc* p);

    /**  */
    virtual TMirrorEventHeader* GetEventHeader();
    /**  */
    virtual TMirrorPixelData* GetPixelData();   

    /**  */
    virtual TFADCData* GetFADCData(FdUtil::Fd::PixelNumber pixel);
    /**  */
    virtual TFADCData* GetFADCData(FdUtil::Fd::PixelNumber pixel,
				   TFADCData::FADCTraceInfo info);    
    /**  */
    virtual TFADCData::PixelMonitorData GetMonitorData(FdUtil::Fd::PixelNumber);

    /**  */
    virtual TMirrorEventHeader::EMiTriggerSource GetTriggerSource();

private:
    //<<< is implemented by using an implementation of VirtualReadout
    VirtualReadout   *fReadout;
    
    //<<< a 'hook' to the FE (=frontend), eventually only faked
    FEPageDesc       *fPage;
};

#endif  // _FEMiEvent_hh_
