
//
// File   : FEPageDesc.hh
//
// Purpose: Declaration of the class FEPageDesc
//
// $Id: FEPageDesc.hh 9725 2010-09-28 07:31:17Z mathes $
//

  /* Generated by Together */

/** @file FEPageDesc.hh
  * Declaration of class FEPageDesc.
  * @author H.-J. Mathes, FzK
  */

#ifndef _FEPageDesc_hh_
#define _FEPageDesc_hh_

#include <MiEvent.hh>

#ifdef USE_MONITOR
# include "ReadoutMonitor.hh"  // ReadoutMonitor::T2InfoRec
#endif // USE_MONITOR

/** This class contains the information necessary for FLT/SLT readout.
  */
class FEPageDesc {
    friend class FEReadout;
    friend class FEFileReadout;
#ifdef USE_MONITOR
    friend class ReadoutMonitor;
#endif // USE_MONITOR

public:
    /** Constructor for class FEPageDesc.
      *
      * Set the page id to an invalid value.
      */
    FEPageDesc(int page_id = 0) : fPageId(page_id) { }
    
    /** Get the curent page id. */
    int GetPageId() { return fPageId; }
    
    /** Get the timestamp of event generation. 
      *
      * We use here the convention, that the time stamps measure in units 
      * of usec as all system times are also measured with usec resolution.
      */
    long long GetTimeStamp() { return fTimeStamp; }
    
    /** Get the current trigger source, i.e. the value read from the FE. */
    TMirrorEventHeader::EMiTriggerSource GetTriggerSource()
     { return fTriggerSource; }

private:
    /** Private destructor of class FEPageDesc. */
    //~FEPageDesc() {}
    
    int            fFltOffset;          // information read from the hardware
    int            fSltOffset;
    unsigned long  fSecond;
    unsigned long  fNanoSecond;

    unsigned int   fNxPgTimeStampLow;   // the time stamps read from the hardware
    unsigned int   fNxPgTimeStampHigh;
    unsigned int   fTriggerTimeStamp;   // time stamp of the trigger
    int            fPageId;
    int            fNPages;             // number of pages filled
    long long      fTimeStamp;          // special time stamp in usec
    
    TMirrorEventHeader::EMiTriggerSource fTriggerSource;      // trigger source as read from the hardware

#if (MIEVENT_VERSION_CODE >= ModuleVersionCode(4,1,0))
    unsigned long long  fRunTimeCounter;
#endif // MIEVENT_VERSION_CODE
    unsigned long long  fDeadTimeCounter;
    unsigned long long  fVetoTimeCounter;
};

#endif // _FEPageDesc_hh_
