
//
// File   : FEReadoutTimeProfile.hh
//
// Purpose: Declaration & implementation of the class FEReadoutTimeProfile
//
// $Id: FEReadoutTimeProfile.cc 9725 2010-09-28 07:31:17Z mathes $
//


/** @file FEReadoutTimeProfile.cc
  * Implementation of the class FEReadoutTimeProfile.
  * @author H.-J. Mathes, Fzk
  */
  
#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cerrno>
#include <cstring>

#include "FEReadoutTimeProfile.hh"

using namespace std;

// --------------------------------------------------------------------------

FEReadoutTimeProfile * FEReadoutTimeProfile::fgInstance = NULL;

FEReadoutTimeProfile::FEReadoutTimeProfile()
 {
  fDeleteEvent.Reset();
  fEventHeader.Reset();
  fFADCReadout.Reset();
  fMonitorReadout.Reset();
  fPixelReadout.Reset();
  fPollNextPage.Reset();
  fTraceReadout.Reset();
  
  // --- install program exit handler ...

  if ( atexit( ExitHandler ) < 0 ) {
    cerr << "FEReadoutTimeProfile::FEReadoutTimeProfile() - "
         << "trying to install 'atexit' handler with error, "
         << errno << " - " << strerror( errno ) << endl;
  }

  fTotal.Start();
}

// --------------------------------------------------------------------------

FEReadoutTimeProfile *FEReadoutTimeProfile::GetInstance()
 {
  if ( fgInstance == NULL ) fgInstance = new FEReadoutTimeProfile();
  return fgInstance;
}
    
// --------------------------------------------------------------------------

FEReadoutTimeProfile::~FEReadoutTimeProfile()
 {
  fTotal.Stop();

  PrintFinalResults();
}

// -------------------------------------------------------------------------

void FEReadoutTimeProfile::ExitHandler()
 {
  if ( fgInstance ) {
    delete fgInstance;
    fgInstance = NULL;
  }
}

// --------------------------------------------------------------------------

void FEReadoutTimeProfile::PrintFinalResults(std::ostream& ostr)
 {
  ostr << "=== FEReadoutTimeProfile: Final results (calls/real/CPU time) === " << endl;
  
  ostr << " Poll NextPg       : " << setw(8) << fPollNextPage.Counter() << " "
       << fPollNextPage.RealTime() << " " << fPollNextPage.CpuTime() << endl;
  ostr << " Event header      : " << setw(8)  << fEventHeader.Counter() << " "
       << fEventHeader.RealTime() << " " << fEventHeader.CpuTime() << endl;
  ostr << " SLT data readout  : " << setw(8)  << fPixelReadout.Counter() << " "
       << fPixelReadout.RealTime() << " " << fPixelReadout.CpuTime() << endl;
  ostr << " FADC data readout : " << setw(8)  << fFADCReadout.Counter() << " "
       << fFADCReadout.RealTime() << " " << fFADCReadout.CpuTime() << endl;
  ostr << " Trace readout     : " << setw(8)  << fTraceReadout.Counter() << " "
       << fTraceReadout.RealTime() << " " << fTraceReadout.CpuTime() << endl;
  ostr << " Monitor data      : " << setw(8)  << fMonitorReadout.Counter() << " "
       << fMonitorReadout.RealTime() << " " << fMonitorReadout.CpuTime() << endl;
  ostr << " Event deletion    : " << setw(8)  << fDeleteEvent.Counter() << " "
       << fDeleteEvent.RealTime() << " " << fDeleteEvent.CpuTime() << endl;
  ostr << endl;
  ostr << " Total time(s)     : "
       << fTotal.RealTime() << " " << fTotal.CpuTime() << endl;
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
