
//
// File   : FEReadoutTimeProfile.hh
//
// Purpose: Declaration & implementation of the class FEReadoutTimeProfile
//
// $Id: FEReadoutTimeProfile.hh 9725 2010-09-28 07:31:17Z mathes $
//


/** @file FEReadoutTimeProfile.hh
  * Declaration of the class FEReadoutTimeProfile.
  * @author H.-J. Mathes, Fzk
  */
  
#ifndef _FEReadoutTimeProfile_hh_
#define _FEReadoutTimeProfile_hh_

#include <iostream>

#include <TStopwatch.h>

/** Class which holds several TStopwatch objects (ROOT) to profile the
  * execution time of various readout steps.
  */

class FEReadoutTimeProfile {
private:
    /** Constructor of the class FEReadoutTimeProfile.
      */
    FEReadoutTimeProfile();

    /** Destructor of the class FEReadoutTimeProfile.
      *
      * The final results of the profiling are printed to cout.
      */
    ~FEReadoutTimeProfile();

public:
    /** Get the one-and-only instance of the class FEReadoutTimeProfile.
      */
    static FEReadoutTimeProfile *GetInstance();
    
    /** Print the final results of the profiling to the specified stream.
      */
    void PrintFinalResults(std::ostream& ostr=std::cout);

    /** TStopwatch for 'event delete' part. */
    TStopwatch            fDeleteEvent;
    /** TStopwatch for 'event header readout'. */
    TStopwatch            fEventHeader;
    /** TStopwatch for 'FADC readout'. */
    TStopwatch            fFADCReadout;
    /** TStopwatch for 'monitor data readout'. */
    TStopwatch            fMonitorReadout;
    /** TStopwatch for 'SLT data readout'. */
    TStopwatch            fPixelReadout;
    /** TStopwatch for 'Next Page polling'. */
    TStopwatch            fPollNextPage;
    /** TStopwatch for 'single DAC trace readout'. */
    TStopwatch            fTraceReadout;

private:
    // these ones are not implemented
    FEReadoutTimeProfile(const FEReadoutTimeProfile&);
    FEReadoutTimeProfile& operator=(const FEReadoutTimeProfile&);

    TStopwatch            fTotal;

    /** Global FEReadoutTimeProfile object.
      */
    static FEReadoutTimeProfile  *fgInstance;

    /** Termination handler called at program exit.
      */
    static void ExitHandler();
};

#endif  // _FEReadoutTimeProfile_hh_
