
//
// File   : FastPulseFinder.cc
//
// Purpose: Implementation of the class FastPulseFinder
//
// $Id: FastPulseFinder.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <iostream>

/** @file FastPulseFinder.cc
  * Implementation of the pulse finder class FastPulseFinder.
  */


#include <TNamed.h>

#include <FdNumbering.hh>
#include <ErrorLogging.hh>
#include <MiEvent.hh>

#include "FastPulseFinder.hh"

using namespace std;
using namespace FdUtil;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class PFStefano - implementation
//

UInt_t FastPulseFinder::fgDebugLevel = 0;

FastPulseFinder::FastPulseFinder()
 : fMinPulseLength(1), fNPulses(0)
 {
  if ( fgDebugLevel >= 2 )
    FD_COUT << "FastPulseFinder::FastPulseFinder(): " 
            << hex << this << dec << endl;
  
  SetName( "FastPulseFinder" );
  
  for ( UInt_t i=0; i<kNMaxPulses; i++ ) {
    fPulseLength[i] = 0;
    fPulseStart[i]  = 0;
  }
}

// -------------------------------------------------------------------------

void FastPulseFinder::FindPulse(TFADCData* fadc)
 {
  bool hitflag = false;
  UInt_t start = 0;
  UInt_t stop = 0;
  
  if ( fgDebugLevel >= 2 ) {
    FD_COUT << "FastPulseFinder::FindPulse(" 
            << Fd::GetColumnNo( fadc->GetPixelNumber() ) << "," 
	    << Fd::GetRowNo( fadc->GetPixelNumber() ) << ")" << endl;
  }

  TFADCData::FADCDataWord fadc_ptr = fadc->GetFADCTrace();
  
  for ( unsigned int i=fadc->GetTraceStartBin();
                     i<=fadc->GetTraceEndBin(); i++ )
   {
//    FD_COUT << "ADC data[" << i << "]: " 
//         << hex << fadc_ptr[i].word << dec << endl;
    if ( !hitflag ) {
#if (MIEVENT_VERSION_CODE >= ModuleVersionCode(4,0,7))
      if ( FADCDataWordHasPixelTrigger( &fadc_ptr[i] ) ) {
#else
      if ( PixelTrigger( &fadc_ptr[i] ) ) {
#endif // MIEVENT_VERSION_CODE
        hitflag = true;
        start = i;
	FD_COUT << "FastPulseFinder::FindPulse() - "
	        << "   found PixelTrigger in bin " << start << endl;
      }
    }
    else {
#if (MIEVENT_VERSION_CODE >= ModuleVersionCode(4,0,7))
      if ( !FADCDataWordHasPixelTrigger( &fadc_ptr[i] ) ) {
#else
      if ( !PixelTrigger( &fadc_ptr[i] ) ) {
#endif // MIEVENT_VERSION_CODE
        hitflag = false;
        stop = i;
	FD_COUT << "FastPulseFinder::FindPulse() - "
	        << "PixelTrigger vanished in bin " << stop << endl;
      
        if ( fNPulses == kNMaxPulses ) {
	  FD_CERR << "FastPulseFinder::FindPulse() - "
	          << "cancelled ::FindPulse() before end of trace !" << endl;
	  break;
	}
	
	if ( (stop - start) > fMinPulseLength ) {  // is pulse long enough ?
	
	  fPulseStart[fNPulses]  = start;
	  fPulseLength[fNPulses] = stop - start;
	
	  FD_COUT << "FastPulseFinder::FindPulse() - "
	          << "PulseLength[" <<  fNPulses << "]= " 
	          << fPulseLength[fNPulses] << endl;
	
	  fNPulses++;
	}
	
	start = stop = 0;
      }
    }
  }

  // now sort the pulses according to their length ...
  
  if ( fNPulses > 1 ) Sort();
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
