
//
// File   : FullPixelReadout.cc
//
// Purpose: Implementation file for class FullPixelReadout
//
// $Id: FullPixelReadout.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <assert.h>
#include <iostream>


/** @file FullPixelReadout.cc
  * Implementation of class FullPixelReadout.
  * @author H.-J. Mathes, Fzk
  */

#include <FdNumbering.hh>
#include <MiEvent.hh>

#include "CheckVersion.hh"

#include "ReadoutConfig.hh"
#include "ExtendedPixelList.hh"
#include "FullPixelReadout.hh"

using namespace std;
using FdUtil::Fd;

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
//
// class FullPixelReadout - implementation
//

FullPixelReadout::FullPixelReadout(MiReadout::SltParameters slt_par)
 : VPixelSelector("FullPixelReadout"),
   fPixelMatrix(new TMirrorBitArray), fExtPixelList(NULL)
 {
  // --- set the initial parameters
  
  MiReadout::SltParametersRec slt_default = DEFAULT_SLT_PARAMETERS;
  
  if ( slt_par == NULL ) 
    fSltPar = slt_default;
  else
    fSltPar = *slt_par;

#ifdef DEBUG_PARAMS
  ReadoutConfig::DumpSltParameters( fSltPar, 
                                    "FullPixelReadout::FullPixelReadout()" );
#endif // DEBUG_PARAMS

  // --- set all pixels in fPixelMatrix
  
  for ( unsigned int col=Fd::kMI_FIRST_COL; col<=Fd::kMI_LAST_COL; col++ ) {
#ifndef AMT_DAQ
    for ( unsigned int row=Fd::kMI_FIRST_ROW; row<=Fd::kMI_LAST_ROW; row++ ) {
#else
    //for ( unsigned int=Fd::kAMT_FIRST_ROW; row<=Fd::kAMT_LAST_ROW; row++ ) {
    for ( unsigned int row=Fd::kMI_FIRST_ROW; row<=16; row++ ) {
#endif // AMT_DAQ
      fPixelMatrix->Set( col, row );
    }
  }
  
  // --- fill and initialize an ExtendedPixelList with the right start/end bins
  
  TMirrorPixelList pixel_list( fPixelMatrix );
  
  fExtPixelList = new ExtendedPixelList( &pixel_list, 
                                         fSltPar.fStartBin, fSltPar.fEndBin );
}

// --------------------------------------------------------------------------

FullPixelReadout::~FullPixelReadout()
 {
  delete fPixelMatrix;
  delete fExtPixelList;
}

// --------------------------------------------------------------------------

void FullPixelReadout::Select(TMirrorPixelData * /* pd */,
                              TMirrorEventHeader::EMiTriggerSource /* trigger_source */,
			      unsigned int column_list)
 {
  // remove columns without modules from the pixel list
  
  for ( unsigned int col=Fd::kMI_FIRST_COL; col<=Fd::kMI_LAST_COL; col++ ) {
    if ( (column_list & (1 << (col - Fd::kMI_FIRST_COL))) == 0 )
      fPixelMatrix->Clear( col );
  }
  
  //fPixelMatrix->Show("FullPixelReadout::Select()");
  
  // --- will be sliced in the base class !!!
  // VPixelSelector::Set( fExtPixelList );
  
  // -> just set the result object
  
  VPixelSelector::Set( fPixelMatrix );
  fExtPixelList->Set( fPixelMatrix );
}

// --------------------------------------------------------------------------

TMirrorPixelList& FullPixelReadout::GetPixelList()
 {
  assert( fExtPixelList != NULL );
  
  //fExtPixelList->Show();
  
  return *fExtPixelList;
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
