
//
// File   : FullPixelReadout.hh
//
// Purpose: Declaration of class FullPixelReadout
//
// $Id: FullPixelReadout.hh 9725 2010-09-28 07:31:17Z mathes $
//


/** @file FullPixelReadout.hh
  * Declaration of class FullPixelReadout.
  * @author H.-J. Mathes, Fzk
  */
  
#ifndef _FullPixelReadout_hh_ 
#define _FullPixelReadout_hh_ 

#include <iostream>

#include <VPixelSelector.hh>

// --- forward declaration(s)

class ExtendedPixelList;
class TMirrorBitArray;
class TMirrorPixelData;

/** Simple (dummy) implementation for the trigger decision.
  *
  * This class implements a trigger decision on the MirrorPC based on 
  * the data read out from the SLT module (the so called PixelData). In this
  * implementation all pixels are readout.
  */
class FullPixelReadout : public VPixelSelector {
public:
    /** Constructor for the class FullPixelReadout.
      *
      * The algorithms parameters are initialized.
      */
    FullPixelReadout(MiReadout::SltParameters);

    /** Destructor of the class FullPixelReadout.
      *
      * Erase all temporary created objects and lists.
      */
    virtual ~FullPixelReadout();
	
    /** Clear the pixel list, i.e. the pixel list contains then no more pixels. 
     *
     * This method does nothing here.
     */
    void Clear() { }

    /** Decide which pixels of the event desribed by its pixel data
      * are to be read out.
      *
      * In this implementation all pixels are read out, only the start and end
      * bin for the FADC readout are configurable parameters.
      */
    void Select(TMirrorPixelData*,TMirrorEventHeader::EMiTriggerSource,
                                  unsigned int column_list=0x0fffff);
    
    /** Returns always true. */
    bool DoesFullReadout()
     { return true; }

    /** Get the list of pixels to be read out (overwrites base class' method). */
    TMirrorPixelList& GetPixelList();

private:
    // default ctor. is not accessible
    FullPixelReadout();
    // copy ctor. and assignment operator are not accessible
    FullPixelReadout(const FullPixelReadout&);
    FullPixelReadout& operator=(const FullPixelReadout&);

    TMirrorBitArray              *fPixelMatrix;
    ExtendedPixelList            *fExtPixelList;
};

#endif  // _FullPixelReadout_hh_
