
//
// File   : LongPixelPattern.cc
//
// Purpose: Implementation of the class LongPixelPattern
//
//
// $Id: LongPixelPattern.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <iostream>

/** @file LongPixelPattern.cc
  * Implementation of the class LongPixelPattern.
  * @author H.-J. Mathes, FzK
  */

#include <FdNumbering.hh>
#include <ErrorLogging.hh>
#include <MiEvent.hh>

#include "LongPixelPattern.hh"

using namespace std;
using namespace FdUtil;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class LongPixelPattern - implementation
//

static const int gIncrement = 10;

LongPixelPattern::LongPixelPattern(PixelPattern& pattern /*,unsigned int initial_store*/ )
 : PixelPattern(pattern), fNMaxPixel(gIncrement)
 {
  fAdcTime  = new unsigned int[gIncrement];
  fCol      = new unsigned int[gIncrement];
  fRow      = new unsigned int[gIncrement];
  fSltTime  = new unsigned int[gIncrement];
  fSltTime2 = new unsigned int[gIncrement];
}

// -------------------------------------------------------------------------

LongPixelPattern::LongPixelPattern(unsigned int size)
 : fNMaxPixel(size)
 {
  fAdcTime  = new unsigned int[size];
  fCol      = new unsigned int[size];
  fRow      = new unsigned int[size];
  fSltTime  = new unsigned int[size];
  fSltTime2 = new unsigned int[size];
}

// -------------------------------------------------------------------------

void LongPixelPattern::Add(PixelPattern* pattern)
 {
  if ( pattern->GetStartTime() < fSltTime[0] )
    fSltTime[0] = pattern->GetStartTime();     // set to earliest time

  // and recalculate all times
  
  // copy all pixels ...
  
  for ( unsigned int n=0; n<pattern->GetNPixel(); n++ ) {

    unsigned int col, row;
    
    pattern->GetColumnRow( n, col, row );
    
    bool found = false;
    
    for ( unsigned int i=0; i<GetNPixel(); i++ ) {    // pixel already there ?
      unsigned int col0, row0;
      GetColumnRow( i, col0, row0 );
      if ( col0 == col && row0 == row ) {
        found = true;
	break;
      }
    }
    
    if ( !found ) {                           // not found, so add it here
      Fd::PixelNumberRec pixel;
      // SetPixelNumberFromColumnRow( &pixel, col, row );
      AddPixel( &pixel );
    }
  }
}

// -------------------------------------------------------------------------

void LongPixelPattern::Add(Fd::PixelNumber pixel)
 {
  if ( fNPixel == fNMaxPixel ) {  // reallocate memory

    FD_COUT << "LongPixelPattern::Add() - "
            << "need to increase the storage area !" << endl;
    
    fNMaxPixel += gIncrement;

    unsigned int *adc = new unsigned int[fNMaxPixel];
    unsigned int *col = new unsigned int[fNMaxPixel];
    unsigned int *row = new unsigned int[fNMaxPixel];
    unsigned int *t   = new unsigned int[fNMaxPixel];
    unsigned int *t2  = new unsigned int[fNMaxPixel];

    for ( unsigned int i=0; i<fNPixel; i++ ) {
      adc[i] = fAdcTime[i];
      col[i] = fCol[i];
      row[i] = fRow[i];
      t[i]   = fSltTime[i];
      t2[i]  = fSltTime2[i];
    }
    
    delete [] fAdcTime;
    fAdcTime = adc;
    
    delete [] fCol;
    fCol = col;
    
    delete [] fRow;
    fRow = row;
    
    delete [] fSltTime;
    fSltTime = t;
    
    delete [] fSltTime2;
    fSltTime2 = t2;
  }
  
  fCol[fNPixel]     = Fd::GetColumnNo( pixel );
  fRow[fNPixel]     = Fd::GetRowNo( pixel );
  fSltTime[fNPixel] = fSltTime[0] + abs( (int)(  fStartColumn / 2 
                                            - Fd::GetColumnNo( pixel ) / 2) );
  fAdcTime[fNPixel] = 0;

  fNPixel++;
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

