
//
// File   : NeighborPixelReadout.hh
//
// Purpose: Declaration of the class NeighborPixelReadout
//
// $Id: NeighborPixelReadout.hh 9725 2010-09-28 07:31:17Z mathes $
//



/** @file NeighborPixelReadout.hh
  * Declaration of the class NeighborPixelReadout.
  * @author H.-J. Mathes, FzK
  */


#ifndef _NeighborPixelReadout_hh_
#define _NeighborPixelReadout_hh_

#include <SelectivePixelReadout.hh>

/** This class steers the cameras pixel readout in the case of a trigger
  * received by the cameras neighbor (L or R trigger).
  *
  * Concerning the strategy of readout of L/R triggers the following was agreed
  * at the meeting on 2006/05/04 (MK + AK + HJM):
  * @li 'NExtCols' of columns are readout in any case
  * @li the usual (SelectivePixelReadout) strategy applies 
  *     for the triggered pixels
  *
  * Added by thjm, 10.11.2007:
  * @li deletion of single pixels must be prevented, this will keep the noise in
  *     the central part of the camera
  */
class NeighborPixelReadout : public SelectivePixelReadout {
public:

    /** Constructor for the class NeighborPixelReadout. */
    NeighborPixelReadout(MiReadout::SltParameters);

    /** Destructor of the class NeighborPixelReadout. */
    ~NeighborPixelReadout();
    
  // --- methods inherited from base class VPixelSelector

    /** Decide if the event described by the PixelData (set through the method
      * SetPixelData()) has a valid signature.
      */
    void Select(TMirrorPixelData*,TMirrorEventHeader::EMiTriggerSource,
                                  unsigned int column_list=0x0fffff);

    /** Set the SltParameters (modifies them before). */
    void SetSltParameters(MiReadout::SltParameters slt);
  
  // -- methods specific for this class

    /** Print the final statistics of this processor. */
    void PrintFinalResults(std::ostream& ostr=std::cout);

private:
    // these ones don't exist
    NeighborPixelReadout();
    NeighborPixelReadout(const NeighborPixelReadout&);
    NeighborPixelReadout& operator=(const NeighborPixelReadout&);
};

#endif  // _NeighborPixelReadout_hh_
