
//
// File   : PixelPattern.cc
//
// Purpose: Implementation file for class PixelPattern
//
// $Id: PixelPattern.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <iostream>
#include <cmath>

/** @file PixelPattern.cc
  * Implementation of the class PixelPattern.
  * @author H.-J. Mathes, FzK
  */

#include <FdNumbering.hh>
#include <ErrorLogging.hh>
#include <MiBitArray.hh>

#include "CheckVersion.hh"

#include "AnalysedFadcData.hh"
#include "PixelPattern.hh"

using namespace std;
using namespace FdUtil;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class PixelPattern - implementation
//

unsigned int PixelPattern::fgPrintLevel = 0;

PixelPattern::PixelPattern()
 : fNPixel(0), fIsValid(kFALSE)
 {
  fAdcTime  = NULL;
  fCol      = NULL;
  fRow      = NULL;
  fSltTime  = NULL;
  fSltTime2 = NULL;
}

// -------------------------------------------------------------------------

PixelPattern::PixelPattern(unsigned int id)
 : fNPixel(0), fIsValid(false), fPatternId(id)
 {
  fAdcTime  = new unsigned int[kNPixel];
  fCol      = new unsigned int[kNPixel];
  fRow      = new unsigned int[kNPixel];
  fSltTime  = new unsigned int[kNPixel];
  fSltTime2 = new unsigned int[kNPixel];

  fSltTime[0] = 0;
}

// -------------------------------------------------------------------------

PixelPattern::PixelPattern(unsigned int id,unsigned int start_col)
 : fNPixel(0), fStartColumn(start_col), 
   fIsValid(false), fPatternId(id)
 {
  fAdcTime  = new unsigned int[kNPixel];
  fCol      = new unsigned int[kNPixel];
  fRow      = new unsigned int[kNPixel];
  fSltTime  = new unsigned int[kNPixel];
  fSltTime2 = new unsigned int[kNPixel];
  
  fSltTime[0] = 0;
}
 
// -------------------------------------------------------------------------

PixelPattern::PixelPattern(PixelPattern& rhs)
 {
  *this = rhs;  // flat copy

  fAdcTime  = new unsigned int[kNPixel];
  fCol      = new unsigned int[kNPixel];
  fRow      = new unsigned int[kNPixel];
  fSltTime  = new unsigned int[kNPixel];
  fSltTime2 = new unsigned int[kNPixel];
  
  for ( unsigned int i=0; i<fNPixel; i++ ) {  // deep copy of the rest
    fAdcTime[i]  = rhs.fAdcTime[i];
    fCol[i]      = rhs.fCol[i];
    fRow[i]      = rhs.fRow[i];
    fSltTime[i]  = rhs.fSltTime[i];
    fSltTime2[i] = rhs.fSltTime2[i];
  }
}

// -------------------------------------------------------------------------

PixelPattern::~PixelPattern()
 {
  delete [] fAdcTime;
  delete [] fCol;
  delete [] fRow;
  delete [] fSltTime;
  delete [] fSltTime2;
}

// -------------------------------------------------------------------------

void PixelPattern::AddPixel(unsigned int col,unsigned int row)
 {
  if ( col < Fd::kMI_FIRST_COL || col > Fd::kMI_LAST_COL ) {
    if ( fgPrintLevel >= 1 ) 
      FD_CERR << "PixelPattern::AddPixel() - "
              << "tried to add pixel with illegal column number " 
              << col << " !" << endl;
    return; // throw;
  }
  
  if ( row < Fd::kMI_FIRST_ROW || row > Fd::kMI_LAST_ROW ) {
    if ( fgPrintLevel >= 1 ) 
      FD_CERR << "PixelPattern::AddPixel() - "
              << "tried to add pixel with illegal row number " 
              << row << " !" << endl;
    return; // throw;
  }
  
  if ( fNPixel >= kNPixel ) {   // should throw an exception !
    FD_CERR << "PixelPattern::AddPixel() - "
            << "Tried to add too much pixels !" << endl;
    return; // throw;
  }

  fCol[fNPixel]       = col;
  fRow[fNPixel]       = row;

  // this is wrong !?
  // SetSltTime( fNPixel, fCol[fNPixel], fStartBin * 10 );

  fAdcTime[fNPixel] = 0;
  
  if ( fgPrintLevel >= 1 ) {
    Fd::PixelNumberRec pixnum;
    Fd::SetPixelNoFromColumnRow( &pixnum, 
                                 (unsigned int)fCol[fNPixel], 
			         (unsigned int)fRow[fNPixel] );
    
    FD_COUT << "Pixel[" << fNPixel << "] (col,row) = (" 
            << fCol[fNPixel] << "," << fRow[fNPixel] << ")"
	    << ", pixnum= " << Fd::GetPixelNo( &pixnum )
	    << ", slttime= " << fSltTime[fNPixel]
	    << "," << fSltTime2[fNPixel] << endl;
  }
  
  fNPixel++;
}

// -------------------------------------------------------------------------

void PixelPattern::SetSltTime(int n,unsigned int col,
                                    unsigned int t_start,unsigned int t_end)
 {
#if 0
  fSltTime[n]  = t_start * 10 + (col / 2);
  fSltTime2[n] = t_end * 10 + (col / 2);
#else
  fSltTime[n]  =   t_start * 10 * TMirrorPixelData::kSLTBinWidth
                 + col * TMirrorPixelData::kSLTBinWidth / 2;
  fSltTime2[n] =   t_end * 10 * TMirrorPixelData::kSLTBinWidth
                 + col * TMirrorPixelData::kSLTBinWidth / 2;
#endif
}

// -------------------------------------------------------------------------

void PixelPattern::SetStart(unsigned int col,unsigned int bin)
 {
  fStartColumn = col;
  fStartBin    = bin;
}

// -------------------------------------------------------------------------

void PixelPattern::Show(const char *where)
 {
  // cout --> FD_LOG_STREAM ???
  cout << "--- Pattern: 0x" << hex << fPatternId << dec;
  
  if ( where && *where != 0 )
    cout << " @ " << where;
  cout << endl;
  
  if ( !fNPixel )
    cout << "   ... no pixels present !" << endl;
  else {
    for ( unsigned int n=0; n<fNPixel; n++ )
      cout << "Pixel[" << n << "] (col,row) = (" 
           << (unsigned int)fCol[n] << "," << (unsigned int)fRow[n] << ")" 
	   << ", slttime= " << fSltTime[n] 
	   << "," << fSltTime2[n] 
	   << ", ADCtime= " << fAdcTime[n] << endl;
  }
}

// -------------------------------------------------------------------------

bool PixelPattern::Validate(TMirrorBitArray *barray)
 { 
  //FD_COUT << "PixelPattern::Validate() - fNPixel= " << fNPixel << endl;
  
  if ( !fNPixel ) return false;
  
  unsigned int result = 0;
  
  for ( unsigned int i=0; i<fNPixel; i++ ) {
    //FD_COUT << "PixelPattern::Validate(" << fCol[i] << "," << fRow[i] << "): " 
    //	    << (int)barray->Test( fCol[i], fRow[i] ) << endl;
    if ( barray->Test( fCol[i], fRow[i] ) ) result++;
  }

  fIsValid = (result == fNPixel) ? true : false;
  
  return fIsValid;
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

