
//
// File   : VPixelSelector.cc
//
// Purpose: Implementation of the class VPixelSelector
//
// $Id: VPixelSelector.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <iostream>
#include <stdexcept>


/** @file VPixelSelector.cc
  * Implementation of the class VPixelSelector.
  * @author H.-J. Mathes, FzK
  */

#include <MiEvent.hh>
#include <ErrorLogging.hh>

#include "ExtendedPixelList.hh"
#include "FullPixelReadout.hh"
#include "NeighborPixelReadout.hh"
#include "SelectivePixelReadout.hh"

#include "VPixelSelector.hh"

using namespace std;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// 
// class VPixelSelector - implementation
// 

VPixelSelector::VPixelSelector(const char *name)
 : fBitArray(NULL), fName("NoName"), fPixelList(NULL)
 {
  SetName( name );
}

// -------------------------------------------------------------------------

VPixelSelector::~VPixelSelector()
 {
  delete fBitArray;
  delete fPixelList;
}

// -------------------------------------------------------------------------

VPixelSelector * VPixelSelector::Create(std::string selector_class,
                                        MiReadout::SltParameters slt_par)
 {
  if ( selector_class == "FullPixelReadout" )
    return new FullPixelReadout(slt_par);
  else if ( selector_class == "NeighborPixelReadout" )
    return new NeighborPixelReadout(slt_par);
  else if ( selector_class == "SelectivePixelReadout" )
    return new SelectivePixelReadout(slt_par);
  else {
    FD_CERR << "VPixelSelector::Create() - unknown class '"
            << selector_class << "' was requested!" << endl;
    return NULL;
  }
}

// -------------------------------------------------------------------------

VPixelSelector * VPixelSelector::Create(MiReadout::SltParameters slt_par)
 {
  if ( slt_par->fDoFullReadout )
    return Create( "FullPixelReadout", slt_par );
  else
    return Create( "SelectivePixelReadout", slt_par );
}

// -------------------------------------------------------------------------

TMirrorPixelList& VPixelSelector::GetPixelList()
 {
  if ( !fPixelList ) {
    FD_CERR << "VPixelSelector::GetPixelList() - no TMirrorPixelList is available !"
         << endl;
    throw runtime_error( "VPixelSelector - no TMirrorPixelList !" );
  }

  return *fPixelList;
}

// -------------------------------------------------------------------------

TMirrorBitArray& VPixelSelector::GetPixels()
 {
  if ( !fBitArray) {
    FD_CERR << "VPixelSelector::GetPixels() - no TMirrorBitArray is available !"
         << endl;
    throw runtime_error( "VPixelSelector - no TMirrorBitArray !" );
  }

  return *fBitArray;
}

// -------------------------------------------------------------------------

void VPixelSelector::Set(TMirrorBitArray* ba)
 {
  if ( ba ) {
    //ba->Show("VPixelSelector::Set()");
    if ( !fBitArray )
      fBitArray = new TMirrorBitArray( *ba );
    else
      *fBitArray = *ba;
  }
}
    
// -------------------------------------------------------------------------

void VPixelSelector::Set(TMirrorPixelList* pl)
 {
  if ( dynamic_cast<ExtendedPixelList *>( pl ) != NULL ) {
    FD_CERR << "VPixelSelector::Set() - received ExtendedPixelList !!!" << endl;
    FD_CERR << "\t---> This object will be sliced !" << endl;
  }
  
  if ( pl ) {
    if ( !fPixelList )
      fPixelList = new TMirrorPixelList( *pl );
    else
      *fPixelList = *pl;
  }
}

// -------------------------------------------------------------------------

void VPixelSelector::SetName(const char *name)
 {
  if ( name == NULL || *name == 0 ) {
    FD_CWARN << "VPixelSelector::SetName() - no name provided !" << endl;
    fName = "NoName";
  }
  else
    fName = name;
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
