
//
// File   : ErrorLoggerStream.cc
//
// Purpose: Implementation of the class ErrorLoggerStream
//
// $Id: ErrorLoggerStream.cc 8758 2010-09-27 12:16:24Z mathes $
//


#include <iostream>
#include <iomanip>
#include <ctime>
#include <string>
#include <sstream>

/** @file ErrorLoggerStream.cc
  * Implementation of the class ErrorLoggerStream.
  * @author H.-J. Mathes, FzK
  */


#include "ErrorLoggerStream.hh"

using namespace FdUtil;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// 
// class ErrorLoggerStream - implementation
// 

unsigned int ErrorLoggerStream::fgPrintLevel = 0;

ErrorLoggerStream::ErrorLoggerStream(const std::string name)
 : std::ostream( fStream.rdbuf() ), fName(name)
 {
  if ( fgPrintLevel > 1 )
    std::cout << "ErrorLoggerStream::ErrorLoggerStream() - source= ..." 
              << std::endl;
}

// -------------------------------------------------------------------------

ErrorLoggerStream& ErrorLoggerStream::GetInstance(const std::string name) 
 {
  ErrorLoggerStream& instance = Singleton<ErrorLoggerStream>::GetInstance();
  
  instance.SetName( name );

  return instance;
}

// -------------------------------------------------------------------------

ErrorLoggerStream& 
ErrorLoggerStream::operator<<(ErrorLogger::ESeverityLevel severity)
 {
  if ( fgPrintLevel > 1 )
    std::cout << "ErrorLoggerStream::operator<<(ESeverityLevel= "
         << (int)severity << ") ... " << std::endl;

  fSeverity = severity;

  return *this;
}

// -------------------------------------------------------------------------

ErrorLoggerStream& 
ErrorLoggerStream::operator<<(ErrorLogger::EVerbosityLevel verbosity)
 {
  std::cout << "ErrorLoggerStream::operator<<(EVerbosityLevel= "
            << (int)verbosity << ") ... " << std::endl;

  fVerbosity = verbosity;
  
  return *this;
}

// -------------------------------------------------------------------------

ErrorLoggerStream& ErrorLoggerStream::endm()
 {
  time_t t_now = time( NULL );
  
  if ( fgPrintLevel > 1 )
    std::cout << "Sending message with severity= " << (int)fSeverity
         << " verbosity= " << (int)fVerbosity << " to stream '"
	 << fName << "' !" << std::endl;

  if ( fgPrintLevel ) {
    std::cout << std::setw(9) << t_now << ": " 
         << (int)fSeverity << ": "
         << "'" << fStream.str() << "'" << std::endl;
  }
  else
    std::cout << fStream.str() << std::endl;

  fStream.str("");
  
  return *this;
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
