
//
// File   : FdNumbering.cc
//
// Purpose: Implementation of the class FdNumbering.
//
// $Id: FdNumbering.cc 8758 2010-09-27 12:16:24Z mathes $
//


#include <iostream>
#include <stdexcept>

/** @file FdNumbering.cc
  * Implementation of the class FdNumbering.
  * @author H.-J. Mathes, FzK
  */

#include "CheckVersion.hh"

#include "FdConfig.hh"
#include "FdNumbering.hh"
#include "ErrorLogging.hh"

using namespace std;
using FdUtil::Fd;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class FdNumbering - implementation
//

ClassImp(Fd)

unsigned int Fd::GetFdPixelNo(Fd::PixelNumber pixel)
 {
  Fd::FdPixelNumber fdpixel = GetFdPixelNumber( pixel );
  
  return fdpixel->fBits.fPixelNo;
}

// -------------------------------------------------------------------------

Fd::FdPixelNumber Fd::GetFdPixelNumber(Fd::PixelNumber pixel)
 {
  static FdPixelNumberRec fdpixel;
  int fd_mirrornum;
  int fd_pixelnum;
  
  fd_mirrornum = GetFdMirrorNo( pixel );

  fd_pixelnum = kMI_NEXTPIXELS * (fd_mirrornum - kFD_FIRST_MIRROR)
	      + GetPixelNo( pixel ) - kMI_FIRST_PIXEL
	      + kFD_FIRST_PIXEL;

  fdpixel.fBits.fPixelNo = fd_pixelnum;
  
  return &fdpixel;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetEyePixelNo(PixelNumber pixel)
 {
  return   GetPixelNo( pixel )
         + ( GetMirrorNo( pixel ) - kEYE_FIRST_MIRROR ) * kMI_NEXTPIXELS;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetEyePixelNo(FdPixelNumber pixel)
 {
  return   (  ( GetPixelNo( pixel ) - kMI_FIRST_PIXEL )
             % kMI_NEXTPIXELS )
         + kMI_NEXTPIXELS * ( ( GetMirrorNo( pixel ) 
	                     - kEYE_FIRST_MIRROR ) % kEYE_NMIRRORS )
	 + kMI_FIRST_PIXEL;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetMirrorPixelNo(FdPixelNumber pixel)
 {
  return   ( GetPixelNo( pixel ) - kMI_FIRST_PIXEL ) % kMI_NEXTPIXELS
         + kMI_FIRST_PIXEL;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetEyeMirrorNo(FdPixelNumber pixel)
 {
  return ( ( pixel->fBits.fPixelNo - kFD_FIRST_PIXEL ) / kMI_NEXTPIXELS )
         % kEYE_NMIRRORS
	 + kEYE_FIRST_MIRROR;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetMirrorNo(FdPixelNumber pixel)
 {
  return   ( pixel->fBits.fPixelNo - kFD_FIRST_PIXEL ) / kMI_NEXTPIXELS
         + kFD_FIRST_MIRROR;
}

// -------------------------------------------------------------------------

Fd::PixelNumber Fd::GetPixelNumber(FdPixelNumber fdpixel)
 {
  static PixelNumberRec pixel;

  SetEyeNo( &pixel, GetEyeNo( fdpixel ) );
  SetMirrorNo( &pixel, GetMirrorNo( fdpixel ) );
  SetPixelNo( &pixel, GetMirrorPixelNo( fdpixel ) );
  
  return &pixel;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetColumnNo(PixelNumber pi)
 {
  if ( pi->fBits.fPixelNo > kMI_LAST_PIXEL )
    return ( pi->fBits.fPixelNo - kMI_LAST_PIXEL - kMI_FIRST_PIXEL ) / 2 + kMI_FIRST_COL;
  else
    return (Int_t)( (pi->fBits.fPixelNo - kMI_FIRST_PIXEL) / kMI_NROWS ) + kMI_FIRST_COL;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetRowNo(PixelNumber pi)
 {
  if ( pi->fBits.fPixelNo > kMI_LAST_PIXEL )
    return kMI_VC1_ROW
           + (pi->fBits.fPixelNo - kMI_LAST_PIXEL - kMI_FIRST_PIXEL) % 2;
  else
    return (pi->fBits.fPixelNo - kMI_FIRST_PIXEL) % kMI_NROWS + kMI_FIRST_ROW;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetEyeNMirrors(unsigned int eye_no)
 {
  if ( eye_no < Fd::kFD_FIRST_EYE || eye_no > Fd::kFD_LAST_EYE ) {
    FD_CERR << "Fd::GetEyeNMirrors() - Eye number "
            << eye_no << " is outside valid range!" << endl;
    throw std::logic_error( "Eye number outside valid range" );
  }
  
  return GetEyeLastMirrorNo( eye_no );
}

// -------------------------------------------------------------------------

unsigned int Fd::GetEyeLastMirrorNo(unsigned int eye_no)
 {
  if ( eye_no < Fd::kFD_FIRST_EYE || eye_no > Fd::kFD_LAST_EYE ) {
    FD_CERR << "Fd::GetEyeLastMirrorNo() - Eye number "
            << eye_no << " is outside valid range!" << endl;
    throw std::logic_error( "Eye number outside valid range" );
  }
  
  if ( eye_no == Fd::kFD_LAST_EYE )
    return Fd::kHEAT_NMIRRORS;
  else
    return Fd::kEYE_NMIRRORS;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetFdFirstMirrorNo(unsigned int eye_no)
 {
  if ( eye_no < Fd::kFD_FIRST_EYE || eye_no > Fd::kFD_LAST_EYE ) {
    FD_CERR << "Fd::GetFdFirstMirrorNo() - Eye number "
            << eye_no << " is outside valid range!" << endl;
    throw std::logic_error( "Eye number outside valid range" );
  }
  
  if ( eye_no == Fd::kFD_FIRST_EYE )
    return Fd::kFD_FIRST_MIRROR;
  else
    return GetFdLastMirrorNo( eye_no - 1 ) - 1;
}

// -------------------------------------------------------------------------

unsigned int Fd::GetFdLastMirrorNo(unsigned int eye_no)
 {
  unsigned int mirror_no = GetEyeLastMirrorNo( eye_no );
  
  return Fd::GetFdMirrorNo( eye_no, mirror_no );
}

// -------------------------------------------------------------------------

void Fd::Set(PixelNumber pi)
 {
  pi->fPixelNumber = 0;
  
  SetEyeNo( pi, GetEyeId() );
  SetMirrorNo( pi, GetMirrorId() );
  
  SetPixelNo( pi, 0 );
}

// -------------------------------------------------------------------------

void Fd::Set(MirrorNumber mi)
 {
  mi->fMirrorNumber = 0;
  
  SetEyeNo( mi, GetEyeId() );
  SetMirrorNo( mi,   (GetEyeId() - Fd::kFD_FIRST_EYE) * Fd::kEYE_NMIRRORS
                   + ((GetMirrorId() - Fd::kEYE_FIRST_MIRROR) % Fd::kEYE_NMIRRORS) 
		   + Fd::kFD_FIRST_MIRROR );
}

// -------------------------------------------------------------------------

void Fd::SetEyeNo(MirrorNumber mi,unsigned int eye_no)
 {
  if ( eye_no < Fd::kFD_FIRST_EYE || eye_no > Fd::kFD_LAST_EYE ) {
    FD_CERR << "Fd::SetEyeNo(MirrorNumber) - Eye number " << eye_no 
            << " is not inside valid range !" << endl;
    return;
  }
  
  // Note: Due to an error in the past, valid values are 1..7, i.e.
  //       EYE 8 would become 0 if there will be any...
  mi->fBits.fEyeNo = eye_no & 7;
}

// -------------------------------------------------------------------------

void Fd::SetMirrorNo(MirrorNumber mi,unsigned int mirror_no)
 {
  if ( mirror_no < Fd::kFD_FIRST_MIRROR || mirror_no > Fd::kFD_LAST_MIRROR ) {
    FD_CERR << "Fd::SetMirrorNo(MirrorNumber) - Mirror number " << mirror_no 
            << " is not inside valid range !" << endl;
    return;
  }
  
  if ( mi->fBits.fEyeNo && mirror_no <= Fd::kEYE_LAST_MIRROR ) // already set ?
    mi->fBits.fMirrorNo = GetFdMirrorNo( mi->fBits.fEyeNo, mirror_no ) & 0x1f;
  else
    mi->fBits.fMirrorNo = mirror_no & 0x1f;
} 

// -------------------------------------------------------------------------

void Fd::SetFdMirrorNo(MirrorNumber mi,unsigned int mirror_no)
 {
  if ( mirror_no < Fd::kFD_FIRST_MIRROR || mirror_no > Fd::kFD_LAST_MIRROR ) {
    FD_CERR << "Fd::SetFdMirrorNo(MirrorNumber) - Mirror number " << mirror_no 
            << " is not inside valid range !" << endl;
    return;
  }
  
  mi->fBits.fMirrorNo = mirror_no & 0x1f;
} 

// -------------------------------------------------------------------------

void Fd::SetEyeNo(FdPixelNumber fd_pixelno,unsigned int eye_no)
 {
  PixelNumber pixel = GetPixelNumber( fd_pixelno );
  
  SetEyeNo( pixel, eye_no );
  
  FdPixelNumber fd_pixelno2 = GetFdPixelNumber( pixel );
  
  *fd_pixelno = *fd_pixelno2;
}

// -------------------------------------------------------------------------

void Fd::SetFdPixelNo(PixelNumber pixel,unsigned int pixel_no)
 {
  FdPixelNumberRec fdpixel;
  
  SetPixelNo( &fdpixel, pixel_no );
  
  PixelNumber pixel2 = GetPixelNumber( &fdpixel );
  
  *pixel = *pixel2;
}

// -------------------------------------------------------------------------

void Fd::SetPixelNoFromColumnRow(PixelNumber pi,
                                 unsigned int col,unsigned int row)
 { 
  if ( row > kMI_LAST_ROW )   /* this is a virtual channel ... */
    pi->fBits.fPixelNo = 
      ( 0x1ff & (  kMI_NPIXELS 
                 + (row - kMI_FIRST_ROW - kMI_LAST_ROW)
                 + (col - kMI_FIRST_COL) * 2
		 + kMI_FIRST_PIXEL ) );
  else                       /* that is a 'normal' channel */
    pi->fBits.fPixelNo = 
      ( 0x1ff & ( (col - kMI_FIRST_COL) * kMI_NROWS 
		 + row - kMI_FIRST_ROW 
		 + kMI_FIRST_PIXEL ) );
}

// -------------------------------------------------------------------------

void Fd::Streamer(TBuffer &b)
 {
  bool reading; reading = b.IsReading();  // just to avoid warning
  
  FD_CERR << "Fd::Streamer() - "
          << "objects of type FdUtil::Fd cannot be streamed !" << endl;
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
