
//
// File   : InifileChain.cc
//
// Purpose: Implementation of the class InifileChain
//
// $Id: InifileChain.cc 8758 2010-09-27 12:16:24Z mathes $
//


#include <iostream>
#include <list>
#include <string>
#include <algorithm>
#include <cassert>

/** @file InifileChain.cc
  * Implementation of the class InifileChain.
  * @author H.-J. Mathes, FzK
  */


#include <ErrorLogging.hh>

#include "InifileChain.hh"

using namespace std;
using namespace FdUtil;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
// 
// class InifileChain - implementation
// 

InifileChain::InifileChain(const char *filename)
 : Inifile(filename)
 {
  assert( filename != NULL && *filename != 0 );
  
  if ( GetPrintLevel() > 1 ) {
    FD_CINFO << "InifileChain::InifileChain('" << filename << "') -" 
             << endl;
  }
  
  if ( GetStatus() == Inifile:: kSUCCESS )  // status of Inifile::Inifile()
    fFiles.push_back( GetFilename() );
}

// -------------------------------------------------------------------------

InifileChain::InifileChain(Inifile& inifile)
 : Inifile(inifile)
 {
  if ( GetPrintLevel() > 1 ) {
    FD_CINFO << "InifileChain::InifileChain('Inifile') - file= " 
             << GetFilename() << endl;
  }
  
  if ( GetStatus() == Inifile:: kSUCCESS )  // status of Inifile::Inifile()
    fFiles.push_back( GetFilename() );
}

// -------------------------------------------------------------------------

InifileChain::~InifileChain()
 {
}

// -------------------------------------------------------------------------

int InifileChain::AddInifile(const char *filename)
 {
  if ( filename == NULL || *filename == 0 ) {
    FD_CWARN << "InifileChain::AddInifile() - "
             << "passed empty filename!" << endl;
    return Inifile::kFAIL;
  }
  
  if ( GetPrintLevel() > 1 ) {
    FD_CINFO << "InifileChain::AddInifile('" << filename << "') -" << endl;
  }
  
  string file = Inifile::Locate( filename );
  
  if ( GetPrintLevel() ) {
    FD_CINFO << "InifileChain::AddInifile() - found " << file << endl;
  }
  
  // --- check for duplicate file names ...
  
  if ( !file.empty() ) {
    if ( find( fFiles.begin(), fFiles.end(), file ) != fFiles.end() ) {
      FD_CWARN << "InifileChain::AddInifile() - "
               << "file: " << filename << " already added to the list!" << endl;
    }
  }
  
  if ( !file.empty() ) {
    fFiles.push_back( file );
    return Inifile::kSUCCESS;
  }
  
  return Inifile::kFAIL;
}

// -------------------------------------------------------------------------

int InifileChain::SpecifyGroup(const char *group)
 {
  int status = Inifile::kFAIL;
  
  Close();
  
  list<string>::iterator file = fFiles.begin();
  
  while ( file != fFiles.end() && status != Inifile::kSUCCESS ) {
  
    Open( (*file).c_str() );
    
    status = Inifile::SpecifyGroup( group );
    if ( status == Inifile::kSUCCESS ) break;
    
    Close();
    
    file++;
  }

  return status;
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
