
//
// File   : TTRawSocketAcceptor.cc
//
// Purpose: Implementation of the class TRawSocketAcceptor
//
// $Id: TRawSocketAcceptor.cc 8758 2010-09-27 12:16:24Z mathes $
// $Revision: 8758 $
// $Date: 2010-09-27 14:16:24 +0200 (Mon, 27 Sep 2010) $
//


#include <iostream>
#include <cstdio>
#include <cstdlib>


/** @file TRawSocketAcceptor.cc
  * Implementation of the class TRawSocketAcceptor.
  * @author H.-J. Mathes, FzK
  */
  

#include <TMessage.h>
#include <TMonitor.h>
#include <TServerSocket.h>
#include <TSocket.h>

#include "TRawSocketAcceptor.hh"

using namespace std;
using namespace FdRoot;
 
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
//
// class TRawSocketAcceptor - implementation
//

TRawSocketAcceptor::TRawSocketAcceptor()
 : fCurrentSocket(0), fMaxClients(1), fMsgLength(1), fMonitor(0), 
   fNumClients(0), fServerSocket(0), fStatus(0)
 {
  fMonitor = new TMonitor;
}

// -------------------------------------------------------------------------

TRawSocketAcceptor::~TRawSocketAcceptor()
 {
  delete fMonitor;
  delete fServerSocket;
}

// -------------------------------------------------------------------------

void TRawSocketAcceptor::_Accept()
 {
  fCurrentSocket = fServerSocket->Accept();
    
  if ( !fCurrentSocket ) return;
    
  if ( ( fStatus = fCurrentSocket->GetErrorCode() ) ) {
    cerr << "Error " << fCurrentSocket->GetErrorCode() 
         << " on accepted socket !" << endl;
    delete fCurrentSocket;
    fCurrentSocket = NULL;
    return;
  }
  
  _Connect();
}

// -------------------------------------------------------------------------

void TRawSocketAcceptor::_Connect()
 {
  // cout << "Client connected !" << endl;
  
  fMonitor->Add( fCurrentSocket );
  fNumClients++;
    
  if ( fNumClients == fMaxClients ) {
    fMonitor->Remove( fServerSocket );  // no further clients allowed
  }
  
  Connect();
}

// -------------------------------------------------------------------------

void TRawSocketAcceptor::_Disconnect()
 {
  // cout << "Client has disconnected !" << endl;

  cout << "Received: " << (float)fCurrentSocket->GetBytesRecv()/1024. 
       << " kBytes" << endl;
  
  fMonitor->Remove( fCurrentSocket );
	  
  fNumClients--;
  if ( fNumClients < fMaxClients )  // add eventually server socket to the TMonitor
    fMonitor->Add( fServerSocket );
  
  Disconnect();
  
  delete fCurrentSocket;
  fCurrentSocket = NULL;
}

// -------------------------------------------------------------------------

Bool_t TRawSocketAcceptor::_HandleRawMessage( char* msg, Int_t msg_length )
 {
  cerr << "Message  of " << msg_length << " bytes not handled !" << endl;
  delete [] msg;
  
  return kTRUE;
}

// -------------------------------------------------------------------------

Bool_t TRawSocketAcceptor::Open( Int_t port )
 {
  fServerSocket = new TServerSocket( port, kTRUE, 0 );
  
  if ( fServerSocket ) {
    if ( ( fStatus = fServerSocket->GetErrorCode() ) ) {
      cerr << "TServerSocket::GetErrorCode()= " 
           << fServerSocket->GetErrorCode() << endl;
      return kFALSE;
    }

    fMonitor->Add( fServerSocket );
    return kTRUE;
  }
  else
    return kFALSE;
}

// -------------------------------------------------------------------------

void TRawSocketAcceptor::Run(Long_t millisec)
 {
  fStatus = 0;   // reset status and error flag
  
  if ( ( fCurrentSocket = fMonitor->Select( millisec ) ) == (TSocket *)-1 ) {
    return;  // TMonitor::Select() timed out
  }
  
  if ( fCurrentSocket == fServerSocket ) {
    _Accept();
    fCurrentSocket = NULL;
    return;
  }
  
  if ( fCurrentSocket->IsValid() ) {

    fCurrentSocket->SetOption( kNoBlock, 1 );
    
    char *msg_buffer = new char[fMsgLength];
    
    Int_t length = fCurrentSocket->RecvRaw ( (void *)msg_buffer, fMsgLength );

    if ( length > 0 ) {  // received some message 
     
      Bool_t handled = kFALSE;
     
      handled = HandleRawMessage( msg_buffer, length );
      
      if ( !handled ) {  // call the standard handler
        
	handled = _HandleRawMessage( msg_buffer, length );

	cerr << "message handled: " << (Int_t)handled << endl;
      }
      
    } // length == 0 => disconnection
    else if ( length == 0 ) {
    
      cerr << "TSocket::RecvRaw(): " << length << endl;
      _Disconnect();
    }
    else if ( length < 0 ) {  // length < 0 = error => disconnection
    
      cerr << "TSocket::RecvRaw(): " << length << endl;
      _Disconnect();
    }

     delete [] msg_buffer;
  }
  else {  // fCurrentSocket->IsValid()
    cerr << "Socket to client not Valid !" << endl;

    delete fCurrentSocket;
    fCurrentSocket = NULL;
  }
}

// -------------------------------------------------------------------------

Int_t TRawSocketAcceptor::Send(const char *msg,Int_t length)
 {
  if ( !fCurrentSocket || !fCurrentSocket->IsValid() ) return -5;
  
  return fCurrentSocket->SendRaw( msg, length );
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
