
//
// File   : EyeDataProc.hh
//
// Purpose: Declaration of class EyeDataProcessor.
//
// $Id: EyeDataProc.hh 8757 2010-09-27 12:16:09Z mathes $
//


#ifndef _EyeDataProc_hh_
#define _EyeDataProc_hh_


/** @file EyeDataProc.hh
  * Declaration of class EyeDataProcessor.
  * @author H.-J. Mathes, FzK
  */

#include <TObject.h>
#include <TNamed.h>

// ----- forward declarations

class TEyeEvent;
class TEyeRunHeader;
class TEyeRunTrailer;

/** This mixin class definess the interface of our simple processing units.
  */
class EyeDataProcessor : public TNamed {
public:
    /** Constructor of the class EyeDataProcessor. The name and the internal
      * status variable are initialized.
      */
    EyeDataProcessor(const char* name="EyeDataProcessor")
     : TNamed(name,"EyeDataProcessor"), fStatus(0) {}
    
    /** Virtual destructor of the class EyeDataProcessor.
      */
    virtual ~EyeDataProcessor() {}

    /** Method to be called for each event. */
    virtual bool ProcessEvent(TEyeEvent*) = 0;
    
    /** Method to be called for the TEyeRunHeader. */
    virtual bool ProcessRunHeader(TEyeRunHeader*) { return true; }

    /** Method to be called for the TEyeRunTrailer. */
    virtual bool ProcessRunTrailer(TEyeRunTrailer*) { return true; }

    /** Set the internal error status of this application. */
    void SetStatus(int status) { fStatus = status; }

    /** Return application internal error status. */
    int GetStatus() { return fStatus; }

private:
    
    int  fStatus;
};

#endif  // _EyeDataProc_hh_
