
//
// File   : T3TestApp.hh
//
// Purpose: Declaration of the class T3TestApplication
//
// $Id: T3TestApp.hh 11539 2010-09-30 14:31:54Z mathes $
//



/** @file T3TestApp.hh
  * Declaration of the class T3TestApplication.
  * @author H.-J. Mathes, FzK
  */


#ifndef _T3TestApp_hh_
#define _T3TestApp_hh_

#include <cstdio>
#include <string>

#include <MiEvent.hh>
#include <MiEventFile.hh>

#include <EyeEvent.hh>
#include <EyeEventFile.hh>

// --- forward declaration(s)

class EyeEventClassifier;
#ifdef HOUGH_TEST
class HoughSel;
#endif // HOUGH_TEST


/** A test framework for testing the T3 (EyeEventClassifier).
  */
class T3TestApplication {
public:
    /** Constructor for the class T3TestApplication.
      *
      * The passed command line arguments are passed and member variables
      * are initialized accordingly.
      */
    T3TestApplication(int argc,char** argv);

    /** Virtual destructor of the class T3TestApplication. */
    virtual ~T3TestApplication();
    
    /** Close the working context of the application. */
    int Finish();
    
    /** Return the applications status. */
    int GetStatus()
     { return fStatus; }

    /** Initialize the working context of the application. */
    int Init();
    
    /** The applications main loop. */
    int Run();
    
protected:

#ifdef BLOB_TEST
    /**  */
#endif // BLOB_TEST
#ifdef HOUGH_TEST
    /**  */
    void DoHoughSel(TEyeEvent *,EyeEventClassifier *,HoughSel *);
#endif // HOUGH_TEST

    /** Parse the command line for parameters. */
    void Parse();
    
    /**  */
    void Usage();
    
    /** 'argc: from programs command line. */
    int                  fArgc;
    
    /** 'argv' from programs command line. */
    char               **fArgv;
    
    /** Current status of the application. */
    int                  fStatus;
    
    unsigned int         fNEventLimit;
    bool                 fReadMiEvent;
    
    std::string          fInputFilename;
    std::string          fLogFilename;
    std::string          fOutputFilename;
    std::string          fRejOutputFilename;
    
    TMirrorEventFile    *fMiInputFile;
    TEyeEventFile       *fEyeInputFile;
    TEyeEventFile       *fOutputFile;
    TEyeEventFile       *fRejOutputFile;
    
    EyeEventClassifier  *fEventClassifier;
};

#endif  // _T3TestApp_hh_
