
//
// File   : TltTestApp.hh
//
// Purpose: Declaration of the class TltTestApplication
//
// $Id: TltTestApp.hh 9725 2010-09-28 07:31:17Z mathes $
//


#include <string>

/** @file TltTestApp.hh
  * Declaration of the class TltTestApplication.
  * @author H.-J. Mathes, FzK
  */


#ifndef _TltTestApp_hh_
#define _TltTestApp_hh_

#include <cstdio>
#include <string>

#include <VTltProcessor.hh>

#include <MiEvent.hh>
#include <MiEventFile.hh>

#include <EyeEvent.hh>
#include <EyeEventFile.hh>


/** A test frame to test various TLT algorithms.
  *
  * A few of them are already implemented in the default application.
  */
class TltTestApplication {
public:
    /** Constructor for the class TltTestApplication.
      *
      * The passed command line arguments are passed and member variables
      * are initialized accordingly.
      */
    TltTestApplication(int argc,char** argv);

    /** Virtual destructor of the class TltTestApplication. */
    virtual ~TltTestApplication();
    
    /** Close the working context of the application. */
    int Finish();
    
    /** Return the applications status. */
    int GetStatus()
     { return fStatus; }
    
    /** Get a handle to the active TLT processor (VTltProcessor). */
    VTltProcessor* GetTltProcessor()
     { return fTltProcessor; }

    /** Get the applications general verbosity level. */
    unsigned int GetVerbosityLevel()
     { return fVerbosity; }
     
    /** Initialize the working context of the application. */
    int Init();
    
    /** The applications main loop. */
    int Run();
    
    /** Show the current setting of this class. */
    void ShowSettings(std::ostream& = std::cout);
    
    /** Set TEyeEvent output format. */
    void SetEyeEventFormat()
     { fUseEyeEvent = true; }

protected:

    /** Parse the command line for parameters. */
    void Parse();
    
    /**  */
    void Usage();
    
    /** 'argc: from programs command line. */
    int               fArgc;
    
    /** 'argv' from programs command line. */
    char            **fArgv;
    
    /** Current status of the application. */
    int               fStatus;
    
    unsigned int      fNEventLimit;
    bool              fUseEyeEvent;
    bool              fFillNtuple;
    
    unsigned int      fVerbosity;
    
    std::string       fInputFilename;
    std::string       fOutputFilename;
    std::string       fRejectOutputFilename;
    std::string       fLogFilename;

    TMirrorEventFile *fInputFile;
    unsigned int      fRunNumber;
    
    std::string       fParameterFilename;
    
    TEyeEventFile    *fEyeOutputFile;
    TMirrorEventFile *fMiOutputFile;
    TEyeEventFile    *fEyeRejectOutputFile;
    TMirrorEventFile *fMiRejectOutputFile;
    
    FILE             *fLogFile;
    
    bool              fUseMTLT;
    VTltProcessor    *fTltProcessor;
};

#endif  // _TltTestApp_hh_
