//
// File   : eventcat.cc
//
// Purpose: Concatenate data files.
//
// $Id: eventcat.cc 12007 2011-02-07 14:57:08Z mathes $
//


#include <iostream>

/** @file eventcat.cc
  * Concatenate data files.
  * @author Stefano Argir
  */
  
static const char CVSId[] = 
"$Id: eventcat.cc 12007 2011-02-07 14:57:08Z mathes $";

#include <TTree.h>

#include <EyeEventFile.hh>
#include <EyeEvent.hh>

using namespace std;

int main(int argc, char** argv) {

  if ( argc < 3 ) {
    cout << "Usage: eventcat <file1.root> <file2.root> .. "
	 << "<filen.root>  <outfile.root> " << endl;
    cout << "Example: eventcat ls *_FD_1*.root hybrid.root" << endl;
    exit(EXIT_FAILURE);
  }
  
  if ( ROOT_VERSION_CODE >= ROOT_VERSION(5,0,0) ) {
    TTree::SetMaxTreeSize( 10000000000LL );
    
    cout << "Setting maximum size of TTree to " << TTree::GetMaxTreeSize() 
         << " !" << endl;
  }
  
  // --- set the print level of the TEye* and TMirror* classes ...
  
  TEyeEventFile::SetPrintLevel( 0 );
  TEyeEvent::SetPrintLevel( 0 );
  TMirrorEvent::SetPrintLevel( 0 );
 
  TEyeRunHeader *header = NULL;
  TEyeRunTrailer *trailer = NULL;
  
  TEyeEvent * inputEvent  = new TEyeEvent();
  TEyeEvent * outputEvent = new TEyeEvent();

  TEyeEventFile outputfile(argv[argc-1], "RECREATE");

  int nevents=0;

  for (int filen = 1; filen < argc-1; filen++ ) {

    cout << argv[filen] << endl;
  
    TEyeEventFile inputfile( argv[filen], "READ" );

    if ( inputfile.IsZombie() ) {
      cout << "File " << argv[filen] << " not found " << endl;
      exit(EXIT_FAILURE);
    }
    
    // try to read header/trailer object (once!)  
    bool header_found = (header !=  NULL);
    
    if ( inputfile.ReadRunHeader( &header ) == kTRUE && !header_found )
      outputfile.WriteRunHeader( header );
    else
      cout << "TEyeRunHeader in file " << argv[filen] << " skipped !" << endl;
    
    bool trailer_found = (trailer !=  NULL);
    
    if ( inputfile.ReadRunTrailer( &trailer ) == kTRUE && !trailer_found )
      outputfile.WriteRunTrailer( trailer );
    else
      cout << "TEyeRunTrailer in file " << argv[filen] << " skipped !" << endl;
    
    // read all events ...
    while ( (inputfile.ReadEvent( &inputEvent ) == kTRUE) ) {
      
      outputEvent->Clear();
      
      *outputEvent = *inputEvent;
      
      // int save_debug = gDebug;
      // gDebug = 2;
      outputfile.WriteEvent( &outputEvent );
      // gDebug = save_debug;
      
      nevents++;

    } // while
    
  } // for

  delete outputEvent;
  delete inputEvent;
  
  cout <<  nevents << " events written to " << outputfile.GetName() << endl;

  return 0;
}

// Configure (x)emacs for this file ...
// Local Variables:
// mode:c++
// compile-command: "make -k"
// End:
