
//
// File   : t3test.cc
//
// Purpose: Simple test program to read events in TEyeEvent format
//
// $Id: t3test.cc 11539 2010-09-30 14:31:54Z mathes $
//


#include <iostream>
#include <iomanip>
#include <cstdlib>


/** @file t3test.cc
  * Simple test read program for Eye event format.
  * @author Manys ...
  */
  
#include <MiEvent.hh>
#include <EyeEvent.hh>
#include <EyeEventFile.hh>

#include <ErrorLogging.hh>

#include "T3TestApp.hh"

using namespace std;
using FdUtil::ErrorLogger;

// -------------------------------------------------------------------------

/*
 * Use with valgrind:
 *
 *  valgrind --tool=memcheck --leak-check=full --leak-resolution=med \ 
 *           --error-limit=no -v ./t3test > valgrind.log 2>&1
 *
 */

int main(int argc, char **argv)
 {
  // --- setup the logging stream
  
  ErrorLogger &logger = ErrorLogger::GetInstance();

  logger.SetVerbosity( ErrorLogger::eTerse );
  logger.SetOption( ErrorLogger::eShowSeverity, false );
  logger.SetOption( ErrorLogger::eShowGPSTime, false );
  
  logger.SetStream( &std::cout );
  
  // --- set lowest possible logging level

  TMirrorEvent::SetPrintLevel( 0 );
  TEyeEvent::SetPrintLevel( 0 );
  TEyeEventFile::SetPrintLevel( 0 );
  
  T3TestApplication app( argc, argv );
  
  if ( app.Init() != 0 ) {
    FD_CERR << argv[0] << ": failed during initialisation!" << endl;
    exit( EXIT_FAILURE );
  }
  
  int status;
  
  // --- main event loop ...
  
  status = app.Run();
  
  if ( app.Finish() != 0 ) {
    FD_CERR << argv[0] << ": failed during shutdown!" << endl;
  }
  
  exit( EXIT_SUCCESS );
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

