
//
// File   : tlttest.cc
//
// Purpose: A test environment for the TLT algorithm. 
//
// $Id: tlttest.cc 9725 2010-09-28 07:31:17Z mathes $
//


#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <string>
#include <climits>


/** @file tlttest.cc
  * A test environment for the TLT algorithm.
  * @author H.-J. Mathes, FzK
  */

#include <FdNumbering.hh>
#include <ErrorLogging.hh>

#include <PrototypeTltProcessor.hh>
#include <AnalysedFadcData.hh>
#include <AnalysedPixelData.hh>

#include <MStcCutTlt.hh>

#include "TltTestApp.hh"


using namespace std;
using FdUtil::ErrorLogger;
using FdUtil::Fd;

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------

int main(int argc, char **argv)
 {
  ErrorLogger &logger = ErrorLogger::GetInstance();
  
  // the default is std::clog
  logger.SetStream( &std::cout );
    
  logger.SetVerbosity( ErrorLogger::eTerse );

  TltTestApplication app( argc, argv );
  
  if ( app.Init() != 0 ) {
    FD_CERR << argv[0] << ": failed during initialisation!" << endl;
    exit( EXIT_FAILURE );
  }
  
  if ( ROOT_VERSION_CODE >= ROOT_VERSION(5,0,0) ) {
    TTree::SetMaxTreeSize( 10000000000LL );
    
    cout << "Setting maximum size of TTree to " << TTree::GetMaxTreeSize() 
         << " !" << endl;
  }
  
  // --- setup the TLT processor ...

  VTltProcessor *tlt = app.GetTltProcessor();
  
  if ( dynamic_cast<PrototypeTltProcessor *>(tlt) != NULL ) {
  
    FD_COUT << argv[0] << ": TLT processor= PrototypeTltProcessor" << endl;
  
    PrototypeTltProcessor *pTlt = dynamic_cast<PrototypeTltProcessor *>(tlt);
    
    //tlt->SetLogfile( tltlog );
    pTlt->SetPrintLevel( app.GetVerbosityLevel() );

    MiReadout::TltParametersRec tltpar = DEFAULT_TLT_PARAMETERS;
  
    // --- modify some of the TLT parameters ...
  
    // tltpar.fCoincidenceWindow = ;
    // tltpar.fNumPatternToAnalyse  = ;
    tltpar.fKeepRejectedFraction = 0;
    // tltpar.fFirstSltBin  = ;
    // tltpar.fLastSltBin = ;
    tltpar.fMaxEventSize = 440;
    tltpar.fCutParameter1 = 0.4;
    // tltpar.fCutParameter2 = ;
  
    pTlt->SetParameters( &tltpar );
  
    TAnalysedPixelData::SetPrintLevel( app.GetVerbosityLevel() );
    TAnalysedFADCData::SetPrintLevel( app.GetVerbosityLevel() );
  }
  else if ( dynamic_cast<MStcCutTlt *>(tlt) != NULL ) {
  
    FD_COUT << argv[0] << ": TLT processor= MStcCutTlt" << endl;
  
    MStcCutTlt * mTlt = dynamic_cast<MStcCutTlt *>(tlt);

#if 0
    MiReadout::MStcCutTltParametersRec tlt_parrec = MSTC_TLT_DEFAULT_PARAMETERS;
  
    // --- modify some of the TLT parameters ...
  
    tlt_parrec.fPercentageRejKept = 0.0;
  
    // --- TEST: modify guessed parameters for V4 hardware
    
    tlt_parrec.fNEarlyCutValue= 8; /* default = 6 */

    tlt_parrec.fPeakBoxcar  = 4; /* default = 2 */
    tlt_parrec.fStcCutValue = 2; /* default = 3 */
    
    mTlt->SetParameters( &tlt_parrec );
#endif

    mTlt->SetPrintLevel( app.GetVerbosityLevel() );

#ifdef MSTC_CALC_MULTIPLICITY
    FD_CWARN << argv[0] << ": MStcCutTlt is calculating the multiplicity !" << endl;
#endif // MSTC_CALC_MULTIPLICITY
  }
  else {
    FD_CERR << argv[0] << ": no TLT algorithm is defined !" << endl;
    exit( EXIT_FAILURE );
  }
    
  tlt->PrintSettings( cout );
  
  app.ShowSettings();
  
  int status;
  
  // --- main event loop ...
  
  status = app.Run();
  
  if ( app.Finish() != 0 ) {
    FD_CERR << argv[0] << ": failed during shutdown!" << endl;
  }
  
  exit( EXIT_SUCCESS );
}

// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
