#ifndef _MD_COUNTER_H
#define _MD_COUNTER_H

#include "TObject.h"
#include "MdModule.h"
#include <vector>

namespace md {

class Counter : public TObject {

public:

   //typedef std::vector< counted_ptr<Module> > ModuleList;
   typedef std::vector< Module* > ModuleList;
   typedef ModuleList::iterator Iterator;
   typedef ModuleList::const_iterator ConstIterator;
 
   Counter(); ///< Default constructor
   Counter( const Counter & ); ///< Copy constructor
   Counter& operator=( const Counter & ); ///< Assignment
   Counter( ID ); ///< ID constructor
   virtual ~Counter(); ///< Default destructor

   Iterator ModulesBegin() { return Iterator( fModList.begin() );} ///< Returns iterator to module beginning
   Iterator ModulesEnd()   { return Iterator( fModList.end() );}   ///< Returns iterator to module end

   ConstIterator ModulesBegin() const { return ConstIterator( fModList.begin() );} ///< Returns const_iterator to module beginning
   ConstIterator ModulesEnd()   const { return ConstIterator( fModList.end() );}   ///< Returns const_iterator to module end

   void ClearModules();     ///< Clear all module list
   void EraseModule( Iterator sPos ); ///< Erase element at sPos
   void EraseModules( Iterator sStart, Iterator sEnd ); ///< Erase element from sStart to sEnd

   Module * MakeModule( ID );
   Module * GetModule ( ID );
   Module * HasModule ( ID );

   size_t ListSize()  { return fModList.size();}  ///< 
   size_t HasModules() { return fModList.size(); }///< Checks module list

   void SetId( ID id ){ fId = id;}///< Set counter id
   ID  GetId( ) { return fId;}///< Get counter id

   void AddModule ( const Module & );

   void SetT3Delay( unsigned int d ) { fT3Delay = d; }
   unsigned int GetT3Delay () { return fT3Delay; }

private:

   ModuleList fModList;///< Collection of modules (Module objects)
   ID      fId;///< Counter Id
   unsigned int fT3Delay; ///< Delay of T3 at station with retrieved T1 (if found)

   void DeleteModules();///< Free memory

   ClassDef(Counter,1)  //Counter structure
};

}//end namespace

#endif
