#include "MdEvent.h"
#include <iostream>
using std::cout;
using std::endl;

namespace md {
//========================================//
Trigger::Trigger():fType( eNone ), fFoldLevel( eVoid ), fT3Sec(0){}
Trigger::Trigger( const Trigger & mdTrg )
{
 fType = mdTrg.fType;
 fFoldLevel = mdTrg.fFoldLevel;
 fT3Sec = mdTrg.fT3Sec;
}
Trigger::~Trigger(){}
//========================================//
////////////// Ctrs and Dtrs /////////////////////
Event::Event():fId(-1) 
{
  time_t rawtime = time(NULL);
  struct tm utc = *(gmtime(&rawtime));
  fUTCtime = mktime ( &utc );
  fUTCdate = ctime( &fUTCtime );
};

Event::Event( ID id ):fId( id ) 
{
  time_t rawtime = time(NULL);
  struct tm utc = *(gmtime(&rawtime));
  fUTCtime = mktime ( &utc );
  fUTCdate = ctime( &fUTCtime );
};

Event::Event( const Event & mdEvt) : TObject()
{  
  *this = mdEvt;
};

Event & Event::operator=(const Event &mdEvt){
  fId      = mdEvt.fId;
  fUTCtime = mdEvt.fUTCtime;
  fUTCdate = mdEvt.fUTCdate;
  fTrigger = mdEvt.fTrigger;
  ClearCounters();
  for(ConstIterator i = mdEvt.CountersBegin(), e = mdEvt.CountersEnd() ; i != e; ++i ){
    fCtrList.push_back(new Counter(**i));
  }
  return *this;
}

Event::~Event()
{
   this->ClearCounters();
};
//////////////////////////////////////////////////
void Event::SetUTCtime( UTCTime time )
{ 
  struct tm utc = *(gmtime(&time));
  fUTCtime = mktime ( &utc );
  fUTCdate = ctime(&fUTCtime);

}
void Event::ClearCounters(){
   DeleteCounters();
   fCtrList.clear();
};

void Event::DeleteCounters(){
   for(Iterator i = CountersBegin(), e = CountersEnd() ; i != e; ++i )
       delete *i;
};

void Event::EraseCounter( Iterator sPos )
{
  delete *sPos;
  fCtrList.erase( sPos );
}

void Event::EraseCounters( Iterator sStart, Iterator sEnd ){
   for(Iterator i = sStart, e = sEnd ; i != e; ++i )
       delete *i;
   fCtrList.erase( sStart, sEnd );
};
/*
void Event::CreateCounter( ID id )
{
  Counter * mdCtr = new Counter( id );
  fCtrList.push_back( counted_ptr<Counter>( mdCtr ) );
};
*/
Counter * Event::MakeCounter( ID id )
{
  Counter * mdCtr = new Counter( id );
  mdCtr->SetId( id );
  fCtrList.push_back( mdCtr );
  //fCtrList.push_back( counted_ptr<Counter>( mdCtr ) );
  return mdCtr;
};

Counter * Event::GetCounter( ID id )
{
  for(Iterator i = CountersBegin(), e = CountersEnd() ; i != e; ++i )
    if( (*i)->GetId() == id )
    {
      //return (*i).get();
      return (*i);
      break;
    }

    //ID not found
    return 0;
};

Counter * Event::HasCounter( ID id )
{
   return GetCounter( id );
};

}//end namespace

