#ifndef _MD_EVENT_H
#define _MD_EVENT_H

#include "TObject.h"
#include "MdCounter.h"

#include "IoMdVersion.h"

#include <vector>
#include <string>
#include <ctime>


namespace md {
//========================================//
class Trigger 
{
  public:
    enum TriggerType { eOccupancy, eT3, eNone };

    enum OccLevel {  eVoid,
           eOcc1, eOcc2, eOcc3, eOcc4, eOcc5, eOcc6, eOcc7, eOcc8,
           eOcc9, eOcc10,eOcc11,eOcc12,eOcc13,eOcc14,eOcc15,eOcc16,
           eOcc17,eOcc18,eOcc19,eOcc20,eOcc21,eOcc22,eOcc23,eOcc24,
           eOcc25,eOcc26,eOcc27,eOcc28,eOcc29,eOcc30,eOcc31,eOcc32,
           eOcc33,eOcc34,eOcc35,eOcc36,eOcc37,eOcc38,eOcc39,eOcc40,
           eOcc41,eOcc42,eOcc43,eOcc44,eOcc45,eOcc46,eOcc47,eOcc48,
           eOcc49,eOcc50,eOcc51,eOcc52,eOcc53,eOcc54,eOcc55,eOcc56,
           eOcc57,eOcc58,eOcc59,eOcc60,eOcc61,eOcc62,eOcc63,eOcc64,
            };

    Trigger();
    Trigger( const Trigger &);
    virtual ~Trigger();
  
    TriggerType Type(){ return fType;}
    void SetType( TriggerType type ){ fType = type; }
  
    OccLevel OccupancyLevel(){return fFoldLevel;}
    void SetOccupancyLevel ( OccLevel occ ){ fFoldLevel = occ; this->fType=eOccupancy; }

    unsigned int GetT3Sec() { return fT3Sec;}
    void SetT3Sec( int t3 ) { fT3Sec = t3;}

#ifdef VER_CHECK
  #if MAJOR_VER > 1 || ( MAJOR_VER == 1 && MINOR_VER > 0 )
    unsigned int GetT3Mic() { return fT3Mic;}
    void SetT3Mic( int t3 ) { fT3Mic = t3;}
//    #warning "COMPILING WITH VERSION > 1.0 (with T3MicroSecond)"
  #endif
#endif

  private:

    TriggerType fType;///< Trigger type
    OccLevel    fFoldLevel;///< Trigger type
    unsigned int fT3Sec; ///< T3Second
    unsigned int fT3Mic; ///< T3iMcroSecond

};
//========================================//
class Event : public TObject {

public:

   //typedef std::vector< counted_ptr<Counter> > CtrList;
   typedef std::vector< Counter* > CtrList;
   typedef CtrList::iterator Iterator;
   typedef CtrList::const_iterator ConstIterator;
   typedef time_t UTCTime; 
 
   Event(); ///< Default constructor
   Event( const Event & ); ///< Copy constructor
   Event& operator=( const Event & ); ///< Assignment operator
   Event( ID );
   virtual ~Event(); ///< Default destructor

   Iterator CountersBegin() { return Iterator( fCtrList.begin() );} ///< Returns iterator to counter beginning
   Iterator CountersEnd()   { return Iterator( fCtrList.end() );}   ///< Returns iterator to counter end

   ConstIterator CountersBegin() const { return ConstIterator( fCtrList.begin() );} ///< Returns const_iterator to counter beginning
   ConstIterator CountersEnd()   const { return ConstIterator( fCtrList.end() );}   ///< Returns const_iterator to counter end

   void ClearCounters(); ///< Clear all counter list
   void EraseCounter( Iterator sPos ); ///< Erase element at sPos
   void EraseCounters( Iterator sStart, Iterator sEnd );///< Erase element from sStart to sEnd

   Counter * MakeCounter( ID );
   Counter * GetCounter ( ID );
   Counter * HasCounter ( ID );

   size_t CounterList()  { return fCtrList.size();}  ///< Size of counters
   size_t HasCounters() { return fCtrList.size(); }///< Checks counter list

   void SetId( ID id ){ fId = id;}///< Set event id
   ID  GetId( ) { return fId;} ///< Get event id

   void SetUTCtime( UTCTime time );
   UTCTime     GetUTCtime(){ return fUTCtime;}
   std::string GetUTCdate(){ return fUTCdate;}

   Trigger & GetTrigger(){ return fTrigger;}///< Get Trigger class

private:

   CtrList fCtrList;///< Collection of counters (Counter objects)
   ID      fId;///< Event Id this value is internally managed when writing (reading) from (to) file 
   UTCTime     fUTCtime;///< UTC time: if not set by user, it is internally initialize by constructor using machine time
   std::string fUTCdate;///< UTC date (uses fUTCtime)
   Trigger   fTrigger;///< Moun detector trigger

   void DeleteCounters();///< Free memory

   ClassDef(Event,1)  //Event structure
};

}//end namespace
#endif
