#ifndef _MdFile_h_
#define _MdFile_h_

#include <vector>
#include <string>

#include "TKey.h"
#include "MdKey.h"
#include "counted_ptr.h"

class TFile;
class TIter;

namespace md {

class Event;

class File {
public:

  typedef std::vector < counted_ptr<TFile> >FileList;
  typedef FileList::iterator FileListIterator;

  /// Open modes
  enum Mode { eRead, eWrite, eAppend };
  /// Return status
  enum Status { eSuccess, eFailure, eEOF };

  File();
  File(const File & theFile);
  File(int argc, char **argv);
  File(std::vector<std::string>);
  /// Construct and open an File
  File(const char *filename, Mode mode = eRead);
  ~File();

  /// Linear read - Return File::eEOF if no more events available
  Status ReadNext(Event & event);
  /**Random access Read - @see Key for indexing rules
   * Return File::eFailure 
   * if no more events available */
  Status Read(Event & event, Key & key);

  /// Write @param event to file
  Status Write(Event & event, bool verbose = false);

  /// Open one file in given @mode
  Status Open(const char *filename, Mode mode);

  /** Open @param nfiles in @param filenames for read only
   *  Files in @param filenames are separated by <space>	
   */
  Status Open(const char *filenames);

  unsigned int NumberOfEvents();

  int GetBytesWritten(); 
  const char * GetFileName();

  /// Close file
  void Close();

private:
  Status ReadObj(TKey * key, Event & event);

  FileList fFiles;
  FileListIterator fFilesIterator;
  TIter *fKeyIterator;
  unsigned long int fCycle;///< manage to have an unique eventId index

};                              // File

}//end namespace

#endif 
