#include "MdModule.h"
#include <iostream>
using std::cout;
using std::endl;
namespace md {
////////////// Ctrs and Dtrs /////////////////////
Module::Module():fId(-1), fHasT1(false)
{
  fPos.X = fPos.Y = fPos.Z = 0;
};

Module::Module( const Module & mdMod) : TObject()
{
  *this = mdMod;
};

Module & Module::operator=( const Module & mdMod)
{
 fId     = mdMod.fId;
 fPos    = mdMod.fPos; 
 fHasT1  = mdMod.fHasT1;
 //fSigMap = mdMod.fSigMap;

 ClearRawSignal();
 for(ConstRawIterator i = mdMod.RawSignalBegin(), e = mdMod.RawSignalEnd() ; i != e; ++i ){
    fSigMap.push_back(new RawSample(**i));
  }

 ClearChannels();
  for(ConstIterator i = mdMod.ChannelsBegin(), e = mdMod.ChannelsEnd() ; i != e; ++i ){
    fChList.push_back(new Channel(**i));
  }
  return *this;
};

Module::Module( ID id):fId( id ), fHasT1( false )
{
  fPos.X = fPos.Y = fPos.Z = 0;
};

Module::~Module()
{
   this->ClearChannels();
   this->ClearRawSignal();
};
//////////////////////////////////////////////////
void Module::ClearRawSignal(){
   DeleteRawSignal();
   fSigMap.clear();
};

void Module::DeleteRawSignal(){
   for(RawIterator i = RawSignalBegin(), e = RawSignalEnd() ; i != e; ++i )
       delete *i;
};

void Module::EraseRawSignal( RawIterator rPos )
{
  delete *rPos;
  fSigMap.erase( rPos );
}

void Module::ClearChannels(){
   DeleteChannels();
   fChList.clear();
};

void Module::DeleteChannels(){
   for(Iterator i = ChannelsBegin(), e = ChannelsEnd() ; i != e; ++i )
       delete *i;
};

void Module::EraseChannel( Iterator sPos )
{
  delete *sPos;
  fChList.erase( sPos );
}

void Module::EraseChannels( Iterator sStart, Iterator sEnd ){
   for(Iterator i = sStart, e = sEnd ; i != e; ++i )
       delete *i;
   fChList.erase( sStart, sEnd );
};

Channel * Module::MakeChannel( ID id )
{
  Channel * mdCh = new Channel( id );
  mdCh->SetId( id );
//  fChList.push_back( counted_ptr<Channel>(mdCh) );
  fChList.push_back( mdCh );
  return mdCh;
};

Channel * Module::GetChannel( ID id )
{
  for(Iterator i = ChannelsBegin(), e = ChannelsEnd() ; i != e; ++i )
    if( (*i)->GetId() == id )
    {
      //return (*i).get();
      return (*i);
      break;
    }

    //ID not found
    return 0;
};

Channel * Module::HasChannel( ID id )
{
   return GetChannel( id );
};
 
void Module::AddRawSample( unsigned int tbin, Long64_t sample )
{
 RawSample * mdraw = new RawSample( tbin, sample ); 
 fSigMap.push_back( mdraw );
}

}//end namespace

