#ifndef _MD_MODULE_H
#define _MD_MODULE_H

#include "TObject.h"
#include "MdChannel.h"
#include "RawSample.h"
#include <vector>

namespace md {

class Module : public TObject {

public:

   typedef std::vector<RawSample *> RawSigMap;///< map of pairs: (bin, 64-bits integer) -> raw signal each 64-bit contains all the channels at a given sample
   typedef RawSigMap::iterator RawIterator;
   typedef RawSigMap::const_iterator ConstRawIterator;
 
   //typedef std::vector< counted_ptr<Channel> > ChannelList;
   typedef std::vector< Channel* > ChannelList;
   typedef ChannelList::iterator Iterator;
   typedef ChannelList::const_iterator ConstIterator;
 
   Module(); ///< Default constructor
   Module( const Module & ); ///< Copy constructor
   Module & operator=( const Module & ); ///< 
   Module( ID ); ///< ID constructor
   virtual ~Module(); ///< Default destructor

   RawIterator RawSignalBegin()  { return RawIterator( fSigMap.begin() );} ///< Returns _iterator to raw signal beginning
   RawIterator RawSignalEnd()    { return RawIterator( fSigMap.end() );}   ///< Returns _iterator to raw signal end
   ConstRawIterator RawSignalBegin() const { return ConstRawIterator( fSigMap.begin() );} ///< Returns const_iterator to raw signal beginning
   ConstRawIterator RawSignalEnd()   const { return ConstRawIterator( fSigMap.end() );}   ///< Returns const_iterator to raw signal end

   Iterator ChannelsBegin() { return Iterator( fChList.begin() );} ///< Returns iterator to channel beginning
   Iterator ChannelsEnd()   { return Iterator( fChList.end() );}   ///< Returns iterator to channel end

   ConstIterator ChannelsBegin() const { return ConstIterator( fChList.begin() );} ///< Returns const_iterator to channel beginning
   ConstIterator ChannelsEnd()   const { return ConstIterator( fChList.end() );}   ///< Returns const_iterator to channel end

   void ClearChannels();   ///< Clear all scintillator list
   void EraseChannel( Iterator cPos );///< Erase element at sPos
   void EraseChannels( Iterator cStart, Iterator cEnd );///< Erase element from sStart to sEnd

   void ClearRawSignal();   ///< Clear raw signal
   void EraseRawSignal( RawIterator rPos );///< Erase element (sample) at rPos

   Channel * MakeChannel( ID );///< Create a scintillator with a given ID
   Channel * GetChannel( ID );///<
   Channel * HasChannel( ID );///<

   size_t ListChannelSize()  { return fChList.size();}  ///<  
   size_t HasChannels() { return fChList.size(); }///< Checks scintillator list

   void SetId( ID id ){ fId = id;}///< Set scintillator id (1-4)
   ID  GetId( ) { return fId;} ///< Get scintillator id (1-4)

   void SetT1( bool T1 ){ this->fHasT1 = T1;}///< Set if this Module had data when the T3 arrived
   bool HasT1( ) { return fHasT1;} ///< True if the T1 was found in this module

   void SetPosition( Pos pos ){ fPos.X = pos.X; fPos.Y = pos.Y; fPos.Z = pos.Z;}
   Pos GetPosition( ){ return fPos;}
   
   void   AddRawSample( unsigned int, Long64_t );   /*{ fSigMap[tbin] = sample; }*/
private:

   ChannelList fChList;///< Collection of scintillators (Channel objects)
   ID      fId;///< Module Id (1-4)
   Pos     fPos;///< Module position with respect to tank 
   bool    fHasT1;

   void DeleteChannels();
   void DeleteRawSignal();
   RawSigMap fSigMap;///< Collection of 64-bit samplings (raw event)

   ClassDef(Module,1)  //Module structure
};

}//end namespace
#endif
