#ifndef _MD_SIGNAL_H
#define _MD_SIGNAL_H

#include <vector>
#include "TObject.h"
#include "RawSample.h"

namespace md{

typedef unsigned int ID;
typedef double XPos;///< X coordinate
typedef double YPos;///< Y coordinate
typedef double ZPos;///< Z coordinate

class Pos {
public:
 Pos():X(0),Y(0),Z(0){};
 Pos( const Pos & pos ){ X = pos.X; Y = pos.Y; Z = pos.Z;}
 virtual ~Pos(){};
 XPos X; 
 YPos Y; 
 ZPos Z;
private:
};

class Signal : public TObject {

public:

   typedef std::vector<RawSample *> SigSample;///< map of pairs: (tbin, sample) -> used for one channel only
   typedef SigSample::iterator       Iterator;
   typedef SigSample::const_iterator ConstIterator;

   typedef unsigned int GPSTime;
   typedef unsigned int eChannel;

   Signal(); ///< default constructor
   Signal( const Signal & ); ///< copy constructor
   Signal & operator=( const Signal & ); ///< 
   Signal( GPSTime, eChannel );///< GPS and electronic channel  Ctor
   virtual ~Signal(); ///< default destructor

   Iterator Begin() { return Iterator( fSigSamples.begin() );} ///< Returns iterator to signal beginning
   Iterator End()   { return Iterator( fSigSamples.end() );}   ///< Returns iterator to signal end

   ConstIterator Begin() const { return ConstIterator( fSigSamples.begin() );} ///< Returns const_iterator to signal beginning
   ConstIterator End()   const { return ConstIterator( fSigSamples.end() );}   ///< Returns const_iterator to signal end

   size_t  Size()  { return fSigSamples.size();}  ///<
   bool IsEmpty() { return fSigSamples.empty();} ///< Size check

   void SetGPS( GPSTime gps ){ fGPSTime = gps;} ///< 
   GPSTime GetGPS( ){ return fGPSTime;}///< 

   void SetChannel( unsigned int ch ){ fEChannel = ch;}///< 
   int  GetChannel(  ){ return fEChannel;}///< 

   void ClearSignal();   ///< Clear raw signal
   void EraseSignal( Iterator );///< Erase element (sample) at rPos
   void EraseSignals( Iterator , Iterator );///< Erase element (sample) at rPos

   void   AddSample( unsigned int, Long64_t ); 

private:

   void DeleteSignal();

   SigSample  fSigSamples; ///< 
   GPSTime  fGPSTime;///< GPS time (should be a T3 from the tank)
   eChannel fEChannel;///< Electronic board channel (Warning: do not correspond to scintillator number neither to PMT pixel number)

   ClassDef(Signal,1)  //Basic raw md data class
};


};//end namespace

#endif
