#include "MdFile.h"
#include <iostream>
using namespace std;

int main(int argc, char*argv[])
{
  std::cout << "Testing Data container\n\n";
  int i;
  double sample;
  time_t rawtime;
  md::ID id;
  md::Trigger::OccLevel occ;


  id = 200;
  //Create an event with ID
  md::Event myEvt ( id );

  rawtime = time(NULL);//set to local machine time
  //rawtime = 0;//This is date: Thu Jan  1 00:00:00 1970

  myEvt.SetUTCtime( rawtime );//will convert local time to UTC time

  occ = md::Trigger::eOcc8;//occupancy level 8
  myEvt.GetTrigger().SetOccupancyLevel( occ );

  cout << "Get utc date " <<  myEvt.GetUTCdate() << endl;
  // or 
  //MdTrigger & myTrg = myEvt.GetTrigger();
  //myTrg.SetOccupancyLevel( occ );

  id = 2;
  md::Counter * myCtr =  myEvt.MakeCounter( id );

  id = 1;
  md::Module * myModule =  myCtr->MakeModule( id );

  if( !(myCtr->HasModule(id)) ) 
      cout << "Module " << id << " does not exist!!\n\n";
  else
      cout << "Module " << id << " does exist!!\n\n";

  id = 31;
  md::Channel * myChannel = myModule->MakeChannel( id );

  if( myChannel->HasSignal() ) cout<< "Data contains no entries!\n";

  //This is a reference, will store data in myChannel object
  md::Signal & mySig = myChannel->GetSignal();
  //Fill signal 
  for( i = 0; i < 20; ++i ){
     sample = (200.-i)/2;
     cout << "filling... " << i << " " << sample << endl;
     //One option
     //myChannel->GetSignal().AddSample( sample );
     //other option
     //mySig.AddSample( sample );

     mySig.AddSample( i, sample );
  }
  cout << endl << endl;

  //Now read signal data
  md::Signal::Iterator sigIt;
  for(  sigIt = myChannel->GetSignal().Begin(); sigIt != myChannel->GetSignal().End(); ++sigIt )
     cout << "reading... tbin "<< (*sigIt)->GetTimeBin() << " " << (*sigIt)->GetSample() << endl;

  md::Channel * otherChannel;

  id = 17;
  if( !(otherChannel=myModule->HasChannel(id)) ) cout << "Channel " << id << " does not exist!!\n\n";

  id = 31;
  if( (otherChannel=myModule->HasChannel(id)) ){ 
     //Now read signal data
     for(  sigIt = otherChannel->GetSignal().Begin(); sigIt != otherChannel->GetSignal().End(); ++sigIt )
     cout << "reading... tbin "<< (*sigIt)->GetTimeBin() << " " << (*sigIt)->GetSample() << endl;
  }
  
  cout << endl << endl;

  md::File::Mode mode;
  md::File ofile;

 if( ofile.Open("example.root", mode = md::File::eWrite ) == md::File::eFailure ){
     cerr << "Error Opening File to read " << endl;
     return -1;//exit(1);
  }

  int verbosity = 1;

  //Write one event
  ofile.Write( myEvt, verbosity );

  //Write another event
  ofile.Write( myEvt, verbosity );
 
  ofile.Close();
  return 0;
}
