#include "MdFile.h"
#include <bitset>
#include <algorithm>
#include <iomanip>
#include <iostream>
using std::cout;
using std::endl;
using std::cerr;
using std::bitset;

int main(int argc, char*argv[])
{

  md::File::Mode mode;
  md::File ifile;
  mode = md::File::eRead;

  if( argc != 2 ) {cerr << "Usage:\n"<< argv[0] << " inputfile.root\n"; return -1;}

  //Opens a single file
  if( ifile.Open(argv[1], mode ) == md::File::eFailure ){
     cerr << "Error Opening " << argv[1] << " to read " << endl;
     return -1;
  } 

  cout << "ifile.NumberOfEvents(): " << ifile.NumberOfEvents()<< endl;
  md::Event myEvt;
  unsigned int eventNo = 0;
  unsigned int eventNoWithData = 0;
  while ( ifile.ReadNext( myEvt ) == md::File::eSuccess )
  {
    cout << "Event Id " <<  myEvt.GetId() << endl;
    cout << "Get utc date " <<  myEvt.GetUTCdate() << endl;
    cout << "Get utc time " <<  myEvt.GetUTCtime() << endl;
    cout << "EventNo " << ++eventNo << endl;

    if(  myEvt.GetTrigger().Type() == md::Trigger::eOccupancy )
        cout << "Occupancy Level " <<  myEvt.GetTrigger().OccupancyLevel() << endl;
    else if( myEvt.GetTrigger().Type() == md::Trigger::eT3 )
        cout << "T3" << endl;
    else
        cout << "Undefined trigger type  " << endl;

   if( myEvt.HasCounters() ) cout << "Number of Counters: " << myEvt.CounterList() << endl;

   md::Event::Iterator   myCtrIt;
   md::Counter::Iterator myModIt;
 
   bool evFlag = true;

   myCtrIt = myEvt.CountersBegin();
   for( ; myCtrIt != myEvt.CountersEnd(); ++myCtrIt ){
     cout << "Counter Id " << (*myCtrIt)->GetId() << endl;
     myModIt = (*myCtrIt)->ModulesBegin(); 
     for( ; myModIt != (*myCtrIt)->ModulesEnd(); ++myModIt ){ 
       cout << "Module Id " << (*myModIt)->GetId() << endl;
        //Constant iterators we dont want to modify the raw signal!
         for( md::Module::ConstRawIterator rIt = (*myModIt)->RawSignalBegin();
                 rIt != (*myModIt)->RawSignalEnd() && (*rIt)->GetTimeBin(); ++rIt ){  
             std::bitset<64> bc ( (*rIt)->GetSample() );
            cout << std::setw( 10 ) << std::setfill(' ') 
            << (*rIt)->GetTimeBin() << " " 
            << bc  << " ";
            cout << " ==> (" << bc.count() << ")\n";
            if( evFlag ) {++eventNoWithData;evFlag=false;} 
         }   
     }//end modules loop
   }//end counters loop
  }//end while loop
  
  cout << "\n==================================\n";
  cout <<    "          Statistics             \n";
  cout <<  endl;
  cout <<  "Total number of events: " << eventNo << endl;
  cout <<  "   events with Md data: " << eventNoWithData << endl;
  cout << "\n==================================\n";
  ifile.Close();
  return 0;
}
