#include "MdFile.h"
#include <iostream>
using std::cout;
using std::endl;
using std::cerr;

using namespace std;

int main(int argc, char*argv[])
{

  md::File::Mode mode;
  md::File ifile;
  mode = md::File::eRead;

  if( argc != 2 ) {cerr << "Usage:\n"<< argv[0] << " inputfile.root\n"; return -1;}

  //Opens a single file
  if( ifile.Open(argv[1], mode ) == md::File::eFailure ){
     cerr << "Error Opening " << argv[1] << " to read " << endl;
     return -1;
  } 

  cout << "ifile.NumberOfEvents(): " << ifile.NumberOfEvents()<< endl;
  md::Event myEvt;

  while ( ifile.ReadNext( myEvt ) == md::File::eSuccess )
  {
    cout << "Event Id " <<  myEvt.GetId() << endl;
    cout << "Get utc date " <<  myEvt.GetUTCdate() << endl;
    cout << "Get utc time " <<  myEvt.GetUTCtime() << endl;

    if(  myEvt.GetTrigger().Type() == md::Trigger::eOccupancy )
        cout << "Occupancy Level " <<  myEvt.GetTrigger().OccupancyLevel() << endl;
    else if( myEvt.GetTrigger().Type() == md::Trigger::eT3 )
        cout << "T3" << endl;
    else
        cout << "Undefined trigger type  " << endl;

   if( myEvt.HasCounters() ) cout << "Number of Counters: " << myEvt.CounterList() << endl;

   md::Event::Iterator   myCtrIt;
   md::Counter::Iterator myModIt;
   md::Module::Iterator  myChnIt;
   md::Signal::Iterator  mySigIt;

   myCtrIt = myEvt.CountersBegin();
   for( ; myCtrIt != myEvt.CountersEnd(); ++myCtrIt ){
     cout << "Counter Id " << (*myCtrIt)->GetId() << endl;
     myModIt = (*myCtrIt)->ModulesBegin(); 
     for( ; myModIt != (*myCtrIt)->ModulesEnd(); ++myModIt ){ 
       cout << "Module Id " << (*myModIt)->GetId() << endl;
       myChnIt = (*myModIt)->ChannelsBegin();
       for( ; myChnIt != (*myModIt)->ChannelsEnd(); ++myChnIt ){ 
          cout << "Channel Id " << (*myChnIt)->GetId() << endl;
          //Now read signal data
          for(  mySigIt = (*myChnIt)->GetSignal().Begin(); mySigIt != (*myChnIt)->GetSignal().End(); ++mySigIt ){
           //cout << "reading... "<< (*mySigIt) << endl;
           cout << "reading... tbin "<< (*mySigIt)->GetTimeBin() << " " << (*mySigIt)->GetSample() << endl;
          }//end signal loop 
       }//end channels loop
     }//end modules loop
   }//end counters loop

  }//end while loop
  
  ifile.Close();
  return 0;
}
