#ifndef CDASUNIV_H
#define CDASUNIV_H

#include "UnivParam.h"

// Usage: UnivCDAS().GetSignal(...) returns signal in VEM, see parameters below

class CDASUniv {
  public:
    double GetSignal(
      double logE, 
      double Xmax,
      double Nmu, 
      double theta, 
      double z,
      double r,
      double psi, 
      int component=-1,
      int atmos=-1,
      double rhoGround=1.04e-3
      ) {
      // logE: log10(Energy [eV])
      // Xmax: depth of maximum of em cascade [g/cm2]
      // Nmu: local relative muon number (1=proton QGSJetII)
      // theta: EAS zenith angle [rad]
      // z: altitude above sea level [m]
      // r: distance to the core [m]
      // psi: asymmetry angle in the shower plane [rad]
      // component: 0: muon, 1: em, 2: muon-induced em, 3: hadronic-induced em, -1: sum of them
      // atmos: month of the year, -1: average
      // rhoGround: density at ground
      return Param->GetSignal(r*100,Xmax,logE,theta,psi,rhoGround,z*100,Nmu,component,atmos);
    };
    static CDASUniv *Instance() {
      if (!_instance) _instance = new CDASUniv();
      return _instance;
    };
  private:
    UnivParamNS::UnivParam *Param;
    static CDASUniv * _instance;
    CDASUniv() {
      Param=new UnivParamNS::UnivParam(0);
    };
    CDASUniv(CDASUniv const&);              // Don't Implement
    void operator=(CDASUniv const&); // Don't implement
};

CDASUniv& UnivCDAS() {
  return *CDASUniv::Instance();
}
#endif
