#include <UnivParam.h>
#include <Atmosphere.h>
#include <GeomAsym.h>
#include <AtmCorr.h>

using namespace UnivParamNS;


UnivParam::UnivParam(int DetectorType)
{
  if (DetectorType<0 || DetectorType>2 )
    {
      printf("UnivParam: Wrong Detector Type %d \n",DetectorType);
      exit(1);
    }
  fDetectorType=DetectorType;
}

double UnivParam::DXFunc(double x, double logE,  double *parDX) 
{ 
  
  double val=0.;  
  int icomp=int(parDX[nParDX-1]);

  double Sref0=parDX[0];
  double fS_e=parDX[1];
  double Sref=Sref0*   pow(10., (logE-19.)*fS_e );

  double DXmax=parDX[2] ;
      
  double L0=parDX[3];
  double fL_e=parDX[4];
  double lambda=L0*   (1.+(logE-19.)*fL_e) ;
      
  if (x<DX0[icomp]) val=0.; 
  else val= Sref * pow((x - DX0[icomp])/(DXref[icomp]-DX0[icomp]), (DXmax-DX0[icomp])/lambda) * exp((DXref[icomp]-x) / lambda );
  
  return val;  
}

//
double UnivParam::RFuncPar( int icomp, int iparDX , int ipar )
{
  double val=0.;
  if       ( iparDX==0 && fDetectorType==0 ) val=parSref_WCD[icomp][ipar];
  else if  ( iparDX==1 && fDetectorType==0 ) val=parGamma_WCD[icomp][ipar];
  else if  ( iparDX==2 && fDetectorType==0 ) val=parDXmax_WCD[icomp][ipar];
  else if  ( iparDX==3 && fDetectorType==0 ) val=parLambda_WCD[icomp][ipar];
  else if  ( iparDX==4 && fDetectorType==0 ) val=parfLambdaE_WCD[icomp][ipar];
  else if  ( iparDX==0 && fDetectorType==1 ) val=parSref_Scin[icomp][ipar];
  else if  ( iparDX==1 && fDetectorType==1 ) val=parGamma_Scin[icomp][ipar];
  else if  ( iparDX==2 && fDetectorType==1 ) val=parDXmax_Scin[icomp][ipar];
  else if  ( iparDX==3 && fDetectorType==1 ) val=parLambda_Scin[icomp][ipar];
  else if  ( iparDX==4 && fDetectorType==1 ) val=parfLambdaE_Scin[icomp][ipar];
  return val;
}

// 
double UnivParam::RFunc(double r, int icomp, int iparDX  )
{
  double par[4];
  for (int ipar=0;ipar<4;ipar++) par[ipar]=RFuncPar(icomp,iparDX,ipar);
  return RFunc(r,  icomp,  iparDX , par );  
} 



double UnivParam::RFunc(double r, int icomp, int iparDX , double *par) 
{ 
  double r_meter=r/1.e2;

  if ( iparDX == (nParDX-1) )
    return icomp;


  //-------------------------------------------------
  // Sref 
  //-------------------------------------------------
  if ( iparDX==0  )                    // NKG
    return par[0]*pow(r_meter/1000.,-par[1])*pow( (par[3]+r_meter)/(par[3]+1000.),-par[2] ); 
  //-------------------------------------------------
  // Scaling
  //-------------------------------------------------
  else if ( iparDX==1 )
    {
      if ( icomp ==1 || icomp==3 )   // expo + cte
	return par[0]+exp(-r_meter/500.)*par[1];
      else              //  pol0
	return par[0];
    }
  //-------------------------------------------------
  // DXmax 
  //-------------------------------------------------

  else if ( iparDX==2  )   
    {
      if (icomp==1)     // gaisser-hillas
	{ 
	  double max=par[0];
	  double rmax=par[1];
	  double rlambda=par[2];
	  double r0=par[3];
	  if ( rmax<r0 || rlambda<0. )
	    return 1.e10;
	  
	  if (r_meter> r0 )
	    return max * pow((r_meter - r0)/(rmax-r0), (rmax-r0)/rlambda) * exp((rmax-r_meter) / rlambda );
	  else
	    return 0.;
	}
      else if ( icomp==3 )   // expo + cte
	return par[0]-par[1]*exp(-par[2]*r_meter/1000.); 
      else                  // pol1
	return par[0]+(r_meter/1000.)*par[1];	
    }
  //-------------------------------------------------
  // Lambda                
  //-------------------------------------------------
  else if ( iparDX==3 )   
    {
      if (icomp==1)                                     // pol1 in r + expo
	return par[0]+r_meter/1000.*par[1]-par[2]*exp(-r_meter/150.); 
      else if (icomp==3)    //cte
	return par[0]; 
      else
	return par[0]+par[1]*exp(par[2]*r_meter/1000.);  // expo + cte
    }
  //-------------------------------------------------
  // fLambda_E
  //-------------------------------------------------
  else if ( iparDX==4 )  
    {
      // pol1  in r
      return par[0]+r_meter/1000.*par[1];
    }
  
  return -100.;
}



double UnivParam::GetS0(double r, double DX, double logE,  int icomp)
{
  //Signal in an spherical tank with shadow effects corrected and for a reference ground density of 1.04e-3 g/cm^3
  //   icomp=0,1  Muon/Em signal in [VEM]
  //   icomp=2,3  Ratio of Em Muon/Em Hadron to muonic signal
  //  
  // r             [cm]
  // DX            [g/cm^2]    Muon/Em Muon-> DX_mu     Em/Em Had -> DX_edep
  // log(E)        [eV]
  // icomp=0,1,2,3 Muon/Em/Em Muon/Em had  -1 sum

  if ( r > rLimit_high ) r= rLimit_high;
  if ( r < rLimit_low  ) r= rLimit_low;
 
  double parDX[nParDX];
  for (int iparDX=0;iparDX<nParDX;iparDX++)
    parDX[iparDX]=RFunc( r, icomp, iparDX );
  
  return DXFunc(DX,logE,parDX);
}

double UnivParam::GetS0(double r, double DX, double logE, int icomp, double Nmu )
{
  if ( icomp<0 || icomp>3 ) return 0.;
  if ( r > rLimit_high ) r= rLimit_high;
  if ( r < rLimit_low  ) r= rLimit_low;

  double val=GetS0(r,DX,logE,icomp);

  if      ( icomp==0 ) return val*Nmu;
  else if ( icomp==1 ) return val;
  else if ( icomp==2 ) return val*Nmu;
  else if ( icomp==3 ) 
    {
      double alpha=GetAlphaFluct(r,DX);
      return val*(1.+(Nmu-1.)*alpha);
    }
  return -100.;
}


double UnivParam::GetSignal(double r, double Xmax_edep , double logE, double theta, double psi,  double rhoGround, double hGround, double Nmu,  int icomp0, int iatm)
{

  // r             [cm]
  // Xmax_edep     [g/cm²]
  // log(E)        [eV]
  // theta/psi     [rad]
  // rhoGround     [g/cm^3]   rhoGround<0. ->  Average density in simulations 1.04e-3  (g/cm^3)
  // hGround       [cm]       hGround<0.   ->  Ground height in simulations
  // Nmu           relative local Nmu
  // iatm=1,..12   otherwise  average over monthly atmospheres
  // icomp=0,1,2,3 Muon/Em/Em Muon/Em had  -1 sum

  AtmosphereNS::Atmosphere fatm;  

  if ( icomp0<-1 || icomp0>3 ) return 0.;
  if ( r > rLimit_high ) r= rLimit_high;
  if ( r < rLimit_low  ) r= rLimit_low;  
  if ( Xmax_edep > GetXmaxLimit(theta) )           { printf("Error: Xmax outside range %lf \n",Xmax_edep); return 0.; }
  if ( Xmax_edep <    Xmax_low         )           { printf("Error: Xmax outside range %lf \n",Xmax_edep); return 0.; }
  if ( theta > 65*M_PI/180.)                       { printf("Error: Theta outside range 0-60 [deg] \n"); return 0.; }
  if ( fDetectorType==1 && theta > 50*M_PI/180.)   { printf("Error: Theta outside range 0-45 [deg] \n"); return 0.; }
  if ( hGround<0. )   hGround=AtmosphereNS::hGroundRef;
  if ( rhoGround<0. ) rhoGround=fatm.GetDensity(hGround,-1);

  AtmCorrNS::AtmCorr fatmcorr(fDetectorType);
  GeomAsymNS::GeomAsym fgeom(fDetectorType);
  
  double sum=0.;
  for ( int icomp=0;icomp<4;icomp++)
    {
      if ( !( icomp==icomp0 || icomp0==-1) ) continue;

      double DX=fatm.GetDX(  r,  psi,  Xmax_edep, -1., theta,  hGround , iatm, icomp );
      if ( DX <= DX0[icomp] ) continue;
      if ( icomp!=1 && DX<0. ) continue; 

      double S0= GetS0(  r,  DX,   logE,   icomp , Nmu )  ;
      double S=  S0 *  fgeom.GetGeomAsym(  DX,    r,  theta,  psi,  icomp  ) 
	/  fatmcorr.GetCorrTypeI(r,icomp,rhoGround,hGround) /  fatmcorr.GetCorrTypeII(r,theta,icomp,hGround) / fatmcorr.GetCorrTypeIII(r,theta,psi,icomp);
      if ( S<=0. ) S=0.;
      sum+=S;
    }
  return sum;
}


double UnivParam:: GetLogE(double vem, double r, double Xmax_edep , double theta, double psi, double rhoGround, double hGround, double Nmu,  int iatm)
{
  double logE=17.00;
  double vemG=GetSignal(r,Xmax_edep,logE,theta,psi,rhoGround,hGround,Nmu,-1,iatm);
  if (vem<vemG) return logE;
  
  double dlogE=0.05;
  while (fabs(vem/vemG-1.)>0.001)
    {
      if (vem>vemG) logE+=dlogE; 
      else { logE-=dlogE; dlogE/=2.; }      
      vemG=GetSignal(r,Xmax_edep,logE,theta,psi,rhoGround,hGround,Nmu,-1,iatm);
    }
  return logE;
}  

double UnivParam::GetAlphaFluct(double r, double DX)
{
  double r_meter=r/1.e2;

  double alpha0=alpha0_par[0] + alpha0_par[1]*(DX-300.)/1000.; // r=0 m
  double alpha1=alpha1_par[0] + alpha1_par[1]*(DX-300.)/1000.; // r=r0

  double B=(alpha1-alpha0)/(1.-exp(1));
  double A=alpha0+B;
  double r0=1500.;
  double val=A-B*exp(r_meter/r0);
  if (val<0.8) val=0.8;
  return val;
}

double UnivParam::GetNmuL(double r, double DX_mu, double logE, double s0Mu ) 
{
  GeomAsymNS::GeomAsym fgeom(fDetectorType);
  double s0MuRef=GetS0( r, DX_mu, logE, 0, 1.00 );
  return s0Mu/s0MuRef;
}


double UnivParam::GetNmuModels(double DX_mu, double logE,  int im, int ip)
{
  // This is the number of muons for different models at 1000 [m] 
  if ( im<0 || im>1 ) return 0.;
  if ( ip<0 || ip>1 ) return 0.;
 
  //--------- DX_mu=300
  double NmuRef19,NmuRef_gamma;
  NmuRef19=     NmuRef_par[im*2+ip][0];
  NmuRef_gamma= NmuRef_par[im*2+ip][1];
  
  double NmuRef=NmuRef19*pow(10.,(logE-19.)*NmuRef_gamma);
  
  //--------- DX attentuation
  double fNmuAtt19,fNmuAtt_gamma;
  fNmuAtt19=     fNmuAtt_par[im*2+ip][0];
  fNmuAtt_gamma= fNmuAtt_par[im*2+ip][1]; 
  double fNmuAtt=fNmuAtt19*pow(10.,(logE-19.)*fNmuAtt_gamma);

  return NmuRef*(1.-(DX_mu-300.)/1000.*fNmuAtt);
}




