#include <iostream>
#include <signal.h>
#include "Es.h"
#include <IoAuger.h>
#include <fstream>
#include "TCdasUtil.h"
#include "ToolsLib.h"

int main (int argc, char *argv[]) {
  if (argc < 3) {
    cout << "Usage: " << argv[0] << " <output.txt> <files>" << endl;
    cout << "   will produce a verbose output for each event" << endl;
    return 1;
  }

  AugerIoSd input(argc-2, argv+2); //creates the object holding the files

  ofstream lOut(argv[1]);

  for (EventPos pos=input.FirstEvent(); pos<input.LastEvent(); pos=input.NextEvent()) {
    TEsEvent event(pos); //creates the event object

    // getting simul informations
    double X0=0,Y0=0,Z0=1400,T0=0.50;
    double lPhi=-444, lTheta=-444;

    IoSdSimul *sim = event.simul();

    if ( sim ) { // there is simul informations
      nea2xyz(sim->corenorthing(), sim->coreeasting(), Z0, &X0, &Y0, &Z0);
      lPhi    = sim->phi()   * kCdasUtil::RAD2DEG ;
      lTheta  = sim->theta() * kCdasUtil::RAD2DEG ;
    } else {
      cerr << "No simulation block. So X Y informations will be ridiculous." << endl;
    }

    // loop over stations
    for (  vector < TCalibStation * >::const_iterator i = event.fSelectedStations.begin();
           i < event.fSelectedStations.end();
           i++) {
  double r;
  double U,V;
  thetaphi2uv(lTheta * kCdasUtil::DEG2RAD,lPhi * kCdasUtil::DEG2RAD,U,V);
  r = compute_distUV(U,V,
                     (*i)->fX - X0 ,
                     (*i)->fY - Y0,
                     (*i)->fZ - Z0 ) ;
 
//      lOut << event.AugerId << "\t" << flush;
//      lOut << (int)( i - event.fSelectedStations.begin() ) << "\t" << flush;
//      lOut << lPhi << " " << lTheta << "\t" << flush;
//      lOut << (*i)->fX - X0 << " " << (*i)->fY - Y0 << " "
//      << (*i)->fZ - Z0 << " " << flush;
	lOut << r << "\t" << flush;
      lOut << (*i)->SigInVEM() << " " << flush ;
//      lOut << (*i)->StartTime() - T0 << flush;
/*
	lOut << X0 << " " << flush;
	lOut << (*i)->fX << " " << flush;
	lOut << (*i)->fX - X0 << " " << flush;
	lOut << Y0 << " " << flush;
	lOut << (*i)->fY << " " << flush;
	lOut << (*i)->fY - Y0 << " " << flush;
	lOut << Z0 << " " << flush;
	lOut << (*i)->fZ << " " << flush;
	lOut << (*i)->fZ - Z0 << " " << flush;
*/	

      lOut << endl;

    }
  }

  lOut.close()
  ;

  return 0;
}
