#include <iostream>
#include <signal.h>
#include <cstdio>
#include <fstream>
#include <string>
#include <iostream>
#include <sys/stat.h>
#include <sys/types.h>

#include "math.h"
#include "Er.h"
#include "IoAuger.h"
#include "TCdasUtil.h"
#include "SdRecLpnhe.h"

//****************************************************************************
//*
//* Version for public event display 5
//* Fri May  4 14:47:27 ART 2007
//* By M. Gomez Berisso (berisso@cab.cnea.gov.ar)
//* And X. Bertou (bertou@cab.cnea.gov.ar)
//*
//****************************************************************************


int main (int argc, char *argv[]) {
  FILE *fo_ev;
  FILE *fo_st;

  if (argc < 3) {
    cout << "Usage: " << argv[0] << " evtid xxx.root" << endl;
    cout << "    builds evid/evid.dat (ascii with data from event evid)" << endl;
    cout << "           evid/evid-zzzz.dat (ascii with data from station zzzz in event evid)" << endl;
    return -1;
  }

  EventId evid=atoll(argv[1]);

  AugerIoSd input(argc-2, argv+2);  //creates the object holding the files
  EventPos min=input.FirstEvent();
  EventPos max=input.LastEvent();
  EventPos pos=min;
  int ok=0;
  while (!ok) {
    IoSdEvent event(pos);
    if (event.Id!=evid && min>=max) ok=1;
    if (event.Id<evid) {
      min=pos+1;
      pos=(int)((min+max)/2);
    }
    if (event.Id>evid) {
      max=pos-1;
      pos=(int)((min+max)/2);
    }
    if (event.Id!=evid) continue;
    ok=1;
    TErEvent ev(event);

    ev.OfficialSelection();

    theLpnheRec().SetDefault();
    theLpnheRec().SetVerbosity(0);
    theLpnheRec().SetRec(eGeomOptimal, eLDFOptimal);

    int reconstructOk=ev.Reconstruct(&theLpnheRec());
    if (!reconstructOk) return 1;


    char nfo_ev[256];
    snprintf(nfo_ev,256,"%012lld",evid);
    mkdir(nfo_ev,0777);
    snprintf(nfo_ev,256,"%012lld/%012lld.dat",evid,evid);
    printf("%s\n",nfo_ev);
    if((fo_ev=fopen(nfo_ev,"w+"))==NULL) {
      fprintf(stderr,"Can't open file '%s'\n",nfo_ev);
      return -1;
    }

    fprintf(fo_ev,"ID               %30d\n"  ,ev.Id);
    fprintf(fo_ev,"AUGERID          %30lld\n"  ,ev.AugerId);
    fprintf(fo_ev,"UTCTIME          %30ld\n"  ,(long int)ev.UTCTime);
    char buffer[25]; snprintf(buffer,25,"%s",ev.UTCDate.c_str());
    fprintf(fo_ev,"UTCDATE          %30s\n"    ,buffer);
    fprintf(fo_ev,"NOEZS            %30d\n"  ,ev.fSelectedStations.size());
    fprintf(fo_ev,"THETA            %30f\n"  ,ev.fRec.Theta("deg"));
    fprintf(fo_ev,"PHI              %30f\n"  ,ev.fRec.Phi("deg"));
    fprintf(fo_ev,"XCORE            %30d\n"  ,(int)ev.fRec.fXCore);
    fprintf(fo_ev,"YCORE            %30d\n"  ,(int)ev.fRec.fYCore);
    fprintf(fo_ev,"EASTING          %30d\n"  ,(int)ev.fRec.Easting());
    fprintf(fo_ev,"NORTHING         %30d\n"  ,(int)ev.fRec.Northing());
    fprintf(fo_ev,"T0               %30d\n"  ,(int)ev.fRec.fT0);
    fprintf(fo_ev,"U                %30f\n"  ,ev.fRec.fU);
    fprintf(fo_ev,"V                %30f\n"  ,ev.fRec.fV);
    fprintf(fo_ev,"R                %30f\n"  ,ev.fRec.fR);
    fprintf(fo_ev,"E                %30f\n"  ,compute_energy("CIC",ev.fRec.fSRef,ev.fRec.Theta(),1));
    if(ev.UseInfillRec) // infill
    fprintf(fo_ev,"S450            %30f\n"  ,ev.fRec.fSRef);
    else
    fprintf(fo_ev,"S1000            %30f\n"  ,ev.fRec.fSRef);
    fprintf(fo_ev,"BETA             %30f\n"  ,ev.fRec.fBeta);
    fprintf(fo_ev,"GAMMA            %30f\n"  ,ev.fRec.fGamma);
    fprintf(fo_ev,"L                %30f\n"  ,ev.fRec.L());
    fprintf(fo_ev,"B                %30f\n"  ,ev.fRec.B());
    fprintf(fo_ev,"DTHETA           %30f\n"  ,ev.fRec.dTheta("deg"));
    fprintf(fo_ev,"DPHI             %30f\n"  ,ev.fRec.dPhi("deg"));
    fprintf(fo_ev,"DXCORE           %30f\n"  ,ev.fRec.dXCore());
    fprintf(fo_ev,"DYCORE           %30f\n"  ,ev.fRec.dYCore());
    fprintf(fo_ev,"DR               %30f\n"  ,ev.fRec.dR());
    if(ev.UseInfillRec) // infill
    fprintf(fo_ev,"DS450            %30f\n"  ,ev.fRec.dSRef());
    else
    fprintf(fo_ev,"DS1000            %30f\n" ,ev.fRec.dSRef());
    fprintf(fo_ev,"GLOBALCHI2       %30f\n"  ,theLpnheRec().globalChi2);
    fprintf(fo_ev,"NDOFCHI2         %30d\n"  ,theLpnheRec().ndof);
    fprintf(fo_ev,"T3TD_ID          %30d\n"  ,ev.Trigger.Id);
    fprintf(fo_ev,"T3TD_PREVIOUSID  %30d\n"  ,ev.Trigger.PreviousId);
    fprintf(fo_ev,"T3TD_NOSTATIONS  %30d\n"  ,ev.Trigger.NumberOfStation);
    fprintf(fo_ev,"T3TD_SECOND      %30d\n"  ,ev.Trigger.Second);
    fprintf(fo_ev,"T3TD_MICROSECOND %30d\n"  ,ev.Trigger.MicroSecond);
    fprintf(fo_ev,"T3TD_SENDER      %30s\n"   ,ev.Trigger.Sender.c_str());
    fprintf(fo_ev,"T3TD_ALGO        %30s\n"   ,ev.Trigger.Algo.c_str());
    fprintf(fo_ev,"T3TD_SDPANGLE    %30f\n"  ,ev.Trigger.SDPAngle);
    fprintf(fo_ev,"IST5             %30d\n"  ,(ev.IsT5()>0));
    fprintf(fo_ev,"HOTESTTANK       %30d\n"  ,ev.IsT5());
    fprintf(fo_ev,"END_DATA\n");
    fclose(fo_ev);
    
    char nfo_st[256];
    snprintf(nfo_st,256,"%012lld/%012lld-nodata.dat",evid,evid);
    if((fo_st=fopen(nfo_st,"w"))==NULL) {
      fprintf(stderr,"Can't open file '%s'\n",nfo_st);
      return -1;
    }
    fclose(fo_st);
    for (unsigned int i=0; i<ev.Stations.size(); i++) {
      if (ev.Stations[i].Id<100) continue;
      if (ev.Stations[i].Error) {
        snprintf(nfo_st,256,"%012lld/%012lld-nodata.dat",evid,evid);
        if((fo_st=fopen(nfo_st,"a"))==NULL) {
          fprintf(stderr,"Can't open file '%s'\n",nfo_st);
          return -1;
        }
	fprintf(fo_st,"ID               %30d\n", ev.Stations[i].Id);
	fprintf(fo_st,"NAME             %30s\n", ev.Stations[i].Name.c_str());
	fprintf(fo_st,"EASTING          %30f\n", ev.Stations[i].Easting);
	fprintf(fo_st,"NORTHING         %30f\n", ev.Stations[i].Northing);
	fprintf(fo_st,"ALTITUDE         %30f\n", ev.Stations[i].Altitude);
	fprintf(fo_st,"END_DATA\n");
        fclose(fo_st);
        continue;
      }
      snprintf(nfo_st,256,"%012lld/%012lld-%04d.dat",evid,evid,ev.Stations[i].Id);
      printf("    %s\n",nfo_st);
      if((fo_st=fopen(nfo_st,"w+"))==NULL) {
        fprintf(stderr,"Can't open file '%s'\n",nfo_st);
        return -1;
      }
      fprintf(fo_st,"ID               %30d\n", ev.Stations[i].Id);
      fprintf(fo_st,"NAME             %30s\n", ev.Stations[i].Name.c_str());
      fprintf(fo_st,"ERROR            %30d\n", ev.Stations[i].Error);
      fprintf(fo_st,"EASTING          %30f\n", ev.Stations[i].Easting);
      fprintf(fo_st,"NORTHING         %30f\n", ev.Stations[i].Northing);
      fprintf(fo_st,"ALTITUDE         %30f\n", ev.Stations[i].Altitude);
      for(unsigned int k=0; k<ev.fRandomStations.size(); k++) {
        if(ev.fRandomStations[k]->Id == ev.Stations[i].Id) {
          fprintf(fo_st,"ES_FLAG          %30s\n", "RANDOM");
        }
      }
      for(unsigned int k=0; k<ev.fSelectedStations.size(); k++) {
        if(ev.fSelectedStations[k]->Id == ev.Stations[i].Id) {
          //DESDE ErRec
          fprintf(fo_st,"ER_SIGINVEM      %30f\n", ev.fSelectedStations[k]->fSigInVEM);
          //fprintf(fo_st,"ER_EXPECTED_VEM  %30f\n", ev.fRec.fS1000*ev.fRec._LDFfunc(ev.fSelectedStations[k]->fDist,theLpnheRec().fBeta,theLpnheRec().fGamma));
          fprintf(fo_st,"ER_PEAKINVEM     %30f %10f %10f %10f\n", ev.fSelectedStations[k]->fPmt[0].fPeakInVEM, ev.fSelectedStations[k]->fPmt[1].fPeakInVEM, ev.fSelectedStations[k]->fPmt[2].fPeakInVEM, ev.fSelectedStations[k]->fPeakInVEM);
          fprintf(fo_st,"ER_STARTTIME     %30.0f\n", 1.e+9*ev.fSelectedStations[k]->StartTime());
          fprintf(fo_st,"ER_STARTBIN      %30d\n", ev.fSelectedStations[k]->fStartBin);
          fprintf(fo_ev,"ER_T0            %30d\n"  ,(int)ev.fRec.fT0);
          fprintf(fo_ev,"ER_DT            %30f\n"  ,ev.Dt(*(ev.fSelectedStations[k])));
          fprintf(fo_ev,"ER_DAXIS         %30f\n"  ,ev.Dist(*(ev.fSelectedStations[k])));
          fprintf(fo_ev,"ER_COS_ZETA      %30f\n"  ,ev.CosZeta(*(ev.fSelectedStations[k])));
          for(int l=0; l<768; l++) {
            double tiempo;
            tiempo = ev.fSelectedStations[k]->Gps->NanoSecond-(768-l)*25;
            fprintf(fo_st,"ER_TRACE         %30.0f %10f %10f %10f\n",
                    tiempo,
                    ev.fSelectedStations[k]->Calib->TubeOk[0]*ev.fSelectedStations[k]->fPmt[0].fTrace[l],
                    ev.fSelectedStations[k]->Calib->TubeOk[1]*ev.fSelectedStations[k]->fPmt[1].fTrace[l],
                    ev.fSelectedStations[k]->Calib->TubeOk[2]*ev.fSelectedStations[k]->fPmt[2].fTrace[l]);
          }
          //T2TRIGGER
#if 0
          fprintf(fo_st,"T2T_TYPE         %30d\n",ev.fSelectedStations[k]->Trigger.Type);
          fprintf(fo_st,"T2T_WINDOW       %30d\n",ev.fSelectedStations[k]->Trigger.Window);
          fprintf(fo_st,"T2T_NAME         %30s\n",ev.fSelectedStations[k]->Trigger.Name.c_str());
          //GPS
          fprintf(fo_st,"GPS_SECOND       %30d\n",ev.fSelectedStations[k]->Gps->Second);
          fprintf(fo_st,"GPS_TICK         %30d\n",ev.fSelectedStations[k]->Gps->Tick);
          fprintf(fo_st,"GPS_CURRENT100   %30d\n",ev.fSelectedStations[k]->Gps->Current100);
          fprintf(fo_st,"GPS_NEXT100      %30d\n",ev.fSelectedStations[k]->Gps->Next100);
          fprintf(fo_st,"GPS_NEXT40       %30d\n",ev.fSelectedStations[k]->Gps->Next40);
          fprintf(fo_st,"GPS_PREVIOUSST   %30d\n",ev.fSelectedStations[k]->Gps->PreviousST);
          fprintf(fo_st,"GPS_CURRENTST    %30d\n",ev.fSelectedStations[k]->Gps->CurrentST);
          fprintf(fo_st,"GPS_NEXTST       %30d\n",ev.fSelectedStations[k]->Gps->NextST);
          fprintf(fo_st,"GPS_OFFSET       %30d\n",ev.fSelectedStations[k]->Gps->Offset);
          fprintf(fo_st,"GPS_NANOSECOND   %30d\n",ev.fSelectedStations[k]->Gps->NanoSecond);
          //HISTO
          for(int m=0; m<10; m++)
            fprintf(fo_st,"HISTO_OFFSET     %30d %10d\n",m,ev.fSelectedStations[k]->Histo->Offset[m]);
          for(int m=0; m<20; m++)
            fprintf(fo_st,"HISTO_BASE       %30d %10d %10d %10d\n",m,ev.fSelectedStations[k]->Histo->Base[0][m],ev.fSelectedStations[k]->Histo->Base[1][m],ev.fSelectedStations[k]->Histo->Base[2][m]);
          for(int m=0; m<150; m++)
            fprintf(fo_st,"HISTO_PEAK       %30d %10d %10d %10d\n",m,ev.fSelectedStations[k]->Histo->Peak[0][m],ev.fSelectedStations[k]->Histo->Peak[1][m],ev.fSelectedStations[k]->Histo->Peak[2][m]);
          for(int m=0; m<600; m++)
            fprintf(fo_st,"HISTO_CHARGE     %30d %10d %10d %10d %10d\n",m,ev.fSelectedStations[k]->Histo->Charge[0][m],ev.fSelectedStations[k]->Histo->Charge[1][m],ev.fSelectedStations[k]->Histo->Charge[2][m],ev.fSelectedStations[k]->Histo->Charge[3][m]);
          for(int m=0; m<20; m++)
            fprintf(fo_st,"HISTO_SHAPE      %30d %10d %10d %10d\n",m,ev.fSelectedStations[k]->Histo->Shape[0][m],ev.fSelectedStations[k]->Histo->Shape[1][m],ev.fSelectedStations[k]->Histo->Shape[2][m]);
          //CALIB
          fprintf(fo_st,"CA_VERSION       %30d\n",ev.fSelectedStations[k]->Calib->Version);
          fprintf(fo_st,"CA_TUBEMASK      %30d\n",ev.fSelectedStations[k]->Calib->TubeMask);
          fprintf(fo_st,"CA_STARTSECOND   %30d\n",ev.fSelectedStations[k]->Calib->StartSecond);
          fprintf(fo_st,"CA_ENDSECOND     %30d\n",ev.fSelectedStations[k]->Calib->EndSecond);
          fprintf(fo_st,"CA_NBT1          %30d\n",ev.fSelectedStations[k]->Calib->NbT1);
          fprintf(fo_st,"CA_NBT2          %30d\n",ev.fSelectedStations[k]->Calib->NbT2);
          fprintf(fo_st,"CA_NBTOT         %30d\n",ev.fSelectedStations[k]->Calib->NbTOT);
          fprintf(fo_st,"CA_NBTDA         %30d %10d %10d\n",ev.fSelectedStations[k]->Calib->NbTDA[0],ev.fSelectedStations[k]->Calib->NbTDA[1],ev.fSelectedStations[k]->Calib->NbTDA[2]);
          fprintf(fo_st,"CA_EVOLUTION     %30d %10d %10d\n",ev.fSelectedStations[k]->Calib->Evolution[0],ev.fSelectedStations[k]->Calib->Evolution[1],ev.fSelectedStations[k]->Calib->Evolution[2]);
          fprintf(fo_st,"CA_RATE          %30f %10f %10f\n",ev.fSelectedStations[k]->Calib->Rate[0],ev.fSelectedStations[k]->Calib->Rate[1],ev.fSelectedStations[k]->Calib->Rate[2]);
          fprintf(fo_st,"CA_VEMPEAK       %30f %10f %10f\n",ev.fSelectedStations[k]->Calib->VemPeak[0],ev.fSelectedStations[k]->Calib->VemPeak[1],ev.fSelectedStations[k]->Calib->VemPeak[2]);
          fprintf(fo_st,"CA_VEMCHARGE     %30f %10f %10f\n",ev.fSelectedStations[k]->Calib->VemCharge[0],ev.fSelectedStations[k]->Calib->VemCharge[1],ev.fSelectedStations[k]->Calib->VemCharge[2]);
          fprintf(fo_st,"CA_BASE          %30f %10f %10f %10f %10f %10f\n",ev.fSelectedStations[k]->Calib->Base[0],ev.fSelectedStations[k]->Calib->Base[1],ev.fSelectedStations[k]->Calib->Base[2],ev.fSelectedStations[k]->Calib->Base[3],ev.fSelectedStations[k]->Calib->Base[4],ev.fSelectedStations[k]->Calib->Base[5]);
          fprintf(fo_st,"CA_SIGMABASE     %30f %10f %10f %10f %10f %10f\n",ev.fSelectedStations[k]->Calib->SigmaBase[0],ev.fSelectedStations[k]->Calib->SigmaBase[1],ev.fSelectedStations[k]->Calib->SigmaBase[2],ev.fSelectedStations[k]->Calib->SigmaBase[3],ev.fSelectedStations[k]->Calib->SigmaBase[4],ev.fSelectedStations[k]->Calib->SigmaBase[5]);
          fprintf(fo_st,"CA_DA            %30f %10f %10f\n",ev.fSelectedStations[k]->Calib->DA[0],ev.fSelectedStations[k]->Calib->DA[1],ev.fSelectedStations[k]->Calib->DA[2]);
          fprintf(fo_st,"CA_SIGMADA       %30f %10f %10f\n",ev.fSelectedStations[k]->Calib->SigmaDA[0],ev.fSelectedStations[k]->Calib->SigmaDA[1],ev.fSelectedStations[k]->Calib->SigmaDA[2]);
          fprintf(fo_st,"CA_STABILITY     %30d %10d %10d\n",ev.fSelectedStations[k]->Calib->Stability[0],ev.fSelectedStations[k]->Calib->Stability[1],ev.fSelectedStations[k]->Calib->Stability[2]);
          fprintf(fo_st,"CA_TUBEOK        %30d %10d %10d\n",ev.fSelectedStations[k]->Calib->TubeOk[0],ev.fSelectedStations[k]->Calib->TubeOk[1],ev.fSelectedStations[k]->Calib->TubeOk[2]);
          fprintf(fo_st,"CA_NTUBESOK      %30d\n",ev.fSelectedStations[k]->Calib->NTubesOk);
          fprintf(fo_st,"CA_T1RATE        %30f\n",ev.fSelectedStations[k]->Calib->T1Rate);
          fprintf(fo_st,"CA_T2RATE        %30f\n",ev.fSelectedStations[k]->Calib->T2Rate);
          fprintf(fo_st,"CA_TOTRATE       %30f\n",ev.fSelectedStations[k]->Calib->TOTRate);
          fprintf(fo_st,"CA_TDARATE       %30f %10f %10f\n",ev.fSelectedStations[k]->Calib->TDARate[0],ev.fSelectedStations[k]->Calib->TDARate[1],ev.fSelectedStations[k]->Calib->TDARate[2]);
          fprintf(fo_st,"CA_SHOWEVOLUTION %30s %10s %10s\n",ev.fSelectedStations[k]->Calib->ShowEvolution[0].c_str(),ev.fSelectedStations[k]->Calib->ShowEvolution[1].c_str(),ev.fSelectedStations[k]->Calib->ShowEvolution[2].c_str());
          //FADC
          fprintf(fo_st,"FADC_NSAMPLE     %30d\n",ev.fSelectedStations[k]->Fadc->NSample);
          for(int m=0; m<768; m++)
            fprintf(fo_st,"FADC_TRACE       %30d %10d %10d %10d %10d %10d %10d\n",
                    m,
                    ev.fSelectedStations[k]->Fadc->Trace[0][IoSdEvent::eHigh][m],
                    ev.fSelectedStations[k]->Fadc->Trace[1][IoSdEvent::eHigh][m],
                    ev.fSelectedStations[k]->Fadc->Trace[2][IoSdEvent::eHigh][m],
                    ev.fSelectedStations[k]->Fadc->Trace[0][IoSdEvent::eLow][m],
                    ev.fSelectedStations[k]->Fadc->Trace[1][IoSdEvent::eLow][m],
                    ev.fSelectedStations[k]->Fadc->Trace[2][IoSdEvent::eLow][m]);
#endif
        }
      }
      fprintf(fo_st,"END_DATA\n");
      fclose(fo_st);
    }//for i
  }//for pos
  return 0;
}
