#include <iostream>
#include <IoAuger.h>


int main (int argc, char *argv[]) {
  if (argc==1) {
    std::cout << std::endl;
    std::cout << "Usage: " << argv[0] << " <files>" << std::endl << std::endl;
    std::cout << "  Reads <files> \"a la IoSd\" independently of <files> format (IoSdEvent files or AugerEvent files)."<< std::endl;
    std::cout << "  and illustrates some writing mechanisms. "  << std::endl;
    std::cout << "  On output it produces  one or two files containing only multiplicity > 10 Sd events." << std::endl;
    std::cout << "  If <files> are IoSdEvent files (e.g. sd_... or sdm_...) output is in the same" << std::endl;
    std::cout << "  format and is called sd_10.root." << std::endl;
    std::cout << "  If <files> are AugerEvent files (e.g. ad_... or hg_... or std4_...)";
    std::cout << " 3 output files are produced: "<< std::endl;
    std::cout << "  sd_10.root in IoSdEvent format, ad_10.root in AugerEvent format, adm_10.root in Auger format with a modified Sd part. " << std::endl;
    std::cout << "  NB: In the later case the AugerEventId is retained in the sd_10.root file." << std::endl;
    return 1;
  }

  AugerIoSd input(argc-1, argv+1);

  // pointer to eventual output files
  IoSd *sd10 = NULL;
  AugerFile *ad10 = NULL, *adm10 = NULL;
  EventPos pos;
  for (pos=input.FirstEvent(); pos<input.LastEvent(); pos=input.NextEvent()) {
    IoSdEvent event(pos);
    if(event.NumberOfErrorZeroStation > 10) {
      if (!sd10)
        sd10 = new IoSd("sd_10.root", "w");
      sd10->Write(event, "VERBOSE");
      if (event.RootClassName == "AugerEvent") {
        if (!ad10)  {
          ad10 = new AugerFile("ad_10.root", AugerFile::eWrite);
          adm10 = new AugerFile("adm_10.root", AugerFile::eWrite);
        }
        ad10->Write(*(event.RawAugerEvent()), "VERBOSE");
        // now modify the IoSdEvent and writeout the modified event as an AugerEvent
        vector<IoSdStation> st;
        st.reserve(10);
        for (unsigned int i = 0; i < event.Stations.size(); i++)
          if (event.Stations[i].Error == 0)
            st.push_back(event.Stations[i]);
        event.Stations = st;

        (event.RawAugerEvent())->PushEvent(event);
        adm10->Write(*(event.RawAugerEvent()));
      }
    }
  }

  if (sd10)
    sd10->Close();
  if (ad10) {
    ad10->Close();
    adm10->Close();
  }
  return 0;
}
