#include <iostream>
#include <IoAuger.h>
#include <Ec.h>

using namespace std;

const int nrOfStations = 61;
const double easierStations[nrOfStations]= {342, 333, 332, 344, 341, 419, 
                                            343, 306, 308, 315, 328, 334, 
                                            403, 408, 422, 425, 431, 434, 
                                            354, 125, 199, 300, 307, 310, 
                                            311, 313, 330, 339, 356, 375,
                                            383, 385, 394, 405, 410, 432, 
                                            433, 121, 196, 197, 201, 296,
                                            305, 312, 319, 325, 329, 337, 
                                            340, 376, 384, 386, 387, 390, 
                                            396, 412, 427, 429, 438, 441, 
                                            1802};

const int pmtNo = 0;
int main (int argc, char *argv[]) {
 
  if ( argc < 3 ) {
    cout << endl
         << "Usage: " << argv[0] << " <output>  <files>" << endl
         << "  <output>: IoSd file as output" << endl
         << "   <files>: IoSd or IoAuger files to be read" << endl;
    return 1;
  }
  
  AugerIoSd input(argc-2, argv+2);
  const char* outputName = argv[1];
  // pointer to eventual output files
  IoSd *output = new IoSd(outputName, "w");
 
  
  for( EventPos pos=input.FirstEvent(); 
       pos<input.LastEvent(); 
       pos=input.NextEvent()) {
    //  IoSdEvent event(pos);
    
    TEcEvent event(pos);
    bool found = false;
    for (unsigned int i = 0; i < event.fCalibStations.size(); i++){
      
      if (event.fCalibStations[i].fSigInVEM < 0.001)
	continue;
      
      for (unsigned int stIter = 0; stIter <nrOfStations; ++stIter){

        const TCalibStation& calibStation = event.fCalibStations[i];
        
        if (calibStation.Id == easierStations[stIter]){
          
          if (calibStation.fPmt[pmtNo].fDA > 1)
	    continue;
          if (!calibStation.fadc())
            continue;
          cout << "Found it: " << easierStations[stIter] << " " 
	       << calibStation.fPmt[0].fDA  << " " 
	       << calibStation.fPmt[1].fDA  << " " 
	       << calibStation.fPmt[2].fDA  << " " 
               << endl;
          double meanValue = 0;
          for (int k = 0; k<kIoSd::MAXSAMPLE; ++k){
            meanValue += calibStation.fadc()->Trace[pmtNo][0][k];
          }
          meanValue /= kIoSd::MAXSAMPLE;
          cout << "=====> " << meanValue << " <=====" << endl;
          if (meanValue > 100)
            found = true; 
	  break;
        }
      }
    }
    if (found)
      output->Write(event, "VERBOSE");
  }
  
  if (output)
    output->Close();
  
  return 0;
}
