#include "math.h"
#include "Er.h"
#include "IoAuger.h"
#include "TCdasUtil.h"
#include "SdRecLpnhe.h"

using namespace std;

int main (int argc, char *argv[]) {
  
  if (argc<2) {
    cerr << endl <<endl << "-------------------------------------------------------------" <<endl;
    cerr << "Usage: "<< endl;
    cerr << "./herald -i [UseInfill] -r [UseInfillRec]  filenames" << endl;
    cerr << "UseInfill (default 0): to chose whether or not infill stations will be used in the event" << endl;
    cerr << "UseInfillRec (default 0): to chose whether or not infill reconstruction will be performed: "<< endl;
    cerr << "                          crowns with d = 750 m, SRef = 450 m, new LDF/CIC/Calib etc." << endl;
    cerr  << "-------------------------------------------------------------" <<endl << endl;
    return 0;}
  
  bool UseInfillStations = false;
  bool UseInfillRec = false;
  unsigned int loop = 1;

  for (int i = 1; i < argc; i++) {
    if (strstr(argv[i], "-i") == argv[i]) {UseInfillStations = atoi(argv[i+1]); i++; loop+=2;}
    if (strstr(argv[i], "-r") == argv[i]) {UseInfillRec = atoi(argv[i+1]); i++; loop+=2;}
  }
  if (UseInfillRec==1) UseInfillStations=1; // there must be infill stations for the rec.
  
  cerr << "Use Infill: " << UseInfillStations << "  Use Infill Rec: " << UseInfillRec << endl;  
  double coslim=cos(15*kCdasUtil::DEG2RAD);
  
  AugerIoSd input (argc-loop, argv+loop);
  int nb=input.LastEvent();
  bool firstIteration = true;

  for(EventPos pos=input.FirstEvent(); pos<input.LastEvent(); pos=input.NextEvent()) {
    if (!(pos%100))
      cerr << pos << "/" << nb << endl;
    TErEvent event(pos);
    
    event.UseQualityCuts=true;
    event.UseCheckT2=true;
    event.UseCheckT2CommsCrisis=true;
 
    if (event.Trigger.Second>=928368000 && event.Trigger.Second<942364800) event.IsCommsCrisis=true;
    if (firstIteration && UseInfillStations){
      kEs::UseInfillStations(UseInfillStations);
      firstIteration = false;
    } 
    event.UseInfillRec=UseInfillRec;

    if (!event.IsT4()) continue;
    event.OfficialSelection();
    theLpnheRec().SetDefault();
    theLpnheRec().SetVerbosity(0);
    
    theLpnheRec().SetRec(eGeomOptimal, eLDFOptimal);
    int ret=event.Reconstruct(&theLpnheRec());

    // Compute rec/estimate difference
    double dcos=event.fEstimated.fU*event.fRec.fU+event.fEstimated.fV*event.fRec.fV+sqrt(1-event.fEstimated.fU*event.fEstimated.fU-event.fEstimated.fV*event.fEstimated.fV)*sqrt(1-event.fRec.fU*event.fRec.fU-event.fRec.fV*event.fRec.fV);
    double dgrnd=sqrt(pow(event.fEstimated.fXCore-event.fRec.fXCore,2)+pow(event.fEstimated.fYCore-event.fRec.fYCore,2));
    int ok=(int)!((dcos<coslim) || (dgrnd>800) || (dgrnd>600 && event.fSelectedStations.size()==4) || (dgrnd>400 && event.fSelectedStations.size()==3));
    // Print result
    if (!ret || (!(event.fRec.Theta()>=0 && event.fRec.Theta()<=M_PI)) || (!(event.fRec.fE>0)) || (!(event.fRec.fSRef>=0)))
      // bad event
      cout <<event.AugerId<<" -1 "<< event.Stations.size() << " " << event.fSelectedStations.size() << " " << event.fEstimated.Theta("deg") << " " << acos(dcos)*kCdasUtil::RAD2DEG << " " << dgrnd << endl;
    else {
      cout<<event.AugerId<< " " << event.fSelectedStations.size() << " " << event.fRec.LocalTheta("deg") << " " << event.fRec.LocalPhi("deg") << " " << event.fRec.fE  << " ";
      cout << event.fRec.L() << " " << event.fRec.B() << " " << event.UTCTime << " " << int(event.fRec.fT0) << " " << event.fRec.fXCore << " " << event.fRec.fYCore << " ";
      cout << event.fRec.fSRef << " " << event.fRec.dSRef() << " " << event.fRec.Ra() << " " << event.fRec.Dec() << " " << event.fRec.Theta("deg") << " ";
      cout << event.fRec.Phi("deg") << " " << event.fRec.dTheta("deg") << " " << event.fRec.dPhi("deg") << " " << event.fRec.dXCore() << " " << event.fRec.dYCore() << " ";
      cout << ok;
      cout << " " << event.IsT5() << " " << event.IsT5(2) << " " << event.IsT5(3) << " ";
      cout << event.IsICRC2005()  << " " << event.trigger().IsFd() << " " << theLpnheRec().angularChi2 << " ";
      cout << theLpnheRec().ldfChi2 << " " << theLpnheRec().globalChi2 << " " << theLpnheRec().ndof << " " << theLpnheRec().fBeta << " " << theLpnheRec().fGamma << " ";
      cout << theLpnheRec().fR << " " << event.Id << " ";
      cout << event.IsICRC2005Posterior(event.fRec.fXCore, event.fRec.fYCore) << " ";
      //New
      if (UseInfillRec){
	cout << compute_energy("CICINFILL",event.fRec.fSRef,event.fRec.Theta(),1) << " " << compute_energy("CICINFILL",event.fRec.fSRef,event.fRec.Theta(),1) << " " << compute_energy("CICINFILL",event.fRec.fSRef,event.fRec.Theta(),1);
      }
      else{
	cout << compute_energy("CIC",event.fRec.fSRef,event.fRec.Theta(),1) << " " << compute_energy("MCPLUSMU",event.fRec.fSRef,event.fRec.Theta(),1) << " " << compute_energy("NEWFD",event.fRec.fSRef,event.fRec.Theta(),1);
      }
      //
      cout << " " << event.fRec.fGPSSecond << " " << UseInfillStations+UseInfillRec;
      cout << " " << event.fRec.fFitBeta << " " << event.GetNeighbours() << " " <<event.IsPos(event.fRec.fXCore, event.fRec.fYCore);    
      cout<<endl;
    }
  }
}
