#include "math.h"
#include "Er.h"
#include "IoAuger.h"
#include "TCdasUtil.h"
#include "SdRecLpnhe.h"

using namespace std;

//herald_td.exe ../../../../NEWDATA/*.root

int main (int argc, char *argv[]) {
  double coslim=cos(15*kCdasUtil::DEG2RAD);
  AugerIoSd input (argc-1, argv+1);
  int nb=input.LastEvent();
  ofstream out;
  out.open("herald.out",ios::out);
  for(EventPos pos=input.FirstEvent(); pos<input.LastEvent(); pos=input.NextEvent()) {
    if (!(pos%100))
      cerr << pos << "/" << nb << endl;
   
    TErEvent event(pos);
    event.UseQualityCuts=true;
    event.UseCheckT2=true;
    event.UseCheckT2CommsCrisis=true;
    if (event.Trigger.Second>=928368000 && event.Trigger.Second<942364800) event.IsCommsCrisis=true;

    if (!event.IsT4()) continue;
    event.TopDownSelection();
    theLpnheRec().SetDefault();
    theLpnheRec().SetVerbosity(0);
    
    theLpnheRec().SetRec(eGeomOptimal, eLDFOptimal);
    int ret=event.Reconstruct(&theLpnheRec());
    
    // Compute rec/estimate difference
    double dcos=event.fEstimated.fU*event.fRec.fU+event.fEstimated.fV*event.fRec.fV+sqrt(1-event.fEstimated.fU*event.fEstimated.fU-event.fEstimated.fV*event.fEstimated.fV)*sqrt(1-event.fRec.fU*event.fRec.fU-event.fRec.fV*event.fRec.fV);
    double dgrnd=sqrt(pow(event.fEstimated.fXCore-event.fRec.fXCore,2)+pow(event.fEstimated.fYCore-event.fRec.fYCore,2));
    int ok=(int)!((dcos<coslim) || (dgrnd>800) || (dgrnd>600 && event.fSelectedStations.size()==4) || (dgrnd>400 && event.fSelectedStations.size()==3));

    // Print result
    if (!ret || (!(event.fRec.Theta()>=0 && event.fRec.Theta()<=M_PI)) || (!(event.fRec.fE>0)) || (!(event.fRec.fSRef>=0)))
    
      // bad event
      out <<event.AugerId<<" -1 "<< event.Stations.size() << " " << event.fSelectedStations.size() << " " << event.fEstimated.Theta("deg") << " " << acos(dcos)*kCdasUtil::RAD2DEG << " " << dgrnd << endl;
    else {
      out<<event.AugerId<< " " << event.fSelectedStations.size() << " " << event.fRec.LocalTheta("deg") << " " << event.fRec.LocalPhi("deg") << " " << event.fRec.fE  << " ";
      out << event.fRec.L() << " " << event.fRec.B() << " " << event.UTCTime << " " << int(event.fRec.fT0) << " " << event.fRec.fXCore << " " << event.fRec.fYCore << " ";
      out << event.fRec.fSRef << " " << event.fRec.dSRef() << " " << event.fRec.Ra() << " " << event.fRec.Dec() << " " << event.fRec.Theta("deg") << " ";
      out << event.fRec.Phi("deg") << " " << event.fRec.dTheta("deg") << " " << event.fRec.dPhi("deg") << " " << event.fRec.dXCore() << " " << event.fRec.dYCore() << " ";
      out << ok;
      out << " " << event.IsT5() << " " << event.IsT5(2) << " " << event.IsT5(3) << " ";
      out << event.IsICRC2005()  << " " << event.trigger().IsFd() << " " << theLpnheRec().angularChi2 << " ";
      out << theLpnheRec().ldfChi2 << " " << theLpnheRec().globalChi2 << " " << theLpnheRec().ndof << " " << theLpnheRec().fBeta << " " << theLpnheRec().fGamma << " ";
      out << theLpnheRec().fR << " " << event.Id << " ";
      out << event.IsICRC2005Posterior(event.fRec.fXCore, event.fRec.fYCore) << " ";
      out << compute_energy("CIC",event.fRec.fSRef,event.fRec.Theta(),1) << " " << compute_energy("MCPLUSMU",event.fRec.fSRef,event.fRec.Theta(),1) << " " << compute_energy("NEWFD",event.fRec.fSRef,event.fRec.Theta(),1);
      out << " " << event.fRec.fGPSSecond;      
      out << endl;
		       											       
    }
  }
}
