
/**
  @file    ldf_out.cc
  @author  Tristan Beau, beau@in2p3.fr
  @brief   This main is a small tool getting out ldf functions. Usefull 
           for comparisons.
*/

#include <iostream>

#include "ToolsLib.h"

int main(int argc, char **argv) {

  int larg=1;

  if ( argc < 7 ) {
    cerr << "Not enough args. Needs ldftype, theta, E or S1000, rmin, rmax, deltar" <<endl;
    exit(1);
  }

#define NEXTARG ( argv[larg++] )
  string ldf   = NEXTARG;
  double theta = atof(NEXTARG);
  double nrj   = atof(NEXTARG);
  double rmin  = atof(NEXTARG);
  double rmax  = atof(NEXTARG);
  double dr    = atof(NEXTARG);

  TSMSPPoint lP;

  TSMSPEstim Estim(SMSPPATH + (string)(SMSPDATA));

  cerr << "Emin = " << Estim.MinMap[SMSP_TYPE_ENERGY]
  << " Emax = " << Estim.MaxMap[SMSP_TYPE_ENERGY] << endl;
  cerr << "Theta min = " << Estim.MinMap[SMSP_TYPE_THETA]
  << " Theta max = " << Estim.MaxMap[SMSP_TYPE_THETA] << endl;
  cerr << "Dist min = " << Estim.MinMap[SMSP_TYPE_DIST]
  << " Dist max = " << Estim.MaxMap[SMSP_TYPE_DIST] << endl;

  double gamma,beta;
  GetBetaGammaFromTheta(0,theta*3.14/180,beta,gamma);

  double S1000 = nrj ; // CAUTION !

  for ( double r=rmin; r<=rmax; r+=dr ) {

    cout << r << "\t" << flush;
    if( ldf == "HP" )
      cout << S1000 * compute_LDFHPFunc(r,theta*3.14/180);
    else if( ldf == "LOG" )
      cout << S1000 * compute_LDFLOGFunc(r,beta,gamma);
    else if ( ldf == "SMSP" ) {
      lP[SMSP_TYPE_DIST] = r;
      lP[SMSP_TYPE_THETA] = theta*3.14/180 ;
      lP[SMSP_TYPE_ENERGY] = nrj;
      cout << Estim.VEM(lP);
    } else if( ldf == "2EXPO" )
      cout << compute_LDF2EXPOFunc(r,theta*3.14/180,nrj,0);
    else {
      cerr << "LDF = " << ldf << " unknown ! " << endl << "Exiting !" << endl;
      exit(1);
    }

    cout << endl;
  }


  return 0;
}
