#include "math.h"
#include "Er.h"
#include "Es.h"
#include "IoAuger.h"
#include "TCdasUtil.h"
#include "SdRecLpnhe.h"

using namespace std;

class ActiveStation{
public:
  double Signal; double X; double Y;
  ActiveStation(double s,double x, double y);
};

ActiveStation::ActiveStation(double s,double x, double y){
  Signal=s;X=x;Y=y;
}

void read_bad_evts(vector <unsigned long long> &IdRemove,char *filename){
  ifstream in1;
  unsigned long long ID;
  in1.open(filename,ios::in);
  while(!in1.eof()){
    in1 >> ID;
    IdRemove.push_back(ID);
  }
  in1.close();
}

bool BadEvent(unsigned long long ID,vector <unsigned long long> &IdRemove){
  for (unsigned int i=0;i<IdRemove.size();++i)
    if (ID==IdRemove[i]) return true;
  return false;
}

int main (int argc, char *argv[]) {

  if (argc<2) {
    cerr << endl <<endl << "-------------------------------------------------------------" <<endl;
    cerr << "Usage: "<< endl;
    cerr << argv[0] << " -i [UseInfill] -r [UseInfillRec]  filenames" << endl;
    cerr << "UseInfill (default 0): to chose whether or not infill stations will be used in the event" << endl;
    cerr << "UseInfillRec (default 0): to chose whether or not infill reconstruction will be performed: "<< endl;
    cerr << "                          crowns with d = 750 m, SRef = 450 m, new LDF/CIC/Calib etc." << endl;
    cerr  << "-------------------------------------------------------------" <<endl << endl;
    return 0;}
  
  vector <unsigned long long> BadIds;
  read_bad_evts(BadIds,"/Users/julienaublin/svn/CDAS/trunk/Work/head/src/EventsToRemove"); 

  bool UseInfillStations = false;
  bool UseInfillRec = false;
  unsigned int loop = 1;
  unsigned int ntot=0,nreject=0;
  for (int i = 1; i < argc; i++) {
    if (strstr(argv[i], "-i") == argv[i]) {UseInfillStations = atoi(argv[i+1]); i++; loop+=2;}
    if (strstr(argv[i], "-r") == argv[i]) {UseInfillRec = atoi(argv[i+1]); i++; loop+=2;}
  }
  if (UseInfillRec==1) UseInfillStations=1; // there must be infill stations for the rec.
  
  cerr << "Use Infill: " << UseInfillStations << "  Use Infill Rec: " << UseInfillRec << endl;  
  double coslim=cos(15*kCdasUtil::DEG2RAD);
  
  AugerIoSd input (argc-loop, argv+loop);
  int nb=input.LastEvent();
  bool firstIteration = true;

  for(EventPos pos=input.FirstEvent(); pos<input.LastEvent(); pos=input.NextEvent()) {
    if (!(pos%100))
      cerr << pos << "/" << nb << endl;
    TErEvent event(pos);
    //cerr << "Reading event: " << event.AugerId << endl;
    ntot++;
    if (BadEvent(event.AugerId,BadIds)) {
      //cerr << "Remove bad period event " << event.AugerId << endl;
      nreject++;
      continue;
    }    
    if (firstIteration && UseInfillStations){
      kEs::UseInfillStations(UseInfillStations);
      firstIteration = false;
    } 
    //kEs::UseInfillStations(UseInfillStations);
    event.UseInfillRec=UseInfillRec;
    
    if (!event.IsT4()) continue;
    event.OfficialSelection();
    theLpnheRec().SetDefault();
    theLpnheRec().SetVerbosity(0);
    
    theLpnheRec().SetRec(eGeomOptimal, eLDFOptimal);
    int ret=event.Reconstruct(&theLpnheRec());

    // Compute rec/estimate difference
    double dcos=event.fEstimated.fU*event.fRec.fU+event.fEstimated.fV*event.fRec.fV+sqrt(1-event.fEstimated.fU*event.fEstimated.fU-event.fEstimated.fV*event.fEstimated.fV)*sqrt(1-event.fRec.fU*event.fRec.fU-event.fRec.fV*event.fRec.fV);
    double dgrnd=sqrt(pow(event.fEstimated.fXCore-event.fRec.fXCore,2)+pow(event.fEstimated.fYCore-event.fRec.fYCore,2));
    int ok=(int)!((dcos<coslim) || (dgrnd>800) || (dgrnd>600 && event.fSelectedStations.size()==4) || (dgrnd>400 && event.fSelectedStations.size()==3));

    // Print result
    if (!ret || (!(event.fRec.Theta()>=0 && event.fRec.Theta()<=M_PI)) || (!(event.fRec.fE>0)) || (!(event.fRec.fSRef>=0)))
      continue;
      // bad event

    if ( (event.fSelectedStations.size() <4) || (event.fRec.LocalTheta("deg") >60.) || (!event.IsT5()) ||  (!event.IsICRC2005Posterior(event.fRec.fXCore, event.fRec.fYCore)) ||  (!ok) )
      continue;
    
    vector<ActiveStation> ActStat;
    unsigned int ns=event.fSelectedStations.size();
    
    for (unsigned int i=0;i<ns;++i){
      ActiveStation s(event.fSelectedStations[i]->SigInVEM(), event.fSelectedStations[i]->fX-event.fRec.fXCore, event.fSelectedStations[i]->fY-event.fRec.fYCore);
      ActStat.push_back(s);
    }
    
    // sort the stations by decreasing value of the integrated signal
    for(int is=1; is<(int) ActStat.size(); is++) 
      for(int js=is; js>0; js--) 
	{
	  if(ActStat[js].Signal > ActStat[js-1].Signal) {
	    ActiveStation temp = ActStat[js];
	    ActStat[js] = ActStat[js-1];
	    ActStat[js-1] = temp;
	  }
	}

    cout<< event.AugerId<< " " <<  event.fRec.fSRef << " " << theLpnheRec().fBeta << " " << theLpnheRec().fdBeta <<  " " << event.fRec.fFitBeta <<" " << theLpnheRec().fGamma << " "<< theLpnheRec().fdGamma<< " " << event.fRec.LocalTheta("rad") << " " << event.fRec.LocalPhi("rad") << " " <<  event.fRec.dXCore()<< " " <<  event.fRec.dYCore()<< " " << theLpnheRec().globalChi2 << " " << theLpnheRec().ndof << " " << event.fSelectedStations.size()<< " " <<  event.fSilentStations.size() <<endl;
    
    for (unsigned int i=0;i<ns;++i)
      //cout << event.fSelectedStations[i]->fX-event.fRec.fXCore << " " <<  event.fSelectedStations[i]->fY-event.fRec.fYCore << " " << event.fSelectedStations[i]->SigInVEM() << endl;
      cout << ActStat[i].X << " " << ActStat[i].Y << " " << ActStat[i].Signal << endl;
    for (unsigned int i=0;i<event.fSilentStations.size();++i)
      cout << event.fSilentStations[i]->fX-event.fRec.fXCore << " " <<  event.fSilentStations[i]->fY-event.fRec.fYCore << endl;

  }
  cerr << ntot << " evts read  " <<  nreject << " rejected " << endl;
  return 0;
}
