/** Sample program to illustrate the usage of the IoAuger library
 *  @version $Id: readhybrid.cc,v 1.3 2005-02-23 15:12:06 bertou Exp $
 */


#include <AugerEvent.h>
#include <AugerFile.h>
#include <IoSdData.h>
#include <EyeEvent.hh>
#include <EyeEventHeader.hh>
#include <TTimeStamp.hh>

#include <iostream>
#include <iomanip>

using namespace std;

int main(int argc, char ** argv) {

  if (argc!=2) {
    cout << "Usage: readhybrid   [filename]" << endl;
    exit(0);
  }

  // open a file for read
  AugerFile file(argv[1], AugerFile::eRead);

  // loop over events on file
  AugerEvent event;
  while (file.ReadNext(event) != AugerFile::eEOF) {

    cout << " Event Id " << event.Id()<< " ";
    unsigned int nstations=0;

    // retrieve  the sd event if available
    if (event.HasSd()) {
      IoSdEvent sd = event.Sd();
      nstations= sd.NumberOfErrorZeroStation;
    }

    cout << " Nstations: " << setw(2)<<nstations;
    cout << " NEyes: " << event.Eyes.size( ) <<endl;

    // loop on eye events
    AugerEvent::EyeIterator eye;
    for(eye= event.EyesBegin(); eye!=event.EyesEnd(); ++eye) {
      cout << "                            ";
      cout << "          eye: "<<eye->GetEventHeader()->GetEyeNo()
      << " sec: " << eye->GetEventHeader()->GetTimeStamp()->GetGPSSec()
      << endl;

    } // loop on eyes


  }// loop on events

}// main


