#include "math.h"
#include "Er.h"
#include "IoAuger.h"
#include "TCdasUtil.h"
#include "SdRecLpnhe.h"

using namespace std;


// From Paul Sommers mail of Subject: Re: Another data set
// Date: 29 juin 2005 19:38:37 GMT-03:00
// E = .162*(S1000/CIC(theta))^1.063
// CIC(theta)=1.049+0.00974*theta-0.000293*theta^2

double cic(double theta /* in degree*/) {
return 1.049+0.00974*theta-0.000293*theta*theta;
}

double IcrcEnergy(double s1000, double theta) {
return 0.162*pow(s1000/cic(theta), 1.063);
}

int main (int argc, char *argv[]) {
  double coslim=cos(15*kCdasUtil::DEG2RAD);
  AugerIoSd input (argc-1, argv+1);
  int nb=input.LastEvent();
  for(EventPos pos=input.FirstEvent(); pos<input.LastEvent(); pos=input.NextEvent()) {
    if (!(pos%100))
      cerr << pos << "/" << nb << endl;
    TErEvent event(pos);
    if (!event.IsT4()) continue;
    event.OfficialSelection();
    theLpnheRec().SetDefault();
    theLpnheRec().SetVerbosity(0);
    
    theLpnheRec().SetRec(eGeomOptimal, eNKG);
    int ret=event.Reconstruct(&theLpnheRec());
    // Compute rec/estimate difference
    double dcos=event.fEstimated.fU*event.fRec.fU+event.fEstimated.fV*event.fRec.fV+sqrt(1-event.fEstimated.fU*event.fEstimated.fU-event.fEstimated.fV*event.fEstimated.fV)*sqrt(1-event.fRec.fU*event.fRec.fU-event.fRec.fV*event.fRec.fV);
    double dgrnd=sqrt(pow(event.fEstimated.fXCore-event.fRec.fXCore,2)+pow(event.fEstimated.fYCore-event.fRec.fYCore,2));

    // Print result
    if (!ret || (!(event.fRec.Theta()>=0 && event.fRec.Theta()<=M_PI)) || (!(event.fRec.fE>0)) || (!(event.fRec.fSRef>=0)))
      // bad event
      cout <<event.AugerId<<" -1 "<< event.Stations.size() << " " << event.fSelectedStations.size() << " " << event.fEstimated.Theta("deg") << " " << acos(dcos)*kCdasUtil::RAD2DEG << " " << dgrnd << endl;
    else {
      int ok=1;
      if ((dcos<coslim) || (dgrnd>800) || (dgrnd>600 && event.fSelectedStations.size()==4) || (dgrnd>400 && event.fSelectedStations.size()==3))
        ok=0;
      double geochi=theLpnheRec().angularChi2;
      if (geochi<0.01) geochi=0;
      double IcrcE = -1.0; // Only valid for zenith below 60 degrees.
      if (event.fRec.LocalTheta("deg") < 60.0) IcrcE = IcrcEnergy(event.fRec.fSRef, event.fRec.LocalTheta("deg"));
      cout<<event.AugerId<< " " << event.fSelectedStations.size() << " " << event.fRec.LocalTheta("deg") << " " << event.fRec.LocalPhi("deg") << " " << IcrcE  << " ";
      cout << event.fRec.L() << " " << event.fRec.B() << " " << event.UTCTime << " " << int(event.fRec.fT0) << " " << event.fRec.fXCore << " " << event.fRec.fYCore << " ";
      cout << event.fRec.fSRef << " " << event.fRec.dSRef() << " " << event.fRec.Ra() << " " << event.fRec.Dec() << " " << event.fRec.Theta("deg") << " ";
      cout << event.fRec.Phi("deg") << " " << event.fRec.dTheta("deg") << " " << event.fRec.dPhi("deg") << " " << event.fRec.dXCore() << " " << event.fRec.dYCore() << " ";
      cout << ok << " " << event.IsT5() << " " << event.IsT5(2) << " " << event.IsT5(3) << " ";
      cout << event.IsICRC2005()  << " " << event.trigger().IsFd() << " " << geochi << " ";
      cout << theLpnheRec().ldfChi2 << " " << theLpnheRec().globalChi2 << " " << theLpnheRec().ndof << " " << theLpnheRec().fBeta << " " << theLpnheRec().fGamma << " ";
      cout << theLpnheRec().fR << " " << event.Id << " " << event.IsICRC2005Posterior(event.fRec.fXCore, event.fRec.fYCore) << " " << event.fRec.fE <<endl;
    }
  }
}
